dnl -*- mode: autoconf; c-basic-offset: 2 -*-
dnl =====================================================================
dnl   General routine for searching out supported languages
dnl =====================================================================
AC_DEFUN([SG_PACKAGE_LANGUAGE],
[
  AC_ARG_WITH([$1],[  --with-$1=<dir>  Directory where $1 is installed],
    [
    ],
    [
      dnl ---------------------------------------------------------------
      dnl  Making an educated guess as to where Language is installed
      dnl ---------------------------------------------------------------
      for path in ${prefix} `echo $PATH | \
	  sed -e 's/s*bin:/ /g' -e 's/:/ /g' -e 's/s*bin$//'`
      do
	if test $2 $path/$3
	then
	  with_$1="$path"
	  break
        fi
      done
    ])
  AC_MSG_CHECKING(for $1 installation)
  case x${with_$1} in
  x|xno)
    AC_MSG_RESULT(no)
    with_$1="" # Normalize the result
    ;;
  *)
    if test $2 ${with_$1}/$3
    then
      AC_MSG_RESULT(${with_$1})
      script_globus="$script_globus lib$1globus.la"
      AC_SUBST(script_globus)
    else
      AC_MSG_ERROR([no (${with_$1})])
    fi
    ;;
  esac
])
dnl =====================================================================
dnl   Search for GLOBUS enumerated types
dnl =====================================================================
AC_DEFUN([SG_CHECK_GLOBUS_ENUM],
[
  sg_cflags_backup="$CFLAGS"
  CFLAGS="$CFLAGS -I${GLOBUS_DIR}/include/${GLOBUS_MODEL}"
  AC_MSG_CHECKING(if $2 is defined in $1)
  AC_TRY_COMPILE([
	$3
	#include "$1"
    ],
    [
      if ( $2 == $2 )
      {
        exit( 0 );
      }
      exit( 1 );
    ],[AC_DEFINE(HAVE_$2,1,Set if $2 is defined by globus header file $1)
       AC_MSG_RESULT(yes)],
    [AC_MSG_RESULT(no)])
  AC_SUBST(HAVE_$2)
  CFLAGS="$sg_cflags_backup"
])
dnl =====================================================================
dnl   Search for GLOBUS header files
dnl =====================================================================
AC_DEFUN([SG_CHECK_GLOBUS_HEADER],
[
  case x${with_globus} in
  x|xno)
    ;;
  *)
    hdr="globus_$1.h"
    AC_MSG_CHECKING(if globus has $hdr)
    if test -f ${with_globus}/include/${GLOBUS_BASE_MODEL}/$hdr
    then
      AC_DEFINE(HAVE_GLOBUS_$2_H,1,Set to 1 if globus installation has globus_$1.h header)
      AC_MSG_RESULT(yes)
      AM_CONDITIONAL(HAVE_GLOBUS_$2,true)
      AC_MSG_CHECKING(if globus has $hdr in threaded model)
      if test -f ${with_globus}/include/${GLOBUS_BASE_MODEL}pthr/$hdr
      then
        AC_MSG_RESULT(yes)
      else
        AC_MSG_RESULT(no)
	default_globus_thread_model=""
      fi
   else
      AM_CONDITIONAL(HAVE_GLOBUS_$2,false)
      AC_MSG_RESULT(no)
    fi
    ;;
  esac
]
)
dnl =====================================================================
dnl   Search for GLOBUS
dnl =====================================================================
AC_DEFUN([SG_PACKAGE_GLOBUS],
[
  AC_ARG_WITH([globus],
  [  --with-globus=<dir>   top level directory containing GLOBUS tools],
  [ case x${with_globus} in
    x|xno)
      AC_MSG_ERROR(Cannot build without GLOBUS tool set)
      ;;
    esac
  ],[
    dnl------------------------------------------------------------------
    dnl Default handler
    dnl------------------------------------------------------------------
    dnl   Making an educatecd guess where the tools are installed
    dnl   The sed expression will look in the user's path for
    dnl     additional directories to try.
    dnl -----------------------------------------------------------------
    for path in ${GLOBUS_LOCATION} ${prefix} `echo $PATH | \
      sed -e 's,/s*bin:, ,g' -e 's/:/ /g' -e 's,/s*bin$,,'`
    do
      if test -x $path/bin/globus-makefile-header
      then
        with_globus="$path"
	break
      fi
    done
  ]) dnl AC_ARG_WITH
  AC_MSG_CHECKING(for globus installation directory)
  if test -d "${with_globus}"
  then
    AC_MSG_RESULT(${with_globus})
    GLOBUS_DIR="${with_globus}"
    AC_SUBST(GLOBUS_DIR)
  else
    AC_MSG_RESULT(FAILED)
    AC_MSG_ERROR(unable to find GLOBUS. Please set the env variable GLOBUS_LOCATION)
  fi
  #----------------------------------------------------------------------
  # Checking if the compiler is supported by Globus
  #----------------------------------------------------------------------
  AC_MSG_CHECKING(if compiler supported by globus tools)
  case x$CXX in
  xg++|xgcc)
    GLOBUS_BASE_MODEL="gcc32dbg"
    ;;
  *)
    AC_MSG_ERROR([no ($CXX)])
    ;;
  esac
  AC_MSG_RESULT(yes)
  #----------------------------------------------------------------------
  # Check for header files
  #  If any of the header files needed do not have a threaded counter
  #  part, then default_globus_thread_model will be set to the empty
  #  string.
  #----------------------------------------------------------------------
  default_globus_thread_model="pthr"

  SG_CHECK_GLOBUS_HEADER(common,COMMON)
  SG_CHECK_GLOBUS_HEADER(common_include,COMMON_INCLUDE)
  SG_CHECK_GLOBUS_HEADER(error,ERROR)
  SG_CHECK_GLOBUS_HEADER(ftp_client,FTP_CLIENT)
  SG_CHECK_GLOBUS_HEADER(ftp_control,FTP_CONTROL)
  SG_CHECK_GLOBUS_HEADER(gass_cache,GASS_CACHE)
  SG_CHECK_GLOBUS_HEADER(gass_copy,GASS_COPY)
  SG_CHECK_GLOBUS_HEADER(gass_server_ez,GASS_SERVER_EZ)
  SG_CHECK_GLOBUS_HEADER(gass_transfer,GASS_TRANSFER)
  SG_CHECK_GLOBUS_HEADER(gass_transfer_proto,GASS_TRANSFER_PROTO)
  SG_CHECK_GLOBUS_HEADER(gram_client,GRAM_CLIENT)
  SG_CHECK_GLOBUS_HEADER(gram_protocol,GRAM_PROTOCOL)
  SG_CHECK_GLOBUS_HEADER(io,IO)
  SG_CHECK_GLOBUS_HEADER(list,LIST)
  SG_CHECK_GLOBUS_HEADER(module,MODULE)
  SG_CHECK_GLOBUS_HEADER(object,OBJECT)
  SG_CHECK_GLOBUS_HEADER(url,URL)
  SG_CHECK_GLOBUS_HEADER(xio,XIO)
  #----------------------------------------------------------------------
  # Checking for Globus Thread support
  #----------------------------------------------------------------------
  AC_MSG_CHECKING(for globus threads)
  AC_ARG_ENABLE([globus-threads],
  [  --enable-globus-threads   model of GLOBUS threading],
  [ case x${enable_globus_threads} in
    xyes)
      globus_threads="pthr"
      case x${default_globus_thread_model} in
      x)
        AC_MSG_ERROR(cannot enable globus threads because not all packages support the threaded model)
	;;
      esac
      ;;
    xno)
      globus_threads=""
      ;;
    esac
  ],[
    dnl------------------------------------------------------------------
    dnl Default handler
    dnl------------------------------------------------------------------
    globus_threads=${default_globus_thread_model}
  ])
  case x$globus_threads in
  x)
    AC_MSG_RESULT(<none>)
    ;;
  *)
    AC_MSG_RESULT($globus_threads)
    AC_DEFINE(HAVE_GLOBUS_THREADED,1,true if globus compiled with threads)
    ;;
  esac
  #----------------------------------------------------------------------
  # Checking if the compiler is supported by Globus
  #----------------------------------------------------------------------
  AC_MSG_CHECKING(if compiler supported by globus tools)
  case x$CXX in
  xg++|xgcc)
    GLOBUS_MODEL="gcc32dbg${globus_threads}"
    ;;
  *)
    AC_MSG_ERROR([no ($CXX)])
    ;;
  esac
  AC_MSG_RESULT(yes ($GLOBUS_MODEL))
  AC_SUBST(GLOBUS_MODEL)
  AC_ARG_ENABLE([globus-experimental])
  AC_MSG_CHECKING(if globus experimental modules should be built)
  case x$enable_globus_experimental in
  xyes)
    AC_MSG_RESULT(yes)
    AM_CONDITIONAL(ENABLE_GLOBUS_EXPERIMENTAL,true)
    ;;
  *)
    AC_MSG_RESULT(no)
    AM_CONDITIONAL(ENABLE_GLOBUS_EXPERIMENTAL,false)
    ;;
  esac
  #----------------------------------------------------------------------
  # Check for certain macros or enums
  #----------------------------------------------------------------------
  SG_CHECK_GLOBUS_ENUM(globus_xio_http.h,
	GLOBUS_XIO_HTTP_ATTR_SET_REQUEST_CALLBACK)
  SG_CHECK_GLOBUS_ENUM(globus_xio_http.h,
	GLOBUS_XIO_HTTP_ATTR_SET_RESPONSE_CALLBACK)
])
dnl =====================================================================
dnl  SWIG support
dnl =====================================================================
AC_DEFUN([SG_PROG_ROBODOC],
[
  dnl -------------------------------------------------------------------
  dnl  Find a useable version of SWIG
  dnl -------------------------------------------------------------------
  AC_CHECK_PROGS(ROBODOC,robodoc)
  dnl -------------------------------------------------------------------
  dnl Make findings public
  dnl -------------------------------------------------------------------
  AM_CONDITIONAL(HAVE_ROBODOC,test -n "$ROBODOC")
])
dnl =====================================================================
dnl  SWIG support
dnl =====================================================================
AC_DEFUN([SG_PROG_SWIG],
[
  dnl -------------------------------------------------------------------
  dnl  Find a useable version of SWIG
  dnl -------------------------------------------------------------------
  AC_PATH_PROG(SWIG,swig)
  dnl -------------------------------------------------------------------
  dnl Make findings public
  dnl -------------------------------------------------------------------
  AC_SUBST(SWIGFLAGS)
  AC_SUBST(SWIG)
])
dnl======================================================================
dnl LDAS_PROG_VALGRIND
dnl======================================================================
AC_DEFUN([SG_PROG_VALGRIND],
[ AC_MSG_CHECKING(for valgrind)
  AC_PATH_PROGS( VALGRIND, valgrind )
  case x$VALGRIND in
  x) AC_MSG_RESULT(no);;
  *) AC_MSG_RESULT($VALGRIND);;
  esac
  AM_CONDITIONAL(HAVE_VALGRIND, test x$VALGRIND != x) ] )
dnl =====================================================================
dnl  Look for TCL scripting language support
dnl =====================================================================
AC_DEFUN([SG_PACKAGE_TCL],
[
  SG_PACKAGE_LANGUAGE([tcl],[-f],[include/tcl.h])
  AM_CONDITIONAL(HAVE_TCL,test -n "${with_tcl}")
  case x${with_tcl} in
  x)
    ;;
  *)
    dnl -----------------------------------------------------------------
    dnl  Configure for tcl
    dnl -----------------------------------------------------------------
    tcl_INCLUDE="${with_tcl}/include"
    AC_SUBST(tcl_INCLUDE)
    dnl -----------------------------------------------------------------
    dnl  Determine if Tcl compiled thread enabled
    dnl -----------------------------------------------------------------
    AC_MSG_CHECKING(if Tcl was compiled thread enabled)
    if tcl -c 'exit [[expr ! [info exists ::tcl_platform(threaded)] ]]'
    then
      AC_MSG_RESULT(yes)
      AC_DEFINE(TCL_IS_THREADED,1,True if TCL interpreter was compiled threaded)
      dnl ---------------------------------------------------------------
      dnl  Determine if Thread package is available
      dnl ---------------------------------------------------------------
      AC_MSG_CHECKING(if Tcl has the Thread package)
      if tcl -c 'catch { package require Thread; exit 0} err; exit 1'
      then
        AC_MSG_RESULT(yes)
        AC_DEFINE(TCL_HAS_PACKAGE_THREAD,1,True if TCL can load the threads package)
      else
        AC_MSG_RESULT(no)
      fi
    else
      AC_MSG_RESULT(no)
    fi
    ;;
  esac
])
EMPTY=""
AC_SUBST(EMPTY)
