#========================================================================
# Allow for some system testing using valgrind if available
#========================================================================
VALGRIND_TESTS_ENVIRONMENT ?= $(TESTS_ENVIRONMENT)
VALGRIND_TESTS ?= $(TESTS)
VALGRIND_ALL_OPTS=--suppressions=$(top_srcdir)/common/tclglobus.supp
VALGRIND_ALL_OPTS+=--num-callers=40
VALGRIND_ALL_OPTS+=$(VALGRIND_TOOL_OPTS)
VALGRIND_ALL_OPTS+=$(VALGRIND_OPTS)

valgrind-memcheck:
	@$(MAKE) \
	  VALGRIND_TOOL="memcheck" \
	  VALGRIND_TOOL_OPTS="--leak-check=yes --show-reachable=yes" \
	  valgrind
	@find ./ -name '*.memcheck' \
	  -exec egrep '(definitely lost:|possibly lost:|still reachable:)' \
	  {} /dev/null \;

valgrind-addrcheck:
	@$(MAKE) \
	  VALGRIND_TOOL="addrcheck" \
	  VALGRIND_TOOL_OPTS="--leak-check=yes --show-reachable=yes" \
	  valgrind
	@find ./ -name '*.addrcheck' \
	  -exec egrep '(ERROR SUMMARY:|definitely lost:|possibly lost:|still reachable:)' \
	  {} /dev/null \;

valgrind: $(VALGRIND_TESTS)
	@echo $(VALGRIND_LD_LIBRARY_PATH)
	@for x in $(VALGRIND_TESTS);				\
	do							\
	  if test -x $(srcdir)/$$x;				\
	  then							\
	    cmd="$(srcdir)/$$x";				\
            extra_opts="$$extra_opts --trace-children=yes";	\
	    logbase="$$x";					\
	  elif test -x .libs/lt-$$x;				\
	  then							\
	    cmd=".libs/lt-$$x";					\
	    logbase="$$cmd";					\
	  elif test -x .libs/$x;				\
	  then							\
	    cmd=".libs/$$x";					\
	    logbase="$$cmd";					\
	  else							\
	    cmd=".libs/$$x";					\
	    logbase="$$cmd";					\
	  fi;							\
	  $(VALGRIND_TESTS_ENVIRONMENT)				\
	  env LD_LIBRARY_PATH="$(VALGRIND_LD_LIBRARY_PATH):$${LD_LIBRARY_PATH}" \
	    $(VALGRIND) --tool=$(VALGRIND_TOOL)			\
	    $(VALGRIND_ALL_OPTS) $$extra_opts			\
            $$cmd > $$logbase.$(VALGRIND_TOOL) 2>&1;		\
	done
