/* -*- mode: c++; c-basic-offset: 2; -*- */

#ifdef SWIG

%include "../../config.h"

/*****************************************************************
 * Hand-wrapped SWIG functions
 *****************************************************************/
%ignore globus_ftp_control_get_parallelism;

%ignore globus_ftp_control_host_port_init;
%ignore globus_ftp_control_local_pasv;

%ignore globus_ftp_control_handle_init;
%ignore globus_ftp_control_handle_destroy;

%ignore globus_ftp_control_command_init;
%ignore globus_ftp_control_command_destroy;

%ignore globus_ftp_control_send_command;

%ignore globus_ftp_control_auth_info_init;

#if 0
%ignore globus_ftp_control_server_handle_init;
%ignore globus_ftp_control_server_handle_destroy;
%ignore globus_ftp_control_server_accept;
#endif

%ignore globus_ftp_control_server_listen;

%ignore globus_ftp_control_data_write;
%ignore globus_ftp_control_data_read;
%ignore globus_ftp_control_data_read_all;

%ignore globus_ftp_control_create_data_info;
%ignore globus_ftp_control_data_write_stripe;
%ignore globus_X_ftp_control_data_write_stripe;

%include "tcl-typemaps.i"

%include "swig/globus_ftp_control.i"

%{
  
#include <tcl.h>
#include "tcl-callback.h"

#include "globus_ftp_control.h"

#include "common/sgdebug.h"
#include "common/sginit.h"
  
#define BUFFER_READ		0  
#define MAX_UNIQUE_NAME		64
  
  /* Callback-related typedef */
  typedef struct callbackData_ {
    long           total_bytes;
    int            iovec_cnt;
    void*          buffer;
    char           unique_name[MAX_UNIQUE_NAME];
  } callbackData;

  
  /**************************************************************************
   * Asynchronous operation completion callback
   **************************************************************************/

  static int
    GlobusFTPControlResponseNoCleanup( )
    {
      static char function[] = "GlobusFTPControlResponseNoCleanup";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      
      return 0;
    }

  static int
    GlobusFTPControlResponseCleanup( )
    {
      static char function[] = "GlobusFTPControlResponseCleanup";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      
      return 1;
    }
  
  static void
    TclFTPControlResponseCallback
    ( void * 					callback_arg, 
      struct globus_ftp_client_handle_s * 	handle,
      globus_object_t * 			error,
      globus_ftp_control_response_t *		ftp_response )
    {
      char 	str_handle[128];
      char	str_ftp_response[128];
      char*	msg;

      static char function[] = "TclFTPControlResponseCallback";
      
      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      
      if ( error )
        {
          msg = globus_object_printable_to_string( error );
	  SGDEBUG_LVL_FMT_1( 10, "msg: %s", msg );
        }
      else
        {
          msg = "";
        }

      SGDEBUG_LVL_FMT_1( 10, "handle = %p", handle );
      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_control_handle_t *" ), 0 );
      SGDEBUG_LVL_FMT_1( 10, "str_handle = %s", str_handle );

      SGDEBUG_LVL_FMT_1( 10, "ftp_response: %p", ftp_response );
      SWIG_MakePtr( str_ftp_response, ftp_response,
		    SWIG_TypeQuery( "globus_ftp_control_response_t *" ), 0 );
      SGDEBUG_LVL_FMT_1( 10, "str_ftp_response: %s", str_ftp_response );
      
      if ( ftp_response )
	{
	  SGDEBUG_LVL_FMT_1( 10, "response code: %d",
			     ftp_response->code );
	  SGDEBUG_LVL_FMT_1( 10, "response buffer: %s",
			     ftp_response->response_buffer );
	  SGDEBUG_LVL_FMT_1( 10, "response length: %d",
			     ftp_response->response_length );
	  SGDEBUG_LVL_FMT_1( 10, "response buffer size: %d",
			     ftp_response->response_buffer_size );

	  if ( ftp_response->code == 150 )
	    {
	      ((callbackStruct*)callback_arg)->s_post_process = GlobusFTPControlResponseNoCleanup;
	    }
	  else
	    {
	      ((callbackStruct*)callback_arg)->s_post_process = GlobusFTPControlResponseCleanup;
	    }
	}

      
      CallCallback( (callbackStruct*)callback_arg,
                    "%s \"%s\" %s",
                    str_handle, msg, str_ftp_response );

      if ( error )
      {
	globus_libc_free(msg);
      }

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }
  
  /**************************************************************************
   * Asynchronous control callback
   **************************************************************************/
  static void
    TclFTPControlCallback
    ( void *					callback_arg,
      struct globus_ftp_control_handle_s *	handle,
      globus_object_t *				error )
    {
      char 	str_handle[128];
      char*	msg;
      static char function[] = "TclFTPControlCallback";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      if ( error )
        {
          msg = globus_object_printable_to_string( error );
	  SGDEBUG_LVL_FMT_1( 10, "msg: %s", msg );
        }
      else
        {
          msg = "";
        }
      
      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_control_handle_t *" ), 0 );
      SGDEBUG_FMT_1( "str_handle = %s", str_handle );
      
      CallCallback( (callbackStruct*)callback_arg,
                    "%s \"%s\"",
                    str_handle, msg );
      if ( error )
      {
	globus_libc_free(msg);
      }
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }


  static void
    TclFTPControlDataConnectCallback
    ( void *					callback_arg,
      struct globus_ftp_control_handle_s *	handle,
      unsigned int				stripe_ndx,
      globus_bool_t				reused,
      globus_object_t *				error )
    {
      char 	str_handle[128];
      char *	msg;
      static char function[] = "TclFTPControlDataConnectCallback";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      if ( error )
        {
          msg = globus_object_printable_to_string( error );
	  SGDEBUG_LVL_FMT_1( 10, "msg: %s", msg );
        }
      else
        {
          msg = "";
        }

      SGDEBUG_LVL_FMT_1( 10, "handle = %p", handle );
      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_control_handle_t *" ), 0 );
      SGDEBUG_LVL_FMT_1( 10, "str_handle = %s", str_handle );
      
      CallCallback( (callbackStruct*)callback_arg,
                    "%s %d %d \"%s\"",
                    str_handle, stripe_ndx, reused, msg);
      if ( error )
	{
	  globus_libc_free(msg);
	}
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }

  /**************************************************************************
   * GLOBUS_FTP_CONTROL_SERVER_LISTEN
   **************************************************************************/
#if 0
  static void
    GlobusFTPControlServerListenInit( callbackStruct* cb )
    {
      static char function[] = "GlobusFTPControlServerListenInit";
      
      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

      globus_ftp_control_server_t * server_handle =
	(globus_ftp_control_server_t*)
	globus_libc_malloc( sizeof(globus_ftp_control_server_t) );

      SGDEBUG_LVL_FMT_1( 10, "server_handle: %p", server_handle );
      
      cb->s_api_data = (globus_ftp_control_server_t *)server_handle;

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }

  static int
    GlobusFTPControlServerListenCleanup( callbackStruct* cb )
    {
      static char function[] = "GlobusFTPControlServerListenCleanup";
      
      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      
      globus_libc_free( cb->s_api_data );
      cb->s_api_data = NULL;

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      return 1;
    }

  
  static void
    TclFTPControlServerListenCallback
    ( void *					callback_arg,
      struct globus_ftp_control_server_s *	server_handle,
      globus_object_t *				error )
    {
      char 	str_handle[128];
      char *	msg;
      static char function[] = "TclFTPControlServerListenCallback";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

      SGDEBUG_LVL_FMT_1( 10, "server_handle: %p", server_handle );
      
      if ( error )
        {
          msg = globus_object_printable_to_string( error );
	  SGDEBUG_LVL_FMT_1( 10, "msg: %s", msg );
        }
      else
        {
          msg = "";
        }
      
      SWIG_MakePtr( str_handle, server_handle,
                    SWIG_TypeQuery( "globus_ftp_control_server_t *" ), 0 );
      SGDEBUG_FMT_1( "str_handle = %s", str_handle );


      
      CallCallback( (callbackStruct*)callback_arg,
                    "%s \"%s\"",
                    (globus_ftp_control_server_t*)((callbackStruct*)callback_arg)->s_api_data,
		    msg );
      if ( error )
      {
	globus_libc_free(msg);
      }
      
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }
#endif
  
  /**************************************************************************
   * GLOBUS_FTP_CONTROL_SERVER_ACCEPT
   * GLOBUS_FTP_CONTROL_SERVER_AUTHENTICATE
   **************************************************************************/

  static void
    TclFTPControlServerCallback
    ( void *					callback_arg,
      struct globus_ftp_control_server_s *	server_handle,
      globus_object_t *				error )
    {
      char 	str_handle[128];
      char *	msg;
      static char function[] = "TclFTPControlServerCallback";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      if ( error )
        {
          msg = globus_object_printable_to_string( error );
	  SGDEBUG_LVL_FMT_1( 10, "msg: %s", msg );
        }
      else
        {
          msg = "";
        }

      SGDEBUG_FMT_1( "server_handle = %p", server_handle );
      SWIG_MakePtr( str_handle, server_handle,
                    SWIG_TypeQuery( "globus_ftp_control_server_t *" ), 0 );
      SGDEBUG_FMT_1( "str_handle = %s", str_handle );

      CallCallback( (callbackStruct*)callback_arg,
                    "%s \"%s\"",
                    str_handle, msg );
      if ( error )
      {
	globus_libc_free(msg);
      }
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }


  static void
    TclFTPControlAuthCallback
    ( void *					callback_arg,
      struct globus_ftp_control_handle_s *	handle,
      globus_object_t *				error,
      globus_ftp_control_auth_info_t *		auth_result )
    {
      char 	str_handle[128];
      char	str_auth_result[128];
      char *	msg;
      static char function[] = "TclFTPControlAuthCallback";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      if ( error )
        {
          msg = globus_object_printable_to_string( error );
	  SGDEBUG_LVL_FMT_1( 10, "msg: %s", msg );
        }
      else
        {
          msg = "";
        }

      SGDEBUG_FMT_1( "handle = %p", handle );
      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_control_handle_t *" ), 0 );
      SGDEBUG_FMT_1( "str_handle = %s", str_handle );

      SGDEBUG_FMT_1( "auth_result = %p", auth_result );
      SWIG_MakePtr( str_auth_result, auth_result,
                    SWIG_TypeQuery( "globus_ftp_control_info_t *" ), 0 );
      SGDEBUG_FMT_1( "str_auth_result = %s", str_auth_result );
      
      CallCallback( (callbackStruct*)callback_arg,
                    "%s \"%s\" %s",
                    str_handle, msg, str_auth_result );
      if ( error )
	{
	  globus_libc_free(msg);
	}
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }

  static void
    TclFTPControlReadCommandsCallback
    ( void *                                   callback_arg,
      struct globus_ftp_control_handle_s *     handle,
      globus_object_t *                        error,
      union globus_ftp_control_command_u *     command )
    {
      char 	str_handle[128];
      char	str_command[128];
      char *	msg;
      static char function[] = "TclFTPControlReadCommandsCallback";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      if ( error )
        {
          msg = globus_object_printable_to_string( error );
	  SGDEBUG_LVL_FMT_1( 10, "msg: %s", msg );
        }
      else
        {
          msg = "";
        }
      
      SGDEBUG_FMT_1( "handle = %p", handle );
      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_control_handle_t *" ), 0 );
      SGDEBUG_FMT_1( "str_handle = %s", str_handle );
      
      CallCallback( (callbackStruct*)callback_arg,
                    "%s \"%s\"",
                    str_handle, msg );
      
      if ( error )
	{
	  globus_libc_free(msg);
	}
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );      
    }
  
  /**************************************************************************
   * Server callback
   **************************************************************************/

  
  /**************************************************************************
   * GLOBUS_FTP_CONTROL_DATA_READ
   **************************************************************************/
  static void
    GlobusFTPControlDataReadInit( callbackStruct* cb, long buffer_length )
    {
      static char function[] = "GlobusFTPControlDataReadInit";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

      SGDEBUG_LVL_FMT_1( 10, "buffer_length: %d", buffer_length );
      
      /* Allocate memory space for our buffer */
      callbackData* data = 
        (callbackData *)globus_libc_malloc( sizeof(callbackData) );
      
      cb->s_api_data = data;

      data->total_bytes = buffer_length; 
      
      /* :TODO: We should not exit if we fail to allocate our buffer */
      /* :TODO: Must return error message instead of just exit */
      if ( ( data->buffer = 
             (globus_byte_t *)globus_libc_malloc( sizeof(globus_byte_t) * 
                                                  buffer_length )) == NULL ) 
        {
          fprintf(stderr, "Memory allocation fails\n");
          exit(1);
        }
      
      memset( data->buffer, '\0', buffer_length );
                                                                               
      TclGlobusCreateUniqueTCLVariableName( data->unique_name );

      SGDEBUG_LVL_FMT_1( 20, "read callback allocation: %p", data );
      SGDEBUG_LVL_FMT_1( 20, "read buffer allocation: %p", data->buffer );
      SGDEBUG_LVL_FMT_1( 10, "data->unique_name: %s", data->unique_name );  

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }

  
  static int
    GlobusFTPControlDataReadCleanup( callbackStruct* cb )
    {
      static char function[] = "GlobusFTPControlDataReadCleanup";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      callbackData* data = (callbackData*)cb->s_api_data;
    
      Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);

      /* Deallocate memory space */
      if ( data->buffer ) 
        {
          SGDEBUG_LVL_FMT_1
            ( 20, "read buffer deallocation: %p", data->buffer ); 
          globus_libc_free( data->buffer );
        }

      if ( cb->s_api_data ) 
        {
          SGDEBUG_LVL_FMT_1
            ( 20, "read callback deallocation: %p", cb->s_api_data );  
          globus_libc_free(cb->s_api_data);
          cb->s_api_data = NULL;
        }

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      
      return 1;
    }


  static void
    GlobusFTPControlDataReadPreCallback( callbackStruct* cb )
    {
      Tcl_Interp* interp;
      Tcl_Obj* obj;
      static char function[] = "GlobusFTPControlDataReadPreCallback";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

      callbackData* data = (callbackData*)(cb->s_api_data);

      interp = cb->s_main_tcl_interp;

      SGDEBUG_LVL_FMT_1( 10, "data->total_bytes: %d", data->total_bytes );
	
      obj = Tcl_NewByteArrayObj((char *)(data->buffer), data->total_bytes);

      Tcl_SetVar2Ex(interp, data->unique_name, NULL, obj, 0);

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }

  
  static void
    TclFTPControlDataReadCallback
    ( void *				callback_arg,
      globus_ftp_control_handle_t *     handle,
      globus_object_t *			error,
      globus_byte_t *                   buffer,
      globus_size_t                     length,
      globus_off_t                      offset,
      globus_bool_t			eof )
    {
      char str_handle[128];
      char*	msg;
      static char function[] = "TclFTPControlDataReadCallback";
      
      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      if ( error )
        {
          msg = globus_object_printable_to_string( error );
        }
      else
        {
          msg = "";
        }

      SGDEBUG_LVL_FMT_1( 10, "handle: %p", handle );
      SGDEBUG_LVL_FMT_1( 10, "error: %s", msg );
#if BUFFER_READ      
      SGDEBUG_LVL_FMT_1( 10, "buffer: %s", buffer );
#endif      
      SGDEBUG_LVL_FMT_1( 10, "length: %d", length );
      SGDEBUG_LVL_FMT_1( 10, "offset: %d", offset );      
      SGDEBUG_LVL_FMT_1( 10, "eof: %d", eof );
      
      SWIG_MakePtr( str_handle, handle,
		    SWIG_TypeQuery( "globus_ftp_control_handle_t *" ), 0 );
      SGDEBUG_LVL_FMT_1( 10, "str_handle: %s", str_handle );

      callbackData*
        data = (callbackData*)((callbackStruct*)callback_arg)->s_api_data;
      
      data->total_bytes = length;

      CallCallback( (callbackStruct*)callback_arg,
		    "%s \"%s\" $%s %d %d %d",
		    str_handle,
		    msg,
		    data->unique_name,
		    eof,
		    data->total_bytes,
		    offset );

      
      if ( error )
      {
	globus_libc_free(msg);
      }
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }

  /**************************************************************************
   * GLOBUS_FTP_CONTROL_DATA_WRITE
   **************************************************************************/
  static void
    GlobusFTPControlDataWriteInit( callbackStruct* cb,
				   unsigned char* buffer )
    {
      static char function[] = "GlobusFTPControlDataWriteInit";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      
      /* Allocate memory space */
      callbackData* data =
	(callbackData *)globus_libc_malloc( sizeof(callbackData) );
      
      SGDEBUG_LVL_FMT_1( 20, "write callback allocation: %p", data );
      
      cb->s_api_data = data;
      data->buffer = buffer;
      
      TclGlobusCreateUniqueTCLVariableName( data->unique_name );

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }

  static int
    GlobusFTPControlDataWriteCleanup( callbackStruct* cb )
    {
      static char function[] = "GlobusFTPControlDataWriteCleanup";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

      callbackData* data = (callbackData*)(cb->s_api_data);
      
      /* Release TCL resources */
      Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);
      
      /* Deallocate memory space */
      if ( data->buffer ) 
	{
	  SGDEBUG_LVL_FMT_1
	    ( 20, "write buffer deallocation: %p", data->buffer );  
	  globus_libc_free( data->buffer );
	}
      
      /* Deallocate memory space */
      if ( cb->s_api_data ) 
	{
	  SGDEBUG_LVL_FMT_1
	    ( 20, "write callback deallocation: %p", cb->s_api_data );
	  globus_libc_free(cb->s_api_data);
	  cb->s_api_data = NULL;
	}
            
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      
      return 1;
    }

  static void
    GlobusFTPControlDataWritePreCallback( callbackStruct* cb )
    {
      Tcl_Interp* interp;
      Tcl_Obj* obj;
      static char function[] = "GlobusFTPControlDataWritePreCallback";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

      callbackData* data = (callbackData*)(cb->s_api_data);
      
      obj = Tcl_NewByteArrayObj( (char *)(data->buffer), data->total_bytes );
      
      interp = cb->s_main_tcl_interp;
      Tcl_SetVar2Ex(interp, data->unique_name, NULL, obj, 0);
            
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }

  static void
    TclFTPControlDataWriteCallback
    ( void *				callback_arg,
      globus_ftp_control_handle_t *     handle,
      globus_object_t *			error,
      globus_byte_t *                   buffer,
      globus_size_t                     length,
      globus_off_t                      offset,
      globus_bool_t			eof )
    {
      char str_handle[128];
      char*	msg;
      static char function[] = "TclFTPControlDataWriteCallback";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

      if ( error )
        {
          msg = globus_object_printable_to_string( error );
        }
      else
        {
          msg = "";
        }
      
      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_control_handle_t *" ), 0 );
      callbackData*
        data = (callbackData*)((callbackStruct*)callback_arg)->s_api_data;
      data->total_bytes = length;
      data->buffer = buffer;
      CallCallback( (callbackStruct*)callback_arg,
                    "%s \"%s\" $%s %d %d %d",
                    str_handle,
                    msg,
                    data->unique_name,
                    data->total_bytes,
                    offset,
                    eof );
      
      if ( error )
	{
	  globus_libc_free(msg);
	}
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }
%}



/**************************************************************************
 *
 *  SWIG INITIALIZATION STUFF
 *
 **************************************************************************/
%init %{

  SGINIT_X509;

  %}

/**************************************************************************
 **************************************************************************
 *			CONTROL CLIENT TYPEMAPS SECTION
 **************************************************************************
 **************************************************************************/

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_GET_PARALLELISM
 **************************************************************************/
%typemap(in,numinputs=0)
  globus_ftp_control_parallelism_t * parallelism ($*1_ltype temp) 
{
  $1 = (globus_ftp_control_parallelism_t *)&temp;
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_parallelism_t * parallelism
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_control_get_parallelism) tcl_globus_ftp_control_get_parallelism;

%exception tcl_globus_ftp_control_get_parallelism GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_control_get_parallelism
    ( globus_ftp_control_handle_t *			handle,
      globus_ftp_control_parallelism_t *		parallelism )
    {
      globus_result_t retval;
      static char function[] = "tcl_globus_ftp_control_get_parallelism";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      retval = globus_ftp_control_get_parallelism( handle,
						   parallelism );
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      return retval;
    }
  
  %}

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_LOCAL_PARALLELISM
 **************************************************************************/
%rename(globus_ftp_control_local_parallelism) tcl_globus_ftp_control_local_parallelism;

%exception tcl_globus_ftp_control_local_parallelism GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_control_local_parallelism
    ( globus_ftp_control_handle_t *			handle,
      globus_ftp_control_parallelism_t *		parallelism )
    {
      globus_result_t retval;
      static char function[] = "tcl_globus_ftp_control_local_parallelism";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      retval = globus_ftp_control_local_parallelism( handle,
						     parallelism );
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      return retval;
    }
  
  %}

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_HOST_PORT_INIT
 **************************************************************************/
%typemap(in,numinputs=0)
     globus_ftp_control_host_port_t * host_port
{
  $1 = (globus_ftp_control_host_port_t *)
    globus_libc_malloc(sizeof(globus_ftp_control_host_port_t));
}

%typemap(argout,fragment="t_output_helper")
     globus_ftp_control_host_port_t * host_port 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );  
}

%rename(globus_ftp_control_host_port_init)
  tcl_globus_ftp_control_host_port_init;

%inline %{

  void
    tcl_globus_ftp_control_host_port_init
    ( globus_ftp_control_host_port_t *              host_port,
      char *                                        host,
      unsigned short                                port )
    {
      globus_result_t retval;
      static char function[] = "tcl_globus_ftp_control_host_port_init";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      globus_ftp_control_host_port_init( host_port,
					 host,
					 port);

      SGDEBUG_LVL_FMT_1( 10, "host_port->host[0]: %d", host_port->host[0] );
      SGDEBUG_LVL_FMT_1( 10, "host_port->host[1]: %d", host_port->host[1] );
      SGDEBUG_LVL_FMT_1( 10, "host_port->host[2]: %d", host_port->host[2] );
      SGDEBUG_LVL_FMT_1( 10, "host_port->host[3]: %d", host_port->host[3] );
      SGDEBUG_LVL_FMT_1( 10, "host_port->port: %d", host_port->port );
      
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    }
  
  %}

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_LOCAL_PASV
 **************************************************************************/

%typemap(argout,fragment="t_output_helper") globus_ftp_control_host_port_t * address
{
  char buf[256];
  int hi;
  int low;
  
  SGDEBUG_LVL_FMT_1( 10, "$1->host[0]: %d\n", $1->host[0]);
  SGDEBUG_LVL_FMT_1( 10, "$1->host[1]: %d\n", $1->host[1]);
  SGDEBUG_LVL_FMT_1( 10, "$1->host[2]: %d\n", $1->host[2]);
  SGDEBUG_LVL_FMT_1( 10, "$1->host[3]: %d\n", $1->host[3]);
  SGDEBUG_LVL_FMT_1( 10, "$1->port: %d\n", $1->port);

  hi = $1->port / 256;
  low = $1->port - (hi * 256);
  
  sprintf(buf, "%d,%d,%d,%d,%d,%d", $1->host[0], $1->host[1], $1->host[2],
	  $1->host[3], hi, low );

  Tcl_Obj* o = Tcl_NewStringObj((char*)buf, -1);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_control_local_pasv) tcl_globus_ftp_control_local_pasv;

%exception tcl_globus_ftp_control_local_pasv GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_control_local_pasv
    ( globus_ftp_control_handle_t *		handle,
      globus_ftp_control_host_port_t *		address )
    {
      globus_result_t retval;
      static char function[] = "tcl_globus_ftp_control_local_pasv";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      retval = globus_ftp_control_local_pasv( handle,
					      address );
      SGDEBUG_LVL_FMT_1( 10, "retval %d", retval );
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      return retval;
    }
  
  %}

#if 0
/**************************************************************************
 *  GLOBUS_FTP_CONTROL_LOCAL_SPAS
 **************************************************************************/
%typemap(in,numinputs=0) globus_ftp_control_host_port_t addresses[]
{
  $1 = (globus_ftp_control_host_port_t *)
    globus_libc_malloc(sizeof(globus_ftp_control_host_port_t));
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_host_port_t addresses[]
{
  int indx;
  char buf[256];
  int hi;
  int low;
  
  SGDEBUG_LVL_FMT_1( 10, "$1->host[0]: %d\n", $1->host[0]);
  SGDEBUG_LVL_FMT_1( 10, "$1->host[1]: %d\n", $1->host[1]);
  SGDEBUG_LVL_FMT_1( 10, "$1->host[2]: %d\n", $1->host[2]);
  SGDEBUG_LVL_FMT_1( 10, "$1->host[3]: %d\n", $1->host[3]);
  SGDEBUG_LVL_FMT_1( 10, "$1->port: %d\n", $1->port);

  hi = $1->port / 256;
  low = $1->port - (hi * 256);
  
  sprintf(buf, "%d,%d,%d,%d,%d,%d", $1->host[0], $1->host[1], $1->host[2],
	  $1->host[3], hi, low );

  Tcl_Obj* o = Tcl_NewStringObj((char*)buf, -1);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_control_local_spas) tcl_globus_ftp_control_local_spas;

%exception tcl_globus_ftp_control_local_spas GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_control_local_spas
    ( globus_ftp_control_handle_t *		handle,
      globus_ftp_control_host_port_t   		addresses[],
      unsigned int				num_addresses )
  {
    globus_result_t retval;
    static char function[] = "tcl_globus_ftp_control_local_spas";

    SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

    retval = globus_ftp_control_local_spas( handle,
					    addresses,
					    num_addresses );
     
    SGDEBUG_LVL_FMT_1( 10, "retval %d", retval );
    SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    return retval;    
  }
  
  %}
#endif

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_HANDLE_INIT
 **************************************************************************/
%typemap(in,numinputs=0) globus_ftp_control_handle_t * outhandle
{
  $1 = (globus_ftp_control_handle_t *)
    globus_libc_malloc(sizeof(globus_ftp_control_handle_t));

  SGDEBUG_LVL_FMT_1
    ( 10,
      "ftp control handle creation: typemap(in): %p", $1 );  
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_handle_t * outhandle 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_control_handle_init) tcl_globus_ftp_control_handle_init;

%exception tcl_globus_ftp_control_handle_init GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_ftp_control_handle_init
    ( globus_ftp_control_handle_t * outhandle )
    {
      int retval;

      retval = globus_ftp_control_handle_init( outhandle );

      return retval;
    }
  %}


/**************************************************************************
 *  GLOBUS_FTP_CONTROL_HANDLE_DESTROY
 **************************************************************************/
%typemap(freearg) globus_ftp_control_handle_t * inhandle
{
  SGDEBUG_LVL_FMT_1
      ( 10,
	"ftp control handle deletion: typemap(in): %p", arg1 );
  if ( arg1 )
    {
      globus_libc_free( arg1 );
    }
}

%rename(globus_ftp_control_handle_destroy)
  tcl_globus_ftp_control_handle_destroy;

%exception tcl_globus_ftp_control_handle_destroy GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_ftp_control_handle_destroy
    ( globus_ftp_control_handle_t * inhandle )
    {
      int retval;

      retval = globus_ftp_control_handle_destroy( inhandle );

      return retval;
    }
  %}

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_SEND_COMMAND
 **************************************************************************/

%rename(globus_ftp_control_send_command) tcl_globus_ftp_control_send_command;

%exception tcl_globus_ftp_control_send_command GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_control_send_command
    ( globus_ftp_control_handle_t *		handle,
      const char *				cmdspec,
      globus_ftp_control_response_callback_t	callback,
      void *					callback_arg )
    {
      static char function[] = "tcl_globus_ftp_control_send_command";
      globus_result_t retval;

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      retval = globus_ftp_control_send_command( handle,
						cmdspec,
						callback,
						callback_arg );
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      return retval;
    }

  %}


/**************************************************************************
 *  GLOBUS_FTP_CONTROL_SEND_RESPONSE
 **************************************************************************/




/**************************************************************************
 *  GLOBUS_FTP_CONTROL_AUTH_INFO_INIT
 **************************************************************************/

%typemap(in,numinputs=0)
  globus_ftp_control_auth_info_t * out_auth_info
{
  $1 = (globus_ftp_control_auth_info_t *)
    globus_libc_malloc(sizeof(globus_ftp_control_auth_info_t));

  SGDEBUG_LVL_FMT_1
    ( 10,
      "ftp control auth info creation: typemap(in): %p", $1 );
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_auth_info_t * out_auth_info
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_control_auth_info_init)
  tcl_globus_ftp_control_auth_info_init;

%exception tcl_globus_ftp_control_auth_info_init GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_ftp_control_auth_info_init
    ( globus_ftp_control_auth_info_t *	out_auth_info,
      gss_cred_id_t			credential_handle,
      globus_bool_t			encrypt,
      char *				user,
      char *				password,
      char *				account,
      char *				subject )
    {
      globus_result_t retval;
      static char function[] = "tcl_globus_ftp_control_auth_info_init";
      
      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      retval = globus_ftp_control_auth_info_init( out_auth_info,
						  credential_handle,
						  encrypt,
						  user,
						  password,
						  account,
						  subject );
      SGDEBUG_LVL_FMT_1( 10, "retval %d", retval );
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      
      return retval;
    }
  %}


/**************************************************************************
 **************************************************************************
 *			CONTROL SERVER TYPEMAPS SECTION
 **************************************************************************
 **************************************************************************/

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_COMMAND_INIT
 **************************************************************************/
%typemap(in,numinputs=0) globus_ftp_control_command_t * outcommand
{
  $1 = (globus_ftp_control_command_t *)
    globus_libc_malloc(sizeof(globus_ftp_control_command_t));

  SGDEBUG_LVL_FMT_1
    ( 10,
      "ftp control command creation: typemap(in): %p", $1 );  
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_command_t * outcommand 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_control_command_init)
  tcl_globus_ftp_control_command_init;

%exception tcl_globus_ftp_control_command_init GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_ftp_control_command_init
    ( globus_ftp_control_command_t * 	outcommand,
      char *				raw_command,
      globus_ftp_control_auth_info_t *	auth_info )
  {
    int retval;
    
    retval = globus_ftp_control_command_init( outcommand,
					      raw_command,
					      auth_info );
    
    return retval;
  }
  %}

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_COMMAND_DESTROY
 **************************************************************************/
%typemap(freearg) globus_ftp_control_command_t * incommand
{
  SGDEBUG_LVL_FMT_1
      ( 10,
	"ftp control command deletion: typemap(in): %p", arg1 );
  if ( arg1 )
    {
      globus_libc_free( arg1 );
    }
}

%rename(globus_ftp_control_command_destroy)
  tcl_globus_ftp_control_command_destroy;

%exception tcl_globus_ftp_control_command_destroy GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_ftp_control_command_destroy
    ( globus_ftp_control_command_t * incommand )
  {
    int retval;
    
    retval = globus_ftp_control_command_destroy( incommand );
    
    return retval;
  }
  %}


/**************************************************************************
 *  GLOBUS_FTP_CONTROL_SERVER_HANDLE_INIT
 **************************************************************************/
#if 0
%typemap(in,numinputs=0) globus_ftp_control_server_t * outhandle
{
  $1 = (globus_ftp_control_server_t *)
    globus_libc_malloc(sizeof(globus_ftp_control_server_t));

  SGDEBUG_LVL_FMT_1
    ( 10,
      "ftp control server handle creation: typemap(in): %p", $1 );  
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_server_t * outhandle 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_control_server_handle_init)
  tcl_globus_ftp_control_server_handle_init;

%exception tcl_globus_ftp_control_server_handle_init GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_ftp_control_server_handle_init
    ( globus_ftp_control_server_t * outhandle )
  {
    int retval;
    static char function[] = "tcl_globus_ftp_control_server_handle_init";
    
    SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
    retval = globus_ftp_control_server_handle_init( outhandle );
    SGDEBUG_LVL_FMT_1( 10, "Server handle: %p", outhandle );
    SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    
    return retval;
  }
  %}
#endif

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_SERVER_HANDLE_DESTROY
 **************************************************************************/
#if 0
%typemap(freearg) globus_ftp_control_server_t * inhandle
{
  SGDEBUG_LVL_FMT_1
      ( 10,
	"ftp control server deletion: typemap(in): %p", arg1 );
  if ( arg1 )
    {
      globus_libc_free( arg1 );
    }
}

%rename(globus_ftp_control_server_handle_destroy)
  tcl_globus_ftp_control_server_handle_destroy;

%exception tcl_globus_ftp_control_server_handle_destroy GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_ftp_control_server_handle_destroy
    ( globus_ftp_control_server_t * inhandle )
  {
    int retval;
    static char function[] = "tcl_globus_ftp_control_server_handle_destroy";

    SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
    retval = globus_ftp_control_server_handle_destroy( inhandle );
    SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    
    return retval;
  }
  %}
#endif


/**************************************************************************
 *  GLOBUS_FTP_CONTROL_SERVER_LISTEN
 **************************************************************************/

%typemap(in) (globus_ftp_control_server_callback_t listen_callback,
	      void * listen_callback_arg )(callbackStruct *cbStruct)
{
  cbStruct = CreateCallback( interp, $input );

#if 0  
  /* Allocate our buffer */
  GlobusFTPControlServerListenInit( cbStruct );
  
  $1 = TclFTPControlServerListenCallback;
#endif
  
  $1 = TclFTPControlServerCallback;
  $2 = cbStruct;

#if 0  
  cbStruct->s_post_process = GlobusFTPControlServerListenCleanup;
#endif  
}

%typemap(in, numinputs=0)
  unsigned short * assigned_port ( $*1_ltype temp )
{
  $1 = &temp;
}

%typemap(argout,fragment="t_output_helper") unsigned short * assigned_port
{
  SGDEBUG_LVL_FMT_1( 10, "port %d", *$1 );
  Tcl_Obj* o = Tcl_NewIntObj( (int)(*$1) );
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_control_server_listen) tcl_globus_ftp_control_server_listen;

%exception tcl_globus_ftp_control_server_listen GLOBUS_EXCEPTION_RESULT_T;

%inline %{  
  globus_result_t
    tcl_globus_ftp_control_server_listen
    ( globus_ftp_control_server_t *		server_handle,
      unsigned short 				port,
      unsigned short *				assigned_port,
      globus_ftp_control_server_callback_t	listen_callback,
      void *					listen_callback_arg )
  {
      static char function[] = "tcl_globus_ftp_control_server_listen";
      globus_result_t retval;
      
      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      SGDEBUG_LVL_FMT_1( 10, "Server handle: %p", server_handle );
      SGDEBUG_LVL_FMT_1( 10, "Before: port %d", port );
      
      retval = globus_ftp_control_server_listen ( server_handle,
						  &port,
						  listen_callback,
						  listen_callback_arg );
      
      SGDEBUG_LVL_FMT_1( 10, "retval %d", retval );
      SGDEBUG_LVL_FMT_1( 10, "After: port %d", port );

      *assigned_port = port;

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      return retval;
  }

  
  %}



/**************************************************************************
 *  GLOBUS_FTP_CONTROL_SERVER_ACCEPT
 **************************************************************************/
#if 0
%typemap(in)
  (globus_ftp_control_callback_t callback,
   void * callback_arg) (callbackStruct *cbStruct)
{
  cbStruct = CreateCallback( interp, $input );

  $1 = (globus_ftp_control_callback_t)TclFTPControlCallback;
  $2 = cbStruct;
}

%rename(globus_ftp_control_server_accept)
  tcl_globus_ftp_control_server_accept;

%exception tcl_globus_ftp_control_server_accept GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_control_server_accept
    ( globus_ftp_control_server_t *		listener,
      globus_ftp_control_handle_t * 		handle,
      globus_ftp_control_callback_t		callback,
      void *					callback_arg )
    {
      static char function[] = "tcl_globus_ftp_control_server_accept";
      globus_result_t retval;

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      retval = globus_ftp_control_server_accept( listener,
						 handle,
						 callback,
						 callback_arg );
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      return retval;
    }

  %}
#endif






/**************************************************************************
 *  GLOBUS_FTP_CONTROL_SEND_RESPONSE
 **************************************************************************/







/**************************************************************************
 **************************************************************************
 *			CONTROL DATA TYPEMAPS SECTION
 **************************************************************************
 **************************************************************************/

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_DATA_READ
 **************************************************************************/

%typemap(in) 
  (globus_ftp_control_data_callback_t read_callback,
   void * read_callback_arg) 
  (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  /* Allocate our buffer */
  GlobusFTPControlDataReadInit( cbStruct, arg2 );

  $1 = TclFTPControlDataReadCallback; 
  $2 = cbStruct;

  cbStruct->s_pre_process = GlobusFTPControlDataReadPreCallback;
    cbStruct->s_post_process = GlobusFTPControlDataReadCleanup;
}

%rename(globus_ftp_control_data_read) tcl_globus_ftp_control_data_read;

#if 1
%exception tcl_globus_ftp_control_data_read GLOBUS_EXCEPTION_RESULT_T;
#endif

#if 0
%exception tcl_globus_ftp_control_data_read {

  globus_result_t error = result =
    tcl_globus_ftp_control_data_read(arg1,arg2,arg3,arg4);

  if ( error != 0 )
    {
      globus_object_t* error_obj = globus_error_get( error );
      
      /*
       * We don't need to catch end-of-file flag and
       * falls through to the next execution
       */
      /*
      if ( error == 17 )
	{
	  char*	msg = globus_object_printable_to_string( error_obj );
	  SGDEBUG_LVL_FMT_1( 10, "msg: %s", msg );
	  globus_libc_free( msg );
	}
      else
	{
      */
      Tcl_Obj*	tcl_result = Tcl_GetObjResult( interp );
      char*	msg = globus_object_printable_to_string( error_obj );
      
      Tcl_SetStringObj( tcl_result, msg, strlen( msg ) );
      globus_libc_free( msg );
      return TCL_ERROR;
      /*} */
    }
}
#endif

%inline %{
  typedef struct {
    globus_byte_t*	buffer;
    globus_size_t	length;
  } tcl_globus_buffer_t;
  
  globus_result_t
    tcl_globus_ftp_control_data_read
    ( globus_ftp_control_handle_t * 		handle,
      globus_size_t 				max_length,
      globus_ftp_control_data_callback_t	read_callback,
      void *					read_callback_arg )
    {
      static char function[] = "tcl_globus_ftp_control_data_read";
      globus_result_t	retval;

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      retval = 
        globus_ftp_control_data_read
        (
         handle,
         (globus_byte_t*)
         ((callbackData*)(((callbackStruct*)read_callback_arg)->s_api_data))->buffer,
         max_length,
         read_callback,
         read_callback_arg 
         );
      SGDEBUG_LVL_FMT_1( 10, "retval: %d", retval );
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      
      return retval;
    }
  %}

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_DATA_READ_ALL
 **************************************************************************/
%rename(globus_ftp_control_data_read_all) tcl_globus_ftp_control_data_read_all;

%exception tcl_globus_ftp_control_data_read_all GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  
  globus_result_t
    tcl_globus_ftp_control_data_read_all
    ( globus_ftp_control_handle_t * 		handle,
      globus_size_t 				length,
      globus_ftp_control_data_callback_t	read_callback,
      void *					read_callback_arg )
    {
      static char function[] = "tcl_globus_ftp_control_data_read_all";
      globus_result_t	retval;

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      retval = 
        globus_ftp_control_data_read_all
        (
         handle,
         (globus_byte_t*)
         ((callbackData*)(((callbackStruct*)read_callback_arg)->s_api_data))->buffer,
         length,
         read_callback,
         read_callback_arg 
         );
      SGDEBUG_LVL_FMT_1( 10, "retval: %d", retval );
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      
      return retval;
    }
  %}

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_DATA_WRITE
 **************************************************************************/
%typemap(in) 
  tcl_globus_buffer_t *rw_in_buffer( $*1_ltype temp ) 
{
  unsigned char*   tbuffer;

  tbuffer = 
    (unsigned char *)Tcl_GetByteArrayFromObj( $input, &(temp.length) );
  temp.buffer = globus_libc_malloc( temp.length );
  SGDEBUG_LVL_FMT_1( 20, "write buffer allocation: = %p", temp.buffer );  
  memcpy( temp.buffer, tbuffer, temp.length );

  $1 = &temp;
}

%typemap(in) 
  (globus_ftp_control_data_callback_t rw_cb, void * rw_user_arg) 
  (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );
  GlobusFTPControlDataWriteInit(cbStruct, arg2->buffer);
  
  $1 = TclFTPControlDataWriteCallback; 
  $2 = cbStruct;
  
  cbStruct->s_pre_process = GlobusFTPControlDataWritePreCallback;
  cbStruct->s_post_process = GlobusFTPControlDataWriteCleanup;
}

%rename(globus_ftp_control_data_write) tcl_globus_ftp_control_data_write;

%exception tcl_globus_ftp_control_data_write GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  globus_result_t
    tcl_globus_ftp_control_data_write
    ( globus_ftp_control_handle_t * 		handle,
      tcl_globus_buffer_t * 			rw_in_buffer,
      globus_off_t   	    	  		offset,
      globus_bool_t  	   	  		eof,
      globus_ftp_control_data_callback_t 	rw_cb,
      void * 					rw_user_arg )
    {
      globus_result_t	retval;
      static char function[] = "tcl_globus_ftp_control_data_write";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
      
      retval = globus_ftp_control_data_write( handle,
					      rw_in_buffer->buffer,
					      rw_in_buffer->length,
					      offset,
					      eof,
					      rw_cb,
					      rw_user_arg );

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      
      return retval;
    }
  %}

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_CREATE_DATA_INFO
 **************************************************************************/
%typemap(in, numinputs=0)
  globus_ftp_control_data_write_info_t * data_info
{
  $1 = globus_libc_malloc(sizeof(globus_ftp_control_data_write_info_t));
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_data_write_info_t * data_info
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%typemap(in)
  (globus_ftp_control_data_callback_t create_data_info_cb,
   void * create_data_info_cb_arg) (callbackStruct *cbStruct)
{
  cbStruct = CreateCallback( interp, $input );
    
  $1 = (globus_ftp_control_data_callback_t)TclFTPControlDataReadCallback;
  $2 = cbStruct;
}

%rename(globus_ftp_control_create_data_info) tcl_globus_ftp_control_create_data_info;

%exception tcl_globus_ftp_control_create_data_info GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  globus_result_t
    tcl_globus_ftp_control_create_data_info
    ( globus_ftp_control_handle_t *		handle,
      globus_ftp_control_data_write_info_t *	data_info,
      globus_byte_t *				buffer,
      globus_size_t				length,
      globus_off_t				offset,
      globus_bool_t				eof,
      globus_ftp_control_data_callback_t	create_data_info_cb,
      void *					create_data_info_cb_arg )
    {
      globus_result_t retval;
      static char function[] = "tcl_globus_ftp_control_create_data_info";
      
      SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

      retval = globus_ftp_control_create_data_info( handle,
						    data_info,
						    buffer,
						    length,
						    offset,
						    eof,
						    create_data_info_cb,
						    create_data_info_cb_arg );

      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
      
      return retval;
    }
  %}

/**************************************************************************
 *  GLOBUS_FTP_CONTROL_DATA_WRITE_STRIPE
 **************************************************************************/

%rename(globus_ftp_control_data_write_stripe) tcl_globus_ftp_control_data_write_stripe;

%exception tcl_globus_ftp_control_data_write_stripe GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_control_data_write_stripe
    ( globus_ftp_control_handle_t *		handle,
      tcl_globus_buffer_t * 			rw_in_buffer,
      globus_off_t				offset,
      globus_bool_t				eof,
      int					stripe_ndx,
      globus_ftp_control_data_callback_t	rw_cb,
      void *					rw_user_arg )
  {
    globus_result_t retval;
    static char function[] = "tcl_globus_ftp_control_data_write_stripe";
    
    SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

    retval = globus_ftp_control_data_write_stripe( handle,
						   rw_in_buffer->buffer,
						   rw_in_buffer->length,
						   offset,
						   eof,
						   stripe_ndx,
						   rw_cb,
						   rw_user_arg );

    SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    
    return retval;    
  }

  %}

/**************************************************************************
 *  GLOBUS_X_FTP_CONTROL_DATA_WRITE_STRIPE
 **************************************************************************/

%rename(globus_X_ftp_control_data_write_stripe) tcl_globus_X_ftp_control_data_write_stripe;

%exception tcl_globus_X_ftp_control_data_write_stripe GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  globus_result_t
      tcl_globus_X_ftp_control_data_write_stripe
      ( globus_ftp_control_handle_t *		handle,
	globus_byte_t *				buffer,
	globus_size_t				length,
	globus_off_t				offset,
	globus_bool_t				eof,
	int					stripe_ndx,
	globus_ftp_control_data_write_info_t *	data_info_P )
  {
    globus_result_t retval;
    static char function[] = "tcl_globus_X_ftp_control_data_write_stripe";
    
    SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );

    retval = globus_X_ftp_control_data_write_stripe( handle,
						     buffer,
						     length,
						     offset,
						     eof,
						     stripe_ndx,
						     data_info_P );
    
    
    SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
    
    return retval;
  }
  
  %}


#endif /* SWIG */


/**************************************************************************
 *
 * ROBODOC
 *
 **************************************************************************/

////////////////////////////////////////////////////////////////////
//
// FTP Client-side Control Connection Management
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_control/globus_ftp_control_handle_init
 * NAME
 *   globus_ftp_control_handle_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_handle_init
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set up (i.e. intialize all mutexes and variables)
 *   a Globus FTP handle. It will also enter the handle in a list used by the
 *   module activation/deactivation functions.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to initialize.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_handle_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_handle_init $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_handle_destroy.
 *****/


/****f* globus_ftp_control/globus_ftp_control_handle_destroy
 * NAME
 *   globus_ftp_control_handle_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_handle_destroy
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to free up all dynamicly allocated memory
 *   associated with a given Globus FTP handle. It will also remove the
 *   handle from a list used by the module activation/deactivation functions.
 *   This command should only be called after a call to either
 *   globus_ftp_control_force_close or globus_ftp_control_quit.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to destroy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_handle_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_handle_destroy $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_handle_init.
 *****/


/****f* globus_ftp_control/globus_ftp_control_connect
 * NAME
 *   globus_ftp_control_connect
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_connect
 *   |html </b>
 *   |html <i>
 *   handle
 *   host
 *   port
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to initiate an FTP control connection.
 *   It creates the socket to the FTP server. When the connection is
 *   made to the server, and the server's identification string is
 *   received, the script will be invoked.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - FTP control handle for all subsequent FTP control operations.
 *
 *   |html <b>
 *   host
 *   |html </b>
 *   - The hostname of the FTP server.
 *
 *   |html <b>
 *   port
 *   |html </b>
 *   - The TCP port number of the FTP server.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the connection to the server is
 *   established, and a response has been read. The command will be
 *   executed exactly once in global scope. If an error occurs while
 *   executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_connect returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_connect $handle $host $port
 *   { connect_response_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_response_destroy
 * NAME
 *   globus_ftp_control_response_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_response_destroy
 *   |html </b>
 *   |html <i>
 *   response
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to free up the memory associated with a
 *   response structure.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   response
 *   |html </b>
 *   - The response structure to destroy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_response_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_response_destroy $response ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_response_copy.
 *****/


/****f* globus_ftp_control/globus_ftp_control_response_copy
 * NAME
 *   globus_ftp_control_response_copy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_response_copy
 *   |html </b>
 *   |html <i>
 *   src_response
 *   dest_response
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to copy one response structure to another.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   src_response
 *   |html </b>
 *   - This parameter indicates the response structure to copy.
 *
 *   |html <b>
 *   dest_response
 *   |html </b>
 *   - This parameter specifies the target response structure.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_response_copy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_response_copy $src_response $dest_response ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_response_destroy.
 *****/


/****f* globus_ftp_control/globus_ftp_control_auth_info_init
 * NAME
 *   globus_ftp_control_auth_info_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_auth_info_init
 *   |html </b>
 *   |html <i>
 *   credential_handle
 *   encrypt
 *   user
 *   password
 *   account
 *   subject
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to initialize a authentication information
 *   structure with the values contained in the first to fourth arguments,
 *   which may be GLOBUS_NULL. No memory is allocated in this function.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   credential_handle
 *   |html </b>
 *   - The credential to use for authentication. This may be
 *   GSS_C_NO_CREDENTIAL to use the user's default credential.
 *
 *   |html <b>
 *   encrypt
 *   |html </b>
 *   - Boolean whether or not to encrypt the control channel for this
 *   handle.
 *
 *   |html <b>
 *   user
 *   |html </b>
 *   - The user name.
 *
 *   |html <b>
 *   password
 *   |html </b>
 *   - The password for the user name.
 *
 *   |html <b>
 *   account
 *   |html </b>
 *   - The account for the user name/password.
 *
 *   |html <b>
 *   subject
 *   |html </b>
 *   - The GSSAPI subject name
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_auth_info_init returns 0 and the
 *   authentication structure.
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.   
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_auth_info_init $gss_c_no_credential
 *   $GLOBUS_TRUE ":globus-mapping:" NULL "/DC=org/DC=doegrids/OU=Services/CN=tclproxy.ligo.caltech.edu" ] result auth_info
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_authenticate
 *****/


/****f* globus_ftp_control/globus_ftp_control_authenticate
 * NAME
 *   globus_ftp_control_authenticate
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_authenticate
 *   |html </b>
 *   |html <i>
 *   handle
 *   auth_info
 *   use_auth
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to perform the authentication handshake with
 *   the FTP server. Depending on which parameters are non-NULL, the
 *   authentication may involve GSSAPI credentials, a username, a password,
 *   and an account name.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A unauthenticated GSIFTP control handle. In the case of GSS
 *   authentication the GSS security context is stored in this handle.
 *
 *   |html <b>
 *   auth_info
 *   |html </b>
 *   - This structure is used to pass the following information:
 *   |html <ul>
 *   |html <li> user, the user's name for login purposes. If this string
 *   is "anonymous", "ftp", GLOBUS_NULL or ":globus-mapping:" then the
 *   password argument is optional. If this string is GLOBUS_NULL or
 *   ":globus-mapping:" and gss_auth is true then the users login is
 *   looked by the FTP server host.
 *   |html <li> password, the password for the above user argument. If
 *   the user argument is "anonymous" or "ftp" or if gss_auth is true
 *   this string may be GLOBUS_NULL.
 *   |html <li> account, this parameter is optional. If not used it
 *   should be set to GLOBUS_NULL. It might be needed by firewalls.
 *   |html <li> auth_gssapi_subject, the GSSAPI subject name of the server
 *   you are connecting to. If this is GLOBUS_NULL, and the gss_auth
 *   parameter is set to GLOBUS_TRUE, then the name will default to the
 *   host name.
 *   |html </ul>
 *
 *   |html <b>
 *   use_auth
 *   |html </b>
 *   - If set to GLOBUS_TRUE the above argument indicates that GSS
 *   authentication should be used, otherwise cleartext user/password
 *   authentication is used.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the authentication process is completed
 *   or when an error occurs. The command will be executed exactly once in
 *   global scope. If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_authenticate returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_authenticate $handle $auth_info $GLOBUS_TRUE
 *   { auth_response_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_auth_info_init
 *****/


/****f* globus_ftp_control/globus_ftp_control_auth_info_compare
 * NAME
 *   globus_ftp_control_auth_info_compare
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_auth_info_compare
 *   |html </b>
 *   |html <i>
 *   auth_info_1
 *   auth_info_2
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to compare two authentication information
 *   structures and return zero if the two structures are deemed equal
 *   and a non-zero value otherwise.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   auth_info_1
 *   |html </b>
 *   - The first authentication structure.
 *
 *   |html <b>
 *   auth_info_2
 *   |html </b>
 *   - The second authentication structure.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_auth_info_compare returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_auth_info_compare $auth_info_1 $auth_info_2 ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_send_command
 * NAME
 *   globus_ftp_control_send_command
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_send_command
 *   |html </b>
 *   |html <i>
 *   handle
 *   cmdspec
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to send an FTP command, and register a handler
 *   to receive the FTP reply (or replies, if an intermediate one is sent).
 *   When the control channel is GSS authenticated, the message and the
 *   reply will be automatically GSS wrapped/unwrapped.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a GSIFTP control handle. The command described by
 *   the cmdspec is issued to the server over the control channel
 *   associated with this handle.
 *
 *   |html <b>
 *   cmdspec
 *   |html </b>
 *   - A printf-style format string containing the text of the command to
 *   send to the server.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the authentication process is completed
 *   or when an error occurs. The command will be executed exactly once in
 *   global scope. If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_send_cmd returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_send_cmd $handle $cmdspec
 *   { send_cmd_response_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_abort
 * NAME
 *   globus_ftp_control_abort
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_abort
 *   |html </b>
 *   |html <i>
 *   handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to send the ABORT message to the FTP server.
 *   The ABORT message is sent out-of-band, and terminates any current
 *   data transfer in progress.
 *
 *   As a result of the ABORT, the data channels used by this control
 *   channel will be closed. The data command callback will be issued
 *   with either a completion reply, or a transfer aborted reply. The
 *   ABORT callback will also be invoked, with the server's response
 *   to the abort command.
 *
 *   Any attempts to register buffers for read or write after an ABORT
 *   has been sent will fail with a "no transfer in progress" error.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a GSIFTP control handle. The ABORT command is issued
 *   to the server over the control channel associated with this handle.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the authentication process is completed
 *   or when an error occurs. The command will be executed exactly once in
 *   global scope. If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_abort returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_abort $handle { response_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_quit
 * NAME
 *   globus_ftp_control_quit
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_quit
 *   |html </b>
 *   |html <i>
 *   handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to send a QUIT message to the FTP server and
 *   register a response handler.
 *
 *   This command is used to close the control channel to the FTP server.
 *   There should be no transfer commands in progress when this is called.
 *   Once the final response callback passed to this function is invoked,
 *   the control handle can no longer be used for any gsiftp control
 *   operations.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a GSIFTP control handle. The QUIT command is issued
 *   to the server over the control channel associated with this handle.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the authentication process is completed
 *   or when an error occurs. The command will be executed exactly once in
 *   global scope. If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_quit returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_quit $handle { response_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_force_close
 * NAME
 *   globus_ftp_control_force_close
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_force_close
 *   |html </b>
 *   |html <i>
 *   handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to force a close of the control connection
 *   without waiting for outstanding commands to complete and without
 *   sending QUIT.
 *
 *   This command is used to close the control channel to the FTP server.
 *   Once the final response callback passed to this function is invoked,
 *   the control handle can no longer be used for any gsiftp control
 *   operations.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a GSIFTP control handle. The QUIT command is issued
 *   to the server over the control channel associated with this handle.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the authentication process is completed
 *   or when an error occurs. The command will be executed exactly once in
 *   global scope. If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_force_close returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_force_close $handle { response_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// FTP Server-side Control Connection Management
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_control/globus_ftp_control_server_handle_init
 * NAME
 *   globus_ftp_control_server_handle_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_server_handle_init
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set up (i.e. intialize all mutexes and variables)
 *   a Globus FTP server handle. It will also enter the handle in a list
 *   used by the module activation/deactivation functions.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to initialize.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_server_handle_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_server_handle_init $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_server_handle_destroy.
 *****/


/****f* globus_ftp_control/globus_ftp_control_server_handle_destroy
 * NAME
 *   globus_ftp_control_server_handle_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_server_handle_destroy
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to free up all dynamicly allocated memory
 *   associated with a given Globus FTP server handle. It will also remove the
 *   handle from a list used by the module activation/deactivation functions.
 *   This command should only be called after a call to either
 *   globus_ftp_control_server_stop.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to destroy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_server_handle_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_server_handle_destroy $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_server_handle_init.
 *****/


/****f* globus_ftp_control/globus_ftp_control_command_init
 * NAME
 *   globus_ftp_control_command_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_command_init
 *   |html </b>
 *   |html <i>
 *   command
 *   raw_command
 *   auth_info
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a command structure based on a
 *   null terminated string representing one line of input from the client.
 *   The command structure is used as a convience to determine what
 *   command the client issued. This function parses a command string
 *   sent by a client and populates the command argument appropriatly.
 *   In the GSSAPI case it will also decode and unwrap the command before
 *   parsing it.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   command
 *   |html </b>
 *   - A pointer to the command structure to be initialized.
 *
 *   |html <b>
 *   raw_command
 *   |html </b>
 *   - A null terminated line of client input. Should contain one command.
 *
 *   |html <b>
 *   auth_info
 *   |html </b>
 *   - Authentication information needed for unwrapping a command.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_command_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_command_init $command $raw_command
 *   $auth_info ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_command_destroy, globus_ftp_control_command_copy.
 *****/


/****f* globus_ftp_control/globus_ftp_control_command_destroy
 * NAME
 *   globus_ftp_control_command_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_command_destroy
 *   |html </b>
 *   |html <i>
 *   command
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to free up the memory allocated to the command
 *   argument.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   command
 *   |html </b>
 *   - The command structure whose associated memory is to be freed.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_command_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_command_destroy $command ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_command_init, globus_ftp_control_command_copy.
 *****/


/****f* globus_ftp_control/globus_ftp_control_command_copy
 * NAME
 *   globus_ftp_control_command_copy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_command_copy
 *   |html </b>
 *   |html <i>
 *   dest_command
 *   src_command
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create a copy of a command structure.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   dest_command
 *   |html </b>
 *   - This parameter specifies the target response structure.
 *
 *   |html <b>
 *   src_command
 *   |html </b>
 *   - The command structure to be copied.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_command_copy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_command_copy $dest_command $src_command ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_command_init, globus_ftp_control_command_destroy.
 *****/


/****f* globus_ftp_control/globus_ftp_control_server_listen
 * NAME
 *   globus_ftp_control_server_listen
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control_server_listen
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_server_listen
 *   |html </b>
 *   |html <i>
 *   server_handle
 *   port
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to start the listening on *port for connections
 *   from FTP clients. When a connection request is made the script gets
 *   invoked. Upon return from this function the server_handle structure
 *   is initialized.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   server_handle
 *   |html </b>
 *   - A pointer to a initialized server handle.
 *
 *   |html <b>
 *   port
 *   |html </b>
 *   - A pointer to the port to listen on. If the initial value is zero
 *   it will be set to the default value.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once connection requests are made.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_server_listen returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_server_listen $server_handle $port
 *   { server_listen_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_server_stop
 *****/


/****f* globus_ftp_control/globus_ftp_control_server_stop
 * NAME
 *   globus_ftp_control_server_stop
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control_server_stop
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_server_stop
 *   |html </b>
 *   |html <i>
 *   listener_handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to stop the GSIFTP server from listening on the
 *   given listener object for client connections. All existing client
 *   connections are left open.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_handle
 *   |html </b>
 *   - The listener handle that is no longer listening for client
 *   connections.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked when the server structure is no longer
 *   listening.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_server_stop returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_server_stop $listener_handle
 *   { stop_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_server_listen
 *****/


/****f* globus_ftp_control/globus_ftp_control_server_accept
 * NAME
 *   globus_ftp_control_server_accept
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control_server_accept
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_server_accept
 *   |html </b>
 *   |html <i>
 *   listener_handle
 *   new_handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to accept a connection request from a client.
 *
 *   When the listen callback is called
 *   (see globus_ftp_control_server_listen) a client has requested
 *   a connection. This function must be called to accept that user
 *   connection request. Once the connection is established or if
 *   a error occurs, the callback function is called.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_handle
 *   |html </b>
 *   - The server object that received the connection request.
 *
 *   |html <b>
 *   new_handle
 *   |html </b>
 *   - The control connection object. This structure will be populated
 *   and passed to the callback when the client is authorized. This
 *   structure represents the control connection between the server
 *   and client. It will be used to read commands from the client and
 *   send responses to the client.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the client connection has been accepted.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_server_accept returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_server_accept $listener_handle $new_handle
 *   { accept_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_server_listen, globus_ftp_control_server_stop
 *****/


/****f* globus_ftp_control/globus_ftp_control_server_authenticate
 * NAME
 *   globus_ftp_control_server_authenticate
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control_server_authenticate
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_server_authenticate
 *   |html </b>
 *   |html <i>
 *   handle
 *   auth_requirements
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to authenticate a connection from a client.
 *
 *   After a client connection has been accepted
 *   (using the globus_ftp_control_server_accept call), this command
 *   should be called to authenticate the client. The caller of this
 *   command may specify certain authentication requirements using
 *   the auth_requirements parameter.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The control connection object. This structure will be populated
 *   and passed to the callback when the client is authorized. This
 *   structure represents the control connection between the server
 *   and client. It will be used to read commands from the client and
 *   send responses to the client.
 *
 *   |html <b>
 *   auth_requirements
 *   |html </b>
 *   - This structure represents the authentication requirements
 *   that the user has for a given connection. For example GSIFTP
 *   user name, password, and account.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the client authentication has been
 *   accepted or rejected.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_server_authenticate returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_server_authenticate $handle $auth_req
 *   { auth_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_server_listen,
 *   globus_ftp_control_server_accept,
 *   globus_ftp_control_server_stop
 *****/


/****f* globus_ftp_control/globus_ftp_control_read_commands
 * NAME
 *   globus_ftp_control_read_commands
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control_read_commands
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_read_commands
 *   |html </b>
 *   |html <i>
 *   handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to read GSIFTP commands on a given control
 *   connection.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The control connection handle that commands will be read from.
 *   Prior to calling this the command globus_ftp_control_handle_t
 *   must be populated via a call to globus_ftp_control_accept.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once commands are read.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_server_read_commands returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_server_read_commands $handle $auth_req
 *   { auth_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_server_listen,
 *   globus_ftp_control_server_accept,
 *   globus_ftp_control_server_stop
 *****/


/****f* globus_ftp_control/globus_ftp_control_send_response
 * NAME
 *   globus_ftp_control_send_response
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control_send_response
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_read_commands
 *   |html </b>
 *   |html <i>
 *   handle
 *   respspec
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to send a response to the GSIFTP client.
 *
 *   This command sends a GSIFTP formatted response to the client.
 *   When a command callback is received the user calls this command
 *   to respond to the clients request.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The control connection to send the response across.
 *
 *   |html <b>
 *   respspec
 *   |html </b>
 *   - A formated string representing the users response.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the response has been sent.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_server_send_response returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_server_send_response $handle $respspec
 *   { send_resp_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_read_commands
 *****/


////////////////////////////////////////////////////////////////////
//
// FTP Data Connection Configuration and Management
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_control/globus_ftp_control_data_force_close
 * NAME
 *   globus_ftp_control_data_force_close
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_force_close
 *   |html </b>
 *   |html <i>
 *   handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to force an imediate close of all data
 *   connections.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle that is have its data connections closed.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the data connections are close
 *   or when an error occurs. The command will be executed exactly once in
 *   global scope. If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_force_close returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_force_close $handle
 *   { close_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_data_connect_read
 * NAME
 *   globus_ftp_control_data_connect_read
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_connect_read
 *   |html </b>
 *   |html <i>
 *   handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to create an incoming FTP data connection.
 *
 *   This command will register a globus_io_{accept, connect}. Further
 *   accepts/connects are done by registering a new accept/connect in
 *   the current accept/connect callback. A call to either
 *   globus_ftp_control_local_pasv() or globus_ftp_control_local_port()
 *   needs to precede this calling this command. This command may be
 *   followed by a globus_ftp_data_read.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a FTP control handle which is configured to create
 *   an incoming data connection.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the connection process is completed
 *   or when an error occurs. The command will be executed exactly once in
 *   global scope. If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_connect_read returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_connect_read $handle 
 *   { connect_read_data_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_data_connect_write
 *****/


/****f* globus_ftp_control/globus_ftp_control_data_set_interface
 * NAME
 *   globus_ftp_control_data_set_interface
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_set_interface
 *   |html </b>
 *   |html <i>
 *   handle
 *   interface_addr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create an outgoing FTP data connection.
 *
 *   This command sets the interface that will be used to send and
 *   receive information along the data channel.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a FTP control handle which is configured to create
 *   an outgoing data connection.
 *
 *   |html <b>
 *   interface_addr
 *   |html </b>
 *   - The specified interface address, i.e., IP address.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_control_data_set_interface returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_set_interface $handle
 *   "131.215.114.146" ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_data_connect_write
 * NAME
 *   globus_ftp_control_data_connect_write
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_connect_write
 *   |html </b>
 *   |html <i>
 *   handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to create an outgoing FTP data connection.
 *
 *   This command will register a globus_io_{accept, connect}. Further
 *   accepts/connects are done by registering a new accept/connect in
 *   the current accept/connect callback. A call to either
 *   globus_ftp_control_local_pasv() or globus_ftp_control_local_port()
 *   needs to precede this calling this command. This command may be
 *   followed by a globus_ftp_data_read.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a FTP control handle which is configured to create
 *   an outgoing data connection.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked once the connection process is completed
 *   or when an error occurs. The command will be executed exactly once in
 *   global scope. If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_connect_write returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_connect_write $handle 
 *   { connect_write_data_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_data_connect_read
 *****/


/****f* globus_ftp_control/globus_ftp_control_data_add_channels
 * NAME
 *   globus_ftp_control_data_add_channels
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_add_channels
 *   |html </b>
 *   |html <i>
 *   handle
 *   num_channels
 *   stripe_ndx
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to open additional data channels (connections)
 *   to the host identified by the stripe parameter.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a FTP control handle. This handle is used to determine
 *   the host corresponding to the stripe number and to store information
 *   about any channels added by this command.
 *
 *   |html <b>
 *   num_channels
 *   |html </b>
 *   - The number of additional channels to add.
 *
 *   |html <b>
 *   stripe_ndx
 *   |html </b>
 *   - An integer identifying the stripe to add channels to. In the case
 *   of non-striped transfer this parameter will be ignored.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_add_channels returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_add_channels $handle $num_channels
 *   $stripe_ndx ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_data_query_channels
 * NAME
 *   globus_ftp_control_data_query_channels
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_query_channels
 *   |html </b>
 *   |html <i>
 *   handle
 *   stripe_ndx
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to return the number of currently open channels
 *   for the host identified by the stripe parameter.
 *
 *   This number may be less then the level of parallelism specified in
 *   local_parallelism, due to the possibility that some channels have
 *   not yet connected.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a FTP control handle. This handle is used to determine
 *   the host corresponding to the stripe number and to store information
 *   about any channels added by this command.
 *
 *   |html <b>
 *   stripe_ndx
 *   |html </b>
 *   - An integer identifying the stripe to return the number of channels.
 *   In the case of non-striped transfer this parameter will be ignored.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_query_channels returns 0 and the
 *   number of channels.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_query_channels $handle $stripe_ndx ]
 *   result num_channels
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_data_remove_channels
 * NAME
 *   globus_ftp_control_data_remove_channels
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_remove_channels
 *   |html </b>
 *   |html <i>
 *   handle
 *   num_channels
 *   stripe_ndx
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to remove data channels (connections) to the
 *   host identified by the stripe parameter.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a FTP control handle. This handle is used to determine
 *   the host corresponding to the stripe number and to store information
 *   about any channels added by this command.
 *
 *   |html <b>
 *   num_channels
 *   |html </b>
 *   - The number of additional channels to remove.
 *
 *   |html <b>
 *   stripe_ndx
 *   |html </b>
 *   - An integer identifying the stripe to remove channels from. In the case
 *   of non-striped transfer this parameter will be ignored.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_remove_channels returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_remove_channels $handle $num_channels
 *   $stripe_ndx ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_data_get_total_data_channels
 * NAME
 *   globus_ftp_control_data_get_total_data_channels
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_get_total_data_channels
 *   |html </b>
 *   |html <i>
 *   handle
 *   stripe_ndx
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to return the total number of data channels
 *   used so far in the current transfer on the given stripe.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A pointer to a FTP control handle. This handle is used to determine
 *   the host corresponding to the stripe number and to store information
 *   about any channels added by this command.
 *
 *   |html <b>
 *   stripe_ndx
 *   |html </b>
 *   - An integer identifying the stripe to return the number of channels.
 *   In the case of non-striped transfer this parameter will be ignored.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_get_total_data_channels returns 0
 *   and the number of channels.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_get_total_data_channels $handle
 *   $stripe_ndx ] result num_channels
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_data_send_eof
 * NAME
 *   globus_ftp_control_data_send_eof
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_send_eof
 *   |html </b>
 *   |html <i>
 *   handle
 *   count
 *   array_size
 *   eof_message
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to send an EOF message to each stripe along
 *   an open data connection.
 *
 *   An EOF message must be sent to all receiving hosts in a transfer.
 *   The message contains the total number of data connections used by
 *   each stripe. Many stripes may be sending to a single receiver but
 *   only one eof message may be sent. The count parameter allows the
 *   user to pass in the total number of data connections used by all
 *   other hosts. The local values are added to the passed in values
 *   and then sent to the receiver.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A FTP control handle containing the connection state.
 *
 *   |html <b>
 *   count
 *   |html </b>
 *   - This array of integers should contain an integer that will be
 *   added to the current parallel data connection count on each stripe.
 *   The order of the integers corresponds to each stripe in the same
 *   order as what was returned from local_port().
 *
 *   |html <b>
 *   array_size
 *   |html </b>
 *   - The number of elements in count.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked when the EOF message has been called.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_send_eof returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_send_eof $handle [ list 1 2 ] 2 
 *   { data_send_eof_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_local_send_eof
 * NAME
 *   globus_ftp_control_local_send_eof
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_send_eof
 *   |html </b>
 *   |html <i>
 *   handle
 *   send_eof
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to determine if the library will automatically
 *   send an EOF message in extended block mode, or if the user will have
 *   to explicitly do it by calling globus_ftp_control_data_send_eof.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP handle you wish to sent the send_eof attribute on.
 *
 *   |html <b>
 *   send_eof
 *   |html </b>
 *   - A boolean representing whether or not to automatically send an
 *   EOF message.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_send_eof returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_send_eof $handle $GLOBUS_TRUE ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_get_stripe_count
 * NAME
 *   globus_ftp_control_get_stripe_count
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_get_stripe_count
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to query stripe counts.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP handle you wish to query for stripe counts.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_get_stripe_count returns 0 and
 *   stripe_count.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_get_stripe_count $handle ]
 *   result stripe_count
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_get_parallelism
 * NAME
 *   globus_ftp_control_get_parallelism
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_get_parallelism
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to query the parallelism information in a
 *   FTP control handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle for which the parallelism information
 *   is to be queried.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_get_parallelism returns 0 and
 *   parallelism information.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_get_parallelism $handle ]
 *   result parallelism
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_local_parallelism.
 *****/


/****f* globus_ftp_control/globus_ftp_control_local_parallelism
 * NAME
 *   globus_ftp_control_local_parallelism
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_parallelism
 *   |html </b>
 *   |html <i>
 *   handle
 *   parallelism
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to set the parallelism information in a
 *   FTP control handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle for which the parallelism information
 *   is to be updated.
 *
 *   |html <b>
 *   parallelism
 *   |html </b>
 *   - A structure containing parallelism information.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_parallelism returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_parallelism $handle $parallelism ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_get_parallelism.
 *****/


/****f* globus_ftp_control/globus_ftp_control_data_get_remote_hosts
 * NAME
 *   globus_ftp_control_data_get_remote_hosts
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_get_remote_hosts
 *   |html </b>
 *   |html <i>
 *   handle
 *   address
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to query the number of remote hosts.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be queried.
 *
 *   |html <b>
 *   address
 *   |html </b>
 *   - The name of the remote address.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_get_remote_hosts returns 0 and
 *   the number of remote address.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_get_remote_hosts $handle $address ]
 *   result addr_count
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_local_pasv
 * NAME
 *   globus_ftp_control_local_pasv
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_pasv
 *   |html </b>
 *   |html <i>
 *   handle
 *   address
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to create a local listening socket, bind it and
 *   return the address the socket is listening to.
 *
 *   If there is a existing data connection it is closed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to store the listening socket information.
 *
 *   |html <b>
 *   address
 *   |html </b>
 *   - The host IP address and port is returned through this parameter.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_pasv returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_pasv $handle $address ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_local_spas
 * NAME
 *   globus_ftp_control_local_spas
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_spas
 *   |html </b>
 *   |html <i>
 *   handle
 *   addresses
 *   num_addresses
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to create local listening sockets, bind them and
 *   return the addresses the sockets are listening to.
 *
 *   If there is a existing data connection it is closed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to store the listening socket information.
 *
 *   |html <b>
 *   addresses
 *   |html </b>
 *   - The host IP address and port is returned through this parameter.
 *   |html|
 *
 *   |html <b>
 *   num_addresses
 *   |html </b>
 *   - The number of listening sockets to create.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_spas returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_spas $handle $num_addresses
 *   $addresses ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_local_port
 * NAME
 *   globus_ftp_control_local_port
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_port
 *   |html </b>
 *   |html <i>
 *   handle
 *   address
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to insert the host/port information returned
 *   by a PASV on the remote host into the local FTP control handle.
 *
 *   If there is a existing data connection it is closed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle into which to insert the host/port information.
 *
 *   |html <b>
 *   address
 *   |html </b>
 *   - The host IP address and port.
 *   |html| </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_port returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_port $handle $addr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_local_spor
 * NAME
 *   globus_ftp_control_local_spor
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_spor
 *   |html </b>
 *   |html <i>
 *   handle
 *   addresses
 *   num_addresses
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to insert the host/port addresses returned by
 *   a SPAS on the remote host into the local FTP control handle.
 *
 *   If there are any outstanding data connections at this point,
 *   they are closed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle into which to insert the host/port addresses.
 *
 *   |html <b>
 *   addresses
 *   |html </b>
 *   - The host IP address and port numbers.
 *
 *   |html <b>
 *   num_addresses
 *   |html </b>
 *   - The number of addresses.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_spor returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_spor $handle $num_addresses
 *   $addresses ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_get_spor
 *****/



/****f* globus_ftp_control/globus_ftp_control_get_spor
 * NAME
 *   globus_ftp_control_get_spor
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_get_spor
 *   |html </b>
 *   |html <i>
 *   handle
 *   addresses
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to query the number of host/port addresses
 *   returned by a SPAS on the remote host into the local FTP control
 *   handle.
 *
 *   If there are any outstanding data connections at this point,
 *   they are closed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle into which to query the host/port addresses.
 *
 *   |html <b>
 *   addresses
 *   |html </b>
 *   - The host IP address and port numbers.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_get_spor returns 0 and number of addresses.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_get_spor $handle $addresses ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_local_spor
 *****/


/****f* globus_ftp_control/globus_ftp_control_local_type
 * NAME
 *   globus_ftp_control_local_type
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_type
 *   |html </b>
 *   |html <i>
 *   handle
 *   type
 *   form_code
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to update the FTP control handle with the
 *   given type information.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be updated.
 *
 *   |html <b>
 *   type
 *   |html </b>
 *   - The type of the data connection. Possible values are: ASCII,
 *   EBCDIC, IMAGE and LOCAL. Currently only ASCII and IMAGE types
 *   are supported.
 *
 *   |html <b>
 *   form_code
 *   |html </b>
 *   - The logical byte size parameter for the LOCAL type.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_type returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_type $handle
 *   $GLOBUS_FTP_CONTROL_TYPE_ASCII 1 ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_get_type
 *****/


/****f* globus_ftp_control/globus_ftp_control_get_type
 * NAME
 *   globus_ftp_control_get_type
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_get_type
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to query the type information from the FTP
 *   control handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be queried.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_get_type returns 0 and type.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_get_type $handle ] result type
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_local_type
 *****/



/****f* globus_ftp_control/globus_ftp_control_local_mode
 * NAME
 *   globus_ftp_control_local_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_mode
 *   |html </b>
 *   |html <i>
 *   handle
 *   mode
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to update the FTP control handle with the
 *   given mode information.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be updated.
 *
 *   |html <b>
 *   mode
 *   |html </b>
 *   - Specifies the mode of the data connection. Possible modes are
 *   STREAM, BLOCK, EXTENDED BLOCK and COMPRESSED. Out of these only
 *   STREAM and EXTENDED BLOCK are supported in this implementation.
 *   Also, EXTENDED BLOCK is only supported in combination with the
 *   IMAGE type.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_mode returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_mode $handle
 *   $GLOBUS_FTP_CONTROL_MODE_STREAM ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_get_mode
 *****/


/****f* globus_ftp_control/globus_ftp_control_get_mode
 * NAME
 *   globus_ftp_control_get_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_get_mode
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to query the mode information from the FTP
 *   control handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be queried.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_get_mode returns 0 and mode.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_get_mode $handle ] result mode
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_local_mode 
 *****/



/****f* globus_ftp_control/globus_ftp_control_local_tcp_buffer
 * NAME
 *   globus_ftp_control_local_tcp_buffer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_tcp_buffer
 *   |html </b>
 *   |html <i>
 *   handle
 *   tcp_buffer
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to update the FTP control handle with
 *   the given socket buffer size information.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be updated.
 *
 *   |html <b>
 *   tcp_buffer
 *   |html </b>
 *   - Specifies the size of the socket buffer in bytes.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_tcp_buffer returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   set tcpbuffer [ new_globus_ftp_control_tcpbuffer_t ]
 *
 *   globus_ftp_control_tcpbuffer_t_mode_set $tcpbuffer	$GLOBUS_FTP_CONTROL_TCPBUFFER_FIXED
 *
 *   lassign [ globus_ftp_control_local_tcp_buffer $handle
 *   $tcp_buffer ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_local_dcau
 * NAME
 *   globus_ftp_control_local_dcau
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_dcau
 *   |html </b>
 *   |html <i>
 *   handle
 *   dcau
 *   delegated_credential_handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to update the FTP control handle with
 *   the given data channel authentication information.
 *
 *   If authentication is set to GLOBUS_FTP_CONTROL_DCAU_NONE, then
 *   protection will also be disabled for this control handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be updated.
 *
 *   |html <b>
 *   dcau
 *   |html </b>
 *   - A parameter specifying the data channel authentication mode.
 *   Possible values are No Authentication, Self Authentication and
 *   Subject-name authentication.
 *
 *   |html <b>
 *   delegated_credential_handle
 *   |html </b>
 *   - A structure containing credential information.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_dcau returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_dcau $handle $dcau
 *   $gss_c_no_credential ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_get_dcau
 *****/


/****f* globus_ftp_control/globus_ftp_control_get_dcau
 * NAME
 *   globus_ftp_control_get_dcau
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_get_dcau
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to query the FTP control handle with
 *   the data channel authentication information.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be queried.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_get_dcau returns 0 and
 *   the data channel authentication information.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_get_dcau $handle ] result dcau
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_local_dcau
 *****/




/****f* globus_ftp_control/globus_ftp_control_local_pbsz
 * NAME
 *   globus_ftp_control_local_pbsz
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_pbsz
 *   |html </b>
 *   |html <i>
 *   handle
 *   bufsize
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to update the FTP control handle with
 *   the given protection buffer size information.
 *
 *   This command sets protection buffer size to be used by this handle.
 *   This value is used to determine how much data will be sent in each
 *   packet during a protected data transfer.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be updated.
 *
 *   |html <b>
 *   bufsize
 *   |html </b>
 *   - A parameter specifying the protection buffer size value.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_pbsz returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_pbsz $handle 256 ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_get_pbsz
 * NAME
 *   globus_ftp_control_get_pbsz
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_get_pbsz
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to query the FTP control handle for the
 *   protection buffer size information.
 *
 *   This command queries the handle to determine the protection buffer
 *   size which is used by this handle. This value is used to determine
 *   how much data will be sent in each packet during a protected data
 *   transfer.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be queried.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_get_pbsz returns 0 and bufsize.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_get_pbsz $handle ] result bufsize
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_local_prot
 * NAME
 *   globus_ftp_control_local_prot
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_prot
 *   |html </b>
 *   |html <i>
 *   handle
 *   protection
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to update the FTP control handle with
 *   the protection information on the structure of the data being
 *   sent on the data channel.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be updated.
 *
 *   |html <b>
 *   protection
 *   |html </b>
 *   - This parameter is used to pass the protection information.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_prot returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_prot $handle
 *   $GLOBUS_FTP_CONTROL_PROTECTION_CLEAR ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_get_prot
 *****/



/****f* globus_ftp_control/globus_ftp_control_get_prot
 * NAME
 *   globus_ftp_control_get_prot
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_get_prot
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to query the FTP control handle with
 *   the protection information on the structure of the data being
 *   sent on the data channel.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be queried.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_get_prot returns 0 and protection.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_get_prot $handle ] result protection
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_local_prot
 *****/


/****f* globus_ftp_control/globus_ftp_control_local_stru
 * NAME
 *   globus_ftp_control_local_stru
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_stru
 *   |html </b>
 *   |html <i>
 *   handle
 *   structure
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to update the FTP control handle with
 *   the information on the structure of the data being sent on
 *   the data channel.
 *
 *   This command updates the handle with the provided structure
 *   information. At this point the only structure type that is
 *   supported is the file type.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be updated.
 *
 *   |html <b>
 *   structure
 *   |html </b>
 *   - This parameter is used to pass the structure information.
 *   Possible values are file, record and page. Only the file type is
 *   supported
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_stru returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_local_stru $handle $GLOBUS_FTP_CONTROL_STRUCTURE_FILE ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_local_layout
 * NAME
 *   globus_ftp_control_local_layout
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_local_layout
 *   |html </b>
 *   |html <i>
 *   handle
 *   layout
 *   data_size
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to update the FTP control handle with
 *   the layout and the size of the data sent over the data channel.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle to be updated.
 *
 *   |html <b>
 *   layout
 *   |html </b>
 *   - A variable containing the layout information.
 *
 *   |html <b>
 *   data_size
 *   |html </b>
 *   - The size of the data that is going to be sent. This may be needed
 *   to interpret the layout information.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_local_layout returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   set layout [ new_globus_ftp_control_layout_t ]
 *
 *   lassign [ globus_ftp_control_local_layout $handle $layout 1028 ] result
 *    
 *   delete_globus_ftp_control_layout_t $layout
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_data_write
 * NAME
 *   globus_ftp_control_data_write
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_write
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   offset
 *   eof
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to write data from the supplied buffer to
 *   data connection(s).
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle contains information about the current
 *   state of the control and data connections.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - A user supplied buffer from which data will written to the data
 *   connection(s).
 *
 *   |html <b>
 *   offset
 *   |html </b>
 *   - The offset in the file at which the data in the buffer starts.
 *
 *   |html <b>
 *   eof
 *   |html </b>
 *   - Indicates that the buffer is that last part of a file. In the
 *   striped case this will cause a EOF block to be send to every data
 *   node involved in the transfer.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the data has been sent. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_write returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_connect $handle $buffer $offset $eof
 *   { data_write_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_data_read
 * NAME
 *   globus_ftp_control_data_read
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_read
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer_length
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to read data from data connection(s) and
 *   put them in the supplied buffer.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle contains information about the current
 *   state of the control and data connections.
 *
 *   |html <b>
 *   buffer_length
 *   |html </b>
 *   - The maximum amount of data that can be stored into the buffer.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the data has been read. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_read returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_read $handle $buffer_length 
 *   { data_read_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_control/globus_ftp_control_data_read_all
 * NAME
 *   globus_ftp_control_data_read_all
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_read_all
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer_length
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to read data from data connection(s) and
 *   put them in the supplied buffer.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle contains information about the current
 *   state of the control and data connections.
 *
 *   |html <b>
 *   buffer_length
 *   |html </b>
 *   - The maximum amount of data that can be stored into the buffer.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the data has been read. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_read_all returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_read_all $handle $buffer_length 
 *   { data_read_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_create_data_info
 * NAME
 *   globus_ftp_control_create_data_info
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_create_data_info
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   length
 *   offset
 *   eof
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to create a globus_ftp_control_data_write_info_t
 *   structure.
 *
 *   This command populates a globus_ftp_control_data_callback_t structure
 *   with valid information. This structure provides the user a way to
 *   register several data writes with a single callback. This is quite
 *   useful to the writter of enqueue functions. It allows a single call
 *   to globus_ftp_control_data_write to be broken up into many writes,
 *   potentially on different stripes, and for a single callback to be
 *   called when all are finished.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle contains information about the current
 *   state of the control and data connections.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - The pointer to the user buffer that will be passed to the callback
 *   argument when there are zero references to data_info. This is intended
 *   to be the start of all the data the user intends to write using
 *   globus_ftp_control_data_write_stripe(), but it does not have to be.
 *
 *   |html <b>
 *   length
 *   |html </b>
 *   - The length of the memory segment pointed to by the argument buffer.
 *
 *   |html <b>
 *   offset
 *   |html </b>
 *   - The file offset of the data segment specified.
 *
 *   |html <b>
 *   eof
 *   |html </b>
 *   - This should be set to true if the user plans on registering
 *   EOF on the data_info structure.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked when all references to data_info are
 *   released. This occurs after all data registered for write from
 *   globus_ftp_control_data_write_stripe have occured and the user
 *   calls globus_ftp_control_release_data_info(). The callback is
 *   passed all of the arguments passed to this function with the
 *   exception of data_info.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_create_data_info returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_create_data_info $handle $buffer $length
 *   $offset $eof { create_data_info_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_release_data_info
 * NAME
 *   globus_ftp_control_release_data_info
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_release_data_info
 *   |html </b>
 *   |html <i>
 *   handle
 *   data_info
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to release all memory and references created
 *   when a call to globus_ftp_control_create_data_info was made.
 *   For every call to globus_ftp_control_create_data_info a call to
 *   this command must be made.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle contains information about the current
 *   state of the control and data connections.
 *
 *   |html <b>
 *   data_info
 *   |html </b>
 *   - The globus_ftp_control_data_write_info_t structure to be released.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_release_data_info returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_release_data_info $handle $data_info ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_ftp_control_data_write_stripe
 * NAME
 *   globus_ftp_control_data_write_stripe
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_control_data_write_stripe
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   offset
 *   eof
 *   stripe_ndx
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to write FTP data to a particular stripe.
 *
 *   This command allows the user to write to a specified stripe.
 *   The stripe index relates to the order passsed into local_spor().
 *   This command differs from globus_ftp_control_data_write in that no
 *   enqueue function is needed since the user specifies the stripe on
 *   which data is written. In order to use this command the user must
 *   have a valid pointer to a globus_ftp_control_data_write_info_t structure.
 *   The data_info structure can be obtained by a call to
 *   globus_ftp_control_create_data_info. Many calls to this command can
 *   be made, but only a single user callback occurs per creation of
 *   a globus_ftp_control_data_write_info_t structure.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle contains information about the current
 *   state of the control and data connections.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - A pointer to the data the user wishes to send along the FTP
 *   data channels.
 *
 *   |html <b>
 *   offset
 *   |html </b>
 *   - The offset into the file of the data.
 *
 *   |html <b>
 *   eof
 *   |html </b>
 *   - A boolean stating that this will be the last chuck of data
 *   registered on the given stripe. In order to properly send an
 *   EOF message the user must register an EOF on every stripe.
 *
 *   |html <b>
 *   stripe_ndx
 *   |html </b>
 *   - The index of the stripe on which the data will be sent.
 *   The index of each stripe is determined by the call to local_spas
 *   or local_spor.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked when all references to data_info are
 *   released. This occurs after all data registered for write from
 *   globus_ftp_control_data_write_stripe have occured and the user
 *   calls globus_ftp_control_release_data_info(). The callback is
 *   passed all of the arguments passed to this function with the
 *   exception of data_info.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_control_data_write_stripe returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_control_data_write_stripe $handle $buffer 
 *   $offset $eof $stripe_ndx { data_write_stripe_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_control/globus_X_ftp_control_data_write_stripe
 * NAME
 *   globus_X_ftp_control_data_write_stripe
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_control
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_X_ftp_control_data_write_stripe
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   length
 *   offset
 *   eof
 *   stripe_ndx
 *   data_info
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to write FTP data to a particular stripe
 *   from an enqueue callback function only.
 *
 *   This command allows the user to register the write of FTP data
 *   on a specfic stripe. This function can only be called from an
 *   enqueue function callback. This command should be used only by
 *   the implementor of an enqueue function. It should be viewed as
 *   unstable and used used only by advanced users. This is the only
 *   command in the library that the enqueue function implementor is
 *   allowed from the enqueue callback.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP control handle contains information about the current
 *   state of the control and data connections.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - A pointer to the data the user wishes to send along the FTP
 *   data channels.
 *
 *   |html <b>
 *   length
 *   |html </b>
 *   - The length of the data pointer to by the parameter buffer.
 *
 *   |html <b>
 *   offset
 *   |html </b>
 *   - The offset into the file of the data.
 *
 *   |html <b>
 *   eof
 *   |html </b>
 *   - A boolean stating that this will be the last chuck of data
 *   registered on the given stripe. In order to properly send an
 *   EOF message the user must register an EOF on every stripe.
 *
 *   |html <b>
 *   stripe_ndx
 *   |html </b>
 *   - The index of the stripe on which the data will be sent.
 *   The index of each stripe is determined by the call to local_spas
 *   or local_spor.
 *
 *   |html <b>
 *   data_info
 *   |html </b>
 *   - An opaque structure that is passed into the enqueue function
 *   and contains reference count and state information. The same
 *   data_info pointer that is passed into the enqueue function must
 *   be used for this parameter.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_X_ftp_control_data_write_stripe returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_X_ftp_control_data_write_stripe $handle $buffer $length
 *   $offset $eof $stripe_ndx $data_info ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



