/* -*- mode: c; basic-c-offset: 2; -*- */

#ifdef SWIG

#include "../../config.h"
  
/*****************************************************************
 * Hand-wrapped SWIG functions
 *****************************************************************/

%ignore globus_gass_cache_open;
%ignore globus_gass_cache_close;

%include "tcl-typemaps.i" 

%include "swig/globus_gass_cache.i"

%{
  
#include <tcl.h>
#include "tcl-callback.h"
#include "common/sgdebug.h"  

%}


/**************************************************************************
 **************************************************************************
 *				TYPEMAPS SECTION
 **************************************************************************
 **************************************************************************/

/**************************************************************************
 * GLOBUS_GASS_CACHE_OPEN
 **************************************************************************/
%typemap(in,numinputs=0) globus_gass_cache_t * cache_handle($*1_ltype temp)
{
  $1 = (globus_gass_cache_t *)&temp;
}

%typemap(argout,fragment="t_output_helper")
     globus_gass_cache_t * cache_handle 
{
  Tcl_Obj* o = SWIG_NewInstanceObj((void *)(*$1), $*1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_gass_cache_open) tcl_globus_gass_cache_open;

%inline %{
  int
    tcl_globus_gass_cache_open
    ( const char *		cache_directory_path,
      globus_gass_cache_t *	cache_handle )
    {
      int retval;

      retval = globus_gass_cache_open( cache_directory_path,
				       cache_handle );

      return retval;
    }
  %}


/**************************************************************************
 * GLOBUS_GASS_CACHE_CLOSE
 **************************************************************************/
%rename(globus_gass_cache_close) tcl_globus_gass_cache_close;

%inline %{

  int
    tcl_globus_gass_cache_close( globus_gass_cache_t cache_handle )
    {
      int retval;

      SGDEBUG_LVL_FMT_1( 10, "globus_gass_cache_close: %p", &cache_handle );
      
      retval = globus_gass_cache_close(&cache_handle);

      SGDEBUG_LVL_FMT_1
	(10,
	 "globus_gass_cache_close: retval: %d", retval);
      
      return retval;
    }
  
  %}

#endif /* SWIG */


/**************************************************************************
 *
 * ROBODOC
 *
 **************************************************************************/

/****f* globus_gass_cache/globus_gass_cache_open
 * NAME
 *   globus_gass_cache_open
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_open
 *   |html </b>
 *   |html <i>
 *   cache_directory_path
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to open the cache specified by the
 *   cache_directory_path argument, and return a cache handle that can be
 *   used in subsequent cache calls.
 *
 *   If cache_directory_path is NULL, then use the value contained in
 *   the GLOBUS_GASS_CACHE_DEFAULT environment variable if it is defined,
 *   otherwise use ~/.gass_cache.
 *
 *   The cache_directory_path must be a directory. If it is a file, this
 *   call will fail with a non-0 return value.
 *
 *   If the specified directory does not exist, then this call will create
 *   the directory.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_directory_path
 *   |html </b>
 *   - Path to the cache directory to open. Can be NULL.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_open returns 0 and cache_handle. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_open "/tmp/test_gass_cache" ] result
 *   cache_handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_cache_close
 *****/




/****f* globus_gass_cache/globus_gass_cache_close
 * NAME
 *   globus_gass_cache_close
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_close
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to close (NOT delete) a previously opened cache.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - The handle to be closed.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_close returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_close $cache_handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_cache_open
 *****/



/****f* globus_gass_cache/globus_gass_cache_add
 * NAME
 *   globus_gass_cache_add
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_add
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   url
 *   tag
 *   create
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create a new cache file or add a tag on it.
 *
 *   If the URL is already in the cache but is locked, then then this
 *   call will block until the cache entry is unlocked, then will proceed
 *   with the subsequent operations.
 * 
 *   If the URL is already in the cache and unlocked, then add the tag
 *   to the cache entry's tag list, return the local cache filename in
 *   local_filename, return the entry's current timestamp in timestamp,
 *   lock the cache entry, and return GLOBUS_GASS_CACHE_ADD_EXISTS.
 *
 *   If the URL is not in the cache, and create==GLOBUS_TRUE, then create
 *   a new unique empty local cache file, add it to the cache with the
 *   specified tag, return the filename in local_filename, return timestamp
 *   set to GLOBUS_GASS_TIMESTAMP_UNKNOWN, lock the cache entry, and return
 *   GLOBUS_GASS_CACHE_ADD_NEW.
 *
 *   If the URL is not in the cache, and create==GLOBUS_FALSE, then do not
 *   add it to the cache, and return GLOBUS_GASS_CACHE_URL_NOT_FOUND.
 *
 *   If this function returns GLOBUS_GASS_CACHE_ADD_EXISTS or
 *   GLOBUS_GASS_CACHE_ADD_NEW, then globus_gass_cache_add_done or
 *   globus_gass_cache_delete() must be subsequently called to unlock the
 *   cache entry.
 *
 *   Subsequent calls to globus_gass_cache_add and
 *   globus_gass_cache_delete_start on the same cache and url, made either
 *   from this process or another, will block until the cache entry is
 *   unlocked.
 *
 *   If tag==NULL, then a tag with the value "null" will be added to the
 *   cache entry's tag list.
 *
 *   The same tag can be used multiple times, in which case this tag will
 *   be added to the entry's tag list multiple times.
 *
 *   The returned local_filename should be subsequently freed by calling
 *   globus_free(local_filename).
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL of the file to be cached. It is used as the main key to the
 *   cache entries.
 *
 *   |html <b>
 *   tag 
 *   |html </b>
 *   - Tag specifying which job is/are using the cache. This
 *   is usually the GLOBUS_GRAM_JOB_CONTACT. Can be NULL or empty; the
 *   tag "null" is then used.
 *
 *   |html <b>
 *   create
 *   |html </b>
 *   - Tells if the cache entry should be created if it is not already
 *   existing.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_add returns 0, timestamp and local filename. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_add $cache_handle $sourcefile $GLOBUS_NULL $GLOBUS_TRUE ] result timestamp local_filename
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_cache_add_done
 *****/



/****f* globus_gass_cache/globus_gass_cache_add_done
 * NAME
 *   globus_gass_cache_add_done
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_add_done
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   url
 *   tag
 *   timestamp
 *   |html </i>
 *
 * DESCRIPTION
 *   globus_gass_cache_add_done MUST be called after globus_gass_cache_add,
 *   to set the timestamp in the cache entry for the URL, and then unlock the
 *   cache entry. (The only case it does not need to be called is if
 *   globus_gass_cache_add has returned GLOBUS_GASS_CACHE_URL_NOT_FOUND, of
 *   course.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL of the cached file to set as "done" (unlock).
 *
 *   |html <b>
 *   tag 
 *   |html </b>
 *   - Tag specifying which job has locked the cache and must
 *   therfor be unlocked.It is an error to call this function
 *   with a tag which does not currently own the cache lock.
 *
 *   |html <b>
 *   timestamp
 *   |html </b>
 *   - Time stamp of the cached file.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_add_done returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_add_done $cache_handle $sourcefile $GLOBUS_NULL timestamp ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_cache_add
 *****/



/****f* globus_gass_cache/globus_gass_cache_query
 * NAME
 *   globus_gass_cache_query
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_query
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   url
 *   tag
 *   wait_for_lock
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query if an item is in the cache.
 *
 *   This call will block only if wait_for_lock is GLOBUS_TRUE.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL of the cached file to query. It is used as the main
 *   key to the cache entries.
 *
 *   |html <b>
 *   tag 
 *   |html </b>
 *   - Tag specifying which job has locked the cache and must
 *   therfor be unlocked.It is an error to call this function
 *   with a tag which does not currently own the cache lock.
 *
 *   |html <b>
 *   wait_for_lock
 *   |html </b>
 *   - If GLOBUS_TRUE, wait for any lock existing lock
 *   to be released.  If GLOBUS_FALSE, doesn't wait for a lock to be
 *   released.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_query returns 0, timestamp, local_filename
 *   and is_locked. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_query $cache_handle $sourcefile $GLOBUS_NULL $GLOBUS_TRUE ] result timestamp local_filename is_locked
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/




/****f* globus_gass_cache/globus_gass_cache_delete_start
 * NAME
 *   globus_gass_cache_delete_start
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_delete_start
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   url
 *   tag
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to lock the cache entry for the URL, and
 *   return the cache entry's current timestamp in timestamp.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL of the cached file to set as "done" (unlock).
 *
 *   |html <b>
 *   tag 
 *   |html </b>
 *   - Tag specifying which job has locked the cache and must
 *   therefore be unlocked.It is an error to call this function
 *   with a tag which does not currently own the cache lock.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_delete_start returns 0 and timestamp. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_delete_start $cache_handle $sourcefile $GLOBUS_NULL ] result timestamp
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_cache_delete
 *****/



/****f* globus_gass_cache/globus_gass_cache_delete
 * NAME
 *   globus_gass_cache_delete
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_delete
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   url
 *   tag
 *   timestamp
 *   is_locked
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to remove one instance of the tag from the
 *   cache entry's tag list.  If there are no more tags in the tag list,
 *   then remove this cache entry and delete the associated local cache
 *   file.  Otherwise, update the timestamp to the passed value.
 *
 *   If is_locked == GLOBUS_TRUE, then this cache entry was locked during
 *   a previous call to globus_gass_cache_add or
 *   globus_gass_cache_delete_start. Otherwise, the entry will be locked
 *   at the start of this operation.
 *
 *   This call will leave the cache entry unlocked.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL of the cached file to set as "done" (unlock).
 *
 *   |html <b>
 *   tag 
 *   |html </b>
 *   - Tag specifying which job is using the cache. It can be NULL of
 *   empty; the tag "null" is then used.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_delete returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_delete $cache_handle $sourcefile $GLOBUS_NULL $timestamp $GLOBUS_TRUE ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_cache_delete_start
 *****/




/****f* globus_gass_cache/globus_gass_cache_cleanup_tag
 * NAME
 *   globus_gass_cache_cleanup_tag
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_cleanup_tag
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   url
 *   tag
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to remove all instances of the tag from the
 *   cache entry's tag list. If there are no more tags in the tag list,
 *   then remove this cache entry and delete the associated local cache file.
 *
 *   If the cache entry is locked with the same tag as is passed to this
 *   function, then the entry is unlocked after removing the tags. Otherwise,
 *   the cache entry's lock is left untouched.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL of the cached file to set as "done" (unlock).
 *
 *   |html <b>
 *   tag 
 *   |html </b>
 *   - Tag specifying which job is using the cache. It can be NULL of
 *   empty; the tag "null" is then used.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_cleanup_tag returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_cleanup_tag $cache_handle $sourcefile $GLOBUS_NULL ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_cache_cleanup_tag_all
 *****/


/****f* globus_gass_cache/globus_gass_cache_cleanup_tag_all
 * NAME
 *   globus_gass_cache_cleanup_tag_all
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_cleanup_tag_all
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   tag
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to remove all instances of the tag from the
 *   cache entry's tag list. If there are no more tags in the tag list,
 *   then remove this cache entry and delete the associated local cache file.
 *
 *   If the cache entry is locked with the same tag as is passed to this
 *   function, then the entry is unlocked after removing the tags. Otherwise,
 *   the cache entry's lock is left untouched.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *
 *   |html <b>
 *   tag 
 *   |html </b>
 *   - Tag specifying which job is using the cache. It can be NULL of
 *   empty; the tag "null" is then used.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_cleanup_tag_all returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_cleanup_tag_all $cache_handle $GLOBUS_NULL ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_cache_cleanup_tag
 *****/



/****f* globus_gass_cache/globus_gass_cache_mangle_url
 * NAME
 *   globus_gass_cache_mangle_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_mangle_url
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to mangle the given URL into a chunk suitable for
 *   using as a file/path name.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *
 *   |html <b>
 *   url 
 *   |html </b>
 *   - The incoming URL to mangle (\0 terminated).
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_mangle_url returns 0, mangled_url and
 *   length. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_mangle_url $cache_handle $sourcefile ] result
 *   mangled_url length
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_cache_mangle_tag
 *****/



/****f* globus_gass_cache/globus_gass_cache_mangle_tag
 * NAME
 *   globus_gass_cache_mangle_tag
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_mangle_tag
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   tag
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to mangle the given tag into a chunk suitable for
 *   using as a file/path name.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *
 *   |html <b>
 *   tag 
 *   |html </b>
 *   - The incoming tag to mangle (\0 terminated).
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_mangle_tag returns 0, mangled_tag and
 *   length. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_mangle_tag $cache_handle $GLOBUS_NULL ] result
 *   mangled_tag length
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_cache_mangle_url
 *****/



/****f* globus_gass_cache/globus_gass_cache_get_dirs
 * NAME
 *   globus_gass_cache_get_dirs
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_get_dirs
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   url
 *   tag
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get a bunch of directories.
 *
 *   This is exported for use in the globus_gass_cache program.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *
 *   |html <b>
 *   url 
 *   |html </b>
 *   - The incoming URL.
 *
 *   |html <b>
 *   tag 
 *   |html </b>
 *   - The incoming tag.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_get_dirs returns 0, local_root,
 *   global_root, tmp_root, log_root, local_dir and global_dir. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_get_dirs $cache_handle $sourcefile $GLOBUS_NULL ] result global_root local_root tmp_root log_root global_dir local_dir
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_cache/globus_gass_cache_get_cache_dir
 * NAME
 *   globus_gass_cache_get_cache_dir
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_get_cache_dir
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get a the root cache of directory.
 *
 *   This is exported for use in the globus_gass_cache program.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_get_cache_dir returns 0 and cache_dir. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_get_cache_dir $cache_handle ] result cache_dir
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gass_cache/globus_gass_cache_get_cache_type_string
 * NAME
 *   globus_gass_cache_get_cache_type_string
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_cache
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_cache_get_cache_type_string
 *   |html </b>
 *   |html <i>
 *   cache_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get a string which describes the cache
 *   type ("normal" or "flat").
 *
 * PARAMETERS
 *   |html <ul><b>
 *   cache_handle
 *   |html </b>
 *   - Handler to be opened cache directory to use.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_cache_get_cache_type_string returns 0 and
 *   cache_type. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_cache_get_cache_type_string $cache_handle ]
 *   result cache_type
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/
