/* -*- mode: c; basic-c-offset: 2; -*- */
#ifdef SWIG

%include "../../config.h"

/*****************************************************************
 * Hand-wrapped SWIG functions
 *****************************************************************/
%ignore globus_gass_copy_register_url_to_url;
%ignore globus_gass_copy_register_url_to_handle;
%ignore globus_gass_copy_register_handle_to_url;
%ignore globus_gass_copy_register_performance_cb;
%ignore globus_gass_copy_cancel;
%ignore globus_gass_copy_glob_expand_url;

%include "tcl-typemaps.i"

%include "swig/globus_gass_copy.i"

%{
    
#include <stdio.h>
    
#include <tcl.h>
    
#include "tcl-callback.h"
    
#include "common/sgdebug.h"
#include "common/sginit.h"
  
/**************************************************************************
 * GLOBUS_GASS_COPY_REGISTER_PERFORMANCE_CB
 **************************************************************************/
  static void
    GASSCopyPerformanceCallback
    ( void *				user_arg,
      globus_gass_copy_handle_t *	handle,
      globus_off_t                      total_bytes,
      float				instantaneous_throughput,
      float                             avg_throughput )
    {
      CallCallback( (callbackStruct*)user_arg,
		    "%p %ld %f %f",
		    handle, total_bytes, instantaneous_throughput,
		    avg_throughput );   
    }
  
/**************************************************************************
 * GLOBUS_GASS_COPY_GLOB_EXPAND_URL
 **************************************************************************/
  static void
    GASSCopyGlobExpandUrlCallback
    ( void *					user_arg,
      const char *				url,
      const globus_gass_copy_glob_stat_t *	info_stat )
    {
      CallCallback( (callbackStruct*)user_arg,
                    "%s %p",
                    url, info_stat );
    }
  
/**************************************************************************
 * GLOBUS_GASS_COPY_REGISTER_URL_TO_URL
 * GLOBUS_GASS_COPY_REGISTER_URL_TO_HANDLE
 * GLOBUS_GASS_COPY_REGISTER_HANDLE_TO_URL
 **************************************************************************/
  static void
    GASSCopyCompleteCallback
    ( void * 				callback_arg, 
      globus_gass_copy_handle_t * 	handle,
      globus_object_t *			error )
    {
      char 	str_handle[128];
      char*	msg;
      
      if ( error )
        {
          msg = globus_object_printable_to_string( error );
	  SGDEBUG_LVL_FMT_1( 10, "GASSCopyCompleteCallback: msg: %s", msg );
        }
      else
        {
          msg = "";
        }

#if 0
      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_gass_copy_handle_t" ), 0 );
      SGDEBUG_LVL_FMT_1( 10, "GASSCopyCompleteCallback: str_handle = %s", 
			 str_handle );
#endif /*0*/

      SGDEBUG_LVL_FMT_1( 10, "GASSCopyCompleteCallback: handle = %d", 
			 handle );
      
      CallCallback( (callbackStruct*)callback_arg,
                    "%d \"%s\"",
                    handle, msg );
      
      if ( error )
      {
	globus_libc_free(msg);
      }
    }
%}

/**************************************************************************
 **************************************************************************
 *				TYPEMAPS SECTION
 **************************************************************************
 **************************************************************************/
%typemap(in) 
     (globus_gass_copy_callback_t callback_func, void * callback_arg)
     (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );
  
  $1 = GASSCopyCompleteCallback;
  $2 = cbStruct;
}

/**************************************************************************
 *  GLOBUS_GASS_COPY_REGISTER_URL_TO_URL
 **************************************************************************/
%rename(globus_gass_copy_register_url_to_url) tcl_globus_gass_copy_register_url_to_url;

%exception tcl_globus_gass_copy_register_url_to_url GLOBUS_EXCEPTION_RESULT_T;     
%inline %{

  globus_result_t
    tcl_globus_gass_copy_register_url_to_url
    ( globus_gass_copy_handle_t * 	handle,
      char * 				source_url,
      globus_gass_copy_attr_t * 	dest_attr,
      char * 				dest_url,
      globus_gass_copy_attr_t * 	source_attr,
      globus_gass_copy_callback_t 	callback_func,
      void * 				callback_arg )
    {
      globus_result_t retval;

      retval = globus_gass_copy_register_url_to_url
	( handle,
	  source_url,
	  dest_attr,
	  dest_url,
	  source_attr,
	  callback_func,
	  callback_arg );

      return retval;
    }
  
  %}

/**************************************************************************
 *  GLOBUS_GASS_COPY_REGISTER_URL_TO_HANDLE
 **************************************************************************/
%rename(globus_gass_copy_register_url_to_handle) tcl_globus_gass_copy_register_url_to_handle;

%exception tcl_globus_gass_copy_register_url_to_handle GLOBUS_EXCEPTION_RESULT_T;  

%inline %{

  globus_result_t
    tcl_globus_gass_copy_register_url_to_handle
    ( globus_gass_copy_handle_t * 	handle,
      char * 				source_url,
      globus_gass_copy_attr_t * 	source_attr,
      globus_io_handle_t *		dest_handle,
      globus_gass_copy_callback_t 	callback_func,
      void * 				callback_arg )
    {
      globus_result_t retval;

      retval = globus_gass_copy_register_url_to_handle
	( handle,
	  source_url,
	  source_attr,
	  dest_handle,
	  callback_func,
	  callback_arg );

      return retval;
    }
  
  %}

/**************************************************************************
 *  GLOBUS_GASS_COPY_REGISTER_HANDLE_TO_URL
 **************************************************************************/
%rename(globus_gass_copy_register_handle_to_url) tcl_globus_gass_copy_register_handle_to_url;

%exception tcl_globus_gass_copy_register_handle_to_url GLOBUS_EXCEPTION_RESULT_T;  

%inline %{

  globus_result_t
    tcl_globus_gass_copy_register_handle_to_url
    ( globus_gass_copy_handle_t * 	handle,
      globus_io_handle_t *		source_handle,
      char * 				dest_url,
      globus_gass_copy_attr_t * 	dest_attr,
      globus_gass_copy_callback_t 	callback_func,
      void * 				callback_arg )
    {
      globus_result_t retval;

      retval = globus_gass_copy_register_handle_to_url
	( handle,
	  source_handle,
	  dest_url,
	  dest_attr,
	  callback_func,
	  callback_arg );

      return retval;
    }
  
  %}

/**************************************************************************
 *  GLOBUS_GASS_COPY_CANCEL
 **************************************************************************/
%rename(globus_gass_copy_cancel) tcl_globus_gass_copy_cancel;

%exception tcl_globus_gass_copy_cancel GLOBUS_EXCEPTION_RESULT_T;  

%inline %{
  globus_result_t
    tcl_globus_gass_copy_cancel
    ( globus_gass_copy_handle_t * 	handle,
      globus_gass_copy_callback_t 	callback_func,
      void * 				callback_arg )
    {
      globus_result_t retval;

      retval = globus_gass_copy_cancel
	( handle,
	  callback_func,
	  callback_arg );

      return retval;
    }
  
  %}


/**************************************************************************
 *
 *  SWIG INITIALIZATION STUFF
 *
 **************************************************************************/
%init %{

  SGINIT_X509;

  %}



/**************************************************************************
 * GLOBUS_GASS_COPY_REGISTER_PERFORMANCE_CB 
 **************************************************************************/
%typemap(in) 
     (globus_gass_copy_performance_cb_t callback, void * user_arg)
     (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );
  
  $1 = GASSCopyPerformanceCallback;
  $2 = cbStruct;
}

%rename(globus_gass_copy_register_performance_cb) tcl_globus_gass_copy_register_performance_cb;

%exception tcl_globus_gass_copy_register_performance_cb GLOBUS_EXCEPTION_RESULT_T;  

%inline %{
  globus_result_t
    tcl_globus_gass_copy_register_performance_cb
    ( globus_gass_copy_handle_t *         handle,
      globus_gass_copy_performance_cb_t   callback,
      void *                              user_arg )
    {
      globus_result_t retval;

      retval = globus_gass_copy_register_performance_cb
	( handle,
	  callback,
	  user_arg );
      
      return retval;
    }
  %}

/**************************************************************************
 *  GLOBUS_GASS_COPY_GLOB_EXPAND_URL
 **************************************************************************/
%typemap(in) 
     (globus_gass_copy_glob_entry_cb_t entry_cb, void * user_arg)
     (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );
  
  $1 = (globus_gass_copy_glob_entry_cb_t)GASSCopyGlobExpandUrlCallback;
  $2 = cbStruct;
}

%rename(globus_gass_copy_glob_expand_url) tcl_globus_gass_copy_glob_expand_url;

%exception tcl_globus_gass_copy_glob_expand_url GLOBUS_EXCEPTION_RESULT_T;  

%inline %{
  globus_result_t
    tcl_globus_gass_copy_glob_expand_url
    ( globus_gass_copy_handle_t * 		handle,
      const char *				url,
      globus_gass_copy_attr_t *			attr,
      globus_gass_copy_glob_entry_cb_t		entry_cb,
      void *					user_arg )
    {
      globus_result_t retval;

      retval = globus_gass_copy_glob_expand_url
	( handle,
	  url,
	  attr,
	  entry_cb,
	  user_arg );
      
      return retval;
    }
  %}

#endif /* SWIG */



/****f* globus_gass_copy/globus_gass_copy_handle_init
 * NAME
 *   globus_gass_copy_handle_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_handle_init
 *   |html </b>
 *   |html <i>
 *   handle
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a GASS Copy handle.
 *
 *   A globus_gass_copy_handle must be initialized before any transfers
 *   may be associated with it. This function initializes a
 *   globus_gass_copy_handle to be used for doing transfers, this includes
 *   initializing a globus_ftp_client_handle which will be used for doing
 *   any FTP/GSIFTP transfers. The same handle may be used to perform
 *   multiple, consecutive transfers. However, there can only be one transfer
 *   associated with a particular handle at any given time. After all
 *   transfers to be associated with this handle have completed, the handle
 *   should be destroyed by calling globus_gass_copy_handle_destroy.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to be initialized.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Initial attributes to be used to create this handle. This can be
 *   NULL.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_handle_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_handle_init $handle NULL ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_handle_destroy, globus_gass_copy_handleattr_init,
 *   globus_ftp_client_handle_init
 *****/


/****f* globus_gass_copy/globus_gass_copy_handle_destroy
 * NAME
 *   globus_gass_copy_handle_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_handle_destroy
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a GASS Copy handle, which was
 *   initialized using globus_gass_copy_handle_init, that will no longer
 *   be used for doing transfers. Once the handle is detroyed, no further
 *   transfers should be associated with it.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to be destroyed.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_handle_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_handle_destroy $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_handle_init, globus_ftp_client_handle_destroy
 *****/


/****f* globus_gass_copy/globus_gass_copy_set_buffer_length
 * NAME
 *   globus_gass_copy_set_buffer_length
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_set_buffer_length
 *   |html </b>
 *   |html <i>
 *   handle
 *   length
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the size of the buffer to be used 
 *   for doing transfers.
 *
 *   This command allows the user to set the size of the buffer that 
 *   will be used for doing transfers, if this command is not called 
 *   the buffer size will default to 1M.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Set the buffer length for transfers associated with this handle.
 *
 *   |html <b>
 *   length
 *   |html </b>
 *   - The length, in bytes, to make the buffer.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success, globus_gass_copy_set_buffer_length returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_set_buffer_length $handle 4096 ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_handle_init, globus_gass_copy_handle_destroy,
 *   globus_gass_copy_get_buffer_length
 *****/


/****f* globus_gass_copy/globus_gass_copy_get_buffer_length
 * NAME
 *   globus_gass_copy_get_buffer_length
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_get_buffer_length
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the size of the buffer being used 
 *   for doing transfers.
 *
 *   This command allows the user to get the size of the buffer that 
 *   is being used for doing transfers.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Get the buffer length for transfers associated with this handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success, globus_gass_copy_get_buffer_length returns 0 and the 
 *   length, in bytes, of the buffer. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_get_buffer_length $handle ] result length
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_handle_init, globus_gass_copy_handle_destroy,
 *   globus_gass_copy_set_buffer_length
 *****/


/****f* globus_gass_copy/globus_gass_copy_set_no_third_party_transfers
 * NAME
 *   globus_gass_copy_set_no_third_party_transfers
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_set_no_third_party_transfers
 *   |html </b>
 *   |html <i>
 *   handle
 *   no_third_party_transfers
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to turn third-party transfers on or off.
 *   (They are on by default.)
 *
 *   This command allows the user to turn third-party transfers 
 *   on or off for ftp to ftp transfers associated with a particular 
 *   handle. This is often desired if one of the servers involved 
 *   in the transfer does not allow third-party transfers.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Turn third-party transfers on or off for transfers associated 
 *   with this handle. They are on by default.
 *
 *   |html <b>
 *   no_third_party_transfers
 *   |html </b>
 *   - GLOBUS_FALSE if third-party transfers should be used. GLOBUS_TRUE if 
 *   third-party transfers should not be used.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success, globus_gass_copy_set_no_third_party_transfers returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_set_no_third_party_transfers
 *   $handle $GLOBUS_FALSE ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_handle_init, globus_gass_copy_handle_destroy,
 *   globus_gass_copy_get_no_third_party_transfers
 *****/


/****f* globus_gass_copy/globus_gass_copy_get_no_third_party_transfers
 * NAME
 *   globus_gass_copy_get_no_third_party_transfers
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_get_no_third_party_transfers
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query if third-party transfers are turned
 *   on or off.
 *
 *   This command allows the user to see if third-party transfers are
 *   turned on or off for ftp to ftp transfers associated with a
 *   particular handle. This is often desired if one of the servers
 *   involved in the transfer does not allow third-party transfers.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - See if third-party transfers are turned on or off for transfers
 *   associated with this handle. They are on by default. 
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success, globus_gass_copy_get_no_third_party_transfers returns 0
 *   and third-party transfers flag. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_get_no_third_party_transfers $handle ]
 *   result no_third_party_transfers
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_handle_init, globus_gass_copy_handle_destroy,
 *   globus_gass_copy_set_no_third_party_transfers
 *****/


/****f* globus_gass_copy/globus_gass_copy_set_partial_offsets
 * NAME
 *   globus_gass_copy_set_partial_offsets
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_set_partial_offsets
 *   |html </b>
 *   |html <i>
 *   handle
 *   start_offset
 *   end_offset
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the offsets to be used for doing
 *   partial transfers.
 *
 *   This command allows the user to set the offsets that will be
 *   used for doing partial transfers. An offset of -1 will disable
 *   partial transfers. An end_offset of -1 means EOF.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Set the offsets for partial transfers associated with this handle.
 *
 *   |html <b>
 *   start_offset
 *   |html </b>
 *   - The starting offset for the partial transfer.
 *
 *   |html <b>
 *   end_offset
 *   |html </b>
 *   - The ending offset for the partial transfer.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success, globus_gass_copy_set_partial_offsets returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_set_partial_offsets $handle 512 1024 ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_handle_init, globus_gass_copy_handle_destroy,
 *   globus_gass_copy_get_partial_offsets
 *****/


/****f* globus_gass_copy/globus_gass_copy_get_partial_offsets
 * NAME
 *   globus_gass_copy_get_partial_offsets
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_get_partial_offsets
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the offsets being used for doing
 *   partial transfers.
 *
 *   This command allows the user to get the offsets that are being
 *   used for doing partial transfers. An offset of -1 means partial
 *   transfers are disabled.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Get the offsets for partial transfers associated with this handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success, globus_gass_copy_get_partial_offsets returns 0, the
 *   starting and ending offsets. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_get_partial_offsets $handle ] result
 *   starting_offset ending_offset
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_handle_init, globus_gass_copy_handle_destroy,
 *   globus_gass_copy_set_partial_offsets
 *****/


/****f* globus_gass_copy/globus_gass_copy_attr_init
 * NAME
 *   globus_gass_copy_attr_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_attr_init
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize an attribute structure.
 *
 *   The command can be used to pass the globus_gass_copy library
 *   information about how a transfer should be performed. It must
 *   first be initialized by calling this command. Then any or all
 *   of the following functions may be called to set attributes
 *   associated with a particular protocol: globus_gass_copy_attr_set_ftp,
 *   globus_gass_copy_attr_set_gass, globus_gass_copy_attr_set_io.
 *   Any comamnd which takes attr as an argument will also accept NULL,
 *   in which case the appropriate set of default attributes will be used.
 *
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute structure to be initialized.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_attr_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_attr_init $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_attr_set_ftp, globus_gass_copy_attr_set_gass,
 *   globus_gass_copy_attr_set_io, globus_gass_copy_get_url_mode
 *****/


/****f* globus_gass_copy/globus_gass_copy_attr_set_ftp
 * NAME
 *   globus_gass_copy_set_ftp
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_set_ftp
 *   |html </b>
 *   |html <i>
 *   globus_gass_copy_attr
 *   ftp_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the attributes for ftp/gsiftp transfers.
 *
 *   In order to specify attributes for ftp/gsiftp transfers,
 *   ftp_attr should be initialized and its values set using the
 *   appropriate globus_ftp_client_operationattr_* commands.
 *   The globus_ftp_client_operationattr_t * can then be passed to the
 *   globus_gass_copy_attr via this command.
 *
 *   |html <ul><b>
 *   globus_gass_copy_attr
 *   |html </b>
 *   - A globus_gass_copy attribute structure.
 *
 *   |html <b>
 *   ftp_attr
 *   |html </b>
 *   - The ftp/gsiftp attributes to be used.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_set_ftp returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_set_ftp $globus_gass_copy_attr
 *   $ftp_attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_attr_init, globus_gass_copy_attr_set_gass,
 *   globus_gass_copy_attr_set_io, globus_gass_copy_get_url_mode
 *****/


/****f* globus_gass_copy/globus_gass_copy_attr_set_io
 * NAME
 *   globus_gass_copy_set_io
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_set_io
 *   |html </b>
 *   |html <i>
 *   globus_gass_copy_attr
 *   io_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the attributes for file transfers.
 *
 *   In order to specify attributes for file transfers,
 *   io_attr should be initialized and its values set using the
 *   appropriate globus_io_attr_* command. The io_attr can then be
 *   passed to the globus_gass_copy_attr
 *   via this command.
 *
 *   |html <ul><b>
 *   globus_gass_copy_attr
 *   |html </b>
 *   - A globus_gass_copy attribute structure.
 *
 *   |html <b>
 *   io_attr
 *   |html </b>
 *   - The file attributes to be used.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_set_io returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_set_io $globus_gass_copy_attr $io_attr ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_attr_init, globus_gass_copy_attr_set_gass,
 *   globus_gass_copy_attr_set_ftp, globus_gass_copy_get_url_mode
 *****/


/****f* globus_gass_copy/globus_gass_copy_attr_set_gass
 * NAME
 *   globus_gass_copy_set_gass
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_set_gass
 *   |html </b>
 *   |html <i>
 *   globus_gass_copy_attr
 *   globus_gass_transfer_requestattr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the attributes for http/https transfers.
 *
 *   In order to specify attributes for http/https transfers,
 *   globus_gass_transfer_requestattr should be 
 *   initialized and its values set using the appropriate
 *   globus_gass_transfer_requestattr_* command. The
 *   globus_gass_transfer_requestattr can then be passed to the
 *   globus_gass_copy_attr via this command.
 *
 *   |html <ul><b>
 *   globus_gass_copy_attr
 *   |html </b>
 *   - A globus_gass_copy attribute structure.
 *
 *   |html <b>
 *   globus_gass_transfer_requestattr
 *   |html </b>
 *   - The http/https attributes to be used
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_set_gass returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_set_gass $globus_gass_copy_attr
 *   $globus_gass_transfer_requestattr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_attr_init, globus_gass_copy_attr_set_io,
 *   globus_gass_copy_attr_set_ftp, globus_gass_copy_get_url_mode
 *****/


/****f* globus_gass_copy/globus_gass_copy_get_url_mode
 * NAME
 *   globus_gass_copy_get_url_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_get_url_mode
 *   |html </b>
 *   |html <i>
 *   url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to classify the URL schema into the transfer
 *   method that will be used to do the actual tranfer.
 *
 *   This command enables the user to determine what protocol will be
 *   used to transfer data to/from a particular url. This information
 *   can then be used to specify the appropriate attributes when
 *   initiating a transfer.
 *
 *   |html <ul><b>
 *   url
 *   |html </b>
 *   - The URL for schema checking
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_get_url_mode returns 0 and url_mode. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_get_url_mode $url ] result url_mode
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_attr_init, globus_gass_copy_attr_set_io,
 *   globus_gass_copy_attr_set_ftp, globus_gass_copy_get_set_gass
 *****/



/****f* globus_gass_copy/globus_gass_copy_url_to_url
 * NAME
 *   globus_gass_copy_url_to_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_url_to_url
 *   |html </b>
 *   |html <i>
 *   handle
 *   source_url
 *   source_attr
 *   dest_url
 *   dest_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to transfer data from source URL to destination
 *   URL in blocking mode.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to perform the copy operation.
 *
 *   |html <b>
 *   source_url
 *   |html </b>
 *   - Transfer data from this URL.
 *
 *   |html <b>
 *   source_attr
 *   |html </b>
 *   - Attributes describing how the transfer form the source should be done.
 *
 *   |html <b>
 *   dest_url
 *   |html </b>
 *   - Transfer data to this URL.
 *
 *   |html <b>
 *   dest_attr
 *   |html </b>
 *   - Attributes describing how the transfer to the destination should be
 *   done.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_url_to_url returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_url_to_url $handle $source_url NULL $dest_url
 *   NULL ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_url_to_handle, globus_gass_copy_handle_to_url
 *****/


/****f* globus_gass_copy/globus_gass_copy_url_to_handle
 * NAME
 *   globus_gass_copy_url_to_handle
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_url_to_handle
 *   |html </b>
 *   |html <i>
 *   handle
 *   source_url
 *   source_attr
 *   dest_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to transfer data from source URL to an I/O
 *   handle in blocking mode.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to perform the copy operation.
 *
 *   |html <b>
 *   source_url
 *   |html </b>
 *   - Transfer data from this URL.
 *
 *   |html <b>
 *   source_attr
 *   |html </b>
 *   - Attributes describing how the transfer form the source should be done.
 *
 *   |html <b>
 *   dest_handle
 *   |html </b>
 *   - transfer data to this IO handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_url_to_handle returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_url_to_handle $handle $source_url
 *   NULL $dest_io_handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_url_to_url, globus_gass_copy_handle_to_url
 *****/


/****f* globus_gass_copy/globus_gass_copy_handle_to_url
 * NAME
 *   globus_gass_copy_handle_to_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_handle_to_url
 *   |html </b>
 *   |html <i>
 *   handle
 *   source_handle
 *   dest_url
 *   dest_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to transfer data from an I/O handle to destination
 *   URL in blocking mode.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to perform the copy operation.
 *
 *   |html <b>
 *   source_handle
 *   |html </b>
 *   - Transfer data from this I/O handle.
 *
 *   |html <b>
 *   dest_url
 *   |html </b>
 *   - Transfer data to this URL.
 *
 *   |html <b>
 *   dest_attr
 *   |html </b>
 *   - Attributes describing how the transfer to the destination should be
 *   done.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_handle_to_url returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_handle_to_url $handle $source_io_handle
 *   $dest_url $dest_attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_url_to_handle, globus_gass_copy_url_to_url
 *****/


/****f* globus_gass_copy/globus_gass_copy_register_url_to_url
 * NAME
 *   globus_gass_copy_register_url_to_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_register_url_to_url
 *   |html </b>
 *   |html <i>
 *   handle
 *   source_url
 *   source_attr
 *   dest_url
 *   dest_attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to transfer data from source URL to destination
 *   URL in asynchronous mode. When the transfer is completed or if the
 *   transfer is aborted, the script will be invoked with the final
 *   status of the transfer.
 *
 *   |html <ul><b>
 *   gass_copy_handle
 *   |html </b>
 *   - The handle to perform the transfer operation.
 *
 *   |html <b>
 *   source_url
 *   |html </b>
 *   - Transfer data from this URL.
 *
 *   |html <b>
 *   source_attr
 *   |html </b>
 *   - Attributes describing how the transfer form the source should be done.
 *
 *   |html <b>
 *   dest_url
 *   |html </b>
 *   - Transfer data to this URL.
 *
 *   |html <b>
 *   dest_attr
 *   |html </b>
 *   - Attributes describing how the transfer to the destination should be
 *   done.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the transfer is completed. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error. 
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_register_url_to_url returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_register_url_to_url $handle
 *   $source_url NULL $dest_url NULL { callback_script user_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_register_url_to_handle,
 *   globus_gass_copy_register_handle_to_url
 *****/


/****f* globus_gass_copy/globus_gass_copy_register_url_to_handle
 * NAME
 *   globus_gass_copy_register_url_to_handle
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_register_url_to_handle
 *   |html </b>
 *   |html <i>
 *   gass_copy_handle
 *   source_url
 *   source_attr
 *   dest_handle
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to transfer data from source URL to destination
 *   I/O handle in asynchronous mode. When the transfer is completed or if the
 *   transfer is aborted, the script will be invoked with the final
 *   status of the transfer.
 *
 *   |html <ul><b>
 *   gass_copy_handle
 *   |html </b>
 *   - The handle to perform the transfer operation.
 *
 *   |html <b>
 *   source_url
 *   |html </b>
 *   - Transfer data from this URL.
 *
 *   |html <b>
 *   source_attr
 *   |html </b>
 *   - Attributes describing how the transfer form the source should be done.
 *
 *   |html <b>
 *   dest_handle
 *   |html </b>
 *   - Transfer data to this I/O handle.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the transfer is completed. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error. 
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_register_url_to_handle returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_register_url_to_handle $handle
 *   $source_url NULL $dest_handle { callback_script user_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_register_url_to_url,
 *   globus_gass_copy_register_handle_to_url
 *****/


/****f* globus_gass_copy/globus_gass_copy_register_handle_to_url
 * NAME
 *   globus_gass_copy_register_handle_to_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_register_handle_to_url
 *   |html </b>
 *   |html <i>
 *   handle
 *   source_handle
 *   dest_url
 *   dest_attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to transfer data from source I/O handle to
 *   destination URL in asynchronous mode. When the transfer is
 *   completed or if the transfer is aborted, the script will be
 *   invoked with the final status of the transfer.
 *
 *   |html <ul><b>
 *   gass_copy_handle
 *   |html </b>
 *   - The handle to perform the transfer operation.
 *
 *   |html <b>
 *   source_handle
 *   |html </b>
 *   - Transfer data from this IO handle
 *
 *   |html <b>
 *   dest_url
 *   |html </b>
 *   - Transfer data to this URL.
 *
 *   |html <b>
 *   dest_attr
 *   |html </b>
 *   - Attributes describing how the transfer to the destination should be
 *   done.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the transfer is completed. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error. 
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_register_handle_to_url returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_register_handle_to_url $handle
 *   $source_handle $dest_url $dest_attr { callback_script user_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_register_url_to_url,
 *   globus_gass_copy_register_url_to_handle
 *****/


/****f* globus_gass_copy/globus_gass_copy_get_status
 * NAME
 *   globus_gass_copy_get_status
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_get_status
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the status code of the current transfer.
 *
 *   Get the status of the last transfer to be initiated using the given
 *   handle. Only one transfer can be active on a handle at a given time,
 *   therefore new transfers may only be initiated when the current status
 *   is one of the following: GLOBUS_GASS_COPY_STATUS_NONE,
 *   GLOBUS_GASS_COPY_STATUS_DONE_SUCCESS,
 *   GLOBUS_GASS_COPY_STATUS_DONE_FAILURE,
 *   GLOBUS_GASS_COPY_STATUS_DONE_CANCELLED
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A globus_gass_copy_handle
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_get_status returns 0 and transfer status. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_get_status $handle ] result transfer_status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_get_status_string
 *****/


/****f* globus_gass_copy/globus_gass_copy_get_status_string
 * NAME
 *   globus_gass_copy_get_status_string
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_get_status_string
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the status string of the current transfer.
 *
 *   Get the status of the last transfer to be initiated using the given
 *   handle. Only one transfer can be active on a handle at a given time,
 *   therefore new transfers may only be initiated when the current status
 *   is one of the following: GLOBUS_GASS_COPY_STATUS_NONE,
 *   GLOBUS_GASS_COPY_STATUS_DONE_SUCCESS,
 *   GLOBUS_GASS_COPY_STATUS_DONE_FAILURE,
 *   GLOBUS_GASS_COPY_STATUS_DONE_CANCELLED
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A globus_gass_copy_handle
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_get_status returns 0 and transfer status. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_get_status_string $handle ] result
 *   transfer_status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_get_status
 *****/



/****f* globus_gass_copy/globus_gass_copy_cache_url_state
 * NAME
 *   globus_gass_copy_cache_url_state
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_cache_url_state
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to explicitly cache connections to URL server.
 *   When an URL is cached, the connection to the URL server will not be
 *   closed after a file transfer completes.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Handle which will contain a cached connection to the URL server.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL of the FTP or GSIFTP server to cache.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_cache_url_state returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_cache_url_state $handle $url ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_flush_url_state
 *****/



/****f* globus_gass_copy/globus_gass_copy_flush_url_state
 * NAME
 *   globus_gass_copy_flush_url_state
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_flush_url_state
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to explicitly remove a cached connection to
 *   an FTP or GSIFTP server. If an idle connection to an FTP server
 *   exists, it will be closed.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Handle which will contain a cached connection to the URL server.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL of the FTP or GSIFTP server to cache.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_flush_url_state returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_flush_url_state $handle $url ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_cache_url_state
 *****/


/****f* globus_gass_copy/globus_gass_copy_set_user_pointer
 * NAME
 *   globus_gass_copy_set_user_pointer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_set_user_pointer
 *   |html </b>
 *   |html <i>
 *   handle
 *   user_pointer
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set a pointer in the handle to point at
 *   user-allocated memory.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus GASS Copy handle.
 *
 *   |html <b>
 *   user_pointer
 *   |html </b>
 *   - The user-specified pointer in the handle structure.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_set_user_pointer returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_set_user_pointer $handle
 *   $user_pointer ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_get_user_pointer
 *****/


/****f* globus_gass_copy/globus_gass_copy_get_user_pointer
 * NAME
 *   globus_gass_copy_get_user_pointer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_get_user_pointer
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the pointer in the handle that points
 *   to user-allocated memory.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus GASS Copy handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_get_user_pointer returns 0 and the
 *   user-specified pointer in the handle structure. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_get_user_pointer $handle ] result user_pointer
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_copy_set_user_pointer
 *****/


/****f* globus_gass_copy/globus_gass_copy_register_performance_cb
 * NAME
 *   globus_gass_copy_register_performance_cb
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_register_performance_cb
 *   |html </b>
 *   |html <i>
 *   handle
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to register a performance information callback.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An initialized Globus GASS Copy handle for which you would like
 *   to see performance info.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error. 
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_register_performance_cb returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_register_performance_cb $handle
 *   { callback_script user_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   None
 *****/


/****f* globus_gass_copy/globus_gass_copy_glob_expand_url
 * NAME
 *   globus_gass_copy_glob_expand_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_glob_expand_url
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to expand wildcards in a globbed URL, and invokes
 *   the script on each one.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A Globus GASS Copy handle to use for the operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to expand. The URL may be an FTP, GSIFTP or file URL.
 *   Wildcard characters supported are '?' '*' '[ ]' in the filename
 *   portion of the URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - GASS Copy attributes for this operation.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked for each entry. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error. 
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_glob_expand_url returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_glob_expand_url $handle $url $attr
 *   { entry_callback user_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   None
 *****/


/****f* globus_gass_copy/globus_gass_copy_mkdir
 * NAME
 *   globus_gass_copy_mkdir
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_copy
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_copy_mkdir
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create a directory given a FTP or file URL.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - A Globus GASS Copy handle to use for the operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL for the directory to create. The URL may be an FTP, GSIFTP
 *   or file URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - GASS Copy attributes for this operation.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_copy_mkdir returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_copy_mkdir $handle $url $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   None
 *****/
