/* -*- mode: c; basic-c-offset: 2; -*- */
#ifdef SWIG

%include "../../config.h"

/*****************************************************************
 * Hand-wrapped SWIG functions
 *****************************************************************/

%ignore globus_gass_transfer_request_get_type;
%ignore globus_gass_transfer_request_get_status;

%ignore globus_gass_transfer_register_get;
%ignore globus_gass_transfer_register_put;
%ignore globus_gass_transfer_register_append;

%ignore globus_gass_transfer_get;
%ignore globus_gass_transfer_put;
%ignore globus_gass_transfer_append;

%ignore globus_gass_transfer_create_listener;
%ignore globus_gass_transfer_register_listen;
%ignore globus_gass_transfer_register_accept;
%ignore globus_gass_transfer_close_listener;

//%ignore globus_gass_transfer_refer;
//%ignore globus_gass_transfer_deny;

%ignore globus_gass_transfer_send_bytes;
%ignore globus_gass_transfer_receive_bytes;
%ignore globus_gass_transfer_fail;

%ignore globus_gass_transfer_request_destroy;

%ignore globus_gass_transfer_request_get_referral;
%ignore globus_gass_transfer_referral_destroy;

%ignore globus_gass_transfer_listenerattr_destroy;

%include "tcl-typemaps.i" 

%include "swig/globus_gass_transfer.i"

%clear char **;

%{
  
#include <tcl.h>
#include "tcl-callback.h"
#include "common/sgdebug.h"
#include "common/sginit.h"
  
#define MAX_UNIQUE_NAME		64

  /* Callback-related typedef */
  typedef struct callbackData_ {
    long	total_bytes;
    int		iovec_cnt;
    void*	buffer;
    char	unique_name[MAX_UNIQUE_NAME];
  } callbackData;

  /***************************************************************
   * GASS Transfer Register Put/Get/Append Callback
   ***************************************************************/
  static void
    GlobusGASSTransferRegisterClientOperationInit( callbackStruct* cb )
    {
      globus_gass_transfer_request_t * request =
        (globus_gass_transfer_request_t *)globus_libc_malloc( sizeof(globus_gass_transfer_request_t) );
      
      cb->s_api_data = request;
    }

  static int
    GlobusGASSTransferRegisterClientOperationCleanup( callbackStruct* cb )
    {
      globus_libc_free( cb->s_api_data );

      cb->s_api_data = NULL;
      
      return 1;
    }

  static void
    TclGASSTransferCallbackClientOperation
    ( void * 				callback_arg, 
      globus_gass_transfer_request_t 	request )
    {
      char request_ptr_buf[256];
      
      /* Special trick for converting request handle */
      SWIG_MakePtr
        ( request_ptr_buf,
          (globus_gass_transfer_request_t*)((callbackStruct*)callback_arg)->s_api_data, 
          SWIG_TypeQuery( "globus_gass_transfer_request_t *" ), 0 );
      
      CallCallback( (callbackStruct*)callback_arg,
                    "%s",
                    request_ptr_buf );
    }
  
  /***************************************************************
   * GASS Transfer Close Callback
   ***************************************************************/
  static void
    TclGASSTransferCloseCallback
    ( void * 				arg, 
      globus_gass_transfer_listener_t 	listener )
    {
      SGDEBUG_LVL_FMT_1( 10, "TclGASSTransferCloseCallback: listener: %p", 
                         &listener );
      
      CallCallback( (callbackStruct*)arg,
                    "%p",
                    &listener );
    }

  /***************************************************************
   * GASS Transfer Listen Callback
   ***************************************************************/
  static void
    TclGASSTransferListenCallback
    ( void * 				arg, 
      globus_gass_transfer_listener_t 	listener )
    {
      SGDEBUG_LVL_FMT_1( 10, "TclGASSTransferListenCallback: listener: %p", 
                         &listener );

      CallCallback( (callbackStruct*)arg,
                    "%p",
                    &listener );
    }

  /***************************************************************
   * GASS Transfer Fail
   ***************************************************************/
  static void
    TclGASSTransferFailCallback
    ( void * 				arg,
      globus_gass_transfer_request_t 	request,
      globus_byte_t * 			buffer,
      globus_size_t 			length,
      globus_bool_t 			last_data )
    {
      CallCallback( (callbackStruct*)arg,
                    "%p %s %d %d",
                    &request,
                    buffer,
                    length,
                    last_data );
    }

  /**************************************************************************
   * GLOBUS_GASS_TRANSFER_REGISTER_ACCEPT
   **************************************************************************/
  static void
    GlobusGASSTransferRegisterAcceptInit( callbackStruct* cb )
    {
      globus_gass_transfer_request_t * request =
        (globus_gass_transfer_request_t *)globus_libc_malloc( sizeof(globus_gass_transfer_request_t) );
      
      cb->s_api_data = request;
    }

  static int
    GlobusGASSTransferRegisterAcceptCleanup( callbackStruct* cb )
    {
      globus_libc_free( cb->s_api_data );

      cb->s_api_data = NULL;
      
      return 1;
    }


  static void
    TclGASSTransferRegisterAcceptCallback
    ( void *               		callback_arg,
      globus_gass_transfer_request_t * 	request )
    {      
      char request_ptr_buf[256];
      
      /* Special trick for converting request handle */
      SWIG_MakePtr
        ( request_ptr_buf,
          (globus_gass_transfer_request_t*)((callbackStruct*)callback_arg)->s_api_data, 
          SWIG_TypeQuery( "globus_gass_transfer_request_t *" ), 0 );

      CallCallback( (callbackStruct*)callback_arg,
                    "%s",
                    request_ptr_buf );
    }
  
  /***************************************************************
   * GASS Transfer Send Bytes
   ***************************************************************/
  static void
    GlobusGASSTransferSendBytesEOFInit( callbackStruct* cb,
					globus_byte_t * send_buffer )
    {
      /* Allocate memory space */
      callbackData* data =
	(callbackData*)globus_libc_malloc( sizeof(callbackData) );

      SGDEBUG_LVL_FMT_1
	( 10, "GlobusGASSTransferSendBytesEOFInit: callback allocation: %p",
	  cb->s_api_data );

      cb->s_api_data = data;
      
      SGDEBUG_LVL_FMT_1
	( 10,
	  "GlobusGASSTransferSendBytesEOFInit: cb->s_api_data: %p",
	  cb->s_api_data );

      data->buffer = send_buffer;
	  
      SGDEBUG_LVL_FMT_1
	( 10,
	  "GlobusGASSTransferSendBytesEOFInit: data->buffer: %p",
	  data->buffer );
    }

  static int
    GlobusGASSTransferSendBytesEOFCleanup( callbackStruct* cb )
    {
      callbackData* data = (callbackData*)(cb->s_api_data);

      /* Deallocate memory space */
      if ( data->buffer ) 
	{
	  SGDEBUG_LVL_FMT_1
	    ( 10, "GlobusGASSTransferSendBytesEOFCleanup: send buffer deallocation: %p", data->buffer );  
	  globus_libc_free( data->buffer );
	}
      
      /* Deallocate memory space */
      if ( cb->s_api_data ) 
	{
	  SGDEBUG_LVL_FMT_1
	    ( 10, "GlobusGASSTransferSendBytesEOFCleanup: callback deallocation: %p", cb->s_api_data );
	  globus_libc_free(cb->s_api_data);
	  cb->s_api_data = NULL;
	}
      
      return 1;
    }

  static void
    TclGASSTransferSendBytesEOFCallback
    ( void * 				arg,
      globus_gass_transfer_request_t 	request,
      globus_byte_t * 			buffer,
      globus_size_t 			length,
      globus_bool_t 			last_data )
    {
      CallCallback( (callbackStruct*)arg,
                    "%p %p %d %d",
                    &request,
                    NULL,
                    0,
                    last_data );
    }


  
  static void
    GlobusGASSTransferSendBytesInit( callbackStruct* cb,
				     globus_byte_t * send_buffer )
    {
      /* Allocate memory space */
      callbackData* data =
	(callbackData*)globus_libc_malloc( sizeof(callbackData) );

      cb->s_api_data = data;
      
      SGDEBUG_LVL_FMT_1
	( 10,
	  "GlobusGASSTransferSendBytesInit: cb->s_api_data: %p",
	  cb->s_api_data );

      data->buffer = send_buffer;
	  
      SGDEBUG_LVL_FMT_1
	( 10,
	  "GlobusGASSTransferSendBytesInit: data->buffer: %p",
	  data->buffer );
	  
      TclGlobusCreateUniqueTCLVariableName( data->unique_name );
    }

  static int
    GlobusGASSTransferSendBytesCleanup( callbackStruct* cb )
    {
      callbackData* data = (callbackData*)(cb->s_api_data);
      
      /* Release TCL resources */
      SGDEBUG_LVL_FMT_1(10, "GlobusGASSTransferSendBytesCleanup: Tcl_UnsetVar2: %s", data->unique_name);
      Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);


      /* Deallocate memory space */
      if ( data->buffer ) 
	{
	  SGDEBUG_LVL_FMT_1
	    ( 10, "GlobusGASSTransferSendBytesCleanup: send buffer deallocation: %p", data->buffer );  
	  globus_libc_free( data->buffer );
	}

      
      /* Deallocate memory space */
      if ( cb->s_api_data ) 
	{
	  SGDEBUG_LVL_FMT_1
	    ( 10, "GlobusGASSTransferSendBytesCleanup: send callback deallocation: %p", cb->s_api_data );
	  globus_libc_free(cb->s_api_data);
	  cb->s_api_data = NULL;
	}
      return 1;
    }

  static void
    GlobusGASSTransferSendBytesPreCallback( callbackStruct* cb )
    {
      Tcl_Interp* interp;
      Tcl_Obj* obj;
      callbackData* data;

      data = (callbackData*)(cb->s_api_data);

      SGDEBUG_LVL_FMT_1
	( 10, "GlobusGASSTransferSendBytesPreCallback: cb->s_api_data: %p",
	  cb->s_api_data );      

      SGDEBUG_LVL_FMT_1
	( 10, "GlobusGASSTransferSendBytesPreCallback: data: %p", data );      

      SGDEBUG_LVL_FMT_1
	( 10, "GlobusGASSTransferSendBytesPreCallback: data->buffer: %p",
	  data->buffer );      
      
      SGDEBUG_LVL_FMT_1
	( 10, "GlobusGASSTransferSendBytesPreCallback: data->total_bytes: %d",
	  data->total_bytes );      

      
      obj = Tcl_NewByteArrayObj( (char *)(data->buffer),
				 data->total_bytes );
      
      interp = cb->s_main_tcl_interp;
      
      Tcl_SetVar2Ex(interp, data->unique_name, NULL, obj, 0);
    }
  
  static void
    TclGASSTransferSendBytesCallback
    ( void * 				arg,
      globus_gass_transfer_request_t 	request,
      globus_byte_t * 			buffer,
      globus_size_t 			length,
      globus_bool_t 			last_data )
    {
      callbackData*
        data = (callbackData*)((callbackStruct*)arg)->s_api_data;
      
      data->total_bytes = length;
      data->buffer = buffer;

      CallCallback( (callbackStruct*)arg,
                    "%p $%s %d %d",
                    &request,
                    data->unique_name,
                    data->total_bytes,
                    last_data );
    }
  
  /***************************************************************
   * GASS Transfer Receive Bytes
   ***************************************************************/
  static void
    GlobusGASSTransferReceiveBytesInit( callbackStruct* cb,
					long buffer_length )
    {
      /* Allocate memory space for our buffer */
      callbackData* data = 
        (callbackData*)globus_libc_malloc( sizeof(callbackData) );
      
      cb->s_api_data = data;
      data->total_bytes = buffer_length; 
      
      /* :TODO: We should not exit if we fail to allocate our buffer */
      /* :TODO: Must return error message instead of just exit */
      if ( ( data->buffer = 
             (globus_byte_t*)globus_libc_malloc( sizeof(globus_byte_t) * 
						 buffer_length )) == NULL ) 
        {
          fprintf(stderr, "Memory allocation fails\n");
          exit(1);
        }
      
      memset( data->buffer, '\0', buffer_length );

      SGDEBUG_LVL_FMT_1( 30, "recv Tcl_SetVar2: %p", data->unique_name );
      
      TclGlobusCreateUniqueTCLVariableName( data->unique_name );

      SGDEBUG_LVL_FMT_1( 30, "recv callback allocation: %p", data );  
      SGDEBUG_LVL_FMT_1( 30, "recv buffer allocation: %p", data->buffer );  
    }

  static int
    GlobusGASSTransferReceiveBytesCleanup( callbackStruct* cb )
    {
      callbackData* data = (callbackData*)cb->s_api_data;

      SGDEBUG_LVL_FMT_1( 30, "read Tcl_UnsetVar2: %p", data->unique_name );
      
      Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);

      /* Deallocate memory space */
      if ( data->buffer ) 
        {
          SGDEBUG_LVL_FMT_1
            ( 30, "recv buffer deallocation: %p", data->buffer ); 
          globus_libc_free( data->buffer );
        }
      
      if ( cb->s_api_data ) 
        {
          SGDEBUG_LVL_FMT_1
            ( 30, "recv callback deallocation: %p", cb->s_api_data );  
          globus_libc_free(cb->s_api_data);
          cb->s_api_data = NULL;
        }
      return 1;
    }

  static void
    GlobusGASSTransferReceiveBytesPreCallback( callbackStruct* cb )
    {
      Tcl_Interp* interp;
      Tcl_Obj* obj;
      
      callbackData* data = (callbackData*)(cb->s_api_data);
      
      interp = cb->s_main_tcl_interp;

      obj = Tcl_NewByteArrayObj((char *)(data->buffer), data->total_bytes);
          
      Tcl_SetVar2Ex(interp, data->unique_name, NULL, obj, 0);
    }

  static void
    TclGASSTransferReceiveBytesCallback
    ( void* 				arg,
      globus_gass_transfer_request_t 	request,
      globus_byte_t* 			bytes,
      globus_size_t 			length,
      globus_bool_t 			last_data )
    {
      callbackData*
        data = (callbackData*)((callbackStruct*)arg)->s_api_data;
      
      data->total_bytes = length;
      
      CallCallback( (callbackStruct*)arg,
                    "%p $%s %d %d",
                    &request,
                    data->unique_name,
                    data->total_bytes,
                    last_data );
    }
%}


/**************************************************************************
 *
 *  SWIG INITIALIZATION STUFF
 *
 **************************************************************************/
%init %{

  SGINIT_X509;

  %}

/**************************************************************************
 **************************************************************************
 *				TYPEMAPS SECTION
 **************************************************************************
 **************************************************************************/

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REQUEST_GET_TYPE
 **************************************************************************/
%typemap(in,numinputs=0) char* transfer_type
{ 
  $1 = globus_libc_malloc(sizeof(char) * 64);
}

%typemap(argout,fragment="t_output_helper") char* transfer_type
{
  Tcl_Obj* o = Tcl_NewStringObj((char*)$1, -1);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%typemap(freearg) char* transfer_type
{
  globus_libc_free($1);
}

%rename(globus_gass_transfer_request_get_type)
     tcl_globus_gass_transfer_request_get_type;

     /*
       %exception tcl_globus_gass_transfer_request_get_type GLOBUS_EXCEPTION_RESULT_T;
     */
     
%inline %{
  void
    tcl_globus_gass_transfer_request_get_type
    ( globus_gass_transfer_request_t request,
      char* transfer_type )
    {
      globus_gass_transfer_request_type_t type;
      
      type = globus_gass_transfer_request_get_type( request );

      switch (type)
	{
	case GLOBUS_GASS_TRANSFER_REQUEST_TYPE_INVALID:
	  sprintf(transfer_type, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_TYPE_INVALID");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_TYPE_GET:
	  sprintf(transfer_type, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_TYPE_GET");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_TYPE_PUT:
	  sprintf(transfer_type, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_TYPE_PUT");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_TYPE_APPEND:
	  sprintf(transfer_type, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_TYPE_APPEND");
	  break;
	default:
	  sprintf(transfer_type, "%s", "Unrecognized GASS Transfer type");
	  break;
	}
    }
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REQUEST_GET_STATUS
 **************************************************************************/
%typemap(in,numinputs=0) char* transfer_status
{ 
  $1 = globus_libc_malloc(sizeof(char) * 64);
}

%typemap(argout,fragment="t_output_helper") char* transfer_status
{
  Tcl_Obj* o = Tcl_NewStringObj((char*)$1, -1);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%typemap(freearg) char* transfer_status
{
  globus_libc_free($1);
}

%rename(globus_gass_transfer_request_get_status) tcl_globus_gass_transfer_request_get_status;

/*
  %exception tcl_globus_gass_transfer_request_get_status GLOBUS_EXCEPTION_RESULT_T; */

%inline %{
  void
    tcl_globus_gass_transfer_request_get_status
    ( globus_gass_transfer_request_t request,
      char* transfer_status )
    {
      globus_gass_transfer_request_status_t status;

      status = globus_gass_transfer_request_get_status( request );

      switch (status)
	{
	case GLOBUS_GASS_TRANSFER_REQUEST_INVALID:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_INVALID");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_STARTING:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_STARTING");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_PENDING:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_PENDING");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_FAILED:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_FAILED");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_REFERRED:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_REFERRED");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_DENIED:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_DENIED");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_DONE:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_DONE");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_ACCEPTING:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_ACCEPTING");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_ACTING:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_ACTING");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_ACTING_TO_PENDING:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_ACTING_TO_PENDING");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_ACTING_TO_FAILING:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_ACTING_TO_FAILING");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_FAILING:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_FAILING");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_USER_FAIL:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_USER_FAIL");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_REFERRING:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_REFERRING");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_ACTING_TO_REFERRING:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_ACTING_TO_REFERRING");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_FINISHING:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_FINISHING");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_SERVER_FAIL1:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_SERVER_FAIL1");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_SERVER_FAIL2:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_SERVER_FAIL2");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_SERVER_FAIL3:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_SERVER_FAIL3");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_STARTING2:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_STARTING2");
	  break;
	case GLOBUS_GASS_TRANSFER_REQUEST_STARTING3:
	  sprintf(transfer_status, "%s",
		  "GLOBUS_GASS_TRANSFER_REQUEST_STARTING3");
	  break;
	default:
	  sprintf(transfer_status, "%s", "Unrecognized GASS Transfer status");
	  break;
	}
    }
  %}


/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REFER
 **************************************************************************/

#if 0
%rename(globus_gass_transfer_refer) tcl_globus_gass_transfer_refer;

%exception tcl_globus_gass_transfer_refer GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gass_transfer_refer
    ( globus_gass_transfer_request_t	request,
      char **				urls,
      globus_size_t			num_urls )
    {
      int retval;

      retval = globus_gass_transfer_refer( request,
					   urls,
					   num_urls );
      
      return retval;
    }
  %}
#endif

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REQUEST_GET_REFERRAL
 **************************************************************************/
%typemap(in,numinputs=0)
     globus_gass_transfer_referral_t * referral
{
  $1 = (globus_gass_transfer_referral_t *)
    globus_libc_malloc(sizeof(globus_gass_transfer_referral_t));

  SGDEBUG_LVL_FMT_1
    ( 10,
      "referral handle creation: typemap(in): %p", $1 );  
}
%typemap(argout,fragment="t_output_helper")
     globus_gass_transfer_referral_t * referral 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_gass_transfer_request_get_referral)
     tcl_globus_gass_transfer_request_get_referral;

%exception tcl_globus_gass_transfer_request_get_referral GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gass_transfer_request_get_referral
    ( globus_gass_transfer_request_t		request,
      globus_gass_transfer_referral_t *		referral )
    {
      int retval;

      retval = globus_gass_transfer_request_get_referral
	( request,
	  referral );

      return retval;
    }
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REFERRAL_DESTROY
 **************************************************************************/

%typemap(freearg) globus_gass_transfer_referral_t * referral_handle
{
  SGDEBUG_LVL_FMT_1
    (10,
     "globus_gass_transfer_referral_destroy: freearg: %p\n", arg1);

  if ( arg1 )
    {
      SGDEBUG_LVL_FMT_1
	( 10,
	  "globus_gass_transfer_referral_destroy: freearg: %p", arg1 );
  
      globus_libc_free(arg1);
    }
}


%rename(globus_gass_transfer_referral_destroy)
     tcl_globus_gass_transfer_referral_destroy;

%exception tcl_globus_gass_transfer_referral_destroy GLOBUS_EXCEPTION_RESULT_T;
     
%inline %{
  int
    tcl_globus_gass_transfer_referral_destroy
    ( globus_gass_transfer_referral_t * referral_handle )
    {
      int retval;
      
      retval = globus_gass_transfer_referral_destroy( referral_handle );
            
      return retval;
    }
  %}


/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REQUEST_DESTROY
 **************************************************************************/

%typemap(freearg) globus_gass_transfer_request_t request_handle
{
  SGDEBUG_LVL_FMT_1
    (10,
     "globus_gass_transfer_request_destroy: freearg: %p\n", argp1);

  if ( argp1 )
    {
      SGDEBUG_LVL_FMT_1
	( 10,
	  "globus_gass_transfer_request_destroy: freearg: %p", argp1 );
  
      globus_libc_free(argp1);
    }
}


%rename(globus_gass_transfer_request_destroy)
     tcl_globus_gass_transfer_request_destroy;

%exception tcl_globus_gass_transfer_request_destroy GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gass_transfer_request_destroy
    ( globus_gass_transfer_request_t request_handle )
    {
      int retval;

      SGDEBUG_LVL_FMT_1
	(10,
	 "globus_gass_transfer_request_destroy: %p\n", &request_handle);

      
      retval = globus_gass_transfer_request_destroy( request_handle );

      SGDEBUG_LVL_FMT_1
	(10,
	 "globus_gass_transfer_request_destroy: retval: %d\n", retval);

      
      return retval;
    }
  %}


/**************************************************************************
 * GLOBUS_GASS_TRANSFER_LISTENERATTR_DESTROY
 **************************************************************************/
%rename(globus_gass_transfer_listenerattr_destroy)
     tcl_globus_gass_transfer_listenerattr_destroy;

%exception tcl_globus_gass_transfer_listenerattr_destroy GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gass_transfer_listenerattr_destroy
    ( globus_gass_transfer_listenerattr_t * attr )
    {
      int retval;

      retval = globus_gass_transfer_requestattr_destroy(attr);
      globus_libc_free(attr);
      
      return retval;
    }
  
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_PUT
 * GLOBUS_GASS_TRANSFER_GET
 * GLOBUS_GASS_TRANSFER_APPEND
 **************************************************************************/
%typemap(in,numinputs=0)
     globus_gass_transfer_request_t * request
{
  $1 = (globus_gass_transfer_request_t *)
    globus_libc_malloc(sizeof(globus_gass_transfer_request_t));

  SGDEBUG_LVL_FMT_1
    ( 10,
      "request handle creation: typemap(in): %p", $1 );  
}
%typemap(argout,fragment="t_output_helper")
     globus_gass_transfer_request_t * request 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_GET
 **************************************************************************/
%rename(globus_gass_transfer_get) tcl_globus_gass_transfer_get;

%exception tcl_globus_gass_transfer_get GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gass_transfer_get
    ( globus_gass_transfer_request_t *		request,
      globus_gass_transfer_requestattr_t *	attr,
      char *					url )
    {
      int retval;

      retval = globus_gass_transfer_get
	( request,
	  attr,
	  url );

      return retval;
    }
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_PUT
 **************************************************************************/
%rename(globus_gass_transfer_put) tcl_globus_gass_transfer_put;

%exception tcl_globus_gass_transfer_put GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gass_transfer_put
    ( globus_gass_transfer_request_t *		request,
      globus_gass_transfer_requestattr_t *	attr,
      char *					url,
      globus_size_t				length )
    {
      int retval;

      retval = globus_gass_transfer_put
	( request,
	  attr,
	  url,
	  length );

      return retval;
    }
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_APPEND
 **************************************************************************/
%rename(globus_gass_transfer_append) tcl_globus_gass_transfer_append;

%exception tcl_globus_gass_transfer_append GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gass_transfer_append
    ( globus_gass_transfer_request_t *		request,
      globus_gass_transfer_requestattr_t *	attr,
      char *					url,
      globus_size_t				length )
    {
      int retval;

      retval = globus_gass_transfer_append
	( request,
	  attr,
	  url,
	  length );

      return retval;
    }
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REGISTER_ACCEPT
 **************************************************************************/
%typemap(in) 
  ( globus_gass_transfer_callback_t	accept_cb,
    void * 				accept_arg )
     (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  /* Allocate our buffer */
  GlobusGASSTransferRegisterAcceptInit( cbStruct );

  $1 = (globus_gass_transfer_callback_t)TclGASSTransferRegisterAcceptCallback; 
  $2 = cbStruct;

  /*
   * :TODO: We must deallocate the memory created for holding TCP
   * :TODO:   connection handle in globus_io_register_close().
   */
}

%rename(globus_gass_transfer_register_accept) tcl_globus_gass_transfer_register_accept;


%exception tcl_globus_gass_transfer_register_accept GLOBUS_EXCEPTION_RESULT_T;


%inline %{
  int
    tcl_globus_gass_transfer_register_accept
    ( globus_gass_transfer_requestattr_t *	attr,
      globus_gass_transfer_listener_t		listener,
      globus_gass_transfer_callback_t		accept_cb,
      void *					accept_arg )
    {
      int retval;

      retval = globus_gass_transfer_register_accept
	( (globus_gass_transfer_request_t*)((callbackStruct*)accept_arg)->s_api_data,
	  attr,
	  listener,
	  accept_cb,
	  accept_arg );
      
      return retval;
    }
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REGISTER_GET
 * GLOBUS_GASS_TRANSFER_REGISTER_PUT
 * GLOBUS_GASS_TRANSFER_REGISTER_APPEND
 **************************************************************************/
%typemap(in) 
     (globus_gass_transfer_callback_t callback, void * user_arg)
     (callbackStruct * cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  /* Allocate our buffer to hold request handle */
  GlobusGASSTransferRegisterClientOperationInit( cbStruct );
  
  $1 = (globus_gass_transfer_callback_t)TclGASSTransferCallbackClientOperation;
  $2 = cbStruct;

  /*
   * :TODO: We must deallocate the memory created for holding TCP
   * :TODO:   connection handle in globus_io_register_close().
   */  
}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REGISTER_PUT
 **************************************************************************/
%rename(globus_gass_transfer_register_put)
     tcl_globus_gass_transfer_register_put;


%exception tcl_globus_gass_transfer_register_put GLOBUS_EXCEPTION_RESULT_T;


%inline %{
  int
    tcl_globus_gass_transfer_register_put
    ( globus_gass_transfer_requestattr_t *	attr,
      char *					url,
      globus_size_t				length,
      globus_gass_transfer_callback_t		callback,
      void *					user_arg )
    {
      int retval;

      retval = globus_gass_transfer_register_put
	( (globus_gass_transfer_request_t*)((callbackStruct*)user_arg)->s_api_data,
	  attr,
	  url,
	  length,
	  callback,
	  user_arg );
      
      return retval;
    }
  %}


/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REGISTER_GET
 **************************************************************************/

%rename(globus_gass_transfer_register_get)
     tcl_globus_gass_transfer_register_get;

%exception tcl_globus_gass_transfer_register_get GLOBUS_EXCEPTION_RESULT_T;


%inline %{
  int
    tcl_globus_gass_transfer_register_get
    ( globus_gass_transfer_requestattr_t *	attr,
      char *					url,
      globus_gass_transfer_callback_t		callback,
      void * 					user_arg )
    {
      int retval;

      retval = globus_gass_transfer_register_get
	( (globus_gass_transfer_request_t*)((callbackStruct*)user_arg)->s_api_data,
	  attr,
	  url,
	  callback,
	  user_arg );

      return retval;
    }
  %}


/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REGISTER_APPEND
 **************************************************************************/
%rename(globus_gass_transfer_register_append)
     tcl_globus_gass_transfer_register_append;


%exception tcl_globus_gass_transfer_register_append GLOBUS_EXCEPTION_RESULT_T;


%inline %{
  int
    tcl_globus_gass_transfer_register_append
    ( globus_gass_transfer_requestattr_t *	attr,
      char *					url,
      globus_size_t				length,
      globus_gass_transfer_callback_t		callback,
      void *					user_arg )
    {
      int retval;

      retval = globus_gass_transfer_register_append
	( (globus_gass_transfer_request_t*)((callbackStruct*)user_arg)->s_api_data,
	  attr,
	  url,
	  length,
	  callback,
	  user_arg );
      
      return retval;
    }
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_FAIL
 **************************************************************************/
%rename(globus_gass_transfer_fail) tcl_globus_gass_transfer_fail;

%exception tcl_globus_gass_transfer_fail GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gass_transfer_fail
    ( globus_gass_transfer_request_t	request,
      globus_gass_transfer_callback_t	callback,
      void * 				user_arg )
    {
      int retval;

      retval = globus_gass_transfer_fail( request,
					  callback,
					  user_arg );
      
      return retval;
    }
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_CREATE_LISTENER
 **************************************************************************/
%typemap(in,numinputs=0)
     globus_gass_transfer_listener_t * listener
{
  $1 = (globus_gass_transfer_listener_t *)
    globus_libc_malloc(sizeof(globus_gass_transfer_listener_t));
}
%typemap(argout,fragment="t_output_helper")
     globus_gass_transfer_listener_t * listener 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_gass_transfer_create_listener) tcl_globus_gass_transfer_create_listener;

%exception tcl_globus_gass_transfer_create_listener GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gass_transfer_create_listener
    ( globus_gass_transfer_listener_t *		listener,
      globus_gass_transfer_listenerattr_t *	attr,
      char *					url )
    {
      int retval;

      retval = globus_gass_transfer_create_listener
	( listener,
	  attr,
	  url );

      return retval;
    }
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_CLOSE_LISTENER
 **************************************************************************/
%typemap(in) 
     (globus_gass_transfer_close_callback_t callback, void * user_arg)
     (callbackStruct * cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );
  SGDEBUG_LVL_FMT_1
    ( 10,
      "globus_gass_transfer_register_listen: typemap(in): %p", cbStruct );
  
  $1 = (globus_gass_transfer_close_callback_t)TclGASSTransferCloseCallback;
  $2 = cbStruct;
}

%rename(globus_gass_transfer_close_listener) tcl_globus_gass_transfer_close_listener;


%exception tcl_globus_gass_transfer_close_listener GLOBUS_EXCEPTION_RESULT_T;


%inline %{
  int
    tcl_globus_gass_transfer_close_listener
    ( globus_gass_transfer_listener_t		listener,
      globus_gass_transfer_close_callback_t 	callback,
      void *					user_arg )
    {
      int retval;

      SGDEBUG_LVL_FMT_1
	( 10,
	  "globus_gass_transfer_close_listener: listener: %p", listener );
      
      retval = globus_gass_transfer_close_listener
	( listener,
	  callback,
	  user_arg );

      SGDEBUG_LVL_FMT_1
	( 10,
	  "globus_gass_transfer_close_listener: retval: %d", retval );
      return retval;
    }
  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_REGISTER_LISTEN
 **************************************************************************/
%typemap(in) 
     (globus_gass_transfer_listen_callback_t callback, void * user_arg)
     (callbackStruct* cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );
  SGDEBUG_LVL_FMT_1
    ( 10,
      "globus_gass_transfer_register_listen: typemap(in): %p", cbStruct );
  
  $1 = (globus_gass_transfer_listen_callback_t)TclGASSTransferListenCallback;
  $2 = cbStruct;
}

%rename(globus_gass_transfer_register_listen) tcl_globus_gass_transfer_register_listen;


%exception tcl_globus_gass_transfer_register_listen GLOBUS_EXCEPTION_RESULT_T;


%inline %{
  int
    tcl_globus_gass_transfer_register_listen
    ( globus_gass_transfer_listener_t		listener,
      globus_gass_transfer_listen_callback_t	callback,
      void *					user_arg )
    {
      int retval;

      retval = globus_gass_transfer_register_listen
	( listener,
	  callback,
	  user_arg );
      SGDEBUG_LVL_FMT_1
	( 10,
	  "globus_gass_transfer_register_listen: retval: %d", retval );
      
      return retval;
    }
  %}


/**************************************************************************
 * GLOBUS_GASS_TRANSFER_SEND_BYTES
 **************************************************************************/

%typemap(in) globus_byte_t * send_buffer
{
  unsigned char *   tbuffer;
  int length;

  if ( arg3 != 0 )
    {
      tbuffer = 
	(unsigned char *)Tcl_GetByteArrayFromObj( $input, &length );
      
      SGDEBUG_LVL_FMT_1( 10, "typemap(in): buffer length = %d", length);
      
      arg2 = globus_libc_malloc(length);
      
      memset(arg2, '\0', length);
      
      memcpy( arg2, tbuffer, length );
    }
  else
    {
      arg2 = globus_libc_malloc(0);
    }
}


%typemap(in)
     (globus_gass_transfer_bytes_callback_t send_cb, void * send_user_arg)
     (callbackStruct* cbStruct)
{
  
  if ( arg3 != 0 )
    {
      cbStruct = CreateCallback( interp, $input );
      
      GlobusGASSTransferSendBytesInit(cbStruct, arg2);
      
      $1 = TclGASSTransferSendBytesCallback;
      $2 = cbStruct;
      
      cbStruct->s_pre_process = GlobusGASSTransferSendBytesPreCallback;
      cbStruct->s_post_process = GlobusGASSTransferSendBytesCleanup;
    }
  else
    {
      cbStruct = CreateCallback( interp, $input );
      
      GlobusGASSTransferSendBytesEOFInit(cbStruct, arg2);
      
      $1 = TclGASSTransferSendBytesEOFCallback;
      $2 = cbStruct;

      cbStruct->s_post_process = GlobusGASSTransferSendBytesEOFCleanup;
    }
}

%rename(globus_gass_transfer_send_bytes) tcl_globus_gass_transfer_send_bytes;

%exception tcl_globus_gass_transfer_send_bytes GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  typedef struct {
    globus_byte_t *	buffer;
    globus_size_t	length;
  } tcl_globus_buffer_t;
  
  int
    tcl_globus_gass_transfer_send_bytes
    ( globus_gass_transfer_request_t		request,
      globus_byte_t *				send_buffer,
      globus_size_t				length,
      globus_bool_t				last_data,
      globus_gass_transfer_bytes_callback_t	send_cb,
      void * 					send_user_arg )
    {
      int retval;

      SGDEBUG_LVL_FMT_1
	( 10, "tcl_globus_gass_transfer_send_bytes: send_buffer = %p",
	  send_buffer );
      SGDEBUG_LVL_FMT_1
	( 10, "tcl_globus_gass_transfer_send_bytes: length = %d",
	  length ); 
      SGDEBUG_LVL_FMT_1
	( 10, "tcl_globus_gass_transfer_send_bytes: last_data = %d",
	  last_data ); 
	
      retval = globus_gass_transfer_send_bytes( request,
						send_buffer,
						length,
						last_data,
						send_cb,
						send_user_arg );
      
      SGDEBUG_LVL_FMT_1
	    ( 10, "globus_gass_transfer_send_bytes returns: %d",
	      retval );
      
      return retval;
    }

  %}

/**************************************************************************
 * GLOBUS_GASS_TRANSFER_RECEIVE_BYTES
 **************************************************************************/
%typemap(in)
     (globus_gass_transfer_bytes_callback_t recv_cb, void * recv_user_arg)
     (callbackStruct * cbStruct)
{
  cbStruct = CreateCallback( interp, $input );

  /* Allocate our buffer */
  GlobusGASSTransferReceiveBytesInit( cbStruct, arg2 );

  $1 = TclGASSTransferReceiveBytesCallback; 
  $2 = cbStruct;

  cbStruct->s_pre_process = GlobusGASSTransferReceiveBytesPreCallback;
  cbStruct->s_post_process = GlobusGASSTransferReceiveBytesCleanup;
}

%rename(globus_gass_transfer_receive_bytes) tcl_globus_gass_transfer_receive_bytes;


%exception tcl_globus_gass_transfer_receive_bytes GLOBUS_EXCEPTION_RESULT_T;


%inline %{

  int
    tcl_globus_gass_transfer_receive_bytes
    ( globus_gass_transfer_request_t		request,
      long 					max_length,
      long 					wait_for_length,
      globus_gass_transfer_bytes_callback_t 	recv_cb,
      void * 					recv_user_arg )
    {
      int retval;

      retval = globus_gass_transfer_receive_bytes
	( request,
	  (globus_byte_t*)
	  ((callbackData*)(((callbackStruct*)recv_user_arg)->s_api_data))->buffer,
	  max_length,
	  wait_for_length,
	  recv_cb,
	  recv_user_arg );

      return retval;
    }
  
  %}

#endif /* SWIG */


/**************************************************************************
 *
 * ROBODOC
 *
 **************************************************************************/

/****f* globus_gass_transfer/globus_gass_transfer_get
 * NAME
 *   globus_gass_transfer_get
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_get
 *   |html </b>
 *   |html <i>
 *   attr
 *   url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a new "get" request of the file
 *   named by url. The entire file will be transfered from the server
 *   if the file exists and user is authorized to do so. This function
 *   blocks until the GASS library has determined whether the file may
 *   be retrievied by the caller, may not because it is a referral to
 *   another URL or URLs, or the server has denied the request.
 *
 *   Upon returning from this command, the request handle is initialized
 *   to refer to the get request's state. This request handle must be
 *   destroyed after the user has finished processing the data associated
 *   with the callback.
 *
 *   If the file doesn't exist at url, but a referral does, then this
 *   command will return with the request's status set to
 *   GLOBUS_GASS_TRANSFER_REQUEST_REFERRED.
 *
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Request attributes.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL to get.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_get returns 0 and request handle. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_get $attr $url ] result request_handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_put, globus_gass_transfer_append.
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_put
 * NAME
 *   globus_gass_transfer_put
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_put
 *   |html </b>
 *   |html <i>
 *   attr
 *   url
 *   length
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a new "put" request of the file
 *   named by url. The entire file will be transfered to the server if
 *   the user is authorized to do so. This command blocks until the
 *   GASS library has determined whether the file may be stored by the
 *   caller, may not because it is a referral to another URL or URLs,
 *   or the server has denied the request.
 *
 *   Upon returning from this command, the request handle is initialized
 *   to refer to the put request's state.
 *
 *   If the server can't store the file at url, but has an alternative
 *   location for the user to store to, then this command return with
 *   request's status set to GLOBUS_GASS_TRANSFER_REQUEST_REFERRED.
 *
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Request attributes.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL to put.
 *
 *   |html <b>
 *   length
 *   |html </b>
 *   - The length of the data to put to the url, if known. If this
 *   parameter is set to GLOBUS_GASS_TRANSFER_LENGTH_UNKNOWN, then the
 *   put may fail if the protocol does not support arbitrarily-length
 *   files.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_put returns 0 and request handle. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_put $attr $url 1024 ] result
 *   request_handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_get, globus_gass_transfer_append.
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_append
 * NAME
 *   globus_gass_transfer_append
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_append
 *   |html </b>
 *   |html <i>
 *   attr
 *   url
 *   length
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a new "append" request of the
 *   file named by url. The entire file will be transfered to the
 *   server if the user is authorized to do so. This function blocks
 *   until the GASS library has determined whether the file may be
 *   retrieved by the caller, may not because it is a referral to
 *   another URL or URLs, or the server has denied the request.
 *
 *   Upon returning from this function, the request handle is
 *   initialized to refer to the append request's state.
 *
 *   If the server can't store the file at url, but has an alternative
 *   location for the user to store to, then this function return with
 *   request's status set to GLOBUS_GASS_TRANSFER_REQUEST_REFERRED.
 *
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Request attributes.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL to append to.
 *
 *   |html <b>
 *   length
 *   |html </b>
 *   - The length of the data to append to the url, if known. If this
 *   parameter is set to GLOBUS_GASS_LENGTH_UNKNOWN, then the append
 *   may fail if the protocol does not support arbitrarily-length files.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_append returns 0 and request handle. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_append $attr $url 1024 ] result
 *   request_handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_get, globus_gass_transfer_put.
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_register_get
 * NAME
 *   globus_gass_transfer_register_get
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_register_get
 *   |html </b>
 *   |html <i>
 *   attr
 *   url
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a new "get" request of the file
 *   named by url. The entire file will be transfered from the server
 *   if the file exists and user is authorized to do so. This command
 *   does not block; instead, the user's script will be called once
 *   the GASS library has determined whether the file can be retrieved
 *   or not.
 *
 *   Upon returning from this command, the request handle is initialized
 *   to refer to the get request's state.
 *
 *   If the file doesn't exist at url, but a referral does, then this
 *   command will return with the request's status set to
 *   GLOBUS_GASS_TRANSFER_REQUEST_REFERRED.
 *
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Request attributes.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL to get.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the transfer is completed. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_register_get returns 0 and request
 *   handle. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_register_get $attr $url
 *   { callback_script user_arg } ] result request_handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_register_put, globus_gass_transfer_register_append.
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_register_put
 * NAME
 *   globus_gass_transfer_register_put
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_register_put
 *   |html </b>
 *   |html <i>
 *   attr
 *   url
 *   length
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a new "put" request of the file
 *   named by url. The entire file will be transfered to the server
 *   if the user is authorized to do so. This command does not block;
 *   instead, the user's script will be called once the GASS library
 *   has determined whether the file may be stored or not.
 *
 *   Upon returning from this command, the request handle is initialized
 *   to refer to the put request's state.
 *
 *   If the server can't store the file at url, but has an alternative
 *   location for the user to store to, then the script will be called
 *   with the request's status set to GLOBUS_GASS_TRANSFER_REQUEST_REFERRED.
 *
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Request attributes.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL to get.
 *
 *   |html <b>
 *   length
 *   |html </b>
 *   - The length of the data to put to the url, if known. If this
 *   parameter is set to GLOBUS_GASS_TRANSFER_LENGTH_UNKNOWN, then the
 *   put may fail if the protocol does not support arbitrarily-length
 *   files.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the transfer is completed. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_register_put returns 0 and request
 *   handle. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_register_put $attr $url 1024
 *   { callback_script user_arg } ] result request_handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_register_get, globus_gass_transfer_register_append.
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_register_append
 * NAME
 *   globus_gass_transfer_register_append
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_register_append
 *   |html </b>
 *   |html <i>
 *   attr
 *   url
 *   length
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a new "append" request of the
 *   file named by url. The entire file will be transfered to the
 *   server if the user is authorized to do so. This function does
 *   not block; instead, the user's callback function will be called
 *   once the GASS library has determined whether the file may be
 *   stored or not.
 * 
 *   Upon returning from this function, the request handle is
 *   initialized to refer to the append request's state.
 *
 *   If the server can't store the file at url, but has an alternative
 *   location for the user to store to, then the callback function
 *   will be called with the request's status set to
 *   GLOBUS_GASS_TRANSFER_REQUEST_REFERRED.
 *
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Request attributes.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL to get.
 *
 *   |html <b>
 *   length
 *   |html </b>
 *   - The length of the data to append to the url, if known. If
 *   this parameter is set to GLOBUS_GASS_LENGTH_UNKNOWN, then the
 *   append may fail if the protocol does not support
 *   arbitrarily-length files.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the transfer is completed. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_register_append returns 0 and request
 *   handle. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_register_append $attr $url 1024
 *   { callback_script user_arg } ] result request_handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_register_get, globus_gass_transfer_register_put.
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_create_listener
 * NAME
 *   globus_gass_transfer_create_listener
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_create_listener
 *   |html </b>
 *   |html <i>
 *   listener_attr
 *   url_scheme
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create a new protocol-specific socket to
 *   listen for client connections to a GASS server. The listener
 *   handle pointer is initialized to contain the a new handle which
 *   can be used in subsequent server operations.
 *
 *   After calling this command, a user may call the
 *   globus_gass_transfer_register_listen or
 *   globus_gass_transfer_close_listener commands with this listener
 *   handle.
 *
 *   |html <ul><b>
 *   listener_attr
 *   |html </b>
 *   - Protocol-specific attributes for the new listener.
 *
 *   |html <b>
 *   url_scheme
 *   |html </b>
 *   - The protocol scheme to implement for the listener.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_create_listener returns 0 and
 *   listener handle. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_create_listener $listener_attr $url_scehme ] *   result listener_handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_close_listener.
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_close_listener
 * NAME
 *   globus_gass_transfer_close_listener
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_close_listener
 *   |html </b>
 *   |html <i>
 *   listener
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to close a GASS server listener port.
 *   Callbacks for any outstanding accepts will be called before
 *   the close callback is invoked.
 *
 *   |html <ul><b>
 *   listener
 *   |html </b>
 *   - Listener handle created by calling
 *   globus_gass_transfer_create_listener
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the transfer is completed. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_close_listener returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_close_listener $listener
 *   { callback_script user_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_create_listener.
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_register_listen
 * NAME
 *   globus_gass_transfer_register_listen
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_register_listen
 *   |html </b>
 *   |html <i>
 *   listener
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command causes the listener handle to listen for new client
 *   connections. When one is ready, it calls the specified callback
 *   script, letting the server implementer continue to accept the
 *   connection and process the request.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener
 *   |html </b>
 *   - The listener handle to register for new connections.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the connection operation is pending.
 *   The command will be executed exactly once in global scope. If
 *   an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_gass_transfer_register_listen returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_register_listen $listener
 *   { callback_script callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_create_listener,
 *   globus_gass_transfer_close_listener.
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_register_accept
 * NAME
 *   globus_gass_transfer_register_accept
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_register_accept
 *   |html </b>
 *   |html <i>
 *   attr
 *   listener
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command causes the listener handle to accept a new connection
 *   on the listener and parse the file request. Once the file request
 *   has been parsed, the specified callback function will be called.
 *   The server implementation must then either authorize, deny, or
 *   refer this request.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Protocol-specific attributes for the new listener.
 *
 *   |html <b>
 *   listener
 *   |html </b>
 *   - The listener handle to register for the new request.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the protocol module has parsed the file
 *   request. The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_gass_transfer_register_accept returns 0 and
 *   the new request handle to be initialized when the operation is completed.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_register_accept $attr $listener
 *   { callback callback_arg } ] result request_handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_listener_get_user_pointer
 * NAME
 *   globus_gass_transfer_listener_get_user_pointer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_listener_get_user_pointer
 *   |html </b>
 *   |html <i>
 *   listener
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the listener's user_pointer field and
 *   return it's value.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener
 *   |html </b>
 *   - The listener handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_listener_get_user_pointer returns
 *   the user-specified pointer in the handle structure. On error, this
 *   command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_listener_get_user_pointer $listener ]
 *   user-pointer
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_listener_set_user_pointer
 * NAME
 *   globus_gass_transfer_listener_set_user_pointer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_listener_set_user_pointer
 *   |html </b>
 *   |html <i>
 *   listener
 *   user_pointer
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the listener's user_pointer field.
 *   The pointer may be used to associate any pointer-sized data with
 *   a listener handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener
 *   |html </b>
 *   - The listener handle.
 *
 *   |html <b>
 *   user-pointer
 *   |html </b>
 *   - The value of the user pointer.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_listener_set_user_pointer returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_listener_set_user_pointer $listener
 *   $user-pointer ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_listener_get_base_url
 * NAME
 *   globus_gass_transfer_listener_get_base_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_listener_get_base_url
 *   |html </b>
 *   |html <i>
 *   listener
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query a listener handle for the base URL
 *   which the server is listening on. For most protocols, this contains
 *   the protocol scheme, host, and port that the listener has registered
 *   itself on.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener
 *   |html </b>
 *   - The listener handle to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_listener_get_base_url returns
 *   a string containing the base URL. On error, this
 *   command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_listener_get_base_url $listener ]
 *   base_url
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_refer
 * NAME
 *   globus_gass_transfer_refer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_refer
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   urls
 *   num_urls
 *   |html </i>
 *
 * DESCRIPTION
 *   This command causes the request to be referred to another URL or
 *   list of URLs. It should be called in response to a request accept
 *   callback when the server wants to refer the client to another server
 *   or servers to process the request.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - A new request handle, passed to the server in an accept callback.
 *
 *   |html <b>
 *   urls
 *   |html </b>
 *   - An array of strings, each being a URL pointing to sources of the
 *   same data as the original URL.
 *
 *   |html <b>
 *   num_urls
 *   |html </b>
 *   - The length of the urls array.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_refer returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_refer $request_handle $urls $num_urls ]
 *   status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_authorize, globus_gass_transfer_deny.
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_authorize
 * NAME
 *   globus_gass_transfer_authorize
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_authorize
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_authorize
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   total_length
 *   |html </i>
 *
 * DESCRIPTION
 *   This command causes the request to be authorized for processing.
 *   It should be called in response to a request accept callback when
 *   the server wants to agree to process this request. After calling
 *   this command, the server implementation should call
 *   globus_gass_transfer_send_bytes or globus_gass_transfer_receive_bytes
 *   to send or receive the data associated with the URL.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - A new request handle, passed to the server in an accept callback.
 *
 *   |html <b>
 *   total_length
 *   |html </b>
 *   - For a "get" request, the total_length of the file to be retrieved
 *   if known. This value may be GLOBUS_GASS_LENGTH_UNKNOWN if the protocol
 *   supports transferring arbitrarily-sized files.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_authorize returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_authorize $request_handle $total_length ]
 *   status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_refer, globus_gass_transfer_deny.
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_deny
 * NAME
 *   globus_gass_transfer_deny
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_deny
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   reason
 *   message
 *   |html </i>
 *
 * DESCRIPTION
 *   This command causes the request to be denied for further processing.
 *   It should be called in response to a request ccept callback when the
 *   server wants to refuse processing this request for the client. After
 *   calling this function, the server implementation need do nothing
 *   further with the request handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - A new request handle, passed to the server in an accept callback.
 *
 *   |html <b>
 *   reason
 *   |html </b>
 *   - A protocol-specific reason code.
 *
 *   |html <b>
 *   message
 *   |html </b>
 *   - An informational message to be sent to the client.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_deny returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_deny $request_handle $reason $message ]
 *   status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_authorize, globus_gass_transfer_refer.
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_send_bytes
 * NAME
 *   globus_gass_transfer_send_bytes
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_send_bytes
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   buffer
 *   length
 *   last_data
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to send a byte array associated with
 *   a request handle.
 *
 *   This command sends a block of data to a server or client as part of
 *   the processing for a request. Multiple data blocks may be registered
 *   with the GASS transfer library at once.
 *
 *   When processing a server request, this command may only be used in
 *   conjunction with "get" requests. The user must call
 *   globus_gass_transfer_authorize before calling this command.
 *
 *   When processing a client request, this command may only be used in
 *   conjunction with "put" or "append" requests. This command may not be
 *   called before either the callback function has been invoked, or the
 *   blocking globus_gass_tranfser_put or globus_gass_transfer_append
 *   command has returned.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle with which this block of bytes is associated.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - A user-supplied buffer containing the data associated with the
 *   request.
 *
 *   |html <b>
 *   length
 *   |html </b>
 *   - The length of the bytes array.
 *
 *   |html <b>
 *   last_data
 *   |html </b>
 *   - A flag to indicate whether this is the final block of data for
 *   the request. If this is true, then the callback function will be
 *   delayed until the server acknowledges that the file has been
 *   completely received.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the operation is completed. The command
 *   will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_send_bytes returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_send_bytes $request_handle $buffer $length
 *   $last_data { callback_script callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_receive_bytes.
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_receive_bytes
 * NAME
 *   globus_gass_transfer_receive_bytes
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_receive_bytes
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   buffer_length
 *   wait_for_length
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to receive a byte array associated with
 *   a request handle.
 *
 *   This command receives a block of data from a server or client
 *   as part of the processing for a request. Multiple data blocks
 *   may be registered with the GASS transfer library at once.
 *
 *   When processing a server request, this command may only be used
 *   in conjunction with "put" or "append" requests. The user must
 *   call globus_gass_transfer_authorize before calling this command.
 *
 *   When processing a client request, this command may only be used
 *   in conjunction with "get" requests. This comamnd may not be called
 *   before either the callback function has been invoked, or the blocking
 *   globus_gass_tranfser_put or globus_gass_transfer_append command has
 *   returned.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle with which this block of bytes is associated.
 *
 *   |html <b>
 *   buffer_length
 *   |html </b>
 *   - The length of the bytes array.
 *
 *   |html <b>
 *   wait_for_length
 *   |html </b>
 *   - The minimum amount of data to wait for before invoking the callback
 *   script. A partial byte array of at least this amount will be returned
 *   in the callback, unless end-of-file is reached before this amount..
 *
 *   |html <b>
 *   last_data
 *   |html </b>
 *   - A flag to indicate whether this is the final block of data for
 *   the request. If this is true, then the callback function will be
 *   delayed until the server acknowledges that the file has been
 *   completely received.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the operation is completed. The command
 *   will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_send_bytes returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_send_bytes $request_handle $buffer $length
 *   $last_data { callback_script callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_receive_bytes.
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_referral_get_count
 * NAME
 *   globus_gass_transfer_referral_get_count
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_referral_get_count
 *   |html </b>
 *   |html <i>
 *   referral
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to get the number of URLs in this referral.
 *
 *   This command examines the referral to determine if the number of
 *   URLs which are contained in it. Each of these URLs should either
 *   point to another referral, or to a URL containing the equivalent
 *   file as the original URL request which caused this referral.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   referral
 *   |html </b>
 *   - The referral structure to query.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_gass_transfer_referral_get_count returns the
 *   number of URL entries in the referral, or 0, if there are none.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_referral_get_count $referral ] num_urls
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_referral_get_url
 * NAME
 *   globus_gass_transfer_referral_get_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_referral_get_url
 *   |html </b>
 *   |html <i>
 *   referral
 *   index
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to get a URL string from a referral.
 *
 *   This command examines the referral to retrieve a URL string from it.
 *   A valid referal will contain one or more strings. They are indexed
 *   from 0 to the value returned by
 *   globus_gass_transfer_referral_get_count - 1.
 *
 *   The string returned by this command must not be freed by the caller.
 *   It will remain valid until the referral structure is destroyed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   referral
 *   |html </b>
 *   - The referral structure to query.
 *
 *   |html <b>
 *   index
 *   |html </b>
 *   - The URL to extract from the referral.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_gass_transfer_referral_get_url returns a string
 *   pointer containing the URL, or NULL if the index or referral were
 *   invalid. 
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_referral_get_url $referral $index] url
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_request_get_type
 * NAME
 *   globus_gass_transfer_request_get_type
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_get_type
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to determine the type of a request.
 *
 *   This command is used by GASS server implementations to discover
 *   what type of operation the client is requesting for an URL.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request to query.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_gass_transfer_request_get_type returns a string
 *   containing the type of the request.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_get_type $request_handle ]
 *   request_type
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_request_get_user_pointer
 * NAME
 *   globus_gass_transfer_request_get_user_pointer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_get_user_pointer
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the user pointer associated with a request.
 *
 *   This command extracts the user pointer from a request handle. The
 *   user-pointer may be used by the application which is generating or
 *   servicing the request to store a pointer to any application-specific
 *   piece of data.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_get_user_pointer returns
 *   the user-specified pointer in the handle structure. On error, this
 *   command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_get_user_pointer $request ]
 *   user-pointer
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_request_set_user_pointer
 * NAME
 *   globus_gass_transfer_request_set_user_pointer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_set_user_pointer
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   user_pointer
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the user pointer associated with a
 *   request handle.
 *
 *   This command sets the user pointer from a request handle. The
 *   user-pointer may be used by the application which is generating or
 *   servicing the request to store a pointer to any application-specific
 *   piece of data.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle to modify.
 *
 *   |html <b>
 *   user-pointer
 *   |html </b>
 *   - The new value of the user pointer for the request.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_set_user_pointer returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_set_user_pointer $request
 *   $user-pointer ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_request_get_status
 * NAME
 *   globus_gass_transfer_request_get_status
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_get_status
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to check the status of a request.
 *
 *   This command queries a request to determine the status of the request.
 *   This command should be called after EOF has been reached, or after the
 *   initial get, put, or append has returned or had it's callback script
 *   called to determine if it is possible to procede, or whether the file
 *   transfer was successfully processed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_get_status returns the
 *   current status of the request. On error, this
 *   command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_get_status $request ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_request_get_referral
 * NAME
 *   globus_gass_transfer_request_get_referral
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_get_referral
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to extract referral information from a request
 *   handle.
 *
 *   This command queries the request handle to determine any referral
 *   information that it contains. This command should only be called on
 *   request handles in the GLOBUS_GASS_TRANSFER_REQUEST_REFERRED state.
 *   If no referral information is stored in the request handle, then the
 *   referral will be initialized to an empty referral. The referral must
 *   be destroyed by calling globus_gass_transfer_referral_destroy by the
 *   caller.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_get_referral returns 0 and the
 *   referral handle. On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_get_referral $request_handle ]
 *   status referral_handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_referral_destroy
 * NAME
 *   globus_gass_transfer_referral_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_referral_destroy
 *   |html </b>
 *   |html <i>
 *   referral_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to free all memory used by a referral.
 *
 *   This command frees all memory used by this referral. After calling
 *   this command, the strings returned by calling
 *   globus_gass_transfer_referral_get_url must not be accessed.
 *   Any further attempts to extract information from this referral will fail.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   referral_handle
 *   |html </b>
 *   - The referral handle to destroy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_referral_destroy returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_referral_destroy $referral_handle ]
 *   status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_request_get_url
 * NAME
 *   globus_gass_transfer_request_get_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_get_url
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the URL from a request handle.
 *
 *   This command queries the request handle to determine the URL
 *   associated with the request. This command is intended to be useful
 *   to GASS server implementors.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_get_url returns 0 and the
 *   URL associated with the request.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_get_url $request_handle ]
 *   status url
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_request_get_length
 * NAME
 *   globus_gass_transfer_request_get_length
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_get_length
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the length of a file to be transferred
 *   using GASS.
 *
 *   This command queries the request handle to determine the amount of
 *   data that will be transferred to copy the URL. The length may be
 *   GLOBUS_GASS_TRANSFER_LENGTH_UNKNOWN if the sender can not determine
 *   the length before making or authorizing the request.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_get_length returns 0 and
 *   the length of the file located at the request's URL, or
 *   GLOBUS_GASS_TRANSFER_LENGTH_UNKNOWN if that cannot be determined.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_get_length $request_handle ]
 *   status file_length
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_request_set_type
 * NAME
 *   globus_gass_transfer_request_set_type
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_set_type
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   request_type
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the type of a request.
 *
 *   This command modifies a request handle by setting the type of
 *   operation that it is being used for. This command may only be
 *   called once per handle, and only from a GASS protocol module
 *   implementation.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle to modify.
 *
 *   |html <b>
 *   request_type
 *   |html </b>
 *   - The type of operation that this request handle will be used for.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_set_type returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_set_type $request $type ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_request_set_url
 * NAME
 *   globus_gass_transfer_request_set_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_set_url
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the URL to which a request handle refers.
 *
 *   This command modifies the given request handle so that it's URL field
 *   is set to string pointed to by url.
 *
 *   No copy is made of the string, so the caller must not free it. It must
 *   be allocated by calling one of the memory allocators in globus_libc,
 *   as it will be freed when the request handle is destroyed.
 *
 *   This command must only be called by protocol modules when constructing
 *   a request handle when accepting a new request. This command can only be
 *   called once per request handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle to modify.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - A string containing the URL that this request will be associated with.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_set_url returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_set_url $request $url ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_request_set_length
 * NAME
 *   globus_gass_transfer_request_set_length
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_set_length
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   length
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the length of a transfer associated
 *   request handle.
 *
 *   This command modifies the given request handle so that it's length
 *   field is set to give length parameter.
 *
 *   This command must only be called by protocol modules when constructing
 *   a request handle when receiving the response to a get request. This
 *   command can only be called once per request handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle to modify.
 *
 *   |html <b>
 *   length
 *   |html </b>
 *   - The length of the file request.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_set_length returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_set_length $request $length ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_request_get_denial_reason
 * NAME
 *   globus_gass_transfer_request_get_denial_reason
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_get_denial_reason
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get an integer code describing why the request
 *   was denied.
 *
 *   This command queries a request which was denied by a server to determine
 *   why it was denied. The denial reason will be expressed in a
 *   protocol-specific response code. Knowledge of the protocol is needed
 *   to understand this response.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - A handle to the request to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_get_denial_reason returns
 *   a protocol-specific integer indicating why the request was denied.
 *   If the request handle is invalid or the request was not denied,
 *   then this command returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_get_denial_reason $request_handle ]
 *   denial_code
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_request_get_denial_message
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_request_get_denial_message
 * NAME
 *   globus_gass_transfer_request_get_denial_message
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_get_denial_message
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get an string describing why a request was
 *   denied.
 *
 *   This command queries a request which was denied by a server to
 *   determine why it was denied. The denial reason will be expressed as
 *   a response string. The string must be freed by the caller.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - A handle to the request to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_get_denial_message returns
 *   a string indicating why the request was denied. If the request handle
 *   is invalid or the request was not denied, then this command returns
 *   GLOBUS_NULL.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_get_string $request_handle ]
 *   denial_string
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_request_get_denial_reason
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_request_get_subject
 * NAME
 *   globus_gass_transfer_request_get_subject
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_get_subject
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the subject string associated with a request.
 *
 *   This command queries a request handle to determine the subject identity
 *   of the client who initiated the request. The string must not be freed
 *   by the caller.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - A handle to the request to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_get_subject returns
 *   a string containing the request initiator's subject identity.
 *   If the request handle is invalid or a credential was not used
 *   to initiate the request, this value will be GLOBUS_NULL.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_get_subject $request_handle ]
 *   subject_string
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_request_destroy
 * NAME
 *   globus_gass_transfer_request_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_request_destroy
 *   |html </b>
 *   |html <i>
 *   request_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a request handle.
 *
 *   This command destroys the caller's reference to a request handle.
 *   It must be called for all request handles which are created by
 *   calling commands in the globus_gass_transfer_client or
 *   globus_gass_transfer_server. After calling the command, the caller
 *   must not attempt to use the request handle for any purpose.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_handle
 *   |html </b>
 *   - The request handle to destroy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_request_destroy returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_request_destroy $request_handle ]
 *   status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_requestattr_init
 * NAME
 *   globus_gass_transfer_requestattr_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_init
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   url_scheme
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize the attributes to contain a
 *   new protocol-specific request attributes.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be initialized.
 *
 *   |html <b>
 *   url_scheme
 *   |html </b>
 *   - The scheme which which the attribute will be used for.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_init returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_init $request_attr http ]
 *   status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_destroy
 * NAME
 *   globus_gass_transfer_requestattr_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_destroy
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy the attribute set specified in
 *   request_attr.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be destroyed.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_destroy returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_destroy $request_attr ]
 *   status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_set_proxy_url
 * NAME
 *   globus_gass_transfer_requestattr_set_proxy_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_set_proxy_url
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   proxy_url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the proxy server attribute for
 *   a GASS transfer attribute set.
 *
 *   This attribute allows the user to use a proxy server to handle
 *   a URL request.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be modified.
 *
 *   |html <b>
 *   proxy_url
 *   |html </b>
 *   - The new value of the proxy_url attribute. This may be GLOBUS_NULL
 *   if no proxy is to be used to access URLs with this attribute set.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_set_proxy_url returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_set_proxy_url $request_attr
 *   $proxy_url ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_get_proxy_url
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_requestattr_set_block_size
 * NAME
 *   globus_gass_transfer_requestattr_set_block_size
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_set_block_size
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   block_size
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the block size attribute for a GASS
 *   transfer attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be modified.
 *
 *   |html <b>
 *   block_size
 *   |html </b>
 *   - The data block size that should be used to process requests
 *   with this attribute set.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_set_block_size returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_set_block_size $request_attr
 *   1024 ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_get_block_size
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_requestattr_set_file_mode
 * NAME
 *   globus_gass_transfer_requestattr_set_file_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_set_file_mode
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   file_mode
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the file mode attribute for a GASS
 *   transfer attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be modified.
 *
 *   |html <b>
 *   file_mode
 *   |html </b>
 *   - The value of the file mode attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_set_file_mode returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_set_file_mode $request_attr
 *   $GLOBUS_GASS_TRANSFER_FILE_MODE_TEXT ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_get_file_mode
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_set_connection_reuse
 * NAME
 *   globus_gass_transfer_requestattr_set_connection_reuse
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_set_connection_reuse
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   connection_reuse
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the connection reuse attribute for
 *   a GASS transfer attribute set.
 *
 *   This attribute allows the user to control whether the connection
 *   associated with a GASS Transfer request should be reused after the
 *   file transfer has completed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be modified.
 *
 *   |html <b>
 *   file_mode
 *   |html </b>
 *   - The value of the connection reuse attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_set_connection_reuse
 *   returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_set_connection_reuse
 *   $request_attr $GLOBUS_TRUE ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_get_connection_reuse
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_requestattr_set_socket_sndbuf
 * NAME
 *   globus_gass_transfer_requestattr_set_socket_sndbuf
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_set_socket_sndbuf
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   sndbuf
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the send buffer size attribute for
 *   a GASS transfer attribute set.
 *
 *   This attribute allows the user to control the socket send buffer
 *   associated with a GASS Transfer request should be reused after
 *   the file transfer has completed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be modified.
 *
 *   |html <b>
 *   sndbuf
 *   |html </b>
 *   - The value of the connection reuse attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_set_socket_sndbuf
 *   returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_set_socket_sndbuf
 *   $request_attr 1024 ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_get_socket_sndbuf
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_requestattr_set_socket_rcvbuf
 * NAME
 *   globus_gass_transfer_requestattr_set_socket_rcvbuf
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_set_socket_rcvbuf
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   rcvbuf
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the receive buffer size attribute for
 *   a GASS transfer attribute set.
 *
 *   This attribute allows the user to control the socket receive buffer
 *   associated with a GASS Transfer request should be reused after
 *   the file transfer has completed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be modified.
 *
 *   |html <b>
 *   rcvbuf
 *   |html </b>
 *   - The value of the socket buffer.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_set_socket_rcvbuf
 *   returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_set_socket_rcvbuf
 *   $request_attr 1024 ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_get_socket_rcvbuf
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_set_socket_nodelay
 * NAME
 *   globus_gass_transfer_requestattr_set_socket_nodelay
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_set_socket_nodelay
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   nodelay
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the TCP nodelay attribute for a GASS
 *   transfer attribute set.
 *
 *   This attribute allows the user to control the socket receive buffer
 *   associated with a GASS Transfer request should be reused after the
 *   file transfer has completed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be modified.
 *
 *   |html <b>
 *   nodelay
 *   |html </b>
 *   - The value of the nodelay attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_set_socket_nodelay
 *   returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_set_socket_nodelay
 *   $request_attr $GLOBUS_TRUE ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_get_socket_nodelay
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_set_authorization
 * NAME
 *   globus_gass_transfer_requestattr_set_authorization
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_set_authorization
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   mode
 *   subject
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the authorization attribute for
 *   a GASS transfer attribute set.
 *
 *   This attribute allows the user to control what type of authorization
 *   should be done when GASS Transfer requests are processed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be modified.
 *
 *   |html <b>
 *   mode
 *   |html </b>
 *   - The authorization mode to use.
 *   |html
 *
 *   |html <b>
 *   subject
 *   |html </b>
 *   - The subject name of the authorized subject, if mode is
 *   GLOBUS_GASS_TRANSFER_AUTHORIZE_SUBJECT
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_set_authorization
 *   returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_set_authorization
 *   $request_attr $mode $subject ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_get_authorization
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_get_proxy_url
 * NAME
 *   globus_gass_transfer_requestattr_get_proxy_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_get_proxy_url
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the proxy server attribute for
 *   a GASS transfer attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be queried.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_get_proxy_url returns 0
 *   and the proxy_url attribute.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_get_proxy_url $request_attr ]
 *   status proxy_url
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_set_proxy_url
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_get_block_size
 * NAME
 *   globus_gass_transfer_requestattr_get_block_size
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_get_block_size
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the block size attribute for a GASS
 *   transfer attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be queried.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_get_block_size returns 0
 *   and block size.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_get_block_size $request_attr ]
 *   status block_size
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_set_block_size
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_requestattr_get_file_mode
 * NAME
 *   globus_gass_transfer_requestattr_get_file_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_get_file_mode
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the file mode attribute for a GASS
 *   transfer attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be queried.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_get_file_mode returns 0
 *   and file mode.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_get_file_mode $request_attr ]
 *   status filemode
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_set_file_mode
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_get_connection_reuse
 * NAME
 *   globus_gass_transfer_requestattr_get_connection_reuse
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_get_connection_reuse
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the connection reuse attribute for
 *   a GASS transfer attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be queried.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_get_connection_reuse
 *   returns 0 and the value of the connection reuse attribute.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_get_connection_reuse
 *   $request_attr ] status connection_reuse
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_set_connection_reuse
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_get_socket_sndbuf
 * NAME
 *   globus_gass_transfer_requestattr_get_socket_sndbuf
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_get_socket_sndbuf
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the send buffer size attribute for
 *   a GASS transfer attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be queried.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_get_socket_sndbuf
 *   returns 0 and the value of the socket buffer.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_get_socket_sndbuf
 *   $request_attr ] status sndbuf
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_set_socket_sndbuf
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_requestattr_get_socket_rcvbuf
 * NAME
 *   globus_gass_transfer_requestattr_get_socket_rcvbuf
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_get_socket_rcvbuf
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the receive buffer size attribute for
 *   a GASS transfer attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be queried.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_get_socket_rcvbuf
 *   returns 0 and the value of the socket buffer.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_get_socket_rcvbuf
 *   $request_attr ] status rcvbuf
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_set_socket_rcvbuf
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_get_socket_nodelay
 * NAME
 *   globus_gass_transfer_requestattr_get_socket_nodelay
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_get_socket_nodelay
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the TCP nodelay attribute for
 *   a GASS transfer attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be queried.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_get_socket_nodelay
 *   returns 0 and the value of the nodelay attribute.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_get_socket_nodelay
 *   $request_attr ] status nodelay
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_set_socket_nodelay
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_requestattr_get_authorization
 * NAME
 *   globus_gass_transfer_requestattr_get_authorization
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_requestattr_get_authorization
 *   |html </b>
 *   |html <i>
 *   request_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the authorization attribute for
 *   a GASS transfer attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   request_attr
 *   |html </b>
 *   - The attribute set to be queried.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_requestattr_get_authorization
 *   returns 0, mode and subject.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_requestattr_get_authorization
 *   $request_attr ] status mode subject 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_requestattr_set_authorization
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_listenerattr_init
 * NAME
 *   globus_gass_transfer_listenerattr_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_listenerattr_init
 *   |html </b>
 *   |html <i>
 *   listener_attr
 *   url_scheme
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize the listener attributes to
 *   contain a new protocol-specific listener attribute.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_attr
 *   |html </b>
 *   - The listener attribute set to be initialized.
 *
 *   |html <b>
 *   url_scheme
 *   |html </b>
 *   - The scheme which which the attribute will be used for.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_listenerattr_init returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_listenerattr_init $listener_attr http ]
 *   status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/




/****f* globus_gass_transfer/globus_gass_transfer_listenerattr_destroy
 * NAME
 *   globus_gass_transfer_listenerattr_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_listenerattr_destroy
 *   |html </b>
 *   |html <i>
 *   listener_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy the attribute set specified in
 *   listener_attr.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_attr
 *   |html </b>
 *   - The attribute set to be destroyed.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_listenerattr_destroy returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_listenerattr_destroy $listener_attr ]
 *   status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_listenerattr_set_backlog
 * NAME
 *   globus_gass_transfer_listenerattr_set_backlog
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_listenerattr_set_backlog
 *   |html </b>
 *   |html <i>
 *   listener_attr
 *   backlog
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the backlog attribute for a GASS transfer
 *   attribute set.
 *
 *   This attribute allows the user to control the number of pending
 *   connections which may exist for this listener.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_attr
 *   |html </b>
 *   - The attribute set to be modified.
 *
 *   |html <b>
 *   backlog
 *   |html </b>
 *   - The number of outstanding connections to allow.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_listenerattr_set_backlog
 *   returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_listenerattr_set_backlog
 *   $listener_attr 1 ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_listenerattr_get_backlog
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_listenerattr_set_port
 * NAME
 *   globus_gass_transfer_listenerattr_set_port
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_listenerattr_set_port
 *   |html </b>
 *   |html <i>
 *   listener_attr
 *   port
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the port attribute for a GASS transfer
 *   attribute set.
 *
 *   This attribute allows the user to set the port to be used by a GASS
 *   Transfer listener.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_attr
 *   |html </b>
 *   - The attribute set to be modified.
 *
 *   |html <b>
 *   port
 *   |html </b>
 *   - The TCP or UDP port number to use.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_listenerattr_set_port
 *   returns 0.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_listenerattr_set_port
 *   $listener_attr 50000 ] status
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_listenerattr_get_port
 *****/


/****f* globus_gass_transfer/globus_gass_transfer_listenerattr_get_backlog
 * NAME
 *   globus_gass_transfer_listenerattr_get_backlog
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_listenerattr_get_backlog
 *   |html </b>
 *   |html <i>
 *   listener_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the backlog attribute for a GASS transfer
 *   attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_attr
 *   |html </b>
 *   - The attribute set to be queried.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_listenerattr_get_backlog
 *   returns 0 and backlog.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_listenerattr_get_backlog
 *   $listener_attr ] status backlog
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_listenerattr_set_backlog
 *****/



/****f* globus_gass_transfer/globus_gass_transfer_listenerattr_get_port
 * NAME
 *   globus_gass_transfer_listenerattr_get_port
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gass_transfer
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gass_transfer_listenerattr_get_port
 *   |html </b>
 *   |html <i>
 *   listener_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the port attribute for a GASS transfer
 *   attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_attr
 *   |html </b>
 *   - The attribute set to be queried.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gass_transfer_listenerattr_get_port
 *   returns 0 and port.
 *   On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gass_transfer_listenerattr_get_port
 *   $listener_attr ] status port
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gass_transfer_listenerattr_set_port
 *****/

