/* -*- mode: c++; c-basic-offset: 2; -*- */

#ifdef SWIG

#include "../../config.h"

/*****************************************************************
 * Hand-wrapped SWIG functions
 *****************************************************************/

%ignore globus_gram_client_attr_init;
%ignore globus_gram_client_attr_destroy;

%ignore globus_gram_client_attr_get_credential;

%include "tcl-typemaps.i"

%include "swig/globus_gram_client-4.0.i"


%{
#include <stdio.h>
#include <tcl.h>

#include "tcl-callback.h"
#include "common/sgdebug.h"
#include "common/sginit.h"

  /*
   * ----------------------------------------------------------------------
   * Callback support
   * ----------------------------------------------------------------------
   */
  
  static void
    gs_gram_client_init( callbackStruct* cb )
    {
      cb->s_api_data = (int*)globus_libc_malloc( sizeof(int) );
    }
  
  static int
    gs_gram_client_post_process( callbackStruct* cb )
    {
      int retval = 0;
      int job_state = *(int*)(cb->s_api_data);
      
      if ( ( job_state == GLOBUS_GRAM_PROTOCOL_JOB_STATE_FAILED ) ||
	   ( job_state == GLOBUS_GRAM_PROTOCOL_JOB_STATE_DONE ) )
	{
	  globus_libc_free( cb->s_api_data );
	  cb->s_api_data = NULL;
	  retval = 1;
	}
      return retval;
    }

  static void
    TclGramCallback( void* user_callback_arg,
		     char* job_contact,
		     int state,
		     int errorcode )
    {
      callbackStruct*	callback = (callbackStruct*)user_callback_arg;
      
      *((int*)callback->s_api_data) = state;

      CallCallback( callback,
		    "%s %d %d",
		    job_contact, state, errorcode );
    }

  
  static void
    TclGramNonblockingCallback( void * user_callback_arg,
				globus_gram_protocol_error_t operation_failure_code,
				const char *job_contact,
				globus_gram_protocol_job_state_t job_state,
				globus_gram_protocol_error_t job_failure_code )
    {
      callbackStruct*	callback = (callbackStruct*)user_callback_arg;

      *((int*)callback->s_api_data) = job_state;
      
      CallCallback( callback,
		    "%s %d %d",
		    job_contact, job_state, job_failure_code );
    }
  
  %}


/**************************************************************************
 *
 *  SWIG INITIALIZATION STUFF
 *
 **************************************************************************/
%init %{

  SGINIT_X509;

  %}


/**************************************************************************
 **************************************************************************
 *
 *				TYPEMAPS SECTION
 *
 **************************************************************************
 **************************************************************************/

/**************************************************************************
 *  GLOBUS_GRAM_CLIENT_ATTR_INIT
 **************************************************************************/
%typemap(in,numinputs=0) globus_gram_client_attr_t * outattr
{
  $1 = (globus_gram_client_attr_t *)
    globus_libc_malloc(sizeof(globus_gram_client_attr_t));

  SGDEBUG_LVL_FMT_1
    ( 10, "globus_gram_client_attr_t creation: typemap(in): %p", $1 );  
}

%typemap(argout,fragment="t_output_helper")
  globus_gram_client_attr_t * outattr 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_gram_client_attr_init) tcl_globus_gram_client_attr_init;

%exception tcl_globus_gram_client_attr_init GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gram_client_attr_init
    ( globus_gram_client_attr_t * outattr )
    {
      int retval;

      retval = globus_gram_client_attr_init( outattr );

      return retval;
    }
  %}

/**************************************************************************
 *  GLOBUS_GRAM_CLIENT_ATTR_DESTROY
 **************************************************************************/
%typemap(freearg) globus_gram_client_attr_t * inattr
{
  SGDEBUG_LVL_FMT_1
      ( 10, "globus_gram_client_attr_t deletion: typemap(in): %p", arg1 );
  if ( arg1 )
    {
      globus_libc_free( arg1 );
    }
}

%rename(globus_gram_client_attr_destroy) tcl_globus_gram_client_attr_destroy;

%exception tcl_globus_gram_client_attr_destroy GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  int
    tcl_globus_gram_client_attr_destroy
    ( globus_gram_client_attr_t * inattr )
  {
    int retval;
    
    retval = globus_gram_client_attr_destroy( inattr );
    
    return retval;
  }
  %}

/**************************************************************************
 *  GLOBUS_GRAM_CLIENT_GET_CREDENTIAL
 **************************************************************************/
%typemap(in,numinputs=0) gss_cred_id_t * out_credential ( $*1_ltype temp )
{
  $1 = (gss_cred_id_t *)&temp;
}

%typemap( argout ) gss_cred_id_t * out_credential
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_gram_client_attr_get_credential) tcl_globus_gram_client_attr_get_credential;

%inline %{

  int
    tcl_globus_gram_client_attr_get_credential
    ( globus_gram_client_attr_t			attr,
      gss_cred_id_t *				out_credential )
  {
    int retval;

    retval = globus_gram_client_attr_get_credential( attr,
						     out_credential );

    return retval;
  }
  
  %}

#endif /* SWIG */


/**************************************************************************
 *
 * ROBODOC
 *
 **************************************************************************/


/****f* globus_gram_client/globus_gram_client_version
 * NAME
 *   globus_gram_client_version
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_version
 *   |html </b>
 *
 * DESCRIPTION
 *   This command is used to check the version of the GRAM protocol
 *   which this implementation of the GRAM client library understands.
 *
 * RETURN VALUE
 *   On success globus_gram_client_version returns an integer representing
 *   the protocol version. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gram_client_version ] result version
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gram_client/globus_gram_client_job_contact_free
 * NAME
 *   globus_gram_client_job_contact_free
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_job_contact_free
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to release the resources storing a job contact
 *   string.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - A job contact string returned in a successful call to
 *   globus_gram_client_job_request.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_job_contact_free returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gram_client_job_contact_free $job_contact ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_job_request
 *****/


/****f* globus_gram_client/globus_gram_client_callback_allow
 * NAME
 *   globus_gram_client_callback_allow
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_callback_allow
 *   |html </b>
 *   |html <i>
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create a callback contact.
 *
 *   This command creates a small GRAM server which can handle
 *   GRAM state updates from job managers. The contact information
 *   for this server is returned and may be used with the
 *   globus_gram_client_job_request or globus_gram_client_callback_register
 *   command.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked when the ping request has completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_callback_allow returns 0 and
 *   callback_contact. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] result
 *   callback_contact
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_disallow
 *****/



/****f* globus_gram_client/globus_gram_client_callback_disallow
 * NAME
 *   globus_gram_client_callback_disallow
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_callback_disallow
 *   |html </b>
 *   |html <i>
 *   callback_contact
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to disable a callback handler.
 *
 *   This command is to disable the GRAM server created by
 *   calling globus_gram_client_callback_allow. This command blocks
 *   until all pending job state updates being handled by this server
 *   are dispatched.
 *
 *   This command can only be used to disable a callback created
 *   in the current process.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   callback_contact
 *   |html </b>
 *   - The callback contact string returned by calling
 *   globus_gram_client_callback_allow.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_callback_disallow returns 0.
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gram_client_callback_disallow $callback_contact ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow
 *****/


/****f* globus_gram_client/globus_gram_client_attr_init
 * NAME
 *   globus_gram_client_attr_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_attr_init
 *   |html </b>
 *
 * DESCRIPTION
 *   This command is used to initialize a GRAM client attribute.
 *
 * RETURN VALUE
 *   On success globus_gram_client_attr_init returns 0 and GRAM client
 *   attribute.
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gram_client_attr_init ] result attr
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gram_client/globus_gram_client_attr_destroy
 * NAME
 *   globus_gram_client_attr_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_attr_destroy
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a GRAM client attribute.
 *
 * RETURN VALUE
 *   On success globus_gram_client_attr_destroy returns 0.
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gram_client_attr_destroy $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gram_client/globus_gram_client_attr_set_credential
 * NAME
 *   globus_gram_client_attr_set_credential
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_attr_set_credential
 *   |html </b>
 *   |html <i>
 *   attr
 *   credential
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to associate a credential with an attribute set.
 *
 * RETURN VALUE
 *   On success globus_gram_client_attr_set_credential returns 0.
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gram_client_attr_set_credential $attr
 *   $gss_c_no_credential ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gram_client/globus_gram_client_attr_get_credential
 * NAME
 *   globus_gram_client_attr_get_credential
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_attr_get_credential
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the credential associated with an
 *   attribute set.
 *
 * RETURN VALUE
 *   On success globus_gram_client_attr_get_credential returns 0 and
 *   credential.
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gram_client_attr_get_credential $attr ] result credential
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gram_client/globus_gram_client_set_credentials
 * NAME
 *   globus_gram_client_set_credentials
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_set_credentials
 *   |html </b>
 *   |html <i>
 *   new_credential
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set new credentials.
 *
 * RETURN VALUE
 *   On success globus_gram_client_set_credentials returns 0.
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_gram_client_set_credentials $gss_c_no_credential ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_gram_client/globus_gram_client_ping
 * NAME
 *   globus_gram_client_ping
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_ping
 *   |html </b>
 *   |html <i>
 *   resource_manager_contact
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to verify that a gatekeeper is running.
 *
 *   This command sends a specially-formated GRAM protocol message
 *   which checks to see if a Globus Gatekeeper is running on a given
 *   PORT, and whether that Gatekeeper is configured to support the
 *   desired job manager service. This is primarily used for diagnostic
 *   purposes.
 *
 *   This command blocks while processing the ping request.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   resource_manager_contact
 *   |html </b>
 *   - A NULL-terminated character string containing a GRAM contact.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_ping returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   lassign [ globus_gram_client_ping $resource_manager_contact ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_register_ping
 *****/



/****f* globus_gram_client/globus_gram_client_register_ping
 * NAME
 *   globus_gram_client_register_ping
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_register_ping
 *   |html </b>
 *   |html <i>
 *   resource_manager_contact
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to verify that a gatekeeper is running
 *   (nonblocking).
 *
 *   This command sends a specially-formated GRAM protocol message
 *   which checks to see if a Globus Gatekeeper is running on a given
 *   PORT, and whether that Gatekeeper is configured to support the
 *   desired job manager service. This is primarily used for diagnostic
 *   purposes.
 *
 *   If this command determines that the ping could not be processed
 *   before contacting the gatekeeper (for example, a malformed
 *   resource_manager_contact), it will return an error, and the script
 *   will not be called.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   resource_manager_contact
 *   |html </b>
 *   - A NULL-terminated character string containing a GRAM contact.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Client attributes to be used. Should be set to
 *   GLOBUS_GRAM_CLIENT_NO_ATTR if no attributes are to be used.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked when the ping request has completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_register_ping returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   lassign [ globus_gram_client_register_ping $resource_manager_contact NULL { register_ping_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_ping
 *****/


/****f* globus_gram_client/globus_gram_client_job_request
 * NAME
 *   globus_gram_client_job_request
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_job_request
 *   |html </b>
 *   |html <i>
 *   resource_manager_contact
 *   description
 *   job_state_mask
 *   callback_contact
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to request a job to be started (blocking).
 *
 *   Request access to interactive resources at the current time.
 *   A job request is atomic: either all of the requested processes
 *   are created, or none are created.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   resource_manager_contact
 *   |html </b>
 *   - A NULL-terminated character string containing a GRAM contact.
 *
 *   |html <b>
 *   description
 *   |html </b>
 *   - An RSL description of the requested job. A GRAM RSL consists
 *   of a conjunction of RSL parameters.
 *
 *   |html <b>
 *   job_state_mask
 *   |html </b>
 *   - 0, a bitwise OR of the GLOBUS_GRAM_PROTOCOL_JOB_STATE states, or
 *   GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL.
 *
 *   |html <b>
 *   callback_contact
 *   |html </b>
 *   - The URL which will receive all messages about the job.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_job_request returns 0 and job_contact. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/date)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL $callback_contact ] result job_contact
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free.
 *****/



/****f* globus_gram_client/globus_gram_client_register_job_request
 * NAME
 *   globus_gram_client_register_job_request
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_register_job_request
 *   |html </b>
 *   |html <i>
 *   resource_manager_contact
 *   description
 *   job_state_mask
 *   callback_contact
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to request a job to be started (nonblocking).
 *
 *   Request access to interactive resources at the current time.
 *   A job request is atomic: either all of the requested processes are
 *   created, or none are created. This is the nonblocking version of
 *   globus_gram_client_job_request. Instead of waiting for the job
 *   manager to acknowledge that the job has been submitted or started,
 *   this command immediately returns after beginning the job submission.
 *   The script will be called to let the caller know whether the job
 *   request has been submitted successfully or not.
 *
 *   If this command determines that the job request could not be processed
 *   before contacting the job manager (for example, a malformed
 *   resource_manager_contact) it will return an error, and the script
 *   will not be called.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   resource_manager_contact
 *   |html </b>
 *   - A NULL-terminated character string containing a GRAM contact.
 *
 *   |html <b>
 *   description
 *   |html </b>
 *   - An RSL description of the requested job. A GRAM RSL consists
 *   of a conjunction of RSL parameters.
 *
 *   |html <b>
 *   job_state_mask
 *   |html </b>
 *   - 0, a bitwise OR of the GLOBUS_GRAM_PROTOCOL_JOB_STATE states, or
 *   GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL.
 *
 *   |html <b>
 *   callback_contact
 *   |html </b>
 *   - The URL which will receive all messages about the job.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Client attributes to be used. Should be set to
 *   GLOBUS_GRAM_CLIENT_NO_ATTR if no attributes are to be used.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked the job request submission has completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_register_job_request returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/date)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_register_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL $callback_contact NULL
 *   { job_request_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free.
 *****/



/****f* globus_gram_client/globus_gram_client_job_cancel
 * NAME
 *   globus_gram_client_job_cancel
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_job_cancel
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to cancel a GRAM-managed job.
 *
 *   Removes a PENDING job request, or kills all processes associated
 *   with an ACTIVE job, releasing any associated resources
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same
 *   value returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_job_cancel returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL $callback_contact ] result job_contact
 *
 *   lassign [ globus_gram_client_job_cancel $job_contact ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_register_job_cancel. 
 *****/



/****f* globus_gram_client/globus_gram_client_register_job_cancel
 * NAME
 *   globus_gram_client_register_job_cancel
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_register_job_cancel
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to cancel of a GRAM-managed job (nonblocking).
 *
 *   Removes a PENDING job request, or kills all processes associated
 *   with an ACTIVE job, releasing any associated resources.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same value
 *   returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Client attributes to be used. Should be set to
 *   GLOBUS_GRAM_CLIENT_NO_ATTR if no attributes are to be used.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked the job request cancel has completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_register_job_cancel returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL $callback_contact ] result job_contact
 *
 *   lassign [ globus_gram_client_register_job_cancel $job_contact NULL
 *   { job_cancel_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_job_cancel.
 *****/


/****f* globus_gram_client/globus_gram_client_job_signal
 * NAME
 *   globus_gram_client_job_signal
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_job_signal
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   signal
 *   signal_arg
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to send a signal to a GRAM job manager to
 *   modify the way it handles a job request. Signals consist of a
 *   signal number, and an optional string argument. The meanings of
 *   the signals supported by the GRAM job manager are defined in
 *   the GRAM Protocol documentation
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same
 *   value returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *
 *   |html <b>
 *   signal
 *   |html </b>
 *   - The signal code to send to the job manager.
 *
 *   |html <b>
 *   signal_arg
 *   |html </b>
 *   - Parameters for the signal, as described in the documentation
 *   for the globus_gram_protocol_job_signal_t.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_job_signal returns 0, job_status and
 *   failure_code. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL $callback_contact ] result job_contact
 *
 *   lassign [ globus_gram_client_job_signal $job_contact $GLOBUS_GRAM_PROTOCOL_JOB_SIGNAL_STOP_MANAGER NULL ] result job_status job_failure_code
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_register_job_signal. 
 *****/



/****f* globus_gram_client/globus_gram_client_register_job_signal
 * NAME
 *   globus_gram_client_register_job_signal
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_register_job_signal
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   signal
 *   signal_arg
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to send a nonblocking signal to a GRAM job
 *   manager to modify the way it handles a job request. Signals
 *   consist of a signal number, and an optional string argument.
 *   The meanings of the signals supported by the GRAM job manager
 *   are defined in the GRAM Protocol documentation.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same value
 *   returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *
 *   |html <b>
 *   signal
 *   |html </b>
 *   - The signal code to send to the job manager.
 *
 *   |html <b>
 *   signal_arg
 *   |html </b>
 *   - Parameters for the signal, as described in the documentation
 *   for the globus_gram_protocol_job_signal_t.

 *   |html <b>
 *   attr
 *   |html </b>
 *   - Client attributes to be used. Should be set to
 *   GLOBUS_GRAM_CLIENT_NO_ATTR if no attributes are to be used.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked the job signal has been completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_register_job_signal returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL $callback_contact ] result job_contact
 *
 *   lassign [ globus_gram_client_register_job_signal $job_contact $GLOBUS_GRAM_PROTOCOL_JOB_SIGNAL_STOP_MANAGER NULL NULL { job_signal_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_register_job_signal.
 *****/



/****f* globus_gram_client/globus_gram_client_job_status
 * NAME
 *   globus_gram_client_job_status
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_job_status
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the status of the job associated
 *   with the job contact, returning it's current job status and
 *   job failure reason if it has failed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same
 *   value returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_job_status returns 0, job_status and
 *   failure_code. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL $callback_contact ] result job_contact
 *
 *   lassign [ globus_gram_client_job_status $job_contact ] result job_status job_failure_code
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_register_job_status. 
 *****/



/****f* globus_gram_client/globus_gram_client_register_job_status
 * NAME
 *   globus_gram_client_register_job_status
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_register_job_status
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the status of the job associated
 *   with the job contact, returning it's current job status and job
 *   failure reason if it has failed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same value
 *   returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Client attributes to be used. Should be set to
 *   GLOBUS_GRAM_CLIENT_NO_ATTR if no attributes are to be used.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked the job request cancel has completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_register_job_status returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL $callback_contact ] result job_contact
 *
 *   lassign [ globus_gram_client_register_job_status $job_contact NULL
 *   { job_status_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_job_status.
 *****/



/****f* globus_gram_client/globus_gram_client_job_callback_register
 * NAME
 *   globus_gram_client_job_callback_register
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_job_callback_register
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   job_state_mask
 *   callback_contact
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to register a callback contact for job state
 *   changes.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same
 *   value returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *
 *   |html <b>
 *   job_state_mask
 *   |html </b>
 *   - A mask indicating which job state changes should be sent to the
 *   callback_contact. This may be 0 (no job state changes),
 *   a bitwise-or of the GLOBUS_GRAM_PROTOCOL_JOB_STATE_* states, or
 *   GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL to register for all job states.
 *
 *   |html <b>
 *   callback_contact
 *   |html </b>
 *   - A URL string containing a GRAM client callback. This string should
 *   normally be generated by a process calling
 *   globus_gram_client_callback_allow.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_job_callback_register returns 0,
 *   job_status and failure_code.  
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL NULL ] result job_contact
 *
 *   set job_state_mask [ expr $GLOBUS_GRAM_PROTOCOL_JOB_STATE_FAILED | 
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_DONE ]
 *
 *   lassign [ globus_gram_client_job_callback_register $job_contact $job_state_mask $callback_contact ] result job_status job_failure_code
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_job_callback_unregister. 
 *****/



/****f* globus_gram_client/globus_gram_client_job_callback_unregister
 * NAME
 *   globus_gram_client_job_callback_unregister
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_job_callback_unregister
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   callback_contact
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to unregister a callback contact from future
 *   job state changes.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same
 *   value returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *
 *   |html <b>
 *   callback_contact
 *   |html </b>
 *   - A URL string containing a GRAM client callback. This string should
 *   normally be generated by a process calling
 *   globus_gram_client_callback_allow.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_job_callback_register returns 0,
 *   job_status and failure_code.  
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL NULL ] result job_contact
 *
 *   lassign [ globus_gram_client_job_callback_unregister $job_contact $callback_contact ] result job_status job_failure_code
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_job_callback_register. 
 *****/



/****f* globus_gram_client/globus_gram_client_job_refresh_credentials
 * NAME
 *   globus_gram_client_job_refresh_credentials
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_job_refresh_credentials
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   credential
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to delegate new credentials to a job manager.
 *
 *   This command performs a new delegation handshake with the job manager,
 *   updating it with a new user proxy. This will allow the job manager
 *   to continue to send job state callbacks after the original proxy
 *   would have expired.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same
 *   value returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *
 *   |html <b>
 *   credential
 *   |html </b>
 *   - A credential which should be used to contact the job manager.
 *   This may be GSS_C_NO_CREDENTIAL to use the process's default credential.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_job_refresh_credentials returns 0.  
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL NULL ] result job_contact
 *
 *   lassign [ globus_gram_client_job_refresh_credentials $job_contact $gss_c_no_credential ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_register_job_refresh_credentials. 
 *****/



/****f* globus_gram_client/globus_gram_client_register_job_refresh_credentials
 * NAME
 *   globus_gram_client_register_job_refresh_credentials
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_register_job_refresh_credentials
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   credential
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to delegate new credentials to a job manager
 *   (nonblocking).
 *
 *   This command performs the same operation as
 *   globus_gram_client_job_refresh_credentials, but without blocking
 *   the calling thread. Once the delegation has completed, it's final
 *   status will be reported in the script.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same
 *   value returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *
 *   |html <b>
 *   credential
 *   |html </b>
 *   - A credential which should be used to contact the job manager.
 *   This may be GSS_C_NO_CREDENTIAL to use the process's default credential.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Client attributes to be used. Should be set to
 *   GLOBUS_GRAM_CLIENT_NO_ATTR if no attributes are to be used.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked when the job refresh has been completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_register_job_refresh_credentials
 *   returns 0.  
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL NULL ] result job_contact
 *
 *   lassign [ globus_gram_client_register_job_refresh_credentials $job_contact $gss_c_no_credential NULL { job_registration_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_job_refresh_credentials. 
 *****/



/****f* globus_gram_client/globus_gram_client_register_job_callback_registration
 * NAME
 *   globus_gram_client_register_job_callback_registration
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_register_job_callback_registration
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   job_state_mask
 *   callback_contact
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to register a callback contact for job state
 *   changes (nonblocking).
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same
 *   value returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *
 *   |html <b>
 *   job_state_mask
 *   |html </b>
 *   - A mask indicating which job state changes should be sent to the
 *   callback_contact. This may be 0 (no job state changes),
 *   a bitwise-or of the GLOBUS_GRAM_PROTOCOL_JOB_STATE_* states, or
 *   GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL to register for all job states.
 *
 *   |html <b>
 *   callback_contact
 *   |html </b>
 *   - A URL string containing a GRAM client callback. This string should
 *   normally be generated by a process calling
 *   globus_gram_client_callback_allow.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Client attributes to be used. Should be set to
 *   GLOBUS_GRAM_CLIENT_NO_ATTR if no attributes are to be used.
 *
  *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked when the job registration has been completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_register_job_callback_registration
 *   returns 0.  
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL NULL ] result job_contact
 *
 *   set job_state_mask [ expr $GLOBUS_GRAM_PROTOCOL_JOB_STATE_FAILED | 
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_DONE ]
 *
 *   lassign [ globus_gram_client_register_job_callback_registration $job_contact $job_state_mask $callback_contact NULL { job_registration_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_register_job_callback_unregistration. 
 *****/



/****f* globus_gram_client/globus_gram_client_register_job_callback_unregistration
 * NAME
 *   globus_gram_client_register_job_callback_unregistration
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_gram_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_gram_client_register_job_callback_unregistration
 *   |html </b>
 *   |html <i>
 *   job_contact
 *   callback_contact
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to unregister a callback contact (nonblocking).
 *
 * PARAMETERS
 *   |html <ul><b>
 *   job_contact
 *   |html </b>
 *   - The job contact string of the job to contact. This is the same
 *   value returned from globus_gram_client_job_request or
 *   globus_gram_client_register_job_request.
 *
 *   |html <b>
 *   callback_contact
 *   |html </b>
 *   - A URL string containing a GRAM client callback. This string should
 *   normally be generated by a process calling
 *   globus_gram_client_callback_allow.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Client attributes to be used. Should be set to
 *   GLOBUS_GRAM_CLIENT_NO_ATTR if no attributes are to be used.
 *
  *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script to be invoked when the job registration has been completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_gram_client_register_job_callback_unregistration
 *   returns 0.  
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   set resource_manager_contact "bosco.ligo.caltech.edu:51939:/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
 *
 *   set job_description "&(executable=/bin/sleep)(arguments=300)"
 *
 *   lassign [ globus_gram_client_callback_allow { done_cb NULL } ] status
 *   callback_contact
 *
 *   lassign [ globus_gram_client_job_request
 *   $resource_manager_contact $job_description
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL NULL ] result job_contact
 *
 *   set job_state_mask [ expr $GLOBUS_GRAM_PROTOCOL_JOB_STATE_FAILED | 
 *   $GLOBUS_GRAM_PROTOCOL_JOB_STATE_DONE ]
 *
 *   lassign [ globus_gram_client_register_job_callback_unregistration $job_contact $callback_contact NULL { job_registration_cb NULL } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gram_client_callback_allow, globus_gram_client_callback_disallow, globus_gram_client_job_request, globus_gram_client_register_job_request, globus_gram_client_job_contact_free, globus_gram_client_register_job_callback_registration. 
 *****/
