/* -*- mode: c++; c-basic-offset: 2; -*- */

#ifdef SWIG

%include "../../config.h"

%include "tcl-typemaps.i"

%ignore globus_io_get_handle_type;

%ignore globus_io_read;
%ignore globus_io_try_read;
%ignore globus_io_register_read;

%ignore globus_io_write;
%ignore globus_io_try_write;
%ignore globus_io_register_write;

%ignore globus_io_writev;
%ignore globus_io_register_writev;

%ignore globus_io_send;
%ignore globus_io_try_send;
%ignore globus_io_register_send;

%ignore globus_io_tcp_get_local_address;
%ignore globus_io_tcp_get_remote_address;

%ignore globus_io_tcp_register_connect;

/*
 * The netlogger calls are all unsupported
 * (and unimplemented in GT 3.2 and greater). 
 */

%ignore globus_io_attr_netlogger_set_handle;
%ignore globus_io_attr_netlogger_copy_handle;
%ignore globus_netlogger_write;
%ignore globus_netlogger_handle_init;
%ignore globus_netlogger_handle_destroy;
%ignore globus_netlogger_get_nlhandle;
%ignore globus_netlogger_set_desc;

%ignore globus_io_tcp_set_credential;
%ignore globus_io_tcp_get_credential;
%ignore globus_io_register_init_delegation;
%ignore globus_io_init_delegation;
%ignore globus_io_register_accept_delegation; 
%ignore globus_io_accept_delegation;

%ignore globus_io_register_select;
%ignore globus_io_tcp_posix_convert;
%ignore globus_io_tcp_posix_convert_listener;
%ignore globus_io_file_posix_convert;

%include "swig/globus_io.i"

typedef struct globus_l_io_handle_s *	globus_io_handle_t;

%{

#include "../../config.h"
#include <tcl.h>
#include "tcl-callback.h"
#include "globus_xio.h"
#include "globus_xio_tcp_driver.h"
#include "globus_xio_file_driver.h"
#include "globus_xio_gsi.h"  
#include "globus_io.h"
#include "common/sgdebug.h"
#include "common/sginit.h"
  
#define MAX_UNIQUE_NAME 64

  /* Callback-related typedef */
  typedef struct callbackData_ {
    long           total_bytes;
    int            iovec_cnt;
    void*          buffer;
    char           unique_name[MAX_UNIQUE_NAME];
  } callbackData;


  /**************************************************************************
   *
   * GLOBUS_IO_REGISTER_CLOSE()
   * GLOBUS_IO_REGISTER_CANCEL()
   * GLOBUS_IO_TCP_REGISTER_LISTEN()
   * GLOBUS_IO_TCP_REGISTER_ACCEPT()
   *
   **************************************************************************/
  static void
    TclIoCallback( void *               callback_arg,
                   globus_io_handle_t * handle,
                   globus_result_t      result )
    {
      char 	str_handle[128];

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_io_handle_t" ), 0 );

      CallCallback( (callbackStruct*)callback_arg,
                    "%s %d", 
                    str_handle, result );
    }


  /**************************************************************************
   *
   * GLOBUS_IO_REGISTER_INIT_DELEGATION()
   * GLOBUS_IO_REGISTER_ACCEPT_DELEGATION()
   *
   **************************************************************************/
  static void
    TclIoDelegationCallback( globus_io_handle_t * handle,
                             globus_result_t      result,
                             gss_cred_id_t        delegated_cred,
                             OM_uint32            time_rec,
                             void *               user_arg )
    {
      char 	str_handle[128];

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_io_handle_t" ), 0 );

      CallCallback( (callbackStruct*)user_arg,
                    "%s %d %p %d",
                    str_handle, result, delegated_cred, time_rec );
    }


  /**************************************************************************
   *
   * GLOBUS_IO_SECURE_AUTHORIZATION_DATA_SET_CALLBACK()
   *
   **************************************************************************/
  static void
    TclIoSecureAuthorizationDataSetCallback
    ( globus_io_handle_t 		      * handle,
      globus_result_t 				result,
      char				      * identity,	
      gss_ctx_id_t			      * context_handle,
      void *                                  	user_arg )
    {
      char 	str_handle[128];

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_io_handle_t" ), 0 );

      CallCallback( (callbackStruct*)user_arg,
                    "%s %d %s %d",
                    str_handle, result, identity, context_handle );
    }


  /**************************************************************************
   *
   * GLOBUS_IO_TCP_REGISTER_CONNECT()
   *
   **************************************************************************/

  static void
    GlobusIoTcpRegisterConnectInit( callbackStruct* cb )
    {
      globus_io_handle_t * handle =
        (globus_io_handle_t *)globus_libc_malloc( sizeof(globus_io_handle_t) );
      
      cb->s_api_data = handle;
    }


  static int
    GlobusIoTcpRegisterConnectCleanup( callbackStruct* cb )
    {
      globus_libc_free( cb->s_api_data );

      cb->s_api_data = NULL;
      return 1;
    }


  static void
    TclIoTcpRegisterConnectCallback( void *               callback_arg,
                                     globus_io_handle_t * handle,
                                     globus_result_t      result )
    {      
      char handle_ptr_buf[256];
      char 	str_handle[128];

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_io_handle_t" ), 0 );
      
      /* Special trick for converting handle */
      SWIG_MakePtr
        ( handle_ptr_buf,
          (globus_io_handle_t*)((callbackStruct*)callback_arg)->s_api_data, 
          SWIG_TypeQuery( "globus_io_handle_t *" ), 0 );

      CallCallback( (callbackStruct*)callback_arg,
                    "%s %d %s",
                    str_handle,
                    result,
                    handle_ptr_buf );
    }
  
  
  /**************************************************************************
   *
   * GLOBUS_IO_REGISTER_READ()
   *
   **************************************************************************/
  static void
    GlobusIoRegisterReadInit( callbackStruct* cb, long buffer_length )
    {
      /* Allocate memory space for our buffer */
      callbackData* data = 
        (callbackData *)globus_libc_malloc( sizeof(callbackData) );
      
      SGDEBUG_LVL_FMT_1( 20, "read callback allocation: %p", data );

      cb->s_api_data = data;

      data->total_bytes = buffer_length; 
      
      /* :TODO: */
      /* We should not exit if we fail to allocate our buffer */
      /* Must return error message instead of just exit */
      if ( ( data->buffer = 
             (globus_byte_t *)globus_libc_malloc( sizeof(globus_byte_t) * 
                                                  buffer_length )) == NULL ) 
        {
          fprintf(stderr, "Memory allocation fails\n");
          exit(1);
        }

      SGDEBUG_LVL_FMT_1( 20, "read buffer allocation: %p", data->buffer );
      
      memset( data->buffer, '\0', buffer_length );
            
      TclGlobusCreateUniqueTCLVariableName( data->unique_name );

      SGDEBUG_LVL_FMT( 10, "read init: TclGlobusCreateUniqueTCLVariableName" );
    }
  
  static int
    GlobusIoRegisterReadCleanup( callbackStruct* cb )
    {
      callbackData* data = (callbackData*)cb->s_api_data;
      
      /* Release TCL resources */
      Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);

      SGDEBUG_LVL_FMT_1( 20, "GlobusIoRegsiterReadCleanup: UnsetVar2: %s", 
			data->unique_name );

      /* Deallocate memory space */
      if ( ((callbackData*)(cb->s_api_data))->buffer )
      {
	SGDEBUG_LVL_FMT_1( 20, "read buffer deallocation: %p",
			   ((callbackData*)(cb->s_api_data))->buffer);
      
	globus_libc_free( ((callbackData*)(cb->s_api_data))->buffer );
	
      }

      if ( cb->s_api_data ) 
      {
        SGDEBUG_LVL_FMT_1
	  ( 20, "read callback deallocation: %p", cb->s_api_data );
        globus_libc_free(cb->s_api_data);
        cb->s_api_data = NULL;
      }
      return 1;
    }


  static void
    GlobusIoRegisterReadPreCallback( callbackStruct* cb )
    {
      Tcl_Obj* obj;
      
      callbackData* data = (callbackData*)(cb->s_api_data);
      
      obj = Tcl_NewByteArrayObj((char *)(data->buffer), data->total_bytes);
          
      Tcl_SetVar2Ex(cb->s_main_tcl_interp, data->unique_name, NULL, obj, 0);
    }
  
  /**************************************************************************
   *
   * GLOBUS_IO_REGISTER_READ()
   *
   **************************************************************************/
  static void
    TclIoRRDataCallback( void *                      arg,
                         globus_io_handle_t *        handle,
                         globus_result_t             result,
                         globus_byte_t *             buf,
                         globus_size_t               nbytes )
    {
      char 	str_handle[128];

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_io_handle_t" ), 0 );
      
      callbackData * data =
	(callbackData*)((callbackStruct*)arg)->s_api_data;
      
      SGDEBUG_LVL_FMT_1( 10, "TclIoRRDataCallback: nbytes: %d", nbytes );

      data->total_bytes = nbytes;

      CallCallback( (callbackStruct*)arg,
                    "%s %d $%s",
                    str_handle, result, data->unique_name );

    }


  /**************************************************************************
   *
   * GLOBUS_IO_REGISTER_WRITE()
   *
   **************************************************************************/
  static void
  GlobusIoRegisterWriteInit( callbackStruct* cb, unsigned char* buffer )
  {
    /* Allocate memory space */
    callbackData * data =
      (callbackData *)globus_libc_malloc( sizeof(callbackData) );

    SGDEBUG_LVL_FMT_1( 20, "write callback allocation: %p", data );
    
    cb->s_api_data = data;
    data->buffer = buffer;
    
    TclGlobusCreateUniqueTCLVariableName( data->unique_name );
    SGDEBUG_LVL_FMT( 10, "write Tcl_SetVar2" );
  }


  static int
  GlobusIoRegisterWriteCleanup( callbackStruct* cb )
  {
    callbackData* data = (callbackData*)(cb->s_api_data);

    /* Release TCL resources */
    SGDEBUG_LVL_FMT( 10, "write Tcl_UnsetVar2" );
    Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);

    /* Deallocate memory space */
    if ( data->buffer ) 
      {
        SGDEBUG_LVL_FMT_1
	  ( 20, "write buffer deallocation: %p", data->buffer ); 
        globus_libc_free( data->buffer );
      }

    /* Deallocate memory space */
    if ( cb->s_api_data ) 
    {
      SGDEBUG_LVL_FMT_1
	( 20, "write callback deallocation: %p", cb->s_api_data ); 
      globus_libc_free(cb->s_api_data);
      cb->s_api_data = NULL;
    }
    return 1;
  }


  static void
  GlobusIoRegisterWritePreCallback( callbackStruct* cb )
  {
    Tcl_Obj* obj;
    callbackData* data = (callbackData*)(cb->s_api_data);

      SGDEBUG_LVL_FMT_1
	( 10, "GlobusIoRegisterWritePreCallback: cb->s_api_data: %p",
	  cb->s_api_data );      

      SGDEBUG_LVL_FMT_1
	( 10, "GlobusIoRegisterWritePreCallback: data: %p", data );      

      SGDEBUG_LVL_FMT_1
	( 10, "GlobusIoRegisterWritePreCallback: data->buffer: %p",
	  data->buffer );      
      
      SGDEBUG_LVL_FMT_1
	( 10, "GlobusIoRegisterWritePreCallback: data->total_bytes: %d",
	  data->total_bytes );      
    
    obj = Tcl_NewByteArrayObj( (char *)(data->buffer), data->total_bytes );
  
    Tcl_SetVar2Ex(cb->s_main_tcl_interp, data->unique_name, NULL, obj, 0);
  }



  /**************************************************************************
   *
   * GLOBUS_IO_REGISTER_WRITE()
   *
   **************************************************************************/

  static void
    TclIoRWDataCallback( void *                       arg,
                         globus_io_handle_t *         handle,
                         globus_result_t              result,
                         globus_byte_t *              buf,
                         globus_size_t                nbytes )
    {
      char 	str_handle[128];

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_io_handle_t" ), 0 );

      callbackData*
        data = (callbackData*)((callbackStruct*)arg)->s_api_data;

      SGDEBUG_LVL_FMT_1( 10, "TclIoRWDataCallback: nbytes: %d", nbytes );

      data->total_bytes = nbytes;
      data->buffer = buf;
      
      CallCallback( (callbackStruct*)arg,
                    "%s %d $%s",
                    str_handle, result, data->unique_name ); 
    }


  /**************************************************************************
   *
   * GLOBUS_IO_REGISTER_WRITEV()
   *
   **************************************************************************/

  static void
  GlobusIoRegisterWriteVectorInit( callbackStruct* 	cb, 
                                   globus_xio_iovec_t* 	iovec,
                                   int 			iovec_cnt )
  {
    /* Allocate memory space */
    callbackData* data =
      (callbackData *)globus_libc_malloc(sizeof(callbackData));
    
    cb->s_api_data = data;
    
    data->buffer = iovec;
    data->iovec_cnt = iovec_cnt;

    TclGlobusCreateUniqueTCLVariableName( data->unique_name );
  }

  static int
    GlobusIoRegisterWriteVectorCleanup( callbackStruct* cb )
    {
      int i;
      callbackData* data = (callbackData*)(cb->s_api_data);

      /* Release TCL resources */
      Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);
      
      for(i = 0; i < data->iovec_cnt; i++)
        {
	  if ( (((globus_xio_iovec_t*)(data->buffer))[i]).iov_base ) 
	    {
	      globus_libc_free
		( (((globus_xio_iovec_t*)(data->buffer))[i]).iov_base );
	      SGDEBUG_LVL_FMT( 10, "globus_libc_free is completed" );
	    }
        }

      /* Deallocate memory space */
      if ( data->buffer )
        {
          globus_libc_free( data->buffer );
        }

      if ( cb->s_api_data ) 
        {
          globus_libc_free(cb->s_api_data);
          cb->s_api_data = NULL;
        }
      return 1;
    }
  
  static void
  GlobusIoRegisterWriteVectorPreCallback( callbackStruct* cb ) 
  {
    int indx = 0;
    Tcl_Interp* interp;
    Tcl_Obj* listptr;
    callbackData*	data = (callbackData*)(cb->s_api_data);
    globus_xio_iovec_t* iovec_buf;
    long totbytes = 0;
    long curbytes = 0;

    listptr = Tcl_NewListObj(0, NULL);
    
    iovec_buf = (globus_xio_iovec_t *)data->buffer;
    
    interp = cb->s_main_tcl_interp;
      
    totbytes = data->total_bytes;

    while ( indx < data->iovec_cnt ) 
    {
      Tcl_ListObjAppendElement
	( interp, listptr, 
	  Tcl_NewByteArrayObj((char *)(iovec_buf[indx]).iov_base, 
			      (iovec_buf[indx]).iov_len ) 
	  );
        
      indx++;
    }
      
    Tcl_SetVar2Ex(interp, data->unique_name, NULL, listptr, 0);
  }
  
  /**************************************************************************
   *
   * GLOBUS_IO_REGISTER_WRITEV()
   *
   **************************************************************************/
  static void
    TclIoRWVectorDataCallback( void *                  arg,
                               globus_xio_handle_t *   handle,
                               globus_result_t         result,
                               struct iovec *          iov,
                               globus_size_t           iovcnt,
                               globus_size_t           nbytes )
    {
      char 	str_handle[128];

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_io_handle_t" ), 0 );

      callbackData*
        data = (callbackData*)((callbackStruct*)arg)->s_api_data;
      
      data->iovec_cnt = iovcnt;
      data->buffer = iov;
      data->total_bytes = nbytes;

      CallCallback( (callbackStruct*)arg,
                    "%s %d $%s %d %d",
                    str_handle, result, data->unique_name, iovcnt, nbytes ); 
    }
  
  %}


/**************************************************************************
 *
 *  SWIG INITIALIZATION STUFF
 *
 **************************************************************************/
%init %{

  SGINIT_X509;

  %}



/**************************************************************************
 *
 *  GLOBUS_IO_GET_HANDLE_TYPE()
 *
 **************************************************************************/

%typemap(in) 
  globus_io_handle_t * inhandle
{
  static void *descr = 0;
  if ( !descr )
    {
      descr = SWIG_TypeQuery( "globus_io_handle_t *" );
      assert( descr );
    }
  if ( (SWIG_ConvertPtr(objv[1],
                        (void **) &arg1,
			descr,
                        SWIG_POINTER_EXCEPTION | 0) != TCL_OK) )
    {
      SWIG_fail;
    }
}

%rename(globus_io_get_handle_type) tcl_globus_io_get_handle_type;


/****f* globus_io/globus_io_get_handle_type
 * NAME
 *   globus_io_get_handle_type
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_get_handle_type
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to return the type of the handle. The type will
 *   be one of those enumerated in globus_io_handle_type_t. Globus I/O
 *   functions do not support all operations on all handle types. This
 *   may be called for any handle type.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_get_handle_type returns the type of the handle.
 *   On error, this command will raise an error and must be 
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_get_handle_type $handle ] handle_type
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_file_open, globus_io_tcp_connect, globus_io_tcp_accept,
 *   globus_io_tcp_create_listener, globus_io_tcp_listen
 *******
 */


%inline %{
  
  globus_io_handle_type_t
    tcl_globus_io_get_handle_type( globus_io_handle_t * inhandle )
    {
      globus_io_handle_type_t handle_type;
      
      handle_type = globus_io_get_handle_type( inhandle );
      
      return handle_type;
    }

  %}



/**************************************************************************
 *
 *  GLOBUS_IO_READ() & GLOBUS_IO_TRY_READ()
 *
 **************************************************************************/

%typemap(in,numinputs=0) 
  tcl_globus_buffer_t* outbuffer( tcl_globus_buffer_t outbuffer ) 
{
  $1 = &outbuffer;
}

%typemap( argout ) tcl_globus_buffer_t* outbuffer 
{
  Tcl_Obj* o = Tcl_NewByteArrayObj( $1->buffer, $1->length );

  /* Return the data buffer */
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%typemap( freearg ) tcl_globus_buffer_t* outbuffer 
{
  globus_libc_free( $1->buffer );
}

//---------------------------------------
// globus_io_read()
//---------------------------------------

/****f* globus_io/globus_io_read
 * NAME
 *   globus_io_read
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_read
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer_length
 *   waitforbytes
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to script the reading of a block of
 *   data from a handle in a blocking mode.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle.
 *
 *   |html <b>
 *   buffer_length
 *   |html </b>
 *   - Maximum number of bytes to read.
 *
 *   |html <b>
 *   waitforbytes
 *   |html </b>
 *   - Minimum number of bytes to write before invoking the script.
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_read returns 0 and a block of data.
 *   On error, this command will raise an error and must be 
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_read $handle $buffer_length 
 *   $waitforbytes ] status buffer
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_try_read, globus_io_register_read
 *******
 */

%rename(globus_io_read) tcl_globus_io_read;

%exception tcl_globus_io_read {

  globus_result_t error = result = tcl_globus_io_read(arg1,arg2,arg3, arg4);

  if ( error != 0 )
    {
      globus_object_t* error_obj = globus_error_get( error );

      if ( globus_io_eof(error_obj) )
        {
          // We don't need to catch end-of-file flag and
          // falls through to the next execution 
        }
      else
        {
          Tcl_Obj*	tcl_result = Tcl_GetObjResult( interp );
          
          char*	msg = globus_object_printable_to_string( error_obj );
          
          Tcl_SetStringObj( tcl_result, msg, strlen( msg ) );
          
          globus_libc_free( msg );

          return TCL_ERROR;
        }
    }

}

%inline %{

  typedef struct {
    globus_byte_t	*buffer;
    globus_size_t	length;
  } tcl_globus_buffer_t;
  
  globus_result_t
    tcl_globus_io_read( globus_io_handle_t *handle,
                        globus_size_t max_nbytes,
                        globus_size_t wait_for_nbytes,
                        tcl_globus_buffer_t *outbuffer )
    {
      
      globus_result_t retval;

      outbuffer->buffer = 
        (globus_byte_t*)globus_libc_malloc( sizeof( globus_byte_t ) *
                                            max_nbytes );
      
      memset(outbuffer->buffer, '\0', max_nbytes);
      
      retval = globus_io_read( handle, outbuffer->buffer,
                               max_nbytes, wait_for_nbytes,
                               &(outbuffer->length) );      
      return retval;
    }

  %}


//---------------------------------------
// globus_io_try_read()
//---------------------------------------

/****f* globus_io/globus_io_try_read
 * NAME
 *   globus_io_try_read
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_try_read
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer_length
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to read whatever data is immediately available
 *   up to buffer_length from a handle in a blocking mode.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle.
 *
 *   |html <b>
 *   buffer_length
 *   |html </b>
 *   - Maximum number of bytes to read.
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_read returns 0 and a block of data.
 *   On error, this command will raise an error and must be 
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_try_read $handle $buffer_length ] status buffer
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_read, globus_io_register_read
 *******
 */

%rename(globus_io_try_read) tcl_globus_io_try_read;

%exception tcl_globus_io_try_read GLOBUS_EXCEPTION_RESULT_T;

%inline %{
  
  globus_result_t
    tcl_globus_io_try_read( globus_io_handle_t *handle,
                            globus_size_t max_nbytes,
                            tcl_globus_buffer_t *outbuffer )
    {
      
      globus_result_t retval;
      
      outbuffer->buffer = 
        (globus_byte_t*)globus_libc_malloc( sizeof( globus_byte_t ) *
                                            max_nbytes );

      memset(outbuffer->buffer, '\0', max_nbytes);
      
      retval = globus_io_try_read( handle,
                                   outbuffer->buffer,
                                   max_nbytes,
                                   &(outbuffer->length) );
            
      return retval;
    }

  %}

/**************************************************************************
 *
 *  GLOBUS_IO_WRITE()
 *
 **************************************************************************/

%typemap(in) 
  tcl_globus_buffer_t *in_buffer( $*1_ltype temp ) 
{
  temp.buffer = 
    (globus_byte_t *)Tcl_GetByteArrayFromObj( $input, &(temp.length) );
  $1 = &temp;
}

//---------------------------------------
// globus_io_write()
//---------------------------------------

/****f* globus_io/globus_io_write
 * NAME
 *   globus_io_write
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_write
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to script the writing of a block of
 *   data in blocking mode.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - Buffer to write.
 *
 *   |html </ul>
 *
 * RESULT
 *   On success globus_io_write returns 0 and number of bytes written.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_write $handle $buffer ] status nbytes_written
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_try_write, globus_io_register_write
 *******
 */

%rename(globus_io_write) tcl_globus_io_write;

%exception tcl_globus_io_write GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_io_write( globus_io_handle_t *handle,
                         tcl_globus_buffer_t *in_buffer,
                         globus_size_t *nbytes_written )
    {
      globus_result_t retval;
      
      retval = globus_io_write( handle,
                                in_buffer->buffer,
                                in_buffer->length,
                                nbytes_written );
      
      return retval;
    }
  
  %}

//---------------------------------------
// globus_io_try_write()
//---------------------------------------

/****f* globus_io/globus_io_try_write
 * NAME
 *   globus_io_try_write
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_try_write
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to write as much as data it can from buffer,
 *   in a blocking mode.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - Buffer to write.
 *
 *   |html </ul>
 *
 * RESULT
 *   On success globus_io_try_write returns 0 and number of bytes written.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_try_write $handle $buffer ] status nbytes_written
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_write, globus_io_register_write
 *******
 */

%rename(globus_io_try_write) tcl_globus_io_try_write;

%exception tcl_globus_io_try_write GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_io_try_write( globus_io_handle_t *handle,
                             tcl_globus_buffer_t *in_buffer,
                             globus_size_t *nbytes_written )
    {
      globus_result_t retval;
      
      retval = globus_io_try_write( handle,
                                    in_buffer->buffer,
                                    in_buffer->length,
                                    nbytes_written );
      
      return retval;
    }
  
  %}



/**************************************************************************
 *
 *  GLOBUS_IO_SEND()
 *
 **************************************************************************/


//---------------------------------------
// globus_io_send()
//---------------------------------------

%rename(globus_io_send) tcl_globus_io_send;

%exception tcl_globus_io_send GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_io/globus_io_send
 * NAME
 *   globus_io_send
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_send
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   flags
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to perform a blocking send operation on the handle.
 *   This will block until the contents of buffer are written to the network,
 *   or an error occurs.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - Buffer to send.
 *
 *   |html <b>
 *   flags
 *   |html </b>
 *   - Flags to be passed to the send system call.
 *   |html </ul>
 *
 * RESULT
 *   On success globus_io_send returns 0 and number of bytes sent.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_send $handle $buffer $flags ] status nbytes_sent
 *
 * BUGS
 *   In the case of an error during a protected data transfer, the value
 *   pointed to by nbytes_sent is not necesarily reliable.
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_register_write, globus_io_try_write
 *******
 */

%inline %{

  globus_result_t
    tcl_globus_io_send( globus_io_handle_t *handle,
                        tcl_globus_buffer_t *in_buffer,
                        int flags,
                        globus_size_t *nbytes_sent )
    {
      globus_result_t retval;
      
      retval = globus_io_send( handle,
                               in_buffer->buffer,
                               in_buffer->length,
                               flags,
                               nbytes_sent );
      
      return retval;
    }
  
  %}


//---------------------------------------
// globus_io_try_send()
//---------------------------------------

%rename(globus_io_try_send) tcl_globus_io_try_send;

%exception tcl_globus_io_try_send GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_io/globus_io_try_send
 * NAME
 *   globus_io_try_send
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_try_send
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   flags
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to perform nonblocking TCP or file send whatever
 *   data can immediately processed by the operating system. 
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - Buffer to send.
 *
 *   |html <b>
 *   flags
 *   |html </b>
 *   - Flags to be passed to the send system call.
 *   |html </ul>
 *
 * RESULT
 *   On success globus_io_try_send returns 0 and number of bytes sent.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_try_send $handle $buffer $flags ] status
 *   nbytes_sent
 *
 * BUGS
 *   This function will always return 0 bytes written for TCP connections
 *   which are configured to use GSSAPI or SSL data wrapping.
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_write, globus_io_register_write
 *******
 */

%inline %{

  globus_result_t
    tcl_globus_io_try_send( globus_io_handle_t *handle,
                            tcl_globus_buffer_t *in_buffer,
                            int flags,
                            globus_size_t *nbytes_sent )
    {
      globus_result_t retval;
      
      retval = globus_io_try_send( handle,
                                   in_buffer->buffer,
                                   in_buffer->length,
                                   flags,
                                   nbytes_sent );
      
      return retval;
    }
  
  %}



/**************************************************************************
 *
 *  GLOBUS_IO_WRITEV()
 *
 **************************************************************************/

%typemap(in) tcl_globus_iovec_t *in_buflist( $*1_ltype temp ) 
{

  Tcl_Obj **listobjv;
  int nitems;
  int i = 0;

  /* Pass in Tcl list of data buffer */
  if ( Tcl_ListObjGetElements(interp, $input, &nitems, &listobjv) == 
       TCL_ERROR )
    return TCL_ERROR;

  temp.iovec = 
    (globus_xio_iovec_t *)globus_libc_malloc( sizeof(struct iovec) * nitems);
  
  temp.iovcnt = nitems;
  
  for (i = 0; i < nitems; i++) 
    {
      temp.iovec[i].iov_base = 
	Tcl_GetStringFromObj(listobjv[i], (int*)&(temp.iovec[i].iov_len));
    }

  $1 = &temp;
}

%typemap( freearg ) tcl_globus_iovec_t* in_buflist 
{
  globus_libc_free( $1->iovec );
}

/****f* globus_io/globus_io_writev
 * NAME
 *   globus_io_writev
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_writev
 *   |html </b>
 *   |html <i>
 *   handle
 *   list_of_buffers
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to script the writing of a block of
 *   data to several memory addresses at once.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle.
 *
 *   |html <b>
 *   list_of_buffers
 *   |html </b>
 *   - TCL list of data buffers.
 *
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_writev returns 0 and number of bytes written.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_writev $handle [list $buf1 $buf2] ] status nbytes
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_write, globus_io_try_write, globus_io_register_write
 *****/

%rename(globus_io_writev) tcl_globus_io_writev;

%exception tcl_globus_io_writev GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  typedef struct {
    char** bufsizearray;
    int iovcnt;
    globus_xio_iovec_t	*iovec;
    globus_size_t	nbytes;
  } tcl_globus_iovec_t;

  globus_result_t
    tcl_globus_io_writev( globus_io_handle_t *handle,
                          tcl_globus_iovec_t *in_buflist,
                          globus_size_t *nbytes )
    {
      globus_result_t retval;

      retval = globus_io_writev( handle,
                                 in_buflist->iovec,
                                 in_buflist->iovcnt,
                                 nbytes );
      
      return retval;
    }
  
  %}


/**************************************************************************
 *
 *  GLOBUS_IO_REGISTER_READ()
 *
 **************************************************************************/

%typemap(in) 
  (globus_io_read_callback_t rr_cb, void * rr_user_arg) 
  (callbackStruct *cbStruct) 
{

  cbStruct = CreateCallback( interp, $input );

  /* Allocate our buffer */
  GlobusIoRegisterReadInit( cbStruct, arg2 );

  $1 = (globus_io_read_callback_t)TclIoRRDataCallback; 
  $2 = cbStruct;

  cbStruct->s_pre_process = GlobusIoRegisterReadPreCallback;
  cbStruct->s_post_process = GlobusIoRegisterReadCleanup; 
}

%rename(globus_io_register_read) tcl_globus_io_register_read;

/****f* globus_io/globus_io_register_read
 * NAME
 *   globus_io_register_read
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_register_read
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer_length
 *   waitforbytes
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to script the reading of a block of
 *   data in the background.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle to read from.
 *
 *   |html <b>
 *   buffer_length
 *   |html </b>
 *   - The maximum number of bytes which will be read
 *
 *   |html <b>
 *   waitforbytes
 *   |html </b>
 *   - The minimum number of bytes to read before calling the callback
 *   function.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked to perform actual read. The command will be
 *   executed exactly once in global scope. If an error occurs while
 *   executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_register_read returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_register_read $handle $buffer_length $waitforbytes
 *   [ subst { callback callback_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_read, globus_io_try_read
 *
 *****/

%exception tcl_globus_io_register_read {

  globus_result_t error = result =
    tcl_globus_io_register_read(arg1,arg2,arg3, arg4, arg5);

  if ( error != 0 )
    {
      globus_object_t* error_obj = globus_error_get( error );

      if ( globus_io_eof(error_obj) )
        {
          // We don't need to catch end-of-file flag and
          // falls through to the next execution 
        }
      else
        {
          Tcl_Obj*	tcl_result = Tcl_GetObjResult( interp );
          
          char*	msg = globus_object_printable_to_string( error_obj );
          
          Tcl_SetStringObj( tcl_result, msg, strlen( msg ) );
          
          globus_libc_free( msg );

          return TCL_ERROR;
        }
    }


}

%inline %{

  globus_result_t
    tcl_globus_io_register_read( globus_io_handle_t *handle,
                                 long max_nbytes,
                                 long wait_for_nbytes,
                                 globus_io_read_callback_t rr_cb,
                                 void * rr_user_arg
                                 )
    {
      globus_result_t	retval;
      
      retval = 
        globus_io_register_read
        (
         handle,
         (globus_byte_t*)
         ((callbackData*)(((callbackStruct*)rr_user_arg)->s_api_data))->buffer,
         max_nbytes,
         wait_for_nbytes,
         rr_cb,
         rr_user_arg 
         );
      
      return retval;

    }
%}


/**************************************************************************
 *
 * GLOBUS_IO_REGISTER_WRITE()
 * GLOBUS_IO_REGISTER_SEND()
 *
 **************************************************************************/

%typemap(in) 
  tcl_globus_buffer_t *rw_in_buffer( $*1_ltype temp ) 
{
  unsigned char*   tbuffer;

  tbuffer = 
    (unsigned char *)Tcl_GetByteArrayFromObj( $input, &(temp.length) );
  temp.buffer = globus_libc_malloc( temp.length );
  SGDEBUG_LVL_FMT_1( 10, "write buffer allocation: = %p", temp.buffer );
  memcpy( temp.buffer, tbuffer, temp.length );
  
  $1 = &temp;
}

%typemap(in) 
  (globus_io_write_callback_t rw_cb, void * rw_user_arg) 
  (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );
  GlobusIoRegisterWriteInit(cbStruct, arg2->buffer);
  
  $1 = (globus_io_write_callback_t)TclIoRWDataCallback; 
  $2 = cbStruct;
  
  cbStruct->s_pre_process = GlobusIoRegisterWritePreCallback;
  cbStruct->s_post_process = GlobusIoRegisterWriteCleanup;
}


//---------------------------------------
// globus_io_register_write()
//---------------------------------------

%rename(globus_io_register_write) tcl_globus_io_register_write;

%exception tcl_globus_io_register_write GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_io/globus_io_register_write
 * NAME
 *   globus_io_register_write
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_register_write
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to script the writing of a block of
 *   data on the handle in the background.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle to write.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - The data to be written.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked to perform actual write. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_register_write returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_register_write $handle $buffer
 *   [ subst { callback $callback_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_write, globus_io_try_write
 *
 *****/

%inline %{

  globus_result_t
    tcl_globus_io_register_write( globus_io_handle_t *handle,
                                  tcl_globus_buffer_t *rw_in_buffer,
                                  globus_io_write_callback_t rw_cb,
                                  void * rw_user_arg )
    {
      globus_result_t	retval;
      
      retval = globus_io_register_write( handle,
                                         rw_in_buffer->buffer,
                                         rw_in_buffer->length,
                                         rw_cb,
                                         rw_user_arg );
      
      return retval;
    }
  %}

//---------------------------------------
// globus_io_register_send()
//---------------------------------------

%rename(globus_io_register_send) tcl_globus_io_register_send;

%exception tcl_globus_io_register_send GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_io/globus_io_register_send
 * NAME
 *   globus_io_register_send
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_register_send
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   flags
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to script the sending of a block of
 *   data on the handle in the background.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle to send.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - The data to be written.
 *
 *   |html <b>
 *   flags
 *   |html </b>
 *   - Flags to be passed to the send() call.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked to perform actual write. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_register_send returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_register_send $handle $buffer $flags
 *   [ subst { callback $callback_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_write, globus_io_try_write
 *
 *****/

%inline %{

  globus_result_t
    tcl_globus_io_register_send( globus_io_handle_t *handle,
                                 tcl_globus_buffer_t *rw_in_buffer,
                                 int flags,
                                 globus_io_write_callback_t rw_cb,
                                 void * rw_user_arg )
    {
      globus_result_t	retval;
      
      retval = globus_io_register_send( handle,
                                        rw_in_buffer->buffer,
                                        rw_in_buffer->length,
                                        flags,
                                        rw_cb,
                                        rw_user_arg );
      
      return retval;
    }
  %}


/**************************************************************************
 *
 * GLOBUS_IO_REGISTER_WRITEV()
 *
 **************************************************************************/

%typemap(in) 
  tcl_globus_iovec_t *rwv_list( $*1_ltype temp ) 
{
  Tcl_Obj **listobjv;
  int nitems;
  int i = 0;

  char*   tbuffer; 
  
  /* Pass in Tcl list of buffer containing actual data */
  if ( Tcl_ListObjGetElements(interp, $input, &nitems, &listobjv) == 
       TCL_ERROR )
    return TCL_ERROR;

  temp.iovec = 
    (globus_xio_iovec_t *)globus_libc_malloc( sizeof(struct iovec) * nitems);
  
  temp.iovcnt = nitems;

  /* nitems == iovec count */
  for (i = 0; i < nitems; i++) 
    {
      /*
      temp.iovec[i].iov_base = 
        Tcl_GetStringFromObj(listobjv[i], &(temp.iovec[i].iov_len)); 
      */

      tbuffer = (char *)
        Tcl_GetStringFromObj(listobjv[i], (int*)&(temp.iovec[i].iov_len));

      SGDEBUG_LVL_FMT_2
        ( 20, 
          "globus_io_register_writev: i: %d, iov_len: %d", 
          i, temp.iovec[i].iov_len );
      
      if ( temp.iovec[i].iov_len > 0 )
	{
	  temp.iovec[i].iov_base = 
	    (char*)globus_libc_malloc( temp.iovec[i].iov_len );
	  
	  memcpy(temp.iovec[i].iov_base, tbuffer, temp.iovec[i].iov_len);
	}
      else
	{
	  temp.iovec[i].iov_base = NULL;
	}
    }
  
  $1 = &temp;
}

%typemap(in) 
  (globus_io_writev_callback_t rwv_cb, void * rwv_user_arg) 
  (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  GlobusIoRegisterWriteVectorInit(cbStruct, 
                                  (globus_xio_iovec_t *)arg2->iovec, 
                                  arg2->iovcnt);
  
  $1 = (globus_io_writev_callback_t)TclIoRWVectorDataCallback; 
  $2 = cbStruct;

  cbStruct->s_pre_process = GlobusIoRegisterWriteVectorPreCallback;
  cbStruct->s_post_process = GlobusIoRegisterWriteVectorCleanup;
}

%rename(globus_io_register_writev) tcl_globus_io_register_writev;

%exception tcl_globus_io_register_writev GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_io/globus_io_register_writev
 * NAME
 *   globus_io_register_writev
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_register_writev
 *   |html </b>
 *   |html <i>
 *   handle
 *   list_of_buffer
 *   waitforbytes
 *   data_descriptor
 *   callback_script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to script the writing of a block of data to
 *   multiple non-contiguous data buffers at once in the background.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle to write to.
 *
 *   |html <b>
 *   list_of_buffer_size
 *   |html </b>
 *   - TCL list of buffer
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked to perform actual write. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * EXAMPLE
 *   lassign [ globus_io_register_writev $handle [list $buffer1 $buffer2] 
 *   [ subst { callback $callback_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_writev
 *
 *****/

%inline %{

  globus_result_t
    tcl_globus_io_register_writev( globus_io_handle_t *handle,
                                   tcl_globus_iovec_t *rwv_list,
                                   globus_io_writev_callback_t rwv_cb,
                                   void * rwv_user_arg)
    {
      globus_result_t retval;
      callbackData * data;

      SGDEBUG_LVL_FMT_1
        ( 10, 
          "tcl_globus_io_register_writev: rwv_list->iovec: %p", 
          rwv_list->iovec );
      
      SGDEBUG_LVL_FMT_1
        ( 10, 
          "tcl_globus_io_register_writev: rwv_list->iovcnt: %d", 
          rwv_list->iovcnt );

      /* 
       * Stick iovec and iovcnt information to callback data structure
       * so we can reclaim the space after the callback is completed.
       */
      data = (callbackData*)((callbackStruct *)rwv_user_arg)->s_api_data;
      
      data->buffer = (globus_xio_iovec_t*)rwv_list->iovec;
      data->iovec_cnt = rwv_list->iovcnt;

      retval = globus_io_register_writev( handle,
                                          rwv_list->iovec,
                                          rwv_list->iovcnt,
                                          rwv_cb,
                                          rwv_user_arg );

      return retval;
    }
  
  %}


/**************************************************************************
 *
 *  GLOBUS_IO_TCP_GET_LOCAL_ADDRESS()
 *  GLOBUS_IO_TCP_GET_REMOTE_ADDRESS()
 *
 **************************************************************************/

%typemap(in, numinputs=0) int * host ( int temp_host[4] ) 
{
  $1 = (int *)&temp_host;
}

%typemap(in, numinputs=0)
  unsigned short * portnumber ( $*1_ltype temp ) 
{
  $1 = &temp;
}

%typemap( argout ) int * host
{
  int i;
  char *ipstring;

  ipstring = (char*)globus_libc_malloc(32*sizeof(char));

  sprintf(ipstring, "%d.%d.%d.%d", $1[0], $1[1], $1[2], $1[3]);
  
  Tcl_Obj* o = Tcl_NewStringObj( (char *)ipstring, -1 );
  
  /* Return the data buffer */
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );

  globus_libc_free(ipstring);
}

%typemap( argout ) unsigned short * portnumber
{   
  Tcl_Obj* o = Tcl_NewIntObj((int)((long)(*$1)));

  /* Return the data buffer */
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

//---------------------------------------
// globus_io_tcp_get_local_address()
//---------------------------------------

%rename(globus_io_tcp_get_local_address) tcl_globus_io_tcp_get_local_address;

%exception tcl_globus_io_tcp_get_local_address GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_io/globus_io_tcp_get_local_address
 * NAME
 *   globus_io_tcp_get_local_address
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_get_local_address
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the IP address associated with a
 *   connected TCP socket.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O TCP connection handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   This function will return the IP address of the socket associated
 *   with this handle. The handle must be already connected, or an error
 *   will be returned.
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_get_local_address handle ] result ipstring port
 *
 * BUGS
 *   None
 *
 *****/

%inline %{

  globus_result_t
    tcl_globus_io_tcp_get_local_address( globus_io_handle_t * handle,
                                         int                * host,
                                         unsigned short     * portnumber )
    {
      globus_result_t retval;

      retval = globus_io_tcp_get_local_address( handle,
                                                host,
                                                portnumber );
      return retval;
    }

  %}

//---------------------------------------
// globus_io_tcp_get_remote_address()
//---------------------------------------

%rename(globus_io_tcp_get_remote_address)
  tcl_globus_io_tcp_get_remote_address;

%exception tcl_globus_io_tcp_get_remote_address GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_io/globus_io_tcp_get_remote_address
 * NAME
 *   globus_io_tcp_get_remote_address
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_get_remote_address
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the IP address associated with the peer of
 *   a connected TCP socket.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O TCP connection handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   This function will return the IP address of the peer of the socket
 *   associated with this handle. The handle must be already connected,
 *   or an error will be returned.
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_get_remote_address handle ] result ipstring port
 *
 * BUGS
 *   None
 *
 *****/

%inline %{

  globus_result_t
    tcl_globus_io_tcp_get_remote_address( globus_io_handle_t * handle,
                                          int                * host,
                                          unsigned short     * portnumber )
    {
      globus_result_t retval;

      retval = globus_io_tcp_get_remote_address( handle,
                                                 host,
                                                 portnumber );
      
      return retval;
    }

  %}


/**************************************************************************
 *
 * GLOBUS_IO_TCP_REGISTER_CONNECT()
 *
 **************************************************************************/

%typemap(in) 
  ( globus_io_callback_t callback,
    void * callback_arg ) (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  /* Allocate our buffer */
  GlobusIoTcpRegisterConnectInit( cbStruct );

  $1 = TclIoTcpRegisterConnectCallback; 
  $2 = cbStruct;

  /*
   * :TODO: We must deallocate the memory created for holding TCP
   * :TODO:   connection handle in globus_io_register_close().
   */
}


%rename(globus_io_tcp_register_connect) tcl_globus_io_tcp_register_connect;

%exception tcl_globus_io_tcp_register_connect GLOBUS_EXCEPTION_RESULT_T;


/****f* globus_io/globus_io_tcp_register_connect
 * NAME
 *   globus_io_tcp_register_connect
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_register_connect
 *   |html </b>
 *   |html <i>
 *   host
 *   port
 *   attr
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to connect a TCP socket on the specified
 *   host/port pair. The connection will be started and a callback
 *   will be invoked when the connection is established.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   host
 *   |html </b>
 *   - The host to connect to.
 *
 *   |html <b>
 *   port
 *   |html </b>
 *   - The TCP port number of the server.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - TCP socket attributes. The attr, if non-NULL, should be initialized
 *   by a call to globus_io_tcp_attr_init(). The attribute settings for the
 *   attr structure can be modified by any of the globus_io_tcp,
 *   globus_io_securesocket, or globus_io_socket attribute accessors. The
 *   security attributes must be compatible with those of the server handle.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the connection operation is completed. For
 *   secure connections, this function will called after the security
 *   handshake is completed. The command will be executed exactly once in
 *   global scope. If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_tcp_register_connect returns 0 and when the
 *   operation is completed successfully the TCP connection handle will
 *   be initialized returned as part of callback script argument.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_register_connect $host $port $attr
 *   [ subst { callback_script callback_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcp_connect
 *****/

%inline %{

  globus_result_t
    tcl_globus_io_tcp_register_connect
    ( char * 			host,
      unsigned short 		port,
      globus_io_attr_t * 	attr,
      globus_io_callback_t 	callback,
      void * 			callback_arg )
    {
      globus_result_t retval;
                                        
      retval = globus_io_tcp_register_connect
        ( host,
          port,
          attr,
          callback,
          callback_arg,
          (globus_io_handle_t*)((callbackStruct*)callback_arg)->s_api_data );

      return retval;
    }

  %}

/**************************************************************************
 *
 * ROBODOC
 *
 **************************************************************************/


////////////////////////////////////////////////////////////////////
//
// I/O Typedefs
//
////////////////////////////////////////////////////////////////////


/****t* globus_io/globus_io_file_flag_t
 * NAME
 *   globus_io_file_flag_t
 *
 * DESCRIPTION
 *   Flags to pass to the globus_io_file_open call.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_CREAT
 *   |html </b>
 *   - Open file with create.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_EXCL
 *   |html </b>
 *   - Exclusive open.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_RDONLY
 *   |html </b>
 *   - Read-only open.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_WRONLY
 *   |html </b>
 *   - Write-only open.
 * 
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_RDWR
 *   |html </b>
 *   - Read-write open.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_TRUNC
 *   |html </b>
 *   - Open and truncate.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_APPEND
 *   |html </b>
 *   - Open for append.
 *   |html </ul>
 *****/


/****t* globus_io/globus_io_file_create_mode_t
 * NAME
 *   globus_io_file_create_mode_t
 *
 * DESCRIPTION
 *   Open mode to pass to the globus_io_file_open() call.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IRWXU
 *   |html </b>
 *   - Read, write, execute: owner.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IRUSR
 *   |html </b>
 *   - Read permission: owner.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IWUSR
 *   |html </b>
 *   - Write permission: owner.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IXUSR
 *   |html </b>
 *   - Execute permission: owner.
 * 
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IRWXO
 *   |html </b>
 *   - Read, write, execute: other users.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IROTH
 *   |html </b>
 *   - Read permission: other users.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IWOTH
 *   |html </b>
 *   - Write permission: other users.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IXOTH
 *   |html </b>
 *   - Execute permission: other users.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IRWXG
 *   |html </b>
 *   - Read, write, execute: group members.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IRGRP
 *   |html </b>
 *   - Read permission: group members.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IWGRP
 *   |html </b>
 *   - Write permission: group members.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_IXGRP
 *   |html </b>
 *   - Execute permission: group members.
 *   |html </ul>
 *****/


/****t* globus_io/globus_io_whence_t
 * NAME
 *   globus_io_whence_t
 *
 * DESCRIPTION
 *   Type for the "whence" parameter of globus_io_file_seek.
 *
 *   This value is used to interpret the "offset" parameter of
 *   globus_io_file_seek. 
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SEEK_SET
 *   |html </b>
 *   - Set the offset of the file handle relative to the beginning
 *   of the file.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SEEK_CUR
 *   |html </b>
 *   - Set the offset of the file handle relative to the current offset.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SEEK_END
 *   |html </b>
 *   - Set the offset of the file handle relative to the end of the file.
 *   |html </ul>
 *****/


/****t* globus_io/globus_io_file_type_t
 * NAME
 *   globus_io_file_type_t
 *
 * DESCRIPTION
 *   Values for the Globus I/O file_type file attribute.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_TYPE_TEXT
 *   |html </b>
 *   - Open the file as a text file.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_FILE_TYPE_BINARY
 *   |html </b>
 *   - Open the file as a binary file.
 *   |html </ul>
 *****/


/****t* globus_io/globus_io_send_flags_t
 * NAME
 *   globus_io_send_flags_t
 *
 * DESCRIPTION
 *   Use this to send a TCP message out of band (Urgent data flag set).
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SEND_MSG_OOB
 *   |html </b>
 *   - To send a TCP message out of band
 *   |html </ul>
 *****/


/****t* globus_io/globus_io_handle_type_t
 * NAME
 *   globus_io_handle_type_t
 *
 * DESCRIPTION
 *   A file or socket handle.
 *
 *   Globus I/O uses the abstraction of a handle to act as a common data
 *   type for multiple I/O media. Currently, Globus I/O provides interfaces
 *   to the BSD socket library and the POSIX file I/O library using handles
 *   instead of file descriptors or win32 file or HANDLE objects.
 *
 *   A handle is created by calling one of the following functions:
 *   globus_io_file_open, globus_io_tcp_create_listener,
 *   globus_io_tcp_accept, globus_io_tcp_register_accept, or
 *   globus_io_tcp_connect.
 * 
 *   Attributes may be bound to a handle at creation time. These provide
 *   an interface to socket options, file access options, and security.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_IO_HANDLE_TYPE_TCP_LISTENER
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_HANDLE_TYPE_TCP_CONNECTED
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_HANDLE_TYPE_UDSS_LISTENER
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_HANDLE_TYPE_UDSS_CONNECTED
 *   |html </b>
 * 
 *   o 
 *   |html <b>
 *   GLOBUS_IO_HANDLE_TYPE_FILE
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_HANDLE_TYPE_UDP_UNCONNECTED
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_HANDLE_TYPE_UDP_CONNECTED
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_HANDLE_TYPE_UDDS_UNCONNECTED
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_HANDLE_TYPE_UDDS_CONNECTED
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_HANDLE_TYPE_INTERNAL
 *   |html </b>
 *   |html </ul>
 *****/


/****t* globus_io/globus_io_secure_authentication_mode_t
 * NAME
 *   globus_io_secure_authentication_mode_t
 *
 * DESCRIPTION
 *   Globus I/O GSI authentication modes.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_AUTHENTICATION_MODE_NONE
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_AUTHENTICATION_MODE_MUTUAL
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_AUTHENTICATION_MODE_ANONYMOUS
 *   |html </b>
 *   |html </ul>
 *****/


/****t* globus_io/globus_io_secure_authorization_mode_t
 * NAME
 *   globus_io_secure_authorization_mode_t
 *
 * DESCRIPTION
 *   Globus I/O GSI authorization modes.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o  
 *   |html <b>
 *   GLOBUS_IO_SECURE_AUTHORIZATION_MODE_NONE
 *   |html </b>
 *   - Do not perform any authorization. This will cause an error when
 *   used in conjunction with delegation on the init/client side.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF
 *   |html </b>
 *   - Authorize the peer if the peer has the same identity as ourselves.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_AUTHORIZATION_MODE_IDENTITY
 *   |html </b>
 *   - Authorize the peer if the peer identity matches the identity set in
 *   the target name.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_AUTHORIZATION_MODE_HOST
 *   |html </b>
 *   - Authorize the peer if the identity of the peer matches the identity
 *   of the peer hostname.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_AUTHORIZATION_MODE_CALLBACK
 *   |html </b>
 *   |html </ul>
 *****/


/****t* globus_io/globus_io_secure_protection_mode_t
 * NAME
 *   globus_io_secure_protection_mode_t
 *
 * DESCRIPTION
 *   Globus I/O GSI protection levels.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o  
 *   |html <b>
 *   GLOBUS_IO_SECURE_PROTECTION_MODE_NONE
 *   |html </b>
 *   - No security.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_PROTECTION_MODE_SAFE
 *   |html </b>
 *   - Messages are signed.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_PROTECTION_MODE_PRIVATE
 *   |html </b>
 *   - Messages are signed and encrypted.
 *   |html </ul>
 *****/


/****t* globus_io/globus_io_secure_delegation_mode_t
 * NAME
 *   globus_io_secure_delegation_mode_t
 *
 * DESCRIPTION
 *   Globus I/O GSI delegation modes.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o  
 *   |html <b>
 *   GLOBUS_IO_SECURE_DELEGATION_MODE_NONE
 *   |html </b>
 *   - No delegation.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY
 *   |html </b>
 *   - Delegate a limited proxy.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_DELEGATION_MODE_FULL_PROXY
 *   |html </b>
 *   - Delegate a full proxy.
 *   |html </ul>
 *****/



/****t* globus_io/globus_io_secure_proxy_mode_t
 * NAME
 *   globus_io_secure_proxy_mode_t
 *
 * DESCRIPTION
 *   Globus I/O GSI proxy modes.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o  
 *   |html <b>
 *   GLOBUS_IO_SECURE_PROXY_MODE_NONE
 *   |html </b>
 *   - Accept full proxies and limited proxies if they are only limited
 *   proxy in the cert chain.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_PROXY_MODE_LIMITED
 *   |html </b>
 *   - Accept only full proxies.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_PROXY_MODE_MANY
 *   |html </b>
 *   - Accept both full and limited proxies unconditionally.
 *   |html </ul>
 *****/


/****t* globus_io/globus_io_secure_channel_mode_t
 * NAME
 *   globus_io_secure_channel_mode_t
 *
 * DESCRIPTION
 *   Globus I/O GSI channel modes.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o  
 *   |html <b>
 *   GLOBUS_IO_SECURE_CHANNEL_MODE_CLEAR
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP
 *   |html </b>
 *
 *   o 
 *   |html <b>
 *   GLOBUS_IO_SECURE_CHANNEL_MODE_SSL_WRAP
 *   |html </b>
 *   |html </ul>
 *****/



////////////////////////////////////////////////////////////////////
//
// I/O Operations
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_register_close
 * NAME
 *   globus_io_register_close
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_register_close
 *   |html </b>
 *   |html <i>
 *   handle
 *   callback_script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to asynchronously close the file or connection
 *   described by handle. As soon as the operation completes, the callback 
 *   script is called.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle to close. Any valid handle may be closed by calling
 *   this function.
 *
 *   |html <b>
 *   callback_script
 *   |html </b>
 *   - TCL script invoked once the operation completes.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_register_close returns 0. On error, this 
 *   command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> mechanism.
 *
 * EXAMPLE
 *   lassign [ globus_io_register_close $handle
 *   [ subst { callback callback_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_register_cancel, globus_io_cancel, globus_io_close
 *****/



/****f* globus_io/globus_io_register_cancel
 * NAME
 *   globus_io_register_cancel
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_register_cancel
 *   |html </b>
 *   |html <i>
 *   handle
 *   perform_callbacks
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to cancel any outstanding operation registered
 *   with the specified handle. If perform_callbacks is set to TRUE, then
 *   any callbacks associated with the registered operation will be called
 *   with the result argument indicating that the operation was cancelled.
 *   Additionally, the callback function will be called, with arg set to
 *   the value of callback_arg, once all outstanding callbacks have been
 *   invoked and either completed or reached their conversion point.
 *
 *   If perform_callbacks is not set to TRUE, callbacks will not be
 *   performed and callback and callback_arg arg ignored. This may be
 *   called for any handle type.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle to cancel. Any valid handle may be cancelled by
 *   calling this function.
 *
 *   |html <b>
 *   perform_callbacks
 *   |html </b>
 *   - A boolean value to indicate whether the script will be invoked or
 *   ignored.
 * 
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the operation completes.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_register_cancel returns 0. On error, this 
 *   command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> mechanism.
 *
 * EXAMPLE
 *   lassign [ globus_io_register_cancel $handle TRUE
 *   [ subst { callback callback_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_cancel, globus_io_close, globus_io_register_close
 *****/


/****f* globus_io/globus_io_cancel
 * NAME
 *   globus_io_cancel
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_cancel
 *   |html </b>
 *   |html <i>
 *   handle
 *   perform_callbacks
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to cancel any outstanding operation registered
 *   with the specified handle. This is a blocking form of
 *   globus_io_register_cancel(). This function blocks until the
 *   cancellation is completely processed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle to cancel. Any valid handle may be closed by
 *   calling this function.
 *
 *   |html <b>
 *   perform_callbacks
 *   |html </b>
 *   - If this parameter is set to true, then this function will
 *   block until all outstanding callbacks for this handle have been
 *   invoked and either completed or reached their conversion point.
 *   Otherwise, this will unregister any pending callbacks, and they
 *   will not be called.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_cancel returns 0. On error, this command will 
 *   raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_cancel $handle true ] result
 *
 * BUGS
 *   Cancel may succeed without cancelling anything, or causing any
 *   callbacks to be invoked. There currently is no way to cancel
 *   specific registered callbacks without cancelling every registered
 *   operation.
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_register_cancel, globus_io_register_close,
 *   globus_io_close.
 *****/


/****f* globus_io/globus_io_close
 * NAME
 *   globus_io_close
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_close
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to close the file or connection associated with
 *   a handle. No further operations may be done on this handle. Any
 *   outstanding requests on this handle will be cancelled (globus_io_cancel
 *   will be called with perform_callbacks set to true). This may be called
 *   for any handle type. This function blocks until any cancelled callbacks
 *   have been dispatched and have either reached their thread conversion
 *   point or completed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle to be closed.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_close returns 0. On error, this command will 
 *   raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_close $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_register_close, globus_io_register_cancel,
 *   globus_io_cancel.
 *****/


/****f* globus_io/globus_io_handle_set_user_pointer
 * NAME
 *   globus_io_handle_set_user_pointer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_handle_set_user_pointer
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the user-specified pointer in the handle
 *   structure to user_pointer. This may be called for any handle type.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle.
 *
 *   |html <b>
 *   user-pointer
 *   |html </b>
 *   - The user-specified pointer in the handle structure.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_handle_set_user_pointer returns 0. On error, this
 *   command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_handle_set_user_pointer $handle $user-pointer] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_io/globus_io_handle_get_user_pointer
 * NAME
 *   globus_io_handle_get_user_pointer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_handle_get_user_pointer
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to return the user-specified pointer in the
 *   handle  structure. This may be called for any handle type.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_handle_get_user_pointer returns 0 and the
 *   user-specified pointer in the handle structure. On error, this
 *   command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_handle_get_user_pointer $handle ] result
 *   user-pointer
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// Files
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_file_open
 * NAME
 *   globus_io_file_open
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_file_open
 *   |html </b>
 *   |html <i>
 *   path
 *   flags
 *   mode
 *   attr
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a Globus I/O handle by opening a file.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   path
 *   |html </b>
 *   - The path to the file to open.
 *
 *   |html <b>
 *   flags
 *   |html </b>
 *   - The flags argument consists of a bitwise-or of the values defined
 *   by the globus_io_file_flag_t.
 *
 *   |html <b>
 *   mode
 *   |html </b>
 *   - The permissions of the file; it consists of a bitwise-or of the values
 *   defined by the globus_io_file_create_mode_t. This variable is unused
 *   by globus_io_open if the file is opened with the GLOBUS_IO_FILE_CREAT
 *   flag.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Globus I/O extensible attribute structure.
 *
 *   |html <b>
 *   handle
 *   |html </b>
 *   - Globus I/O file or socket handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_file_open returns 0. On error,
 *   this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_open $path $flags $mode $attr $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_io/globus_io_file_seek
 * NAME
 *   globus_io_file_seek
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_file_seek
 *   |html </b>
 *   |html <i>
 *   handle
 *   offset
 *   whence
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to seek to a new position on an open file.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to perform the seek upon.
 *
 *   |html <b>
 *   offset
 *   |html </b>
 *   - The new position of the file, relative to the whence parameter.
 *
 *   |html <b>
 *   whence
 *   |html </b>
 *   - The whence parameter determines how to interpret the offset.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_file_open returns 0. On error, this command
 *   will raise an error and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_file_seek $handle $offset $whence ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/




////////////////////////////////////////////////////////////////////
//
// Attributes
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_fileattr_init
 * NAME
 *   globus_io_fileattr_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_fileattr_init
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a file attribute structure.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Attribute to initialize.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_fileattr_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_fileattr_init $attr ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_fileattr_destroy
 *****/


/****f* globus_io/globus_io_fileattr_destroy
 * NAME
 *   globus_io_fileattr_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_fileattr_destroy
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a previously allocated file
 *   attribute structure.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute structure to destroy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_fileattr_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_fileattr_destroy $attr ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_fileattr_init
 *****/


/****f* globus_io/globus_io_tcpattr_init
 * NAME
 *   globus_io_tcpattr_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcpattr_init
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a TCP attribute structure.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Attribute to initialize.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_tcpattr_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_tcpattr_init $attr ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_destroy
 *****/


/****f* globus_io/globus_io_tcpattr_destroy
 * NAME
 *   globus_io_tcpattr_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcpattr_destroy
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a previously allocated TCP
 *   attribute structure.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute structure to destroy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_tcpattr_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_tcpattr_destroy $attr ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


////////////////////////////////////////////////////////////////////
//
// File Type
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_file_type
 * NAME
 *   globus_io_attr_set_file_type
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_file_type
 *   |html </b>
 *   |html <i>
 *   attr
 *   file_type
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the file type attribute in the specified
 *   attribute set. This attribute is used to choose whether the file is
 *   opened in text or binary mode on systems where those are different.
 *   This attribute may not be applied to an existing file handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to modify.
 *
 *   |html <b>
 *   file_type
 *   |html </b>
 *   - The new value of the file type.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_file_type returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_file_type $attr $file_type ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_fileattr_init
 *****/


/****f* globus_io/globus_io_attr_get_file_type
 * NAME
 *   globus_io_attr_get_file_type
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_file_type
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the file-type attribute in the
 *   specified attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_file_type returns 0 and the file-type
 *   attribute. On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_file_type $attr ] result file-type 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_fileattr_init
 *****/


////////////////////////////////////////////////////////////////////
//
// TCP Port Restriction
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_tcp_restrict_port
 * NAME
 *   globus_io_attr_set_tcp_restrict_port
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_tcp_restrict_port
 *   |html </b>
 *   |html <i>
 *   attr
 *   restrict_port
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the restrict-port attribute in the
 *   specified TCP attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init and must be destroyed by calling
 *   globus_io_tcpattr_destroy.
 *
 *   |html <b>
 *   restrict_port
 *   |html </b>
 *   - The new value of the restrict_port attribute, 1 means TRUE and 0
 *   means FALSE.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_tcp_restrict_port returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_tcp_restrict_port $attr 1 ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_tcpattr_destroy
 *****/


/****f* globus_io/globus_io_attr_get_tcp_restrict_port
 * NAME
 *   globus_io_attr_get_tcp_restrict_port
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_tcp_restrict_port
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the restrict-port attribute in the
 *   specified TCP attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init and must be destroyed by calling
 *   globus_io_tcpattr_destroy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_tcp_restrict_port returns 0 and the TCP
 *   restrict_port attribute. On error, this command will raise an error
 *   and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_tcp_restrict_port $attr ] result
 *   restrict_port
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_tcpattr_destroy
 *****/


////////////////////////////////////////////////////////////////////
//
// Reuse Address
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_socket_reuseaddr
 * NAME
 *   globus_io_attr_set_socket_reuseaddr
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_socket_reuseaddr
 *   |html </b>
 *   |html <i>
 *   attr
 *   reuseaddr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the reuse-addr attribute in the specified
 *   attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init and must be destroyed by calling
 *   globus_io_tcpattr_destroy.
 *
 *   |html <b>
 *   reuseaddr
 *   |html </b>
 *   - The value of the reuseaddr attribute, 1 means TRUE and 0
 *   means FALSE.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_socket_reuseaddr returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_socket_reuseaddr $attr 1 ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_tcpattr_destroy
 *****/


/****f* globus_io/globus_io_attr_get_socket_reuseaddr
 * NAME
 *   globus_io_attr_get_socket_reuseaddr
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_socket_reuseaddr
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the reuseaddr attribute in the
 *   specified attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_socket_reuseaddr returns 0 and the 
 *   reuseaddr attribute. On error, this command will raise an error
 *   and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_socket_reuseaddr $attr ] result
 *   reuseaddr
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_tcpattr_destroy
 *****/


////////////////////////////////////////////////////////////////////
//
// Socket Keepalive
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_socket_keepalive
 * NAME
 *   globus_io_attr_set_socket_keepalive
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_socket_keepalive
 *   |html </b>
 *   |html <i>
 *   attr
 *   keepalive
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the keepalive attribute in the specified
 *   attribute set. This attribute is used to determine whether or not to
 *   periodically send "keepalive" messages on a connected socket handle.
 *   This may enable earlier detection of broken connections.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init and must be destroyed by calling
 *   globus_io_tcpattr_destroy.
 *
 *   |html <b>
 *   keepalive
 *   |html </b>
 *   - The value of the keepalive attribute, 1 means TRUE and 0
 *   means FALSE.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_socket_keepalive returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_socket_keepalive $attr 1 ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_tcpattr_destroy
 *****/


/****f* globus_io/globus_io_attr_get_socket_keepalive
 * NAME
 *   globus_io_attr_get_socket_keepalive
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_socket_keepalive
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the keepalive attribute in the
 *   specified attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init and must be destroyed by
 *   globus_io_tcpattr_destroy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_socket_keepalive returns 0 and the 
 *   keepalive attribute. On error, this command will raise an error
 *   and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_socket_keepalive $attr ] result
 *   keepalive
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_tcpattr_destroy
 *****/


////////////////////////////////////////////////////////////////////
//
// Socket Linger
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_socket_linger
 * NAME
 *   globus_io_attr_set_socket_linger
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_socket_linger
 *   |html </b>
 *   |html <i>
 *   attr
 *   linger
 *   linger_time
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the linger attribute in the specified
 *   attribute set. This attribute is used to determine what to do when
 *   data is in the socket's buffer when the socket is closed. If linger
 *   is set to true, then the close operation will block until the socket
 *   buffers are empty, or the linger_time has expired.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   linger
 *   |html </b>
 *   - The value of the linger attribute.
 *
 *   |html <b>
 *   linger_time
 *   |html </b>
 *   - The time (in seconds) to block at close time if linger is true
 *   and data is queued in the socket buffer.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_socket_linger returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_socket_linger $attr 1 60 ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


/****f* globus_io/globus_io_attr_get_socket_linger
 * NAME
 *   globus_io_attr_get_socket_linger
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_socket_linger
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the linger attribute in the
 *   specified attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_socket_linger returns 0, the value of
 *   the linger attribute and the time (in seconds) to block at close time
 *   if linger is true and data is queued in the socket buffer.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_socket_linger $attr ] result linger
 *   linger_time
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


////////////////////////////////////////////////////////////////////
//
// Out of Band Data Handling
//
////////////////////////////////////////////////////////////////////

/****f* globus_io/globus_io_attr_set_socket_oobinline
 * NAME
 *   globus_io_attr_set_socket_oobinline
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_socket_oobinline
 *   |html </b>
 *   |html <i>
 *   attr
 *   oobinline 
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the oobinline attribute in the specified
 *   attribute set. This attribute is used to choose whether out-of-band
 *   data is received in the normal data queue, or must be received
 *   specially by calling globus_io_recv or globus_io_register_recv with
 *   the MSG_OOB flag set to true.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   oobinline
 *   |html </b>
 *   - The value of the oobinline attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_socket_oobinline returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_socket_oobinline $attr true ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


/****f* globus_io/globus_io_attr_get_socket_oobinline
 * NAME
 *   globus_io_attr_get_socket_oobinline
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_socket_oobinline
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the oobinline attribute in the
 *   specified attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_socket_oobinline returns 0 and the
 *   value of the oobinline attribute.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_socket_oobinline $attr ] result oobinline
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


////////////////////////////////////////////////////////////////////
//
// Socket Send Buffer
//
////////////////////////////////////////////////////////////////////

/****f* globus_io/globus_io_attr_set_socket_sndbuf
 * NAME
 *    globus_io_attr_set_socket_sndbuf
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_socket_sndbuf
 *   |html </b>
 *   |html <i>
 *   attr
 *   sndbuf 
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the sndbuf attribute in the specified
 *   attribute set. This attribute is used to choose the size of the
 *   send buffer for the socket handle this attribute is applied to.
 *   The send buffer is often used by the operating system to choose
 *   the appropriate TCP window size.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   sndbuf
 *   |html </b>
 *   - The value of the send buffer in bytes.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_socket_sndbuf returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_socket_sndbuf $attr 2048 ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


/****f* globus_io/globus_io_attr_get_socket_sndbuf
 * NAME
 *   globus_io_attr_get_socket_sndbuf
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_socket_sndbuf
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the sndbuf attribute in the
 *   specified attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_socket_sndbuf returns 0 and the
 *   value of the sndbuf attribute.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_socket_sndbuf $attr ] result sndbuf
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/

////////////////////////////////////////////////////////////////////
//
// Socket Receive Buffer
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_socket_rcvbuf
 * NAME
 *    globus_io_attr_set_socket_rcvbuf
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_socket_rcvbuf
 *   |html </b>
 *   |html <i>
 *   attr
 *   rcvbuf 
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the rcvbuf attribute in the specified
 *   attribute set. This attribute is used to choose the size of the
 *   send buffer for the socket handle this attribute is applied to.
 *   The send buffer is often used by the operating system to choose
 *   the appropriate TCP window size.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   rcvbuf
 *   |html </b>
 *   - The value of the send buffer in bytes.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_socket_rcvbuf returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_socket_rcvbuf $attr 2048 ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


/****f* globus_io/globus_io_attr_get_socket_rcvbuf
 * NAME
 *   globus_io_attr_get_socket_rcvbuf
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_socket_rcvbuf
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the rcvbuf attribute in the
 *   specified attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_socket_rcvbuf returns 0 and the
 *   value of the rcvbuf attribute.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_socket_rcvbuf $attr ] result rcvbuf
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


////////////////////////////////////////////////////////////////////
//
// TCP No delay
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_tcp_nodelay
 * NAME
 *   globus_io_attr_set_tcp_nodelay
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_tcp_nodelay
 *   |html </b>
 *   |html <i>
 *   attr
 *   nodelay 
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the nodelay attribute in the specified
 *   TCP attribute set. This attribute is used to determine whether or
 *   not to disable Nagle's algorithm. If set to true, the socket will
 *   send packets as soon as possible with no unnecessary delays introduced.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   nodelay
 *   |html </b>
 *   - The new value of the nodelay attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_tcp_nodelay returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_tcp_nodelay $attr true ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


/****f* globus_io/globus_io_attr_get_tcp_nodelay
 * NAME
 *   globus_io_attr_get_tcp_nodelay
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_tcp_nodelay
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the nodelay attribute in the
 *   specified attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_tcp_nodelay returns 0 and the
 *   value of the nodelay attribute.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_tcp_nodelay $attr ] result nodelay
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


////////////////////////////////////////////////////////////////////
//
// TCP Interface
//
////////////////////////////////////////////////////////////////////

/****f* globus_io/globus_io_attr_set_tcp_interface
 * NAME
 *   globus_io_attr_set_tcp_interface
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_tcp_interface
 *   |html </b>
 *   |html <i>
 *   attr
 *   interface 
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the TCP interface attribute in the
 *   specified TCP attribute set. This attribute is used to determine
 *   whether to bind TCP sockets to a particular interface. This attribute
 *   must be set before calling globus_io_tcp_create_listener or
 *   globus_io_tcp_connect.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   interface
 *   |html </b>
 *   - The value of the interface attribute. The interface string must be
 *   in dotted-ip format (ie "127.0.0.1").
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_tcp_interface returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_tcp_interface $attr 127.0.0.1 ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


/****f* globus_io/globus_io_attr_get_tcp_interface
 * NAME
 *   globus_io_attr_get_tcp_interface
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_tcp_interface
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the TCP interface attribute in the
 *   specified TCP attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_tcp_interface returns 0 and the
 *   value of the interface attribute.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_tcp_interface $attr ] result interface
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init
 *****/


////////////////////////////////////////////////////////////////////
//
// Authentication Mode
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_secure_authentication_mode
 * NAME
 *   globus_io_attr_set_secure_authentication_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_secure_authentication_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   mode
 *   credential
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the authentication mode attribute in
 *   the specified socket attribute set. 
 *
 *   This attribute is used to determine whether or not to call the
 *   GSSAPI security context establishment functions once a socket
 *   connection is established.
 *
 *   If the authentication_mode value is
 *   GLOBUS_IO_SECURE_AUTHENTICATION_MODE_NONE, then the channel mode,
 *   delegation mode, protection mode, and authorization mode will all
 *   be reset to disable all security on the socket attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   mode
 *   |html </b>
 *   - The value of the authentication-mode attribute. The values for
 *   mode are described in the documentation for the
 *   globus_io_secure_authentication_mode_t type.
 *
 *   |html <b>
 *   credential
 *   |html </b>
 *   - A GSSAPI credential to be used when authenticating. If the
 *   credential is equal to GSS_C_NO_CREDENTIAL, then Globus I/O will
 *   use the process's default credentials.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_secure_authentication_mode returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_secure_authentication_mode $attr
 *   $GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI $gss_c_no_credential ]
 *   result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_authorization_mode_t
 *****/


/****f* globus_io/globus_io_attr_get_secure_authentication_mode
 * NAME
 *   globus_io_attr_get_secure_authentication_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_secure_authentication_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the authentication mode attribute in
 *   the specified socket attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_secure_authentication_mode returns 0 and 
 *   the values of mode and credential attributes. On error, this command
 *   will raise an error and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_secure_authentication_mode $attr ]
 *   result mode credential
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_authorization_mode_t
 *****/


////////////////////////////////////////////////////////////////////
//
// Authorization Mode
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_secure_authorization_mode
 * NAME
 *   globus_io_attr_set_secure_authorization_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_secure_authorization_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   mode
 *   data
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the authorization-mode attribute in the
 *   specified socket attribute set. 
 *
 *   This attribute is used to determine what security identities to
 *   authorize as the peer to the security handshake done when making an
 *   authenticated connection.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   mode
 *   |html </b>
 *   - The new value of the authorization-mode attribute. The values for
 *   mode are described in the documentation for the
 *   globus_io_secure_authorization_mode_t type.
 *
 *   |html <b>
 *   data
 *   |html </b>
 *   - A globus_io_secure_authorization_data_t structure containing any
 *   additional information necessary to perform the specified type of
 *   authorization.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_secure_authorization_mode returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_secure_authorization_mode $attr
 *   $GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF $auth_data ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_authorization_mode_t
 *****/


/****f* globus_io/globus_io_attr_get_secure_authorization_mode
 * NAME
 *   globus_io_attr_get_secure_authorization_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_secure_authorization_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the authorization mode attribute in
 *   the specified socket attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attributes to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_secure_authorization_mode returns 0 and 
 *   the values of mode and data attributes. On error, this command
 *   will raise an error and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_secure_authorization_mode $attr ]
 *   result mode data
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_authorization_mode_t
 *****/


////////////////////////////////////////////////////////////////////
//
// Authorization Data
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_secure_authorization_data_initialize
 * NAME
 *   globus_io_secure_authorization_data_initialize
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_secure_authorization_data_initialize
 *   |html </b>
 *   |html <i>
 *   data
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a data structure to hold
 *   authorization-mode specific data.
 *
 *   This function must be called before any of the data accessors
 *   for this structure can be used.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   data
 *   |html </b>
 *   - The data structure to initialize.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_secure_authorization_data_initialize returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_secure_authorization_data_initialize $data ]
 *   result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_secure_authorization_data_destroy,
 *   globus_io_secure_authorization_data_get_callback,
 *   globus_io_secure_authorization_data_set_callback,
 *   globus_io_secure_authorization_data_get_identity,
 *   globus_io_secure_authorization_data_set_identity
 *****/


/****f* globus_io/globus_io_secure_authorization_data_destroy
 * NAME
 *   globus_io_secure_authorization_data_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_secure_authorization_data_destroy
 *   |html </b>
 *   |html <i>
 *   data
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to free a data structure to hold
 *   authorization-mode specific data.
 *
 *   This function must be after the data structure is no longer
 *   being used to free memory allocated by setting it's member data.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   data
 *   |html </b>
 *   - The data structure to free. Once this function returns, the
 *   data structure must not be used until another call to
 *   globus_io_secure_authorization_data_initialize is made with this
 *   structure.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_secure_authorization_data_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_secure_authorization_data_destroy $data ]
 *   result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_secure_authorization_data_initialize,
 *   globus_io_secure_authorization_data_get_callback,
 *   globus_io_secure_authorization_data_set_callback,
 *   globus_io_secure_authorization_data_get_identity,
 *   globus_io_secure_authorization_data_set_identity
 *****/


/****f* globus_io/globus_io_secure_authorization_data_set_identity
 * NAME
 *   globus_io_secure_authorization_data_set_identity
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_secure_authorization_data_set_identity
 *   |html </b>
 *   |html <i>
 *   data
 *   identity 
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the "identity" member of the
 *   authorization-mode specific data structure.
 *
 *   This member should be set to a string containing the security
 *   distinguished name of the peer which you will authorize for
 *   this connection. The identity is only when the authorization-mode
 *   attribute is set to GLOBUS_IO_SECURE_AUTHORIZATION_MODE_IDENTITY.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   data
 *   |html </b>
 *   - The data structure previously initialized by a call to
 *   globus_io_secure_authorization_data_initialize.
 *
 *   |html <b>
 *   identity
 *   |html </b>
 *   - The string (a specific subject name) naming the security identity
 *   to authorize. 
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_secure_authorization_data_set_identity returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_secure_authorization_data_set_identity
 *   $data
 *   "/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621/CN=2095438022" ]
 *   result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_secure_authorization_data_initialize,
 *   globus_io_secure_authorization_data_destroy,
 *   globus_io_secure_authorization_data_get_callback,
 *   globus_io_secure_authorization_data_set_callback,
 *   globus_io_secure_authorization_data_get_identity,
 *****/


/****f* globus_io/globus_io_secure_authorization_data_get_identity
 * NAME
 *   globus_io_secure_authorization_data_get_identity
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_secure_authorization_data_get_identity
 *   |html </b>
 *   |html <i>
 *   data
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the "identity" member of the
 *   authorization-mode specific data structure.
 *
 *   This member is set by a call to
 *   globus_io_secure_authorization_mode_set_identity.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   data
 *   |html </b>
 *   - The data structure previously initialized by a call to
 *   globus_io_secure_authorization_data_initialize.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_secure_authorization_data_set_identity returns 0
 *   and the identity string set in the data structure. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_secure_authorization_data_get_identity $data ]
 *   result identity
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_secure_authorization_data_initialize,
 *   globus_io_secure_authorization_data_destroy,
 *   globus_io_secure_authorization_data_get_callback,
 *   globus_io_secure_authorization_data_set_callback,
 *   globus_io_secure_authorization_data_set_identity,
 *****/


/****f* globus_io/globus_io_secure_authorization_data_set_callback
 * NAME
 *   globus_io_secure_authorization_data_set_callback
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_secure_authorization_data_set_callback
 *   |html </b>
 *   |html <i>
 *   data
 *   callback
 *   callback_arg
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the callback and callback_arg members
 *   of the authorization-mode specific data structure.
 *
 *   This member is used when setting the authorization-mode attribute
 *   of a TCP attribute set to GLOBUS_IO_SECURE_AUTHORIZATION_MODE_CALLBACK
 *   in a call to globus_io_attr_set_secure_authorization_mode.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   data
 *   |html </b>
 *   - The data structure previously initialized by a call to
 *   globus_io_secure_authorization_data_initialize.
 *
 *   |html <b>
 *   callback
 *   |html </b>
 *   - The new value of the callback member.
 *
 *   |html <b>
 *   callback_arg
 *   |html </b>
 *   - The new value of the callback_arg member.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_secure_authorization_data_set_callback returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_secure_authorization_data_set_callback
 *   $data { callback_proc callback_arg } ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_secure_authorization_data_initialize,
 *   globus_io_secure_authorization_data_destroy,
 *   globus_io_secure_authorization_data_get_callback,
 *   globus_io_secure_authorization_data_set_identity,
 *   globus_io_secure_authorization_data_get_identity,
 *****/



/****f* globus_io/globus_io_secure_authorization_data_get_callback
 * NAME
 *   globus_io_secure_authorization_data_get_callback
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_secure_authorization_data_get_callback
 *   |html </b>
 *   |html <i>
 *   data
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the callback and callback-arg members of
 *   the authorization-mode specific data structure.
 *
 *   This member is set by a call to the function
 *   globus_io_secure_authorization_data_set_callback.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   data
 *   |html </b>
 *   - The data structure previously initialized by a call to
 *   globus_io_secure_authorization_data_initialize.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_secure_authorization_data_get_callback returns 0
 *   and the current values of the callback and callback-arg in the data
 *   structure. On error, this command will raise an error and must be
 *   caught using the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_secure_authorization_data_initialize,
 *   globus_io_secure_authorization_data_destroy,
 *   globus_io_secure_authorization_data_get_callback,
 *   globus_io_secure_authorization_data_set_identity,
 *   globus_io_secure_authorization_data_get_identity,
 *****/


////////////////////////////////////////////////////////////////////
//
// Channel Mode
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_secure_channel_mode
 * NAME
 *   globus_io_attr_set_secure_channel_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_secure_channel_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   mode
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the channel mode attribute in
 *   the specified socket attribute set. 
 *
 *   This attribute is used to determine if any data wrapping should
 *   be done on the socket connection. This is required to use the
 *   data protection attribute.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   mode
 *   |html </b>
 *   - The new value of the channel-mode attribute. The values for
 *   mode are described in the globus_io_secure_channel_mode_t type.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_secure_channel_mode returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_secure_channel_mode $attr
 *   $GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_channel_mode_t
 *****/


/****f* globus_io/globus_io_attr_get_secure_channel_mode
 * NAME
 *   globus_io_attr_get_secure_channel_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_secure_channel_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the channel mode attribute in
 *   the specified socket attribute set. 
 *
 *   This attribute is used to determine if any data wrapping should
 *   be done on the socket connection. This is required to use the
 *   data protection attribute.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_secure_channel_mode returns 0 and
 *   the value of the channel-mode attribute. On error, this command
 *   will raise an error and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_secure_channel_mode $attr ] result
 *   channel-mode
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_channel_mode_t
 *****/


////////////////////////////////////////////////////////////////////
//
// Data Protection
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_secure_protection_mode
 * NAME
 *   globus_io_attr_set_secure_protection_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_secure_protection_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   protection-mode
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the protection mode attribute in
 *   the specified socket attribute set. 
 *
 *   This attribute is used to determine if any data protection
 *   should be done on the socket connection.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   protection-mode
 *   |html </b>
 *   - The new value of the protection-mode attribute. The values for
 *   mode are described in the globus_io_secure_protection_mode_t type.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_secure_protection_mode returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_secure_protection_mode $attr
 *   $GLOBUS_IO_SECURE_PROTECTION_MODE_PRIVATE ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_protection_mode_t
 *****/


/****f* globus_io/globus_io_attr_get_secure_protection_mode
 * NAME
 *   globus_io_attr_get_secure_protection_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_secure_protection_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the protection mode attribute in
 *   the specified socket attribute set. 
 *
 *   This attribute is used to determine if any data protection
 *   should be done on the socket connection.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_secure_protection_mode returns 0 and
 *   the value of the protection-mode attribute. On error, this command
 *   will raise an error and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_secure_protection_mode $attr ] result
 *   protection-mode
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_protection_mode_t
 *****/


////////////////////////////////////////////////////////////////////
//
// Delegation
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_secure_delegation_mode
 * NAME
 *   globus_io_attr_set_secure_delegation_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_secure_delegation_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   delegation-mode
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the delegation mode attribute in
 *   the specified socket attribute set. 
 *
 *   This attribute is used to determine whether the process's
 *   credentials should be delegated to the other side of the connection.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   delegation-mode
 *   |html </b>
 *   - The new value of the delegation-mode attribute. The values for
 *   mode are described in the globus_io_secure_delegation_mode_t type.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_secure_delegation_mode returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_secure_delegation_mode $attr
 *   $GLOBUS_IO_SECURE_DELEGATION_MODE_NONE ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_delegation_mode_t
 *****/


/****f* globus_io/globus_io_attr_get_secure_delegation_mode
 * NAME
 *   globus_io_attr_get_secure_delegation_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_secure_delegation_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the delegation mode attribute in
 *   the specified socket attribute set. 
 *
 *   This attribute is used to determine if any data delegation
 *   should be done on the socket connection.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_secure_delegation_mode returns 0 and
 *   the value of the delegation-mode attribute. On error, this command
 *   will raise an error and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_secure_delegation_mode $attr ] result
 *   delegation-mode
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_delegation_mode_t
 *****/


////////////////////////////////////////////////////////////////////
//
// Authentication using Proxy
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_secure_proxy_mode
 * NAME
 *   globus_io_attr_set_secure_proxy_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_secure_proxy_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   proxy-mode
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the proxy mode attribute in
 *   the specified socket attribute set. 
 *
 *   This attribute is used to determine whether the process should
 *   accept limited proxy certificates for authentication.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *
 *   |html <b>
 *   proxy-mode
 *   |html </b>
 *   - The new value of the proxy-mode attribute. The values for
 *   mode are described in the globus_io_secure_proxy_mode_t type.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_secure_proxy_mode returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_set_secure_proxy_mode $attr
 *   $GLOBUS_IO_SECURE_PROXY_MODE_LIMITED ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_proxy_mode_t
 *****/


/****f* globus_io/globus_io_attr_get_secure_proxy_mode
 * NAME
 *   globus_io_attr_get_secure_proxy_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_secure_proxy_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the proxy mode attribute in
 *   the specified socket attribute set. 
 *
 *   This attribute is used to determine if any data proxy
 *   should be done on the socket connection.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to query. The attr parameter must be initialized
 *   by globus_io_tcpattr_init.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_secure_proxy_mode returns 0 and
 *   the value of the proxy-mode attribute. On error, this command
 *   will raise an error and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_io_attr_get_secure_proxy_mode $attr ] result
 *   proxy-mode
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcpattr_init, globus_io_secure_proxy_mode_t
 *****/


////////////////////////////////////////////////////////////////////
//
// Extension OIDs
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_attr_set_secure_extension_oids
 * NAME
 *   globus_io_attr_set_secure_extension_oids
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_set_secure_extension_oids
 *   |html </b>
 *   |html <i>
 *   attr
 *   extension_oids
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the extension OID (Object Identifiers)
 *   attribute in the specified attribute set. 
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to modify.
 *
 *   |html <b>
 *   extension_oids
 *   |html </b>
 *   - The new value of the extension_oids attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_set_secure_extension_oids returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_io/globus_io_attr_get_secure_extension_oids
 * NAME
 *   globus_io_attr_get_secure_extension_oids
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_attr_get_secure_extension_oids
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the extension_oids attribute in
 *   the specified attribute set. 
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_io_attr_get_secure_extension_oids returns 0 and
 *   the value of the extension_oids attribute. On error, this command
 *   will raise an error and must be caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// TCP Sockets
//
////////////////////////////////////////////////////////////////////


/****f* globus_io/globus_io_tcp_connect
 * NAME
 *   globus_io_tcp_connect
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_connect
 *   |html </b>
 *   |html <i>
 *   host
 *   port
 *   attr
 *   handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to perform blocking TCP connection establishment
 *   on the specified host/port pair.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   host
 *   |html </b>
 *   - The host to connect to.
 *
 *   |html <b>
 *   port
 *   |html </b>
 *   - The TCP port number of the server.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - TCP socket attributes. The attr, if non-NULL, should be initialized
 *   by a call to globus_io_tcp_attr_init. The attribute settings for the
 *   attr structure can be modified by any of the globus_io_tcp attribute
 *   accessors. The security attributes must be compatible with those of
 *   the server handle.
 *
 *   |html <b>
 *   handle
 *   |html </b>
 *   - Globus I/O handle to be initialized when the connection is completed.
 *   The application must not use this handle until this function has
 *   returned.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_tcp_connect returns 0 and the Globus
 *   I/O handle to be initialized when the connection is completed.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_connect $host $port $attr $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcp_register_connect
 *****/


/****f* globus_io/globus_io_tcp_create_listener
 * NAME
 *   globus_io_tcp_create_listener
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_create_listener
 *   |html </b>
 *   |html <i>
 *   port
 *   backlog
 *   attr
 *   handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to create a socket handle capable of accepting
 *   new TCP connections from other hosts or processes.
 *
 *   In order to actively listen for connections, you must call
 *   globus_io_tcp_register_listen or globus_io_tcp_listen with the new
 *   handle returned by this function.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   port
 *   |html </b>
 *   - The TCP port that the socket will listen for connections on. If the
 *   port number is 0, then an arbitrary TCP port will be selected. If this
 *   is true, and the restrict_port attribute is set to TRUE (the default)
 *   and the GLOBUS_TCP_PORT_RANGE environment variable was set when Globus
 *   I/O was initialized, then the port will be selected from that range.
 *   Otherwise, any port number may be chosen.
 *
 *   |html <b>
 *   backlog
 *   |html </b>
 *   - The backlog parameter indicates the maximum length of the system's
 *   queue of pending connections. Any connection attempts when the queue
 *   is full will fail. If backlog is equal to -1, then the system-specific
 *   maximum queue length will be used.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - The attributes of this server. The attributes will be used as the
 *   default for handles created by accepting a connection on this handle.
 *
 *   |html <b>
 *   handle
 *   |html </b>
 *   - The new handle which will be created for this server. It can only
 *   be used for listening for and establishing connections, and may be
 *   closed by calling globus_io_register_close or globus_io_close; other
 *   I/O operations on this handle will fail.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_tcp_create_listener returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_create_listener 0 -1 $attr $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcp_register_listen, globus_io_tcp_listen,
 *   globus_io_tcp_register_accept, globus_io_tcp_accept
 *****/



/****f* globus_io/globus_io_tcp_register_accept
 * NAME
 *   globus_io_tcp_register_accept
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_register_accept
 *   |html </b>
 *   |html <i>
 *   listener_handle
 *   attr
 *   new_handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to post asynchronous server-side TCP connection
 *   establishment.
 *
 *   Once the connection has been accepted, the script will be
 *   called, with the callback_arg and a newly created, connected, handle
 *   argument passed to the script.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_handle
 *   |html </b>
 *   - A Globus I/O handle created by globus_io_tcp_create_listener.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - The attributes of the new connection. If this is NULL, than the
 *   attributes passed to globus_io_tcp_create_listener will be used. The
 *   attributes should be compatible with those passed to that function.
 *   Security attributes can not be changed between those two calls.
 *
 *   |html <b>
 *   new_handle
 *   |html </b>
 *   - A handle to the new connection. This handle must not be used for
 *   any operation until the callback function is called. 
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the connection operation is established,
 *   and any security exchange has taken place. The "handle" passed to the
 *   script will be the "new_handle" passed to this function.The command
 *   will be executed exactly once in global scope. If an error occurs
 *   while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_tcp_register_accept returns 0 and the new Globus
 *   I/O handle to be initialized when the connection is completed.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_register_accept $listener_handle $attr
 *   $new_handle [ subst { callback callback_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_io/globus_io_tcp_accept
 * NAME
 *   globus_io_tcp_accept
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_accept
 *   |html </b>
 *   |html <i>
 *   listener_handle
 *   attr
 *   new_handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to perform blocking TCP connection establishment
 *   on the specified host/port pair.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_handle
 *   |html </b>
 *   - A Globus I/O handle created by globus_io_tcp_create_listener.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - The attributes of the new connection. If this is NULL, than the
 *   attributes passed to globus_io_tcp_create_listener will be used. The
 *   attributes should be compatible with those passed to that function.
 *   Security attributes can not be changed between those two calls.
 *
 *   |html <b>
 *   new_handle
 *   |html </b>
 *   - A handle to the new connection. This handle must not be used for
 *   any operation until this function returns successfully.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_tcp_accept returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_accept $listener_handle $attr $new_handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_io/globus_io_tcp_get_attr
 * NAME
 *   globus_io_tcp_get_attr
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_get_attr
 *   |html </b>
 *   |html <i>
 *   handle
 *   attr
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to extract the TCP attributes from a
 *   Globus I/O handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The Globus I/O handle to query.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - The attribute structure to be initialized to the same attributes
 *   as the Globus I/O handle.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_tcp_get_attr returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_get_attr $handle $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_io/globus_io_tcp_get_security_context
 * NAME
 *   globus_io_tcp_get_security_context
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_get_security_context
 *   |html </b>
 *   |html <i>
 *   handle
 *   context
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to extract the security context from a
 *   Globus I/O handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The Globus I/O handle to query.
 *
 *   |html <b>
 *   context
 *   |html </b>
 *   - The handle's context will be copied into this parameter. Note that
 *   a shallow copy operation is done. If the context is needed beyond the
 *   lifetime of the handle, it should be exported and then re-imported using
 *   the GSSAPI. If message integrity is desired it is often easier to use
 *   the secure channel modes defined by Globus I/O. If no security
 *   attributes are set on the handle, then this parameter will be set to
 *   GSS_C_NO_CONTEXT.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_tcp_get_security_context returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_io/globus_io_tcp_get_delegated_credential
 * NAME
 *   globus_io_tcp_get_delegated_credential
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_get_delegated_credential
 *   |html </b>
 *   |html <i>
 *   handle
 *   cred
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to retrieve the delegated credential from a
 *   a secure TCP connection. If the TCP handle is set to
 *   GLOBUS_IO_SECURE_AUTHENTICATION_MODE_NONE or the client did not
 *   delegate a credential, the cred value will be set to NULL.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The Globus I/O handle to query. This may only be used on handles
 *   created by calling globus_io_tcp_register_accept or
 *   globus_io_tcp_accept.
 *
 *   |html <b>
 *   cred
 *   |html </b>
 *   - The handle's delegated credential will be copied into this parameter.
 *   Note that a shallow copy operation is done. If the credential is needed
 *   beyond the lifetime of the handle, it should be exported and then
 *   re-imported using the GSSAPI. If no security attributes are set on the
 *   handle, then this parameter will be set to GSS_C_NO_CREDENTIAL.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_tcp_get_delegated_credential returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/




/****f* globus_io/globus_io_tcp_register_listen
 * NAME
 *   globus_io_tcp_register_listen
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_register_listen
 *   |html </b>
 *   |html <i>
 *   handle
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to post asynchronous wait until a client
 *   connection is pending.
 *
 *   This function will invoke the script when the Globus I/O handle
 *   has a connection pending. Calls to this function or
 *   globus_io_tcp_listen must be made before each call to
 *   globus_io_tcp_accept or globus_io_tcp_register_accept.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The listener handle which will be monitored.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the connection operation is pending.
 *   The command will be executed exactly once in global scope. If
 *   an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_tcp_register_listen returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_register_listen $handle
 *   [ subst { callback callback_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcp_listen, globus_io_tcp_register_accept,
 *   globus_io_tcp_accept.
 *****/


/****f* globus_io/globus_io_tcp_listen
 * NAME
 *   globus_io_tcp_listen
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_io
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_io_tcp_listen
 *   |html </b>
 *   |html <i>
 *   listener_handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command will block until a client connection is pending on
 *   the listener_handle. Once this function returns, the user application
 *   can call globus_io_tcp_register_accept or globus_io_tcp_accept
 *   to create a newly connected handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   listener_handle
 *   |html </b>
 *   - The listener handle which will be monitored.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_io_tcp_listen returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_io_tcp_listen $listener_handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_io_tcp_register_listen, globus_io_tcp_register_accept,
 *   globus_io_tcp_accept.
 *****/


#endif /* SWIG */


