#include "config.h"

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "common/sgdebug.h"
#include "common/sginit.h"

void
sginit ( )
{
  char* x509_user_key;
  char* x509_user_cert;
  int x509_user_key_f_ok;
  int x509_user_cert_f_ok;
  
  static char function[] = "sginit";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
  
  x509_user_key = getenv(X509_USER_KEY);
  if ( x509_user_key )
    {
      SGDEBUG_LVL_FMT_1( 10, "X509_USER_KEY => before: %s", x509_user_key);
    }

  x509_user_cert = getenv(X509_USER_CERT);
  if ( x509_user_cert )
    {
      SGDEBUG_LVL_FMT_1( 10, "X509_USER_CERT => before: %s", x509_user_cert);
    }

  if ( x509_user_key )
    {
      x509_user_key_f_ok = access(x509_user_key, F_OK);
    }
  if ( x509_user_cert )
    {
      x509_user_cert_f_ok = access(x509_user_cert, F_OK);
    }
  
  /* Both env variables AREN'T null */
  if ( x509_user_key && x509_user_cert )
    {
      /* Both env variables point to existing files */
      if ( x509_user_key_f_ok == 0 && x509_user_cert_f_ok == 0 )
	{
	  /* X509_USER_KEY ISN'T readable */
	  if ( access(x509_user_key, R_OK) != 0 )
	    {
	      unsetenv(X509_USER_KEY);
	      unsetenv(X509_USER_CERT);
	    }
	}
      else
	{
	  /* One of the env variables points to non-existing file */
	  if ( x509_user_key_f_ok != 0 || x509_user_cert_f_ok != 0 )
	    {
	      unsetenv(X509_USER_KEY);
	      unsetenv(X509_USER_CERT);
	    }
	}
    }
  else
    {
      /* One of the env variables IS null */
      if ( ( ! x509_user_key && x509_user_cert ) ||
	   ( x509_user_key && ! x509_user_cert ) )
	{
	  unsetenv(X509_USER_KEY);
	  unsetenv(X509_USER_CERT);
	}
    }

  x509_user_key = getenv(X509_USER_KEY);
  if ( x509_user_key )
    {
      SGDEBUG_LVL_FMT_1( 10, "X509_USER_KEY => after: %s", x509_user_key);
    }

  x509_user_cert = getenv(X509_USER_CERT);
  if ( x509_user_cert )
    {
      SGDEBUG_LVL_FMT_1( 10, "X509_USER_CERT => after: %s", x509_user_cert);
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
}
