#include "tcl.h"
#include "tcl_vararg.h"

#include "common/sgdebug.h"

#include "globus_xio_gsi.h"

extern struct swig_type_info *ty;

/**************************************************************************
 *
 * GET UTILITY FUNCTIONS
 *
 **************************************************************************/

/*
 *-------------------------------------------------------------------------
 * Get an integer object from Tcl interpreter
 *-------------------------------------------------------------------------
 */
 
int
GetVarArgAsInt( driver_call_info_type* Parms,
                unsigned int Offset,
                int* Result )
{
  TclInterpInfo_type*	interp_info =
    (TclInterpInfo_type*)(Parms->m_interp_info);
  
  if ( Tcl_GetIntFromObj( interp_info->m_interp,
                          ((Tcl_Obj**)(Parms->m_var_argv))[ Offset ],
                          Result ) == TCL_ERROR )
    {
    return 0;
  }
  return 1;
}


/*
 *-------------------------------------------------------------------------
 * Get a string object from Tcl interpreter 
 *-------------------------------------------------------------------------
 */

char*
GetVarArgAsString( driver_call_info_type* Parms,
                   unsigned int Offset,
                   int* Length )
{
  SGDEBUG_LVL_FMT_1( 10,
		     "GetVarArgAsString: m_var_argv: %p", Parms->m_var_argv );
  SGDEBUG_LVL_FMT_1( 10, "GetVarArgAsString: m_interp_info: %p", 
		     Parms->m_interp_info );

  return( Tcl_GetStringFromObj( ((Tcl_Obj**)(Parms->m_var_argv))[ Offset ],
				Length ) );
}

/*
 *-------------------------------------------------------------------------
 * Get a pointer object from Tcl interpreter 
 *-------------------------------------------------------------------------
 */

void
GetVarArgAsPointer( driver_call_info_type* Parms,
                    unsigned int Offset,
                    void* Result )
{
  TclInterpInfo_type*	interp_info =
    (TclInterpInfo_type*)(Parms->m_interp_info);

  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_GSI_SET_CREDENTIAL:
    case GLOBUS_XIO_GSI_INIT_DELEGATION:
      {
        ty =
          (struct swig_type_info *)GlobusXio_SWIG_TypeQuery("gss_cred_id_t");
      }
      break;

    case GLOBUS_XIO_GSI_SET_TARGET_NAME:
      {
        ty =
          (struct swig_type_info *)GlobusXio_SWIG_TypeQuery("gss_name_t");
      }
      break;

    default:
      //:TODO: Invalid command
      break;
    }
  
  SGDEBUG_LVL_FMT_1( 10,
		     "Before GlobusXio_SWIG_ConvertPtr: Result: %ld", 
		     *((gss_cred_id_t*)Result) );

  SGDEBUG_LVL_FMT_1( 10, "m_var_argv: %p", Parms->m_var_argv );

  SGDEBUG_LVL_FMT_1( 10, "m_interp_info: %p", interp_info );

  SGDEBUG_LVL_FMT_1
    ( 10, "m_var_argv: %s", 
      Tcl_GetString( ((Tcl_Obj**)(Parms->m_var_argv))[ Offset ] ) );

  if ( (GlobusXio_SWIG_ConvertPtr
        (interp_info->m_interp,
         ((Tcl_Obj**)(Parms->m_var_argv))[ Offset ],
         (void**)&Result, 
         ty, 
         0) != TCL_OK) )
    {
      SGDEBUG_LVL_FMT( 10, "GlobusXio_SWIG_ConvertPtr fails");
    }

  SGDEBUG_LVL_FMT_1( 10, "After GlobusXio_SWIG_ConvertPtr: Result: %ld", 
                 Result );
}



/**************************************************************************
 *
 * APPEND UTILITY FUNCTIONS
 *
 **************************************************************************/

/*
 *-------------------------------------------------------------------------
 * Append an integer object into Tcl interpreter 
 *-------------------------------------------------------------------------
 */

void
AppendInt( driver_call_info_type* Parms,
           int Value )
{
  int len;
  
  SGDEBUG_LVL_FMT_1( 10, "AppendInt: Value: %d", Value );
  
  TclInterpInfo_type*	interp_info =
    (TclInterpInfo_type*)(Parms->m_interp_info);

  Tcl_ListObjLength( interp_info->m_interp, interp_info->m_results,
                     &len );
  
  SGDEBUG_LVL_FMT_1( 10, "AppendInt: before adding to list: len: %d", len );
  
  Tcl_ListObjAppendElement( interp_info->m_interp,
                            interp_info->m_results,
                            Tcl_NewIntObj( Value ) );
  
  Tcl_ListObjLength( interp_info->m_interp, interp_info->m_results,
                     &len );

  SGDEBUG_LVL_FMT_1( 10, "AppendInt: done: len: %d", len );
}


/*
 *-------------------------------------------------------------------------
 * Append a string object into Tcl interpreter 
 *-------------------------------------------------------------------------
 */

void
AppendString( driver_call_info_type* Parms,
              char* Value )
{
  TclInterpInfo_type*	interp_info =
    (TclInterpInfo_type*)(Parms->m_interp_info);

  
  Tcl_ListObjAppendElement( interp_info->m_interp,
                            interp_info->m_results,
                            Tcl_NewStringObj(Value, strlen(Value)) );
}


/*
 *-------------------------------------------------------------------------
 * Append a pointer object into Tcl interpreter 
 *-------------------------------------------------------------------------
 */

void
AppendPointer( driver_call_info_type* Parms,
               void* Value )
{
  TclInterpInfo_type*	interp_info =
    (TclInterpInfo_type*)(Parms->m_interp_info);

  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_GSI_GET_CREDENTIAL:
    case GLOBUS_XIO_GSI_GET_DELEGATED_CRED:
      {
        ty =
          (struct swig_type_info *)GlobusXio_SWIG_TypeQuery("gss_cred_id_t *");
      }
      break;

    case GLOBUS_XIO_GSI_GET_TARGET_NAME:
    case GLOBUS_XIO_GSI_GET_PEER_NAME:
    case GLOBUS_XIO_GSI_GET_LOCAL_NAME:
      {
        ty =
          (struct swig_type_info *)GlobusXio_SWIG_TypeQuery("gss_name_t *");
      }
      break;

    case GLOBUS_XIO_GSI_GET_CONTEXT:
      {
         ty =
          (struct swig_type_info *)GlobusXio_SWIG_TypeQuery("gss_ctx_id_t *");
      }
      break;

    case GLOBUS_XIO_GSI_ACCEPT_DELEGATION:
      {
        ty =
          (struct swig_type_info *)GlobusXio_SWIG_TypeQuery("gss_cred_id_t *");
      }
      break;
      
    default:
      // Invalid command
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "AppendPointer: m_interp_info: %p",
		     Parms->m_interp_info );
  SGDEBUG_LVL_FMT_1( 10, "AppendPointer: Value: %d",
		     *(gss_cred_id_t*)Value );
  
  Tcl_Obj* o =
    (Tcl_Obj *)GlobusXio_SWIG_NewInstanceObj(interp_info->m_interp,
                                             Value, ty, 0);

  SGDEBUG_LVL_FMT( 10, "AppendPointer: After GlobusXio_SWIG_NewInstanceObj" );
  
  Tcl_ListObjAppendElement( interp_info->m_interp,
                            interp_info->m_results,
                            (Tcl_Obj*)o );

  SGDEBUG_LVL_FMT( 10, "AppendPointer: After Tcl_ListObjAppendElement" );
}
