//:TODO: Need to have this come from the compiler command line.
#define TCL

#include "globus_xio.h"
#include "globus_xio_file_driver.h"

#include "common/sgdebug.h"

#include "xio_driver.h"

#ifdef TCL
#include "tcl_vararg.h"
#endif /* TCL */

//----------------------------------------------------------------------------
// Support for file driver overloaded globus_xio_attr_cntl()
//----------------------------------------------------------------------------
static globus_result_t
file_driver_attr_cntl( globus_xio_attr_t Attr,
                       globus_xio_driver_t Driver,
                       driver_call_info_type* Parms )
{
  globus_result_t result;
  static char func_name[] = "file_driver_attr_cntl";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );
   
  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_FILE_SET_MODE:
      {
        int mode;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
#if 0	    
	    Tcl_AppendResult
	      ( ((TclInterpInfo_type*)(Parms->m_interp_info))->m_interp, 
		"Missing arguments",
		NULL );
	    
#endif	    
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &mode ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, mode );
      }
      break;

      
    case GLOBUS_XIO_FILE_GET_MODE:
      {
        int mode;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, &mode );
        
        AppendInt( Parms, mode );
      } 
      break;

      
    case GLOBUS_XIO_FILE_SET_FLAGS:
      {
        int flags;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &flags ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, flags );
      }
      break;

      
    case GLOBUS_XIO_FILE_GET_FLAGS:
      {
        int flags;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, &flags );

        AppendInt( Parms, flags );
      }
      break;

      
    case GLOBUS_XIO_FILE_SET_TRUNC_OFFSET:
      {
        globus_off_t offset;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        //:TODO: globus_off_t could be short, int, long long ???
        if ( GetVarArgAsInt( Parms, 0, (int*)&offset ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, offset );
      }
      break;

      
    case GLOBUS_XIO_FILE_GET_TRUNC_OFFSET:
      {
        globus_off_t offset;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &offset );

        AppendInt( Parms, offset );
      }
      break;

      
    case GLOBUS_XIO_FILE_SET_HANDLE:
      {
        globus_xio_system_handle_t handle;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &handle ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, handle );
      }
      break;

      
    case GLOBUS_XIO_FILE_GET_HANDLE:
      {
        globus_xio_system_handle_t handle;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, &handle );

        AppendInt( Parms, handle );
      }
      break;
      
    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return result;
}


//----------------------------------------------------------------------------
// Support for file driver overloaded globus_xio_handle_cntl()
//----------------------------------------------------------------------------
static globus_result_t
file_driver_handle_cntl( globus_xio_handle_t Handle,
                         globus_xio_driver_t Driver,
                         driver_call_info_type* Parms )
{
  globus_result_t result;
  static char func_name[] = "file_driver_handle_cntl";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );

  switch( Parms->m_cmd ) 
    {
    case GLOBUS_XIO_FILE_GET_HANDLE:
      {
        globus_xio_system_handle_t handle;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &handle );

        AppendInt( Parms, handle );
      }
      break;


    case GLOBUS_XIO_FILE_SEEK:
      {
        globus_off_t offset;
        unsigned int whence;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &whence ) == 0 )
          {
            //:TODO: Return some error status, currently just returning
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        if ( whence != GLOBUS_XIO_FILE_SEEK_SET ||
             whence != GLOBUS_XIO_FILE_SEEK_CUR ||
             whence != GLOBUS_XIO_FILE_SEEK_END )
          {
            //:TODO: Return some error status, currently just returning
            //:TODO: generic error
            return GLOBUS_FAILURE;            
          }
        
        result = globus_xio_handle_cntl ( Handle, Driver, Parms->m_cmd,
                                          &offset, whence );

        AppendInt( Parms, offset );
      }
      break;

    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return result;
}


//----------------------------------------------------------------------------
// This function gets called to initialize the entry for Tcl function map
// hash table
//----------------------------------------------------------------------------

XIODriverFunctionMap_type*
DriverFileInit( )
{
  static char func_name[] = "DriverFileInit";
  
  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  XIODriverFunctionMap_type* retval = 
    (XIODriverFunctionMap_type*)
    globus_malloc( sizeof( XIODriverFunctionMap_type ) );
  
  /* 
   * Provide overloaded function for globus_xio_attr_cntl() 
   */
  retval->m_attr_cntl.m_function_call =
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (file_driver_attr_cntl);
  
  /*
   * Provide overloaded function for globus_xio_handle_cntl() 
   */
  retval->m_handle_cntl.m_function_call =
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (file_driver_handle_cntl);
  
  /*
   * Currently, globus_xio_server_cntl() doesn't need to support 
   * file driver attributes. 
   */
  retval->m_server_cntl.m_function_call = NULL;
  
  /*
   * Currently, globus_xio_data_descriptor_cntl() doesn't need to 
   * support file driver attributes. 
   */
  retval->m_data_desc_cntl.m_function_call = NULL;

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return retval;
}
