//:TODO: Need to have this come from the compiler command line.
#define TCL

#include "globus_xio.h"
#include "globus_xio_gsi.h"

#include "xio_driver.h"

#include "common/sgdebug.h"

#ifdef TCL
#include "tcl_vararg.h"
#endif /* TCL */

//----------------------------------------------------------------------------
// Support for GSI driver - globus_xio_attr_cntl()
//----------------------------------------------------------------------------

static globus_result_t
gsi_driver_attr_cntl( globus_xio_attr_t Attr,
                      globus_xio_driver_t Driver,
                      driver_call_info_type* Parms )
{
  
  globus_result_t result;
  static char func_name[] = "gsi_driver_attr_cntl";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );
  
  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_GSI_SET_CREDENTIAL:
      {
        gss_cred_id_t credential;
        char *s;
        int length;

        if ( Parms->m_var_argc != 1 )
        {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
        }
        
        /* 
         * Check if we pass a string "NULL" literally from Tcl layer 
         */
        s = GetVarArgAsString( Parms, 0, &length );

        /* 
         * If it does, just use call the actual globus_xio_attr_cntl
         * with NULL as the default credential 
         */
        if ( strcmp(s, "NULL") == 0 )
          {
            SGDEBUG_LVL_FMT_1( 10, "credential from Tcl layer: %s", s );

            result = globus_xio_attr_cntl( Attr, 
                                           Driver, 
                                           Parms->m_cmd,
                                           NULL );
          }
        
        /*
         * Otherwise, we must get the true pointer information
         * created at the Tcl layer
         */
        else
          {
            SGDEBUG_LVL_FMT_1( 10, "Before GetVarArgAsPointer: credential: %ld", 
                           credential );
            
            GetVarArgAsPointer( Parms, 0, (gss_cred_id_t*)&credential );
            
            SGDEBUG_LVL_FMT_1( 10, "After GetVarArgAsPointer: credential: %ld", 
                           credential );
            
            result = globus_xio_attr_cntl( Attr, 
                                           Driver, 
                                           Parms->m_cmd,
                                           credential );
            
            SGDEBUG_LVL_FMT_1( 10, "After globus_xio_attr_cntl: result: %d", result );
          }
      }
      break;

      
    case GLOBUS_XIO_GSI_GET_CREDENTIAL:
      {
        gss_cred_id_t *credential;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        credential = (gss_cred_id_t*)globus_libc_malloc(sizeof(gss_cred_id_t));
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       credential );
        
        SGDEBUG_LVL_FMT_1( 10, "After globus_xio_attr_cntl: result: %d", result);
        
        if ( *credential == 0 )
          {
            SGDEBUG_LVL_FMT_1( 10, "Before AppendPointer: credential: %p", 
			       credential );
            SGDEBUG_LVL_FMT_1( 10, "After globus_xio_attr_cntl: credential: %d", 
			       *credential );
            AppendInt( Parms, 0 );
          }
        else
          {
            SGDEBUG_LVL_FMT_1( 10, "Before AppendPointer: credential: %p", 
			       credential );
            AppendPointer( Parms, (gss_cred_id_t *)&credential );
          }

        globus_libc_free(credential);
      }
      break;

    case GLOBUS_XIO_GSI_SET_TARGET_NAME:
      {
        gss_name_t target_name;

        char *s;
        int length;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        /* 
         * Check if we pass a string "NULL" literally from Tcl layer 
         */
        s = GetVarArgAsString( Parms, 0, &length);
        
        /* 
         * If it does, just use call the actual globus_xio_attr_cntl
         * with NULL as the default target name 
         */
        if ( strcmp( s, "NULL" ) == 0 )
          {
            SGDEBUG_LVL_FMT_1( 10, "target name from Tcl layer: %s", s );

            result = globus_xio_attr_cntl( Attr, 
                                           Driver,
                                           Parms->m_cmd, 
                                           NULL );
          }
        /*
         * Otherwise, we must get the true pointer information
         * created at the Tcl layer
         */
        else
          {
            GetVarArgAsPointer( Parms, 0, (gss_name_t*)&target_name );

            result = globus_xio_attr_cntl( Attr, 
                                           Driver, 
                                           Parms->m_cmd, 
                                           target_name );
          }
      }
      break;


      // :TODO: Must allocate space to hold the value
    case GLOBUS_XIO_GSI_GET_TARGET_NAME:
      {
        gss_name_t *target_name;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        target_name = (gss_name_t*)globus_malloc(sizeof(gss_name_t));
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       target_name );

        if ( *target_name == 0 )
          {
            AppendInt( Parms, 0 );
          }
        else
          {
            AppendPointer( Parms, target_name);
          }
        
        globus_free(target_name);
      }
      break;

      
    case GLOBUS_XIO_GSI_SET_GSSAPI_REQ_FLAGS:
      {
        OM_uint32 req_flags;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &req_flags ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            //return GLOBUS_FAILURE;
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       req_flags );
      }
      break;

      
    case GLOBUS_XIO_GSI_GET_GSSAPI_REQ_FLAGS:
      {
        OM_uint32 req_flags;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       &req_flags );

        AppendInt( Parms, req_flags );
      }
      break;

      
    case GLOBUS_XIO_GSI_SET_PROXY_MODE:
      {
        globus_xio_gsi_proxy_mode_t proxy_mode;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, (int *)&proxy_mode ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       proxy_mode );
      }
      break;

      
    case GLOBUS_XIO_GSI_GET_PROXY_MODE:
      {
        globus_xio_gsi_proxy_mode_t proxy_mode;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       &proxy_mode );

        AppendInt( Parms, proxy_mode );
      }
      break;

      
    case GLOBUS_XIO_GSI_SET_AUTHORIZATION_MODE:
      {
        globus_xio_gsi_authorization_mode_t authz_mode;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, (int *)&authz_mode ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       authz_mode );
      }
      break;

      
    case GLOBUS_XIO_GSI_GET_AUTHORIZATION_MODE:
      {
        globus_xio_gsi_authorization_mode_t authz_mode;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       &authz_mode );

        AppendInt( Parms, authz_mode );
      }
      break;

      
    case GLOBUS_XIO_GSI_SET_DELEGATION_MODE:
      {
        globus_xio_gsi_delegation_mode_t delegation_mode;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, (int *)&delegation_mode ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd, 
                                       delegation_mode );
      }
      break;

      
    case GLOBUS_XIO_GSI_GET_DELEGATION_MODE:
      {
        globus_xio_gsi_delegation_mode_t delegation_mode;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       &delegation_mode );

        AppendInt( Parms, delegation_mode );
      }      
      break;

      
    case GLOBUS_XIO_GSI_SET_SSL_COMPATIBLE:
      {
        globus_bool_t ssl_mode;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &ssl_mode ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd, 
                                       ssl_mode );
      }
      break;

      
    case GLOBUS_XIO_GSI_SET_ANON:
      {
        globus_bool_t anon_mode;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &anon_mode ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       anon_mode );
      }
      break;

      
    case GLOBUS_XIO_GSI_SET_WRAP_MODE:
      {
        globus_bool_t wrap_mode;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &wrap_mode ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       wrap_mode );
      }
      break;

      
    case GLOBUS_XIO_GSI_GET_WRAP_MODE:
      {
        globus_bool_t wrap_mode;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       &wrap_mode );

        AppendInt( Parms, wrap_mode );
      }
      break;

      
    case GLOBUS_XIO_GSI_SET_BUFFER_SIZE:
      {
        globus_size_t buffer_size;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &buffer_size ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       buffer_size );
      }
      break;


    case GLOBUS_XIO_GSI_GET_BUFFER_SIZE:
      {
        globus_size_t buffer_size;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       &buffer_size );

        AppendInt( Parms, buffer_size );

      }
      break;


    case GLOBUS_XIO_GSI_SET_PROTECTION_LEVEL:
      {
        globus_xio_gsi_protection_level_t protection_level;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, (int *)&protection_level ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd, 
                                       protection_level );
      }
      break;


    case GLOBUS_XIO_GSI_GET_PROTECTION_LEVEL:
      {
        globus_xio_gsi_protection_level_t protection_level;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd,
                                       &protection_level );

        AppendInt( Parms, protection_level );
      }
      break;



      
    case GLOBUS_XIO_GSI_FORCE_SERVER_MODE:
      {
        globus_bool_t server_mode;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &server_mode ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, 
                                       Driver, 
                                       Parms->m_cmd, 
                                       server_mode );
      }
      break;

      
    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return result;
}

//----------------------------------------------------------------------------
// Support for GSI driver - globus_xio_handle_cntl()
//----------------------------------------------------------------------------

static globus_result_t
gsi_driver_handle_cntl( globus_xio_handle_t Handle,
                        globus_xio_driver_t Driver,
                        driver_call_info_type* Parms )
{
  globus_result_t result;
  static char func_name[] = "gsi_driver_handle_cntl";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );
  
  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_GSI_SET_CREDENTIAL:
      {
        gss_cred_id_t credential;
        char *s;
        int length;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        /* 
         * Check if we pass a string "NULL" literally from Tcl layer 
         */
        s = GetVarArgAsString( Parms, 0, &length );

        /* 
         * If it does, just use call the actual globus_xio_attr_cntl
         * with NULL as the default credential 
         */
        if ( strcmp(s, "NULL") == 0 )
          {
            SGDEBUG_LVL_FMT_1( 10, "credential from Tcl layer: %s", s );

            result = globus_xio_handle_cntl( Handle, 
                                             Driver, 
                                             Parms->m_cmd,
                                             NULL );
          }
        /*
         * Otherwise, we must get the true pointer information
         * created at the Tcl layer
         */
        else
          {
            SGDEBUG_LVL_FMT_1( 10, "Before GetVarArgAsPointer: credential: %ld", 
                           credential );
            
            GetVarArgAsPointer( Parms, 0, (gss_cred_id_t*)&credential );
            
            SGDEBUG_LVL_FMT_1( 10, "After GetVarArgAsPointer: credential: %ld", 
                           credential );

            result = globus_xio_handle_cntl( Handle, 
                                             Driver, 
                                             Parms->m_cmd,
                                             credential );
            
            SGDEBUG_LVL_FMT_1( 10, "After globus_xio_attr_cntl: result: %d", result );
          }
      }
      break;


    case GLOBUS_XIO_GSI_GET_CREDENTIAL:
      {
        gss_cred_id_t *credential;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        credential = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
        
        result = globus_xio_handle_cntl( Handle, 
                                         Driver, 
                                         Parms->m_cmd,
                                         credential );

        SGDEBUG_LVL_FMT_1( 10, "After globus_xio_attr_cntl: result: %d", result);

        if ( *credential == 0 )
          {
            SGDEBUG_LVL_FMT_1( 10, "Before AppendPointer: credential: %p", 
                           credential );
            SGDEBUG_LVL_FMT_1( 10, "After globus_xio_attr_cntl: credential: %d", 
                           *credential );
            AppendInt( Parms, 0 );
          }
        else
          {
            SGDEBUG_LVL_FMT_1( 10, "Before AppendPointer: credential: %p", 
                           credential );
            AppendPointer( Parms, (gss_cred_id_t *)&credential );
          }

        globus_free(credential);
      }
      break;


    case GLOBUS_XIO_GSI_GET_CONTEXT:
      {
        gss_ctx_id_t* context;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        context = (gss_ctx_id_t*)globus_malloc(sizeof(gss_ctx_id_t));

        result = globus_xio_handle_cntl( Handle, 
                                         Driver, 
                                         Parms->m_cmd,
                                         context );
        
        if ( *context == 0 )
          {
            AppendInt( Parms, 0 );
          }
        else
          {
            AppendPointer( Parms, (gss_ctx_id_t*)&context );
          }
        globus_free(context);
      }
      break;


    case GLOBUS_XIO_GSI_GET_DELEGATED_CRED:
      {
        gss_cred_id_t* credential;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        credential = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
        
        result = globus_xio_handle_cntl( Handle, 
                                         Driver, 
                                         Parms->m_cmd,
                                         credential );

        if ( *credential == 0 )
          {
            AppendInt( Parms, 0 );
          }
        else
          {
            AppendPointer( Parms, (gss_cred_id_t*)&credential );
          }

        globus_free(credential);
      }
      break;

            
    case GLOBUS_XIO_GSI_GET_PEER_NAME:
      {
        gss_name_t *peer_name;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        peer_name = (gss_name_t*)globus_malloc(sizeof(gss_name_t));
        
        result = globus_xio_handle_cntl( Handle, 
                                         Driver, 
                                         Parms->m_cmd,
                                         peer_name );
        
        if ( *peer_name == 0 )
          {
            AppendInt( Parms, 0 );
          }
        else
          {
            AppendPointer( Parms, (gss_name_t*)&peer_name );
          }

        globus_free(peer_name);
      }
      break;


    case GLOBUS_XIO_GSI_GET_LOCAL_NAME:
      {
        gss_name_t *local_name;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        local_name = (gss_name_t*)globus_malloc(sizeof(gss_name_t));
        
        result = globus_xio_handle_cntl( Handle, 
                                         Driver, 
                                         Parms->m_cmd,
                                         local_name );

        if ( *local_name == 0 )
          {
            AppendInt( Parms, 0 );
          }
        else
          {
            AppendPointer( Parms, (gss_name_t*)&local_name );
          }

        globus_free(local_name);
      }
      break;


    case GLOBUS_XIO_GSI_INIT_DELEGATION:
      {
        gss_cred_id_t credential;
        gss_OID_set restriction_oids;
        gss_buffer_set_t restriction_buffers;
        OM_uint32 time_req;
        char *s;
        int length;

        if ( Parms->m_var_argc != 4 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        /* 
         *Check if we pass a string "NULL" literally from Tcl layer 
         */
        s = GetVarArgAsString( Parms, 0, &length );

        if ( strcmp(s, "NULL") == 0 )
          {
            credential = NULL;
          }
        else
          {
            // Get credential from argument
            GetVarArgAsPointer( Parms, 0, &credential );
          }

        s = GetVarArgAsString( Parms, 1, &length );
        
        if ( strcmp(s, "NULL") == 0 )
          {
            restriction_oids = NULL;
          }
        else
          {
            // Get restriction_oids
            GetVarArgAsPointer( Parms, 1, &restriction_oids );
          }

        s = GetVarArgAsString( Parms, 2, &length );
        
        if ( strcmp(s, "NULL") == 0 )
          {
            restriction_buffers = NULL;
          }
        else
          {
            // Get restriction_buffers
            GetVarArgAsPointer( Parms, 2, &restriction_buffers );
          }

        // Get time_req
        if ( GetVarArgAsInt( Parms, 3, &time_req ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            result = GLOBUS_FAILURE;
          }

        result = globus_xio_handle_cntl( Handle, 
                                         Driver, 
                                         Parms->m_cmd, 
                                         credential,
                                         restriction_oids,
                                         restriction_buffers,
                                         time_req );
      }
      break;

      
    case GLOBUS_XIO_GSI_ACCEPT_DELEGATION:
      {
        gss_cred_id_t *credential;
        gss_OID_set restriction_oids;
        gss_buffer_set_t restriction_buffers;
        OM_uint32 time_req;
        char *s;
        int length;

        if ( Parms->m_var_argc != 4 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        credential = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));

        if ( strcmp(s, "NULL") == 0 )
          {
            restriction_oids = NULL;
          }
        else
          {
            // Get restriction_oids
            GetVarArgAsPointer( Parms, 1, &restriction_oids );
          }

        s = GetVarArgAsString( Parms, 2, &length );
        
        if ( strcmp(s, "NULL") == 0 )
          {
            restriction_buffers = NULL;
          }
        else
          {
            // Get restriction_buffers
            GetVarArgAsPointer( Parms, 2, &restriction_buffers );
          }

        // Get time_req
        if ( GetVarArgAsInt( Parms, 3, &time_req ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            result = GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, 
                                         Driver, 
                                         Parms->m_cmd,
                                         credential,
                                         restriction_oids,
                                         restriction_buffers,
                                         time_req );

        if ( *credential == 0 )
          {
            AppendInt( Parms, 0 );
          }
        else
          {
            AppendPointer( Parms, (gss_cred_id_t *)&credential );
          }

        globus_free(credential);
      }
      break;
      
#if WORKING
    case GLOBUS_XIO_GSI_REGISTER_INIT_DELEGATION:
      {
        gss_cred_id_t credential;
        gss_OID_set restriction_oids;
        gss_buffer_set_t restriction_buffers;
        OM_uint32 time_req;

        if ( Parms->m_var_argc != 6 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        // Get credential from argument
        GetVarArgAsPointer( Parms, 0, &credential );
        
        // Get restriction_oids
        GetVarArgAsPointer( Parms, 1, &restriction_oids );
        
        // Get restriction_buffers
        GetVarArgAsPointer( Parms, 2, &restriction_buffers );
        
        // Get time_req
        if ( GetVarArgAsInt( Parms, 3, &time_req ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }


        // Get callback function


        // Get callback arguments

        result = globus_xio_handle_cntl( Handle, 
                                         Driver, 
                                         Parms->m_cmd, 
                                         credential,
                                         restriction_oids,
                                         restriction_buffers,
                                         time_req );
      }
      break;

    case GLOBUS_XIO_GSI_REGISTER_ACCEPT_DELEGATION:
      {
        gss_cred_id_t *credential;
        gss_OID_set restriction_oids;
        gss_buffer_set_t restriction_buffers;
        OM_uint32 time_req;

        if ( Parms->m_var_argc != 6 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        credential = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));

        result = globus_xio_handle_cntl( Handle, 
                                         Driver, 
                                         Parms->m_cmd,
                                         &credential,
                                         restriction_oids,
                                         restriction_buffers,
                                         time_req );

        AppendPointer( Parms, (gss_cred_id_t *)&credential );

        AppendPointer( Parms, restriction_oids );

        AppendPointer( Parms, restriction_buffers );

        AppendInt( Parms, time_req );

        // Append callback function

        
        // Append callback arguments
        
        globus_free(credential);
      }
      break;
#endif /* 0 */
            
    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return result;
}

//----------------------------------------------------------------------------
// This function gets called to initialize the entry for Tcl function map
// hash table
//----------------------------------------------------------------------------

XIODriverFunctionMap_type*
DriverGSIInit( )
{
  static char func_name[] = "DriverGSIInit";
  
  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  XIODriverFunctionMap_type* retval = 
    (XIODriverFunctionMap_type*)
    globus_malloc( sizeof( XIODriverFunctionMap_type ) );

  // :TODO: Fix compiler warning for the next 4 statements
  // :TODO: => assignment from incompatible pointer type

  /* 
   * Provide overloaded function for globus_xio_attr_cntl() 
   */
  retval->m_attr_cntl.m_function_call = 
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (gsi_driver_attr_cntl);

  /*
   * Provide overloaded function for globus_xio_handle_cntl() 
   */
  retval->m_handle_cntl.m_function_call =
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (gsi_driver_handle_cntl);

  /*
   * Currently, globus_xio_server_cntl() doesn't need to support 
   * file driver. 
   */
  retval->m_server_cntl.m_function_call = NULL;
  
  /*
   * Currently, globus_xio_data_descriptor_cntl() doesn't need to 
   * support file driver. 
   */
  retval->m_data_desc_cntl.m_function_call = NULL;

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return retval;
}
