/* :TODO: Need to have this come from the compiler command line. */
#define TCL

#include "globus_xio.h"
#include "globus_xio_http.h"

#include "common/sgdebug.h"

#include "xio_driver.h"

#ifdef TCL
#include "tcl_vararg.h"
#endif /* TCL */

/*---------------------------------------------------------------------------
 * Support for HTTP driver
 *---------------------------------------------------------------------------*/
static globus_result_t
http_driver_attr_cntl( globus_xio_attr_t Attr,
                       globus_xio_driver_t Driver,
                       driver_call_info_type* Parms )
{
  globus_result_t result;
  static char func_name[] = "http_driver_handle_cntl";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );
  
  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_HTTP_ATTR_SET_REQUEST_METHOD:
      {
        char *method;
        int length;
        
        if ( Parms->m_var_argc != 1 )
          {
            /* :TODO: inform user of error condition */
	    return GLOBUS_FAILURE;
          }
        
        method = GetVarArgAsString( Parms, 0, &length);

        if ( method == NULL )
          {
            /* :TODO: Return some error status, currently just returning
	     * :TODO:   generic error
	     */
            return GLOBUS_FAILURE;
          }

	result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, method );
      }
      break;


    case GLOBUS_XIO_HTTP_ATTR_SET_REQUEST_HTTP_VERSION:
    {
      unsigned int version;

      if ( Parms->m_var_argc != 1)
        {
	  /* :TODO: inform user of error condition */
	  return GLOBUS_FAILURE;
        }

        if ( GetVarArgAsInt( Parms, 0, &version ) == 0 )
          {
            /* :TODO: Return some error status, currently just returning
	     * :TODO:   generic error
	     */
            return GLOBUS_FAILURE;
          }

	result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, version );
      }
      break;


    case GLOBUS_XIO_HTTP_ATTR_SET_REQUEST_HEADER:
     {
       char *header_name;
       char *header_value;
       int length;

       if ( Parms->m_var_argc != 2 )
         {
           /* :TODO: inform user of error condition */
	   return GLOBUS_FAILURE;
         }
       
       header_name = GetVarArgAsString( Parms, 0, &length);

       if ( header_name == NULL )
         {
           /* :TODO: Return some error status, currently just returning
	    * :TODO:    generic error
	    */
           return GLOBUS_FAILURE;
         }

       header_value = GetVarArgAsString( Parms, 1, &length);
       
       if ( header_value == NULL )
         {
           /* :TODO: Return some error status, currently just returning
	    * :TODO:   generic error
	    */
           return GLOBUS_FAILURE;
         }
       
       result =  globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
				       header_name, header_value );
     }
     break;

#if HAVE_GLOBUS_XIO_HTTP_ATTR_SET_REQUEST_CALLBACK
    case GLOBUS_XIO_HTTP_ATTR_SET_REQUEST_CALLBACK:
      {
        /* :TODO: handle callback types */
      }
      break;
#endif /* HAVE_GLOBUS_XIO_HTTP_ATTR_SET_REQUEST_CALLBACK */
    
    
#if HAVE_GLOBUS_XIO_HTTP_ATTR_SET_RESPONSE_CALLBACK
    case GLOBUS_XIO_HTTP_ATTR_SET_RESPONSE_CALLBACK:
      {
        /* :TODO: handle callback types */
      }
      break;
#endif /* GLOBUS_XIO_HTTP_ATTR_SET_RESPONSE_CALLBACK */

    default:
      /* :TODO: inform user of error condition */
      result = GLOBUS_FAILURE;
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );

  return result;
}


/*---------------------------------------------------------------------------
 * 
 *---------------------------------------------------------------------------*/
static globus_result_t
http_driver_handle_cntl( globus_xio_handle_t Handle,
                         globus_xio_driver_t Driver,
                         driver_call_info_type* Parms )
{
  globus_result_t result;
  static char func_name[] = "http_driver_handle_cntl";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );

  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_HTTP_HANDLE_SET_RESPONSE_HEADER:
     {
       char *header_name;
       char *header_value;
       int length;

       if ( Parms->m_var_argc != 2 )
          {
            /* :TODO: inform user of error condition */
	    return GLOBUS_FAILURE;
          }

       header_name = GetVarArgAsString( Parms, 0, &length);
       
       if ( header_name == NULL )
         {
           /* :TODO: Return some error status, currently just returning
	    * :TODO:   generic error
	    */
           return GLOBUS_FAILURE;
         }
       
       header_value = GetVarArgAsString( Parms, 1, &length);
       
       if ( header_value == NULL )
         {
           /* :TODO: Return some error status, currently just returning
	    * :TODO:   generic error
	    */
           return GLOBUS_FAILURE;
         }
       
       result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
					header_name, header_value );
     }
     break;


    case GLOBUS_XIO_HTTP_HANDLE_SET_RESPONSE_STATUS_CODE:
    {
      unsigned int status;

      if ( Parms->m_var_argc != 1)
        {
          /* :TODO: inform user of error condition */
	  return GLOBUS_FAILURE;
        }

      if ( GetVarArgAsInt( Parms, 0, &status ) == 0 )
        {
          /* :TODO: Return some error status, currently just returning
	   * :TODO:   generic error
	   */
          return GLOBUS_FAILURE;
        }

      result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
				       status );
      }
      break;

      
    case GLOBUS_XIO_HTTP_HANDLE_SET_RESPONSE_REASON_PHRASE:
     {
       char *reason;
       int length;

       if ( Parms->m_var_argc != 1 )
         {
           /* :TODO: inform user of error condition */
	   return GLOBUS_FAILURE;
         }

       reason = GetVarArgAsString( Parms, 0, &length);

       if ( reason == NULL )
         {
           /* :TODO: Return some error status, currently just returning
	    * :TODO:   generic error
	    */
           return GLOBUS_FAILURE;
         }

       result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd, reason );
     }
     break;


    case GLOBUS_XIO_HTTP_HANDLE_SET_RESPONSE_HTTP_VERSION:
    {
      unsigned int version;

      if ( Parms->m_var_argc != 1)
        {
          /* :TODO: inform user of error condition */
	  return GLOBUS_FAILURE;
        }

      if ( GetVarArgAsInt( Parms, 0, &version ) == 0 )
        {
          /* :TODO: Return some error status, currently just returning
	   * :TODO:   generic error
	   */
          return GLOBUS_FAILURE;
        }
      
      result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd, version );
    }
    break;

    
    case GLOBUS_XIO_HTTP_HANDLE_SET_END_OF_ENTITY:
      {
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd );
      }
      break;

      
    default:
      /* :TODO: inform user of error condition */
      result = GLOBUS_FAILURE;
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return result;
}

/*---------------------------------------------------------------------------
 *
 *---------------------------------------------------------------------------*/
XIODriverFunctionMap_type*
DriverHTTPInit( )
{
  XIODriverFunctionMap_type* retval =
    (XIODriverFunctionMap_type*)
    globus_malloc( sizeof( XIODriverFunctionMap_type ) );

  retval->m_attr_cntl.m_function_call =
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (http_driver_attr_cntl);

  retval->m_handle_cntl.m_function_call =
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (http_driver_handle_cntl);

  retval->m_server_cntl.m_function_call = NULL;

  retval->m_data_desc_cntl.m_function_call = NULL;
  
  return retval;
}


