//:TODO: Need to have this come from the compiler command line.
#define TCL

#include "globus_xio.h"
#include "globus_xio_tcp_driver.h"

#include "common/sgdebug.h"

#include "xio_driver.h"

#ifdef TCL
#include "tcl_vararg.h"
#endif /* TCL */

//----------------------------------------------------------------------------
// Support for TCP driver
//----------------------------------------------------------------------------

static globus_result_t
tcp_driver_attr_cntl( globus_xio_attr_t Attr,
                      globus_xio_driver_t Driver,
                      driver_call_info_type* Parms )
{
  globus_result_t result;
  static char func_name[] = "tcp_driver_attr_cntl";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );
  
  switch( Parms->m_cmd )
    {      
    case GLOBUS_XIO_TCP_SET_SERVICE:
      {
        char *service_name;
        int length;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        service_name = GetVarArgAsString( Parms, 0, &length);

        if ( service_name == NULL )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       service_name );
      }
      break;

      
    case GLOBUS_XIO_TCP_GET_SERVICE:
      {
        char *service_name;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        service_name = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(service_name, '\0', MAX_NAME);
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &service_name );

        AppendString( Parms, service_name );

        globus_free(service_name);
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_PORT:
      {
        int listener_port;
        
        if ( Parms->m_var_argc != 1)
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &listener_port ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       listener_port );
      }
      break;

      
    case GLOBUS_XIO_TCP_GET_PORT:
      {
        int listener_port;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &listener_port );
        
        AppendInt( Parms, listener_port );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_BACKLOG:
      {
        int listener_backlog;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &listener_backlog ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       listener_backlog );
      }
      break;


    case GLOBUS_XIO_TCP_GET_BACKLOG:
      {
        int listener_backlog;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &listener_backlog );
        
        AppendInt( Parms, listener_backlog );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_LISTEN_RANGE:
      {
        int min_port, max_port;

        if ( Parms->m_var_argc != 2 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &min_port ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 1, &max_port ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       min_port, max_port );
      }
      break;


    case GLOBUS_XIO_TCP_GET_LISTEN_RANGE:
      {
        int min_port, max_port;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &min_port, &max_port );
        
        AppendInt( Parms, min_port );
        
        AppendInt( Parms, max_port );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_HANDLE:
      {
        globus_xio_system_handle_t handle;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &handle ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, handle );
      }
      break;


    case GLOBUS_XIO_TCP_GET_HANDLE:
      {
        globus_xio_system_handle_t handle;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &handle );

        AppendInt( Parms, handle );
      }
      break;

    case GLOBUS_XIO_TCP_SET_INTERFACE:
    {
        char* interface;
        int length;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        interface = GetVarArgAsString( Parms, 0, &length );

        if ( interface == NULL )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       interface );
      }
      break;


    case GLOBUS_XIO_TCP_GET_INTERFACE:
      {
        char* interface;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        interface = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(interface, '\0', MAX_NAME);
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &interface );

        AppendString( Parms, interface );

        globus_free(interface);
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_RESTRICT_PORT:
      {
        globus_bool_t restrict_port;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &restrict_port ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       restrict_port );
      }
      break;


    case GLOBUS_XIO_TCP_GET_RESTRICT_PORT:
      {
        globus_bool_t restrict_port;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &restrict_port );

        AppendInt( Parms, restrict_port );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_REUSEADDR:
      {
        globus_bool_t reuseaddr;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &reuseaddr ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       reuseaddr );
      }
      break;


    case GLOBUS_XIO_TCP_GET_REUSEADDR:
      {
        globus_bool_t reuseaddr;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &reuseaddr );

        AppendInt( Parms, reuseaddr );
      }
      break;
      
      
    case GLOBUS_XIO_TCP_SET_NO_IPV6:
      {
        globus_bool_t no_ipv6;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &no_ipv6 ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       no_ipv6 );
      }
      break;


    case GLOBUS_XIO_TCP_GET_NO_IPV6:
      {
        globus_bool_t no_ipv6;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &no_ipv6 );

        AppendInt( Parms, no_ipv6 );
      }
      break;
      
      
    case GLOBUS_XIO_TCP_SET_CONNECT_RANGE:
      {
        int connector_min_port, connector_max_port;

        if ( Parms->m_var_argc != 2 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &connector_min_port ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 1, &connector_max_port ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       connector_min_port,
                                       connector_max_port );
      }
      break;


    case GLOBUS_XIO_TCP_GET_CONNECT_RANGE:
      {
        int connector_min_port, connector_max_port;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &connector_min_port,
                                       &connector_max_port );
        
        AppendInt( Parms, connector_min_port );
        AppendInt( Parms, connector_max_port );
      }
      break;
      
      
    case GLOBUS_XIO_TCP_SET_KEEPALIVE:
      {
        globus_bool_t keepalive;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &keepalive ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       keepalive );
      }
      break;


    case GLOBUS_XIO_TCP_GET_KEEPALIVE:
      {
        globus_bool_t keepalive;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &keepalive );

        AppendInt( Parms, keepalive );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_LINGER:
      {
        globus_bool_t linger;
        int linger_time;
        
        if ( Parms->m_var_argc != 2 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &linger ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 1, &linger_time ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       linger, linger_time );
      }
      break;


    case GLOBUS_XIO_TCP_GET_LINGER:
      {
        globus_bool_t linger;
        int linger_time;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &linger, &linger_time );

        AppendInt( Parms, linger );
        AppendInt( Parms, linger_time );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_OOBINLINE:
      {
        globus_bool_t handle;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &handle ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, handle );
      }
      break;


    case GLOBUS_XIO_TCP_GET_OOBINLINE:
      {
        globus_bool_t oobinline;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &oobinline );

        AppendInt( Parms, oobinline );
      }
      break;
      
      
    case GLOBUS_XIO_TCP_SET_SNDBUF:
      {
        int sndbuf;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &sndbuf ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, sndbuf );
      }
      break;


    case GLOBUS_XIO_TCP_GET_SNDBUF:
      {
        int sndbuf;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &sndbuf );

        AppendInt( Parms, sndbuf );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_RCVBUF:
      {
        int rcvbuf;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &rcvbuf ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, rcvbuf );
      }
      break;


    case GLOBUS_XIO_TCP_GET_RCVBUF:
      {
        int rcvbuf;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &rcvbuf );

        AppendInt( Parms, rcvbuf );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_NODELAY:
      {
        globus_bool_t nodelay;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &nodelay ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       nodelay );
      }
      break;


    case GLOBUS_XIO_TCP_GET_NODELAY:
      {
        globus_bool_t nodelay;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &nodelay );

        AppendInt( Parms, nodelay );
      }
      break;
      

#ifdef GLOBUS_XIO_TCP_AFFECT_ATTR_DEFAULTS
    case GLOBUS_XIO_TCP_AFFECT_ATTR_DEFAULTS:
      {
        globus_bool_t affect_global;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &affect_global ) == 0 )
          {
            //:TODO: Return some error status, currently just returning 
            //:TODO: generic error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_data_descriptor_cntl( Attr, Driver, Parms->m_cmd,
                                                  affect_global );
      }
      break;
#endif /* GLOBUS_XIO_TCP_AFFECT_ATTR_DEFAULTS */


    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return result;
}


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
static globus_result_t
tcp_driver_handle_cntl( globus_xio_handle_t Handle,
                        globus_xio_driver_t Driver,
                        driver_call_info_type* Parms )
{
  globus_result_t result;
  static char function[] = "tcp_driver_handle_cntl";
  
  SGDEBUG_LVL_FMT_1( 10, "Entering %s", function );
  
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );

  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_TCP_GET_HANDLE:
      {
        globus_xio_system_handle_t handle;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &handle );
        
        AppendInt( Parms, handle );
      }
      break;
      
      
    case GLOBUS_XIO_TCP_SET_KEEPALIVE:
      {
        globus_bool_t keepalive;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &keepalive ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

	SGDEBUG_LVL_FMT( 10, "Prior to calling globus_xio_handle_cntl()" );
	
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         keepalive );
	SGDEBUG_LVL_FMT_1( 10, "result: %d", result );
	
      }
      break;


    case GLOBUS_XIO_TCP_GET_KEEPALIVE:
      {
        globus_bool_t keepalive;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &keepalive );
        
        AppendInt( Parms, keepalive );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_LINGER:
      {
        globus_bool_t linger;
        int linger_time;
        
        if ( Parms->m_var_argc != 2 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &linger ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 1, &linger_time ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         linger, linger_time );
      }
      break;


    case GLOBUS_XIO_TCP_GET_LINGER:
      {
        globus_bool_t linger;
        int linger_time;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &linger, &linger_time );

        AppendInt( Parms, linger );
        AppendInt( Parms, linger_time );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_OOBINLINE:
      {
        globus_bool_t oobinline;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &oobinline ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         oobinline );
      }
      break;


    case GLOBUS_XIO_TCP_GET_OOBINLINE:
      {
        globus_bool_t oobinline;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &oobinline );

        AppendInt( Parms, oobinline );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_SNDBUF:
      {
        int sndbuf;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &sndbuf ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         sndbuf );
      }
      break;


    case GLOBUS_XIO_TCP_GET_SNDBUF:
      {
        int sndbuf;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &sndbuf );

        AppendInt( Parms, sndbuf );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_RCVBUF:
      {
        int rcvbuf;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &rcvbuf ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         rcvbuf );
      }
      break;


    case GLOBUS_XIO_TCP_GET_RCVBUF:
      {
        int rcvbuf;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &rcvbuf );

        AppendInt( Parms, rcvbuf );
      }
      break;

      
    case GLOBUS_XIO_TCP_SET_NODELAY:
      {
        globus_bool_t nodelay;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &nodelay ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         nodelay );
      }
      break;


    case GLOBUS_XIO_TCP_GET_NODELAY:
      {
        globus_bool_t nodelay;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &nodelay );

        AppendInt( Parms, nodelay );
      }
      break;

      
    case GLOBUS_XIO_TCP_GET_LOCAL_CONTACT:
      {
        char *contact_string;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        contact_string = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(contact_string, '\0', MAX_NAME);
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &contact_string );

        AppendString( Parms, contact_string );

        globus_free(contact_string);
      }
      break;


    case GLOBUS_XIO_TCP_GET_LOCAL_NUMERIC_CONTACT:
      {
        char *contact_string;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        contact_string = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(contact_string, '\0', MAX_NAME);
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &contact_string );

        AppendString( Parms, contact_string );

        globus_free(contact_string);
      }
      break;


    case GLOBUS_XIO_TCP_GET_REMOTE_CONTACT:
      {
        char *contact_string;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        contact_string = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(contact_string, '\0', MAX_NAME);
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &contact_string );

        AppendString( Parms, contact_string );

        globus_free(contact_string);
      }
      break;


    case GLOBUS_XIO_TCP_GET_REMOTE_NUMERIC_CONTACT:
      {
        char *contact_string;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        contact_string = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(contact_string, '\0', MAX_NAME);
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &contact_string );

        AppendString( Parms, contact_string );

        globus_free(contact_string);
      }
      break;

      
    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", function );
  
  return result;
}


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
static globus_result_t
tcp_driver_data_descriptor_cntl( globus_xio_data_descriptor_t Data_Descriptor,
                                 globus_xio_driver_t Driver,
                                 driver_call_info_type* Parms )
{
  globus_result_t result;
  static char func_name[] = "tcp_driver_data_descriptor_cntl";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );
  
  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_TCP_SET_SEND_FLAGS:
      {
        int send_flags;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &send_flags ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_data_descriptor_cntl( Data_Descriptor, Driver,
                                                  Parms->m_cmd, send_flags );
      }
      break;

      
    case GLOBUS_XIO_TCP_GET_SEND_FLAGS:
      {
        int send_flags;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_data_descriptor_cntl( Data_Descriptor, Driver,
                                                  Parms->m_cmd, &send_flags );
        
        AppendInt( Parms, send_flags );

        return result;
      }
      break;

      
    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return result;
}


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
static globus_result_t
tcp_driver_server_cntl( globus_xio_server_t Server,
                        globus_xio_driver_t Driver,
                        driver_call_info_type* Parms )
{
  globus_result_t result;
  static char func_name[] = "tcp_driver_server_cntl";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );
  
  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_TCP_GET_HANDLE:
      {
        globus_xio_system_handle_t handle;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        result = globus_xio_server_cntl( Server, Driver,
                                         Parms->m_cmd, &handle );

        AppendInt( Parms, handle );

      }
      break;

      
    case GLOBUS_XIO_TCP_GET_LOCAL_CONTACT:
      {
        char *contact_string;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        contact_string = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(contact_string, '\0', MAX_NAME);
        
        result = globus_xio_server_cntl( Server, Driver, Parms->m_cmd,
                                         &contact_string );

        AppendString( Parms, contact_string );

        globus_free(contact_string);
      }
      break;


    case GLOBUS_XIO_TCP_GET_LOCAL_NUMERIC_CONTACT:
      {
        char *contact_string;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
            return GLOBUS_FAILURE;
          }

        contact_string = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(contact_string, '\0', MAX_NAME);
        
        result = globus_xio_server_cntl( Server, Driver, Parms->m_cmd,
                                         &contact_string );

        AppendString( Parms, contact_string );

        globus_free(contact_string);
      }
      break;

      
    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return result;
}


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
XIODriverFunctionMap_type*
DriverTCPInit( )
{
  static char func_name[] = "DriverTCPInit";
  
  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );
  
  XIODriverFunctionMap_type* retval = 
    (XIODriverFunctionMap_type*)
    globus_malloc( sizeof( XIODriverFunctionMap_type ) );

  retval->m_attr_cntl.m_function_call = 
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (tcp_driver_attr_cntl);

  retval->m_handle_cntl.m_function_call =
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (tcp_driver_handle_cntl);

  retval->m_server_cntl.m_function_call =
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (tcp_driver_server_cntl);

  retval->m_data_desc_cntl.m_function_call =
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (tcp_driver_data_descriptor_cntl);

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return retval;
}
