//:TODO: Need to have this come from the compiler command line.
#define TCL

#include "globus_xio.h"
#include "globus_xio_udp_driver.h"

#include "xio_driver.h"

#include "common/sgdebug.h"

#ifdef TCL
#include "tcl_vararg.h"
#endif /* TCL */

//----------------------------------------------------------------------------
// Support for UDP driver
//----------------------------------------------------------------------------
static globus_result_t
udp_driver_attr_cntl( globus_xio_attr_t Attr,
                      globus_xio_driver_t Driver,
                      driver_call_info_type* Parms )
{
  globus_result_t result;
  static char func_name[] = "udp_driver_attr_cntl";

  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  SGDEBUG_LVL_FMT_1( 10, "Parms->m_var_argc: %d", Parms->m_var_argc );
  SGDEBUG_LVL_FMT_1( 10, "Parms->m_cmd: %d", Parms->m_cmd );
  
  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_UDP_SET_HANDLE:
      {
        globus_xio_system_handle_t handle;
        
        if ( Parms->m_var_argc != 1)
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &handle ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, handle );
      }
      break;


    case GLOBUS_XIO_UDP_GET_HANDLE:
      {
        globus_xio_system_handle_t handle;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &handle );

        AppendInt( Parms, handle );
      }
      break;
    
      
    case GLOBUS_XIO_UDP_SET_SERVICE:
      {
        char *service_name;
        int length;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        service_name = GetVarArgAsString( Parms, 0, &length);
        
        if ( service_name == NULL)
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       service_name );
      }
      break;


    case GLOBUS_XIO_UDP_GET_SERVICE:
      {
        char *service_name;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        service_name = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(service_name, '\0', MAX_NAME);
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &service_name );

        AppendString( Parms, service_name );

        globus_free(service_name);
      }
      break;

      
    case GLOBUS_XIO_UDP_SET_PORT:
    {
      int listener_port;
        
      if ( Parms->m_var_argc != 1)
        {
	  //:TODO: inform user of error condition
	  return GLOBUS_FAILURE;
        }

        if ( GetVarArgAsInt( Parms, 0, &listener_port ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
	            
	SGDEBUG_LVL_FMT_1( 10, "GLOBUS_XIO_UDP_SET_PORT: listener_port: %d", 
			   listener_port );
	
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       listener_port );
      }
      break;


    case GLOBUS_XIO_UDP_GET_PORT:
      {
        int listener_port;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &listener_port );

	SGDEBUG_LVL_FMT_1( 10, "GLOBUS_XIO_UDP_GET_PORT: listener_port: %d", 
			   listener_port );

        AppendInt( Parms, listener_port );
      }
      break;
      
      
    case GLOBUS_XIO_UDP_SET_LISTEN_RANGE:
      {
        unsigned int listener_min_port;
        unsigned int listener_max_port;
        
        if ( Parms->m_var_argc != 2 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &listener_min_port ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 1, &listener_max_port ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       listener_min_port,
                                       listener_max_port );
      }

      break;


    case GLOBUS_XIO_UDP_GET_LISTEN_RANGE:
      {
        int min_port, max_port;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &min_port, &max_port );
        
        AppendInt( Parms, min_port );
        AppendInt( Parms, max_port );
      }
      break;

      
    case GLOBUS_XIO_UDP_SET_INTERFACE:
      {
        char *interface;
        int length;
        
        if ( Parms->m_var_argc != 1 )
          {
	    //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        interface = GetVarArgAsString( Parms, 0, &length);
        
        if ( interface == NULL)
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       interface );
      }
    break;


    case GLOBUS_XIO_UDP_GET_INTERFACE:
      {
        char* interface;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        interface = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(interface, '\0', MAX_NAME);
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &interface );

        AppendString( Parms, interface );

        globus_free(interface);
      }
      break;

      
    case GLOBUS_XIO_UDP_SET_RESTRICT_PORT:
      {
        globus_bool_t restrict_port;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &restrict_port ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       restrict_port );
      }
      break;


    case GLOBUS_XIO_UDP_GET_RESTRICT_PORT:
      {
        globus_bool_t restrict_port;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &restrict_port );

        AppendInt( Parms, restrict_port );
      }
      break;

      
    case GLOBUS_XIO_UDP_SET_REUSEADDR:
      {
        globus_bool_t reuse_addr;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &reuse_addr ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       reuse_addr );
      }
      break;


    case GLOBUS_XIO_UDP_GET_REUSEADDR:
      {
        globus_bool_t reuseaddr;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &reuseaddr );

        AppendInt( Parms, reuseaddr );
      }
      break;

      
    case GLOBUS_XIO_UDP_SET_NO_IPV6:
      {
        globus_bool_t no_ipv6;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &no_ipv6 ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, no_ipv6 );
      }
      break;


    case GLOBUS_XIO_UDP_GET_NO_IPV6:
      {
        globus_bool_t no_ipv6;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &no_ipv6 );

        AppendInt( Parms, no_ipv6 );
      }
      break;

      
    case GLOBUS_XIO_UDP_SET_SNDBUF:
      {
        int sndbuf;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &sndbuf ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, sndbuf );
      }
      break;


    case GLOBUS_XIO_UDP_GET_SNDBUF:
      {
        int sndbuf;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &sndbuf );

        AppendInt( Parms, sndbuf );
      }
      break;

      
    case GLOBUS_XIO_UDP_SET_RCVBUF:
      {
        int rcvbuf;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &rcvbuf ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd, rcvbuf );
      }
      break;


    case GLOBUS_XIO_UDP_GET_RCVBUF:
      {
        int rcvbuf;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        result = globus_xio_attr_cntl( Attr, Driver, Parms->m_cmd,
                                       &rcvbuf );

        AppendInt( Parms, rcvbuf );
      }
      break;

      
    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  return result;
}


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
static globus_result_t
udp_driver_handle_cntl( globus_xio_handle_t Handle,
                        globus_xio_driver_t Driver,
                        driver_call_info_type* Parms )
{
  globus_result_t result;
  
  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_UDP_GET_HANDLE:
      {
        globus_xio_system_handle_t handle;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &handle );
        
        AppendInt( Parms, handle );
      }
      break;
      
      
    case GLOBUS_XIO_UDP_SET_SNDBUF:
      {
        int sndbuf;

        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        if ( GetVarArgAsInt( Parms, 0, &sndbuf ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         sndbuf );
      }
      break;


    case GLOBUS_XIO_UDP_GET_SNDBUF:
      {
        int sndbuf;
        
        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &sndbuf );

        AppendInt( Parms, sndbuf );
      }
      break;

      
    case GLOBUS_XIO_UDP_SET_RCVBUF:
      {
        int rcvbuf;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        if ( GetVarArgAsInt( Parms, 0, &rcvbuf ) == 0 )
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         rcvbuf );
      }
      break;


    case GLOBUS_XIO_UDP_GET_RCVBUF:
      {
        int rcvbuf;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &rcvbuf );

        AppendInt( Parms, rcvbuf );
      }
      break;

      
    case GLOBUS_XIO_UDP_CONNECT:
      {
        char *contact_string;
        int length;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        contact_string = GetVarArgAsString( Parms, 0, &length);
        
        if ( contact_string == NULL)
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         contact_string );
      }
      break;


    case GLOBUS_XIO_UDP_GET_CONTACT:
      {
        char *contact_string;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        contact_string = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(contact_string, '\0', MAX_NAME);
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &contact_string );
        
        AppendString( Parms, contact_string );
        
        globus_free(contact_string);
      }
      break;


    case GLOBUS_XIO_UDP_GET_NUMERIC_CONTACT:
      {
        char *contact_string;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        contact_string = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(contact_string, '\0', MAX_NAME);
        
        result = globus_xio_handle_cntl( Handle, Driver, Parms->m_cmd,
                                         &contact_string );

        AppendString( Parms, contact_string );

        globus_free(contact_string);
      }
      break;

      
    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  return result;
}


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
static globus_result_t
udp_driver_data_descriptor_cntl( globus_xio_data_descriptor_t Data_Descriptor,
                                 globus_xio_driver_t Driver,
                                 driver_call_info_type* Parms )
{
  globus_result_t result;
  
  switch( Parms->m_cmd )
    {
    case GLOBUS_XIO_UDP_SET_CONTACT:
      {
        char *contact_string;
        int length;
        
        if ( Parms->m_var_argc != 1 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }
        
        contact_string = GetVarArgAsString( Parms, 0, &length);
        
        if ( contact_string == NULL)
          {
            //:TODO: Return some error status, currently just returning generic
            //:TODO:   error
            return GLOBUS_FAILURE;
          }

        result = globus_xio_data_descriptor_cntl( Data_Descriptor,
                                                  Driver, Parms->m_cmd,
                                                  contact_string);
      }
      break;


    case GLOBUS_XIO_UDP_GET_CONTACT:
      {
        char *contact_string;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        contact_string = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(contact_string, '\0', MAX_NAME);
        
        result = globus_xio_data_descriptor_cntl( Data_Descriptor,
                                                  Driver, Parms->m_cmd,
                                                  &contact_string );

        AppendString( Parms, contact_string );
        
        globus_free(contact_string);
      }
      break;
      

    case GLOBUS_XIO_UDP_GET_NUMERIC_CONTACT:
      {
        char *contact_string;

        if ( Parms->m_var_argc != 0 )
          {
            //:TODO: inform user of error condition
	    return GLOBUS_FAILURE;
          }

        contact_string = (char*)globus_malloc(sizeof(char)*MAX_NAME);
        
        memset(contact_string, '\0', MAX_NAME);
        
        result = globus_xio_data_descriptor_cntl( Data_Descriptor,
                                                  Driver, Parms->m_cmd,
                                                  &contact_string );

        AppendString( Parms, contact_string );
        
        globus_free(contact_string);
      }
      break;

      
    default:
      // :TODO: inform user of error condition
      result = GLOBUS_FAILURE;
      break;
    }

  return result;
}

//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
XIODriverFunctionMap_type*
DriverUDPInit( )
{
  static char func_name[] = "DriverUDPInit";
  
  SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );

  XIODriverFunctionMap_type* retval = 
    (XIODriverFunctionMap_type*)
    globus_malloc( sizeof( XIODriverFunctionMap_type ) );

  retval->m_attr_cntl.m_function_call = 
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (udp_driver_attr_cntl);

  retval->m_handle_cntl.m_function_call =
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (udp_driver_handle_cntl);

  retval->m_server_cntl.m_function_call = NULL;
  
  retval->m_data_desc_cntl.m_function_call =
    (globus_result_t (*)(struct driver_call_info_struct* Parms))
    (udp_driver_data_descriptor_cntl);

  SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
  
  return retval;
}
