#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit

# In wish, we don't need to display the window
# But, we need to send it out of the visible part of the screen
if { [info commands wm] == "wm" } {
    wm iconify .
    #wm geometry . 1x1+4000+4000
}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_ftp_utils.tcl"

verifyProxy

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_ftp_client \
		  globus_ftp_client_debug_plugin } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
set SECTION general
validate_section $SECTION [list PORT]
set port $TestEnv($SECTION,PORT)

set SECTION ftp_test
validate_section $SECTION [list GSIFTP BUFFERSIZE TIMEOUT]
set timeout_value $TestEnv($SECTION,TIMEOUT)

verifyServer $TestEnv($SECTION,GSIFTP) $port

#------------------------------------------------------------------------
# Run test loop
#------------------------------------------------------------------------
proc run_test { } {
    if { [ catch {
	#----------------------------------------------------------------
	# Create remote directory
	#----------------------------------------------------------------
	create_remote_directory $::remote_topdir $::remote_subdir
	
	#----------------------------------------------------------------
	# Transfer files to remote site
	#----------------------------------------------------------------
	transfer_files_to_remote_site $::remote_subdir
	
	#----------------------------------------------------------------
	# Temporary local directory
	#----------------------------------------------------------------
	copy_init $::localdir
	
	#----------------------------------------------------------------
	# Transfer files from remote site
	#----------------------------------------------------------------
	transfer_files_from_remote_site $::remote_subdir
	
	#----------------------------------------------------------------
	# MD5 checksum
	#----------------------------------------------------------------
	foreach srcfile $::filelist {
	    if { [file isdirectory $srcfile] } {
		continue
	    }
	    set dstfile [ file join $::localdir [ file tail $srcfile ] ]
	    check_md5sum $srcfile $dstfile
	}
	
	#----------------------------------------------------------------
	# Remove local files and directory
	#----------------------------------------------------------------
	copy_deinit $::localdir
	
	#----------------------------------------------------------------
	# Delete all remote files 
	#----------------------------------------------------------------
	remove_remote_files $::remote_subdir
	
	#----------------------------------------------------------------
	# Remove remote directory
	#----------------------------------------------------------------
	remove_remote_directory $::remote_topdir $::remote_subdir
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------

if { [ catch {
    puts_info 10 "[info nameofexecutable]"
    
    # Initialization step
    ftp_client_init

    # Remote top level directory
    set remote_topdir [ construct_gsiftp_url $TestEnv($SECTION,GSIFTP) ]
    puts_info 10 "remote_topdir: $remote_topdir"

    # Remote sub-directory 
    set remote_subdir $remote_topdir
    append remote_subdir "/"
    append remote_subdir [constructOutputDirectory $::argv0]
    puts_info 10 "remote_subdir: $remote_subdir"
    
    # Set up input file(s)
    set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]

    # globus_ftp_client_get
    set localdir [file join /tmp \
		      [file join "$::env(USER)" \
			   [constructOutputDirectory $::argv0]]]
    puts_info 10 "localdir: $localdir"

    # Getting file list
    set filelist [ constructFileList ]
    puts_info 10 "Number of files: [ llength $filelist ]"

    # Run the test in infinite loop mode or not
    if { [IsInfinite] == 1 } {
	while 1 {
	    run_test
	}
    } else {
	run_test
    }

    # Cleanup the test
    ftp_client_cleanup
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
