#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------

source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_ftp_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_ftp_client \
		  globus_ftp_client_debug_plugin } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
set SECTION ftp_test
validate_section $SECTION [list GSIFTP]
puts_info 10 "GSIFTP: $TestEnv($SECTION,GSIFTP)"
#------------------------------------------------------------------------
# Main script
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "[info nameofexecutable]"
    
    ftp_client_init
    ftp_client_extra_init
    
    set url [ construct_gsiftp_url $TestEnv($SECTION,GSIFTP) ]
    
    # globus_ftp_client_handleattr_copy
    set dst_t	[ new_p_globus_ftp_client_handleattr_t ]
    puts_info 10 "dst_t: $dst_t"
    puts_info 10 "dst_t value: [ p_globus_ftp_client_handleattr_t_value $dst_t]"
    
    set src_t	[ new_p_globus_ftp_client_handleattr_t ]
    puts_info 10 "src_t: $src_t"
    puts_info 10 "src_t value: [ p_globus_ftp_client_handleattr_t_value $src_t]"
    
    puts_info 10 "globus_ftp_client_handleattr_init"
    globus_ftp_client_handleattr_init $dst_t 
    
    puts_info 10 "globus_ftp_client_handleattr_init"
    globus_ftp_client_handleattr_init $src_t 
    
    puts_info 10 "globus_ftp_client_handleattr_copy"
    globus_ftp_client_handleattr_copy \
	$dst_t \
	$src_t 
    
    puts_info 10 "globus_ftp_client_handleattr_destroy"
    globus_ftp_client_handleattr_destroy $dst_t 
    
    puts_info 10 "globus_ftp_client_handleattr_destroy"
    globus_ftp_client_handleattr_destroy $src_t 

    puts_info 10 "Reclaim memory space"
    delete_p_globus_ftp_client_handleattr_t $dst_t
    delete_p_globus_ftp_client_handleattr_t $src_t
    puts_info 10 "Done reclaiming memory space"
    
    #--------------------------------------------------------------------
    # globus_ftp_client_handleattr_add_cached_url
    #--------------------------------------------------------------------    
    puts_info 10 "globus_ftp_client_handleattr_add_cached_url"
    globus_ftp_client_handleattr_add_cached_url $handle_attr $url 

    #--------------------------------------------------------------------
    # globus_ftp_client_handleattr_remove_cached_url
    #--------------------------------------------------------------------
    puts_info 10 "globus_ftp_client_handleattr_remove_cached_url"
    globus_ftp_client_handleattr_remove_cached_url $handle_attr $url

    #--------------------------------------------------------------------
    # globus_ftp_client_handleattr_set_cache_all
    #--------------------------------------------------------------------
    puts_info 10 "globus_ftp_client_handleattr_set_cache_all"
    globus_ftp_client_handleattr_set_cache_all $handle_attr $GLOBUS_TRUE

    #--------------------------------------------------------------------
    # globus_ftp_client_handleattr_get_cache_all
    #--------------------------------------------------------------------
    lassign [ globus_ftp_client_handleattr_get_cache_all \
		  $handle_attr \
		 ] result cache_all
    puts_info 10 "globus_ftp_client_handleattr_get_cache_all returns $cache_all"

    #--------------------------------------------------------------------
    # globus_ftp_client_handleattr_set_rfc1738_url
    #--------------------------------------------------------------------
    puts_info 10 "globus_ftp_client_handleattr_set_rfc1738_url"
    globus_ftp_client_handleattr_set_rfc1738_url $handle_attr $GLOBUS_FALSE
    
    #--------------------------------------------------------------------
    # globus_ftp_client_handleattr_get_rfc1738_url
    #--------------------------------------------------------------------
    lassign [ globus_ftp_client_handleattr_get_rfc1738_url \
		  $handle_attr \
		 ] result rfc1738_url
    
    puts_info 10 "globus_ftp_client_handleattr_get_rfc1738_url returns $rfc1738_url"
    
    #--------------------------------------------------------------------
    # globus_ftp_client_handle_add_plugin
    #--------------------------------------------------------------------
    puts_info 10 "globus_ftp_client_handleattr_add_plugin"
    globus_ftp_client_handleattr_add_plugin $handle_attr $plugin_t
    
    #--------------------------------------------------------------------
    # globus_ftp_client_handle_remove_plugin
    #--------------------------------------------------------------------
    puts_info 10 "globus_ftp_client_handleattr_remove_plugin"
    globus_ftp_client_handleattr_remove_plugin $handle_attr $plugin_t 

    ftp_client_cleanup
    ftp_client_extra_cleanup
} err ] } {
    err_handler $err
}

#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
