#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------

source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_ftp_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_ftp_control \
		  globus_ftp_client \
		  globus_ftp_client_debug_plugin \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Main script
#------------------------------------------------------------------------

if { [ catch {
    puts_info 10 "[info nameofexecutable]"
    
    ftp_client_init

    #--------------------------------------------------------------------
    #
    # The sampel code below is to demonstrate on how to set/get
    # parallelism values related to globus_ftp_client operation
    # attributes
    #
    #--------------------------------------------------------------------

    set PARALLELISM_SIZE		4
    puts_info 10 "PARALLELISM_SIZE value: $PARALLELISM_SIZE"

    puts_info 10 "new_globus_ftp_parallelism_fixed_t"
    set parallelism_fixed_t [ new_globus_ftp_parallelism_fixed_t ]
    puts_info 10 "parallelism_fixed: $parallelism_fixed_t"

    puts_info 10 "Calling globus_ftp_parallelism_fixed_t_size_set"
    globus_ftp_parallelism_fixed_t_size_set $parallelism_fixed_t \
	$PARALLELISM_SIZE

    puts_info 10 "new_globus_ftp_control_parallelism_t"
    set in_parallelism_t [ new_globus_ftp_control_parallelism_t ]
    puts_info 10 "parallelism: $in_parallelism_t"

    puts_info 10 "Calling globus_ftp_control_parallelism_t_mode_set"
    globus_ftp_control_parallelism_t_mode_set $in_parallelism_t \
	$GLOBUS_FTP_CONTROL_PARALLELISM_FIXED

    puts_info 10 "Calling globus_ftp_control_parallelism_t_fixed_get"
    lassign [ globus_ftp_control_parallelism_t_fixed_get \
		  $in_parallelism_t \
		 ] parallelism_fixed_value
    puts_info 10 "globus_ftp_control_parallelism_t_fixed_get returns $parallelism_fixed_value"

    puts_info 10 "Calling globus_ftp_parallelism_fixed_t_size_get"
    lassign [ globus_ftp_parallelism_fixed_t_size_get \
		  $parallelism_fixed_value \
		 ] parallelism_size_value
    puts_info 10 "globus_ftp_parallelism_fixed_t_size_get returns $parallelism_size_value"

    
    puts_info 10 "Calling globus_ftp_control_parallelism_t_mode_get"
    lassign [ globus_ftp_control_parallelism_t_mode_get \
		  $in_parallelism_t \
		 ] mode_value
    puts_info 10 "globus_ftp_control_parallelism_t_mode_get returns $mode_value"
    
    puts_info 10 "Calling globus_ftp_control_parallelism_t_fixed_set"
    globus_ftp_control_parallelism_t_fixed_set \
	$in_parallelism_t \
	$parallelism_fixed_t

    lassign [ globus_ftp_control_parallelism_t_fixed_get \
		  $in_parallelism_t \
		 ] result
    puts_info 10 "globus_ftp_control_parallelism_t_fixed_get returns $result"
    
    #--------------------------------------------------------------------
    #
    # Parallelism
    #
    #--------------------------------------------------------------------
    
    lassign [ globus_ftp_client_operationattr_set_parallelism \
		  $operation_attr \
		  $in_parallelism_t \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_parallelism returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_parallelism \
		  $operation_attr \
		 ] result out_parallelism
    puts_info 10 "globus_ftp_client_operationattr_get_parallelism returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_parallelism returns $out_parallelism"
    
    #--------------------------------------------------------------------
    #
    # Striped Data Movement
    #
    #--------------------------------------------------------------------

    lassign [ globus_ftp_client_operationattr_set_striped \
		  $operation_attr \
		  $GLOBUS_TRUE \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_striped returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_striped \
		  $operation_attr \
		 ] result striped
    puts_info 10 "globus_ftp_client_operationattr_get_striped returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_striped returns $striped"
    
    
    #--------------------------------------------------------------------
    #
    # Striped File Layout
    #
    #--------------------------------------------------------------------

    puts_info 10 "new_globus_ftp_control_layout_t"
    set layout_t [ new_globus_ftp_control_layout_t ]
    puts_info 10 "layout: $layout_t"
    
    lassign [ globus_ftp_client_operationattr_set_layout \
		  $operation_attr \
		  $layout_t \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_layout returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_layout \
		  $operation_attr \
		 ] result out_layout
    puts_info 10 "globus_ftp_client_operationattr_get_layout returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_layout returns $out_layout"
    
    #--------------------------------------------------------------------
    #
    # TCP Buffer
    #
    #--------------------------------------------------------------------

    puts_info 10 "new_globus_ftp_control_tcpbuffer_t"
    set tcpbuffer_t [ new_globus_ftp_control_tcpbuffer_t ]
    puts_info 10 "tcpbuffer_t: $tcpbuffer_t"
    
    lassign [ globus_ftp_client_operationattr_set_tcp_buffer \
		  $operation_attr \
		  $tcpbuffer_t \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_tcp_buffer returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_tcp_buffer \
		  $operation_attr \
		 ] result out_tcpbuffer
    puts_info 10 "globus_ftp_client_operationattr_get_tcp_buffer returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_tcp_buffer returns $out_tcpbuffer"
    
    #--------------------------------------------------------------------
    #
    # Transfer Mode
    #
    #--------------------------------------------------------------------

    lassign [ globus_ftp_client_operationattr_set_mode \
		  $operation_attr \
		  $GLOBUS_FTP_CONTROL_MODE_EXTENDED_BLOCK \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_mode returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_mode \
		  $operation_attr \
		 ] result mode
    puts_info 10 "globus_ftp_client_operationattr_get_mode returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_mode returns $mode"

    lassign [ globus_ftp_client_operationattr_set_list_uses_data_mode \
		  $operation_attr \
		  $GLOBUS_TRUE \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_list_uses_data_mode returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_list_uses_data_mode \
		  $operation_attr \
		 ] result list_uses_data_mode
    puts_info 10 "globus_ftp_client_operationattr_get_list_uses_data_mode returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_list_uses_data_mode returns $list_uses_data_mode"
    
    #--------------------------------------------------------------------
    #
    # Authorization
    #
    #--------------------------------------------------------------------

    if { 0 } {
	
	lassign [ globus_ftp_client_operationattr_set_authorization \
		      $operation_attr \
		      $gss_c_no_credential \
		      user \
		      password \
		      account \
		      subject \
		     ] result
	
	if { $result != $GLOBUS_SUCCESS } {
	    Debug "globus_ftp_client_operationattr_set_authorization"
	    incr FAIL
	}
	
	
	lassign [ globus_ftp_client_operationattr_get_authorization \
		      $operation_attr \
		      $gss_c_no_credential \
		     ] result user password account subject
	
	if { $result != $GLOBUS_SUCCESS } {
	    Debug "globus_ftp_client_operationattr_get_authorization"
	    incr FAIL
	}
    }
	
    #--------------------------------------------------------------------
    #
    # Data Channel Authentication
    #
    #--------------------------------------------------------------------

    puts_info 10 "new_globus_ftp_control_dcau_t"
    set dcau_t [ new_globus_ftp_control_dcau_t ]
    puts_info 10 "dcau_t: $dcau_t"
    
    lassign [ globus_ftp_client_operationattr_set_dcau \
		  $operation_attr \
		  $dcau_t \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_dcau returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_dcau \
		  $operation_attr \
		 ] result out_dcau
    puts_info 10 "globus_ftp_client_operationattr_get_dcau returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_dcau returns $out_dcau"

    #--------------------------------------------------------------------
    #
    # Data Channel Protection
    #
    #--------------------------------------------------------------------

    lassign [ globus_ftp_client_operationattr_set_data_protection \
		  $operation_attr \
		  $GLOBUS_FTP_CONTROL_PROTECTION_CLEAR \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_data_protection returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_data_protection \
		  $operation_attr \
		 ] result data_protection_t
    puts_info 10 "globus_ftp_client_operationattr_get_data_protection returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_data_protection returns $data_protection_t"
    
    #--------------------------------------------------------------------
    #
    # Control Channel Protection
    #
    #--------------------------------------------------------------------
    
    lassign [ globus_ftp_client_operationattr_set_control_protection \
		  $operation_attr \
		  $GLOBUS_FTP_CONTROL_PROTECTION_SAFE \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_control_protection returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_control_protection \
		  $operation_attr \
		 ] result control_protection_t
    puts_info 10 "globus_ftp_client_operationattr_get_control_protection returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_control_protection returns $control_protection_t"
 
    #--------------------------------------------------------------------
    #
    # Append
    #
    #--------------------------------------------------------------------

    lassign [ globus_ftp_client_operationattr_set_append \
		  $operation_attr \
		  $GLOBUS_FALSE \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_append returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_append \
		  $operation_attr \
		 ] result append_value
    puts_info 10 "globus_ftp_client_operationattr_get_append returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_append returns $append_value"
    
    #--------------------------------------------------------------------
    #
    # Read into a Single Buffer
    #
    #--------------------------------------------------------------------

    
    #--------------------------------------------------------------------
    #
    # Resume third party transfer
    #
    #--------------------------------------------------------------------

    lassign [ globus_ftp_client_operationattr_set_resume_third_party_transfer \
		  $operation_attr \
		  $GLOBUS_TRUE \
		 ] result
    puts_info 10 "globus_ftp_client_operationattr_set_resume_third_party_transfer returns $result"
    
    lassign [ globus_ftp_client_operationattr_get_resume_third_party_transfer \
		  $operation_attr \
		 ] result resume_value
    puts_info 10 "globus_ftp_client_operationattr_get_resume_third_party_transfer returns $result"
    puts_info 10 "globus_ftp_client_operationattr_get_resume_third_party_transfer returns $resume_value"
        
    #--------------------------------------------------------------------
    #
    # Free up the operation attributes
    #
    #--------------------------------------------------------------------

    puts_info 10 "delete_globus_ftp_control_dcau_t"
    delete_globus_ftp_control_dcau_t $dcau_t
    puts_info 10 "delete_globus_ftp_control_tcpbuffer_t"
    delete_globus_ftp_control_tcpbuffer_t $tcpbuffer_t
    puts_info 10 "delete_globus_ftp_control_layout_t"
    delete_globus_ftp_control_layout_t $layout_t
    puts_info 10 "delete_globus_ftp_parallelism_fixed_t"
    delete_globus_ftp_parallelism_fixed_t $parallelism_fixed_t
    puts_info 10 "delete_globus_ftp_control_parallelism_t"
    delete_globus_ftp_control_parallelism_t $in_parallelism_t
    
    ftp_client_cleanup

} err ] } {
    err_handler $err
}

#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
