#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit

# In wish, we don't need to display the window
# But, we need to send it out of the visible part of the screen
if { [info commands wm] == "wm" } {
    wm iconify .
    wm geometry . 1x1+4000+4000
}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_ftp_utils.tcl"

verifyProxy

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io \
		  globus_ftp_client } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

set SECTION general
validate_section $SECTION [list PORT]
set port $TestEnv($SECTION,PORT)

set SECTION ftp_test
validate_section $SECTION [list GSIFTP]
validate_section $SECTION [list FILENAME]
validate_section $SECTION [list NEWFILENAME]
validate_section $SECTION [list TIMEOUT]

set timeout_value $TestEnv($SECTION,TIMEOUT)
puts_info 10 "timeout value: $timeout_value"

verifyServer $TestEnv($SECTION,GSIFTP) $port

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "[info nameofexecutable]"
    
    # Initialization step
    ftp_client_init

    # Create top level directory
    set remote_topdir [construct_gsiftp_url $TestEnv($SECTION,GSIFTP)]
    puts_info 10 "remote_topdir: $remote_topdir"

    # Create sub-directory
    set remote_subdir $remote_topdir
    append remote_subdir "/"
    append remote_subdir [constructOutputDirectory $::argv0]
    puts_info 10 "remote_subdir: $remote_subdir"

    # Create remote directories
    create_remote_directory $remote_topdir $remote_subdir

    # Setup srcfile information
    set srcfile [ file join [ file join [ file dirname [ pwd ] ] swig ]\
		      $TestEnv($SECTION,FILENAME) ]
    puts_info 10 "srcfile: $::srcfile"
    set copy_info(srcId) [ open $srcfile r ]
    set filetype [ fileType $srcfile ]
    if { [ fileType $srcfile ] == "binary executable elf" } {
	fconfigure $::copy_info(srcId) -translation binary -encoding binary
    }
    
    # Construct target filename
    set targetfile $remote_subdir
    append targetfile "/"
    append targetfile $TestEnv($SECTION,FILENAME)
    puts_info 10 "targetfile: $targetfile"

    # globus_ftp_client_put
    ftp_client_put $targetfile
    
    # globus_ftp_client_size
    ftp_client_size $targetfile

    # Modification time
    ftp_client_modification_time $targetfile

    # globus_ftp_client_chmod
    ftp_client_chmod $targetfile

    # Construct new target filename
    set new_targetfile $remote_subdir
    append new_targetfile "/"
    append new_targetfile $TestEnv($SECTION,NEWFILENAME)
    puts_info 10 "new_targetfile: $new_targetfile"
    
    # globus_ftp_client_move
    ftp_client_move $targetfile $new_targetfile

    # globus_ftp_client_move
    ftp_client_move $new_targetfile $targetfile

    # Setup local directory
    set localdir [file join /tmp \
		      [file join "$::env(USER)" \
			   [constructOutputDirectory $::argv0]]]
    puts_info 10 "localdir: $localdir"

    # Create temporary directory
    copy_init $localdir

    # Setup destination info
    set dstfile [ file join $localdir $TestEnv($SECTION,FILENAME) ]
    puts_info 10 "dstfile: $::dstfile"
    set copy_info(dst) [ open $dstfile w 0600 ]
    puts_info 10 "copy_info(dst): $copy_info(dst)"
    
    # globus_ftp_client_get
    ftp_client_get $targetfile

    # Run MD5 checksum
    check_md5sum $srcfile $dstfile

    # globus_ftp_client_delete
    ftp_client_delete $targetfile

    # Remove remote directory
    remove_remote_directory $remote_topdir $remote_subdir

    # Cleanup the test
    close $::copy_info(srcId)
    close $::copy_info(dst)

    # Remove temporary directory
    copy_deinit $localdir

    # Cleanup step
    ftp_client_cleanup
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
