#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit

# In wish, we don't need to display the window
# But, we need to send it out of the visible part of the screen
if { [info commands wm] == "wm" } {
    wm iconify .
    wm geometry . 1x1+4000+4000
}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_ftp_utils.tcl"

verifyProxy

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_ftp_client \
		  globus_ftp_client_debug_plugin } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

set SECTION general
validate_section $SECTION [list PORT]
set port $TestEnv($SECTION,PORT)

set SECTION ftp_test
validate_section $SECTION [list GSIFTP GSIFTP_2 BUFFERSIZE TIMEOUT]

set timeout_value $TestEnv($SECTION,TIMEOUT)
puts_info 10 "timeout value: $timeout_value"

verifyServer $TestEnv($SECTION,GSIFTP) $port
verifyServer $TestEnv($SECTION,GSIFTP_2) $port

#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc run_test { } {
    global first_gsiftp
    global second_gsiftp
    global first_subdir
    global second_subdir
    global filelist
    global localdir
    
    if { [ catch {
	# Create first GSIFTP directory
	create_remote_directory $first_gsiftp $first_subdir

	# Transfer files to remote site
	transfer_files_to_remote_site $first_subdir
    
	# Create first GSIFTP directory
	create_remote_directory $second_gsiftp $second_subdir

	# Do third party transfer
	foreach file $filelist {
	    set f [ file tail $file ]

	    set srcfile $first_subdir
	    append srcfile "/"
	    append srcfile $f
	    puts_info 10 "srcfile: $srcfile"
	    
	    set dstfile $second_subdir
	    append dstfile "/"
	    append dstfile $f
	    puts_info 10 "dstfile: $dstfile"

	    ftp_client_third_party_transfer $srcfile $dstfile
	}

	# Temporary local directory
	copy_init $localdir

	# Transfer files from remote site
	transfer_files_from_remote_site $second_subdir
	
	# MD5 checksum
	foreach srcfile $filelist {
	    if { [file isdirectory $srcfile] } {
		continue
	    }
	    set dstfile [ file join $localdir [ file tail $srcfile ] ]
	    check_md5sum $srcfile $dstfile
	}
	
	# Remove local files and directory
	copy_deinit $localdir

	# Remove remote files
	remove_remote_files $first_subdir
	remove_remote_files $second_subdir
	
	# Remove remote directory
	remove_remote_directory $first_gsiftp $first_subdir
	remove_remote_directory $second_gsiftp $second_subdir
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "[info nameofexecutable]"

    # Initialization step
    ftp_client_init

    # First GSIFTP - top level directory
    set first_gsiftp [construct_gsiftp_url $TestEnv($SECTION,GSIFTP)]
    puts_info 10 "first_gsiftp: $first_gsiftp"
    # First GSIFTP - Remote sub-directory 
    set first_subdir $first_gsiftp
    append first_subdir "/"
    append first_subdir [constructOutputDirectory $::argv0]
    puts_info 10 "first_subdir: $first_subdir"

    # Second GSIFTP - top level directory
    set second_gsiftp [construct_gsiftp_url $TestEnv($SECTION,GSIFTP_2)]
    puts_info 10 "second_gsiftp: $second_gsiftp"
    # Second GSIFTP - Remote sub-directory 
    set second_subdir $second_gsiftp
    append second_subdir "/"
    append second_subdir [constructOutputDirectory $::argv0]
    puts_info 10 "second_subdir: $second_subdir"

    # Set up input file(s)
    set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
    puts_info 10 "sourcedir: $sourcedir"
    
    # globus_ftp_client_get
    set localdir [file join /tmp \
		      [file join "$::env(USER)" \
			   [constructOutputDirectory $::argv0]]]
    puts_info 10 "localdir: $localdir"
    
    # Getting file list
    set filelist [ constructFileList ]
    puts_info 10 "Number of files: [ llength $filelist ]"
    
    # Run the test in infinite loop mode or not
    if { [IsInfinite] == 1 } {
	while 1 {
	    run_test
	}
    } else {
	run_test
    }
    
    # Cleanup step
    ftp_client_cleanup
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
