#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

puts_info 10 "env(TOPSRCDIR) $env(TOPSRCDIR)"
puts_info 10 "env(SRCDIR) $env(SRCDIR)"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_ftp_control \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------

set host bosco.ligo.caltech.edu
set port 35000

#------------------------------------------------------------------------
# Response callback
#------------------------------------------------------------------------
proc response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering response_cb"
	upvar 0 $user_parms user_args
	puts "user_parms $user_parms handle $handle err $err ftp_response $ftp_response"
	set ::cb_done 1
	puts_info 10 "leaving response_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Main
#------------------------------------------------------------------------
if { [ catch {
    globus_module_activate $::globus_i_ftp_control_module

    lassign [ globus_ftp_control_handle_init ] result handle
    puts_info 10 "handle $handle"
    
    set cb_done 0
    puts_info 10 "globus_ftp_control_connect"
    lassign [ globus_ftp_control_connect \
		  $handle \
		  $host \
		  $port \
		  { response_cb NULL } \
		 ] result
    vwait cb_done

    puts_info 10 "globus_ftp_control_auth_info_init"
    puts_info 10 "gss_c_no_credential $gss_c_no_credential"
    lassign [ globus_ftp_control_auth_info_init \
		  $gss_c_no_credential \
		  $GLOBUS_TRUE \
		  "install" \
		  "password" \
		  $GLOBUS_NULL \
		  "/DC=org/DC=doegrids/OU=Services/CN=host/tclproxy.ligo.caltech.edu" \
		 ] result auth_info
    puts_info "auth_info $auth_info"

    set cb_done 0
    puts_info 10 "globus_ftp_control_authenticate"
    lassign [ globus_ftp_control_authenticate \
		  $handle \
		  $auth_info \
		  $GLOBUS_TRUE \
		  { response_cb NULL } \
		 ] result
    vwait cb_done
    
    set cb_done 0
    puts_info 10 "globus_ftp_control_send_command"
    lassign [ globus_ftp_control_send_command \
		  $handle \
		  "PWD\r\n" \
		  { response_cb NULL } \
		 ] result
    vwait cb_done
    
    set cb_done 0
    puts_info 10 "globus_ftp_control_quit"
    lassign [ globus_ftp_control_quit \
		  $handle \
		  { response_cb NULL } \
		 ] result
    vwait cb_done

    if { 0 } {
	lassign [ globus_ftp_control_abort \
		      $handle \
		      { response_cb NULL } \
		     ] result
	lassign [ globus_ftp_control_force_close \
		      $handle \
		      { response_cb NULL } \
		     ] result
	lassign [ globus_ftp_control_quit \
		      $handle \
		      { response_cb NULL } \
		     ] result
    }
    
    puts_info 10 "globus_ftp_control_handle_destroy"
    lassign [ globus_ftp_control_handle_destroy $handle ] result

    globus_module_deactivate $::globus_i_ftp_control_module
} err ] } {
    err_handler $err
}    
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
