#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_ftp_control_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_ftp_control \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
set SECTION general
validate_section $SECTION [list PORT]
set port $TestEnv($SECTION,PORT)

set SECTION ftp_test
validate_section $SECTION [list GSIFTP BUFFERSIZE FILENAME]
set host $TestEnv($SECTION,GSIFTP)
set buffersize $TestEnv($SECTION,BUFFERSIZE)
set filename $TestEnv($SECTION,FILENAME)

puts_info 10 "host $host port $port"
puts_info 10 "buffersize $buffersize"
puts_info 10 "filename $filename"

set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
puts_info 10 "sourcedir: $sourcedir"

set localdir [ file join /tmp \
		  [ file join "$env(USER)" \
			[ constructOutputDirectory $argv0 ] ] ]
puts_info 10 "localdir: $localdir"

verifyProxy

#------------------------------------------------------------------------
# Run test
#------------------------------------------------------------------------
proc run_test { control_handle } {
    if { [ catch {
	puts_info 10 "entering run_test"

	puts_info 10 "control_handle $control_handle"

	set dcau [ new_globus_ftp_control_dcau_t ]
	set dcau_subject [ new_globus_ftp_control_dcau_subject_t ]
	
	# Send PASV command
	ftp_control_send_pasv_cmd $control_handle
	
	# Data channel inialization
	data_channel_init $control_handle $dcau $dcau_subject
	
	# Retrieve the file
	set dest_file [ file join $::localdir $::filename ]
	puts_info 10 "dest_file $dest_file"
	set ::fid [ open $dest_file w ]
	fconfigure $::fid -translation binary
	fconfigure $::fid -encoding binary

	ftp_control_send_retr_cmd $control_handle $::filename

	close $::fid
	
	# Delete remote file
	#set cmd "DELE "
	#append cmd [file tail $::filename]
	#append cmd "\r\n"
	#ftp_control_send_single_command $control_handle $cmd
	
	# Run MD5 checksum
	#check_md5sum $::filename
	
	delete_globus_ftp_control_dcau_subject_t $dcau_subject
	delete_globus_ftp_control_dcau_t $dcau
	
	puts_info 10 "leaving run_test"
    } err ] } {
	return -code error $err
    }	
}

#------------------------------------------------------------------------
# Main
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "pid: [ pid ]"

    # Activate Globus FTP control module
    ftp_control_activation

    # Create Globus FTP control handle 
    set control_handle [ ftp_control_handle_initialization ]
    
    # Connect to remote server
    ftp_control_connect $control_handle $host $port
    
    if { $connect_failed == 0 } {
	# Create temporary directory
	copy_init $localdir
	
	# Initialize local host & port information
	set addr [ ftp_control_host_port_init 0 ]

	set cmd "CWD "
	append cmd $::env(USER)
	append cmd "/"
	#append cmd [constructOutputDirectory $::argv0]
	append cmd "test_ftp_control_file_copy-fc4-gt4-threaded"
	append cmd "\r\n"
	ftp_control_send_single_command $control_handle $cmd
	
	# Run the test in infinite loop mode or not
	if { [IsInfinite] == 1 } {
	    while 1 {
		run_test $control_handle
	    }
	} else {
	    run_test $control_handle
	}

	if { 0 } {
	    set cmd "CWD "
	    append cmd ".."
	    append cmd "\r\n"
	    ftp_control_send_single_command $control_handle $cmd
	    
	    # Remove remote directory
	    set cmd "RMD "
	    #append cmd [constructOutputDirectory $::argv0]
	    append cmd "test_ftp_control_file_copy-fc4-gt4-threaded"
	    append cmd "\r\n"
	    ftp_control_send_single_command $control_handle $cmd
	    
	    set cmd "CWD "
	    append cmd ".."
	    append cmd "\r\n"
	    ftp_control_send_single_command $control_handle $cmd
	    
	    # Remove remote directory
	    set cmd "RMD "
	    append cmd $::env(USER)
	    append cmd "\r\n"
	    ftp_control_send_single_command $control_handle $cmd
	}
	
	# Quit
	ftp_control_quit $control_handle

	# Remove temporary directory
	#copy_deinit $localdir
    }
    
    # Destroy control handle
    ftp_control_handle_destroy $control_handle

    # Deactivate Globus FTP control module
    ftp_control_deactivation
    
} err ] } {
    err_handler $err
}    
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
