#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

puts_info 10 "env(TOPSRCDIR) $env(TOPSRCDIR)"
puts_info 10 "env(SRCDIR) $env(SRCDIR)"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_ftp_control \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
set SECTION ftp_test
validate_section $SECTION [list GSIFTP]
set host $TestEnv($SECTION,GSIFTP)

set SECTION general
validate_section $SECTION [list PORT]
set port $TestEnv($SECTION,PORT)

puts_info 10 "host $host port $port"

verifyServer $host $port

verifyProxy

#------------------------------------------------------------------------
# Destroy callback
#------------------------------------------------------------------------
proc destroy_cb { user_parms handle err } {
    if { [ catch {
	puts_info 10 "entering destroy_cb"
	upvar 0 $user_parms user_args
	puts_info 10 "\tuser_parms $user_parms handle $handle err $err"
	set ::cb_done 1
	puts_info 10 "leaving destroy_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Response callback
#------------------------------------------------------------------------
proc response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering response_cb"
	upvar 0 $user_parms user_args
	puts_info 10 "\tuser_parms $user_parms handle $handle err $err ftp_response $ftp_response"

	if { [string length $err] == 0 } {
	    lassign [ globus_ftp_control_response_t_code_get \
			  $ftp_response \
			 ] code
	    puts_info 10 "code $code"
	}
	
	set ::cb_done 1

	puts_info 10 "leaving response_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Main
#------------------------------------------------------------------------
if { [ catch {
    globus_module_activate $globus_i_ftp_control_module

    # globus_ftp_control_handle_init
    puts_info 10 "globus_ftp_control_handle_init"
    lassign [ globus_ftp_control_handle_init ] result handle
    puts_info 10 "\tresult $result handle $handle"

    # globus_ftp_control_connect
    puts_info 10 "globus_ftp_control_connect"
    set cb_done 0
    lassign [ globus_ftp_control_connect \
		  $handle \
		  $host \
		  $port \
		  { response_cb NULL } \
		 ] result
    puts_info 10 "\tresult $result"
    vwait cb_done

    # globus_ftp_control_host_port_init
    puts_info 10 "globus_ftp_control_host_port_init"
    lassign [ globus_ftp_control_host_port_init \
		  [info hostname] \
		  0 \
		 ] addr
    puts_info 10 "\taddr $addr"

    # globus_ftp_control_local_pasv
    puts_info 10 "globus_ftp_control_local_pasv"
    lassign [ globus_ftp_control_local_pasv \
		  $handle \
		  $addr \
		 ] result nodeinfo
    puts_info 10 "\tresult $result nodeinfo $nodeinfo"

    # globus_ftp_control_local_port
    puts_info 10 "globus_ftp_control_local_port"
    lassign [ globus_ftp_control_local_port \
		  $handle \
		  $addr \
		 ] result 
    puts_info 10 "\tresult $result"

    # globus_ftp_control_get_parallelism
    puts_info 10 "globus_ftp_control_get_parallelism"
    lassign [ globus_ftp_control_get_parallelism \
		  $handle \
		 ] result parallelism
    puts_info 10 "\tresult $result parallelism $parallelism"

    # globus_ftp_control_local_parallelism
    puts_info 10 "globus_ftp_control_local_parallelism"
    lassign [ globus_ftp_control_local_parallelism \
		  $handle \
		  $parallelism \
		 ] result
    puts_info 10 "\tresult $result"

    # globus_ftp_control_local_type
    puts_info 10 "globus_ftp_control_local_type"
    lassign [ globus_ftp_control_local_type \
		  $handle \
		  $GLOBUS_FTP_CONTROL_TYPE_ASCII \
		  1 \
		 ] result
    puts_info 10 "\tresult $result"

    # globus_ftp_control_get_type
    puts_info 10 "globus_ftp_control_get_type"
    lassign [ globus_ftp_control_get_type \
		  $handle \
		 ] result type
    puts_info 10 "\tresult $result type $type"

    # globus_ftp_control_local_mode
    puts_info 10 "globus_ftp_control_local_mode"
    lassign [ globus_ftp_control_local_mode \
		  $handle \
		  $GLOBUS_FTP_CONTROL_MODE_STREAM \
		 ] result
    puts_info 10 "\tresult $result"

    # globus_ftp_control_get_mode
    puts_info 10 "globus_ftp_control_get_mode"
    lassign [ globus_ftp_control_get_mode \
		  $handle \
		 ] result mode
    puts_info 10 "\tresult $result mode $mode"

    # globus_ftp_control_get_stripe_count
    puts_info 10 "globus_ftp_control_get_stripe_count"
    lassign [ globus_ftp_control_get_stripe_count \
		  $handle \
		 ] result stripe_count
    puts_info 10 "\tresult $result stripe_count $stripe_count"

    if { 0 } {
	# globus_ftp_control_data_add_channels
	puts_info 10 "globus_ftp_control_data_add_channels"
	lassign [ globus_ftp_control_data_add_channels \
		      $handle \
		      1 \
		      1 \
		     ] result
	puts_info 10 "\tresult $result"

    
	# globus_ftp_control_data_remove_channels
	puts_info 10 "globus_ftp_control_data_add_channels"
	lassign [ globus_ftp_control_data_remove_channels \
		      $handle \
		      1 \
		      1 \
		     ] result
	puts_info 10 "\tresult $result"
    }
    
    # globus_ftp_control_data_get_total_data_channels
    puts_info 10 "globus_ftp_control_data_get_total_data_channels"
    lassign [ globus_ftp_control_data_get_total_data_channels \
		  $handle \
		  0 \
		 ] result num_channels
    puts_info 10 "\tresult $result num_channels $num_channels"

    #globus_ftp_control_data_query_channels
    puts_info 10 "globus_ftp_control_data_query_channels"
    lassign [ globus_ftp_control_data_query_channels \
		  $handle \
		  0 \
		 ] result num_channels
    puts_info 10 "\tresult $result num_channels $num_channels"

    # globus_ftp_control_local_layout
    set layout [ new_globus_ftp_control_layout_t ]

    puts_info 10 "globus_ftp_control_local_layout"
    lassign [ globus_ftp_control_local_layout \
		  $handle \
		  $layout \
		  1028 \
		 ] result
    puts_info 10 "\tresult $result"
    
    delete_globus_ftp_control_layout_t $layout

    # globus_ftp_control_data_set_interface
    puts_info 10 "globus_ftp_control_data_set_interface"
    lassign [ globus_ftp_control_data_set_interface \
		  $handle \
		  "131.215.114.146" \
		 ] result
    puts_info 10 "\tresult $result"

    set tcpbuffer [ new_globus_ftp_control_tcpbuffer_t ]
    
    globus_ftp_control_tcpbuffer_t_mode_set \
	$tcpbuffer \
	$GLOBUS_FTP_CONTROL_TCPBUFFER_FIXED

    # globus_ftp_control_local_tcp_buffer
    puts_info 10 "globus_ftp_control_local_tcp_buffer"
    lassign [ globus_ftp_control_local_tcp_buffer \
		  $handle \
		  $tcpbuffer \
		 ] result
    puts_info 10 "\tresult $result"

    delete_globus_ftp_control_tcpbuffer_t $tcpbuffer

    set dcau [ new_globus_ftp_control_dcau_t ]

    puts_info 10 "globus_ftp_control_dcau_t_mode_set"
    globus_ftp_control_dcau_t_mode_set \
	$dcau \
	$GLOBUS_FTP_CONTROL_DCAU_SELF

    # globus_ftp_control_dcau_t_mode_get
    puts_info 10 "globus_ftp_control_dcau_subject_t_mode_get"
    lassign [ globus_ftp_control_dcau_t_mode_get \
		  $dcau \
		 ] dcau_mode
    puts_info 10 "\tdcau_mode $dcau_mode"

    if { 0 } {
	set dcau_subject [ new_globus_ftp_control_dcau_subject_t ]
	
	puts_info 10 "globus_ftp_control_dcau_subject_t_subject_set"
	globus_ftp_control_dcau_subject_t_subject_set \
	    $dcau_subject \
	    /DC=org/DC=doegrids/OU=Services/CN=tclproxy.ligo.caltech.edu
	
	puts_info 10 "globus_ftp_control_dcau_subject_t_subject_get"
	lassign [ globus_ftp_control_dcau_subject_t_subject_get \
		      $dcau_subject \
		     ] dcau_subject
	puts_info 10 "dcau_subject $dcau_subject"
    }

    # globus_ftp_control_local_dcau
    puts_info 10 "globus_ftp_control_local_dcau"
    lassign [ globus_ftp_control_local_dcau \
		  $handle \
		  $dcau \
		  $gss_c_no_credential \
		 ] result
    puts_info 10 "\tresult $result"

    # globus_ftp_control_get_dcau
    puts_info 10 "globus_ftp_control_get_dcau"
    lassign [ globus_ftp_control_get_dcau \
		  $handle \
		  $dcau \
		 ] result
    puts_info 10 "\tresult $result"

    # globus_ftp_control_local_prot
    puts_info 10 "globus_ftp_control_local_prot"
    lassign [ globus_ftp_control_local_prot \
		  $handle \
		  $GLOBUS_FTP_CONTROL_PROTECTION_PRIVATE \
		 ] result
    puts_info 10 "\tresult $result"

    # globus_ftp_control_get_prot
    puts_info 10 "globus_ftp_control_get_prot"
    lassign [ globus_ftp_control_get_prot \
		  $handle \
		 ] result protection
    puts_info 10 "\tresult $result protection $protection"

    if { 0 } {
	delete_globus_ftp_control_dcau_subject_t $dcau_subject
    }
    delete_globus_ftp_control_dcau_t $dcau

    # globus_ftp_control_local_stru
    puts_info 10 "globus_ftp_control_local_stru"
    lassign [ globus_ftp_control_local_stru \
		  $handle \
		  $GLOBUS_FTP_CONTROL_STRUCTURE_FILE \
		 ] result
    puts_info 10 "\tresult $result"

    # globus_ftp_control_local_send_eof
    puts_info 10 "globus_ftp_control_local_send_eof"
    lassign [ globus_ftp_control_local_send_eof \
		  $handle \
		  $GLOBUS_TRUE \
		 ] result
    puts_info 10 "\tresult $result"

    if { 0 } {
	set cb_done 0
	lassign [ globus_ftp_control_data_force_close \
		      $handle \
		      { destroy_cb NULL } \
		     ] result
	puts_info 10 "\tresult: $result"
	vwait cb_done
    }
    
    if { 1 } {
	# globus_ftp_control_quit
	set cb_done 0
	puts_info 10 "globus_ftp_control_quit"
	lassign [ globus_ftp_control_quit \
		      $handle \
		      { response_cb NULL } \
		     ] result
	puts_info 10 "\tresult: $result"
	vwait cb_done
    }

    # globus_ftp_control_handle_destroy
    puts_info 10 "globus_ftp_control_handle_destroy"
    lassign [ globus_ftp_control_handle_destroy $handle ] result
    puts_info 10 "\tresult: $result"

    globus_module_deactivate $globus_i_ftp_control_module
} err ] } {
    err_handler $err
}    
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
