#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_ftp_control_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_ftp_control \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
set SECTION ftp_test
validate_section $SECTION [list GSIFTP BUFFERSIZE]
set host $TestEnv($SECTION,GSIFTP)
set buffersize $TestEnv($SECTION,BUFFERSIZE)

set SECTION general
validate_section $SECTION [list PORT]
set port $TestEnv($SECTION,PORT)

puts_info 10 "host $host port $port"
puts_info 10 "buffersize $buffersize"

verifyProxy

#------------------------------------------------------------------------
# Main
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "pid: [ pid ]"

    # Activate Globus FTP control module
    ftp_control_activation

    # Create Globus FTP control handle 
    set control_handle [ ftp_control_handle_initialization ]
    	
    # Connect to remote server
    ftp_control_connect $control_handle $host $port

    if { $connect_failed == 0 } {

	# Initialize local host & port information
	set addr [ ftp_control_host_port_init 42500 ]

	# Allocate DCAU resources
	set dcau [ new_globus_ftp_control_dcau_t ]
	set dcau_subject [ new_globus_ftp_control_dcau_subject_t ]
	
	# Data channel inialization
	data_channel_init $control_handle $dcau $dcau_subject
	
	ftp_control_send_port_cmd $control_handle $addr
	
	# Send LIST command
	set cmd "PWD"
	append cmd "\r\n"
	ftp_control_send_cmd $control_handle $cmd
	
	# Delete DCAU resources
	delete_globus_ftp_control_dcau_subject_t $dcau_subject
	delete_globus_ftp_control_dcau_t $dcau

	# Quit
	ftp_control_quit $control_handle
    }

    # Destroy control handle
    ftp_control_handle_destroy $control_handle

    # Deactivate Globus FTP control module
    ftp_control_deactivation
    
} err ] } {
    err_handler $err
}    
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
