#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_ftp_control_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_ftp_control \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
set SECTION ftp_test
validate_section $SECTION [list GSIFTP]
set host $TestEnv($SECTION,GSIFTP)

set SECTION general
validate_section $SECTION [list PORT]
set port $TestEnv($SECTION,PORT)

puts_info 10 "host $host port $port"

verifyServer $host $port

verifyProxy
#------------------------------------------------------------------------
# Quit response callback
#------------------------------------------------------------------------
proc quit_response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering quit_response_cb"

	puts_info 10 "user_parms $user_parms handle $handle err $err ftp_response $ftp_response"
	
	set ::cb_done 1
	
	puts_info 10 "leaving quit_response_cb"
    } err ] } {
	return -code error $err
    }	
}

#------------------------------------------------------------------------
# Response callback
#------------------------------------------------------------------------
proc connect_response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering response_cb"

	puts_info 10 "user_parms $user_parms"
	puts_info 10 "handle $handle"
	puts_info 10 "err $err"
	puts_info 10 "ftp_response $ftp_response"

	if { [string length $err] != 0 } {
	    set ::cb_done 1
	} else {
	    lassign [ globus_ftp_control_response_t_code_get \
			  $ftp_response \
			 ] code
	    puts_info 10 "code $code"
	    
	    if { $code == 220 } {
		puts_info 10 "globus_ftp_control_auth_info_init"
		puts_info 10 "\tgss_c_no_credential $::gss_c_no_credential"
		lassign [ globus_ftp_control_auth_info_init \
			      $::gss_c_no_credential \
			      $::GLOBUS_TRUE \
			      ":globus-mapping:" \
			      "" \
			      NULL \
			      "/DC=org/DC=doegrids/OU=Services/CN=tclproxy.ligo.caltech.edu" \
			     ] result auth_info
		puts_info 10 "\tresult $result auth_info $auth_info"

		if { 0 } {
		    puts_info 10 "globus_ftp_control_data_get_remote_hosts"
		    lassign [ globus_ftp_control_data_get_remote_hosts \
				  $handle \
				  $addr \
				 ] result addr_count
		    puts_info 10 "result $result addr_count $addr_count"
		}
		
		puts_info 10 "globus_ftp_control_command_init"
		lassign [ globus_ftp_control_command_init \
			      "PASV\r\n" \
			      $auth_info \
			     ] result src_command
		puts_info 10 "\tresult $result src_command $src_command"

		puts_info 10 "globus_ftp_control_command_init"
		lassign [ globus_ftp_control_command_init \
			      "SITE\r\n" \
			      $auth_info \
			     ] result dest_command
		puts_info 10 "\tresult $result dest_command $dest_command"

		puts_info 10 "globus_ftp_control_command_copy"
		lassign [ globus_ftp_control_command_copy \
			      $dest_command \
			      $src_command \
			     ] result
		puts_info 10 "\tresult $result"

		puts_info 10 "globus_ftp_control_command_destroy"
		lassign [ globus_ftp_control_command_destroy \
			      $src_command \
			     ] result
		puts_info 10 "\tresult $result"

		puts_info 10 "globus_ftp_control_command_destroy"
		lassign [ globus_ftp_control_command_destroy \
			      $dest_command \
			     ] result
		puts_info 10 "\tresult $result"
	    }
	}
	
	set ::cb_done 1

	puts_info 10 "leaving response_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Main
#------------------------------------------------------------------------
if { [ catch {
    globus_module_activate $globus_i_ftp_control_module

    puts_info 10 "globus_ftp_control_handle_init"
    lassign [ globus_ftp_control_handle_init \
		 ] result handle
    puts_info 10 "\thandle $handle"
    
    set cb_done 0
    puts_info 10 "globus_ftp_control_connect"
    lassign [ globus_ftp_control_connect \
		  $handle \
		  $host \
		  $port \
		  { connect_response_cb NULL } \
		 ] result
    puts_info 10 "\tresult $result"
    vwait cb_done
    
    set cb_done 0
    puts_info 10 "globus_ftp_control_quit"
    lassign [ globus_ftp_control_quit \
		  $handle \
		  { quit_response_cb NULL } \
		 ] result
    puts_info 10 "\tresult: $result"
    vwait cb_done
    
    
    puts_info 10 "globus_ftp_control_handle_destroy"
    lassign [ globus_ftp_control_handle_destroy $handle \
		 ] result
    puts_info 10 "\tresult: $result"
    
    globus_module_deactivate $globus_i_ftp_control_module
} err ] } {
    err_handler $err
}    
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
