#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

puts_info 10 "env(TOPSRCDIR) $env(TOPSRCDIR)"
puts_info 10 "env(SRCDIR) $env(SRCDIR)"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_ftp_control \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
set SECTION general
validate_section $SECTION [list PORT]
set port $TestEnv($SECTION,PORT)

#------------------------------------------------------------------------
# Stop callback
#------------------------------------------------------------------------
proc stop_cb { user_parms server_handle err } {
    if { [ catch {
	puts_info 10 "entering stop_cb"

	puts "user_parms $user_parms server_handle $server_handle err $err"

	set ::cb_done 1
	
	puts_info 10 "leaving stop_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Authenticate response callback
#------------------------------------------------------------------------
proc auth_resp_cb { user_parms handle err } {
    if { [ catch {
	puts_info 10 "entering auth_cb"

	puts "user_parms $user_parms server_handle $server_handle err $err"
	
	puts_info 10 "globus_ftp_control_server_stop"
	lassign [ globus_ftp_control_server_stop \
		      $listener_handle \
		      [ subst { stop_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult $result"
	
	puts_info 10 "leaving auth_cb"
    } err ] } {
	err_handler $err
    }
}

#------------------------------------------------------------------------
# Authenticate callback
#------------------------------------------------------------------------
proc auth_cb { user_parms handle err } {
    if { [ catch {
	puts_info 10 "entering auth_cb"

	if { [string length $err] == 0 } {
	    puts_info 10 "\tglobus_ftp_control_send_response"
	    lassign [ globus_ftp_control_send_response \
			  $handle \
			  "230 User %s logged on.\r\n" \
			  [ subst { auth_resp_cb NULL } ] \
			 ] result
	    puts_info 10 "\tresult $result"
	}
	
	puts_info 10 "leaving auth_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Response callback
#------------------------------------------------------------------------
proc response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering response_cb"

	upvar 0 $user_parms user_args
	puts "user_parms $user_parms handle $handle err $err ftp_response $ftp_response"

	if { [ string length $err ] == 0 } {
	    set auth_req [ GLOBUS_FTP_CONTROL_AUTH_REQ_GSSAPI |
			   GLOBUS_FTP_CONTROL_AUTH_REQ_USER |
			   GLOBUS_FTP_CONTROL_AUTH_REQ_PASS ]
	    
	    puts_info 10 "\tglobus_ftp_control_server_authenticate"
	    lassign [ globus_ftp_control_server_authenticate \
			  $handle \
			  $auth_req \
			  [ subst { auth_cb NULL } ] \
			 ] result
	    puts_info 10 "\tresult $result"
	}
	
	puts_info 10 "leaving response_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Accept callback
#------------------------------------------------------------------------
proc accept_cb { user_parms handle err } {
    if { [ catch {
	puts_info 10 "entering accept_cb"

	upvar 0 $user_parms user_args
	puts "user_parms $user_parms handle $handle err $err"

	if { [string length $err] == 0 } {
	    puts_info 10 "\tglobus_ftp_control_send_response"
	    lassign [ globus_ftp_control_send_response \
			  $handle \
			  "220 TclGlobus FTP Control Server\r\n" \
			  [ subst { response_cb NULL } ] \
			 ] result
	    puts_info 10 "\tresult $result"
	}

	puts_info 10 "leaving accept_cb"
    } err ] } {
	err_handler $err
    }
}

#------------------------------------------------------------------------
# Listen callback
#------------------------------------------------------------------------
proc listen_cb { user_parms listener_handle err } {
    if { [ catch {
	puts_info 10 "entering listen_cb"

	upvar 0 $user_parms user_args
	puts "user_parms $user_parms listener_handle $listener_handle err $err"

	if { [string length $err] == 0 } {
	    puts_info 10 "globus_ftp_control_handle_init"
	    lassign [ globus_ftp_control_handle_init \
			 ] result new_handle
	    puts_info 10 "\tresult $result new_handle $new_handle"
	    
	    puts_info 10 "\tglobus_ftp_control_server_accept"
	    lassign [ globus_ftp_control_server_accept \
			  $listener_handle \
			  $new_handle \
			  [ subst { accept_cb NULL } ] \
			 ] result
	    puts_info 10 "\tresult $result"
	}
	
	puts_info 10 "leaving listen_cb"
    } err ] } {
	err_handler $err
    }
}

#------------------------------------------------------------------------
# Main
#------------------------------------------------------------------------
if { [ catch {
    globus_module_activate $globus_i_ftp_control_module

    set listener_handle [ new_globus_ftp_control_server_t ]
    puts_info 10 "listener_handle $listener_handle"
    
    puts_info 10 "globus_ftp_control_server_handle_init"
    lassign [ globus_ftp_control_server_handle_init \
		  $listener_handle \
		 ] result
    puts_info 10 "\tresult $result"
    
    set cb_done 0
    set port 35000
    puts_info 10 "globus_ftp_control_server_listen"
    lassign [ globus_ftp_control_server_listen \
		  $listener_handle \
		  $port \
		  { listen_cb NULL } \
		 ] result assigned_port
    puts_info 10 "\tresult $result assigned_port $assigned_port"
    vwait cb_done

    puts_info 10 "globus_ftp_control_handle_destroy"
    lassign [ globus_ftp_control_server_handle_destroy \
		  $listener_handle \
		 ] result
    puts_info 10 "\tresult $result"

    delete_globus_ftp_control_server_t $listener_handle
    
    globus_module_deactivate $globus_i_ftp_control_module
} err ] } {
    err_handler $err
}    
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
