#------------------------------------------------------------------------
# Globus FTP Control module activation
#------------------------------------------------------------------------
proc ftp_control_activation { } {
    if { [ catch {
	puts_info 10 "entering ftp_control_activation"
	globus_module_activate $::globus_i_ftp_control_module
	puts_info 10 "leaving ftp_control_activation"
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Globus FTP Control module deactivation
#------------------------------------------------------------------------
proc ftp_control_deactivation { } {
    if { [ catch {
	puts_info 10 "entering ftp_control_deactivation"
	globus_module_deactivate $::globus_i_ftp_control_module
	puts_info 10 "leaving ftp_control_deactivation"
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Globus FTP Control handle initialization
#------------------------------------------------------------------------
proc ftp_control_handle_initialization { } {

    puts_info 10 "entering ftp_control_handle_initialization"
    
    puts_info 10 "globus_ftp_control_handle_init"
    lassign [ globus_ftp_control_handle_init \
		 ] result handle
    puts_info 10 "\thandle $handle"
    
    puts_info 10 "leaving ftp_control_handle_initialization"
    
    return $handle

}

#------------------------------------------------------------------------
# Globus FTP Control handle destroy
#------------------------------------------------------------------------
proc ftp_control_handle_destroy { handle } {
    if { [ catch {
	puts_info 10 "entering ftp_control_handle_destroy"
	
	puts_info 10 "globus_ftp_control_handle_destroy"
	lassign [ globus_ftp_control_handle_destroy \
		      $handle \
		     ] result
	puts_info 10 "\tresult: $result"

	puts_info 10 "leaving ftp_control_handle_destroy"
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Connect response callback
#------------------------------------------------------------------------
proc connect_response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering connect_response_cb"
	
	puts_info 10 "user_parms $user_parms"
	puts_info 10 "handle $handle"
	puts_info 10 "err $err"
	puts_info 10 "ftp_response $ftp_response"

	if { [ string length $err ] == 0 } {
	    	    
	    lassign [ globus_ftp_control_response_t_code_get \
			  $ftp_response \
			 ] ftp_response_code
	    puts_info 10 "ftp_response_code $ftp_response_code"

	    lassign [ globus_ftp_control_response_t_response_buffer_get \
			  $ftp_response \
			 ] ftp_response_buffer
	    puts_info 10 "ftp_response_buffer $ftp_response_buffer"
	    
	    lassign [ globus_ftp_control_response_t_response_length_get \
			  $ftp_response \
			 ] ftp_response_length
	    puts_info 10 "ftp_response_length $ftp_response_length"

	    lassign [ globus_ftp_control_response_t_response_buffer_size_get \
			  $ftp_response \
			 ] ftp_response_buffer_size
	    puts_info 10 "ftp_response_buffer_size $ftp_response_buffer_size"
	    
	    if { $ftp_response_code == 530 } {
		puts_info 10 "Not logged in" 
		set ::connect_done 1
		set ::connect_failed 1
		
	    } elseif { $ftp_response_code == 220 } {
		puts_info 10 "globus_ftp_control_auth_info_init"
		puts_info 10 "gss_c_no_credential $::gss_c_no_credential"
		lassign [ globus_ftp_control_auth_info_init \
			      $::gss_c_no_credential \
			      $::GLOBUS_TRUE \
			      ":globus-mapping:" \
			      "" \
			      NULL \
			      "/DC=org/DC=doegrids/OU=Services/CN=tclproxy.ligo.caltech.edu" \
			     ] result auth_info
		puts_info 10 "\tresult $result auth_info $auth_info"
		
		puts_info 10 "globus_ftp_control_authenticate"
		lassign [ globus_ftp_control_authenticate \
			      $handle \
			      $auth_info \
			      $::GLOBUS_TRUE \
			      [ subst { connect_response_cb NULL } ] \
			     ] result
		puts_info 10 "\tresult: $result"
		
	    } elseif { $ftp_response_code == 221 ||
		       $ftp_response_code == 230 ||
		       $ftp_response_code == 250 } {
		set ::connect_done 1
		
	    } elseif { $ftp_response_code >= 500 } {
		set ::connect_failed 1
		set ::connect_done 1
	    }
	    
	} else {
	    set ::connect_failed 1
	    set ::connect_done 1
	}
	
	puts_info 10 "leaving connect_response_cb"
    } err ] } {
	return -code error $err
    }	
}

#------------------------------------------------------------------------
# Connect to remote server
#------------------------------------------------------------------------
proc ftp_control_connect { handle host port } {
    if { [ catch {
	puts_info 10 "entering ftp_control_connect"

	puts_info 10 "handle $handle"
	
	set ::connect_done 0
	set ::connect_failed 0
	puts_info 10 "globus_ftp_control_connect"
	lassign [ globus_ftp_control_connect \
		      $handle \
		      $host \
		      $port \
		      [ subst { connect_response_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult $result"
	vwait ::connect_done

	puts_info 10 "leaving ftp_control_connect"
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Quit response callback
#------------------------------------------------------------------------
proc quit_response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering quit_response_cb"

	puts_info 10 "user_parms $user_parms"
	puts_info 10 "handle $handle"
	puts_info 10 "err $err"
	puts_info 10 "ftp_response $ftp_response"
	
	set ::quit_done 1
	
	puts_info 10 "leaving quit_response_cb"
    } err ] } {
	return -code error $err
    }	
}

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc ftp_control_quit { handle } {
    if { [ catch {
	puts_info 10 "entering ftp_control_quit"

	puts_info 10 "handle $handle"
	
	set ::quit_done 0
	puts_info 10 "globus_ftp_control_quit"
	lassign [ globus_ftp_control_quit \
		      $handle \
		      [ subst { quit_response_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
	vwait ::quit_done

	puts_info 10 "leaving ftp_control_quit"
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc ftp_control_host_port_init { port } {
    puts_info 10 "entering ftp_control_host_port_init"
    
    puts_info 10 "globus_ftp_control_host_port_init"
    lassign [ globus_ftp_control_host_port_init \
		  [info hostname] \
		  $port \
		 ] addr
    puts_info 10 "\taddr $addr"

    puts_info 10 "leaving ftp_control_host_port_init"
    
    return $addr
}

#------------------------------------------------------------------------
# Passive response callback
#------------------------------------------------------------------------
proc passive_response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering passive_response_cb"
	
	puts_info 10 "user_parms $user_parms"
	puts_info 10 "handle $handle"
	puts_info 10 "err $err"
	puts_info 10 "ftp_response $ftp_response"

	if { [ string length $err ] == 0 } {
	    lassign [ globus_ftp_control_response_t_code_get \
			  $ftp_response \
			 ] ftp_response_code
	    puts_info 10 "ftp_response_code $ftp_response_code"
	    
	    if { $ftp_response_code == 227 } {

		lassign [ globus_ftp_control_response_t_response_buffer_get \
			      $ftp_response \
			     ] ftp_response_buffer
		puts_info 10 "ftp_response_buffer $ftp_response_buffer"

		regexp "(\[0-9]{1,3}),(\[0-9]{1,3}),(\[0-9]{1,3}),(\[0-9]{1,3}),(\[0-9]{1,3}),(\[0-9]{1,3})" $ftp_response_buffer all first second third fourth fifth sixth
		
		puts_info 20  "$all \n $first \n $second \n $third \n $fourth \n $fifth \n $sixth"

		set hostname "$first.$second.$third.$fourth"
		puts_info 10 "hostname $hostname"
		
		set port [expr $fifth * 256 | $sixth]
		puts_info 10 "port $port"
		
		puts_info 10 "globus_ftp_control_host_port_init"
		lassign [ globus_ftp_control_host_port_init \
			      $hostname \
			      $port \
			     ] addr
		puts_info 10 "\taddr $addr"
		
		puts_info 10 "globus_ftp_control_local_port"
		lassign [ globus_ftp_control_local_port \
			      $handle \
			      $addr \
			     ] result
		puts_info 10 "\tresult $result"
		
		set ::pasv_cmd_done 1
		
	    } elseif { $ftp_response_code == 229 } {

		set ::pasv_cmd_done 1
		
	    } else {
		set ::pasv_cmd_done 1
	    }
	} else {
	    set ::pasv_cmd_done 1
	}
	
	puts_info 10 "leaving passive_response_cb"
    } err ] } {
	return -code error $err
    }	
}

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc ftp_control_send_pasv_cmd { handle } {
    if { [ catch {
	puts_info 10 "entering ftp_control_send_pasv_cmd"
	
	set ::pasv_cmd_done 0
	puts_info 10 "globus_ftp_control_send_command -> PASV"
	lassign [ globus_ftp_control_send_command \
		      $handle \
		      "PASV\r\n" \
		      [ subst { passive_response_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
	vwait ::pasv_cmd_done

	puts_info 10 "leaving ftp_control_send_pasv_cmd"
    } err ] } {
	return -code error $err
    }

}

#------------------------------------------------------------------------
# Data channel initialization
#------------------------------------------------------------------------
proc data_channel_init { handle dcau dcau_subject } {
    if { [ catch {
	puts_info 10 "entering data_channel_init"

	puts_info 10 "handle $handle"

	puts_info 10 "globus_ftp_control_dcau_subject_t_mode_set"
	globus_ftp_control_dcau_subject_t_mode_set \
	    $dcau_subject \
	    $::GLOBUS_FTP_CONTROL_DCAU_SELF
	
	puts_info 10 "globus_ftp_control_dcau_subject_t_subject_set"
	globus_ftp_control_dcau_subject_t_subject_set \
	    $dcau_subject \
	    $::GLOBUS_NULL
	
	puts_info 10 "globus_ftp_control_dcau_t_mode_set"
	globus_ftp_control_dcau_t_mode_set \
	    $dcau \
	    $::GLOBUS_FTP_CONTROL_DCAU_SELF
	
	puts_info 10 "globus_ftp_control_dcau_t_subject_set"
	globus_ftp_control_dcau_t_subject_set \
	    $dcau \
	    $dcau_subject
	
	puts_info 10 "globus_ftp_control_local_dcau"
	lassign [ globus_ftp_control_local_dcau \
		      $handle \
		      $dcau \
		      $::gss_c_no_credential \
		     ] result
	puts_info 10 "\tresult $result"
	
	puts_info 10 "globus_ftp_control_local_prot"
	lassign [ globus_ftp_control_local_prot \
		      $handle \
		      $::GLOBUS_FTP_CONTROL_PROTECTION_CLEAR \
		     ] result
	puts_info 10 "\tresult $result"
	
	puts_info 10 "globus_ftp_control_local_mode"
	lassign [ globus_ftp_control_local_mode \
		      $handle \
		      $::GLOBUS_FTP_CONTROL_MODE_STREAM \
		     ] result
	puts_info 10 "\tresult $result"
	
	puts_info 10 "globus_ftp_control_local_type"
	lassign [ globus_ftp_control_local_type \
		      $handle \
		      $::GLOBUS_FTP_CONTROL_TYPE_ASCII \
		      1 \
		     ] result
	puts_info 10 "\tresult $result"

	puts_info 10 "leaving data_channel_init"
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Response callback
#------------------------------------------------------------------------
proc response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering response_cb"

	puts_info 10 "user_parms $user_parms"
	puts_info 10 "handle $handle"
	puts_info 10 "err $err"
	puts_info 10 "ftp_response $ftp_response"
	
	if { [ string length $err ] == 0 } {
	    lassign [ globus_ftp_control_response_t_code_get \
			  $ftp_response \
			 ] code
	    puts_info 10 "code $code"

	    lassign [ globus_ftp_control_response_t_response_buffer_get \
			  $ftp_response \
			 ] ftp_response_buffer
	    puts_info 10 "ftp_response_buffer $ftp_response_buffer"
	}

	set ::cb_done 1
	
	puts_info 10 "leaving response_cb"
    } err ] } {
	return -code error $err
    }	
}    

#------------------------------------------------------------------------
# Send single command - CWD, DELE, MKD, RMD, RNFR, RNTO
#------------------------------------------------------------------------
proc ftp_control_send_single_command { handle cmd } {
    if { [ catch {
	puts_info 10 "entering ftp_control_send_single_command"

	puts_info 10 "handle $handle"
	puts_info 10 "cmd \"$cmd\""
	set ::cb_done 0
	lassign [ globus_ftp_control_send_command \
		      $handle \
		      $cmd \
		      [ subst { response_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
	vwait ::cb_done
	
	puts_info 10 "leaving create_remote_dirs"
    } err ] } {
	return -code error $err
    }	
}

#------------------------------------------------------------------------
# Data write callback
#------------------------------------------------------------------------
proc data_write_cb { user_parms handle err buffer length offset eof } {
    if { [ catch {
	puts_info 10 "entering data_write_cb"

	puts_info 10 "handle $handle"
	puts_info 10 "err $err"
	puts_info 10 "length $length"
	puts_info 10 "offset $offset"
	puts_info 10 "eof $eof"

	set ::data_done 1
	
	puts_info 10 "leaving data_write_cb"
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Connect write callback
#------------------------------------------------------------------------
proc connect_write_cb { user_parms handle stripe_ndx reused err } {
    if { [ catch {
	puts_info 10 "entering connect_write_cb"

	puts_info 10 "user_parms $user_parms"
	puts_info 10 "handle $handle"
	puts_info 10 "stripe_ndx $stripe_ndx"
	puts_info 10 "reused $reused"
	puts_info 10 "err $err"

	set srcdir [ file join [ file dirname [ pwd ] ] swig ]
	puts_info 10 "srcdir $srcdir"
	set srcfile [ file join $srcdir $::filename ]
	puts_info 10 "srcfile $srcfile"
	set fid [ open $srcfile r ]
	fconfigure $fid -buffering full
	
	if { [ fileType $srcfile ] == "binary executable elf" } {
	    fconfigure $fid -translation binary
	    fconfigure $fid -encoding binary
	}
	
	set offset [ file size $srcfile ]
	puts_info 10 "offset: $offset"

	while { ! [eof $fid] } {
	    set data [ read $fid $::buffersize ]
	    puts_info 10 "data size [string length $data]" 
	    
	    puts_info 10 "globus_ftp_control_data_write"
	    set ::data_done 0
	    lassign [ globus_ftp_control_data_write \
			  $handle \
			  $data \
			  $offset \
			  [eof $fid] \
			  [ subst { data_write_cb NULL } ] \
			 ] result
	    puts_info 10 "\tresult: $result"
	    vwait ::data_done
	}

	close $fid
	set ::transfer_done 1
		
	puts_info 10 "leaving connect_write_cb"
    } err ] } {
	return -code error $err
    }	    
}

#------------------------------------------------------------------------
# Store response callback
#------------------------------------------------------------------------
proc stor_response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering stor_response_cb"

	puts_info 10 "user_parms $user_parms"
	puts_info 10 "handle $handle"
	puts_info 10 "err $err"
	puts_info 10 "ftp_response $ftp_response"

	if { [string length $err] == 0 } {
	    lassign [ globus_ftp_control_response_t_code_get \
			  $ftp_response \
			 ] code
	    puts_info 10 "code $code"	    
	}
	
	#set ::cb_done 1
	
	puts_info 10 "leaving stor_response_cb"
    } err ] } {
	return -code error $err
    }	
}

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc ftp_control_send_stor_cmd { handle filename } {
    if { [ catch {
	puts_info 10 "entering ftp_control_send_stor_cmd"
	
	set cmd "STOR "
	append cmd $filename
	append cmd "\r\n"
	puts_info 10 "cmd \"$cmd\""
	
	set ::transfer_done 0
	puts_info 10 "globus_ftp_control_send_command => STOR"
	lassign [ globus_ftp_control_send_command \
		      $handle \
		      $cmd \
		      [ subst { stor_response_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
	
	puts_info 10 "globus_ftp_control_data_connect_write"
	lassign [ globus_ftp_control_data_connect_write \
		      $handle \
		      [ subst { connect_write_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
	vwait ::transfer_done

	puts_info 10 "leaving ftp_control_send_stor_cmd"
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Port response callback
#------------------------------------------------------------------------
proc port_response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering port_response_cb"

	if { [ string length $err ] == 0 } {
	    lassign [ globus_ftp_control_response_t_code_get \
			  $ftp_response \
			 ] code
	    puts_info 10 "code $code"
	}

	set ::port_done 1
	
	puts_info 10 "leaving port_response_cb"
    } err ] } {
	return -code error $err
    }	
}    

#------------------------------------------------------------------------
# Globus FTP Control - PORT command
#------------------------------------------------------------------------
proc ftp_control_send_port_cmd { handle addr } {
    if { [ catch {
	puts_info 10 "entering ftp_control_send_port_cmd"
	
	puts_info 10 "globus_ftp_control_local_pasv"
	lassign [ globus_ftp_control_local_pasv \
		      $handle \
		      $addr \
		     ] result nodeinfo
	puts_info 10 "\tresult $result nodeinfo $nodeinfo"
	
	puts_info 10 "globus_ftp_control_send_command => PORT"
	set ::port_done 0
	lassign [ globus_ftp_control_send_command \
		      $handle \
		      "PORT $nodeinfo\r\n" \
		      [ subst { port_response_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
	vwait ::port_done

	puts_info 10 "leaving ftp_control_send_port_cmd"
    } err ] } {
	return -code error $err
    }
}


#------------------------------------------------------------------------
# List response callback
#------------------------------------------------------------------------
proc cmd_response_cb { user_parms handle err ftp_response } {
    if { [ catch {
	puts_info 10 "entering cmd_response_cb"
	
	puts_info 10 "handle $handle"
	puts_info 10 "err $err"
	puts_info 10 "ftp_response $ftp_response"

	if { [string length $err] == 0 } {
	    lassign [ globus_ftp_control_response_t_code_get \
			  $ftp_response \
			 ] code
	    puts_info 10 "code $code"
	}
	
	puts_info 10 "leaving cmd_response_cb"
    } err ] } {
	return -code error $err
    }	
}

#------------------------------------------------------------------------
# Read callback
#------------------------------------------------------------------------
proc read_data_cb { user_parms handle err buffer end_of_line length offset } {
    if { [ catch {
	puts_info 10 "entering read_data_cb"
	
	puts_info 10 "handle $handle"
	puts_info 10 "err $err"
	puts_info 10 "end_of_line $end_of_line"
	puts_info 10 "length $length"
	puts_info 10 "offset $offset"

	#puts -nonewline $buffer
	
	if { $end_of_line == 1 } {
	    set ::transfer_done 1
	} else {
	    puts_info 10 "globus_ftp_control_data_read"
	    lassign [ globus_ftp_control_data_read \
			  $handle \
			  $::buffersize \
			  [ subst { read_data_cb NULL } ] \
			 ] result
	    puts_info 10 "\tresult: $result"
	}
	
	puts_info 10 "leaving read_data_cb"
    } err ] } {
	return -code error $err
    }	    
}

#------------------------------------------------------------------------
# Connect read callback
#------------------------------------------------------------------------
proc connect_read_data_cb { user_parms handle stripe_ndx reused err } {
    if { [ catch {
	puts_info 10 "entering connect_read_data_cb"

	puts_info 10 "user_parms $user_parms"
	puts_info 10 "handle $handle"
	puts_info 10 "stripe_ndx $stripe_ndx"
	puts_info 10 "reused $reused"
	puts_info 10 "err $err"

	puts_info 10 "globus_ftp_control_data_read"
	lassign [ globus_ftp_control_data_read \
		      $handle \
		      $::buffersize \
		      [ subst { read_data_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
		
	puts_info 10 "leaving connect_read_data_cb"
    } err ] } {
	return -code error $err
    }	    
}

#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc ftp_control_send_cmd { handle cmd } {
    if { [ catch {
	puts_info 10 "entering ftp_control_send_cmd"

	puts_info 10 "handle $handle"
	puts_info 10 "cmd $cmd"

	puts_info 10 "globus_ftp_control_send_command"
	lassign [ globus_ftp_control_send_command \
		      $handle \
		      $cmd \
		      [ subst { cmd_response_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
	
	set ::transfer_done 0
	puts_info 10 "globus_ftp_control_data_connect_read"
	lassign [ globus_ftp_control_data_connect_read \
		      $handle \
		      [ subst { connect_read_data_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
	vwait ::transfer_done

	puts_info 10 "leaving ftp_control_send_cmd"
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Read callback
#------------------------------------------------------------------------
proc read_file_cb { user_parms handle err buffer end_of_line length offset } {
    if { [ catch {
	puts_info 10 "entering read_file_cb"
	
	puts_info 10 "handle $handle"
	puts_info 10 "err $err"
	puts_info 10 "end_of_line $end_of_line"
	puts_info 10 "length $length"
	puts_info 10 "offset $offset"

	puts -nonewline $::fid $buffer
	
	if { $end_of_line == 1 } {
	    set ::transfer_done 1
	} else {
	    puts_info 10 "globus_ftp_control_data_read"
	    lassign [ globus_ftp_control_data_read \
			  $handle \
			  $::buffersize \
			  [ subst { read_file_cb NULL } ] \
			 ] result
	    puts_info 10 "\tresult: $result"
	}
	
	puts_info 10 "leaving read_file_cb"
    } err ] } {
	return -code error $err
    }	    
}

#------------------------------------------------------------------------
# Connect read callback
#------------------------------------------------------------------------
proc connect_read_file_cb { user_parms handle stripe_ndx reused err } {
    if { [ catch {
	puts_info 10 "entering connect_read_file_cb"

	puts_info 10 "user_parms $user_parms"
	puts_info 10 "handle $handle"
	puts_info 10 "stripe_ndx $stripe_ndx"
	puts_info 10 "reused $reused"
	puts_info 10 "err $err"

	puts_info 10 "globus_ftp_control_data_read"
	lassign [ globus_ftp_control_data_read \
		      $handle \
		      $::buffersize \
		      [ subst { read_file_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
		
	puts_info 10 "leaving connect_read_file_cb"
    } err ] } {
	return -code error $err
    }	    
}

#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc ftp_control_send_retr_cmd { handle filename } {
    if { [ catch {
	puts_info 10 "entering ftp_control_send_retr_cmd"

	puts_info 10 "handle $handle"
	puts_info 10 "filename: $filename"
	
	set cmd "RETR "
	append cmd $filename
	append cmd "\r\n"
	puts_info 10 "cmd \"$cmd\""

	puts_info 10 "globus_ftp_control_send_command"
	lassign [ globus_ftp_control_send_command \
		      $handle \
		      $cmd \
		      [ subst { cmd_response_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
	
	set ::transfer_done 0
	puts_info 10 "globus_ftp_control_data_connect_read"
	lassign [ globus_ftp_control_data_connect_read \
		      $handle \
		      [ subst { connect_read_file_cb NULL } ] \
		     ] result
	puts_info 10 "\tresult: $result"
	vwait ::transfer_done

	puts_info 10 "leaving ftp_control_send_retr_cmd"
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc ftp_control_file_put { } {	


}

#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc ftp_control_file_get { } {



}
