
#------------------------------------------------------------------------
# Construct GSIFTP URL
#------------------------------------------------------------------------
proc construct_gsiftp_url { gsiftp } {
    set url "gsiftp://"
    append url $gsiftp
    append url "/"
    append url tmp
    append url "/"
    append url $::env(USER)
	
    return $url
}
#------------------------------------------------------------------------
# The procedure to initialize the FTP test
#------------------------------------------------------------------------
proc ftp_client_init { } {
    if { [ catch {
	set SECTION ftp_test
	validate_section $SECTION [list BUFFERSIZE]
	set ::max_nbytes $::TestEnv($SECTION,BUFFERSIZE)
	
	# Activate Globus FTP Client module
	puts_info 10 "Activate FTP Client module"
	globus_module_activate $::globus_i_ftp_client_module
	
	# FTP Client Attributes
	set ::handle_attr 	[ new_p_globus_ftp_client_handleattr_t ]
	puts_info 10 "handle_attr: $::handle_attr"
	puts_info 10 "handle_attr value: [ p_globus_ftp_client_handleattr_t_value $::handle_attr]"
	
	set ::operation_attr 	[ new_p_globus_ftp_client_operationattr_t ]
	puts_info 10 "operation_attr: $::operation_attr"
	puts_info 10 "operation_attr value: [ p_globus_ftp_client_operationattr_t_value $::operation_attr]"
	
	# Attribute initialization
	puts_info 10 "globus_ftp_client_handleattr_init"
	globus_ftp_client_handleattr_init $::handle_attr 
	
	puts_info 10 "globus_ftp_client_operationattr_init"
	globus_ftp_client_operationattr_init $::operation_attr 
	
	# Handle creation
	set ::copy_info(handle)	[ new_p_globus_ftp_client_handle_t ]
	puts_info 10 "handle: $::copy_info(handle)"
	puts_info 10 "handle value: [ p_globus_ftp_client_handle_t_value $::copy_info(handle) ]"
	
	# Handle initialization
	puts_info 10 "globus_ftp_client_handle_init"
	globus_ftp_client_handle_init $::copy_info(handle) $::handle_attr

    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to initialize extra stuff for the FTP test
#------------------------------------------------------------------------
proc ftp_client_extra_init { } {
    if { [ catch {
	# Activate Globus FTP Client Debug Plugin module
	puts_info 10 "Activate FTP Client Debug Plugin module"
	globus_module_activate $::globus_i_ftp_client_debug_plugin_module

	# Plugin creation
	set ::plugin_t [ new_p_globus_ftp_client_plugin_t ]
	puts_info 10 "plugin_t: $::plugin_t"
	puts_info 10 "plugin_t value: [ p_globus_ftp_client_plugin_t_value $::plugin_t ]"
	
	# Plugin initialization
	puts_info 10 "globus_ftp_client_debug_plugin_init"
	globus_ftp_client_debug_plugin_init $::plugin_t NULL NULL

	# Allocate memory for restart marker
	set ::restart_marker_t     [ new_globus_ftp_client_restart_marker_t ]
	puts_info 10 "restart_marker_t: $::restart_marker_t"
    
	# globus_ftp_client_restart_marker_init()
	puts_info 10 "globus_ftp_client_restart_marker_init"
    	globus_ftp_client_restart_marker_init $::restart_marker_t
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to cleanup the FTP test
#------------------------------------------------------------------------
proc ftp_client_cleanup { } {
    if { [ catch {
	# Destroy handle
	puts_info 10 "globus_ftp_client_handle_destroy"
	globus_ftp_client_handle_destroy $::copy_info(handle) 
	
	# Destroy attributes
	puts_info 10 "globus_ftp_client_handleattr_destroy"
	globus_ftp_client_handleattr_destroy $::handle_attr 
	
	puts_info 10 "globus_ftp_client_operationattr_destroy"
	globus_ftp_client_operationattr_destroy $::operation_attr 

	# Free up dynamic allocation
	puts_info 10 "Reclaiming memory space"
	delete_p_globus_ftp_client_handle_t $::copy_info(handle)
	delete_p_globus_ftp_client_handleattr_t $::handle_attr
	delete_p_globus_ftp_client_operationattr_t $::operation_attr
	puts_info 10 "Done reclaiming memory space"
	
	# Deactivate Globus FTP Client module
	puts_info 10 "Deactivate FTP Client module"
	globus_module_deactivate $::globus_i_ftp_client_module
    } err ] } {
	return -code error $err
    }	
}
#------------------------------------------------------------------------
# The procedure to cleanup the FTP test
#------------------------------------------------------------------------
proc ftp_client_extra_cleanup { } {
    if { [ catch {
	# Destroy plugin
	puts_info 10 "globus_ftp_client_debug_plugin_destroy"
	globus_ftp_client_debug_plugin_destroy $::plugin_t
	
	# Destroy restart marker
	puts_info 10 "globus_ftp_client_restart_marker_destroy"
        globus_ftp_client_restart_marker_destroy $::restart_marker_t

	# Free up dynamic allocation
	puts_info 10 "Reclaiming memory space"
	delete_globus_ftp_client_restart_marker_t $::restart_marker_t
	delete_p_globus_ftp_client_plugin_t $::plugin_t
	puts_info 10 "Done reclaiming memory space"

	puts_info 10 "Deactivate FTP Client Debug Plugin module"
	globus_module_deactivate $::globus_i_ftp_client_debug_plugin_module
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Generic FTP complete callback
#------------------------------------------------------------------------
proc ftp_client_complete_cb { user_parms handle err } {
    if { [ catch {
	puts_info 20 "Entering ftp_client_complete_cb"
	puts_info 20 "handle: $handle"
	puts_info 20 "err: $err"
	
	# In wish, we need to trigger event loop
	if { [info commands wm] == "wm" } {
	    wm iconify .
	}

	set status 1
	if { [ string length $err ] != 0 } {
	    puts_info 20 "err: $err"
	    set ::global_err $err
	    set status 2
	} 
	set ::cb_done $status

	puts_info 20 "Leaving ftp_client_complete_cb"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Callback for globus_ftp_client_list and globus_ftp_client_verbose_list
#------------------------------------------------------------------------
proc ftp_client_dirlist_cb { user_parms handle err buffer length offset eof } {
    if { [ catch {
	puts_info 20 "Entering ftp_client_dirlist_cb"
	upvar 0 $user_parms user_args
	
	puts_info 20 "err: $err"
	puts_info 20 "buffer: $buffer"
	puts_info 20 "length: $length"
	puts_info 20 "offset: $offset"
	puts_info 20 "eof: $eof"

	# Append the buffer to output
	append ::listing_output $buffer
	
	puts_info 20 "globus_ftp_client_register_read"
	
	globus_ftp_client_register_read \
	    $user_args(handle) \
	    $::max_nbytes \
	    [ subst { ftp_client_dirlist_cb $user_parms } ] 
	
	puts_info 20 "Leaving ftp_client_dirlist_cb"
    
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Callback for globus_ftp_client_size
#------------------------------------------------------------------------
proc ftp_client_size_cb { user_parms handle err filesize } {
    if { [ catch {
	puts_info 20 "Entering ftp_client_size_cb"
	puts_info 20 "handle: $handle"
	puts_info 20 "err: $err"

	set status 1
	# Setting checksum value
	set ::filesize_value $filesize 
	if { [ string length $err ] != 0 } {
	    append ::global_err $err
	    set status 2
	} 
	# In wish, we need to trigger event loop
	if { [info commands wm] == "wm" } {
	    wm iconify .
	}

	set ::cb_done $status

	puts_info 20 "Leaving ftp_client_size_cb"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Callback for globus_ftp_client_cksm
#------------------------------------------------------------------------
proc ftp_client_cksm_cb { user_parms handle err cksm } {
    if { [ catch {
	puts_info 20 "Entering ftp_client_cksm_cb"
	puts_info 20 "handle: $handle"
	puts_info 20 "err: $err"

	set status 1
	# Setting checksum value
	set ::cksm_value $cksm 
	if { [ string length $err ] != 0 } {
	    append ::global_err $err
	    set status 2
	} 
	set ::cb_done $status
	
	puts_info 20 "Leaving ftp_client_cksm_cb"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Callback for globus_ftp_client_modification_time
#------------------------------------------------------------------------
proc ftp_client_mdtm_cb { user_parms handle err tv_sec tv_nsec } {
    if { [ catch {
	puts_info 20 "Entering ftp_client_mdtm_cb"
	puts_info 20 "handle: $handle"
	puts_info 20 "err: $err"

	set status 1
	set ::tv_sec_value $tv_sec
	set ::tv_nsec_value $tv_nsec

	if { [ string length $err ] != 0 } {
	    append ::global_err $err
	    set status 2
	}
	# In wish, we need to trigger event loop
	if { [info commands wm] == "wm" } {
	    wm iconify .
	}

	set ::cb_done $status
	puts_info 20 "Leaving ftp_client_mdtm_cb"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Read callback
#------------------------------------------------------------------------
proc ftp_client_read_cb { user_parms handle err buffer length offset eof } {
    if { [ catch {
	puts_info 20 "Entering ftp_client_read_cb"
	upvar 0 $user_parms user_args
	puts_info 20 "handle: $handle"
	puts_info 20 "err: $err"
	puts_info 20 "length of buffer: [string length $buffer]"
	puts_info 20 "length: $length"
	puts_info 20 "offset: $offset"
	puts_info 20 "eof: $eof"
	
	puts -nonewline $user_args(dst) $buffer
	flush $user_args(dst)
	if { $length == $::TestEnv($::SECTION,BUFFERSIZE) } {
	    puts_info 20 "globus_ftp_client_register_read"
	    globus_ftp_client_register_read \
		$user_args(handle) \
		$::max_nbytes \
		[ subst { ftp_client_read_cb $user_parms } ] 
	} 
	puts_info 20 "Leaving ftp_client_read_cb"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Write callback
#------------------------------------------------------------------------
proc ftp_client_write_cb { user_parms handle err buffer length offset eof } {
    if { [ catch {
	puts_info 20 "Entering ftp_client_write_cb"
	upvar 0 $user_parms user_args
	puts_info 20 "handle: $handle"
	puts_info 20 "err: $err"
	puts_info 20 "length of buffer: [string length $buffer]"
	puts_info 20 "length: $length"
	puts_info 20 "offset: $offset"
	puts_info 20 "eof: $eof"
	
	# Read the next data
	set data [ read $user_args(srcId) $::max_nbytes ]
	puts_info 20 "length of data: [ string length $data ]"
	
	if { [ string length $data ] > 0 } {
	    globus_ftp_client_register_write \
		$user_args(handle) \
		$data \
		$length \
		[ eof $user_args(srcId) ] \
		[ subst { ftp_client_write_cb $user_parms } ] 
	} 
	puts_info 20 "Leaving ftp_client_write_cb"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to report any FTP callback error message
#------------------------------------------------------------------------
proc ftp_client_callback_err_handler { } {
    switch -regexp -- $::global_err {
	{does not exist} {
	    puts_fail "Error: $::global_err"
	}
	
	{gss_init_sec_context failed} {
	    append message "Error: No valid proxy certificate. (" 
	    append message $::global_err
	    append message ")"
	    puts_fail $message
	    
	    ftp_client_cleanup

	    TestExit
	}
	
	{ the server responded } {
	    puts_info 20 "Error: $::global_err."
	}

	default {
	    puts_fail "Error: $::global_err."
	}
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_exists
#------------------------------------------------------------------------
proc ftp_client_exists { url } {
    if { [ catch {
	# vwait variable
	set ::cb_done	0
	
	# globus_ftp_client_exists
	puts_info 20 "globus_ftp_client_exists"
	puts_info 20 "url: $url"

	# globus_ftp_client_exists
	globus_ftp_client_exists \
	    $::copy_info(handle) \
	    $url \
	    $::operation_attr \
	    { ftp_client_complete_cb NULL }
	
	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"
	
	# We have an error coming back from ftp_client_complet_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	} 
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_mkdir
#------------------------------------------------------------------------
proc ftp_client_mkdir { url } {
    if { [ catch {
	# vwait variable
	set ::cb_done 0

	puts_info 20 "globus_ftp_client_mkdir"
	puts_info 20 "url: $url"

	# globus_ftp_client_mkdir
	globus_ftp_client_mkdir \
	    $::copy_info(handle) \
	    $url \
	    $::operation_attr \
	    { ftp_client_complete_cb NULL } 

	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"
    
	# We have an error coming back from ftp_client_complet_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	} 	
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_rmdir
#------------------------------------------------------------------------
proc ftp_client_rmdir { url } {
    if { [ catch {
	# vwait variable
	set ::cb_done 0

	puts_info 20 "globus_ftp_client_rmdir"
	puts_info 20 "url: $url"

	# globus_ftp_client_rmdir
	globus_ftp_client_rmdir \
	    $::copy_info(handle) \
	    $url \
	    $::operation_attr \
	    { ftp_client_complete_cb NULL }
	
	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"
	
	# We have an error coming back from ftp_client_complet_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	} 
    } err ] } {
	return -code error $err
    }	
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_delete
#------------------------------------------------------------------------
proc ftp_client_delete { url } {
    if { [ catch {
	# vwait variable
	set ::cb_done	0
    
	puts_info 20 "globus_ftp_client_delete"
	puts_info 20 "url: $url"

	# globus_ftp_client_delete
	globus_ftp_client_delete \
	    $::copy_info(handle) \
	    $url \
	    $::operation_attr \
	    { ftp_client_complete_cb NULL } 
	
	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"
	
	# We have an error coming back from complete_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	} 
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_list
#------------------------------------------------------------------------
proc ftp_client_list { url } {
    if { [ catch {
	# vwait variable
	set ::cb_done	0
	
	puts_info 20 "globus_ftp_client_list"
	puts_info 20 "url: $url"

	# globus_ftp_client_list
	lassign [ globus_ftp_client_list \
		      $::copy_info(handle) \
		      $url \
		      $::operation_attr \
		      { ftp_client_complete_cb NULL } 
		 ] result
	
	if { $result == $::GLOBUS_SUCCESS } {
	    puts_info 20 "globus_ftp_client_register_read"
	    
	    # globus_ftp_client_register_read
	    globus_ftp_client_register_read \
		$::copy_info(handle) \
		$::max_nbytes \
		{ ftp_client_dirlist_cb ::copy_info }
	}
	
	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"
	
	# We have an error coming back from complete_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	} 
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_verbose_list
#------------------------------------------------------------------------
proc ftp_client_verbose_list { url } {
    if { [ catch {
	# vwait variable
	set ::cb_done	0
	
	puts_info 20 "globus_ftp_client_verbose_list"
	puts_info 20 "url: $url"

	# globus_ftp_client_verbose_list
	lassign [ globus_ftp_client_verbose_list \
		      $::copy_info(handle) \
		      $url \
		      $::operation_attr \
		      { ftp_client_complete_cb NULL } \
		     ] result
	
	if { $result == $::GLOBUS_SUCCESS } {
	    puts_info 20 "globus_ftp_client_register_read"

	    # globus_ftp_client_register_read
	    globus_ftp_client_register_read \
		$::copy_info(handle) \
		$::max_nbytes \
		{ ftp_client_dirlist_cb ::copy_info } 
	}

	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"

	# We have an error coming back from complete_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	} 
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_chmod
#------------------------------------------------------------------------
proc ftp_client_chmod { url } {
    if { [ catch {
	puts_info 10 "Entering ftp_client_chmod"
	# vwait variable
	set ::cb_done	0
	
	# globus_ftp_client_chmod
	puts_info 20 "globus_ftp_client_chmod"
	puts_info 20 "url: $url"

	globus_ftp_client_chmod \
	    $::copy_info(handle) \
	    $url \
	    [ expr $::GLOBUS_IO_FILE_IRUSR | \
		  $::GLOBUS_IO_FILE_IWUSR | \
		  $::GLOBUS_IO_FILE_IRGRP | \
		  $::GLOBUS_IO_FILE_IWGRP | \
		  $::GLOBUS_IO_FILE_IROTH | \
		  $::GLOBUS_IO_FILE_IWOTH ] \
	    $::operation_attr \
	    { ftp_client_complete_cb NULL } 
	
	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"
	
	# We have an error coming back from ftp_client_complet_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	}
	puts_info 10 "Leaving ftp_client_chmod"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_get
#------------------------------------------------------------------------
proc ftp_client_get { url } {
    if { [ catch {
	puts_info 10 "Entering ftp_client_get"
	# Get the file
	set ::cb_done	0
	
	puts_info 20 "globus_ftp_client_get"
	puts_info 20 "url: $url"
	
	# globus_ftp_client_get
	globus_ftp_client_get \
	    $::copy_info(handle) \
	    $url \
	    $::operation_attr \
	    NULL \
	    { ftp_client_complete_cb NULL } 

	# Determine the file type
	lassign [  globus_ftp_client_operationattr_get_type \
		       $::operation_attr \
		      ] result filetype
	
	puts_info 20 "globus_ftp_client_operationattr_get_type"
	puts_info 20 "result: $result"
	puts_info 20 "filetype: $filetype"
	
	if { [string match $filetype "GLOBUS_FTP_CONTROL_TYPE_IMAGE"] } {
	    fconfigure $::copy_info(dst) -translation binary -encoding binary
	}

	# Get the file we've just transferred from an FTP server
	if { $result == $::GLOBUS_SUCCESS } {
	    puts_info 20 "globus_ftp_client_register_read"
	    
	    # globus_ftp_client_register_read
	    globus_ftp_client_register_read \
		$::copy_info(handle) \
		$::max_nbytes \
		{ ftp_client_read_cb ::copy_info } 
	}
	
	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"

	# We have an error coming back from ftp_client_complete_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	}
	puts_info 10 "Leaving ftp_client_get"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_put
#------------------------------------------------------------------------
proc ftp_client_put { url } {
    if { [ catch {
	puts_info 10 "Entering ftp_client_put"
	# globus_ftp_client_put starts the protocol exchange on the control
	# channel. Note that this does NOT start moving data over the data
	# channel.
	set ::cb_done 0
	puts_info 20 "url: $url"
	lassign [ globus_ftp_client_put \
		      $::copy_info(handle) \
		      $url \
		      $::operation_attr \
		      NULL \
		      { ftp_client_complete_cb NULL } ] result
	puts_info 20 "globus_ftp_client_put returns $result"

	# This is where the data movement over the data channel is
	# initiated. You read a buffer, and call register_write. This
	# is an asynch call which returns immediately. 
	if { $result == $::GLOBUS_SUCCESS } {
	    set buffer [ read $::copy_info(srcId) $::max_nbytes ]
	    puts_info 20 "globus_ftp_client_register_write"
	    puts_info 20 "length: [ string bytelength $buffer ]"
	    globus_ftp_client_register_write \
		$::copy_info(handle) \
		$buffer \
		[ string bytelength $buffer ] \
		[ eof $::copy_info(srcId) ] \
		{ ftp_client_write_cb ::copy_info } 
	}

	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done

	# We have an error coming back from ftp_client_complete_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	}
	puts_info 10 "Leaving ftp_client_put"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_size
#------------------------------------------------------------------------
proc ftp_client_size { url } {
    if { [ catch {
	puts_info 10 "Entering ftp_client_size"
	# vwait variable
	set ::cb_done	0
	
	puts_info 20 "globus_ftp_client_size"
	puts_info 20 "url: $url"

	# globus_ftp_client_size
	globus_ftp_client_size \
	    $::copy_info(handle) \
	    $url \
	    $::operation_attr \
	    { ftp_client_size_cb NULL } 
	
	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"
	
	# We have an error coming back from complete_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	}
	puts_info 10 "Leaving ftp_client_size"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_cksm
#------------------------------------------------------------------------
proc ftp_client_cksm { url } {
    if { [ catch {
	# vwait variable
	set ::cb_done	0
	
	#globus_ftp_client_cksm
	puts_info 20 "globus_ftp_client_cksm"
	puts_info 20 "url: $url"

	globus_ftp_client_cksm \
	    $::copy_info(handle) \
	    $url \
	    $::operation_attr \
	    0 \
	    -1 \
	    MD5 \
	    { ftp_client_cksm_cb NULL }
	
	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"
	
	# We have an error coming back from complete_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	} 
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_modification_time
#------------------------------------------------------------------------
proc ftp_client_modification_time { url } {
    if { [ catch {
	puts_info 10 "Entering ftp_client_modification_time"
	# vwait variable
	set ::cb_done	0

	# globus_ftp_client_modification_time 
	puts_info 20 "globus_ftp_client_modification_time"
	puts_info 20 "url: $url"
	
	# globus_ftp_client_modification_time
	globus_ftp_client_modification_time \
	    $::copy_info(handle) \
	    $url \
	    $::operation_attr \
	    { ftp_client_mdtm_cb NULL } 
        
	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"
	
	# We have an error coming back from complete_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	}
	puts_info 10 "Leaving ftp_client_modification_time"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_move
#------------------------------------------------------------------------
proc ftp_client_move { oldUrl newUrl } {
    if { [ catch {
	puts_info 10 "Entering ftp_client_move"
	# vwait variable
	set ::cb_done 0

	# globus_ftp_client_modification_time 
	puts_info 20 "globus_ftp_client_move"
	puts_info 20 "oldUrl: $oldUrl"
	puts_info 20 "newUrl: $newUrl"
	
	# globus_ftp_client_move
	globus_ftp_client_move \
	    $::copy_info(handle) \
	    $oldUrl \
	    $newUrl \
	    $::operation_attr \
	    { ftp_client_complete_cb NULL }

	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"

	# We have an error coming back from complete_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	}
	puts_info 10 "Leaving ftp_client_move"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# The procedure to execute globus_ftp_client_third_party_transfer
#------------------------------------------------------------------------
proc ftp_client_third_party_transfer { srcUrl dstUrl } {
    if { [ catch {
	# vwait variable
	set ::cb_done 0

	# globus_ftp_client_modification_time 
	puts_info 20 "globus_ftp_client_third_party_transfer"
	puts_info 20 "srcUrl: $srcUrl"
	puts_info 20 "dstUrl: $dstUrl"

	# globus_ftp_client_third_party_transfer
	globus_ftp_client_third_party_transfer \
	    $::copy_info(handle) \
	    $srcUrl \
	    $::operation_attr \
	    $dstUrl \
	    $::operation_attr \
	    NULL \
	    { ftp_client_complete_cb NULL }
	
	# Go into the tcl event loop waiting for thread callbacks
	puts_info 20 "Go into the Tcl event loop waiting for thread callbacks"
	vwait ::cb_done
	puts_info 20 "Done with Tcl event loop"

	# We have an error coming back from complete_cb proc
	if { $::cb_done == 2 } {
	    ftp_client_callback_err_handler
	} 
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc create_remote_directory { remote_topdir remote_subdir } {
    if { [ catch {
	puts_info 10 "Entering create_remote_directory"
	# globus_ftp_client_mkdir
	set eventId [ after $::timeout_value set ::cb_done 2 ]
	ftp_client_mkdir $remote_topdir
	after cancel $eventId

	set eventId [ after $::timeout_value set ::cb_done 2 ]
	ftp_client_mkdir $remote_subdir
	after cancel $eventId
	puts_info 10 "Leaving create_remote_directory"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc remove_remote_directory { remote_topdir remote_subdir } {
    if { [ catch {
	puts_info 10 "Entering remove_remote_directory"
	# Remove remote directory
	ftp_client_rmdir $remote_subdir
	ftp_client_rmdir $remote_topdir
	puts_info 10 "Leaving remove_remote_directory"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc remove_remote_files { remote_subdir } {
    if { [ catch {
	# Delete all remote files 
	foreach srcfile $::filelist {
	    if { [file isdirectory $srcfile] } {
		continue
	    }
	    
	    set f [ file tail $srcfile ]
	    set srcfile $remote_subdir
	    append srcfile "/"
	    append srcfile $f
	    puts_info 10 "srcfile: $srcfile"
	    
	    ftp_client_delete $srcfile 
	}
    } err ] } {
	return -code error $err
    }				
}
#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc transfer_files_to_remote_site { remote_subdir } {
    if { [ catch {
	foreach srcfile $::filelist {
	    if { [file isdirectory $srcfile] } {
		continue
	    }
	    puts_info 10 "srcfile: $srcfile"
	    
	    # Open srcfile
	    set ::copy_info(srcId) [ open $srcfile r ]
	    
	    set filetype [ fileType $srcfile ]
	    if { [ fileType $srcfile ] == "binary executable elf" } {
		fconfigure $::copy_info(srcId) -translation binary -encoding binary
	    }
	    
	    # Construct dstfile
	    set f [ file tail $srcfile ]
	    set dstfile $remote_subdir
	    append dstfile "/"
	    append dstfile $f
	    puts_info 10 "dstfile: $dstfile"
	    
	    # globus_ftp_client_put
	    ftp_client_put $dstfile
	    
	    # Close srcfile
	    close $::copy_info(srcId)
	}
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc transfer_files_from_remote_site { remote_subdir } {
    if { [ catch {
	foreach file $::filelist {
	    if { [file isdirectory $file] } {
		continue
	    }
	    
	    set f [ file tail $file ]
	    set srcfile $remote_subdir
	    append srcfile "/"
	    append srcfile $f
	    puts_info 10 "srcfile: $srcfile"
	    
	    set dstfile [ file join $::localdir $f ]
	    puts_info 10 "dstfile: $dstfile"
	    
	    set ::copy_info(dst) [ open $dstfile w 0600 ]
	    
	    # globus_ftp_client_get
	    ftp_client_get $srcfile
	    
	    close $::copy_info(dst)
	}
    } err ] } {
	return -code error $err
    }		
}
#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
