#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_gass_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_gass_cache } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

set SECTION gass_cache
validate_section $SECTION [list FILENAME]

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    #--------------------------------------------------------------------
    # GASS Cache module activation
    #--------------------------------------------------------------------
    puts_info 10 "Activate GASS Cache module"
    globus_module_activate $globus_i_gass_cache_module

    set cache_directory_path [ file join [ file dirname [ pwd ] ] $env(USER) ]
    puts_info 10 "cache_directory_path $cache_directory_path"

    #--------------------------------------------------------------------
    # globus_gass_cache_open
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_open"
    lassign [ globus_gass_cache_open \
		  $cache_directory_path \
		 ] result cache_handle
    puts_info 10 "\tresult: $result"
    puts_info 10 "\tcache handle: $cache_handle"

    #--------------------------------------------------------------------
    # globus_gass_cache_add
    # globus_gass_cache_add_done
    #--------------------------------------------------------------------
    set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
    puts_info 10 "sourcedir: $sourcedir"

    set sourcefile file://
    append sourcefile [ file join $sourcedir $TestEnv($SECTION,FILENAME) ]
    puts_info 10 "sourcefile: $sourcefile"
    
    puts_info 10 "globus_gass_cache_add"
    lassign [ globus_gass_cache_add \
		  $cache_handle \
		  $sourcefile \
		  $GLOBUS_NULL \
		  $GLOBUS_TRUE \
		 ] result timestamp local_filename
    puts_info 10 "\tresult: $result"
    puts_info 10 "\ttimestamp: $timestamp"
    puts_info 10 "\tlocal_filename: $local_filename"

    if { $result == 1 } {
	puts_info 10 "globus_gass_cache_add_done"
	lassign [ globus_gass_cache_add_done \
		      $cache_handle \
		      $sourcefile \
		      $GLOBUS_NULL \
		      $timestamp \
		     ] result
	puts_info 10 "\tresult: $result"
    }

    #--------------------------------------------------------------------
    # globus_gass_cache_query
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_query"
    lassign [ globus_gass_cache_query \
		  $cache_handle \
		  $sourcefile \
		  $GLOBUS_NULL \
		  $GLOBUS_TRUE \
		 ] result timestamp local_filename is_locked
    puts_info 10 "\tresult: $result"
    puts_info 10 "\ttimestamp: $timestamp"
    puts_info 10 "\tlocal_filename: $local_filename"
    puts_info 10 "\tis_locked: $is_locked"

    #--------------------------------------------------------------------
    # globus_gass_cache_get_dirs
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_get_dirs"
    lassign [ globus_gass_cache_get_dirs \
		  $cache_handle \
		  $sourcefile \
		  $GLOBUS_NULL \
		 ] result global_root local_root tmp_root log_root global_dir local_dir
    puts_info 10 "\tresult: $result"
    puts_info 10 "\tglobal_root: $global_root"
    puts_info 10 "\tlocal_root: $local_root"
    puts_info 10 "\ttmp_root: $tmp_root"
    puts_info 10 "\tlog_root: $log_root"
    puts_info 10 "\tglobal_dir: $global_dir"
    puts_info 10 "\tlocal_dir: $local_dir"

    #--------------------------------------------------------------------
    # globus_gass_cache_get_cache_dir
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_get_cache_dir"
    lassign [ globus_gass_cache_get_cache_dir \
		  $cache_handle \
		 ] result cache_dir
    puts_info 10 "\tresult: $result"
    puts_info 10 "\tcache_dir: $cache_dir"
    
    #--------------------------------------------------------------------
    # globus_gass_cache_get_cache_type_string
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_get_cache_type_string"
    lassign [ globus_gass_cache_get_cache_type_string \
		  $cache_handle \
		 ] result cache_type
    puts_info 10 "\tresult: $result"
    puts_info 10 "\tcache_type: $cache_type"

    #--------------------------------------------------------------------
    # globus_gass_cache_mangle_url
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_mangle_url"
    lassign [ globus_gass_cache_mangle_url \
		  $cache_handle \
		  $sourcefile \
		 ] result mangled_url length
    puts_info 10 "\tresult: $result"
    puts_info 10 "\tmangled_url: $mangled_url"
    puts_info 10 "\tlength: $length"
    
    #--------------------------------------------------------------------
    # globus_gass_cache_mangle_tag
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_mangle_tag"
    lassign [ globus_gass_cache_mangle_tag \
		  $cache_handle \
		  $GLOBUS_NULL \
		 ] result mangled_tag length
    puts_info 10 "\tresult: $result"
    puts_info 10 "\tmangled_tag: $mangled_tag"
    puts_info 10 "\tlength: $length"
     
    #--------------------------------------------------------------------
    # globus_gass_cache_delete_start
    # globus_gass_cache_delete
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_delete_start"
    lassign [ globus_gass_cache_delete_start \
		  $cache_handle \
		  $sourcefile \
		  $GLOBUS_NULL \
		 ] result timestamp
    puts_info 10 "\tresult: $result"
    puts_info 10 "\ttimestamp: $timestamp"
    
    if { $result == $GLOBUS_SUCCESS } {
	puts_info 10 "globus_gass_cache_delete"
	lassign [ globus_gass_cache_delete \
		      $cache_handle \
		      $sourcefile \
		      $GLOBUS_NULL \
		      $timestamp \
		      $GLOBUS_TRUE \
		     ] result
	puts_info 10 "\tresult: $result"
    }

    #--------------------------------------------------------------------
    # globus_gass_cache_cleanup_tag
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_cleanup_tag"
    lassign [ globus_gass_cache_cleanup_tag \
		  $cache_handle \
		  $sourcefile \
		  $GLOBUS_NULL \
		 ] result
    puts_info 10 "\tresult: $result"

    #--------------------------------------------------------------------
    # globus_gass_cache_cleanup_tag_all
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_cleanup_tag_all"
    lassign [ globus_gass_cache_cleanup_tag_all \
		  $cache_handle \
		  $GLOBUS_NULL \
		 ] result
    puts_info 10 "\tresult: $result"
    
    #--------------------------------------------------------------------
    # globus_gass_cache_close
    #--------------------------------------------------------------------
    puts_info 10 "globus_gass_cache_close"
    lassign [ globus_gass_cache_close \
		  $cache_handle \
		 ] result
    puts_info 10 "\tresult: $result"

    #--------------------------------------------------------------------
    # file delete
    #--------------------------------------------------------------------
    file delete -force $cache_directory_path
    
    #--------------------------------------------------------------------
    # GASS module deactivation
    #--------------------------------------------------------------------
    puts_info 10 "Deactivate GASS Cache module"
    globus_module_deactivate $globus_i_gass_cache_module

} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit

