#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_gass_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_gass_cache } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    #--------------------------------------------------------------------
    # GASS Cache module activation
    #--------------------------------------------------------------------
    puts_info 10 "Activate GASS Cache module"
    globus_module_activate $globus_i_gass_cache_module

    set cache_directory_path [ file join [ file dirname [ pwd ] ] $env(USER) ]
    puts_info 10 "cache_directory_path $cache_directory_path"
    
    puts_info 10 "globus_gass_cache_open"
    lassign [ globus_gass_cache_open \
		  $cache_directory_path \
		 ] result cache_handle
    puts_info 10 "\tresult: $result"
    puts_info 10 "\tcache handle: $cache_handle"
    
    puts_info 10 "globus_gass_cache_close"
    lassign [ globus_gass_cache_close \
		  $cache_handle \
		 ] result
    puts_info 10 "\tresult: $result"

    #--------------------------------------------------------------------
    # file delete
    #--------------------------------------------------------------------
    file delete -force $cache_directory_path

    #--------------------------------------------------------------------
    # GASS module deactivation
    #--------------------------------------------------------------------
    puts_info 10 "Deactivate GASS Cache module"
    globus_module_deactivate $globus_i_gass_cache_module

} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit

