#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit

# In wish, we don't need to display the window
# But, we need to send it out of the visible part of the screen
if { [info commands wm] == "wm" } {
    wm iconify .
    wm geometry . 1x1+4000+4000
}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_gass_utils.tcl"
source "$env(SRCDIR)/test_ftp_utils.tcl"

verifyProxy

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_gass_copy \
		  globus_gass_transfer \
		  globus_ftp_client \
		  globus_io } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

set SECTION general
validate_section $SECTION [list PORT]
set port $TestEnv($SECTION,PORT)

set SECTION gass_copy
validate_section $SECTION [list GSIFTP]
puts_info 10 "GSIFTP: $TestEnv($SECTION,GSIFTP)"

set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
puts_info 10 "sourcedir: $sourcedir"

set localdir [file join /tmp \
		  [file join "$env(USER)" \
		       [constructOutputDirectory $argv0]]]
puts_info 10 "localdir: $localdir"

verifyServer $TestEnv($SECTION,GSIFTP) $port

#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc done_callback { user_parms handle err } {
    if { [ catch {
	puts_info 10 "entering done_callback"
	# In wish, we need to trigger event loop
	if { [info commands wm] == "wm" } {
	    wm iconify .
	}
	set ::cb_done 1
	puts_info 10 "leaving done_callback"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# GASS Copy Handle to URL (blocking)
#------------------------------------------------------------------------
proc gass_copy_handle_to_url { read_cs write_cs } {
    if { [ catch {
	# Create read handle
	set readhandle  [ new_p_globus_io_handle_t ]
	
	# globus_io_file_open() for reading
	puts_info 10 "globus_io_file_open"
	globus_io_file_open \
	    $read_cs \
	    $::GLOBUS_IO_FILE_RDONLY \
	    0 \
	    NULL \
	    $readhandle

	# Test sending of the data file
	puts_info 10 "Executing globus_gass_copy_handle_to_url"
	set ::cb_done 0
	globus_gass_copy_register_handle_to_url \
	    $::gh \
	    $readhandle \
	    $write_cs \
	    $::dstAttr \
	    { done_callback NULL }
	vwait ::cb_done

	# Close read handle
	puts_info 10 "globus_io_close"
	globus_io_close $readhandle

	# Release memory space
	delete_p_globus_io_handle_t $readhandle
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# GASS Copy URL to Handle (blocking)
#------------------------------------------------------------------------
proc gass_copy_url_to_handle { read_cs write_cs } {
    if { [ catch {
	# Create write handle
	set writehandle [ new_p_globus_io_handle_t ]
	# Set write flags and mode
	set write_flags [ expr $::GLOBUS_IO_FILE_CREAT | \
			      $::GLOBUS_IO_FILE_TRUNC | \
			      $::GLOBUS_IO_FILE_WRONLY ]
	
	set write_mode [ expr $::GLOBUS_IO_FILE_IRUSR | \
			     $::GLOBUS_IO_FILE_IWUSR ]

	# globus_io_file_open() for writing
	globus_io_file_open \
	    $write_cs \
	    $write_flags \
	    $write_mode \
	    NULL \
	    $writehandle

	# Test retrieving of the data file
	set ::cb_done 0
	globus_gass_copy_register_url_to_handle \
	    $::gh \
	    $read_cs \
	    $::srcAttr \
	    $writehandle \
	    { done_callback NULL }
	vwait ::cb_done
	
	# Close write handle
	puts_info 10 "globus_io_close"
	globus_io_close $writehandle

	# Release memory space
	delete_p_globus_io_handle_t $writehandle
    } err ] } {
	return -code error $err
    }	
}
#------------------------------------------------------------------------
# Run test loop
#------------------------------------------------------------------------
proc run_test { } {
    if { [ catch {
	# Create local temporary directory
	copy_init $::localdir

	# Create remote top directory
	globus_gass_copy_mkdir \
	    $::gh \
	    $::remote_topdir \
	    $::dstAttr
		
	# Create remote sub-directory
	globus_gass_copy_mkdir \
	    $::gh \
	    $::remote_subdir \
	    $::dstAttr
	
	# Do globus_gass_copy operation
	foreach orig_file $::filelist {
	    if { [file isdirectory $orig_file] } {
		continue
	    }
	    set remote_file $::remote_subdir
	    append remote_file "/"
	    append remote_file [file tail $orig_file]

	    # globus_gass_copy_handle_to_url
	    puts_info 10 "orig_file: $orig_file"
	    puts_info 10 "remote_file: $remote_file"
	    gass_copy_handle_to_url $orig_file $remote_file

	    # globus_gass_copy_url_to_handle
	    set target_file [ file join $::localdir [ file tail $orig_file ] ]
	    puts_info 10 "target_file: $target_file"
	    gass_copy_url_to_handle $remote_file $target_file
	    
	    # MD5 checksum
	    check_md5sum $orig_file $target_file
	}

	# Remove remote files using FTP Client API
	foreach file $::filelist {
	    if { [file isdirectory $file] } {
		continue
	    }
	    set f [ file tail $file ]
	    set file $::remote_subdir
	    append file "/"
	    append file $f
	    puts_info 10 "file: $file"
	    # File deletion
	    ftp_client_delete $file 
	}
	
	# Remove remote directories using FTP Client API
	ftp_client_rmdir $::remote_subdir
	ftp_client_rmdir $::remote_topdir

	# Remove local temporary directory
	copy_deinit $::localdir
    } err ] } {
	return -code error $err
    }		
}	
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # GASS Transfer module activation
    gass_module_activation

    # GASS Copy initialization
    gass_copy_init
    
    # Activate Globus I/O module
    puts_info 10 "Activate I/O module"
    globus_module_activate $globus_l_io_module

    # FTP initialization step
    ftp_client_init
    
    # Get file list
    set filelist [ constructFileList ]
    puts_info 10 "Number of files: [ llength $filelist ]"
    
    # Remote top directory
    set remote_topdir [ construct_gsiftp_url $TestEnv($SECTION,GSIFTP) ]
    puts_info 10 "remote_topdir: $remote_topdir"
    
    # Remote sub directory
    set remote_subdir $remote_topdir
    append remote_subdir "/"
    append remote_subdir [constructOutputDirectory $::argv0]
    puts_info 10 "remote_subdir: $remote_subdir"
    
    # Run the test in infinite loop mode or not
    if { [IsInfinite] == 1 } {
	while 1 {
	    run_test
	}
    } else {
	run_test
    }
    
    # FTP cleanup step
    ftp_client_cleanup

    # Deactivate Globus I/O module
    puts_info 10 "Deactivate I/O module"
    globus_module_deactivate $globus_l_io_module

    # GASS Copy cleanup
    gass_copy_cleanup

    # GASS Transfer module deactivation
    gass_module_deactivation    
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
