#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit

# In wish, we don't need to display the window
# But, we need to send it out of the visible part of the screen
if { [info commands wm] == "wm" } {
    wm iconify .
    wm geometry . 1x1+4000+4000
}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_gass_utils.tcl"
source "$env(SRCDIR)/test_ftp_utils.tcl"

verifyProxy

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io \
		  globus_gass_copy \
		  globus_gass_transfer \
		  globus_ftp_client } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

set SECTION general
validate_section $SECTION [list PORT]
set port $TestEnv($SECTION,PORT)

set SECTION gass_copy
validate_section $SECTION [list GSIFTP]
puts_info 10 "GSIFTP: $TestEnv($SECTION,GSIFTP)"

set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
puts_info 10 "sourcedir: $sourcedir"

set localdir [file join /tmp \
		  [file join "$env(USER)" \
		       [constructOutputDirectory $argv0]]]
puts_info 10 "localdir: $localdir"

verifyServer $TestEnv($SECTION,GSIFTP) $port

#------------------------------------------------------------------------
# 
#------------------------------------------------------------------------
proc done_callback { user_parms handle err } {
    if { [ catch {
	puts_info 10 "entering done_callback"
	# In wish, we need to trigger event loop
	if { [info commands wm] == "wm" } {
	    wm iconify .
	}

	set ::cb_done 1
	puts_info 10 "leaving done_callback"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# GASS Copy URL to URL (asynchronous)
#------------------------------------------------------------------------
proc gass_copy_async_url_to_url { read_cs write_cs } {
    if { [ catch {	    
	# Construct URL for remote file
	set dstfile $::remote_subdir
	append dstfile "/"
	append dstfile [file tail $write_cs]
	puts_info 10 "dstfile: $dstfile"

	# Test sending of the data file
	set ::cb_done 0
	globus_gass_copy_register_url_to_url \
	    $::gh \
	    file:///$read_cs \
	    $::srcAttr \
	    $dstfile \
	    $::dstAttr \
	    { done_callback NULL }
	vwait ::cb_done
	
	# Test retrieving of the data file
	set ::cb_done 0
	globus_gass_copy_register_url_to_url \
	    $::gh \
	    $dstfile \
	    $::srcAttr \
	    file:///$write_cs \
	    $::dstAttr \
	    { done_callback NULL }
	vwait ::cb_done
	
	# MD5 checksum
	check_md5sum $read_cs $write_cs
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Run test loop
#------------------------------------------------------------------------
proc run_test { } {
    if { [ catch {
	# Create local temporary directory
	copy_init $::localdir
	
	# Create remote top directory
	globus_gass_copy_mkdir \
	    $::gh \
	    $::remote_topdir \
	    $::dstAttr

	# Create remote sub-directory
	globus_gass_copy_mkdir \
	    $::gh \
	    $::remote_subdir \
	    $::dstAttr
	
	# globus_copy_url_to_url
	foreach read_cs $::filelist {
	    if { [file isdirectory $read_cs] } {
		continue
	    }
	    set write_cs [ file join $::localdir [ file tail $read_cs ] ]
	    
	    puts_info 10 "read_cs: $read_cs"
	    puts_info 10 "write_cs: $write_cs"
	    
	    gass_copy_async_url_to_url $read_cs $write_cs
	}
	
	# Remove remote files using FTP Client API
	foreach file $::filelist {
	    if { [file isdirectory $file] } {
		continue
	    }
	    set f [ file tail $file ]
	    set file $::remote_subdir
	    append file "/"
	    append file $f
	    puts_info 10 "file: $file"
	    # Do deletion
	    ftp_client_delete $file 
	}
	
	# Remove remote directories using FTP Client API
	ftp_client_rmdir $::remote_subdir
	ftp_client_rmdir $::remote_topdir

	# Remove local temporary directory
	copy_deinit $::localdir
    } err ] } {
	return -code error $err
    }		
}	
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # GASS Transfer module activation
    gass_module_activation

    # FTP initialization
    ftp_client_init

    # GASS Copy initialization
    gass_copy_init

    # Get file list
    set filelist [ constructFileList ]
    puts_info 10 "Number of files: [ llength $filelist ]"

    # Remote top directory
    set remote_topdir [ construct_gsiftp_url $TestEnv($SECTION,GSIFTP) ]
    puts_info 10 "remote_topdir: $remote_topdir"

    # Remote sub directory
    set remote_subdir $remote_topdir
    append remote_subdir "/"
    append remote_subdir [constructOutputDirectory $::argv0]
    puts_info 10 "remote_subdir: $remote_subdir"
    
    # Run the test in infinite loop mode or not
    if { [IsInfinite] == 1 } {
	while 1 {
	    run_test
	}
    } else {
	run_test
    }

    # GASS Copy cleanup
    gass_copy_cleanup
        
    # FTP cleanup
    ftp_client_cleanup

    # GASS Transfer module deactivation
    gass_module_deactivation    
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
