#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_gass_utils.tcl"
source "$env(SRCDIR)/test_ftp_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_gass_copy \
		  globus_gass_transfer \
		  globus_ftp_client \
		  globus_ftp_control \
		  globus_io } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Buffer length
#------------------------------------------------------------------------
proc buffer_length { } {
    if { [ catch {
	puts_info 10 "Testing globus_gass_copy_set_buffer_length"
	globus_gass_copy_set_buffer_length $::gh 1024

	puts_info 10 "Testing globus_gass_copy_get_buffer_length"
	lassign [ globus_gass_copy_get_buffer_length $::gh \
		     ] result blocksize

	if { $blocksize != 1024 } {
	    set ::ExitStatus 0
	}
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# No third party transfers
#------------------------------------------------------------------------
proc no_third_party_transfers { } {
    if { [ catch {
	puts_info 10 "Testing globus_gass_copy_set_no_third_party_transfers"
	globus_gass_copy_set_no_third_party_transfers $::gh $::GLOBUS_TRUE

	puts_info 10 "Testing globus_gass_copy_get_no_third_party_transfers"
	lassign [ globus_gass_copy_get_no_third_party_transfers \
		      $::gh \
		     ] result third_party

	if { $third_party != 1 } {
	    set ::ExitStatus 0
	}
    } err ] } {
	return -code error $err
    }	
}
#------------------------------------------------------------------------
# Partial offsets
#------------------------------------------------------------------------
proc partial_offsets { } {
    if { [ catch {
	puts_info 10 "Testing globus_gass_copy_set_partial_offsets"
	globus_gass_copy_set_partial_offsets $::gh 512 1024

	puts_info 10 "Testing globus_gass_copy_get_partial_offsets"
	lassign [ globus_gass_copy_get_partial_offsets $::gh \
		     ] result start_offset end_offset
	
	if { $start_offset != 512 || $end_offset != 1024 } {
	    set ::ExitStatus 0
	}
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# URL mode
#------------------------------------------------------------------------
proc url_mode { } {
    if { [ catch {
	set url gsiftp://ldas-grid.ligo.caltech.edu/tmp/msamidi/test_gass_copy_async_url_to_url-rh9-gt4-threaded/Makefile

	puts_info 10 "Testing globus_gass_copy_get_url_mode"
	lassign [ globus_gass_copy_get_url_mode $url ] result url_mode

	if { ! [string equal $url_mode "GLOBUS_GASS_COPY_URL_MODE_FTP"] } {
	    set ::ExitStatus 0
	}
    } err ] } {
	return -code error $err
    }	
}
#------------------------------------------------------------------------
# Set attributes for specific protocols
#------------------------------------------------------------------------
proc set_attrs_for_specific_protocols { } {
    if { [ catch {
	set gass_copy_attr [ new_p_globus_gass_copy_attr_t ]
	set io_attr	   [ new_p_globus_io_attr_t ]
	set request_attr   [ new_p_globus_gass_transfer_requestattr_t ]
	
	globus_io_tcpattr_init $io_attr

	puts_info 10 "Testing globus_gass_copy_attr_set_ftp"
	globus_gass_copy_attr_set_ftp $gass_copy_attr $::operation_attr
	puts_info 10 "Testing globus_gass_copy_attr_set_io"
	globus_gass_copy_attr_set_io $gass_copy_attr $io_attr
	puts_info 10 "Testing globus_gass_copy_attr_set_gass"
	globus_gass_copy_attr_set_gass $gass_copy_attr $request_attr

	delete_p_globus_gass_transfer_requestattr_t $request_attr
	delete_p_globus_io_attr_t $io_attr
	delete_p_globus_gass_copy_attr_t $gass_copy_attr
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Cache handles functions
#------------------------------------------------------------------------
proc cache_handles { } {
    if { [ catch {
	set url \
	    gsiftp://ldas-grid.ligo.caltech.edu/tmp/msamidi/test_gass_copy_async_url_to_url-rh9-gt4-threaded/Makefile

	puts_info 10 "Testing globus_gass_copy_cache_url_state"
	globus_gass_copy_cache_url_state $::gh $url
	puts_info 10 "Testing globus_gass_copy_flush_url_state"
	globus_gass_copy_flush_url_state $::gh $url
    } err ] } {
	return -code error $err
    }    
}
#------------------------------------------------------------------------
# Get/set user pointers from/to GASS copy handles
#------------------------------------------------------------------------
proc user_pointers { } {
    if { [ catch {
	puts_info 10 "Testing globus_gass_copy_set_user_pointer"
	globus_gass_copy_set_user_pointer $::gh NULL
	puts_info 10 "Testing globus_gass_copy_get_user_pointer"
	lassign [ globus_gass_copy_get_user_pointer $::gh \
		     ] result user_pointer

	if { ! [string match $user_pointer NULL] } {
	    set ::ExitStatus 0
	}
    } err ] } {
	return -code error $err
    }        
}
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "[info nameofexecutable]"
    
    # GASS Transfer module activation
    gass_module_activation

    # FTP initialization
    ftp_client_init

    # GASS Copy initialization
    gass_copy_init

    # $gha is globus_gass_copy_handleattr_t *
    # $handle_attr is globus_ftp_client_handleattr *
    globus_gass_copy_handleattr_set_ftp_attr $gha $handle_attr

    buffer_length

    no_third_party_transfers

    partial_offsets

    url_mode

    set_attrs_for_specific_protocols

    puts_info 10 "Testing globus_gass_copy_get_status"
    lassign [ globus_gass_copy_get_status $gh ] result status
    if { ! [string equal $status "GLOBUS_GASS_COPY_STATUS_NONE"] } {
	set ::ExitStatus 0
    }
    puts_info 10 "Testing globus_gass_copy_get_status_string"
    lassign [ globus_gass_copy_get_status_string $gh ] status
    if { ! [string equal $status "GLOBUS_GASS_COPY_STATUS_NONE"] } {
	set ::ExitStatus 0
    }

    cache_handles

    user_pointers

    # GASS Copy cleanup
    gass_copy_cleanup
        
    # FTP cleanup
    ftp_client_cleanup

    # GASS Transfer module deactivation
    gass_module_deactivation
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
