#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_gass_transfer \
		  globus_gass_server_ez } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "[info nameofexecutable]"
    
    puts_info 10 "Activate GASS Server EZ module"
    globus_module_activate $globus_i_gass_server_ez_module
    
    set listenerattr [ new_p_globus_gass_transfer_listenerattr_t ]
    set requestattr [ new_p_globus_gass_transfer_requestattr_t ]
    
    set default_options \
	[ expr $GLOBUS_GASS_SERVER_EZ_LINE_BUFFER | \
	      $GLOBUS_GASS_SERVER_EZ_TILDE_EXPAND | \
	      $GLOBUS_GASS_SERVER_EZ_TILDE_USER_EXPAND | \
	      $GLOBUS_GASS_SERVER_EZ_READ_ENABLE | \
	      $GLOBUS_GASS_SERVER_EZ_WRITE_ENABLE | \
	      $GLOBUS_GASS_SERVER_EZ_STDOUT_ENABLE | \
	      $GLOBUS_GASS_SERVER_EZ_STDERR_ENABLE | \
	      $GLOBUS_GASS_SERVER_EZ_CLIENT_SHUTDOWN_ENABLE ]
        
    puts_info 10 "Testing globus_gass_transfer_listenerattr_init"
    globus_gass_transfer_listenerattr_init \
	$listenerattr \
	http
    
    puts_info 10 "Testing globus_gass_transfer_listenerattr_set_port"
    lassign [ globus_gass_transfer_listenerattr_set_port \
		  $listenerattr \
		  50000 \
		 ] retval
    puts_info 10 "\treturn value: $retval"

    puts_info 10 "Testing globus_gass_server_ez_init"
    lassign [ globus_gass_server_ez_init \
		  $listenerattr \
		  http \
		  $requestattr \
		  $default_options \
		 ] retval listener
    puts_info 10 "\treturn value: $retval"
    puts_info 10 "\tlistener: $listener"
    
    puts_info 10 "Testing globus_gass_transfer_listenerattr_get_port"
    lassign [ globus_gass_transfer_listenerattr_get_port \
		  $listenerattr \
		 ] retval port
    puts_info 10 "\treturn value: $retval"
    puts_info 10 "\tport: $port"

    if { $port != 50000 } {
	set ExitStatus 1
    }
    
    puts_info 10 "Testing globus_gass_transfer_listener_get_base_url"
    lassign [ globus_gass_transfer_listener_get_base_url \
		  $listener \
		 ] url
    puts_info 10 "\turl: $url"

    puts_info 10 "Testing globus_gass_server_ez_shutdown"
    lassign [ globus_gass_server_ez_shutdown \
		  $listener \
		 ] retval
    puts_info 10 "\treturn value: $retval"
    
    puts_info 10 "Releasing resources"
    globus_gass_transfer_requestattr_destroy $requestattr
    delete_p_globus_gass_transfer_requestattr_t $requestattr

    delete_p_globus_gass_transfer_listenerattr_t $listenerattr
    
    puts_info 10 "Deactivate GASS Server EZ module"
    globus_module_deactivate $globus_i_gass_server_ez_module    
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
    
