#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_gass_transfer } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "[info nameofexecutable]"
    
    # Activate Globus GASS Transfer module
    puts_info 10 "Activate GASS Transfer module"
    globus_module_activate $globus_i_gass_transfer_module

    # Create request handle and attributes
    puts_info 10 "Allocate resources"
    set request_attr [ new_p_globus_gass_transfer_requestattr_t ]
    set listener_attr [ new_p_globus_gass_transfer_listenerattr_t ]
    #----------------------------------------------------------------
    # Initialize request and listener attributes
    #----------------------------------------------------------------
    puts_info 10 "Testing globus_gass_transfer_requestattr_init"
    globus_gass_transfer_requestattr_init \
	$request_attr \
	"http"

    puts_info 10 "Testing globus_gass_transfer_listenattr_init"
    globus_gass_transfer_listenerattr_init \
	$listener_attr \
	"http"
    #----------------------------------------------------------------
    # globus_gass_transfer_requestattr_set_proxy_url
    # globus_gass_transfer_requestattr_get_proxy_url
    #----------------------------------------------------------------
    puts_info 10 "Testing globus_gass_transfer_requestattr_set_proxy_url"
    globus_gass_transfer_requestattr_set_proxy_url \
	$request_attr \
	NULL
    puts_info 10 "Testing globus_gass_transfer_requestattr_get_proxy_url"
    lassign [ globus_gass_transfer_requestattr_get_proxy_url \
		  $request_attr \
		 ] retval proxy_url
    puts_info 10 "proxy_url: $proxy_url"
    if { ! [string match $proxy_url "NULL"] } {
	puts_info 10 "Setting/getting proxy URL request attribute fails"
	set ::ExitStatus 0
    }
    #----------------------------------------------------------------
    # globus_gass_transfer_requestattr_set_block_size
    # globus_gass_transfer_requestattr_get_block_size
    #----------------------------------------------------------------
    puts_info 10 "Testing globus_gass_transfer_requestattr_set_block_size"
    globus_gass_transfer_requestattr_set_block_size \
	$request_attr \
	1024
    puts_info 10 "Testing globus_gass_transfer_requestattr_get_block_size"
    lassign [ globus_gass_transfer_requestattr_get_block_size \
		  $request_attr \
		 ] retval block_size
    puts_info 10 "block_size: $block_size"
    if { $block_size != 1024 } {
	puts_info 10 "Setting/getting block size request attribute fails"
	set ::ExitStatus 0
    }
    #----------------------------------------------------------------
    # globus_gass_transfer_requestattr_set_file_mode
    # globus_gass_transfer_requestattr_get_file_mode
    #----------------------------------------------------------------
    puts_info 10 "Testing globus_gass_transfer_requestattr_set_file_mode"
    globus_gass_transfer_requestattr_set_file_mode \
	$request_attr \
	$GLOBUS_GASS_TRANSFER_FILE_MODE_TEXT
    puts_info 10 "Testing globus_gass_transfer_requestattr_get_file_mode"
    lassign [ globus_gass_transfer_requestattr_get_file_mode \
		  $request_attr \
		 ] retval file_mode
    puts_info 10 "file_mode: $file_mode"
    if { ! [ string match $file_mode "GLOBUS_GASS_TRANSFER_FILE_MODE_TEXT"] } {
	puts_info 10 "Setting/getting file mode request attribute fails"
	set ::ExitStatus 0
    }
    #----------------------------------------------------------------
    # globus_gass_transfer_requestattr_set_connection_reuse
    # globus_gass_transfer_requestattr_get_connection_reuse
    #----------------------------------------------------------------
    puts_info 10 "Testing globus_gass_transfer_requestattr_set_connection_reuse"
    globus_gass_transfer_requestattr_set_connection_reuse \
	$request_attr \
	$GLOBUS_TRUE
    puts_info 10 "Testing globus_gass_transfer_requestattr_get_connection_reuse"
    lassign [ globus_gass_transfer_requestattr_get_connection_reuse \
		  $request_attr \
		 ] retval conn_reuse
    puts_info 10 "connection_reuse: $conn_reuse"
    if { $conn_reuse != 1 } {
	puts_info 10 "Setting/getting connection reuse request attribute fails"
	set ::ExitStatus 0
    }
    #----------------------------------------------------------------
    # globus_gass_transfer_requestattr_set_socket_sndbuf
    # globus_gass_transfer_requestattr_get_socket_sndbuf
    #----------------------------------------------------------------
    puts_info 10 "Testing globus_gass_transfer_requestattr_set_socket_sndbuf"
    globus_gass_transfer_requestattr_set_socket_sndbuf \
	$request_attr \
	512
    puts_info 10 "Testing globus_gass_transfer_requestattr_get_socket_sndbuf"
    lassign [ globus_gass_transfer_requestattr_get_socket_sndbuf \
		  $request_attr \
		 ] retval sndbuf
    puts_info 10 "sndbuf: $sndbuf"
    if { $sndbuf != 512 } {
	puts_info 10 "Setting/getting socket sndbuf request attribute fails"
	set ::ExitStatus 0
    }
    #----------------------------------------------------------------
    # globus_gass_transfer_requestattr_set_socket_rcvbuf
    # globus_gass_transfer_requestattr_get_socket_rcvbuf
    #----------------------------------------------------------------
    puts_info 10 "Testing globus_gass_transfer_requestattr_set_socket_rcvbuf"
    globus_gass_transfer_requestattr_set_socket_rcvbuf \
	$request_attr \
	1024
    puts_info 10 "Testing globus_gass_transfer_requestattr_get_socket_rcvbuf"
    lassign [ globus_gass_transfer_requestattr_get_socket_rcvbuf \
		  $request_attr \
		 ] retval rcvbuf
    puts_info 10 "rcvbuf: $rcvbuf"
    if { $rcvbuf != 1024 } {
	puts_info 10 "Setting/getting socket rcvbuf request attribute fails"
	set ::ExitStatus 0
    }
    #----------------------------------------------------------------
    # globus_gass_transfer_requestattr_set_socket_nodelay
    # globus_gass_transfer_requestattr_get_socket_nodelay
    #----------------------------------------------------------------
    puts_info 10 "Testing globus_gass_transfer_requestattr_set_socket_nodelay"
    globus_gass_transfer_requestattr_set_socket_nodelay \
	$request_attr \
	$GLOBUS_FALSE
    puts_info 10 "Testing globus_gass_transfer_requestattr_get_socket_nodelay"
    lassign [ globus_gass_transfer_requestattr_get_socket_nodelay \
		  $request_attr \
		 ] retval nodelay
    puts_info 10 "nodelay: $nodelay"
    if { $nodelay != 0 } {
	puts_info 10 "Setting/getting socket nodelay request attribute fails"
	set ::ExitStatus 0
    }
    #----------------------------------------------------------------
    # globus_gass_transfer_secure_requestattr_set_authorization
    # globus_gass_transfer_secure_requestattr_get_authorization
    #----------------------------------------------------------------
    if {0 } {
	puts_info 10 "Testing globus_gass_transfer_secure_requestattr_set_authorization"
	globus_gass_transfer_secure_requestattr_set_authorization \
	    $request_attr \
	    $GLOBUS_GASS_TRANSFER_AUTHORIZE_SELF \
	    "/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621/CN=1423281965"
        
	puts_info 10 "Testing globus_gass_transfer_secure_requestattr_get_authorization"
	lassign [ globus_gass_transfer_secure_requestattr_get_authorization \
		      $request_attr \
		     ] retval mode subject
	puts_info 10 "mode: $mode"
	puts_info 10 "subject: $subject"
	if { ! [string match $mode "GLOBUS_GASS_TRANSFER_AUTHORIZE_SUBJECT"] } {
	    puts_info 10 "Setting/getting secure authorization fails"
	    set ::ExitStatus 0
	}
    }
    #----------------------------------------------------------------
    # globus_gass_transfer_listenerattr_set_backlog
    # globus_gass_transfer_listenerattr_get_backlog
    #----------------------------------------------------------------
    puts_info 10 "Testing globus_gass_transfer_listenerattr_set_backlog"
    globus_gass_transfer_listenerattr_set_backlog $listener_attr 1

    puts_info 10 "Testing globus_gass_transfer_listenerattr_get_backlog"
    lassign [ globus_gass_transfer_listenerattr_get_backlog \
		  $listener_attr \
		 ] retval backlog
    puts_info 10 "backlog: $backlog"
    if { $backlog != 1 } {
	set ::ExitStatus 0
    }
    #----------------------------------------------------------------
    # globus_gass_transfer_listenerattr_set_port
    # globus_gass_transfer_listenerattr_get_port
    #----------------------------------------------------------------
    puts_info 10 "Testing globus_gass_transfer_listenerattr_set_port"
    globus_gass_transfer_listenerattr_set_port $listener_attr 50000
    puts_info 10 "Testing globus_gass_transfer_listenerattr_get_port"
    lassign [ globus_gass_transfer_listenerattr_get_port \
		  $listener_attr \
		 ] retval port
    puts_info 10 "port: $port"
    if { $port != 50000 } {
	set ::ExitStatus 0
    }
    #----------------------------------------------------------------
    # Destroy request attributes
    #----------------------------------------------------------------
    puts_info 10 "Destroy attributes"
    globus_gass_transfer_requestattr_destroy $request_attr
    #globus_gass_transfer_listenerattr_destroy $listener_attr
    
    # Deallocate resources
    puts_info 10 "Deallocate resources"
    delete_p_globus_gass_transfer_requestattr_t $request_attr
    delete_p_globus_gass_transfer_listenerattr_t $listener_attr
    
    # Deactivate Globus GASS Copy module
    puts_info 10 "Deactivate GASS Transfer module"
    globus_module_deactivate $globus_i_gass_transfer_module    
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
