#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Determine if we have threads or not
#------------------------------------------------------------------------
set ::EXEC_MAIN 1

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_ftp_utils.tcl"
source "$env(SRCDIR)/test_gass_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io \
		  globus_gass_transfer } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
set SECTION gass_transfer
validate_section $SECTION [list BUFFERSIZE URL_SCHEME]

set buffersize $TestEnv($SECTION,BUFFERSIZE)
puts_info 10 "buffersize: $buffersize"

set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
puts_info 10 "sourcedir: $sourcedir"

set localdir [file join /tmp "$env(USER)"]
set localdir [file join $localdir [constructOutputDirectory $argv0]]
puts_info 10 "GASS client directory: $localdir"

set gass_server_dir [file join /tmp "$env(USER)"]
set gass_server_dir [file join $gass_server_dir server]
set gass_server_dir [file join $gass_server_dir \
			 [constructOutputDirectory $argv0]]
puts_info 10 "GASS server directory: $gass_server_dir"

#------------------------------------------------------------------------
# Namespace Globus
#------------------------------------------------------------------------
namespace eval globus {
    proc activation { } {
	global globus_i_gass_transfer_module
	global globus_l_io_module
	
	#----------------------------------------------------------------
	# Activate Globus GASS Transfer module
	#----------------------------------------------------------------
	puts_info 10 "Activate GASS Transfer module"
	globus_module_activate $globus_i_gass_transfer_module
	
	#----------------------------------------------------------------
	# Activate Globus I/O module
	#----------------------------------------------------------------
	puts_info 10 "Activate I/O module"
	globus_module_activate $globus_l_io_module
    }

    proc deactivation { } {
	global globus_i_gass_transfer_module
	global globus_l_io_module

	#----------------------------------------------------------------
	# Deactivate Globus I/O module
	#----------------------------------------------------------------
	puts_info 10 "Deactivate I/O module"
	globus_module_deactivate $globus_l_io_module
	
	#----------------------------------------------------------------
	# Deactivate Globus GASS Copy module
	#----------------------------------------------------------------
	puts_info 10 "Deactivate GASS Transfer module"
	globus_module_deactivate $globus_i_gass_transfer_module
    }
}

#------------------------------------------------------------------------
# Namespace Client
#------------------------------------------------------------------------
namespace eval client {
    variable data
    variable cb_done

    #------------------------------------------------------------------------
    # Client-initiated callback
    #------------------------------------------------------------------------
    proc client_initiated_cb { user_parms request } {
	if { [ catch {
	    puts_info 10 "entering client_initiated_cb"
	    
	    upvar 0 $user_parms user_args
	    
	    puts_info 10 "\trequest: $request"
	    
	    set user_args(requesthandle) $request
	    
	    lassign [ globus_gass_transfer_request_get_status \
			  $request \
			 ] request_status
	    puts_info 10 "\trequest status: $request_status"
	    
	    set cb_done 1
	    
	    puts_info 10 "leaving client_initiated_cb"
	} err ] } {
	    err_handler $err
	}		
    }

    #--------------------------------------------------------------------
    # Send bytes callback
    #--------------------------------------------------------------------
    proc send_bytes_cb { user_parms request buffer length last_data } {
	variable cb_done
	
	if { [ catch {
	    puts_info 20 "Entering client::send_bytes_cb"
	    
	    upvar 0 $user_parms user_args
	    
	    puts_info 20 "\trequest: $request"
	    puts_info 20 "\tbuffer length: [string length $buffer]"
	    puts_info 20 "\tlength: $length"
	    puts_info 20 "\tlast_data: $last_data"

	    if { ! $last_data } {
		lassign [ globus_gass_transfer_request_get_status \
			      $user_args(requesthandle) \
			     ] request_status
		puts_info 20 "\trequest status: $request_status"
		
		set buffer [ read $user_args(srcId) 1024 ]
		puts_info 20 "\tbuffer length of Tcl next read: [string length $buffer]"
		
		if { [string length $buffer] == 0 } {
		    puts_info 20 "\tbuffer: $buffer"
		    globus_gass_transfer_send_bytes \
			$user_args(requesthandle) \
			$buffer \
			0 \
			$::GLOBUS_TRUE \
			[ subst { [namespace current]::send_bytes_cb \
				      $user_parms } ]
		} else {
		    globus_gass_transfer_send_bytes \
			$user_args(requesthandle) \
			$buffer \
			[string length $buffer] \
			$::GLOBUS_FALSE \
			[ subst { [namespace current]::send_bytes_cb  \
				      $user_parms } ]
		}
	    } else {
		set cb_done 1
	    }
	    
	    puts_info 20 "Leaving client::send_bytes_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    
    #--------------------------------------------------------------------
    # GASS Transfer client-initiated put operation
    #--------------------------------------------------------------------
    proc put_operation { server_url sourcefile mode } {
	variable data
	variable gass_server_dir
	variable cb_done
	
	if { [ catch {
	    puts_info 10 "Entering client::put_operation"
	    	    
	    #------------------------------------------------------------
	    # Setup target file
	    #------------------------------------------------------------
	    set targetfile $server_url
	    append targetfile $::gass_server_dir
	    append targetfile "/"
	    append targetfile [file tail $sourcefile]
	    puts_info 10 "\ttargetfile: $targetfile"
	    
	    # Open srcfile
	    set data(srcId) [ open $sourcefile r ]
	    
	    set filetype [ fileType $sourcefile ]
	    if { [ fileType $sourcefile ] == "binary executable elf" } {
		puts_info 10 "\tBinary mode"
		fconfigure $data(srcId) -translation binary -encoding binary
	    }

	    set buffer [ read $data(srcId) 1024 ]
	    puts_info 10 "\tbuffer length: [string length $buffer]"
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_put
	    #------------------------------------------------------------
	    if { $mode == "blocking" } {
		puts_info 10 "\tglobus_gass_transfer_put"
		lassign [ globus_gass_transfer_put \
			      $data(requestattr) \
			      $targetfile \
			      $::GLOBUS_GASS_TRANSFER_LENGTH_UNKNOWN \
			     ] retval data(requesthandle)
		puts_info 20 "\tglobus_gass_transfer_put return value: $retval"
		puts_info 10 "\tglobus_gass_transfer_put request handle : $data(requesthandle)"
	    }
	
	    #------------------------------------------------------------
	    # globus_gass_transfer_register_put
	    #------------------------------------------------------------
	    if { $mode == "nonblocking" } {
		set cb_done 0
		puts_info 10 "\tglobus_gass_transfer_register_put"
		lassign [ globus_gass_transfer_register_put \
			      $data(requestattr) \
			      $targetfile \
			      $::GLOBUS_GASS_TRANSFER_LENGTH_UNKNOWN \
			      [subst { [namespace current]::client_initiated_cb \
					   [namespace current]::data }] \
			     ] retval
		puts_info 20 "\tglobus_gass_transfer_put return value: $retval"
		vwait cb_done
		puts_info 10 "\tglobus_gass_transfer_put request handle : $data(requesthandle)"
		
	    }

	    #------------------------------------------------------------
	    # globus_gass_transfer_request_set_url
	    #------------------------------------------------------------
	    if { 0 } {
		puts_info 10 "\tTesting globus_gass_transfer_request_set_url"
		globus_gass_transfer_request_set_url \
		    $data(requesthandle) \
		    $server_url
	    }
	    
	    #--------------------------------------------------------
	    # globus_gass_transfer_request_get_url
	    #--------------------------------------------------------
	    puts_info 10 "\tTesting globus_gass_transfer_request_get_url"
	    lassign [ globus_gass_transfer_request_get_url \
			  $data(requesthandle) \
			 ] url
	    puts_info 10 "\turl: $url"
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_request_set_user_pointer
	    #------------------------------------------------------------
	    globus_gass_transfer_request_set_user_pointer \
		$data(requesthandle) \
		NULL

	    #------------------------------------------------------------
	    # globus_gass_transfer_request_get_user_pointer
	    #------------------------------------------------------------
	    lassign [ globus_gass_transfer_request_get_user_pointer \
			  $data(requesthandle) \
			 ] user_pointer
	    puts_info 10 "\tuser pointer: $user_pointer"
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_request_get_type
	    #------------------------------------------------------------
	    lassign [ globus_gass_transfer_request_get_type \
			  $data(requesthandle) \
			 ] request_type
	    puts_info 10 "\trequest type: $request_type"
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_request_get_status
	    #------------------------------------------------------------
	    lassign [ globus_gass_transfer_request_get_status \
			  $data(requesthandle) \
			 ] request_status
	    puts_info 10 "\trequest status: $request_status"
	    
	    #------------------------------------------------------------
	    # Request is denied
	    #------------------------------------------------------------
	    if { [string equal $request_status \
		      "GLOBUS_GASS_TRANSFER_REQUEST_DENIED"] } {
		lassign [ globus_gass_transfer_request_get_denial_reason \
			      $data(requesthandle) \
			     ] denial_reason
		puts_info 10 "\tdenial reason: $denial_reason"
		
		lassign [ globus_gass_transfer_request_get_denial_message \
			      $data(requesthandle) \
			     ] denial_message
		puts_info 10 "\tdenial message: $denial_message"
	    } else {
		#--------------------------------------------------------
		# Request is granted
		#--------------------------------------------------------
		lassign [ globus_gass_transfer_request_get_url \
			      $data(requesthandle) \
			     ] remote_url
		puts_info 10 "\tremote url: $remote_url"
		
		#--------------------------------------------------------
		# Determine transfer file mode, binary or text
		#--------------------------------------------------------
		if { 0 } {
		    set filetype [ fileType $sourcefile ]
		    if { [ fileType $sourcefile ] == "binary executable elf" } {
			puts_info 10 "Initialize binary mode"
			globus_gass_transfer_requestattr_set_file_mode \
			    $requestattr \
			    $::GLOBUS_GASS_TRANSFER_FILE_MODE_BINARY
		    } else {
			puts_info 10 "Initialize text mode"
			globus_gass_transfer_requestattr_set_file_mode \
			    $requestattr \
			    $::GLOBUS_GASS_TRANSFER_FILE_MODE_TEXT
		    }
		}

		if { 0 } {
		    #----------------------------------------------------
		    # globus_gass_transfer_request_set_subject
		    #----------------------------------------------------
		    globus_gass_transfer_request_set_subject \
			$data(requesthandle) \
			"/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
		}
		
		set cb_done 0
		globus_gass_transfer_send_bytes \
		    $data(requesthandle) \
		    $buffer \
		    [string length $buffer] \
		    $::GLOBUS_FALSE \
		    [subst  { [namespace current]::send_bytes_cb \
				  [namespace current]::data }]
		vwait [namespace current]::cb_done		
	    }
	    
	    puts_info 10 "\tglobus_gass_transfer_request_destroy: $data(requesthandle)"
	    globus_gass_transfer_request_destroy $data(requesthandle)
	    	    
	    puts_info 10 "Leaving client::put_operation"
	} err ] } {
	    return -code error $err
	}    
    }

   #---------------------------------------------------------------------
    # Client cleanup
    #--------------------------------------------------------------------    
    proc cleanup { } {
	variable data
	
	if { [ catch {
	    puts_info 10 "Entering client::cleanup"
	    
	    # Cleanup I/O resources
	    puts_info 10 "Cleanup I/O resources"
	    globus_io_fileattr_destroy $data(fileattr)
	    delete_p_globus_io_attr_t $data(fileattr)
	    
	    # Cleanup GASS Transfer resources
	    puts_info 10 "Cleanup GASS Transfer resources"
	    globus_gass_transfer_requestattr_destroy $data(requestattr)
	    delete_p_globus_gass_transfer_requestattr_t $data(requestattr)

	    puts_info 10 "Leaving client::cleanup"
	} err ] } {
	    return -code error $err
	}
    }

    #--------------------------------------------------------------------
    # Client initialize
    #--------------------------------------------------------------------
    proc initialize { } {
	variable data

	if { [ catch {
	    puts_info 10 "Entering client::initialize"
	    
	    #------------------------------------------------------------
	    # I/O attributes
	    #------------------------------------------------------------
	    set data(fileattr) [ new_p_globus_io_attr_t ]
	    puts_info 10 "\tfileattr value: [p_globus_io_attr_t_value $data(fileattr)]"
	    
	    puts_info 10 "\tglobus_io_fileattr_init"
	    globus_io_fileattr_init $data(fileattr)
	    
	    #------------------------------------------------------------
	    # Create GASS Transfer request handle and attributes
	    #------------------------------------------------------------
	    set data(requestattr) [new_p_globus_gass_transfer_requestattr_t]
	    puts_info 10 "\trequestattr value: [ p_globus_gass_transfer_requestattr_t_value $data(requestattr) ]"
	    	    
	    #------------------------------------------------------------
	    # Initialize GASS Transfer request attributes
	    #------------------------------------------------------------
	    puts_info 10 "\tInitialize request attributes"
	    globus_gass_transfer_requestattr_init \
		$data(requestattr) \
		$::url_scheme
	    
	    puts_info 10 "\tInitialize connection reuse attribute"
	    globus_gass_transfer_requestattr_set_connection_reuse \
		$data(requestattr) \
		$::GLOBUS_TRUE

	    puts_info 10 "Leaving client::initialize"
	} err ] } {
	    return -code error $err
	}
    }
}

#------------------------------------------------------------------------
# Namespace Server
#------------------------------------------------------------------------
namespace eval server {
    variable data
    variable url
    variable cb_done
        
    #--------------------------------------------------------------------
    # Write bytes callback
    #--------------------------------------------------------------------
    proc write_bytes_cb { user_parms handle result buffer } {
	variable cb_done
	
	if { [ catch {
	    puts_info 20 "Entering server::write_bytes_cb"
	    
	    upvar 0 $user_parms user_args
	    
	    puts_info 20 "\thandle: $handle"
	    puts_info 20 "\tresult: $result"
	    puts_info 20 "\tbuffer  length: [string length $buffer]"
	    puts_info 20 "\trecv last data: $user_args(recv_last_data)"

	    if { $user_args(recv_last_data) == 1 } {
		set cb_done 1
	    } else {
		globus_gass_transfer_receive_bytes \
		    $user_args(requesthandle) \
		    1024 \
		    1 \
		    [ subst { [namespace current]::recv_bytes_cb \
				  $user_parms } ]
	    }
	    puts_info 20 "Leaving server::write_bytes_cb"
	} err ] } {
	    err_handler $err
	}	
    }
    
    #--------------------------------------------------------------------
    # Put bytes callback
    #--------------------------------------------------------------------
    proc recv_bytes_cb { user_parms request buffer length last_data } {
	variable cb_done
	
	if { [ catch {
	    puts_info 20 "Entering server::recv_bytes_cb"
	    
	    upvar 0 $user_parms user_args
	    
	    puts_info 20 "\tbuffer length: [string length $buffer]"
	    puts_info 20 "\tlength: $length"
	    puts_info 20 "\tlast_data: $last_data"
	    	    
	    lassign [ globus_gass_transfer_request_get_status \
			  $user_args(requesthandle) \
			 ] request_status
	    puts_info 20 "\trequest status: $request_status"

	    if { $last_data == 1 } {
		set user_args(recv_last_data) 1
	    } else {
		set user_args(recv_last_data) 0
	    }

	    globus_io_register_write \
		$user_args(writehandle) \
		$buffer \
		[ subst { [namespace current]::write_bytes_cb \
			      $user_parms } ]

	    puts_info 20 "Leaving server::recv_bytes_cb"
	} err ] } {
	    err_handler $err
	}	
    }

    
    #--------------------------------------------------------------------
    # Accept callback
    #--------------------------------------------------------------------
    proc accept_cb { user_parms request } {
	variable data
	variable cb_done
	global gass_server_dir
	
	if { [ catch {
	    puts_info 10 "Entering server::accept_cb"
	    
	    upvar 0 $user_parms user_args

	    puts_info 10 "\trequest: $request"
	    set user_args(requesthandle) $request

	    #------------------------------------------------------------
	    # globus_gass_transfer_deny
	    #------------------------------------------------------------
	    if { 0 } {
		puts_info 10 "\tTesting globus_gass_transfer_deny"
		globus_gass_transfer_deny \
		    $user_args(requesthandle) \
		    100 \
		    "Testing"
	    }

	    #------------------------------------------------------------
	    # globus_gass_transfer_refer
	    #------------------------------------------------------------
	    if { 0 } {
		puts_info 10 "\tTesting globus_gass_transfer_refer"
		globus_gass_transfer_refer \
		    $user_args(requesthandle) \
		    "http://tclglobus.ligo.caltech.edu:33684" \
		    1
	    }

	    #------------------------------------------------------------
	    # globus_gass_transfer_request_get_type
	    #------------------------------------------------------------
	    lassign [ globus_gass_transfer_request_get_type \
			  $user_args(requesthandle) \
			 ] request_type
	    puts_info 10 "\trequest type: $request_type"

	    #------------------------------------------------------------
	    # globus_gass_transfer_request_get_status
	    #------------------------------------------------------------
	    lassign [ globus_gass_transfer_request_get_status \
			  $user_args(requesthandle) \
			 ] request_status
	    puts_info 10 "\trequest status: $request_status"

	    if { 0 } {
		#--------------------------------------------------------
		# globus_gass_transfer_request_get_subject
		#--------------------------------------------------------
		lassign [  globus_gass_transfer_request_get_subject \
			       $user_args(requesthandle) \
			      ] subject
		puts_info 10 "\tsubject: $subject"
		
		#--------------------------------------------------------
		# globus_gass_transfer_request_get_url
		#--------------------------------------------------------
		lassign [ globus_gass_transfer_request_get_url \
			      $user_args(requesthandle) \
			     ] client_url
		puts_info 10 "\tclient_url: $client_url"
	    }
	    
	    #------------------------------------------------------------
	    # Write flags and modes
	    #------------------------------------------------------------
	    if { $request_type == "GLOBUS_GASS_TRANSFER_REQUEST_TYPE_APPEND" } {
		set write_flags [ expr $::GLOBUS_IO_FILE_CREAT | \
				      $::GLOBUS_IO_FILE_APPEND | \
				      $::GLOBUS_IO_FILE_WRONLY ]
		puts_info 10 "\twrite_flags: $write_flags"
	    }
	    
	    if { $request_type == "GLOBUS_GASS_TRANSFER_REQUEST_TYPE_PUT" } {
		set write_flags [ expr $::GLOBUS_IO_FILE_CREAT | \
				      $::GLOBUS_IO_FILE_TRUNC | \
				      $::GLOBUS_IO_FILE_WRONLY ]
		puts_info 10 "\twrite_flags: $write_flags"
	    }

	    #------------------------------------------------------------
	    # Actual data transmission
	    #------------------------------------------------------------
	    if { $request_type == \
		     "GLOBUS_GASS_TRANSFER_REQUEST_TYPE_APPEND" || \
		     $request_type == \
			   "GLOBUS_GASS_TRANSFER_REQUEST_TYPE_PUT" } {
		
		#--------------------------------------------------------
		# globus_gass_transfer_authorize
		#--------------------------------------------------------
		lassign [ globus_gass_transfer_authorize \
			      $user_args(requesthandle) \
			      0 \
			     ] retval
		puts_info 10 "\tglobus_gass_transfer_authorize: retval: $retval"
		
		#--------------------------------------------------------
		# Create write handle
		#--------------------------------------------------------
		set data(writehandle) [ new_p_globus_io_handle_t ]
				
		set write_mode [ expr $::GLOBUS_IO_FILE_IRUSR | \
				     $::GLOBUS_IO_FILE_IWUSR ]
		puts_info 10 "\twrite_mode: $write_mode"

		set targetfile $gass_server_dir
		append targetfile "/"
		append targetfile $::TestEnv($::SECTION,FILENAME)
		puts_info 10 "\ttargetfile: $targetfile"
		
		globus_io_file_open \
		    $targetfile \
		    $write_flags \
		    $write_mode \
		    NULL \
		    $user_args(writehandle) 
		puts_info 10 "\tglobus_io_file_open"
		
		#---------------------------------------------------------
		# globus_gass_transfer_receive_bytes
		#---------------------------------------------------------
		set cb_done 0
		lassign [ globus_gass_transfer_receive_bytes \
			      $request \
			      1024 \
			      1 \
			      [subst { [namespace current]::recv_bytes_cb \
					   $user_parms }] \
			     ] retval 
		vwait [namespace current]::cb_done

		puts_info 10 "\tglobus_gass_transfer_receive_bytes is completed"
		#---------------------------------------------------------
		# Close I/O write handle
		#---------------------------------------------------------
		globus_io_close $user_args(writehandle)	
		delete_p_globus_io_handle_t $data(writehandle)		
		
	    } elseif { $request_type == \
			   "GLOBUS_GASS_TRANSFER_REQUEST_TYPE_GET" } {

		if { 0 } {
		    set data(readhandle) [ new_p_globus_io_handle_t ]
		    puts_info 10 \
			"\treadhandle: [p_globus_io_handle_t_value $data(readhandle)]"
		}

		set targetfile $gass_server_dir
		append targetfile "/"
		append targetfile $::TestEnv($::SECTION,FILENAME)
		puts_info 10 "targetfile: $targetfile"

		if { 0 } {
		    #-----------------------------------------------------
		    # globus_io_file_open
		    #-----------------------------------------------------
		    puts_info 10 "\tglobus_io_file_open"
		    globus_io_file_open \
			$targetfile \
			$::GLOBUS_IO_FILE_RDONLY \
			0 \
			NULL \
			$user_args(readhandle)
		}

		# Open srcfile
		set data(srcId) [ open $targetfile r ]
		
		set filetype [ fileType $targetfile ]
		if { [ fileType $targetfile ] == "binary executable elf" } {
		    puts_info 10 "\tBinary mode"
		    fconfigure $data(srcId) -translation binary -encoding binary
		}

		#---------------------------------------------------------
		# globus_gass_transfer_authorize
		#---------------------------------------------------------
		globus_gass_transfer_authorize \
		    $user_args(requesthandle) \
		    [file size $targetfile]

		if { 0 } {
		    #-----------------------------------------------------
		    # globus_io_register_read
		    #-----------------------------------------------------
		    puts_info 10 "globus_io_register_read"
		    set cb_done 0
		    globus_io_register_read \
			$user_args(readhandle) \
			1024 \
			1 \
			[subst { operation::read_bytes_cb \
				     $user_parms }]
		    vwait cb_done
		}

		set buffer [ read $data(srcId) 1024 ]

		set cb_done 0
		globus_gass_transfer_send_bytes \
		    $user_args(requesthandle) \
		    $buffer \
		    [string length $buffer] \
		    $::GLOBUS_FALSE \
		    [subst  { [namespace current]::send_bytes_cb \
				  $user_parms }]
		#vwait cb_done
		vwait [namespace current]::cb_done
		
		puts_info 10 "\tserver: globus_gass_transfer_send_bytes is completed"
		#--------------------------------------------------------
		# Close I/O read handle
		#--------------------------------------------------------
		#globus_io_close $user_args(readhandle)	
		#delete_p_globus_io_handle_t $data(readhandle)
		
	    } elseif { $request_type == \
			   "GLOBUS_GASS_TRANSFER_REQUEST_REFERRED" } {
		
		lassign [ globus_gass_transfer_request_get_referral \
			      $user_args(requesthandle) \
			     ] retval user_args(referral)
		puts_info 10 "referral: $referral"
		
		lassign [ globus_gass_transfer_referral_get_count \
			      $user_args(referral) \
			     ] referral_count
		puts_info 10 "referral count: $referral_count"

		lassign [ globus_gass_transfer_referral_get_url \
			      $user_args(referral) \
			      $referral_count
			     ] referral_url
		puts_info 10 "referral URL: $referral_url"
				
		puts_info 10 "Testing globus_gass_transfer_referral_destroy"
		globus_gass_transfer_referral_destroy \
		    $user_args(referral)
	    }

	    #------------------------------------------------------------
	    # Destroy request handle & attributes
	    #------------------------------------------------------------
	    globus_gass_transfer_request_destroy $request
	    
	    puts_info 10 "Leaving server::accept_cb"
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Listener callback
    #--------------------------------------------------------------------
    proc listen_cb { user_parms listener } {
	variable data
	variable cb_done
	
	if { [ catch {
	    puts_info 10 "Entering server::listen_cb"
	    
	    upvar 0 $user_parms user_args

	    puts_info 10 "\tlistener: $listener"
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_register_accept
	    #------------------------------------------------------------
	    puts_info 10 "\tglobus_gass_transfer_register_accept"

	    lassign [ globus_gass_transfer_register_accept \
			  $user_args(requestattr) \
			  $user_args(listener) \
			  [subst { [namespace current]::accept_cb \
				       $user_parms }] \
			 ] retval

	    #------------------------------------------------------------
	    # Destroy request handle & attributes
	    #------------------------------------------------------------
	    #globus_gass_transfer_request_destroy $user_args(requesthandle)
	    
	    puts_info 10 "Leaving server::listen_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server shutdown
    #--------------------------------------------------------------------
    proc shutdown { } {
	variable data
	variable cb_done
	
	if { [ catch {
	    puts_info 10 "Entering server::shutdown"
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_close_listener
	    #------------------------------------------------------------
	    puts_info 10 "\tglobus_gass_transfer_close_listener"
	    set cb_done 0
	    globus_gass_transfer_close_listener \
		$data(listener) \
		{ close_listener_cb NULL }
	    vwait cb_done
	    
	    #------------------------------------------------------------
	    # Close I/O read handle
	    #------------------------------------------------------------
	    #globus_io_close $data(readhandle)	
	    #delete_p_globus_io_handle_t $data(readhandle)

	    #------------------------------------------------------------
	    # Close I/O read handle
	    #------------------------------------------------------------
	    #globus_io_close $data(writehandle)	
	    #delete_p_globus_io_handle_t $data(writehandle)		

	    #------------------------------------------------------------
	    # Cleanup I/O resources
	    #------------------------------------------------------------
	    #puts_info 10 "\tCleanup I/O resources"
	    #globus_io_fileattr_destroy $data(fileattr)
	    #delete_p_globus_io_attr_t $data(fileattr)
	    
	    #------------------------------------------------------------
	    # Cleanup GASS Transfer resources
	    #------------------------------------------------------------
	    puts_info 10 "\tCleanup GASS Transfer resources"

	    #delete_p_globus_gass_transfer_listener_t $data(listener)

	    #globus_gass_transfer_listenerattr_destroy $data(listenerattr)
	    delete_p_globus_gass_transfer_listenerattr_t $data(listenerattr)

	    globus_gass_transfer_requestattr_destroy $data(requestattr)
	    delete_p_globus_gass_transfer_requestattr_t $data(requestattr)
	    
	    puts_info 10 "Leaving server::shutdown"
	} err ] } {
	    return -code error $err
	}
    }

    #--------------------------------------------------------------------
    # Server initialization
    #--------------------------------------------------------------------
    proc initialize { } {
	variable data
	variable url
		
	if { [ catch {
	    puts_info 10 "Entering server::initialize"
	    
	    #------------------------------------------------------------
	    # I/O attributes
	    #------------------------------------------------------------
	    #set data(fileattr) [ new_p_globus_io_attr_t ]
	    #puts_info 10 "\tfileattr value: [p_globus_io_attr_t_value $data(fileattr)]"
	    
	    #puts_info 10 "\tglobus_io_fileattr_init"
	    #globus_io_fileattr_init $data(fileattr)

	    #set data(readhandle)  [ new_p_globus_io_handle_t ]
	    set data(writehandle) [ new_p_globus_io_handle_t ]
	    
	    #------------------------------------------------------------
	    # GASS Transfer attributes
	    #------------------------------------------------------------
	    set data(listenerattr) [new_p_globus_gass_transfer_listenerattr_t]
	    set data(requestattr)  [new_p_globus_gass_transfer_requestattr_t]
	    
	    #------------------------------------------------------------
	    # Initialize listener and request attributes
	    #------------------------------------------------------------
	    puts_info 10 "\tglobus_gass_transfer_listenerattr_init"
	    globus_gass_transfer_listenerattr_init \
		$data(listenerattr) \
		$::url_scheme
	    
	    puts_info 10 "\tglobus_gass_transfer_requestattr_init"
	    globus_gass_transfer_requestattr_init \
		$data(requestattr) \
		$::url_scheme

	    #------------------------------------------------------------
	    # globus_gass_transfer_listenerattr_set_backlog
	    #------------------------------------------------------------
	    puts_info 10 "\tglobus_gass_transfer_listenerattr_set_backlog"
	    globus_gass_transfer_listenerattr_set_backlog \
		$data(listenerattr) \
		1
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_listenerattr_set_port
	    #------------------------------------------------------------
	    puts_info 10 "\tglobus_gass_transfer_listenerattr_set_port"
	    globus_gass_transfer_listenerattr_set_port \
		$data(listenerattr) \
		0
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_create_listener
	    #------------------------------------------------------------
	    puts_info 10 "\tglobus_gass_transfer_create_listener"
	    lassign [ globus_gass_transfer_create_listener \
			  $data(listenerattr) \
			  $::url_scheme \
			 ] retval data(listener)
	    puts_info 10 "\tglobus_gass_transfer_create_listener: $data(listener)"
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_listener_set_user_pointer
	    #------------------------------------------------------------
	    globus_gass_transfer_listener_set_user_pointer \
		$data(listener) \
		NULL

	    #------------------------------------------------------------
	    # globus_gass_transfer_listener_get_user_pointer
	    #------------------------------------------------------------
	    lassign [ globus_gass_transfer_listener_get_user_pointer \
			  $data(listener) \
			 ] user_pointer
	    puts_info 10 "\tuser_pointer: $user_pointer"

	    #------------------------------------------------------------
	    # globus_gass_transfer_listener_get_base_url
	    #------------------------------------------------------------
	    lassign [ globus_gass_transfer_listener_get_base_url \
			  $data(listener) \
			 ] url
	    puts_info 10 "\turl: $url"

	    #------------------------------------------------------------
	    # globus_gass_transfer_register_listen
	    #------------------------------------------------------------
	    puts_info 10 "\tglobus_gass_transfer_register_listen"
	    globus_gass_transfer_register_listen \
		$data(listener) \
		[subst { [namespace current]::listen_cb \
			     [namespace current]::data }]
	    		
	    puts_info 10 "Leaving server::initialize"
	} err ] } {
	    return -code error $err
	}
    }
}
#------------------------------------------------------------------------
# Run test loop
#------------------------------------------------------------------------
proc run_test { } {
    if { [ catch {
	puts_info 10 "Creating temporary local directory"
	copy_init $::localdir

	puts_info 10 "Creating server temporary directory"
	copy_init $::gass_server_dir

	validate_section $::SECTION [list FILENAME]
	set sourcefile [file join $::sourcedir \
			    $::TestEnv($::SECTION,FILENAME) ]
	puts_info 10 "sourcefile: $sourcefile"
	
	# Initialize the server
	puts_info 10 "Initialize the server"
	server::initialize
	
	# Client put operation
	client::put_operation $::server::url $sourcefile blocking
	
	# Shutdown the server
	server::shutdown

	# MD5 checksum
	check_md5sum \
	    $sourcefile \
	    [file join $::gass_server_dir [file tail $sourcefile]]
	
	puts_info 10 "Removing server temporary directory"
	copy_deinit $::gass_server_dir
	
	puts_info 10 "Removing temporary local directory"
	copy_init $::localdir
    } err ] } {
	return -code error $err
    }    
}
#------------------------------------------------------------------------
# Main
#------------------------------------------------------------------------
proc main { } {
    if { [ catch {
	# Globus activation
	globus::activation

	set ::url_scheme $::TestEnv($::SECTION,URL_SCHEME)
	
	# Initialize the client
	puts_info 10 "Initialize the client"
	client::initialize

	run_test

	puts_info 10 "Clean up the client"
	client::cleanup
		
	# Globus deactivation
	globus::deactivation
    } err ] } {
	err_handler $err
    }
	
    #--------------------------------------------------------------------
    # Exit with normal exit status since test completed without error.
    #--------------------------------------------------------------------
    TestExit
}

if { $::EXEC_MAIN } {
    main
}
    
