#------------------------------------------------------------------------
# GASS Transfer module activation
#------------------------------------------------------------------------
proc gass_module_activation { } {
    global globus_i_gass_copy_module
    global globus_i_gass_transfer_module
    #global globus_i_gass_cache_module
    #global globus_l_io_module
    
    if { [ catch {
	# Activate Globus GASS Copy module
	puts_info 10 "Activate GASS Copy module"
	globus_module_activate $globus_i_gass_copy_module

	# Activate Globus GASS Transfer module
	puts_info 10 "Activate GASS Transfer module"
	globus_module_activate $globus_i_gass_transfer_module

	# Activate Globus GASS Cache module
	#puts_info 10 "Activate GASS Cache module"
	#globus_module_activate $globus_i_gass_cache_module
	
	# Activate Globus I/O module
	#puts_info 10 "Activating I/O module"
	#globus_module_activate $globus_l_io_module
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# GASS Transfer module deactivation
#------------------------------------------------------------------------
proc gass_module_deactivation { } {
    global globus_i_gass_copy_module
    global globus_i_gass_transfer_module
    #global globus_i_gass_cache_module
    #global globus_l_io_module
    
    if { [ catch {
	# Deactivate Globus GASS Transfer module
	puts_info 10 "Deactivate GASS Transfer module"
	globus_module_deactivate $globus_i_gass_transfer_module

	# Deactivate Globus GASS Copy module
	puts_info 10 "Deactivate GASS Copy module"
	globus_module_deactivate $globus_i_gass_copy_module

	# Deactivate Globus GASS Cache module
	#puts_info 10 "Deactivate GASS Cache module"
	#globus_module_deactivate $globus_i_gass_cache_module
	
	# Deactivate Globus I/O module
	#puts_info 10 "Deactivate I/O module"
	#globus_module_deactivate $globus_l_io_module	
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Proc to initialize GASS Copy test
#------------------------------------------------------------------------
proc gass_copy_init { } {
    if { [ catch {
	# Handle attribute initialization
	set ::gha [ new_p_globus_gass_copy_handleattr_t ]
	puts_info 10 "gha: $::gha"
	globus_gass_copy_handleattr_init $::gha
	
	# Handle initialization
	set ::gh [ new_p_globus_gass_copy_handle_t ]
	puts_info 10 "gh: $::gh"
	globus_gass_copy_handle_init $::gh $::gha
	
	# Attribute initialization
	set ::srcAttr [ new_p_globus_gass_copy_attr_t ]
	puts_info 10 "srcAttr: $::srcAttr"

	set ::dstAttr [ new_p_globus_gass_copy_attr_t ]
	puts_info 10 "dstAttr: $::dstAttr"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Proc to cleanup GASS Copy test
#------------------------------------------------------------------------
proc gass_copy_cleanup { } {
    if { [ catch {
	# Destroy handle
	puts_info 10 "Executing globus_gass_copy_handle_destroy"
	globus_gass_copy_handle_destroy $::gh
	globus_gass_copy_handleattr_destroy $::gha
	
	# Reclaim memory space
	puts_info 10 "Reclaim memory space"
	delete_p_globus_gass_copy_attr_t $::srcAttr
	delete_p_globus_gass_copy_attr_t $::dstAttr
	delete_p_globus_gass_copy_handleattr_t $::gha
	delete_p_globus_gass_copy_handle_t $::gh
    } err ] } {
	return -code error $err
    }	
}
#------------------------------------------------------------------------
# Proc to initialize client GASS Copy test
#------------------------------------------------------------------------
proc gass_transfer_init { mode } {
    if { [ catch {
	#----------------------------------------------------------------
	# I/O attributes
	#----------------------------------------------------------------
	set ::fileattr    [ new_p_globus_io_attr_t ]
	puts_info 10 "fileattr: $::fileattr"
	puts_info 10 "fileattr value: [ p_globus_io_attr_t_value $::fileattr ]"
	
	puts_info 10 "globus_io_fileattr_init"
	globus_io_fileattr_init $::fileattr
	
	#----------------------------------------------------------------
	# Create GASS Transfer request handle and attributes
	#----------------------------------------------------------------
	set ::requestattr [ new_p_globus_gass_transfer_requestattr_t ]
	puts_info 10 "requestattr value: [ p_globus_gass_transfer_requestattr_t_value $::requestattr ]"

	if { $mode == "client" } {
	    #------------------------------------------------------------
	    # Initialize GASS Transfer request attributes
	    #------------------------------------------------------------
	    puts_info 10 "Initialize request attributes"
	    globus_gass_transfer_requestattr_init \
		$::requestattr \
		$::TestEnv($::SECTION,URL_SCHEME)
	    
	    puts_info 10 "Initialize connection reuse attribute"
	    globus_gass_transfer_requestattr_set_connection_reuse \
		$::requestattr \
		$::GLOBUS_TRUE
	    
	} elseif { $mode == "server" } {
	    set ::listener [ new_p_globus_gass_transfer_listener_t ]
	    set ::listenerattr [ new_p_globus_gass_transfer_listenerattr_t ]
	    
	    puts_info 10 "Testing globus_gass_transfer_listenerattr_init"
	    globus_gass_transfer_listenerattr_init \
		$::listenerattr \
		$::TestEnv($::SECTION,URL_SCHEME)
	    
	    puts_info 10 "Testing globus_gass_transfer_requestattr_init"
	    globus_gass_transfer_requestattr_init \
		$::requestattr \
		$::TestEnv($::SECTION,URL_SCHEME)
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_listenerattr_set_backlog
	    #------------------------------------------------------------
	    puts_info 10 "Testing globus_gass_transfer_listenerattr_set_backlog"
	    globus_gass_transfer_listenerattr_set_backlog $::listenerattr 1
	    
	    #------------------------------------------------------------
	    # globus_gass_transfer_listenerattr_set_port
	    #------------------------------------------------------------
	    puts_info 10 "Testing globus_gass_transfer_listenerattr_set_port"
	    globus_gass_transfer_listenerattr_set_port $::listenerattr 0
	}
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Proc to cleanup client GASS Copy test
#------------------------------------------------------------------------
proc gass_transfer_cleanup { mode } {
    if { [ catch {
	#----------------------------------------------------------------
	# Cleanup I/O resources
	#----------------------------------------------------------------
	puts_info 10 "Cleanup I/O resources"
	globus_io_fileattr_destroy $::fileattr
	delete_p_globus_io_attr_t $::fileattr
	
	#----------------------------------------------------------------
	# Cleanup GASS Transfer resources
	#----------------------------------------------------------------
	puts_info 10 "Cleanup GASS Transfer resources"

	globus_gass_transfer_requestattr_destroy $::requestattr
	delete_p_globus_gass_transfer_requestattr_t $::requestattr

	if { $mode == "server" } {
	    puts_info 10 "Destroy listener attributes"
	    #globus_gass_transfer_listenerattr_destroy $::listenerattr
	    delete_p_globus_gass_transfer_listenerattr_t $::listenerattr

	    delete_p_globus_gass_transfer_listener_t $::listener
	}
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Close callback
#------------------------------------------------------------------------
proc close_listener_cb { user_parms listener } {
    if { [ catch {
	puts_info 10 "Entering close_listener_cb"
	set ::cb_done 1
	puts_info 10 "Leaving close_listener_cb"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# Listener callback
#------------------------------------------------------------------------
proc listener_cb { user_parms listener } {
    if { [ catch {
	puts_info 10 "Entering listener_cb"
	set ::cb_done 1
	puts_info 10 "Leaving listener_cb"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# Accept callback
#------------------------------------------------------------------------
proc accept_cb { user_parms request } {
    if { [ catch {
	puts_info 10 "Entering accept_cb"
	set ::cb_done 1
	puts_info 10 "Leaving accept_cb"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# Client-initiated callback
#------------------------------------------------------------------------
proc client_initiated_cb { user_parms request } {
    if { [ catch {
	puts_info 10 "entering client_initiated_cb"
	
	upvar 0 $user_parms user_args

	puts_info 10 "\trequest: $request"

	set user_args(requesthandle) $request
	
	lassign [ globus_gass_transfer_request_get_status \
		      $request \
		     ] request_status
	puts_info 10 "\trequest status: $request_status"
	
	set ::cb_done 1

	puts_info 10 "leaving client_initiated_cb"
    } err ] } {
	err_handler $err
    }		
}
#------------------------------------------------------------------------
# Close callback
#------------------------------------------------------------------------
proc close_cb { user_parms handle result buffer } {
    if { [ catch {
	set ::cb_done 1
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# Send bytes callback
#------------------------------------------------------------------------
proc send_bytes_cb { user_parms request buffer length last_data } {
    if { [ catch {
	puts_info 20 "Entering send_bytes_cb"
	
	upvar 0 $user_parms user_args

	puts_info 20 "\trequest: $request"
	puts_info 20 "\tbuffer length: [string length $buffer]"
	puts_info 20 "\tlength: $length"
	puts_info 20 "\tlast_data: $last_data"

	if { ! $last_data } {
	    if { 1 } {
		lassign [ globus_gass_transfer_request_get_status \
			      $user_args(requesthandle) \
			     ] request_status
		puts_info 20 "\trequest status: $request_status"
	    }

	    if { 0 } {
		# Read the next data
		globus_io_register_read \
		    $user_args(readhandle) \
		    $::buffersize \
		    1 \
		    [ subst { read_bytes_cb $user_parms } ]
	    }

	    set buffer [ read $user_args(srcId) 1024 ]
	    puts_info 20 "\tbuffer length of Tcl next read: [string length $buffer]"

	    if { [string length $buffer] == 0 } {
		puts_info 20 "\tbuffer: $buffer"
		globus_gass_transfer_send_bytes \
		    $user_args(requesthandle) \
		    $buffer \
		    0 \
		    $::GLOBUS_TRUE \
		    [ subst { send_bytes_cb $user_parms } ]
	    } else {
		globus_gass_transfer_send_bytes \
		    $user_args(requesthandle) \
		    $buffer \
		    [string length $buffer] \
		    $::GLOBUS_FALSE \
		    [ subst { send_bytes_cb  $user_parms } ]
	    }

	    
	} else {
	    set ::cb_done 1
	}
	
	puts_info 20 "Leaving send_bytes_cb"
    } err ] } {
	err_handler $err
    }	
}
#------------------------------------------------------------------------
# Read bytes callback
#------------------------------------------------------------------------
proc read_bytes_cb { user_parms handle result buffer } {
    if { [ catch {
	puts_info 20 "Entering read_bytes_cb"
	
	upvar 0 $user_parms user_args

	puts_info 20 "\thandle: $handle"
	puts_info 20 "\tresult: $result"
	puts_info 20 "\tbuffer length: [string length $buffer]"	
	
	if { 0 } {
	    set user_args(filesize) [ expr $user_args(filesize) - \
					  [string length $buffer] ]
	    
	    puts_info 20 "\tfilesize: $user_args(filesize)"
	}
	
	if { $result != 0 } {
	    globus_gass_transfer_send_bytes \
		$user_args(requesthandle) \
		$buffer \
		0 \
		$::GLOBUS_TRUE \
		[ subst { send_bytes_cb $user_parms } ]
	} else {
	    globus_gass_transfer_send_bytes \
		$user_args(requesthandle) \
		$buffer \
		[string length $buffer] \
		$::GLOBUS_FALSE \
		[ subst { send_bytes_cb $user_parms } ]
	}
		
	puts_info 20 "Leaving read_bytes_cb"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# Write bytes callback
#------------------------------------------------------------------------
proc write_bytes_cb { user_parms handle result buffer } {
    if { [ catch {
	puts_info 20 "Entering write_bytes_cb"

	upvar 0 $user_parms user_args

	puts_info 20 "\thandle: $handle"
	puts_info 20 "\tresult: result"
	puts_info 20 "\tbuffer length: [string length $buffer]"
	puts_info 20 "\trecv last data: $user_args(recv_last_data)"

	if { $user_args(recv_last_data) == 1 } {
	    set ::cb_done 1
	} else {
	    globus_gass_transfer_receive_bytes \
		$user_args(requesthandle) \
		$::buffersize \
		1 \
		[ subst { recv_bytes_cb $user_parms } ]
	}
	puts_info 20 "Leaving write_bytes_cb"
    } err ] } {
	err_handler $err
    }	
}
#------------------------------------------------------------------------
# Recv bytes callback
#------------------------------------------------------------------------
proc recv_bytes_cb { user_parms request buffer length last_data } {
    if { [ catch {
	puts_info 20 "Entering recv_bytes_cb"
	
	upvar 0 $user_parms user_args
	
	puts_info 20 "\tbuffer length: [string length $buffer]"
	puts_info 20 "\tlength: $length"
	puts_info 20 "\tlast_data: $last_data"
	
	lassign [ globus_gass_transfer_request_get_status \
		      $user_args(requesthandle) \
		     ] request_status
	puts_info 20 "\trequest status: $request_status"
	
	# Get the next data
	#if { $last_data == 0 } {

	if { $last_data == 1 } {
	    set user_args(recv_last_data) 1
	} else {
	    set user_args(recv_last_data) 0
	}
	
	globus_io_register_write \
	    $user_args(writehandle) \
	    $buffer \
	    [ subst { write_bytes_cb \
			  $user_parms } ]
	
	#} else {
	if { 0 } {
	    globus_io_register_write \
		$user_args(writehandle) \
		$buffer \
		[ subst { close_cb NULL } ]
	}
	#}
	
	puts_info 20 "Leaving recv_bytes_cb"
    } err ] } {
	err_handler $err
    }	
}
#------------------------------------------------------------------------
# GASS Transfer client-initiated put operation
#------------------------------------------------------------------------
proc gass_transfer_put_operation { url gass_server_dir sourcefile mode } {
    global fileattr
    global requestattr
    global data
    global cb_done
    global buffersize
    
    if { [ catch {
	if { 0 } {
	    set data(readhandle)  [new_p_globus_io_handle_t]
	    puts_info 10 \
		"readhandle: [p_globus_io_handle_t_value $data(readhandle)]"
	}
	
	#----------------------------------------------------------------
	# Setup target file
	#----------------------------------------------------------------
	set targetfile $url
	append targetfile $gass_server_dir
	append targetfile "/"
	append targetfile [file tail $sourcefile]
	puts_info 10 "targetfile: $targetfile"
	
	if { 0 } {
	    #----------------------------------------------------------------
	    # globus_io_file_open
	    #----------------------------------------------------------------
	    puts_info 10 "globus_io_file_open"
	    globus_io_file_open \
		$sourcefile \
		$::GLOBUS_IO_FILE_RDONLY \
		0 \
		$fileattr \
		$data(readhandle)
	    
	    set data(filesize) [file size $sourcefile]
	}

	# Open srcfile
	set data(srcId) [ open $sourcefile r ]
	
	set filetype [ fileType $sourcefile ]
	if { [ fileType $sourcefile ] == "binary executable elf" } {
	    puts_info 10 "Binary mode"
	    fconfigure $data(srcId) -translation binary -encoding binary
	}
	
	

	#----------------------------------------------------------------
	# globus_gass_transfer_put
	#----------------------------------------------------------------
	if { $mode == "blocking" } {
	    puts_info 10 "Testing globus_gass_transfer_put"
	    lassign [ globus_gass_transfer_put \
			  $requestattr \
			  $targetfile \
			  $::GLOBUS_GASS_TRANSFER_LENGTH_UNKNOWN \
			 ] retval data(requesthandle)
	    puts_info 20 "globus_gass_transfer_put return value: $retval"
	    puts_info 10 "globus_gass_transfer_put request handle : $data(requesthandle)"
	}
	
	#----------------------------------------------------------------
	# globus_gass_transfer_register_put
	#----------------------------------------------------------------
	if { $mode == "nonblocking" } {
	    puts_info 10 "\tTesting globus_gass_transfer_register_put"
	    set cb_done 0
	    lassign [ globus_gass_transfer_register_put \
			  $requestattr \
			  $targetfile \
			  $::GLOBUS_GASS_TRANSFER_LENGTH_UNKNOWN \
			  { client_initiated_cb ::data } \
			 ] retval
	    puts_info 10 "\tglobus_gass_transfer_put return value: $retval"
	    vwait cb_done
	    puts_info 10 "\tglobus_gass_transfer_register_put: request handle: $data(requesthandle)"
	}
	
	#----------------------------------------------------------------
	# globus_gass_transfer_request_get_type
	#----------------------------------------------------------------
	lassign [ globus_gass_transfer_request_get_type \
		      $data(requesthandle) \
		     ] request_type
	puts_info 10 "request type: $request_type"
	
	#----------------------------------------------------------------
	# globus_gass_transfer_request_get_status
	#----------------------------------------------------------------
	lassign [ globus_gass_transfer_request_get_status \
		      $data(requesthandle) \
		     ] request_status
	puts_info 10 "request status: $request_status"
	
	#----------------------------------------------------------------
	# Request is denied
	#----------------------------------------------------------------
	if { [string equal $request_status \
		  "GLOBUS_GASS_TRANSFER_REQUEST_DENIED"] } {
	    lassign [ globus_gass_transfer_request_get_denial_reason \
			  $data(requesthandle) \
			 ] denial_reason
	    puts_info 10 "denial reason: $denial_reason"
	    
	    lassign [ globus_gass_transfer_request_get_denial_message \
			  $data(requesthandle) \
			 ] denial_message
	    puts_info 10 "denial message: $denial_message"
	} else {
	    #------------------------------------------------------------
	    # Request is granted
	    #------------------------------------------------------------
	    lassign [ globus_gass_transfer_request_get_url \
			  $data(requesthandle) \
			 ] serverurl
	    puts_info 10 "server url: $serverurl"
	    
	    #------------------------------------------------------------
	    # Determine transfer file mode, binary or text
	    #------------------------------------------------------------
	    if { 0 } {
		set filetype [ fileType $sourcefile ]
		if { [ fileType $sourcefile ] == "binary executable elf" } {
		    puts_info 10 "Initialize binary mode"
		    globus_gass_transfer_requestattr_set_file_mode \
			$requestattr \
			$::GLOBUS_GASS_TRANSFER_FILE_MODE_BINARY
		} else {
		    puts_info 10 "Initialize text mode"
		    globus_gass_transfer_requestattr_set_file_mode \
			$requestattr \
			$::GLOBUS_GASS_TRANSFER_FILE_MODE_TEXT
		}
	    }

	    if { 0 } {
		#------------------------------------------------------------
		# globus_io_register_read
		#------------------------------------------------------------
		set cb_done 0
		puts_info 10 "globus_io_register_read"
		globus_io_register_read \
		    $data(readhandle) \
		    $buffersize \
		    1 \
		    { read_bytes_cb ::data }
		vwait cb_done
	    }

	    set buffer [ read $data(srcId) 1024 ]

	    #if { [string length $buffer] < 1024 } {
	    #set last_data $::GLOBUS_TRUE
	    #} else {
	    #set last_data $::GLOBUS_FALSE
	    #}
	    
	    set cb_done 0
	    globus_gass_transfer_send_bytes \
		$data(requesthandle) \
		$buffer \
		[string length $buffer] \
		$::GLOBUS_FALSE \
		{ send_bytes_cb ::data }
	    vwait cb_done
	}

	puts_info 10 "globus_gass_transfer_request_destroy: $data(requesthandle)"

	close $data(srcId)
	
	globus_gass_transfer_request_destroy $data(requesthandle)
		
	#------------------------------------------------------------
	# Close I/O read handle
	#------------------------------------------------------------
	#globus_io_close $data(readhandle)	
	#delete_p_globus_io_handle_t $data(readhandle)
		
    } err ] } {
	return -code error $err
    }    
}
#------------------------------------------------------------------------
# GASS Transfer client-initiated get operation
#------------------------------------------------------------------------
proc gass_transfer_get_operation { url localdir gass_server_dir sourcefile mode } {
    global fileattr
    global requestattr
    global data
    global cb_done
    global request_status
    global buffersize
    
    if { [ catch {
	#----------------------------------------------------------------
	# Create read and write handles
	#----------------------------------------------------------------
	set data(writehandle) [ new_p_globus_io_handle_t ]
	puts_info 10 \
	    "writehandle: [p_globus_io_handle_t_value $data(writehandle)]"
	
	#----------------------------------------------------------------
	# Setup target file
	#----------------------------------------------------------------
	set remotefile $url
	append remotefile $gass_server_dir
	append remotefile "/"
	append remotefile [file tail $sourcefile]
	puts_info 10 "remotefile: $remotefile"
	
	#----------------------------------------------------------------
	# Write flags and modes
	#----------------------------------------------------------------
	set write_flags [ expr $::GLOBUS_IO_FILE_CREAT | \
			      $::GLOBUS_IO_FILE_TRUNC | \
			      $::GLOBUS_IO_FILE_WRONLY ]
	puts_info 10 "write_flags: $write_flags"
	
	set write_mode [ expr $::GLOBUS_IO_FILE_IRUSR | \
			     $::GLOBUS_IO_FILE_IWUSR ]
	puts_info 10 "write_mode: $write_mode"
	
	#----------------------------------------------------------------
	# globus_io_file_open for write
	#----------------------------------------------------------------
	puts_info 10 "globus_io_file_open"
	globus_io_file_open \
	    [file join $localdir [file tail $sourcefile]] \
	    $write_flags \
	    $write_mode \
	    $fileattr \
	    $data(writehandle) 

	#----------------------------------------------------------------
	# globus_gass_transfer_get
	#----------------------------------------------------------------
	if { $mode == "blocking" } { 
	    lassign [ globus_gass_transfer_get \
			  $requestattr \
			  $remotefile \
			 ] retval data(requesthandle)
	    puts_info 10 "globus_gass_transfer_get return value: $retval"
	    puts_info 10 "globus_gass_transfer_get request handle : $data(requesthandle)"
	}
	
	#----------------------------------------------------------------
	# globus_gass_transfer_register_get
	#----------------------------------------------------------------
	if { $mode == "nonblocking" } {
	    puts_info 10 "Testing globus_gass_transfer_register_get"
	    set cb_done 0
	    lassign [ globus_gass_transfer_register_get \
			  $requestattr \
			  $remotefile \
			  { client_initiated_cb ::data } \
			 ] retval
	    puts_info 10 "\tglobus_gass_transfer_put return value: $retval"
	    vwait cb_done
	    puts_info 10 "\tglobus_gass_transfer_get: request handle : $data(requesthandle)"
	}
	
	#------------------------------------------------------------
	# Request is granted
	#------------------------------------------------------------
	lassign [ globus_gass_transfer_request_get_url \
		      $data(requesthandle) \
		     ] serverurl
	puts_info 10 "server url: $serverurl"

	if { 0 } {
	    #----------------------------------------------------------------
	    # globus_gass_transfer_request_get_length
	    #----------------------------------------------------------------
	    lassign [ globus_gass_transfer_request_get_length \
			  $data(requesthandle) \
			 ] length
	    puts_info 10 "length: $length"
	}
	
	#----------------------------------------------------------------
	# globus_gass_transfer_request_get_type
	# globus_gass_transfer_request_get_status
	#----------------------------------------------------------------
	lassign [ globus_gass_transfer_request_get_type \
		      $data(requesthandle) \
		     ] request_type
	puts_info 10 "request type: $request_type"
	
	lassign [ globus_gass_transfer_request_get_status \
		      $data(requesthandle) \
		     ] request_status
	puts_info 10 "request status: $request_status"

	#----------------------------------------------------------------
	# Request is denied
	#----------------------------------------------------------------
	if { [string equal $request_status \
		  "GLOBUS_GASS_TRANSFER_REQUEST_DENIED"] } {
	    lassign [ globus_gass_transfer_request_get_denial_reason \
			  $data(requesthandle) \
			 ] denial_reason
	    puts_info 10 "denial reason: $denial_reason"
	    
	    lassign [ globus_gass_transfer_request_get_denial_message \
			  $data(requesthandle) \
			 ] denial_message
	    puts_info 10 "denial message: $denial_message"
	} else {
	    #------------------------------------------------------------
	    # The actual file transmission from GASS server.
	    #------------------------------------------------------------
	    set cb_done 0
	    globus_gass_transfer_receive_bytes \
		$data(requesthandle) \
		$buffersize \
		1 \
		{ recv_bytes_cb ::data } 
	    vwait cb_done
	}

	puts_info 10 "globus_gass_transfer_request_destroy: $data(requesthandle)"
	globus_gass_transfer_request_destroy $data(requesthandle)

	#------------------------------------------------------------
	# Close I/O read handle
	#------------------------------------------------------------
	globus_io_close $data(writehandle)	
	delete_p_globus_io_handle_t $data(writehandle)
    } err ] } {
	return -code error $err
    }       
}
	
