#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Load standard packages
#------------------------------------------------------------------------
package require Expect

source "$env(SRCDIR)/test_utils.tcl"
source "$env(SRCDIR)/test_gram_client_utils.tcl"

verifyProxy
    
foreach lib { globus_error \
		  globus_object \
		  globus_module \
		  globus_gram_client \
		  globus_gram_protocol_constants \
		  globus_gram_protocol } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

set done 0

#------------------------------------------------------------------------
# Start the personal gatekeeper
#------------------------------------------------------------------------
init_globus_personal_gatekeeper

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "[info nameofexecutable]"
    # Activate the modules that are to be used
    puts_info 10 "Activate GRAM module"
    globus_module_activate $globus_gram_client_module

    # Register allowable callback
    puts_info 10 "globus_gram_client_callback_allow"
    lassign [ globus_gram_client_callback_allow \
		  { done_cb NULL } \
		 ] status callback_contact
    puts_info 10 "\tstatus $status callback_contact $callback_contact"

    # Perform remote request
    set host "$::GRAM"
    set job "&(executable=/bin/sleep)(arguments=90)"

    puts_info 10 "globus_gram_client_job_request"
    lassign [ globus_gram_client_job_request \
		  $host \
		  $job \
		  $GLOBUS_GRAM_PROTOCOL_JOB_STATE_ALL \
		  $GLOBUS_NULL \
		 ] status job_contact
    puts_info 10 "\tstatus $status job_contact $job_contact"

    set job_state_mask [ expr $GLOBUS_GRAM_PROTOCOL_JOB_STATE_FAILED | \
			     $GLOBUS_GRAM_PROTOCOL_JOB_STATE_DONE ]
    
    puts_info 10 "globus_gram_client_job_callback_register"
    lassign [ globus_gram_client_job_callback_register \
		  $job_contact \
		  $job_state_mask \
		  $callback_contact \
		 ] status job_status failure_code
    puts_info 10 "\tstatus $status job_status $job_status failure_code $failure_code" 

    after 5000
    
    puts_info 10 "globus_gram_client_job_callback_unregister"
    lassign [ globus_gram_client_job_callback_unregister \
		  $job_contact \
		  $callback_contact \
		 ] status job_status failure_code
    puts_info 10 "\tstatus $status job_status $job_status failure_code $failure_code" 

    puts_info 10 "globus_gram_client_callback_disallow"
    lassign [ globus_gram_client_callback_disallow \
		  $callback_contact \
		 ] status
    puts_info 10 "\tstatus $status"
    
    # Deactivate the modules that are to be used
    puts_info 10 "Deactivate GRAM module"
    globus_module_deactivate $globus_gram_client_module
} err ] } {
    puts_fail "EXCEPTION: $err"
    terminate
}
terminate
