
#------------------------------------------------------------------------
# Start the personal gatekeeper
#------------------------------------------------------------------------
proc init_globus_personal_gatekeeper {} {
    set timeout 10
    if { [IsVerbose] < 0} {
	log_user 0
    }
    spawn -noecho globus-personal-gatekeeper -start
    expect {
	-re "^(.*GRAM.*)$" {
	    regsub {GRAM contact: (.*)$} $expect_out(1,string) {\1} ::GRAM
	    set ::GRAM [string trim $::GRAM]
	}
	timeout { puts_info 10 "took too much time"}
    }
}

#------------------------------------------------------------------------
# Terminate the personal gatekeeper
#------------------------------------------------------------------------
proc terminate {} {
    puts_info 10 "Entering terminate"
    puts_info 10 "\tKilling globus-personal-gatekeeper for $::GRAM"
    exec globus-personal-gatekeeper -kill $::GRAM 2>/dev/null
    puts_info 10 "\tKilling globus-personal-gatekeeper: no GRAM"
    exec globus-personal-gatekeeper -kill "" 2>/dev/null
    puts_info 10 "\tExiting with exit status: $::ExitStatus"
    puts_info 10 "Leaving terminate"
    TestExit
}

#------------------------------------------------------------------------
# Nonblocking callback
#------------------------------------------------------------------------
proc register_ping_cb { user_args job_contact job_state job_failure_code } {
    if  { [ catch {
	puts_info 10 "Entering register_ping_cb"

	puts_info 10 "\tuser_args $user_args job_contact $job_contact job_state $job_state job_failure_code $job_failure_code"

	set ::done 1
	
	puts_info 10 "Leaving register_ping_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Nonblocking callback
#------------------------------------------------------------------------
proc nonblocking_cb { user_args job_contact job_state job_failure_code } {
    if  { [ catch {
	puts_info 10 "Entering nonblocking_cb"

	puts_info 10 "\tuser_args $user_args job_contact $job_contact job_state $job_state job_failure_code $job_failure_code"

	puts_info 10 "Leaving nonblocking_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Nonblocking callback
#------------------------------------------------------------------------
proc job_registration_cb { user_args job_contact job_state job_failure_code } {
    if  { [ catch {
	puts_info 10 "Entering register_ping_cb"

	puts_info 10 "\tuser_args $user_args job_contact $job_contact job_state $job_state job_failure_code $job_failure_code"

	set ::done_1 1
	
	puts_info 10 "Leaving register_ping_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Job request callback
#------------------------------------------------------------------------
proc job_request_cb { user_args job_contact job_state job_failure_code } {
    if  { [ catch {
	puts_info 10 "Entering job_request_cb"
	
	puts_info 10 "\tuser_args $user_args job_contact $job_contact job_state $job_state job_failure_code $job_failure_code"

	puts_info 10 "\tglobus_gram_client_register_job_status"
	lassign [ globus_gram_client_register_job_status \
		      $job_contact \
		      NULL \
		      { nonblocking_cb NULL } \
		     ] status
	puts_info 10 "\t\tstatus $status"
	
	puts_info 10 "Leaving job_request_cb"
    } err ] } {
	err_handler $err
    }	
}

#------------------------------------------------------------------------
# Done callback
#------------------------------------------------------------------------
proc done_cb {user_args job_contact job_state errorcode} {
    global done
    if  { [ catch {
	puts_info 10 "Entering done_cb"
	puts_info 10 "\tuser_args $user_args job_contact $job_contact job_state $job_state errorcode $errorcode"

	if { $job_state == $::GLOBUS_GRAM_PROTOCOL_JOB_STATE_PENDING} {
	    puts_info 10 "\tGot GLOBUS_GRAM_PROTOCOL_JOB_STATE_PENDING from job manager"
	} elseif { $job_state == $::GLOBUS_GRAM_PROTOCOL_JOB_STATE_ACTIVE } {
	    puts_info 10 "\tGot GLOBUS_GRAM_PROTOCOL_JOB_STATE_ACTIVE from job manager"
	} elseif { $job_state == $::GLOBUS_GRAM_PROTOCOL_JOB_STATE_FAILED } {
	    set err [ globus_gram_protocol_error_string $errorcode ]
	    puts_info 10 "\tFailed with: $err"
	    set done 1
	} elseif { $job_state == $::GLOBUS_GRAM_PROTOCOL_JOB_STATE_DONE } {
	    puts_info 10 "\tGRAM job is completed"
	    set done 1
	} 
	puts_info 10 "Leaving done_cb"
    } err ] } {
	set done 1
	err_handler $err
    }
}
