#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Load standard packages
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
    
foreach lib { globus_error \
		  globus_object \
		  globus_module \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    globus_module_activate $globus_i_gsi_gssapi_module
    
    set cred_handle [ new_p_gss_cred_id_t ]
    puts_info 10 "cred_handle $cred_handle"

    set status_string [ new_p_gss_buffer_desc ]
    puts_info 10 "status_string $status_string"
    
    set buffer_set [ new_p_gss_buffer_set_t ]
    puts_info 10 "buffer_set $buffer_set"
    
    set oid_set [ new_p_gss_OID_set ]
    puts_info 10 "oid_set $oid_set"
    
    # Create an empty OID set
    puts_info 10 "gss_create_empty_oid_set"
    lassign [ gss_create_empty_oid_set \
		  $oid_set \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    # Determine underlying security mechanisms
    puts_info 10 "gss_indicate_mechs"
    lassign [ gss_indicate_mechs \
		  $oid_set \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    # Add GSS_C_NT_USER_NAME gss_oid_set
    puts_info 10 "gss_add_oid_set_member"
    lassign [ gss_add_oid_set_member \
		  $GSS_C_NT_USER_NAME \
		  $oid_set \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    # Inquire GSS_C_NT_USER_NAME 
    puts_info 10 "p_gss_ctx_id_t_value"
    lassign [ p_gss_OID_set_value \
		  $oid_set \
		 ] oid_set_addr_value
    puts_info 10 "\toid_set_addr_value $oid_set_addr_value"
    
    puts_info 10 "gss_test_oid_set_member"
    lassign [ gss_test_oid_set_member \
		  $GSS_C_NT_USER_NAME \
		  $oid_set_addr_value \
		 ] maj_stat min_stat present
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tpresent $present"
    
    # Create an empty buffer set
    puts_info 10 "gss_create_empty_buffer_set"
    lassign [ gss_create_empty_buffer_set \
		  $buffer_set \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    # Acquire the credential
    puts_info 10 "gss_acquire_cred"
    lassign [ gss_acquire_cred \
		  NULL \
		  $GSS_C_INDEFINITE \
		  $gss_c_no_oid_set \
		  $GSS_C_BOTH \
		  $cred_handle \
		  NULL \
		 ] maj_stat min_stat time_rec
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\ttime_rec $time_rec"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_acquire_cred"
	TestExit
    }

    if { 0 } {
	# Inquire credential by OID mechanism
	set name [ new_p_gss_name_t ]
	
	puts_info 10 "gss_inquire_cred_by_mech"
	lassign [ gss_inquire_cred_by_mech \
		      $gss_c_no_credential \
		      $GSS_C_NT_USER_NAME \
		      $name \
		      NULL \
		     ] maj_stat min_stat initiator_lifetime acceptor_lifetime 
	puts_info 10 "\tmaj_stat $maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	puts_info 10 "\tinitiator_lifetime $initiator_lifetime"
	puts_info 10 "\tacceptor_lifetime $acceptor_lifetime"
	
	puts_info 10 "gss_inquire_names_for_mech"
	lassign [ gss_inquire_names_for_mech \
		      $GSS_C_NT_USER_NAME \
		      $oid_set \
		     ] maj_stat min_stat
	puts_info 10 "\tmaj_stat $maj_stat"
	puts_info 10 "\tmin_stat $min_stat"

	lassign [ p_gss_name_t_value \
		      $name \
		     ] name_addr_value

	puts_info 10 "gss_inquire_mechs_for_name"
	lassign [ gss_inquire_mechs_for_name \
		      $name_addr_value \
		      $oid_set \
		     ] maj_stat min_stat
	puts_info 10 "\tmaj_stat $maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
    }
    
    puts_info 10 "gss_display_status"
    lassign [ gss_display_status \
		  $maj_stat \
		  $GSS_C_GSS_CODE \
		  $gss_c_null_oid \
		 ] maj_stat min_stat message_context status_string
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tmessage_context $message_context"
    puts_info 10 "\tstatus_string $status_string"
    
    # Destroy the credential
    puts_info 10 "gss_release_cred"
    lassign [ gss_release_cred \
		  $cred_handle \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_cred"
	TestExit
    }

    if { 0 } {
	# Release buffer
	puts_info 10 "gss_release_buffer"
	lassign [ gss_release_buffer \
		      $status_string \
		     ] maj_stat min_stat
	puts_info 10 "\tmaj_stat $maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	
	if { $maj_stat != $GSS_S_COMPLETE } {
	    puts_fail "ERR: gss_release_buffer"
	    TestExit
	}
    }

    # Release buffer empty
    puts_info 10 "gss_release_buffer_set"
    lassign [ gss_release_buffer_set \
		  $buffer_set \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    
    # Release OID set 
    puts_info 10 "gss_release_oid_set"
    lassign [ gss_release_oid_set \
		  $oid_set \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    delete_p_gss_cred_id_t $cred_handle
    
    globus_module_deactivate $globus_i_gsi_gssapi_module
} err ] } {
    puts_fail "EXCEPTION: $err"
}

#---------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#---------------------------------------------------------------------
TestExit
