#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Load standard packages
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
    
foreach lib { globus_error \
		  globus_object \
		  globus_module \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    globus_module_activate $globus_i_gsi_gssapi_module
    
    set cred_handle [ new_p_gss_cred_id_t ]
    
    set init_context [ new_p_gss_ctx_id_t ]
    set accept_context [ new_p_gss_ctx_id_t ]

    set send_tok [ new_p_gss_buffer_desc ]
    set recv_tok [ new_p_gss_buffer_desc ]
    set output_name [ new_p_gss_buffer_desc ]

    set target_name [ new_p_gss_name_t ]
    set source_name [ new_p_gss_name_t ]
    set alias_name [ new_p_gss_name_t ]
    
    set mech_type [ new_p_gss_OID ]

    set data_set [ new_p_gss_buffer_set_t ]
    
    ####################################################################
    # Initialize variables
    ####################################################################
    p_gss_cred_id_t_assign $cred_handle $gss_c_no_credential
    
    p_gss_ctx_id_t_assign $init_context $gss_c_no_context
    p_gss_ctx_id_t_assign $accept_context $gss_c_no_context

    set subject "/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
    
    ####################################################################
    # Acquire the credential
    ####################################################################
    puts_info 10 "gss_acquire_cred"
    lassign [ gss_acquire_cred \
		  NULL \
		  $GSS_C_INDEFINITE \
		  $gss_c_no_oid_set \
		  $GSS_C_BOTH \
		  $cred_handle \
		  NULL \
		 ] maj_stat min_stat time_rec
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tcred_handle $cred_handle"
    puts_info 10 "\ttime_rec $time_rec"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_acquire_cred"
	TestExit
    }

    puts_info 10 "gss_buffer_desc_value_set"
    gss_buffer_desc_value_set \
	$send_tok \
	$subject

    puts_info 10 "gss_buffer_desc_length_set"
    gss_buffer_desc_length_set \
	$send_tok \
	[string length $subject]
    
    ####################################################################
    # Import the subject name
    ####################################################################
    puts_info 10 "gss_import_name"
    lassign [ gss_import_name \
		  $send_tok \
		  $GSS_C_NT_USER_NAME \
		  $target_name \
		 ] maj_stat min_stat 
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\ttarget_name $target_name"
    
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_import_name"
	TestExit
    }

    puts_info 10 "p_gss_name_t_value"
    lassign [ p_gss_name_t_value \
		  $target_name \
		 ] target_name_addr_value
    puts_info 10 "\ttarget_name_addr_value $target_name_addr_value"

    puts_info 10 "gss_canonicalize_name"
    lassign [ gss_canonicalize_name \
		  $target_name_addr_value \
		  $GSS_C_NT_USER_NAME \
		  NULL \
		 ] maj_stat min_stat 
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    ####################################################################
    # Export the subject name
    ####################################################################
    puts_info 10 "p_gss_name_t_value"
    lassign [ p_gss_name_t_value \
		  $target_name \
		 ] target_name_addr_value
    puts_info 10 "\ttarget_name_addr_value $target_name_addr_value"

    set exported_name [ new_p_gss_buffer_desc ]
    
    puts_info 10 "gss_export_name"
    lassign [ gss_export_name \
		  $target_name_addr_value \
		  $exported_name \
		 ] maj_stat min_stat 
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    puts_info 10 "p_gss_name_t_value"
    lassign [ p_gss_name_t_value \
		  $target_name \
		 ] target_name_addr_value
    puts_info 10 "\ttarget_name_addr_value $target_name_addr_value"

    puts_info 10 "gss_duplicate_name"
    lassign [ gss_duplicate_name \
		  $target_name_addr_value \
		  $alias_name \
		 ] maj_stat min_stat 
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    puts_info 10 "p_gss_name_t_value"
    lassign [ p_gss_name_t_value \
		  $alias_name \
		 ] alias_name_addr_value
    puts_info 10 "\talias_name_addr_value $alias_name_addr_value"

    puts_info 10 "gss_compare_name"
    lassign [ gss_compare_name \
		  $target_name_addr_value \
		  $alias_name_addr_value \
		 ] maj_stat min_stat name_equal
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tname_equal $name_equal"
    
    ####################################################################
    # Set up the first security context
    ####################################################################
    set req_flags [ expr $GSS_C_ANON_FLAG | $GSS_C_CONF_FLAG ]

    puts_info 10 "p_gss_name_t_value"
    lassign [ p_gss_name_t_value \
		  $target_name \
		 ] target_name_addr_value
    puts_info 10 "\ttarget_name_addr_value $target_name_addr_value"
    
    puts_info 10 "gss_init_sec_context"
    lassign [ gss_init_sec_context \
		  $gss_c_no_credential \
		  $init_context \
		  $target_name_addr_value \
		  $gss_c_null_oid \
		  $req_flags \
		  0 \
		  $gss_c_no_channel_bindings \
		  $gss_c_no_buffer \
		  NULL \
		  $send_tok \
		 ] init_maj_stat min_stat ret_flags time_rec
    puts_info 10 "\tinit_maj_stat $init_maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tret_flags $ret_flags"
    puts_info 10 "\ttime_rec $time_rec"
    puts_info 10 "\tinit_context $init_context"

    if { $init_maj_stat != $GSS_S_COMPLETE && \
	     $init_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	puts_fail "ERR: gss_init_sec_context"
	TestExit
    }
    
    while 1 {
	puts_info 10 "gss_accept_sec_context"
	lassign [ gss_accept_sec_context \
		      $accept_context \
		      $gss_c_no_credential \
		      $send_tok \
		      $gss_c_no_channel_bindings \
		      $source_name \
		      $mech_type \
		      $recv_tok \
		      $gss_c_no_credential \
		     ] accept_maj_stat min_stat ret_flags time_rec
	puts_info 10 "\taccept_maj_stat $accept_maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	puts_info 10 "\tsource_name $source_name"
	puts_info 10 "\tret_flags $ret_flags"
	puts_info 10 "\ttime_rec $time_rec"

	if { $accept_maj_stat != $GSS_S_COMPLETE && \
		 $accept_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	    puts_fail "ERR: gss_accept_sec_context"
	    TestExit
	} elseif { $accept_maj_stat == $GSS_S_COMPLETE } {
	    break
	}
	
	puts_info 10 "gss_init_sec_context"
	lassign [ gss_init_sec_context \
		      $gss_c_no_credential \
		      $init_context \
		      $target_name_addr_value \
		      $gss_c_null_oid \
		      $req_flags \
		      0 \
		      $gss_c_no_channel_bindings \
		      $recv_tok \
		      NULL \
		      $send_tok \
		     ] init_maj_stat min_stat ret_flags time_rec
	puts_info 10 "\tinit_maj_stat $init_maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	puts_info 10 "\tret_flags $ret_flags"
	puts_info 10 "\ttime_rec $time_rec"

	if { $init_maj_stat != $GSS_S_COMPLETE && \
		 $init_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	    puts_fail "ERR: gss_init_sec_context"
	    TestExit
	}
    }

    if { 0 } {
	lassign [ p_gss_ctx_id_t_value \
		      $accept_context \
		     ] accept_context_addr_value
	
	puts_info 10 "gss_inquire_sec_context_by_oid"
	lassign [ gss_inquire_sec_context_by_oid \
		      $accept_context_addr_value \
		      $gss_c_null_oid \
		      $gss_c_no_buffer_set \
		     ] maj_stat min_stat
	puts_info 10 "\tmaj_stat $maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
    }
    
    set context_token [ new_p_gss_buffer_t ]

    lassign [ p_gss_buffer_t_value \
		  $context_token \
		 ] context_token_addr_value

    puts_info 10 "p_gss_ctx_id_t_value"
    lassign [ p_gss_ctx_id_t_value \
		  $init_context \
		 ] init_context_addr_value
    puts_info 10 "\tinit_context_addr_value $init_context_addr_value"

    puts_info 10 "gss_export_sec_context"
    lassign [ gss_export_sec_context \
		  $init_context \
		  $context_token_addr_value \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    puts_info 10 "gss_import_sec_context"
    lassign [ gss_import_sec_context \
		  $context_token_addr_value \
		  $init_context \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    puts_info 10 "Successfully established anonymous security context"
    
    ####################################################################
    # Inquire security context
    ####################################################################
    puts_info 10 "p_gss_ctx_id_t_value"
    lassign [ p_gss_ctx_id_t_value \
		  $init_context \
		 ] init_context_addr_value
    puts_info 10 "\tinit_context_addr_value $init_context_addr_value"

    puts_info 10 "p_gss_ctx_id_t_value"
    lassign [ p_gss_ctx_id_t_value \
		  $accept_context \
		 ] accept_context_addr_value
    puts_info 10 "\taccept_context_addr_value $accept_context_addr_value"

    puts_info 10 "gss_inquire_context"
    lassign [ gss_inquire_context \
		  $accept_context_addr_value \
		  $source_name \
		  $target_name \
		  $mech_type \
		 ] maj_stat min_stat lifetime_rec ctx_flags locally_initiated open
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tlifetime_rec $lifetime_rec"
    puts_info 10 "\tctx_flags $ctx_flags"
    puts_info 10 "\tlocally_initiated $locally_initiated"
    puts_info 10 "\topen $open"
    
    ####################################################################
    # Display name
    ####################################################################
    puts_info 10 "gss_display_name"
    lassign [ gss_display_name \
		  $source_name \
		  $output_name \
		  NULL \
		 ] maj_stat min_stat recv_subject_name
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\trecv_subject_name $recv_subject_name"

    ####################################################################
    # Delete security context
    ####################################################################
    puts_info 10 "gss_delete_sec_context"
    lassign [ gss_delete_sec_context \
		  $init_context \
		  $output_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_delete_sec_context"
	TestExit
    }

    puts_info 10 "gss_delete_sec_context"
    lassign [ gss_delete_sec_context \
		  $accept_context \
		  $output_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_delete_sec_context"
	TestExit
    }

    ####################################################################
    # Release target name
    ####################################################################
    puts_info 10 "gss_release_name"
    lassign [ gss_release_name \
		  $target_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
	
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_name"
	TestExit
    }

    puts_info 10 "gss_release_name"
    lassign [ gss_release_name \
		  $source_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
	
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_name"
	TestExit
    }

    ####################################################################
    # Destroy the credential
    ####################################################################
    puts_info 10 "gss_release_cred"
    lassign [ gss_release_cred \
		  $cred_handle \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_cred"
	TestExit
    }

    ####################################################################
    # Release buffer
    ####################################################################
    puts_info 10 "gss_release_buffer"
    lassign [ gss_release_buffer \
		  $recv_tok \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_buffer"
	TestExit
    }
    
    puts_info 10 "gss_release_buffer"
    lassign [ gss_release_buffer \
		  $send_tok \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_buffer"
	TestExit
    }

    puts_info 10 "gss_release_buffer"
    lassign [ gss_release_buffer \
		  $output_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_buffer"
	TestExit
    }

    ####################################################################
    # Delete all pointers
    ####################################################################
    delete_p_gss_cred_id_t $cred_handle
    
    delete_p_gss_buffer_desc $send_tok
    delete_p_gss_buffer_desc $recv_tok
    delete_p_gss_buffer_desc $output_name
    
    delete_p_gss_name_t $target_name
    delete_p_gss_name_t $source_name
    
    delete_p_gss_ctx_id_t $init_context
    delete_p_gss_ctx_id_t $accept_context
    
    delete_p_gss_OID $mech_type

    globus_module_deactivate $globus_i_gsi_gssapi_module
} err ] } {
    puts_fail "EXCEPTION: $err"
}

#---------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#---------------------------------------------------------------------
TestExit
