#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Load standard packages
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
    
foreach lib { globus_error \
		  globus_object \
		  globus_module \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    globus_module_activate $globus_i_gsi_gssapi_module
    
    set cred_handle [ new_p_gss_cred_id_t ]
    set delegated_cred_handle [ new_p_gss_cred_id_t ]
    set imported_cred_handle [ new_p_gss_cred_id_t ]

    set send_tok [ new_p_gss_buffer_desc ]
    set recv_tok [ new_p_gss_buffer_desc ]
    set output_name [ new_p_gss_buffer_desc ]

    set target_name [ new_p_gss_name_t ]
    set source_name [ new_p_gss_name_t ]
    
    set init_context [ new_p_gss_ctx_id_t ]
    set accept_context [ new_p_gss_ctx_id_t ]
    set del_init_context [ new_p_gss_ctx_id_t ]
    set del_accept_context [ new_p_gss_ctx_id_t ]
    
    set mech_type [ new_p_gss_OID ]

    # Initialize variables
    p_gss_ctx_id_t_assign $init_context $gss_c_no_context
    p_gss_ctx_id_t_assign $accept_context $gss_c_no_context
    p_gss_ctx_id_t_assign $del_init_context $gss_c_no_context
    p_gss_ctx_id_t_assign $del_accept_context $gss_c_no_context
    
    ####################################################################
    # Acquire the credential
    ####################################################################
    puts_info 10 "gss_acquire_cred"
    lassign [ gss_acquire_cred \
		  NULL \
		  $GSS_C_INDEFINITE \
		  $gss_c_no_oid_set \
		  $GSS_C_BOTH \
		  $cred_handle \
		  NULL \
		 ] maj_stat min_stat time_rec 
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tcred_handle $cred_handle"
    puts_info 10 "\ttime_rec $time_rec"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_acquire_cred"
	TestExit
    }

    ####################################################################
    # Get the subject name
    ####################################################################
    puts_info 10 "p_gss_cred_id_t_value"
    lassign [ p_gss_cred_id_t_value \
		  $cred_handle \
		 ] cred_handle_addr_value
    puts_info 10 "\tcred_handle_addr_value $cred_handle_addr_value"
    
    puts_info 10 "gss_inquire_cred"
    lassign [ gss_inquire_cred \
		  $cred_handle_addr_value \
		  $target_name \
		  NULL \
		  NULL \
		 ] maj_stat min_stat lifetime
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\ttarget_name $target_name"
    puts_info 10 "\tlifetime $lifetime"
    
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_inquire_cred"
	TestExit
    }

    ####################################################################
    # Set up the first security context
    ####################################################################
    puts_info 10 "p_gss_name_t_value"
    lassign [ p_gss_name_t_value \
		  $target_name \
		 ] target_name_addr_value
    puts_info 10 "\ttarget_name_addr_value $target_name_addr_value"
    
    puts_info 10 "gss_init_sec_context"
    lassign [ gss_init_sec_context \
		  $cred_handle_addr_value \
		  $init_context \
		  $target_name_addr_value \
		  $gss_c_null_oid \
		  0 \
		  0 \
		  $gss_c_no_channel_bindings \
		  $gss_c_no_buffer \
		  NULL \
		  $send_tok \
		 ] init_maj_stat min_stat ret_flags time_rec
    puts_info 10 "\tinit_maj_stat $init_maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tret_flags $ret_flags"
    puts_info 10 "\ttime_rec $time_rec"
    puts_info 10 "\tinit_context $init_context"

    if { $init_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	puts_fail "ERR: gss_init_sec_context"
	TestExit
    }
        
    while 1 {
	puts_info 10 "gss_accept_sec_context"
	lassign [ gss_accept_sec_context \
		      $accept_context \
		      $gss_c_no_credential \
		      $send_tok \
		      $gss_c_no_channel_bindings \
		      NULL \
		      $mech_type \
		      $recv_tok \
		      $gss_c_no_credential \
		     ] accept_maj_stat min_stat ret_flags time_rec
	puts_info 10 "\taccept_maj_stat $accept_maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	puts_info 10 "\tret_flags $ret_flags"
	puts_info 10 "\ttime_rec $time_rec"

	if { $accept_maj_stat != $GSS_S_COMPLETE && \
		 $accept_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	    puts_fail "ERR: gss_accept_sec_context"
	    TestExit
	} elseif { $accept_maj_stat == $GSS_S_COMPLETE } {
	    break
	}

	puts_info 10 "gss_init_sec_context"
	lassign [ gss_init_sec_context \
		      $gss_c_no_credential \
		      $init_context \
		      $target_name_addr_value \
		      $gss_c_null_oid \
		      0 \
		      0 \
		      $gss_c_no_channel_bindings \
		      $recv_tok \
		      NULL \
		      $send_tok \
		     ] init_maj_stat min_stat ret_flags time_rec
	puts_info 10 "\tinit_maj_stat $init_maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	puts_info 10 "\tret_flags $ret_flags"
	puts_info 10 "\ttime_rec $time_rec"

	if { $init_maj_stat != $GSS_S_COMPLETE && \
		 $init_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	    puts_fail "ERR: gss_init_sec_context"
	    TestExit
	}	
    }

    puts_info 10 "Successfully established initial security context"

    ####################################################################
    # Delegate our credential over the initial security context and
    # insert a restriction extension into the delegated credential.
    # This is a post GT 2.0 feature.
    ####################################################################
    puts_info 10 "p_gss_ctx_id_t_value"
    lassign [ p_gss_ctx_id_t_value \
		  $init_context \
		 ] init_context_addr_value
    puts_info 10 "\tinit_context_addr_value $init_context_addr_value"
    
    puts_info 10 "gss_init_delegation"
    lassign [ gss_init_delegation \
		  $init_context_addr_value\
		  $cred_handle_addr_value \
		  $gss_c_null_oid \
		  $gss_c_no_oid_set \
		  $gss_c_no_buffer_set \
		  $gss_c_no_buffer \
		  0 \
		  0 \
		  $send_tok \
		 ] init_maj_stat min_stat
    puts_info 10 "\tinit_maj_stat $init_maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    if { $init_maj_stat != $GSS_S_COMPLETE && \
	     $init_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	puts_fail "ERR: gss_init_delegation"
	TestExit
    } 

    puts_info 10 "p_gss_ctx_id_t_value"
    lassign [ p_gss_ctx_id_t_value \
		  $accept_context \
		 ] accept_context_addr_value
    puts_info 10 "\taccept_context_addr_value $accept_context_addr_value"

    while 1 {
	puts_info 10 "gss_accept_delegation"
	lassign [ gss_accept_delegation \
		      $accept_context_addr_value \
		      $gss_c_no_oid_set \
		      $gss_c_no_buffer_set \
		      $send_tok \
		      0 \
		      0 \
		      $delegated_cred_handle \
		      $mech_type \
		      $recv_tok \
		     ] accept_maj_stat min_stat time_rec 
	puts_info 10 "\taccept_maj_stat $accept_maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	puts_info 10 "\ttime_rec $time_rec"
	puts_info 10 "\tdelegated_cred_handle $delegated_cred_handle"

	if { $accept_maj_stat != $GSS_S_COMPLETE && \
		 $accept_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	    puts_fail "ERR: gss_accept_delegation"
	    TestExit
	} elseif { $accept_maj_stat == $GSS_S_COMPLETE } {
	    break
	}

	puts_info 10 "gss_init_delegation"
	lassign [ gss_init_delegation \
		      $init_context_addr_value \
		      $cred_handle_addr_value \
		      $gss_c_null_oid \
		      $gss_c_no_oid_set \
		      $gss_c_no_buffer_set \
		      $recv_tok \
		      0 \
		      0 \
		      $send_tok \
		     ] init_maj_stat min_stat
	puts_info 10 "\tinit_maj_stat $init_maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	
	if { $init_maj_stat != $GSS_S_COMPLETE && \
		 $init_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	    puts_fail "ERR: gss_init_delegation"
	    TestExit
	} 
    }

    puts_info 10 "Successfully delegated credential"

    ####################################################################
    # Export and import the delegated credential
    ####################################################################
    puts_info 10 "p_gss_cred_id_t_value"
    lassign [ p_gss_cred_id_t_value \
		  $delegated_cred_handle \
		 ] delegated_cred_handle_addr_value
    puts_info 10 "\tdelegated_cred_handle_addr_value $delegated_cred_handle_addr_value"
    
    puts_info 10 "gss_export_cred"
    lassign [ gss_export_cred \
		  $delegated_cred_handle_addr_value \
		  $gss_c_null_oid \
		  0 \
		  $send_tok \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    puts_info 10 "gss_import_cred"
    lassign [ gss_import_cred \
		  $imported_cred_handle \
		  $gss_c_null_oid \
		  0 \
		  $send_tok \
		  0 \
		 ] maj_stat min_stat time_rec
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\timported_cred_handle $imported_cred_handle"
    puts_info 10 "\ttime_rec $time_rec"
    
    puts_info 10 "Successfully exported/imported the delegated credential"
    
    ####################################################################
    # Set up another security context using the delegated credential
    ####################################################################
    puts_info 10 "p_gss_cred_id_t_value"
    lassign [ p_gss_cred_id_t_value \
		  $imported_cred_handle \
		 ] imported_cred_handle_addr_value
    puts_info 10 "\timported_cred_handle_addr_value $imported_cred_handle_addr_value"

    puts_info 10 "gss_init_sec_context"
    lassign [ gss_init_sec_context \
		  $imported_cred_handle_addr_value \
		  $del_init_context \
		  $target_name_addr_value \
		  $gss_c_null_oid \
		  0 \
		  0 \
		  $gss_c_no_channel_bindings \
		  $gss_c_no_buffer \
		  NULL \
		  $send_tok \
		 ] init_maj_stat min_stat ret_flags time_rec
    puts_info 10 "\tinit_maj_stat $init_maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tret_flags $ret_flags"
    puts_info 10 "\ttime_rec $time_rec"

    
    if { $init_maj_stat != $GSS_S_COMPLETE && \
	     $init_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	puts_fail "ERR: gss_init_sec_context"
	TestExit
    }

    while 1 {
	puts_info 10 "gss_accept_sec_context"
	lassign [ gss_accept_sec_context \
		      $del_accept_context \
		      $imported_cred_handle_addr_value \
		      $send_tok \
		      $gss_c_no_channel_bindings \
		      $target_name \
		      $mech_type \
		      $recv_tok \
		      $gss_c_no_credential \
		     ] accept_maj_stat min_stat ret_flags time_rec
	puts_info 10 "\taccept_maj_stat $accept_maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	puts_info 10 "\tret_flags $ret_flags"
	puts_info 10 "\ttime_rec $time_rec"

	if { $accept_maj_stat != $GSS_S_COMPLETE && \
		 $accept_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	    puts_fail "ERR: gss_accept_sec_context"
	    TestExit
	} elseif { $accept_maj_stat == $GSS_S_COMPLETE } {
	    break
	}

	puts_info 10 "gss_init_sec_context"
	lassign [ gss_init_sec_context \
		      $imported_cred_handle_addr_value \
		      $del_init_context \
		      $target_name_addr_value \
		      $gss_c_null_oid \
		      0 \
		      0 \
		      $gss_c_no_channel_bindings \
		      $recv_tok \
		      NULL \
		      $send_tok \
		     ] init_maj_stat min_stat ret_flags time_rec
	puts_info 10 "\tinit_maj_stat $init_maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	puts_info 10 "\tret_flags $ret_flags"
	puts_info 10 "\ttime_rec $time_rec"

	if { $init_maj_stat != $GSS_S_COMPLETE && \
		 $init_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	    puts_fail "ERR: gss_init_sec_context"
	    TestExit
	}	
    }

    puts_info 10 "Successfully established security context with delegated credential"


    ####################################################################
    # Display name
    ####################################################################
    puts_info 10 "gss_display_name"
    lassign [ gss_display_name \
		  $target_name \
		  $output_name \
		  NULL \
		 ] maj_stat min_stat recv_subject_name
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\trecv_subject_name $recv_subject_name"

    
    ####################################################################
    # Release target name
    ####################################################################
    puts_info 10 "gss_release_name"
    lassign [ gss_release_name \
		  $target_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
	
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_name"
	TestExit
    }

    puts_info 10 "gss_release_name"
    lassign [ gss_release_name \
		  $source_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
	
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_name"
	TestExit
    }

    ####################################################################
    # Delete security context
    ####################################################################
    puts_info 10 "gss_delete_sec_context"
    lassign [ gss_delete_sec_context \
		  $init_context \
		  $output_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_delete_sec_context"
	TestExit
    }

    puts_info 10 "gss_delete_sec_context"
    lassign [ gss_delete_sec_context \
		  $accept_context \
		  $output_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_delete_sec_context"
	TestExit
    }

    puts_info 10 "gss_delete_sec_context"
    lassign [ gss_delete_sec_context \
		  $del_init_context \
		  $output_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_delete_sec_context"
	TestExit
    }

    puts_info 10 "gss_delete_sec_context"
    lassign [ gss_delete_sec_context \
		  $del_accept_context \
		  $output_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_delete_sec_context"
	TestExit
    }

    ####################################################################
    # Release buffer
    ####################################################################
    puts_info 10 "gss_release_buffer"
    lassign [ gss_release_buffer \
		  $send_tok \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_buffer"
	TestExit
    }

    puts_info 10 "gss_release_buffer"
    lassign [ gss_release_buffer \
		  $recv_tok \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_buffer"
	TestExit
    }

    puts_info 10 "gss_release_buffer"
    lassign [ gss_release_buffer \
		  $output_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_buffer"
	TestExit
    }

    ####################################################################
    # Destroy the credential
    ####################################################################
    puts_info 10 "gss_release_cred"
    lassign [ gss_release_cred \
		  $cred_handle \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_cred"
	TestExit
    }

    ####################################################################
    # Release resources 
    ####################################################################
    delete_p_gss_cred_id_t $cred_handle
    delete_p_gss_cred_id_t $delegated_cred_handle
    delete_p_gss_cred_id_t $imported_cred_handle
    
    delete_p_gss_buffer_desc $send_tok
    delete_p_gss_buffer_desc $recv_tok
    delete_p_gss_buffer_desc $output_name
    
    delete_p_gss_name_t $target_name
    delete_p_gss_name_t $source_name
    
    delete_p_gss_ctx_id_t $init_context
    delete_p_gss_ctx_id_t $accept_context
    delete_p_gss_ctx_id_t $del_init_context
    delete_p_gss_ctx_id_t $del_accept_context
    
    delete_p_gss_OID $mech_type

    globus_module_deactivate $globus_i_gsi_gssapi_module
} err ] } {
    puts_fail "EXCEPTION: $err"
}

#---------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#---------------------------------------------------------------------
TestExit
