#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Load standard packages
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
    
foreach lib { globus_error \
		  globus_object \
		  globus_module \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    globus_module_activate $globus_i_gsi_gssapi_module
    
    set cred_handle [ new_p_gss_cred_id_t ]
    
    set target_name [ new_p_gss_name_t ]

    set init_context [ new_p_gss_ctx_id_t ]
    set accept_context [ new_p_gss_ctx_id_t ]
    
    set input_token [ new_p_gss_buffer_desc ]
    set output_token [ new_gss_buffer_desc ]

    set in_buf [ new_gss_buffer_desc ]
    set out_buf [ new_gss_buffer_desc ]

    set msg_buf [ new_gss_buffer_desc ]
    
    set output_name [ new_p_gss_buffer_desc ]

    set mech_type [ new_p_gss_OID ]

    ####################################################################
    # Acquire the credential
    ####################################################################
    puts_info 10 "gss_acquire_cred"
    lassign [ gss_acquire_cred \
		  NULL \
		  $GSS_C_INDEFINITE \
		  $gss_c_no_oid_set \
		  $GSS_C_BOTH \
		  $cred_handle \
		  NULL \
		 ] maj_stat min_stat time_rec 
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tcred_handle $cred_handle"
    puts_info 10 "\ttime_rec $time_rec"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_acquire_cred"
	TestExit
    }

    ####################################################################
    # Get the subject name
    ####################################################################
    puts_info 10 "p_gss_cred_id_t_value"
    lassign [ p_gss_cred_id_t_value \
		  $cred_handle \
		 ] cred_handle_addr_value
    puts_info 10 "\tcred_handle_addr_value $cred_handle_addr_value"
    
    puts_info 10 "gss_inquire_cred"
    lassign [ gss_inquire_cred \
		  $cred_handle_addr_value \
		  $target_name \
		  NULL \
		  NULL \
		 ] maj_stat min_stat lifetime
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\ttarget_name $target_name"
    puts_info 10 "\tlifetime $lifetime"
    
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_inquire_cred"
	TestExit
    }

    ####################################################################
    # Set up the security context
    ####################################################################
    puts_info 10 "p_gss_name_t_value"
    lassign [ p_gss_name_t_value \
		  $target_name \
		 ] target_name_addr_value
    puts_info 10 "\ttarget_name_addr_value $target_name_addr_value"

    puts_info 10 "gss_init_sec_context"
    lassign [ gss_init_sec_context \
		  $cred_handle_addr_value \
		  $init_context \
		  $target_name_addr_value \
		  $gss_c_null_oid \
		  0 \
		  0 \
		  $gss_c_no_channel_bindings \
		  $input_token \
		  NULL \
		  $output_token \
		 ] init_maj_stat min_stat ret_flags time_rec
    puts_info 10 "\tinit_maj_stat $init_maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tret_flags $ret_flags"
    puts_info 10 "\ttime_rec $time_rec"
    puts_info 10 "\tinit_context $init_context"

    if { $init_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	puts_fail "ERR: gss_init_sec_context"
	TestExit
    }

    while 1 {
	puts_info 10 "gss_accept_sec_context"
	lassign [ gss_accept_sec_context \
		      $accept_context \
		      $gss_c_no_credential \
		      $output_token \
		      $gss_c_no_channel_bindings \
		      NULL \
		      $mech_type \
		      $input_token \
		      $gss_c_no_credential \
		     ] accept_maj_stat min_stat ret_flags time_rec
	puts_info 10 "\taccept_maj_stat $accept_maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	puts_info 10 "\tret_flags $ret_flags"
	puts_info 10 "\ttime_rec $time_rec"

	if { $accept_maj_stat != $GSS_S_COMPLETE && \
		 $accept_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	    puts_fail "ERR: gss_accept_sec_context"
	    TestExit
	} elseif { $accept_maj_stat == $GSS_S_COMPLETE } {
	    break
	}

	puts_info 10 "gss_init_sec_context"
	lassign [ gss_init_sec_context \
		      $gss_c_no_credential \
		      $init_context \
		      $target_name_addr_value \
		      $gss_c_null_oid \
		      0 \
		      0 \
		      $gss_c_no_channel_bindings \
		      $input_token \
		      NULL \
		      $output_token \
		     ] init_maj_stat min_stat ret_flags time_rec
	puts_info 10 "\tinit_maj_stat $init_maj_stat"
	puts_info 10 "\tmin_stat $min_stat"
	puts_info 10 "\tret_flags $ret_flags"
	puts_info 10 "\ttime_rec $time_rec"

	if { $init_maj_stat != $GSS_S_COMPLETE && \
		 $init_maj_stat != $GSS_S_CONTINUE_NEEDED } {
	    puts_fail "ERR: gss_init_sec_context"
	    TestExit
	}	
    }

    puts_info 10 "Successfully established initial security context"

    # Construct hello message
    set msg "Hello Globus from GSSAPI"

    ####################################################################
    # gss_wrap
    ####################################################################
    puts_info 10 "gss_buffer_desc_value_set"
    gss_buffer_desc_value_set \
	$in_buf \
	$msg

    puts_info 10 "gss_buffer_desc_length_set"
    gss_buffer_desc_length_set \
	$in_buf \
	[expr [string length $msg] + 1]

    puts_info 10 "p_gss_ctx_id_t_value"
    lassign [ p_gss_ctx_id_t_value \
		  $init_context \
		 ] init_context_addr_value
    puts_info 10 "\tinit_context_addr_value $init_context_addr_value"

    puts_info 10 "gss_wrap"
    lassign [ gss_wrap \
		  $init_context_addr_value \
		  1 \
		  $GSS_C_QOP_DEFAULT \
		  $in_buf \
		  $out_buf \
		 ] maj_stat min_stat state 
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tstate $state"
    
    ####################################################################
    # gss_unwrap
    ####################################################################
    puts_info 10 "p_gss_ctx_id_t_value"
    lassign [ p_gss_ctx_id_t_value \
		  $accept_context \
		 ] accept_context_addr_value
    puts_info 10 "\taccept_context_addr_value $accept_context_addr_value"
    
    puts_info 10 "gss_unwrap"
    lassign [ gss_unwrap \
		  $accept_context_addr_value \
		  $out_buf \
		  $msg_buf \
		 ] maj_stat min_stat conf_state qop_state
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    puts_info 10 "\tconf_state $conf_state"
    puts_info 10 "\tqop_state $qop_state"
    
    ####################################################################
    # Produce a signature block for the message
    ####################################################################
    puts_info 10 "gss_verify_mic"
    lassign [ gss_get_mic \
		  $accept_context_addr_value \
		  $GSS_C_QOP_DEFAULT \
		  $msg_buf \
		  $out_buf \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
    
    ####################################################################
    # Release target name
    ####################################################################
    puts_info 10 "gss_release_name"
    lassign [ gss_release_name \
		  $target_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"
	
    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_name"
	TestExit
    }

    ####################################################################
    # Delete security context
    ####################################################################
    puts_info 10 "gss_delete_sec_context"
    lassign [ gss_delete_sec_context \
		  $init_context \
		  $output_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_delete_sec_context"
	TestExit
    }

    puts_info 10 "gss_delete_sec_context"
    lassign [ gss_delete_sec_context \
		  $accept_context \
		  $output_name \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_delete_sec_context"
	TestExit
    }

    ####################################################################
    # Destroy the credential
    ####################################################################
    puts_info 10 "gss_release_cred"
    lassign [ gss_release_cred \
		  $cred_handle \
		 ] maj_stat min_stat
    puts_info 10 "\tmaj_stat $maj_stat"
    puts_info 10 "\tmin_stat $min_stat"

    if { $maj_stat != $GSS_S_COMPLETE } {
	puts_fail "ERR: gss_release_cred"
	TestExit
    }

    ####################################################################
    # Release resources
    ####################################################################
    delete_p_gss_cred_id_t $cred_handle

    delete_p_gss_ctx_id_t $init_context
    delete_p_gss_ctx_id_t $accept_context
    
    delete_p_gss_name_t $target_name

    delete_p_gss_buffer_desc $output_name

    delete_p_gss_OID $mech_type
    
    globus_module_deactivate $globus_i_gsi_gssapi_module
} err ] } {
    puts_fail "EXCEPTION: $err"
}

#---------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#---------------------------------------------------------------------
TestExit
