#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Determine if we have threads or not
#------------------------------------------------------------------------
set ::HAVE_PACKAGE_THREAD 0
set ::EXEC_MAIN 1
if { 0 } {
    if { ! [catch {package require Thread}] } {
	set ::HAVE_PACKAGE_THREAD 1
	if { ! [info exists ::LOADING_THREAD_INTERP] } {
	    set ::LOADING_THREAD_INTERP 0
	} else {
	    set ::EXEC_MAIN 0
	}
    }
}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io} {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Initialize necessary variables 
#------------------------------------------------------------------------
set SECTION io_test
validate_section $SECTION [list BUFFERSIZE]
validate_section $SECTION [list WAITFORNBYTES]

set bufsize $TestEnv($SECTION,BUFFERSIZE)
puts_info 10 "bufsize: $bufsize"

set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
puts_info 10 "sourcedir: $sourcedir"

set localdir [file join /tmp \
		  [file join "$env(USER)" \
		       [constructOutputDirectory $argv0]]]	
puts_info 10 "localdir: $localdir"

set waitfornbytes $TestEnv($SECTION,WAITFORNBYTES)
puts_info 10 "waitfornbytes: $waitfornbytes"

set cb_client_done 0

namespace eval globus {
    #--------------------------------------------------------------------
    # Activate Globus I/O module
    #--------------------------------------------------------------------
    proc initialize { } {
	global globus_l_io_module
	
	puts_info 10 "Activate I/O module"
	globus_module_activate $globus_l_io_module
	puts_info 10 "Done activating I/O module"
    }

    #--------------------------------------------------------------------
    # Deactivate Globus XIO module
    #--------------------------------------------------------------------
    proc cleanup { } {
	global globus_l_io_module
	
	puts_info 10 "Deactivate I/O module"
	globus_module_deactivate $globus_l_io_module
	puts_info 10 "Done deactivating I/O module"
    }
}

namespace eval client {
    variable data
    #--------------------------------------------------------------------
    # Destroy client attribute
    #--------------------------------------------------------------------
    proc cleanup { } {
	puts_info 10 "Client: destroying TCP and file attributes"
	
	variable data
	
	globus_io_tcpattr_destroy $data(tcp_attr)
	globus_io_fileattr_destroy $data(file_attr)
	delete_p_globus_io_attr_t $data(tcp_attr)
	delete_p_globus_io_attr_t $data(file_attr)
	foreach handle [list write_handle] {
	    if { [info exists [namespace current]::data($handle)] } {
		set h [subst $[namespace current]::data($handle)]
		puts_info 10 "Client: deleting handle: $handle ref: $h"
		delete_p_globus_io_handle_t $h
	    }
	}
	
	globus::cleanup
    }

    #--------------------------------------------------------------------
    # Initialize client attributes
    #--------------------------------------------------------------------
    proc initialize { Filename } {
	globus::initialize
	
	puts_info 10 "Initialize client attributes"
	
	variable data
	variable filename $Filename
	set data(tcp_attr) [ new_p_globus_io_attr_t ]
	set data(file_attr) [ new_p_globus_io_attr_t ]
	
	globus_io_tcpattr_init $data(tcp_attr)
	globus_io_fileattr_init $data(file_attr)
    }
    #--------------------------------------------------------------------#
    #--------------------------------------------------------------------#
    # CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT     #
    #--------------------------------------------------------------------#
    #--------------------------------------------------------------------#

    #--------------------------------------------------------------------
    # Dest close callback
    #--------------------------------------------------------------------
    proc client_done_cb { user_parms handle result } {
	if { [ catch {
	    variable server_thread
	    global cb_client_done
	    global HAVE_PACKAGE_THREAD
	    
	    puts_info 10 "in dest_close_cb"
	    puts_info 10 "handle: $handle"
	    puts_info 10 "leaving dest_close_cb"
	    
	    if { $HAVE_PACKAGE_THREAD } {
		[namespace current]::cleanup
		thread::send -async $server_thread [subst {
		    set ::cb_client_done 1
		}]
		thread::release [thread::id]
	    } else {
		set cb_client_done 1
	    }
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Client close callback
    #--------------------------------------------------------------------
    proc client_close_cb { user_parms handle result } {
	if { [ catch {
	    upvar 0 $user_parms user_args
	    puts_info 10 "entering client_close_cb"
	    puts_info 10 "handle: $handle"
	    globus_io_register_close \
		$user_args(write_handle) \
		[ subst { [namespace current]::client_done_cb $user_parms } ] 
	    puts_info 10 "leaving client_close_cb"
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Dest read callback
    #--------------------------------------------------------------------
    proc client_read_cb { user_parms handle result buffer } {
	if { [ catch {
	    upvar 0 $user_parms user_args
	    puts_info 20 "in client_read_cb"
	    puts_info 20 "handle: $handle"
	    puts_info 20 "conn_handle: $user_args(conn_handle)"
	    
	    if { [ string length $buffer ] == 0 } {
		puts_info 20 "zero length buffer"
		puts_info 20 "globus_io_register_close"
		globus_io_register_close \
		    $user_args(conn_handle) \
		    [ subst { [namespace current]::client_close_cb \
				  $user_parms } ] 
	    } else {
		if { [ catch {
		    puts_info 20 "globus_io_register_read"
		    globus_io_register_read \
			$user_args(conn_handle) \
			$::bufsize \
			$::waitfornbytes \
			[ subst { [namespace current]::client_write_cb \
				      $user_parms } ] 
		} err ] } {
		    puts_info 10 "err: $err"
		    puts_info 10 "globus_io_register_close"
		    globus_io_register_close \
			$user_args(conn_handle) \
			[ subst { [namespace current]::client_close_cb \
				      $user_parms } ] 
		}
	    }
	    puts_info 20 "leaving client_read_cb" 
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Client destination write callback
    #--------------------------------------------------------------------
    proc client_write_cb { user_parms handle result buffer } {
	if { [ catch {
	    upvar 0 $user_parms user_args
	    puts_info 20 "entering client_write_cb"
	    puts_info 20 "handle: $handle"
	    puts_info 20 "client write buffer length: [ string length $buffer ]"
	    puts_info 20 "globus_io_register_write"
	    globus_io_register_write \
		$user_args(write_handle) \
		$buffer \
		[ subst { [namespace current]::client_read_cb \
			      $user_parms } ] 
	    puts_info 20 "leaving client_write_cb"
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Client destination open callback
    #--------------------------------------------------------------------
    proc client_dest_open_cb { user_parms handle result conn_handle } {
	if { [ catch {
	    puts_info 10 "entering client_dest_open_cb"
	    upvar 0 $user_parms user_args
	    puts_info 10 "[namespace current]: handle: $handle"
	    puts_info 10 "[namespace current]: conn_handle: $conn_handle"
	    set user_args(conn_handle) $conn_handle
	    lassign [ globus_io_tcp_get_remote_address \
			  $conn_handle ] result ipstring port
	    puts_info 10 "Accepted connection on my peer $ipstring:$port"
	    if { [ catch {
		puts_info 10 "globus_io_register_read"
		globus_io_register_read \
		    $conn_handle \
		    $::bufsize \
		    $::waitfornbytes \
		    [ subst { [namespace current]::client_write_cb \
				  $user_parms } ] 
	    } err ] } {
		puts_info 10 "err: $err"
		globus_io_register_close \
		    $conn_handle \
		    [ subst { [namespace current]::client_close_cb \
				  $user_parms } ] 
	    }
	    puts_info 10 "leaving client_dest_open_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Client
    #--------------------------------------------------------------------
    proc client { port } {
	if { [ catch {
	    global HAVE_PACKAGE_THREAD
	    variable data
	    puts_info 10 "entering client proc"
	    if { $HAVE_PACKAGE_THREAD } {
		puts_info 10 "Running client in thread: [thread::id]"
	    }
	    
	    #------------------------------------------------------------
	    # Handle for writing data
	    #------------------------------------------------------------
	    set data(write_handle) [ new_p_globus_io_handle_t ]
	    
	    #------------------------------------------------------------
	    # Write flags and mode
	    #------------------------------------------------------------
	    set write_flags [ expr $::GLOBUS_IO_FILE_CREAT | \
				  $::GLOBUS_IO_FILE_TRUNC | \
				  $::GLOBUS_IO_FILE_WRONLY ]
	    
	    set write_mode [ expr $::GLOBUS_IO_FILE_IRUSR | \
				 $::GLOBUS_IO_FILE_IWUSR ]
	    
	    #------------------------------------------------------------
	    # Client opens file handle to write data 
	    #------------------------------------------------------------
	    variable filename
	    puts_info 10 "globus_io_file_open"
	    
	    puts_info 10 "client: filename: $filename"
	    puts_info 10 "client: write_flags: $write_flags"
	    puts_info 10 "client: write_mode: $write_mode"
	    puts_info 10 "client: data(file_attr): $data(file_attr)"
	    puts_info 10 "client: data(write_handle): $data(write_handle)"
	    
	    globus_io_file_open \
		$filename \
		$write_flags \
		$write_mode \
		$data(file_attr) \
		$data(write_handle) 
	    
	    #------------------------------------------------------------
	    # Handle for connection socket
	    #------------------------------------------------------------
	    set host [info hostname]
	    
	    puts_info 10 "globus_io_tcp_register_connect"
	    puts_info 10 "client: host: $host"
	    puts_info 10 "client: port: $port"
	    puts_info 10 "client: TCP attrs: $data(tcp_attr)"
	    
	    globus_io_tcp_register_connect \
		$host \
		$port \
		$data(tcp_attr) \
		[subst { [namespace current]::client_dest_open_cb \
			     [namespace current]::data }]
	    
	    puts_info 10 "leaving client proc"
	} err ] } {
	    return -code error $err
	}
    }
};# namespace client


namespace eval server {
    #--------------------------------------------------------------------#
    #--------------------------------------------------------------------#
    # SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER     #
    #--------------------------------------------------------------------#
    #--------------------------------------------------------------------#
    variable port 0
    variable data

    #--------------------------------------------------------------------
    # Destroy server attribute
    #--------------------------------------------------------------------
    proc cleanup { } {
	variable data
	
	globus_io_tcpattr_destroy $data(tcp_attr)
	globus_io_fileattr_destroy $data(file_attr)
	delete_p_globus_io_attr_t $data(tcp_attr)
	delete_p_globus_io_attr_t $data(file_attr)
	foreach handle [list read_handle list_handle conn_handle] {
	    if { [info exists [namespace current]::data($handle)] } {
		set h [subst $[namespace current]::data($handle)]
		puts_info 10 "Deleting handle: $handle ref: $h"
		delete_p_globus_io_handle_t $h
	    }
	}
    }

    #--------------------------------------------------------------------
    # Initialize server attributes
    #--------------------------------------------------------------------
    proc initialize { Filename } {
	puts_info 10 "Initialize server attributes"
	
	variable data
	variable filename $Filename
	
	set data(tcp_attr) [ new_p_globus_io_attr_t ]
	set data(file_attr) [ new_p_globus_io_attr_t ]
	
	globus_io_tcpattr_init $data(tcp_attr)
	globus_io_fileattr_init $data(file_attr)
    }
	
    #--------------------------------------------------------------------
    # Server done callback
    #--------------------------------------------------------------------
    proc server_done_cb { user_parms handle result } {
	if { [ catch {
	    puts_info 10 "in server_done_cb"
	    puts_info 10 "leaving server_done_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server listening close callback
    #--------------------------------------------------------------------
    proc server_list_close_cb { user_parms handle result } {
	if { [ catch {
	    upvar 0 $user_parms user_args
	    puts_info 10 "entering server_list_close_cb"
	    globus_io_register_close \
		$user_args(list_handle) \
		[ subst { [namespace current]::server_done_cb $user_parms } ] 
	    puts_info 10 "leaving server_list_close_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server close callback
    #--------------------------------------------------------------------
    proc server_close_cb { user_parms handle result } {
	if { [ catch {
	    upvar 0 $user_parms user_args
	    puts_info 10 "entering server_close_cb"
	    puts_info 10 "globus_io_register_close"
	    
	    globus_io_register_close \
		$user_args(conn_handle) \
		[ subst { [namespace current]::server_list_close_cb \
			      $user_parms } ] 
	    
	    puts_info 10 "leaving server_close_cb"
	} err ] } {
	    err_handler $err
	}
    }
	
    #--------------------------------------------------------------------
    # Dest write callback
    #--------------------------------------------------------------------
    proc server_write_cb { user_parms handle result buffer } {
	if { [ catch {
	    puts_info 20 "entering dest_write_cb"
	    
	    # We need to access our associative array
	    upvar 0 $user_parms user_args
	    
	    if { [ catch {
		puts_info 20 "globus_io_register_read"
		puts_info 20 "server read buffer length: [string length $buffer]"
		
		globus_io_register_read \
		    $user_args(read_handle) \
		    $::bufsize \
		    $::waitfornbytes \
		    [ subst { [namespace current]::source_read_cb \
				  $user_parms } ] 
	    } err ] } {
		puts_info 10 "err: $err"
		puts_info 10 "globus_io_register_close"
		
		globus_io_register_close \
		    $user_args(read_handle) \
		    [ subst { [namespace current]::server_close_cb \
				  $user_parms } ] 
	    }
	    
	    puts_info 20 "leaving dest_write_cb"
	}  err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Source read callback
    #--------------------------------------------------------------------
    proc source_read_cb { user_parms handle result buffer } {
	if { [ catch {
	    upvar 0 $user_parms user_args
	    
	    puts_info 20 "entering source_read_cb"
	    puts_info 20 "globus_io_register_send"
	    puts_info 20 "server read buffer length: [string length $buffer]"
	    
	    if { [ string length $buffer] == 0 } {
		puts_info 10 "globus_io_register_close"
		
		globus_io_register_close \
		    $user_args(read_handle) \
		    [ subst { [namespace current]::server_close_cb \
				  $user_parms } ] 
	    } else {
		globus_io_register_send \
		    $user_args(conn_handle) \
		    $buffer \
		    0 \
		    [ subst { [namespace current]::server_write_cb \
				  $user_parms } ] 
	    }
	    puts_info 20 "leaving source_read_cb"
	} err ] } {
	    err_handler $err
	}	
    }

    #--------------------------------------------------------------------
    # Source open callback
    #--------------------------------------------------------------------
    proc source_open_cb { user_parms conn_handle result } {
	if { [ catch {
	    puts_info 10 "entering source_open_cb"
	    upvar 0 $user_parms user_args
	    #------------------------------------------------------------
	    # Get local and remote info
	    #------------------------------------------------------------
	    lassign [ globus_io_tcp_get_local_address \
			  $user_args(conn_handle) ] result ipstring port
	    puts_info 10 "Accepted connection on my socket $ipstring:$port"
	    lassign [ globus_io_tcp_get_remote_address \
			  $user_args(conn_handle) ] result ipstring port
	    puts_info 10 "Accepted connection on my peer $ipstring:$port"
	    #------------------------------------------------------------
	    # Open the file handle to read data from source file
	    #------------------------------------------------------------
	    puts_info 10 "globus_io_file_open"
	    variable filename
	    globus_io_file_open \
		$filename \
		$::GLOBUS_IO_FILE_RDONLY \
		0 \
		$user_args(file_attr) \
		$user_args(read_handle)
	    
	    #------------------------------------------------------------
	    # Open source file and perform asynchronous read operation
	    # on the handle
	    #------------------------------------------------------------
	    if { [ catch {
		puts_info 10 "globus_io_register_read"
		globus_io_register_read \
		    $user_args(read_handle) \
		    $::bufsize \
		    $::waitfornbytes \
		    [ subst { [namespace current]::source_read_cb \
				  $user_parms } ] 
	    } err ] } {
		puts_info 10 "err: $err"
		puts_info 10 "globus_io_register_close"
		
		globus_io_register_close \
		    $user_args(read_handle) \
		    [ subst { [namespace current]::server_close_cb \
				  $user_parms } ] 
	    }
			
	    puts_info 10 "leaving source_open_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server accept callback
    #--------------------------------------------------------------------
    proc server_accept_cb { user_parms handle result } {
	if { [ catch {
	    puts_info 10 "entering server_accept_cb"
	    
	    upvar 0 $user_parms user_args
	    
	    puts_info 10 "user_args(tcp_attr): $user_args(tcp_attr)"
	    puts_info 10 "user_args(list_handle): $user_args(list_handle)"
	    puts_info 10 "user_args(read_handle): $user_args(read_handle)"
	    puts_info 10 "user_args(conn_handle): $user_args(conn_handle)"
	    
	    globus_io_tcp_register_accept \
		$user_args(list_handle) \
		$user_args(tcp_attr) \
		$user_args(conn_handle) \
		[ subst { [namespace current]::source_open_cb $user_parms } ] 
	    
	    puts_info 10 "leaving server_accept_cb"
	    
	} err ] } {
	    err_handler $err
	}
    }
	
    #--------------------------------------------------------------------
    # Server
    #--------------------------------------------------------------------
    proc server { } {
	if { [ catch {
	    variable data
	    
	    puts_info 10 "entering server proc"
	    # Handle for writing data to a file
	    set data(read_handle) [ new_p_globus_io_handle_t ]
	    puts_info 10 "server: data(read_handle): $data(read_handle)"
	    # Handle for listening socket
	    set data(list_handle) [ new_p_globus_io_handle_t ]
	    puts_info 10 "server: data(list_handle): $data(list_handle)"
	    # Handle for connection socket
	    set data(conn_handle) [ new_p_globus_io_handle_t ]
	    puts_info 10 "server: data(conn_handle): $data(conn_handle)"
	    
	    # Create a listening handle
	    variable port
	    lassign [ globus_io_tcp_create_listener \
			  0 \
			  -1 \
			  $data(tcp_attr) \
			  $data(list_handle) ] result port
	    puts_info 10 "server listening port is $port"
	    #------------------------------------------------------------
	    # Wait for a connection from client
	    #------------------------------------------------------------
	    puts_info 10 "globus_io_tcp_register_listen"
	    
	    globus_io_tcp_register_listen \
		$data(list_handle) \
		[subst { [namespace current]::server_accept_cb \
			     [namespace current]::data }]
	    puts_info 10 "leaving server proc"
	} err ] } {
	    return -code error $err
	}
    }
};# Namespace server


#------------------------------------------------------------------------
# Run test loop
#------------------------------------------------------------------------
proc run_test { } {
    # Create temporary directory
    copy_init $::localdir
    # Construct file list
    set in_filelist [ constructFileList ]
    puts_info 10 "Number of files: [ llength $in_filelist ]"
    
    foreach source_filename $in_filelist {
	if { [file isdirectory $source_filename] } {
	    continue
	}
	set destination_filename \
	    [ file join $::localdir [ file tail $source_filename ] ]
	puts_info 10 "Source file: $source_filename"
	puts_info 10 "Dest file: $destination_filename"
	
	#----------------------------------------------------------------
	# In both models, the server will run in the main thread
	#----------------------------------------------------------------
	server::initialize $source_filename
	
	if { $::HAVE_PACKAGE_THREAD } {
	    #------------------------------------------------------------
	    # Load the client thread with the commands it will need
	    #------------------------------------------------------------
	    set c [thread::create {
		set ::LOADING_THREAD_INTERP 1
		source "$env(SRCDIR)/test_io_async_client_server.tcl"
		puts_info 10 "Finished loading client thread"
		thread::wait
	    }]
	    puts_info 10 "Created client thread: $c"
	    
	    #------------------------------------------------------------
	    # Start the server
	    #------------------------------------------------------------
	    server::server
	    
	    #------------------------------------------------------------
	    # Initialize the client thread and then call its main entry
	    #   point. This will start the data exchange between the
	    #   client and server. The client thread will terminate
	    #   once it has written all bytes to the output file.
	    #------------------------------------------------------------
	    thread::send -async $c [subst {
		::client::initialize $destination_filename
		set ::client::server_thread [thread::id]
		::client::client $::server::port
	    }]
	    #------------------------------------------------------------
	    # wait for the client to complete
	    #------------------------------------------------------------
	    vwait cb_client_done
	    # thread::wait
	} else {
	    #------------------------------------------------------------
	    # Don't have the Thread package so just run everything in
	    #   in the main thread. This ONLY works because we are using
	    #   the asyncronous calls.
	    #------------------------------------------------------------
	    client::initialize $destination_filename
	    
	    server::server
	    client::client $::server::port
	    vwait cb_client_done
	    
	    client::cleanup
	}
	puts_info 10 "All joined up"
	server::cleanup
	
	puts_info 10 "Client/Server done"
	
	#-----------------------------------------------------------------
	# Run MD5 checksum before exiting
	#-----------------------------------------------------------------
	check_md5sum $source_filename $destination_filename
    }	
    #-----------------------------------------------------------------
    # Remove temporary directory
    #-----------------------------------------------------------------
    copy_deinit $::localdir
}

#------------------------------------------------------------------------
# Main
#------------------------------------------------------------------------
proc main { } {
    #--------------------------------------------------------------------#
    #--------------------------------------------------------------------#
    # MAIN                                                               #
    #--------------------------------------------------------------------#
    #--------------------------------------------------------------------#
    global localdir
    global SECTION
    global TestEnv
    global env
    global HAVE_PACKAGE_THREAD

    if { [ catch {
	#----------------------------------------------------------------
	# Do initialation of globus
	#----------------------------------------------------------------
	globus::initialize
	
	# Run the test in infinite loop mode or not
	if { [IsInfinite] == 1 } {
	    while 1 {
		run_test
	    }
	} else {
	    run_test
	}
	
	#----------------------------------------------------------------
	# Do cleanup of globus
	#----------------------------------------------------------------
	globus::cleanup
    } err ] } {
	err_handler $err
    }
	
    #--------------------------------------------------------------------
    # Exit with normal exit status since test completed without error.
    #--------------------------------------------------------------------
    TestExit
}

puts_info 10 "HAVE_PACKAGE_THREAD: $HAVE_PACKAGE_THREAD"
if { $::EXEC_MAIN } {
    main
}
