#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Load shared libraries
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
set SECTION io_test
validate_section $SECTION [list BUFFERSIZE WAITFORNBYTES]

set copy_info(bufsize) $TestEnv($SECTION,BUFFERSIZE)
set copy_info(wait_for_nbytes) $TestEnv($SECTION,WAITFORNBYTES)

puts_info 10 "bufsize: $copy_info(bufsize)"

set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
puts_info 10 "sourcedir : $sourcedir"

set localdir [file join /tmp \
		  [file join "$env(USER)" \
		       [constructOutputDirectory $argv0]]]
puts_info 10 "localdir: $localdir"
#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc done_callback { user_parms handle result } {
    if { [ catch {
	puts_info 10 "entering done_callback"
	# In wish, we need to trigger event loop
	if { [info commands wm] == "wm" } {
	    wm iconify .
	}
	set ::cb_done 1
	puts_info 10 "leaving done_callback"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc close_callback { user_parms handle result } {
    if { [ catch {
	upvar 0 $user_parms user_args
	puts_info 10 "entering close_callback"
	globus_io_register_close \
	    $user_args(writehandle) \
	    [ subst { done_callback $user_parms } ] 
	puts_info 10 "leaving close_callback"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc read_callback { user_parms handle result buffer } {
    if { [ catch {
	upvar 0 $user_parms user_args
	puts_info 20 "entering read_callback"
	puts_info 20 "read_callback: result: $result buffer length: [string length $buffer]"
	if { [string length $buffer] == 0 } {
	    #set ::cb_done 1
	    puts_info 10 "globus_io_register_close"
	    globus_io_register_close \
		$user_args(readhandle) \
		[subst { close_callback $user_parms } ]
	} else {
	    if { [ catch {
		globus_io_register_read \
		    $user_args(readhandle) \
		    $user_args(bufsize) \
		    $user_args(wait_for_nbytes) \
		    [ subst { write_callback $user_parms } ] 
	    } err ] } {
		puts_info 10 "err: $err"
		set ::cb_done 1
	    }
	}
	puts_info 20 "leaving read_callback"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc write_callback { user_parms handle result buffer } {
    if { [ catch {
	upvar 0 $user_parms user_args
	puts_info 20 "entering write_callback"
	globus_io_register_write \
	    $user_args(writehandle) \
	    $buffer \
	    [ subst { read_callback $user_parms } ] 
	puts_info 20 "leaving write_callback"
    } err ] } {
	err_handler $err
    }	
}
#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc run_test { } {
    if { [ catch {
	# Initialization step
	copy_init $::localdir
	
	set in_filelist [ constructFileList ]
	puts_info 10 "Number of files: [ llength $in_filelist ]"
	
	foreach read_cs $in_filelist {
	    if { [file isdirectory $read_cs] } {
		continue
	    }
	    
	    set write_cs [ file join $::localdir [ file tail $read_cs ] ]
	    
	    set ::copy_info(readhandle)        [ new_p_globus_io_handle_t ]
	    set ::copy_info(writehandle)       [ new_p_globus_io_handle_t ]
	    
	    # creating a read handle by calling
	    # globus_io_file_open() to perform read operation
	    
	    puts_info 20 "read_cs: $read_cs"
	    puts_info 20 "write_cs: $write_cs"
	    
	    globus_io_file_open \
		$read_cs \
		$::GLOBUS_IO_FILE_RDONLY \
		0 \
		$::fileattr \
		$::copy_info(readhandle)
	    
	    globus_io_file_open \
		$write_cs \
		$::write_flags \
		$::write_mode \
		$::fileattr \
		$::copy_info(writehandle)

	    # In wish, we don't need to display the window
	    # But, we need to send it out of the visible part of the screen
	    if { [info commands wm] == "wm" } {
		wm iconify .
		wm geometry . 1x1+4000+4000
	    }

	    #------------------------------------------------------------
	    # globus_io_register_read
	    #------------------------------------------------------------
	    set ::cb_done 0
	    puts_info 10 "globus_io_register_read"
	    globus_io_register_read \
		$::copy_info(readhandle) \
		$::copy_info(bufsize) \
		$::copy_info(wait_for_nbytes) \
		{ write_callback ::copy_info } 
	    vwait ::cb_done

	    #------------------------------------------------------------
	    # Release memory space
	    #------------------------------------------------------------
	    delete_p_globus_io_handle_t $::copy_info(readhandle)
	    delete_p_globus_io_handle_t $::copy_info(writehandle)
	    
	    #------------------------------------------------------------
	    # MD5 checksum
	    #------------------------------------------------------------
	    check_md5sum $read_cs $write_cs
	}
	# Deinitialization step
	copy_deinit $::localdir
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus I/O module
    puts_info 10 "Activating I/O module"
    globus_module_activate $globus_l_io_module
		
    # I/O attributes
    set fileattr [ new_p_globus_io_attr_t ]
    puts_info 20 "fileattr value: [ p_globus_io_attr_t_value $fileattr ]"

    puts_info 10 "globus_io_fileattr_init"
    globus_io_fileattr_init $fileattr
    

    # Set write flags and modes
    set write_flags [ expr $::GLOBUS_IO_FILE_CREAT | \
			  $::GLOBUS_IO_FILE_TRUNC | \
			  $::GLOBUS_IO_FILE_WRONLY ]
    
    set write_mode [ expr $::GLOBUS_IO_FILE_IRUSR | \
			 $::GLOBUS_IO_FILE_IWUSR ]

    # Run the test in infinite loop mode or not
    if { [IsInfinite] == 1 } {
	while 1 {
	    run_test
	}
    } else {
	run_test
    }
    
    # Destroy I/O attribute
    puts_info 10 "Destroying I/O attribute"
    globus_io_fileattr_destroy $fileattr
    delete_p_globus_io_attr_t $fileattr
    
    # Deactivate Globus I/O module
    puts_info 10 "Deactivating I/O module"
    globus_module_deactivate $globus_l_io_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
