#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
case x$PROGRAM in
# ... and otherwise use tclsh. \
*) PROGRAM="tclsh";;
# End of bourne script \
esac
# execution after the exec line. \
exec $PROGRAM "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
set SECTION io_test
validate_section $SECTION [list BUFFERSIZE]
validate_section $SECTION [list WAITFORNBYTES]

set bufsize         $TestEnv($SECTION,BUFFERSIZE)
set waitforbytes    $TestEnv($SECTION,WAITFORNBYTES)
set read_cs         [ lindex $argv 0 ]
set AUTHENTICATE    1
set ENCRYPTION      1
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus I/O module
    globus_module_activate $globus_l_io_module
    
    # I/O attributes of the server
    set file_attr  [ new_p_globus_io_attr_t ]
    set tcp_attr   [ new_p_globus_io_attr_t ]
    set auth_data  [ new_p_globus_io_secure_authorization_data_t ]
    
    # Handle for listening socket, connection socket and write operation
    set list_handle  [ new_p_globus_io_handle_t ]
    set conn_handle  [ new_p_globus_io_handle_t ]
    set read_handle  [ new_p_globus_io_handle_t ]
    
    # Initialize the authorization data
    globus_io_secure_authorization_data_initialize $auth_data
    
    # Open the file handle to read data from source file
    globus_io_fileattr_init $file_attr
    globus_io_tcpattr_init $tcp_attr
    
    globus_io_file_open \
	$read_cs \
	$GLOBUS_IO_FILE_RDONLY \
	0 \
	$file_attr \
	$read_handle 
    
    lassign [ globus_io_get_handle_type $read_handle ] handle_type

    # Initialize the attributes structure with authentication and
    # encryption
    if { $AUTHENTICATE == 1 } {
	# Use GSS-API authentication
	globus_io_attr_set_secure_authentication_mode \
	    $tcp_attr \
	    $GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI \
	    $gss_c_no_credential
	
	# Set the authorization mode
	globus_io_attr_set_secure_authorization_mode \
	    $tcp_attr \
	    $GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF \
	    $auth_data
	
	# Set the proxy mode
	globus_io_attr_set_secure_proxy_mode \
	    $tcp_attr \
	    $GLOBUS_IO_SECURE_PROXY_MODE_LIMITED 
	
	if { $ENCRYPTION == 1 } {
	    
	    # Set the delegation mode to None
	    globus_io_attr_set_secure_delegation_mode \
		$tcp_attr \
		$GLOBUS_IO_SECURE_DELEGATION_MODE_NONE 
	    
	    # Set the channel mode to use wrapping
	    globus_io_attr_set_secure_channel_mode \
		$tcp_attr \
		$GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP 
	    
	    # Use encryption, SAFE instead of PRIVATE is also possible
	    # but gives only integrity protection
	    globus_io_attr_set_secure_protection_mode \
		$tcp_attr \
		$GLOBUS_IO_SECURE_PROTECTION_MODE_PRIVATE 
	} 
    }
    
    globus_io_attr_set_socket_reuseaddr \
	$tcp_attr \
	$GLOBUS_TRUE 
    
    # Create the listening socket
    lassign [ globus_io_tcp_create_listener \
		  0 \
		  -1 \
		  $tcp_attr \
		  $list_handle ] result port
    
    lassign [ globus_io_get_handle_type $list_handle ] list_handle_type
    
    puts "[info hostname]:$port"
    
    # Create the listening socket
    globus_io_tcp_listen $list_handle
    
    # Connection arrived, accept it
    globus_io_tcp_accept \
	$list_handle \
	$tcp_attr \
	$conn_handle
    
    lassign [ globus_io_get_handle_type $conn_handle ] conn_handle_type
    
    # Get local and remote address information
    lassign [ globus_io_tcp_get_local_address \
		  $conn_handle ] result ipstring port
    lassign [ globus_io_tcp_get_remote_address \
		  $conn_handle ] result ipstring port
    
    # Testing globus_io_tcp_get_security_context()
    lassign [ globus_io_tcp_get_security_context \
		  $conn_handle \
		 ] result context
    
    # Testing globus_io_tcp_get_delegated_credential()
    lassign [ globus_io_tcp_get_delegated_credential \
		  $conn_handle \
		 ] result cred
    
    # Read and write data 
    do {
	lassign [ globus_io_read \
		      $read_handle \
		      $bufsize \
		      $waitforbytes \
		     ] result buffer
	lassign [ globus_io_write \
		      $conn_handle \
		      $buffer \
		     ] result nbytes
    } until { [string length $buffer] == 0 }
    
    # Close all handles
    globus_io_close $list_handle 
    globus_io_close $conn_handle
    globus_io_close $read_handle
    
    # Destroy the attribute
    globus_io_tcpattr_destroy $tcp_attr
    globus_io_fileattr_destroy $file_attr
    globus_io_secure_authorization_data_destroy $auth_data
    
    # Reclaim memory space
    delete_p_globus_io_attr_t $tcp_attr
    delete_p_globus_io_attr_t $file_attr
    delete_p_globus_io_secure_authorization_data_t $auth_data

    delete_p_globus_io_handle_t $read_handle
    delete_p_globus_io_handle_t $list_handle
    delete_p_globus_io_handle_t $conn_handle

    # Deactivate Globus I/O module
    globus_module_deactivate $globus_l_io_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
