#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
case x$PROGRAM in
# ... and otherwise use tclsh. \
*) PROGRAM="tclsh";;
# End of bourne script \
esac
# execution after the exec line. \
exec $PROGRAM "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
set SECTION io_test
validate_section $SECTION [list BUFFERSIZE]
validate_section $SECTION [list WAITFORNBYTES]

set bufsize         $TestEnv($SECTION,BUFFERSIZE)
set waitforbytes    $TestEnv($SECTION,WAITFORNBYTES)

set server_info     [ lindex $argv 0 ]
set hostname        [ lindex [ split $server_info : ] 0 ]
set port            [ lindex [ split $server_info : ] 1 ]
set write_cs	    [ lindex $argv 1 ]
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus I/O module
    globus_module_activate $globus_l_io_module

    # I/O attributes of the client
    set tcp_attr  [ new_p_globus_io_attr_t ]
    set file_attr [ new_p_globus_io_attr_t ]
    
    # Handle for reading data from a file
    set write_handle  [ new_p_globus_io_handle_t ]
    
    # Handle for connection socket
    set conn_handle  [ new_p_globus_io_handle_t ]
    
    # Client opens file handle to write data 
    globus_io_fileattr_init $file_attr
    
    set write_flags [ expr $GLOBUS_IO_FILE_CREAT | \
			  $GLOBUS_IO_FILE_TRUNC | \
			  $GLOBUS_IO_FILE_WRONLY ]
    
    set write_mode [ expr $GLOBUS_IO_FILE_IRUSR | \
			 $GLOBUS_IO_FILE_IWUSR ]
    
    globus_io_file_open \
	$write_cs \
	$write_flags \
	$write_mode \
	$file_attr \
	$write_handle
    
    # Connect to remote server
    globus_io_tcpattr_init $tcp_attr
    
    globus_io_tcp_connect \
	$hostname \
	$port \
	$tcp_attr \
	$conn_handle 

    # Read and write data 
    do {
	# Read data from the connection handle
	lassign [ globus_io_read \
		      $conn_handle \
		      $bufsize \
		      $waitforbytes \
		     ] result buffer
	# Write data to the destination file using write handle
	lassign [ globus_io_write \
		      $write_handle \
		      $buffer 
		 ] result nbytes_written
    } until { $nbytes_written == 0 }
    
    # Close connection handle and read handle
    globus_io_close $conn_handle
    globus_io_close $write_handle
    
    # Destroy the attribute
    globus_io_tcpattr_destroy $tcp_attr
    globus_io_fileattr_destroy $file_attr
    
    # Reclaim memory space
    delete_p_globus_io_attr_t $tcp_attr
    delete_p_globus_io_attr_t $file_attr
    delete_p_globus_io_handle_t $write_handle
    
    # Deactivate Globus I/O module
    globus_module_deactivate $globus_l_io_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
