#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
case x$PROGRAM in
# ... and otherwise use tclsh. \
*) PROGRAM="tclsh";;
# End of bourne script \
esac
# execution after the exec line. \
exec $PROGRAM "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
set SECTION io_test
validate_section $SECTION [list BUFFERSIZE]
validate_section $SECTION [list WAITFORNBYTES]

set bufsize         $TestEnv($SECTION,BUFFERSIZE)
set waitforbytes    $TestEnv($SECTION,WAITFORNBYTES)

set read_cs         [ lindex $argv 0 ] 
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus I/O module
    globus_module_activate $globus_l_io_module
    
    # I/O attributes of the server
    set tcp_attr  [ new_p_globus_io_attr_t ]
    set file_attr [ new_p_globus_io_attr_t ]
    
    # Handle for writing data to a file
    set read_handle  [ new_p_globus_io_handle_t ]
    
    # Handle for listening socket
    set list_handle  [ new_p_globus_io_handle_t ]
    
    # Handle for connection socket
    set conn_handle  [ new_p_globus_io_handle_t ]
    
    # Initialize the attribute
    globus_io_tcpattr_init $tcp_attr
    globus_io_fileattr_init $file_attr
    
    # Open the file handle to read data from source file
    globus_io_file_open \
	$read_cs \
	$GLOBUS_IO_FILE_RDONLY \
	0 \
	$file_attr \
	$read_handle
    
    # Create a listening handle
    # 0 indicates a port number will be allocated by the system
    lassign [ globus_io_tcp_create_listener \
		  0 \
		  -1 \
		  $tcp_attr \
		  $list_handle ] result port
    puts "[info hostname]:$port"
    
    # Wait a connection from client
    globus_io_tcp_listen $list_handle
    
    # Connection arrived, accept it
    globus_io_tcp_accept \
	$list_handle \
	$tcp_attr \
	$conn_handle
    
    # Check remote and local addresses
    lassign [ globus_io_tcp_get_local_address \
		  $conn_handle ] result ipstring port
    puts_info 10 "Accepted connection on my socket $ipstring:$port"

    lassign [ globus_io_tcp_get_remote_address \
		  $conn_handle ] result ipstring port
    puts_info 10 "Accepted connection on my peer $ipstring:$port"
    
    # Read and write data 
    do {
	lassign [ globus_io_read \
		      $read_handle \
		      $bufsize \
		      $waitforbytes ] result buffer
	lassign [ globus_io_send \
		      $conn_handle \
		      $buffer \
		      0 \
		     ] result nbytes_sent
    } until { [string length $buffer] == 0 } 
    
    # Close listening handle and connection handle
    globus_io_close $list_handle
    globus_io_close $conn_handle
    
    # Close write handle
    globus_io_close $read_handle
    
    # Destroy the attribute
    globus_io_tcpattr_destroy $tcp_attr
    globus_io_fileattr_destroy $file_attr
    
    # Reclaim memory space
    delete_p_globus_io_attr_t $tcp_attr
    delete_p_globus_io_attr_t $file_attr
    delete_p_globus_io_handle_t $read_handle
    delete_p_globus_io_handle_t $list_handle
    
    # Deactivate Globus I/O module
    globus_module_deactivate $globus_l_io_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
