#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

# In wish, we don't need to display the window
# But, we need to send it out of the visible part of the screen
if { [info commands wm] == "wm" } {
    wm iconify .
    wm geometry . 1x1+4000+4000
}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
set directory	[ file join [ file dirname [ pwd ] ] swig ]
set filename	[ file join $directory Makefile ]
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
proc cancel_cb { user_parms handle result } {
    if { [ catch {
	puts_info 10 "in cancel_cb"
	# In wish, we need to trigger event loop
	if { [info commands wm] == "wm" } {
	    wm iconify .
	}
	set ::cb_done 1
	puts_info 10 "leaving cancel_cb"
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Main script
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "[info nameofexecutable]"
    
    # Activate Globus I/O module
    globus_module_activate $globus_l_io_module

    set handle  [ new_p_globus_io_handle_t ]

    # globus_io_file_open
    globus_io_file_open \
	$filename \
	$GLOBUS_IO_FILE_RDONLY \
	0 \
	NULL \
	$handle

    # globus_io_cancel
    globus_io_cancel $handle $GLOBUS_FALSE 

    # globus_io_handle_set_user_pointer
    # globus_io_handle_get_user_pointer
    globus_io_handle_set_user_pointer $handle NULL 
    
    lassign [ globus_io_handle_get_user_pointer \
		  $handle \
		 ] result user_pointer
    puts_info 10 "user_pointer: $user_pointer"

    # globus_io_file_seek
    globus_io_file_seek $handle 512 $GLOBUS_IO_SEEK_SET 
    set bufsize 32
    lassign [ globus_io_read \
		  $handle \
		  $bufsize \
		  1 \
		 ] result buffer
    puts_info 10 "buffer: $buffer"
    puts_info 10 "buffer length: [ string bytelength $buffer ]"

    if { $result != $GLOBUS_SUCCESS || \
	     [ string bytelength $buffer ] != $bufsize } {
	set ExitStatus 0
    }
    
    set cb_done 0
    globus_io_register_cancel \
	$handle \
	$GLOBUS_TRUE \
	{ cancel_cb NULL } 
    vwait cb_done

    # Delete handle
    delete_p_globus_io_handle_t $handle

    # Deactivate Globus I/O module
    globus_module_deactivate $globus_l_io_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
