#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

set ::EXEC_MAIN 1

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
set SECTION io_test
validate_section $SECTION [list BUFFERSIZE WAITFORNBYTES]

set bufsize         $TestEnv($SECTION,BUFFERSIZE)
puts_info 10 "bufsize: $bufsize"
set waitforbytes    $TestEnv($SECTION,WAITFORNBYTES)
puts_info 10 "waitforbytes: $waitforbytes"

set sourcedir [ file join [ file dirname [ pwd ] ] swig ] 
puts_info 10 "sourcedir: $sourcedir"

set localdir [file join /tmp \
		  [file join "$env(USER)" \
		       [constructOutputDirectory $argv0]]]	
puts_info 10 "localdir: $localdir"

set filename $TestEnv($SECTION,FILENAME)
puts_info 10 "filename: $filename"

set port [ lindex $argv 0 ]
puts_info 10 "port: $port"

if { [ string length $port ] == 0 } {
    set mode -server
} else {
    set mode -client
}
puts_info 10 "mode: $mode"

#------------------------------------------------------------------------
# Globus setup
#------------------------------------------------------------------------
namespace eval globus {
    proc initialize { } {
	global globus_l_io_module
	
	#----------------------------------------------------------------
	# Activate Globus XIO module
	#----------------------------------------------------------------
	puts_info 10 "globus_module_activate"
	globus_module_activate $globus_l_io_module	
    }
    
    proc cleanup { } {
	global globus_l_io_module
		
	#----------------------------------------------------------------
	# Deactivate Globus XIO module
	#----------------------------------------------------------------
	puts_info 10 "globus_module_deactivate"
	globus_module_deactivate $globus_l_io_module
    }
}

#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
# CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT  #
#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
namespace eval client {
    variable filename

    #--------------------------------------------------------------------
    # Client
    #--------------------------------------------------------------------
    proc client { Filename } {

	variable filename $Filename

	if { [ catch {
	    # I/O attributes of the client
	    set tcp_attr  [ new_p_globus_io_attr_t ]
	    set file_attr [ new_p_globus_io_attr_t ]
	    
	    # Handle for reading data from a file
	    set write_handle  [ new_p_globus_io_handle_t ]
	    
	    # Handle for connection socket
	    set conn_handle  [ new_p_globus_io_handle_t ]
	    
	    # Client opens file handle to write data 
	    globus_io_fileattr_init $file_attr
	    
	    set write_flags [ expr $::GLOBUS_IO_FILE_CREAT | \
				  $::GLOBUS_IO_FILE_TRUNC | \
				  $::GLOBUS_IO_FILE_WRONLY ]
	    
	    set write_mode [ expr $::GLOBUS_IO_FILE_IRUSR | \
				 $::GLOBUS_IO_FILE_IWUSR ]
	    
	    globus_io_file_open \
		$filename \
		$write_flags \
		$write_mode \
		$file_attr \
		$write_handle
	    
	    # Connect to remote server
	    globus_io_tcpattr_init $tcp_attr
	    
	    globus_io_tcp_connect \
		[info hostname] \
		$::port \
		$tcp_attr \
		$conn_handle 
	    
	    # Read and write data 
	    do {
		# Read data from the connection handle
		lassign [ globus_io_read \
			      $conn_handle \
			      $::bufsize \
			      $::waitforbytes \
			     ] result buffer
		
		# Write data to the destination file using write handle
		lassign [ globus_io_write \
			      $write_handle \
			      $buffer 
			 ] result nbytes_written
	    } until { $nbytes_written == 0 }
	    
	    # Close connection handle and read handle
	    globus_io_close $conn_handle
	    globus_io_close $write_handle
	    
	    # Destroy the attribute
	    globus_io_tcpattr_destroy $tcp_attr
	    globus_io_fileattr_destroy $file_attr
	    
	    # Reclaim memory space
	    delete_p_globus_io_attr_t $tcp_attr
	    delete_p_globus_io_attr_t $file_attr
	    delete_p_globus_io_handle_t $write_handle

	    # Run MD5 checksum before exiting
	    check_md5sum $::read_cs $::write_cs
	    
	    # Deinitialization step
	    copy_deinit $::localdir

	} err ] } {
	    err_handler $err
	}	
    }
	
};# namespace client    


#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
# SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER  #
#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
namespace eval server {    
    variable tcp_attr
    variable file_attr
    variable read_handle
    variable list_handle
    variable conn_handle
    variable filename

    #--------------------------------------------------------------------
    # Server
    #--------------------------------------------------------------------
    proc server { } {	
	variable tcp_attr
	variable file_attr
	variable read_handle
	variable list_handle
	variable conn_handle
	variable filename

	if { [ catch {
	    puts_info 10 "Entering server"

	    # Wait a connection from client
	    globus_io_tcp_listen $list_handle

	    # Connection arrived, accept it
	    globus_io_tcp_accept \
		$list_handle \
		$tcp_attr \
		$conn_handle
	    
	    # Open the file handle to read data from source file
	    globus_io_file_open \
		$filename \
		$::GLOBUS_IO_FILE_RDONLY \
		0 \
		$file_attr \
		$read_handle
	    
	    # Check remote and local addresses
	    lassign [ globus_io_tcp_get_local_address \
			  $conn_handle ] result ipstring port
	    puts_info 10 "Accepted connection on my socket $ipstring:$port"
	    
	    lassign [ globus_io_tcp_get_remote_address \
			  $conn_handle ] result ipstring port
	    puts_info 10 "Accepted connection on my peer $ipstring:$port"
	    
	    # Read and write data 
	    do {
		lassign [ globus_io_read \
			      $read_handle \
			      $::bufsize \
			      $::waitforbytes ] result buffer
		
		lassign [ globus_io_send \
			      $conn_handle \
			      $buffer \
			      0 \
			     ] result nbytes_sent
	    } until { [string length $buffer] == 0 } 
	    
	    # Close listening handle and connection handle
	    globus_io_close $list_handle
	    globus_io_close $conn_handle
	    
	    # Close write handle
	    globus_io_close $read_handle
	    
	    puts_info 10 "Leaving server"
	} err ] } {
	    err_handler $err
	}	    
    }
    
    #--------------------------------------------------------------------
    # Server cleanup
    #--------------------------------------------------------------------
    proc cleanup { } {
	variable tcp_attr
	variable file_attr
	variable read_handle
	variable list_handle
	variable conn_handle

	if { [ catch {
	    puts_info 10 "server: Entering cleanup"
	    # Destroy the attribute
	    globus_io_tcpattr_destroy $tcp_attr
	    globus_io_fileattr_destroy $file_attr
	    
	    # Reclaim memory space
	    delete_p_globus_io_attr_t $tcp_attr
	    delete_p_globus_io_attr_t $file_attr
	    delete_p_globus_io_handle_t $read_handle
	    delete_p_globus_io_handle_t $list_handle
	    
	    puts_info 10 "server: Leaving cleanup"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server initialization
    #--------------------------------------------------------------------
    proc initialize { Filename } {
	variable tcp_attr
	variable file_attr
	variable read_handle
	variable list_handle
	variable conn_handle

	variable filename $Filename
	
	# I/O attributes of the server
	set tcp_attr  [ new_p_globus_io_attr_t ]
	set file_attr [ new_p_globus_io_attr_t ]
	
	# Handle for writing data to a file
	set read_handle  [ new_p_globus_io_handle_t ]
	
	# Handle for listening socket
	set list_handle  [ new_p_globus_io_handle_t ]
	
	# Handle for connection socket
	set conn_handle  [ new_p_globus_io_handle_t ]
	
	# Initialize the attribute
	globus_io_tcpattr_init $tcp_attr
	globus_io_fileattr_init $file_attr
	
	# Create a listening handle
	# 0 indicates a port number will be allocated by the system
	lassign [ globus_io_tcp_create_listener \
		      0 \
		      -1 \
		      $tcp_attr \
		      $list_handle ] result port
	puts_info 10 "server: initialize: port: $port"
	
	return $port
    }
};# Namespace server


#------------------------------------------------------------------------
# MAIN
#------------------------------------------------------------------------
proc main { } {
    if { [ catch {
	#----------------------------------------------------------------
	# Do initialization of globus
	#----------------------------------------------------------------
	globus::initialize
	
	# Set up local directory name
	set ::localdir [file join /tmp \
			  [file join "$::env(USER)" \
			       [constructOutputDirectory $::argv0]]]
	puts_info 10 "localdir: $::localdir"
	
	set ::write_cs [ file join $::localdir $::filename ] 
	puts_info 10 "write_cs: $::write_cs"
	
	set ::read_cs [ file join [ file dirname [ pwd ] ] swig $::filename ]
	puts_info 10 "read_cs: $::read_cs"

	# Create temporary directory
	copy_init $::localdir
	
	# Runs server process
	switch -- $::mode {
	    -client {
		client::client $::write_cs
	    }
	    
	    -server {
		puts_info 10 "argv0: $::argv0"
		puts_info 10 "argc: $::argc"
		set port [ server::initialize $::read_cs ]
		puts_info 10 "main: port: $port"
		
		# Exec client process
		catch { exec $::argv0 $port &  } client_pid
		
		puts_info 10 "client_pid: $client_pid"
		
		# Continue with the server
		server::server

		puts_info 10 "Server is completed"

		server::cleanup
	    }
	}

	#----------------------------------------------------------------
	# Do cleanup of globus
	#----------------------------------------------------------------
	globus::cleanup
    } err ] } {
	err_handler $err
    }
    #---------------------------------------------------------------------
    # Exit with normal exit status since test completed without error.
    #---------------------------------------------------------------------
    TestExit
}

if { $::EXEC_MAIN } {
    main
}
	
