#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
set SECTION io_test
validate_section $SECTION [list BUFFERSIZE]

set bufsize           $TestEnv($SECTION,BUFFERSIZE)

set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
puts_info 10 "sourcedir: $sourcedir"

set localdir [file join /tmp \
		  [file join "$env(USER)" \
		       [constructOutputDirectory $argv0]]]
puts_info 10 "localdir: $localdir"
#------------------------------------------------------------------------
# globus_io_try_read() and globus_io_try_write()
#------------------------------------------------------------------------
proc try_copy { read_cs write_cs } {
    global fileattr
    global bufsize
    
    if { [ catch {
	# creating a read handle
	set readhandle  [ new_p_globus_io_handle_t ]
	puts_info 10 \
	    "readhandle value: [p_globus_io_handle_t_value $readhandle]"

	# creating a write handle
	set writehandle [ new_p_globus_io_handle_t ]
	puts_info 10 \
	    "writehandle value: [p_globus_io_handle_t_value $writehandle]"

	# globus_io_file_open
	globus_io_file_open \
	    $read_cs \
	    $::GLOBUS_IO_FILE_RDONLY \
	    0 \
	    $fileattr \
	    $readhandle

	# globus_io_file_open to perform write operation
	set write_flags [ expr $::GLOBUS_IO_FILE_CREAT | \
			      $::GLOBUS_IO_FILE_TRUNC | \
			      $::GLOBUS_IO_FILE_WRONLY ]
	puts_info 10 "write_flags: $write_flags"
	set write_mode [ expr $::GLOBUS_IO_FILE_IRUSR | \
			     $::GLOBUS_IO_FILE_IWUSR ]
	puts_info 10 "write_mode: $write_mode"
	
	globus_io_file_open \
	    $write_cs \
	    $write_flags \
	    $write_mode \
	    $fileattr \
	    $writehandle

	# Actual read/write operation
	set remainingbytestoread [ file size $read_cs ]
	do {
	    lassign [ globus_io_try_read \
			  $readhandle \
			  $bufsize \
			 ] result buffer
	    puts_info 20 "globus_io_try_read returns $result"
	    puts_info 20 "bytes read: [ string length $buffer ]"
	    
	    lassign [ globus_io_try_write \
			  $writehandle \
			  $buffer \
			 ] result nbytes_written
	    puts_info 20 "globus_io_try_write returns $result"
	    puts_info 20 "nbytes_written: $nbytes_written"
	    
	    set remainingbytestoread [ expr { $remainingbytestoread - \
						  $nbytes_written } ]
	} until { $remainingbytestoread == 0 }
	
	# MD5 checksum
	check_md5sum $read_cs $write_cs

	# Close both handles
	globus_io_close $readhandle 
	globus_io_close $writehandle

	# Cleanup memory space
	delete_p_globus_io_handle_t $readhandle
	delete_p_globus_io_handle_t $writehandle
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Run test loop
#------------------------------------------------------------------------
proc run_test { } {
    global localdir
    
    if { [ catch {
	copy_init $localdir
	set filelist [ constructFileList ]
	puts_info 10 "Number of files: [ llength $filelist ]"
	foreach read_cs $filelist {
	    if { [file isdirectory $read_cs] } {
		continue
	    }
	    set write_cs [ file join $localdir [ file tail $read_cs ] ]
	    puts_info 20 "filename: $read_cs"
	    puts_info 20 "filename: $write_cs"
	    try_copy $read_cs $write_cs
	}
	copy_deinit $localdir
    } err ] } {
	return -code error $err
    }		
}
#------------------------------------------------------------------------
# Main script
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus I/O module
    puts_info 10 "Activating I/O module"
    globus_module_activate $globus_l_io_module
    
    # I/O attributes
    set fileattr [ new_p_globus_io_attr_t ]
    puts_info 20 "fileattr: $fileattr"
    puts_info 20 "fileattr value: [ p_globus_io_attr_t_value $fileattr ]"
    
    puts_info 10 "globus_io_fileattr_init"
    globus_io_fileattr_init $fileattr

    # Run the test in infinite loop mode or not
    if { [IsInfinite] == 1 } {
	while 1 {
	    run_test
	}
    } else {
	run_test
    }
    
    # Destroy I/O attribute
    puts_info 10 "globus_io_fileattr_destroy"
    globus_io_fileattr_destroy $fileattr
    
    # Reclaim memory space
    puts_info 10 "Reclaiming memory space"
    delete_p_globus_io_attr_t $fileattr
    
    # Deactivate Globus I/O module
    puts_info 10 "Deactivating I/O module"
    globus_module_deactivate $globus_l_io_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
