#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io \
		  globus_xio } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
set SECTION io_test
validate_section $SECTION [list WAITFORNBYTES]

set wait_for_nbytes   $TestEnv($SECTION,WAITFORNBYTES)

set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
puts_info 10 "sourcedir: $sourcedir"

set localdir [file join /tmp \
		  [file join "$env(USER)" \
		       [constructOutputDirectory $argv0]]]
puts_info 10 "localdir: $localdir"
#------------------------------------------------------------------------
# globus_xio_readv and globus_io_writev
#------------------------------------------------------------------------
proc vcopy { read_cs write_cs } {
    global stack
    global fileattr
    global wait_for_nbytes
    
    if { [ catch {
	set bufsize1      	1024
	set bufsize2      	2048
	set bufsize3		4096
	
	puts_info 10 "read_cs: $read_cs"
	puts_info 10 "write_cs: $write_cs"

	# creating read handle
	puts_info 10 "globus_xio_handle_create"
	lassign [ globus_xio_handle_create $stack ] result readhandle
	
	# Open both handles
	puts_info 10 "globus_xio_open"
	globus_xio_open $readhandle $read_cs NULL

	# creating write handle
	set writehandle [ new_p_globus_io_handle_t ]
	puts_info 10 "writehandle: $writehandle"
	puts_info 10 \
	    "writehandle value: [p_globus_io_handle_t_value $writehandle]"
	
	# globus_io_file_open() to perform write operation
	set write_flags [ expr $::GLOBUS_IO_FILE_CREAT | \
			      $::GLOBUS_IO_FILE_TRUNC | \
			      $::GLOBUS_IO_FILE_WRONLY ]
	puts_info 10 "write_flags: $write_flags"
	set write_mode [ expr $::GLOBUS_IO_FILE_IRUSR | \
			     $::GLOBUS_IO_FILE_IWUSR ]
	puts_info 10 "write_mode: $write_mode"
	
	# globus_io_file_open() for write
	puts_info 10 "globus_io_file_open"
	globus_io_file_open \
	    $write_cs \
	    $write_flags \
	    $write_mode \
	    $fileattr \
	    $writehandle
	
	# Actual read/write operation
	set remainingbytestoread [ file size $read_cs ]
	do {
	    lassign [ globus_xio_readv \
			  $readhandle \
			  [list $bufsize1 $bufsize2 $bufsize3] \
			  $wait_for_nbytes \
			  NULL ] status buf1 buf2 buf3
	    puts_info 20 "globus_xio_readv returns $status"
	    puts_info 20 "length of buf1: [string length $buf1]"
	    puts_info 20 "length of buf2: [string length $buf2]"
	    puts_info 20 "length of buf3: [string length $buf3]"
	    	    
	    lassign [ globus_io_writev \
			  $writehandle \
			  [list $buf1 $buf2 $buf3] \
			 ] result nbytes_written
	    puts_info 20 "globus_io_writev returns $result"
	    puts_info 20 "nbytes_written: $nbytes_written"
	    
	    set remainingbytestoread [ expr { $remainingbytestoread - \
						  $nbytes_written } ]
	} until { $remainingbytestoread == 0 }
	
	# MD5 checksum
	check_md5sum $read_cs $write_cs
	
	# Close read handle
	puts_info 10 "globus_xio_close"
	globus_xio_close $readhandle NULL
	
	# Close write handle
	globus_io_close $writehandle
	# Cleanup memory space
	delete_p_globus_io_handle_t $writehandle
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Run test loop
#------------------------------------------------------------------------
proc run_test { } {
    global localdir
    
    if { [ catch {
	copy_init $localdir
	set filelist [ constructFileList ]
	puts_info 10 "Number of files: [ llength $filelist ]"
	foreach read_cs $filelist {
	    if { [file isdirectory $read_cs] } {
		continue
	    }
	    set write_cs [ file join $localdir [ file tail $read_cs ] ]
	    puts_info 20 "filename: $read_cs"
	    puts_info 20 "filename: $write_cs"
	    vcopy $read_cs $write_cs
	}
	copy_deinit $localdir
    } err ] } {
	return -code error $err
    }		
}
#------------------------------------------------------------------------
# Main script
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus I/O module
    puts_info 10 "Activating I/O module"
    globus_module_activate $globus_l_io_module
    # Activate Globus XIO module
    puts_info 10 "Activating XIO module"
    globus_module_activate $globus_i_xio_module

    # I/O attributes
    set fileattr [ new_p_globus_io_attr_t ]
    puts_info 20 "fileattr value: [ p_globus_io_attr_t_value $fileattr ]"

    # globus_io_fileattr_init
    globus_io_fileattr_init $fileattr

    # Load the driver
    puts_info 10 "globus_xio_driver_load"
    lassign [ globus_xio_driver_load file ] result driver
    
    # Create the driver stack and push the driver onto the stack
    puts_info 10 "globus_xio_stack_init"
    lassign [ globus_xio_stack_init NULL ] result stack
    puts_info 10 "globus_xio_stack_push_driver"
    globus_xio_stack_push_driver $stack $driver
	
    # Run the test in infinite loop mode or not
    if { [IsInfinite] == 1 } {
	while 1 {
	    run_test
	}
    } else {
	run_test
    }
    
    # Destroy the stack
    puts_info 10 "globus_xio_stack_destroy"
    globus_xio_stack_destroy $stack

    # Unload the driver
    puts_info 10 "globus_xio_driver_unload"
    globus_xio_driver_unload $driver
	
    # Destroy I/O attribute
    puts_info 10 "globus_io_fileattr_destroy"
    globus_io_fileattr_destroy $fileattr

    # Reclaim memory space
    puts_info 10 "Reclaiming memory space"
    delete_p_globus_io_attr_t $fileattr

    # Deactivate globus module
    puts_info 10 "Deactivating XIO module"
    globus_module_deactivate $globus_i_xio_module
    # Deactivate Globus I/O module
    puts_info 10 "Deactivating I/O module"
    globus_module_deactivate $globus_l_io_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
