#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_io \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc auth_cb { user_parms handle result identity context_handle } {
    if { [ catch {
	puts_info 10 "entering auth_cb"
	upvar 0 $user_parms user_args
	puts_info 10 "leaving auth_cb"
    } err ] } {
	return -code error $err
    }	
}
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "[info nameofexecutable]"
    
    # Activate Globus I/O module
    globus_module_activate $globus_l_io_module

    set handle  [ new_p_globus_io_handle_t ]
    
    # Initialize the attribute
    set tcp_attr	[ new_p_globus_io_attr_t ]
    set auth_data	[ new_p_globus_io_secure_authorization_data_t ]
    
    globus_io_tcpattr_init $tcp_attr
    globus_io_secure_authorization_data_initialize $auth_data

    # globus_io_attr_set_callback_space
    # globus_io_attr_get_callback_space
    if { 0 } {
	globus_io_attr_set_callback_space \
	    $tcp_attr \
	    [ expr 0 ] 
	
	lassign [ globus_io_attr_get_callback_space \
		      $tcp_attr \
		     ] result callback_space
    }
    
    # Testing TCP restrict_port attribute
    globus_io_attr_set_tcp_restrict_port \
	$tcp_attr \
	$GLOBUS_TRUE 
    
    lassign [ globus_io_attr_get_tcp_restrict_port \
		  $tcp_attr \
		 ] result restrict_port

    puts_info 10 "restrict_port: $restrict_port"

    if { $restrict_port != $GLOBUS_TRUE } {
	incr ExitStatus
    }
    
    # Testing TCP reuseaddr attribute
    globus_io_attr_set_socket_reuseaddr \
		  $tcp_attr \
		  $GLOBUS_TRUE 

    lassign [ globus_io_attr_get_socket_reuseaddr \
		  $tcp_attr \
		 ] result reuseaddr
    
    puts_info 10 "reuseaddr: $reuseaddr"

    if { $reuseaddr != $GLOBUS_TRUE } {
	incr ExitStatus
    }
    
    # Testing TCP keepalive attribute
    globus_io_attr_set_socket_keepalive \
	$tcp_attr \
	$GLOBUS_TRUE 
    
    lassign [ globus_io_attr_get_socket_keepalive \
		  $tcp_attr \
		 ] result keepalive

    puts_info 10 "keepalive: $keepalive"

    if { $keepalive != $GLOBUS_TRUE } {
	incr ExitStatus
    }
    
    # Testing TCP linger attribute 
    set linger_value 60

    globus_io_attr_set_socket_linger \
	$tcp_attr \
	$GLOBUS_TRUE \
	$linger_value 

    lassign [ globus_io_attr_get_socket_linger \
		  $tcp_attr \
		 ] result linger linger_time

    puts_info 10 "linger: $linger"
    puts_info 10 "linger_time: $linger_time"

    if { $linger != $GLOBUS_TRUE && $linger_time != $linger_value } {
	incr ExitStatus
    }
    
    # Testing TCP oobinline attribute
    globus_io_attr_set_socket_oobinline \
	$tcp_attr \
	$GLOBUS_FALSE 

    lassign [ globus_io_attr_get_socket_oobinline \
		  $tcp_attr \
		 ] result oobinline

    puts_info 10 "oobinline: $oobinline"

    if { $oobinline != $GLOBUS_FALSE } {
	incr ExitStatus
    }
    
    # Testing TCP sndbuf attribute
    set sndbuf_value 256

    globus_io_attr_set_socket_sndbuf \
	$tcp_attr \
	$sndbuf_value 

    lassign [ globus_io_attr_get_socket_sndbuf \
		  $tcp_attr \
		 ] result sndbuf

    puts_info 10 "sndbuf: $sndbuf"

    if { $sndbuf != $sndbuf_value } {
	incr ExitStatus
    }
    
    # Testing TCP rcvbuf attribute
    set rcvbuf_value 512

    lassign [ globus_io_attr_set_socket_rcvbuf \
		  $tcp_attr \
		  $rcvbuf_value \
		 ] result

    lassign [ globus_io_attr_get_socket_rcvbuf \
		  $tcp_attr \
		 ] result rcvbuf

    puts_info 10 "rcvbuf: $rcvbuf"

    if { $rcvbuf != $rcvbuf_value } {
	incr ExitStatus
    }
    
    # Testing TCP nodelay attribute
    globus_io_attr_set_tcp_nodelay \
	$tcp_attr \
	$GLOBUS_TRUE 

    lassign [ globus_io_attr_get_tcp_nodelay \
		  $tcp_attr \
		 ] result nodelay

    puts_info 10 "nodelay: $nodelay"

    if { $nodelay != $GLOBUS_TRUE } {
	incr ExitStatus
    }
    
    # Testing TCP interface attribute
    set interface_value 127.0.0.1

    globus_io_attr_set_tcp_interface \
	$tcp_attr \
	$interface_value 
    
    lassign [ globus_io_attr_get_tcp_interface \
		  $tcp_attr \
		 ] result interface

    puts_info 10 "interface: $interface"

    if { [ string compare $interface $interface_value ] != 0 } {
	incr ExitStatus
    }
    
    # Testing authentication mode
    globus_io_attr_set_secure_authentication_mode \
	$tcp_attr \
	$GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI \
	$gss_c_no_credential 
    
    lassign [ globus_io_attr_get_secure_authentication_mode \
		  $tcp_attr \
		 ] result authentication_mode credential

    puts_info 10 "authentication_mode: $authentication_mode"
    puts_info 10 "credential: $credential"

    if { $result != $GLOBUS_SUCCESS } { 
	incr ExitStatus
    }
    
    # Testing extension OID
    globus_io_attr_set_secure_extension_oids \
	$tcp_attr \
	$gss_c_no_oid_set

    lassign [ globus_io_attr_get_secure_extension_oids \
		  $tcp_attr \
		 ] result extension_oids

    puts_info 10 "extension_oids: $extension_oids"

    if { $extension_oids != $gss_c_no_oid_set } {
	incr ExitStatus
    }

    # globus_io_secure_authorization_data_set_callback
    # globus_io_secure_authorization_data_get_callback
    if { 0 } {
	set cb_done	0
	
	globus_io_secure_authorization_data_set_callback \
	    $auth_data \
	    { auth_cb NULL } 
	vwait cb_done
	
	lassign [  globus_io_secure_authorization_data_get_callback \
		       $auth_data \
		      ] result 
	
	if { $result != $GLOBUS_SUCCESS } {
	    incr FAIL
	}
    }
    
    # Testing authorization mode
    globus_io_attr_set_secure_authorization_mode \
	$tcp_attr \
	$GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF \
	$auth_data
    
    lassign [ globus_io_attr_get_secure_authorization_mode \
		  $tcp_attr \
		  $auth_data \
		 ] result authorization_mode

    puts_info 10 "authorization_mode: $authorization_mode"

    if { $authorization_mode != $GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF } {
	incr ExitStatus
    }

    # Testing authorization data identity
    set id	\
	"/DC=org/DC=doegrids/OU=People/CN=Michael Samidi 288621"
    
    globus_io_secure_authorization_data_set_identity $auth_data $id 

    lassign [ globus_io_secure_authorization_data_get_identity \
		  $auth_data \
		 ] result identity

    puts_info 10 "identity: $identity"

    if { $identity != $id } {
	incr ExitStatus
    }
    
    # Testing channel mode
    globus_io_attr_set_secure_channel_mode \
	$tcp_attr \
	$GLOBUS_IO_SECURE_CHANNEL_MODE_SSL_WRAP 

    lassign [ globus_io_attr_get_secure_channel_mode \
		  $tcp_attr \
		 ] result channel_mode

    puts_info 10 "channel_mode: $channel_mode"

    if { $channel_mode != $GLOBUS_IO_SECURE_CHANNEL_MODE_SSL_WRAP } {
	incr ExitStatus
    }
    
    # Testing protection mode
    globus_io_attr_set_secure_protection_mode \
	$tcp_attr \
	$GLOBUS_IO_SECURE_PROTECTION_MODE_PRIVATE 
    
    lassign [ globus_io_attr_get_secure_protection_mode \
		  $tcp_attr \
		 ] result protection_mode

    puts_info 10 "protection_mode: $protection_mode"

    if { $protection_mode != $GLOBUS_IO_SECURE_PROTECTION_MODE_PRIVATE } {
	incr ExitStatus
    }
    
    # Testing delegation mode
    globus_io_attr_set_secure_delegation_mode \
	$tcp_attr \
	$GLOBUS_IO_SECURE_DELEGATION_MODE_FULL_PROXY 

    lassign [ globus_io_attr_get_secure_delegation_mode \
		  $tcp_attr \
		 ] result delegation_mode

    puts_info 10 "delegation_mode: $delegation_mode"

    if { $delegation_mode != $GLOBUS_IO_SECURE_DELEGATION_MODE_FULL_PROXY } {
	incr ExitStatus
    }
    
    # Testing proxy mode
    globus_io_attr_set_secure_proxy_mode \
	$tcp_attr \
	$GLOBUS_IO_SECURE_PROXY_MODE_NONE 

    lassign [ globus_io_attr_get_secure_proxy_mode \
		  $tcp_attr \
		 ] result proxy_mode

    puts_info 10 "proxy_mode: $proxy_mode"

    if { $proxy_mode != $GLOBUS_IO_SECURE_PROXY_MODE_NONE } {
	incr ExitStatus
    }

    # Create a listening handle
    lassign [ globus_io_tcp_create_listener \
		  0 \
		  -1 \
		  $tcp_attr \
		  $handle ] result port
    puts_info 10 "[info hostname]:$port"

    # Testing globus_io_tcp_get_attr()
    set new_attr	[ new_p_globus_io_attr_t ]
    
    globus_io_tcp_get_attr $handle $new_attr
    
    # Destroy the attribute
    globus_io_secure_authorization_data_destroy $auth_data 

    globus_io_tcpattr_destroy $tcp_attr

    # Reclaim memory space
    delete_p_globus_io_secure_authorization_data_t $auth_data
    delete_p_globus_io_attr_t $tcp_attr
    delete_p_globus_io_attr_t $new_attr
    delete_p_globus_io_handle_t $handle

    # Deactivate Globus I/O module
    globus_module_deactivate $globus_l_io_module
    
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
