#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Determine if we have threads or not
#------------------------------------------------------------------------
set ::HAVE_PACKAGE_THREAD 0
set ::EXEC_MAIN 1
if { 0 } {
    if { ! [catch {package require Thread}] } {
	set ::HAVE_PACKAGE_THREAD 1
	if { ! [info exists ::LOADING_THREAD_INTERP] } {
	    set ::LOADING_THREAD_INTERP 0
	} else {
	    set ::EXEC_MAIN 0
	}
    }
}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Initialize necessary variables 
#------------------------------------------------------------------------
set SECTION xio_test
validate_section $SECTION [list BUFFERSIZE]
validate_section $SECTION [list WAITFORNBYTES]

set bufsize      $TestEnv($SECTION,BUFFERSIZE)
set waitforbytes $TestEnv($SECTION,WAITFORNBYTES)

set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
puts_info 10 "sourcedir: $sourcedir"

set localdir [file join /tmp \
		  [file join "$env(USER)" \
		       [constructOutputDirectory $argv0]]]

puts_info 10 "bufsize: $bufsize"
puts_info 10 "waitforbytes: $waitforbytes"
puts_info 10 "localdir: $localdir"

set cb_client_done 0

namespace eval globus {
    proc initialize { } {
	global globus_i_xio_module
	global tcpdriver
	global filedriver
	
	#----------------------------------------------------------------
	# Activate Globus XIO module
	#----------------------------------------------------------------
	puts_info 10 "globus_module_activate"
	globus_module_activate $globus_i_xio_module
	
	#----------------------------------------------------------------
	# Load transport driver
	#----------------------------------------------------------------
	puts_info 10 "globus_xio_driver_load"
	lassign [ globus_xio_driver_load tcp ] result tcpdriver
	lassign [ globus_xio_driver_load file ] result filedriver
    }
    
    proc cleanup { } {
	global globus_i_xio_module
	global tcpdriver
	global filedriver
	
	#----------------------------------------------------------------
	# Unload both drivers
	#----------------------------------------------------------------
	puts_info 10 "globus_xio_driver_unload"
	lassign [ globus_xio_driver_unload $tcpdriver ] result
	lassign [ globus_xio_driver_unload $filedriver ] result
	
	#----------------------------------------------------------------
	# Deactivate Globus XIO module
	#----------------------------------------------------------------
	puts_info 10 "globus_module_deactivate"
	globus_module_deactivate $globus_i_xio_module
    }
}

#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
# CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT  #
#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
namespace eval client {
    variable client_filestack
    variable client_tcpstack
    variable data
    variable filename
    
    #--------------------------------------------------------------------
    # Dest close callback
    #--------------------------------------------------------------------
    proc dest_close_cb { user_parms handle result } {
	global cb_client_done
	global HAVE_PACKAGE_THREAD
	variable server_thread
	
	if { [ catch {	    
	    puts_info 10 "in dest_close_cb"
	    if { $HAVE_PACKAGE_THREAD } {
		[namespace current]::cleanup
		thread::send -async $server_thread [subst {
		    set ::cb_client_done 1
		}]
		thread::release [thread::id]
	    } else {
		set cb_client_done 1
	    }
	    puts_info 10 "leaving dest_close_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Client close callback
    #--------------------------------------------------------------------
    proc client_close_cb { user_parms handle result } {
	if { [ catch {
	    puts_info 10 "in client_close_cb"
	    upvar 0 $user_parms user_args
	    lassign [ globus_xio_register_close \
			  $user_args(dest_handle) \
			  NULL \
			  [ subst { [namespace current]::dest_close_cb \
					$user_parms } ] \
			 ] result
	    
	    puts_info 10 "leaving client_close_cb"
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Dest read callback
    #--------------------------------------------------------------------
    proc dest_read_cb { user_parms handle result buffer data_desc } {
	if { [ catch {
	    puts_info 20 "in dest_read_cb"
	    upvar 0 $user_parms user_args
	    lassign [ globus_xio_register_read \
			  $user_args(client_handle) \
			  $::bufsize \
			  $::waitforbytes \
			  NULL \
			  [ subst { [namespace current]::dest_write_cb \
					$user_parms } ] \
			 ] result
	    
	    puts_info 20 "leaving dest_read_cb" 
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Dest write callback
    #--------------------------------------------------------------------
    proc dest_write_cb { user_parms handle result buffer data_desc } {
	if { [ catch {
	    upvar 0 $user_parms user_args
	    puts_info 20 "in dest_write_cb"
	    puts_info 30 "client buffer: $buffer"
	    puts_info 40 "buffer length : [ string length $buffer ]"
	    
	    if { [string length $buffer] > 0 } {
		lassign [ globus_xio_register_write \
			      $user_args(dest_handle) \
			      $buffer \
			      $::waitforbytes \
			      NULL \
			      [ subst { [namespace current]::dest_read_cb \
					    $user_parms } ] \
			     ] result
	    } else {

		lassign [ globus_xio_register_close \
			      $user_args(client_handle) \
			      NULL \
			      [ subst { [namespace current]::client_close_cb \
					    $user_parms } ] \
			     ] result
	    }
	    puts_info 20 "leaving dest_write_cb"
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Client read callback
    #--------------------------------------------------------------------
    proc client_read_cb { user_parms handle result } {
	if { [ catch {
	    puts_info 10 "in client_read_cb"
	    upvar 0 $user_parms user_args
	    puts_info 10 "user_args(client_handle): $user_args(client_handle)"

	    lassign [ globus_xio_register_read \
			  $user_args(client_handle) \
			  $::bufsize \
			  $::waitforbytes \
			  NULL \
			  [ subst { [namespace current]::dest_write_cb \
					$user_parms } ] \
			 ] result
	    
		      puts_info 10 "leaving client_read_cb" 
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Open callback
    #--------------------------------------------------------------------
    proc dest_open_cb { user_parms handle result } {
	if { [ catch {
	    puts_info 10 "in dest_open_cb"
	    upvar 0 $user_parms user_args
	    variable filename
	    # Open the destination
	    lassign [ globus_xio_register_open \
			  $user_args(dest_handle) \
			  $filename \
			  NULL \
			  [ subst { [namespace current]::client_read_cb \
					$user_parms } ] \
			 ] result
	    
	    puts_info 10 "leaving dest_open_cb"
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Client
    #--------------------------------------------------------------------
    proc client { server_cs } {
	global HAVE_PACKAGE_THREAD
	variable client_filestack
	variable client_tcpstack
	variable data
	
	if { [ catch {
	    if { $HAVE_PACKAGE_THREAD } {
		puts_info 10 "Running client in thread: [thread::id]"
	    }
	    lassign [ globus_xio_handle_create \
			  $client_tcpstack \
			 ] result data(client_handle)
	    puts_info 10 "client: globus_xio_handle_create: $data(client_handle)"
	    lassign [ globus_xio_handle_create \
			  $client_filestack \
			 ] result data(dest_handle)
	    puts_info 10 "client: globus_xio_handle_create: $data(dest_handle)"
	    
	    #while { ! [info exists $server_cs] } {
	    #}
	    
	    lassign [ globus_xio_register_open \
			  $data(client_handle) \
			  $server_cs \
			  NULL \
			  [subst { [namespace current]::dest_open_cb \
				       [namespace current]::data }] \
			 ] result
	    
	    puts_info 10 "Connected to $server_cs"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Cleanup client
    #--------------------------------------------------------------------
    proc cleanup { } {
	variable client_filestack
	variable client_tcpstack
	global HAVE_PACKAGE_THREAD
	
	if { [ catch {
	    puts_info 10 "globus_xio_stack_destroy - client"

	    lassign [ globus_xio_stack_destroy $client_filestack ] result
	    
	    lassign [ globus_xio_stack_destroy $client_tcpstack ] result
	    
	    if { $HAVE_PACKAGE_THREAD } {
		globus::cleanup
	    }
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Initialize client
    #--------------------------------------------------------------------
    proc initialize { Filename } {
	puts_info 10 "Initialize client"
	global tcpdriver
	global filedriver
	global HAVE_PACKAGE_THREAD
	
	variable filename $Filename
	variable client_filestack
	variable client_tcpstack
	
	if { $HAVE_PACKAGE_THREAD } {
	    globus::initialize
	}
	if { [ catch {
	    puts_info 10 "client: globus_xio_stack_init"
	    lassign [ globus_xio_stack_init NULL ] result client_filestack
	    lassign [ globus_xio_stack_init NULL ] result client_tcpstack
	    
	    puts_info 10 "client: globus_xio_stack_push_driver"
	    lassign [ globus_xio_stack_push_driver \
			  $client_tcpstack \
			  $tcpdriver \
			 ] result
	    
	    lassign [ globus_xio_stack_push_driver \
			  $client_filestack \
			  $filedriver \
			 ] result
	    
	} err ] } {
	    err_handler $err
	}
    }
};# namespace client

#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
# SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER  #
#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
namespace eval server {
    variable data
    variable server_cs
    variable server_filestack
    variable server_tcpstack
    variable filename

    #--------------------------------------------------------------------
    # Server close callback
    #--------------------------------------------------------------------
    proc server_done_cb { user_parms handle result } {
	if { [ catch {
	    puts_info 10 "in server_done_cb"
	    puts_info 10 "leaving server_done_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server close callback
    #--------------------------------------------------------------------
    proc server_close_cb { user_parms handle result } {
	if { [ catch {
	    puts_info 10 "in server_close_cb"
	    upvar 0 $user_parms user_args
	    puts_info 10 "server: globus_xio_server_register_close: $user_args(server)"
	    lassign [ globus_xio_server_register_close \
			  $user_args(server) \
			  [ subst { [namespace current]::server_done_cb \
					$user_parms } ] \
			 ] result
	    
	    puts_info 10 "leaving server_close_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Source close callback
    #--------------------------------------------------------------------
    proc source_close_cb { user_parms handle result } {
	if { [ catch {
	    puts_info 10 "in source_close_cb"
	    upvar 0 $user_parms user_args
	    puts_info 10 "server: globus_xio_register_close: $user_args(source_handle)"
	    lassign [ globus_xio_register_close \
			  $user_args(source_handle) \
			  NULL \
			  [ subst { [namespace current]::server_close_cb \
					$user_parms } ]  \
			 ] result
	    
	    puts_info 10 "leaving source_close_cb"
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Server read callback
    #--------------------------------------------------------------------
    proc server_read_cb { user_parms handle result buffer data_desc } {
	if { [ catch {
	    puts_info 20 "in server_read_cb"
	    # We need to access our associative array
	    upvar 0 $user_parms user_args
	    lassign [ globus_xio_register_read \
			  $user_args(source_handle) \
			  $::bufsize \
			  $::waitforbytes \
			  NULL \
			  [ subst { [namespace current]::server_write_cb \
					$user_parms } ] \
			 ] result
	    
	    puts_info 20 "leaving server_read_cb"
	}  err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server write callback
    #--------------------------------------------------------------------
    proc server_write_cb { user_parms handle result buffer data_desc } {
	if { [ catch {
	    upvar 0 $user_parms user_args
	    puts_info 20 "in server_write_cb"
	    puts_info 20 "length of data: [string length $buffer]"
	    
	    if { [string length $buffer] > 0 } {
		lassign [globus_xio_register_write \
			     $user_args(server_handle) \
			     $buffer \
			     $::waitforbytes \
			     NULL \
			     [ subst { [namespace current]::server_read_cb \
					   $user_parms } ] \
			    ] result
	    } else {
		puts_info 10 "server: globus_xio_register_close: $user_args(server_handle)"
		lassign [ globus_xio_register_close \
			      $user_args(server_handle) \
			      NULL \
			      [ subst { [namespace current]::source_close_cb \
					    $user_parms } ] \
			     ] result
	    }
	    puts_info 20 "leaving server_write_cb" 
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Source read callback
    #--------------------------------------------------------------------
    proc source_read_cb { user_parms handle result } {
	if { [ catch {
	    puts_info 20 "in source_read_cb"
	    # We need to access our associative array
	    upvar 0 $user_parms user_args

	    lassign [ globus_xio_register_read \
			  $user_args(source_handle) \
			  $::bufsize \
			  $::waitforbytes \
			  NULL \
			  [ subst { [namespace current]::server_write_cb \
					$user_parms } ]  \
			 ] result
	    
	    puts_info 20 "leaving source_read_cb"
	}  err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Source open callback
    #--------------------------------------------------------------------
    proc source_open_cb { user_parms handle result } {
	if { [ catch {
	    variable filename
	    puts_info 10 "in source_open_cb"
	    upvar 0 $user_parms user_args
	    puts_info 10 "server: globus_xio_register_open: $user_args(source_handle)"
	    # Open the destination
	    lassign [ globus_xio_register_open \
			  $user_args(source_handle) \
			  $filename \
			  NULL \
			  [ subst { [namespace current]::source_read_cb \
					$user_parms } ] \
			 ] result
	    
	    puts_info 10 "leaving source_open_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server accept callback
    #--------------------------------------------------------------------
    proc server_accept_cb { user_parms server server_handle result } {
	variable server_cs
	if { [ catch {
	    puts_info 10 "in server_accept_cb"
	    upvar 0 $user_parms user_args
	    set user_args(server_handle) $server_handle
	    puts_info 10 "server: globus_xio_register_open: $server_handle"
	    lassign [ globus_xio_register_open \
			  $server_handle \
			  $server_cs \
			  NULL \
			  [ subst { [namespace current]::source_open_cb \
					$user_parms } ]  \
			 ] result
	    
	    puts_info 10 "leaving server_accept_cb"
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server
    #--------------------------------------------------------------------
    proc server { } {	
	variable data
	variable server_filestack
	variable server_tcpstack
	
	if { [ catch {
	    # Create server object
	    lassign [ globus_xio_server_create \
			  NULL \
			  $server_tcpstack \
			 ] result data(server)
	    puts_info 10 "server: globus_xio_server_create: $data(server)"
	    # Create source file handle
	    lassign [ globus_xio_handle_create \
			  $server_filestack \
			 ] result data(source_handle)
	    puts_info 10 "server: globus_xio_handle_create: $data(source_handle)"
	    # Get server contact string
	    variable server_cs
	    lassign [ globus_xio_server_get_contact_string \
			  $data(server) \
			 ] result server_cs
	    puts_info 10 "Accepting connections $server_cs"
	    # Register server accept callback
	    lassign [ globus_xio_server_register_accept \
			  $data(server) \
			  [subst { [namespace current]::server_accept_cb \
				       [namespace current]::data }] \
			 ] result
	} err ] } {
	    err_handler $err
	}
    }
	
    #--------------------------------------------------------------------
    # Server cleanup
    #--------------------------------------------------------------------
    proc cleanup { } {
	variable server_filestack
	variable server_tcpstack
	
	if { [ catch {
	    # Stack construction
	    puts_info 10 "globus_xio_stack_destroy - server"

	    lassign [ globus_xio_stack_destroy $server_filestack ] result

	    lassign [ globus_xio_stack_destroy $server_tcpstack ] result

	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server initialization
    #--------------------------------------------------------------------
    proc initialize { Filename } {
	puts_info 10 "Initialize server"
	global filedriver
	global tcpdriver
	
	variable server_filestack
	variable server_tcpstack
	variable filename $Filename
	
	if { [ catch {
	    puts_info 10 "server: filename: $filename"
	    
	    # Stack construction
	    puts_info 10 "server: globus_xio_stack_init"
	    lassign [ globus_xio_stack_init NULL ] result server_filestack
	    lassign [ globus_xio_stack_init NULL ] result server_tcpstack
	    
	    # Push drivers onto stack
	    puts_info 10 "server: globus_xio_stack_push_driver"
	    lassign [ globus_xio_stack_push_driver \
			  $server_filestack \
			  $filedriver  \
			 ] result
	    
	    lassign [ globus_xio_stack_push_driver \
			  $server_tcpstack \
			  $tcpdriver \
			 ] result
	} err ] } {
	    err_handler $err
	}
    }
};# Namespace server

#------------------------------------------------------------------------
# Run test loop
#------------------------------------------------------------------------
proc run_test { } {
    global localdir
    global SECTION
    global TestEnv
    global env
    global HAVE_PACKAGE_THREAD

    # Create temporary directory
    copy_init $localdir
    
    # Construct file list
    set in_filelist [ constructFileList ]
    puts_info 10 "Number of files: [ llength $in_filelist ]"
    
    foreach source_filename $in_filelist {
	if { [file isdirectory $source_filename] } {
	    continue
	}
	set destination_filename \
	    [ file join $localdir [ file tail $source_filename ] ]
	puts_info 10 "Source file: $source_filename"
	puts_info 10 "Dest file: $destination_filename"
	
	#----------------------------------------------------------------
	# In both models, the server will run in the main thread
	#----------------------------------------------------------------
	server::initialize $source_filename
	
	if { $HAVE_PACKAGE_THREAD } {
	    #------------------------------------------------------------
	    # Load the client thread with the commands it will need
	    #------------------------------------------------------------
	    set c [thread::create {
		set ::LOADING_THREAD_INTERP 1
		source "$env(SRCDIR)/test_xio_async_client_server.tcl"
		puts_info 10 "Finished loading client thread"
		thread::wait
	    }]
	    puts_info 10 "Created client thread: $c"
	    
	    #------------------------------------------------------------
	    # Start the server
	    #------------------------------------------------------------
	    server::server
	    
	    #------------------------------------------------------------
	    # Initialize the client thread and then call its main entry
	    #   point. This will start the data exchange between the
	    #   client and server. The client thread will terminate
	    #   once it has written all bytes to the output file.
	    #------------------------------------------------------------
	    thread::send -async $c [subst {
		::client::initialize $destination_filename
		set ::client::server_thread [thread::id]
		::client::client $::server::server_cs
	    }]
	    #------------------------------------------------------------
	    # wait for the client to complete
	    #------------------------------------------------------------
	    vwait cb_client_done
	    
	} else {
	    #------------------------------------------------------------
	    # Don't have the Thread package so just run everything in
	    #   in the main thread. This ONLY works because we are using
	    #   the asyncronous calls.
	    #------------------------------------------------------------
	    client::initialize $destination_filename
	    
	    server::server
	    client::client $::server::server_cs
	    vwait cb_client_done
	    
	    client::cleanup
	}
	server::cleanup
	puts_info 10 "Client/Server done"
	
	#-----------------------------------------------------------------
	# Run MD5 checksum before exiting
	#-----------------------------------------------------------------
	check_md5sum $source_filename $destination_filename
    }
    #-----------------------------------------------------------------
    # Remove temporary directory
    #-----------------------------------------------------------------
    copy_deinit $localdir
}

#------------------------------------------------------------------------
# MAIN
#------------------------------------------------------------------------
proc main { } {
    if { [ catch {
	#----------------------------------------------------------------
	# Do initialization of globus
	#----------------------------------------------------------------
	globus::initialize
	
	# Run the test in infinite loop mode or not
	if { [IsInfinite] == 1 } {
	    while 1 {
		run_test
	    }
	} else {
	    run_test
	}
	
	#----------------------------------------------------------------
	# Do cleanup of globus
	#----------------------------------------------------------------
	globus::cleanup
    } err ] } {
	err_handler $err
    }
    #---------------------------------------------------------------------
    # Exit with normal exit status since test completed without error.
    #---------------------------------------------------------------------
    TestExit
}

puts_info 10 "HAVE_PACKAGE_THREAD: $HAVE_PACKAGE_THREAD"
if { $::EXEC_MAIN } {
    main
}
