#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Tcl application really starts here
#------------------------------------------------------------------------
#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Sanity check to verify that the version of Globus being used supports
#   the xio library.
#------------------------------------------------------------------------
if { [ info exists globus_i_xio_module ] == 0 } {
    #--------------------------------------------------------------------
    # No xio extensions as is the case with globus 2.4.x
    #--------------------------------------------------------------------
    puts_info 10 "No XIO extensions as is the case with globus 2.4.x"
    TestExit    
}

#------------------------------------------------------------------------
# Initialize necessary variables 
#------------------------------------------------------------------------
set SECTION xio_test
validate_section $SECTION [list WAITFORNBYTES]

set drivername    file

set waitforbytes  $TestEnv($SECTION,WAITFORNBYTES)

set bufsize1      1024
set bufsize2      2048
set bufsize3      4096

set bufsize_list	[ list $bufsize1 $bufsize2 $bufsize3 ]

set sourcedir         	[ file join [ file dirname [ pwd ] ] swig ]
set localdir [file join /tmp \
		  [file join "$env(USER)" [constructOutputDirectory $argv0]]]
puts_info 10 "localdir: $localdir"

#------------------------------------------------------------------------
# Close callback
#------------------------------------------------------------------------
proc close_cb { user_parms handle result } {
    if { [ catch {
	puts_info 10 "in close_cb"

	# In wish, we need to trigger event loop
	if { [info commands wm] == "wm" } {
	    wm iconify .
	}
	set ::cb_done 1

	puts_info 10 "leaving close_cb"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# Dest close callback
#------------------------------------------------------------------------
proc dest_close_cb { user_parms handle result } {
    if { [ catch {
	puts_info 10 "in dest_close_cb"
	upvar 0 $user_parms user_args
	globus_xio_register_close \
	    $user_args(dest_handle) \
	    NULL \
	    [ subst { close_cb $user_parms } ] 
	puts_info 10 "leaving dest_close_cb"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# Destination write callback
#------------------------------------------------------------------------
proc dest_write_cb { user_parms handle result iovec count nbytes data_desc } {
    if { [ catch {
	puts_info 20 "in dest_write_cb"
	upvar 0 $user_parms user_args
	puts_info 20 "nbytes: $nbytes"
	puts_info 20 "iovec length: [llength $iovec]"
	
	globus_xio_register_readv \
	    $user_args(source_handle) \
	    $::bufsize_list \
	    $::waitforbytes \
	    NULL \
	    [ subst { source_read_cb $user_parms } ] 
	
	puts_info 20 "leaving dest_write_cb"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# Source read callback
#------------------------------------------------------------------------
proc source_read_cb { user_parms handle result iovec count nbytes data_desc } {
    if { [ catch {
	puts_info 20 "in source_read_cb"
	upvar 0 $user_parms user_args
	puts_info 20 "nbytes: $nbytes"
	puts_info 20 "iovec length: [llength $iovec]"
	set buflist {}
	foreach element $iovec {
	    if { [subst {llength $element}] > 0 } {
		lappend buflist $element
	    }
	}
	if { $nbytes > 0 } { 
	    globus_xio_register_writev \
		$user_args(dest_handle) \
		$buflist \
		$::waitforbytes \
		NULL \
		[ subst { dest_write_cb $user_parms } ] 
	} else {
	    globus_xio_register_close \
		$user_args(source_handle) \
		NULL \
		[ subst { dest_close_cb $user_parms } ] 
	}
	puts_info 20 "leaving source_read_cb"
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# Destination open callback
#------------------------------------------------------------------------
proc dest_open_cb { user_parms handle result } {
    if { [ catch {
	puts_info 10 "in dest_open_cb"
	# We need to access our associative array
	upvar 0 $user_parms user_args
	
	# We have opened the source and destination.
	# Let's start reading from the source
	# Pay a careful attention on how we construct the cb !!!
	globus_xio_register_readv \
	    $user_args(source_handle) \
	    $::bufsize_list \
	    $::waitforbytes \
	    NULL \
	    [ subst { source_read_cb $user_parms } ] 
	
	puts_info 10 "leaving dest_open_cb" 
    }  err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
# Source open callback
#------------------------------------------------------------------------
proc source_open_cb { user_parms handle result } {
    if { [ catch {
	puts_info 10 "in source_open_cb"
	upvar 0 $user_parms user_args
	# Open the destination
        globus_xio_register_open \
	    $user_args(dest_handle) \
	    $user_args(dest_cs) \
	    NULL \
	    [ subst { dest_open_cb $user_parms } ] 
	
	puts_info 10 "leaving source_open_cb" 
    } err ] } {
	err_handler $err
    }
}
#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
proc run_test { } {
    if { [ catch {
	# Initialization step
	copy_init $::localdir
	
	set in_filelist [ constructFileList ]
	puts_info 10 "Number of files: [ llength $in_filelist ]"
	
	# Start asynchronous vector copy test
	foreach read_cs $in_filelist {
	    puts_info 10 "read_cs: $read_cs"
	    if { [file isdirectory $read_cs] } {
		continue
	    }
	    set ::copy_info(source_cs) $read_cs
	    set ::copy_info(dest_cs) [ file join $::localdir \
					   [ file tail $read_cs ] ]
	    # Create the destination handle and put it into 
	    # our associative array
	    lassign [ globus_xio_handle_create \
			  $::dest_stack \
			 ] result ::copy_info(dest_handle)
	    # Create the source handle and put it into our associative array
	    lassign [ globus_xio_handle_create \
			  $::source_stack \
			 ] result ::copy_info(source_handle)

	    # In wish, we don't need to display the window
	    # But, we need to send it out of the visible part of the screen
	    if { [info commands wm] == "wm" } {
		wm iconify .
		wm geometry . 1x1+4000+4000
	    }

	    set ::cb_done 0

	    # Open the source
	    globus_xio_register_open \
		$::copy_info(source_handle) \
		$::copy_info(source_cs) \
		NULL \
		{ source_open_cb ::copy_info }
	    
	    # Go into the tcl event loop waiting for thread callbacks
	    vwait ::cb_done

	    # MD5 checksum
	    check_md5sum $::copy_info(source_cs) $::copy_info(dest_cs)
	}

	if { [info commands wm] == "wm" } {
	    destroy .
	}
	
	# Deinitialization step
	copy_deinit $::localdir
	
    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus XIO module
    puts_info 10 "globus_module_activate"
    globus_module_activate $globus_i_xio_module

    # Load the driver
    puts_info 10 "globus_xio_driver_load"
    lassign [ globus_xio_driver_load $drivername ] result driver

    # Create the driver stack
    puts_info 10 "globus_xio_stack_init - source"
    lassign [ globus_xio_stack_init NULL ] result source_stack
    puts_info 10 "globus_xio_stack_init - dest"
    lassign [ globus_xio_stack_init NULL ] result dest_stack

    # Push the driver onto the stack
    puts_info 10 "globus_xio_stack_push_driver - source"
    globus_xio_stack_push_driver $source_stack $driver 
    puts_info 10 "globus_xio_stack_push_driver - dest"
    globus_xio_stack_push_driver $dest_stack $driver
    
    # Run the test in infinite loop mode or not
    if { [IsInfinite] == 1 } {
	while 1 {
	    run_test
	}
    } else {
	run_test
    }
	
    # Destroy the stack
    puts_info 10 "globus_xio_stack_destroy - source"
    globus_xio_stack_destroy $source_stack
    puts_info 10 "globus_xio_stack_destroy - dest"
    globus_xio_stack_destroy $dest_stack
    
    # Unload the driver
    puts_info 10 "globus_xio_driver_unload"
    globus_xio_driver_unload $driver
    
    # Deactivate globus module
    puts_info 10 "globus_module_deactivate"
    globus_module_deactivate $globus_i_xio_module

} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
