#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Sanity check to verify that the version of Globus being used supports
#   the xio library.
#------------------------------------------------------------------------
if { [ info exists globus_i_xio_module ] == 0 } {
    #--------------------------------------------------------------------
    # No xio extensions as is the case with globus 2.4.x
    #--------------------------------------------------------------------
    TestExit
}

#------------------------------------------------------------------------
# Initialize necessary variables 
#------------------------------------------------------------------------
set SECTION xio_test
validate_section $SECTION [list BUFFERSIZE]
validate_section $SECTION [list WAITFORNBYTES]

set tcpdrivername   tcp
set filedrivername  file

set bufsize         $TestEnv($SECTION,BUFFERSIZE)
set waitforbytes    $TestEnv($SECTION,WAITFORNBYTES)

set contact_string  [ lindex $argv 0 ]
set write_cs	    [ lindex $argv 1 ]

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus XIO module
    globus_module_activate $globus_i_xio_module

    # Load transport driver
    lassign [ globus_xio_driver_load $tcpdrivername ] result tcpdriver
    lassign [ globus_xio_driver_load $filedrivername ] result filedriver

    # Stack construction
    lassign [ globus_xio_stack_init NULL ] result filestack
    lassign [ globus_xio_stack_init NULL ] result tcpstack
    
    # Push drivers onto stack
    globus_xio_stack_push_driver $filestack $filedriver
    globus_xio_stack_push_driver $tcpstack $tcpdriver

    # File handle
    lassign [ globus_xio_handle_create $filestack ] result filehandle	
    globus_xio_open $filehandle $write_cs NULL
	
    # Network handle
    lassign [ globus_xio_handle_create $tcpstack ] result networkhandle
    globus_xio_open $networkhandle $contact_string NULL

    # Blocking read and write
    do {
	if { [ catch {
	    # Read from network
	    lassign [ globus_xio_read \
			  $networkhandle \
			  $bufsize \
			  $waitforbytes \
			  NULL \
			 ] status buffer
	    
	    # Write to file
	    lassign [ globus_xio_write \
			  $filehandle \
			  $buffer \
			  $waitforbytes \
			  NULL \
			 ] status nwritten
	} err ] } {
	    switch -regex -- $err {
		{end of file} { 
		    # Reach end of file and exit from the loop
		    break
		}
	    }
	}
    } until { $nwritten == 0 }
    
    # Close both handles
    globus_xio_close $networkhandle NULL
    globus_xio_close $filehandle NULL  
       
    # Destroy both stacks
    globus_xio_stack_destroy $filestack
    globus_xio_stack_destroy $tcpstack

    # Unload both drivers
    globus_xio_driver_unload $tcpdriver
    globus_xio_driver_unload $filedriver

    # Deactivate Globus XIO module
    globus_module_deactivate $globus_i_xio_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
