#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Sanity check to verify that the version of Globus being used supports
# the xio library.
#------------------------------------------------------------------------
if { [ info exists globus_i_xio_module ] == 0 } {
    #--------------------------------------------------------------------
    # No xio extensions as is the case with globus 2.4.x
    #--------------------------------------------------------------------
    puts_info 10 "No XIO extensions as is the case with globus 2.4.x"
    TestExit
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus XIO module
    puts_info 10 "Activating XIO module"
    globus_module_activate $globus_i_xio_module

    # Load the driver
    puts_info 10 "globus_xio_driver_load"
    lassign [ globus_xio_driver_load file ] result filedriver

    puts_info 10 "globus_xio_attr_init"
    lassign [ globus_xio_attr_init ] result attr
    
    # Create XIO stack and push the drivers onto the stack
    puts_info 10 "globus_xio_stack_init"
    lassign [ globus_xio_stack_init NULL ] result stack
    
    puts_info 10 "globus_xio_stack_push_driver"
    lassign [ globus_xio_stack_push_driver \
		  $stack \
		  $filedriver \
		 ] result
    
    # Create handle
    puts_info 10 "globus_xio_handle_create"
    lassign [ globus_xio_handle_create \
		  $stack \
		 ] result handle

    puts_info 10 "GLOBUS_XIO_FILE_SET_MODE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $filedriver \
		  $GLOBUS_XIO_FILE_SET_MODE \
		  $GLOBUS_XIO_FILE_IRWXU \
		 ] result
    
    puts_info 10 "GLOBUS_XIO_FILE_GET_MODE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $filedriver \
		  $GLOBUS_XIO_FILE_GET_MODE \
		 ] result file_mode
    puts_info 10 "file_mode $file_mode"
    
    puts_info 10 "GLOBUS_XIO_FILE_SET_FLAGS"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $filedriver \
		  $GLOBUS_XIO_FILE_SET_FLAGS \
		  $GLOBUS_XIO_FILE_TEXT \
		 ] result

    puts_info 10 "GLOBUS_XIO_FILE_GET_FLAGS"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $filedriver \
		  $GLOBUS_XIO_FILE_GET_FLAGS \
		 ] result file_flag
    puts_info 10 "file_flag $file_flag"

    puts_info 10 "GLOBUS_XIO_FILE_SET_TRUNC_OFFSET"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $filedriver \
		  $GLOBUS_XIO_FILE_SET_TRUNC_OFFSET \
		  1025 \
		 ] result
    
    puts_info 10 "GLOBUS_XIO_FILE_GET_TRUNC_OFFSET"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $filedriver \
		  $GLOBUS_XIO_FILE_GET_TRUNC_OFFSET \
		 ] result offset
    puts_info 10 "offset $offset"

    puts_info 10 "GLOBUS_XIO_FILE_SET_HANDLE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $filedriver \
		  $GLOBUS_XIO_FILE_SET_HANDLE \
		  10
		 ] result
    
    puts_info 10 "GLOBUS_XIO_FILE_GET_HANDLE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $filedriver \
		  $GLOBUS_XIO_FILE_GET_HANDLE \
		 ] result handle
    puts_info 10 "handle $handle"
    
    # Destroy the stack
    puts_info 10 "globus_xio_stack_destroy"
    lassign [ globus_xio_stack_destroy \
		  $stack \
		 ] result

    # Destroy both attributes
    puts_info 10 "globus_xio_attr_destroy"
    lassign [ globus_xio_attr_destroy \
		  $attr \
		 ] result
    
    # Unload the driver
    lassign [ globus_xio_driver_unload \
		  $filedriver \
		 ] result
    
    # Deactivate globus module
    puts_info 10 "Deactivating XIO module"
    globus_module_deactivate $globus_i_xio_module
} err ] } {
    err_handler $err
}

puts_info 10 "Done"

#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
