#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Sanity check to verify that the version of Globus being used supports
# the xio library.
#------------------------------------------------------------------------
if { [ info exists globus_i_xio_module ] == 0 } {
    #--------------------------------------------------------------------
    # No xio extensions as is the case with globus 2.4.x
    #--------------------------------------------------------------------
    puts_info 10 "No XIO extensions as is the case with globus 2.4.x"
    TestExit
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus XIO module
    puts_info 10 "Activating XIO module"
    globus_module_activate $globus_i_xio_module

    # Load the driver
    puts_info 10 "globus_xio_driver_load"
    lassign [ globus_xio_driver_load tcp ] status tcpdriver
    puts_info 10 "\tstatus $status"
    lassign [ globus_xio_driver_load gsi ] status gsidriver
    puts_info 10 "\tstatus $status"
    
    puts_info 10 "globus_xio_attr_init"
    lassign [ globus_xio_attr_init ] status attr
    puts_info 10 "\tstatus $status"
    
    # Create XIO stack and push the drivers onto the stack
    puts_info 10 "globus_xio_stack_init"
    lassign [ globus_xio_stack_init NULL ] status stack
    puts_info 10 "\tstatus $status"
    
    puts_info 10 "globus_xio_stack_push_driver"
    lassign [ globus_xio_stack_push_driver \
		  $stack \
		  $tcpdriver \
		 ] status
    puts_info 10 "\tstatus $status"
    
    puts_info 10 "globus_xio_stack_push_driver"
    lassign [ globus_xio_stack_push_driver \
		  $stack \
		  $gsidriver \
		 ] status
    puts_info 10 "\tstatus $status"
    
    puts_info 10 "GLOBUS_XIO_GSI_SET_CREDENTIAL"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_CREDENTIAL \
		  $gss_c_no_credential \
		 ] status
    puts_info 10 "\tstatus $status"

    puts_info 10 "GLOBUS_XIO_GSI_GET_CREDENTIAL"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_GET_CREDENTIAL \
		 ] status cred_handle
    puts_info 10 "\tstatus $status cred_handle $cred_handle"

    puts_info 10 "GLOBUS_XIO_GSI_SET_GSSAPI_REQ_FLAGS"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_GSSAPI_REQ_FLAGS \
		  $GSS_C_ANON_FLAG \
		 ] status
    puts_info 10 "\tstatus $status"
    
    puts_info 10 "GLOBUS_XIO_GSI_GET_GSSAPI_REQ_FLAGS"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_GET_GSSAPI_REQ_FLAGS \
		 ] status req_flags
    puts_info 10 "\tstatus $status req_flags $req_flags"

    puts_info 10 "GLOBUS_XIO_GSI_SET_PROXY_MODE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_PROXY_MODE \
		  $GLOBUS_XIO_GSI_PROXY_MODE_LIMITED \
		 ] status
    puts_info 10 "\tstatus $status"

    puts_info 10 "GLOBUS_XIO_GSI_GET_PROXY_MODE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_GET_PROXY_MODE \
		 ] status proxy_mode
    puts_info 10 "\tstatus $status proxy_mode $proxy_mode"

    puts_info 10 "GLOBUS_XIO_GSI_SET_AUTHORIZATION_MODE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_AUTHORIZATION_MODE \
		  $GLOBUS_XIO_GSI_SELF_AUTHORIZATION \
		 ] status
    puts_info 10 "\tstatus $status"

    puts_info 10 "GLOBUS_XIO_GSI_GET_AUTHORIZATION_MODE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_GET_AUTHORIZATION_MODE \
		 ] status auth_mode
    puts_info 10 "\tstatus $status auth_mode $auth_mode"

    puts_info 10 "GLOBUS_XIO_GSI_SET_DELEGATION_MODE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_DELEGATION_MODE \
		  $GLOBUS_XIO_GSI_DELEGATION_MODE_FULL \
		 ] status
    puts_info 10 "\tstatus $status"

    puts_info 10 "GLOBUS_XIO_GSI_GET_DELEGATION_MODE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_GET_DELEGATION_MODE \
		 ] status deleg_mode
    puts_info 10 "\tstatus $status deleg_mode $deleg_mode"

    puts_info 10 "GLOBUS_XIO_GSI_SET_SSL_COMPATIBLE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_SSL_COMPATIBLE \
		  $GLOBUS_TRUE \
		 ] status
    puts_info 10 "\tstatus $status"

    puts_info 10 "GLOBUS_XIO_GSI_SET_ANON"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_ANON \
		  $GLOBUS_TRUE \
		 ] status
    puts_info 10 "\tstatus $status"

    puts_info 10 "GLOBUS_XIO_GSI_SET_WRAP_MODE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_WRAP_MODE \
		  $GLOBUS_TRUE \
		 ] status
    puts_info 10 "\tstatus $status"

    puts_info 10 "GLOBUS_XIO_GSI_GET_WRAP_MODE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_GET_WRAP_MODE \
		 ] status wrap_mode
    puts_info 10 "\tstatus $status wrap_mode $wrap_mode"

    puts_info 10 "GLOBUS_XIO_GSI_SET_BUFFER_SIZE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_BUFFER_SIZE \
		  1024 \
		 ] status
    puts_info 10 "\tstatus $status"

    puts_info 10 "GLOBUS_XIO_GSI_GET_BUFFER_SIZE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_GET_BUFFER_SIZE \
		 ] status buffer_size
    puts_info 10 "\tstatus $status buffer_size $buffer_size"

    puts_info 10 "GLOBUS_XIO_GSI_SET_PROTECTION_LEVEL"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_PROTECTION_LEVEL \
		  $GLOBUS_XIO_GSI_PROTECTION_LEVEL_PRIVACY \
		 ] status
    puts_info 10 "\tstatus $status"
    
    puts_info 10 "GLOBUS_XIO_GSI_GET_PROTECTION_LEVEL"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_GET_PROTECTION_LEVEL \
		 ] status prot_level
    puts_info 10 "\tstatus $status prot_level $prot_level"

    puts_info 10 "GLOBUS_XIO_GSI_SET_TARGET_NAME"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_SET_TARGET_NAME \
		  $gss_c_no_name \
		 ] status
    puts_info 10 "\tstatus $status"
    
    puts_info 10 "GLOBUS_XIO_GSI_GET_TARGET_NAME"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $gsidriver \
		  $GLOBUS_XIO_GSI_GET_TARGET_NAME \
		 ] status target_name
    puts_info 10 "\tstatus $status target_name $target_name"
    
    # Destroy the stack
    puts_info 10 "globus_xio_stack_destroy"
    lassign [ globus_xio_stack_destroy \
		  $stack \
		 ] result

    # Destroy both attributes
    puts_info 10 "globus_xio_attr_destroy"
    lassign [ globus_xio_attr_destroy \
		  $attr \
		 ] result
    
    # Unload the driver
    lassign [ globus_xio_driver_unload \
		  $gsidriver \
		 ] result
    
    # Deactivate globus module
    puts_info 10 "Deactivating XIO module"
    globus_module_deactivate $globus_i_xio_module
} err ] } {
    err_handler $err
}

puts_info 10 "Done"

#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
