#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec wish "$0" ${1+"$@"}

set ::EXEC_MAIN 1

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

verifyProxy

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio \
		  gssapi } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Sanity check to verify that the version of Globus being used supports
#   the xio library.
#------------------------------------------------------------------------
if { [ info exists globus_i_xio_module ] == 0 } {
    #--------------------------------------------------------------------
    # No xio extensions as is the case with globus 2.4.x
    #--------------------------------------------------------------------
    TestExit
}

#------------------------------------------------------------------------
# Initialize necessary variables 
#------------------------------------------------------------------------
set SECTION xio_test
validate_section $SECTION [list FILENAME BUFFERSIZE WAITFORNBYTES]

set bufsize $TestEnv($SECTION,BUFFERSIZE)
puts_info 10 "bufsize: $bufsize"
set waitforbytes $TestEnv($SECTION,WAITFORNBYTES)
puts_info 10 "waitforbytes: $waitforbytes"

set sourcedir [ file join [ file dirname [ pwd ] ] swig ] 
puts_info 10 "sourcedir: $sourcedir"

set localdir [file join /tmp \
		  [file join "$env(USER)" \
		       [constructOutputDirectory $argv0]]]	
puts_info 10 "localdir: $localdir"

set filename $TestEnv($SECTION,FILENAME)
puts_info 10 "filename: $filename"

set contact_string [ lindex $argv 0 ]
puts_info 10 "contact_string: $contact_string"

if { [ string length $contact_string ] == 0 } {
    set mode -server
} else {
    set mode -client
}
puts_info 10 "mode: $mode"

#------------------------------------------------------------------------
# Globus setup
#------------------------------------------------------------------------
namespace eval globus {
    proc initialize { } {
	global globus_i_xio_module
	global tcpdriver
	global filedriver
	global gsidriver
	
	#----------------------------------------------------------------
	# Activate Globus XIO module
	#----------------------------------------------------------------
	puts_info 10 "globus_module_activate"
	globus_module_activate $globus_i_xio_module
	
	#----------------------------------------------------------------
	# Load transport driver
	#----------------------------------------------------------------
	puts_info 10 "globus_xio_driver_load"
	lassign [ globus_xio_driver_load tcp ] status tcpdriver
	lassign [ globus_xio_driver_load file ] status filedriver
	lassign [ globus_xio_driver_load gsi ] result gsidriver
    }
    
    proc cleanup { } {
	global globus_i_xio_module
	global tcpdriver
	global filedriver
	global gsidriver
	
	#----------------------------------------------------------------
	# Unload both drivers
	#----------------------------------------------------------------
	puts_info 10 "globus_xio_driver_unload"
	lassign [ globus_xio_driver_unload $tcpdriver ] status
	lassign [ globus_xio_driver_unload $filedriver ] status
	lassign [ globus_xio_driver_unload $gsidriver ] status
	
	#----------------------------------------------------------------
	# Deactivate Globus XIO module
	#----------------------------------------------------------------
	puts_info 10 "globus_module_deactivate"
	globus_module_deactivate $globus_i_xio_module
    }
}


#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
# CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT CLIENT  #
#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
namespace eval client {
    variable client_filestack
    variable client_tcpstack
    variable filename
    
    #--------------------------------------------------------------------
    # Initialize client
    #--------------------------------------------------------------------
    proc client { Filename } {
	puts_info 10 "Initialize client"
	global tcpdriver
	global filedriver
	global gsidriver
	
	variable filename $Filename
	
	if { [ catch {
	        # GSI driver specific attribute initialization
	    lassign [ globus_xio_attr_init ] result gsiattr 

	    puts_info 10 "client: globus_xio_stack_init"
	    lassign [ globus_xio_stack_init NULL ] status client_filestack
	    lassign [ globus_xio_stack_init NULL ] status client_tcpstack
	    	    
	    lassign [ globus_xio_stack_push_driver \
			  $client_tcpstack \
			  $tcpdriver \
			 ] status
	    puts_info 10 "client: globus_xio_stack_push_driver: tcpdriver"

	    lassign [ globus_xio_stack_push_driver \
			  $client_tcpstack \
			  $gsidriver \
			 ] status
	    puts_info 10 "client: globus_xio_stack_push_driver: gsidriver"
	    
	    lassign [ globus_xio_stack_push_driver \
			  $client_filestack \
			  $filedriver \
			 ] status
	    puts_info 10 "client: globus_xio_stack_push_driver: filedriver"
	    
	    lassign [ globus_xio_handle_create \
			  $client_tcpstack \
			 ] result network_handle
	    puts_info 10 "client: globus_xio_handle_create: $network_handle"
	       
	    lassign [ globus_xio_handle_create \
			  $client_filestack \
			 ] result file_handle
	    puts_info 10 "client: globus_xio_handle_create: $file_handle"

	    # GSI driver specific attribute settings
	    puts_info 10 "gss_c_no_credential: $::gss_c_no_credential"
	    
	    # Authentication part - Set GSS-API authentication mode
	    globus_xio_attr_cntl $gsiattr \
		$gsidriver \
		$::GLOBUS_XIO_GSI_SET_CREDENTIAL \
		$::gss_c_no_credential
	    
	    lassign [ globus_xio_attr_cntl \
			  $gsiattr \
			  $gsidriver \
			  $::GLOBUS_XIO_GSI_GET_CREDENTIAL \
			 ] result credential
	    puts_info 10 "credential: $credential"
	    
	    # Authentication part - Set GSI authorization mode
	    globus_xio_attr_cntl \
		$gsiattr \
		$gsidriver \
		$::GLOBUS_XIO_GSI_SET_AUTHORIZATION_MODE \
		$::GLOBUS_XIO_GSI_SELF_AUTHORIZATION 
	    
	    lassign [ globus_xio_attr_cntl \
			  $gsiattr \
			  $gsidriver \
			  $::GLOBUS_XIO_GSI_GET_AUTHORIZATION_MODE \
			 ] result authz_mode
	    puts_info 10 "authz_mode: $authz_mode"
	    
	    # Encryption part - Set the channel mode to use wrapping
	    globus_xio_attr_cntl \
		$gsiattr \
		$gsidriver \
		$::GLOBUS_XIO_GSI_SET_WRAP_MODE \
		$::GLOBUS_TRUE 
	    
	    lassign [ globus_xio_attr_cntl \
			  $gsiattr \
			  $gsidriver \
			  $::GLOBUS_XIO_GSI_GET_WRAP_MODE \
			 ] result wrap_mode
	    puts_info 10 "wrap_mode: $wrap_mode"
	    
	    # Encryption part - Messages are signed
	    globus_xio_attr_cntl \
		$gsiattr \
		$gsidriver \
		$::GLOBUS_XIO_GSI_SET_PROTECTION_LEVEL \
		$::GLOBUS_XIO_GSI_PROTECTION_LEVEL_PRIVACY
    
	    lassign [ globus_xio_attr_cntl \
			  $gsiattr \
			  $gsidriver \
			  $::GLOBUS_XIO_GSI_GET_PROTECTION_LEVEL \
			 ] result protection_level
	    puts_info 10 "protection_level: $protection_level"
	    
	    puts_info 10 "client: filename: $filename"
	    lassign [ globus_xio_open $file_handle $filename NULL ] status
	    puts_info 10 "client: globus_xio_open: status: $status"
	    
	    lassign [ globus_xio_open \
			  $network_handle \
			  $::contact_string \
			  $gsiattr \
			 ] status
	    puts_info 10 "Connected to $::contact_string"

	    # Blocking read and write
	    do {
		if { [ catch {
		    # Read from network
		    lassign [ globus_xio_read \
				  $network_handle \
				  $::bufsize \
				  $::waitforbytes \
				  NULL \
				 ] status buffer
		    puts_info 10 "client: status: $status"
		    puts_info 10 "client: buffer length: [string length $buffer]"
		   
		    # Write to file
		    lassign [ globus_xio_write \
				  $file_handle \
				  $buffer \
				  $::waitforbytes \
				  NULL \
				 ] status nwritten
		    
		    puts_info 10 "client: nwritten: $nwritten"
		} err ] } {
		    switch -regex -- $err {
			{end of file} { 
			    # Reach end of file and exit from the loop
			    break
			}
		    }
		}
	    } until { [string length $buffer] == 0 }
	    
	    # Close both handles
	    puts_info 10 "client: globus_xio_close"
	    lassign [ globus_xio_close $network_handle NULL ] status
	    lassign [ globus_xio_close $file_handle NULL ] status  

	    puts_info 10 "client: globus_xio_stack_destroy - client"
	    lassign [ globus_xio_stack_destroy $client_filestack ] status
	    lassign [ globus_xio_stack_destroy $client_tcpstack ] status
	    
	    # Destroy both attributes
	    globus_xio_attr_destroy $gsiattr

	    # Run MD5 checksum before exiting
	    check_md5sum $::read_cs $::write_cs
	    
	    # Deinitialization step
	    copy_deinit $::localdir
	} err ] } {
	    err_handler $err
	}
    }
};# namespace client    


#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
# SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER SERVER  #
#------------------------------------------------------------------------#
#------------------------------------------------------------------------#
namespace eval server {
    variable server_cs
    variable server_filestack
    variable server_tcpstack
    variable filename
    variable server
    variable filehandle
    variable gsiattr
    
    #--------------------------------------------------------------------
    # Server
    #--------------------------------------------------------------------
    proc server { } {	
	variable data
	variable server_filestack
	variable server_tcpstack
	variable filename
	variable server_cs
	variable server
	variable filehandle
	variable gsiattr
	
	if { [ catch {
	    puts_info 10 "server: globus_xio_server_accept"
	    lassign [ globus_xio_server_accept $server ] status tcphandle

	    puts_info 10 "server: globus_xio_open"
	    lassign [ globus_xio_open $tcphandle $server_cs $gsiattr ] status

	    # Blocking read and write
	    set remainingbytestoread [ file size $filename ]
	    set totalbufsizes [ expr $::bufsize ]
	    do {
		if { [ catch {
		    # Read from file
		    lassign [ globus_xio_read \
				  $filehandle \
				  $::bufsize \
				  $::waitforbytes \
				  NULL \
				 ] read_status buffer
		    puts_info 10 "server: buffer: [string length $buffer]"
		    # Write to network
		    lassign [ globus_xio_write \
				  $tcphandle \
				  $buffer \
				  $::waitforbytes \
				  NULL \
				 ] write_status nbytes
		    puts_info 10 "server: nbytes: $nbytes"
		} err ] } {
		    switch -regex -- $err {
			{end of file} { 
			    # Reach end of file and exit from the loop
			    break
			}
		    }
		}
		
		set remainingbytestoread [ expr { $remainingbytestoread - \
						      $totalbufsizes } ]
		
	    } until { $remainingbytestoread < 0 }

	    # Close server
	    globus_xio_close $tcphandle NULL
	    globus_xio_server_close $server
	    
	    # Close file handle
	    globus_xio_close $filehandle NULL
    
	} err ] } {
	    err_handler $err
	}
    }

    #--------------------------------------------------------------------
    # Server cleanup
    #--------------------------------------------------------------------
    proc cleanup { } {
	variable server_filestack
	variable server_tcpstack
	variable gsiattr
	if { [ catch {
	    
	    # Stack construction
	    puts_info 10 "globus_xio_stack_destroy - server"
	    globus_xio_stack_destroy $server_filestack
	    globus_xio_stack_destroy $server_tcpstack

	    # Destroy both attribute variables
	    globus_xio_attr_destroy $gsiattr
	    
	} err ] } {
	    err_handler $err
	}
    }
    
    #--------------------------------------------------------------------
    # Server initialization
    #--------------------------------------------------------------------
    proc initialize { Filename } {
	puts_info 10 "Initialize server"
	global filedriver
	global tcpdriver
	global gsidriver
	
	variable server_filestack
	variable server_tcpstack
	variable filename $Filename
	variable server
	variable server_cs
	variable filehandle
	variable gsiattr
	
	puts_info 10 "server: filename: $filename"
	
	# Stack construction
	puts_info 10 "server: globus_xio_stack_init"
	lassign [ globus_xio_stack_init NULL ] result server_filestack
	lassign [ globus_xio_stack_init NULL ] result server_tcpstack
	
	# Push drivers onto stack
	puts_info 10 "server: globus_xio_stack_push_driver"
	lassign [ globus_xio_stack_push_driver \
		      $server_filestack \
		      $filedriver \
		     ] status
	
	lassign [ globus_xio_stack_push_driver \
		      $server_tcpstack \
		      $tcpdriver \
		     ] status

	lassign [ globus_xio_stack_push_driver \
		      $server_tcpstack \
		      $gsidriver \
		     ] status

	lassign [ globus_xio_attr_init ] result gsiattr

	# GSI driver specific attribute settings
	puts_info 10 "gss_c_no_credential: $::gss_c_no_credential"
	
	# Authentication part
	# $gss_c_no_credential indicates a default credential 
	globus_xio_attr_cntl \
	    $gsiattr \
	    $gsidriver \
	    $::GLOBUS_XIO_GSI_SET_CREDENTIAL \
	    $::gss_c_no_credential
	
	lassign [ globus_xio_attr_cntl \
		      $gsiattr \
		      $gsidriver \
		      $::GLOBUS_XIO_GSI_GET_CREDENTIAL \
		     ] result credential
	puts_info 10 "credential: $credential"
	
	# Authentication part - Set GSI authorization mode
	globus_xio_attr_cntl \
	    $gsiattr \
	    $gsidriver \
	    $::GLOBUS_XIO_GSI_SET_AUTHORIZATION_MODE \
	    $::GLOBUS_XIO_GSI_SELF_AUTHORIZATION 
	
	lassign [ globus_xio_attr_cntl \
		      $gsiattr \
		      $gsidriver \
		      $::GLOBUS_XIO_GSI_GET_AUTHORIZATION_MODE \
		     ] result authz_mode
	puts_info 10 "authz_mode: $authz_mode"
	
	# Encryption part - Set the channel mode to use wrapping
	globus_xio_attr_cntl \
	    $gsiattr \
	    $gsidriver \
	    $::GLOBUS_XIO_GSI_SET_WRAP_MODE \
	    $::GLOBUS_TRUE 
	
	lassign [ globus_xio_attr_cntl \
		      $gsiattr \
		      $gsidriver \
		      $::GLOBUS_XIO_GSI_GET_WRAP_MODE \
		     ] result wrap_mode
	puts_info 10 "wrap_mode: $wrap_mode"
	
	# Encryption part - Messages are signed
	globus_xio_attr_cntl \
	    $gsiattr \
	    $gsidriver \
	    $::GLOBUS_XIO_GSI_SET_PROTECTION_LEVEL \
	    $::GLOBUS_XIO_GSI_PROTECTION_LEVEL_PRIVACY 
	
	lassign [ globus_xio_attr_cntl \
		      $gsiattr \
		      $gsidriver \
		      $::GLOBUS_XIO_GSI_GET_PROTECTION_LEVEL \
		     ] result protection_level
	puts_info 10 "protection_level: $protection_level"
	
	# File handle
	puts_info 10 "server: globus_xio_handle_create"
	lassign [ globus_xio_handle_create \
		      $server_filestack\
		     ] status filehandle
	
	lassign [ globus_xio_open $filehandle $filename NULL ] status
	
	# Server handle
	puts_info 10 "server: globus_xio_server_create"
	lassign [ globus_xio_server_create \
		      NULL \
		      $server_tcpstack \
		     ] result server
	
	lassign [ globus_xio_server_get_contact_string \
		      $server \
		     ] result server_cs
	
	puts_info 10 "server: $server_cs"
	
	return $server_cs
    }
};# Namespace server


#------------------------------------------------------------------------
# MAIN
#------------------------------------------------------------------------
proc main { } {
    if { [ catch {
	#----------------------------------------------------------------
	# Do initialization of globus
	#----------------------------------------------------------------
	globus::initialize
	
	# Set up local directory name
	set ::localdir [file join /tmp \
			  [file join "$::env(USER)" \
			       [constructOutputDirectory $::argv0]]]
	puts_info 10 "localdir: $::localdir"
	
	set ::write_cs [ file join $::localdir $::filename ] 
	puts_info 10 "write_cs: $::write_cs"
	
	set ::read_cs [ file join [ file dirname [ pwd ] ] swig $::filename ]
	puts_info 10 "read_cs: $::read_cs"

	# Create temporary directory
	copy_init $::localdir
	
	# Runs server process
	switch -- $::mode {
	    -client {
		client::client $::write_cs
	    }
	    
	    -server {
		puts_info 10 "argv0: $::argv0"
		puts_info 10 "argc: $::argc"
		set cs [ server::initialize $::read_cs ]
		puts_info 10 "cs: $cs"
		
		# Exec client process
		catch { exec $::argv0 $cs &  } client_pid
		
		puts_info 10 "client_pid: $client_pid"
		
		# Continue with the server
		server::server
				
		puts_info 10 "Server is completed"

		server::cleanup
	    }
	}
	
	#----------------------------------------------------------------
	# Do cleanup of globus
	#----------------------------------------------------------------
	globus::cleanup
    } err ] } {
	err_handler $err
    }
    #---------------------------------------------------------------------
    # Exit with normal exit status since test completed without error.
    #---------------------------------------------------------------------
    TestExit
}

if { $::EXEC_MAIN } {
    main
}
