#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}
    
#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Sanity check to verify that the version of Globus being used supports
#   the xio library.
#------------------------------------------------------------------------
if { [ info exists globus_i_xio_module ] == 0 } {
    #--------------------------------------------------------------------
    # No xio extensions as is the case with globus 2.4.x
    #--------------------------------------------------------------------
    TestExit
}
#------------------------------------------------------------------------
# Initialize necessary variables 
#------------------------------------------------------------------------
set SECTION xio_test
validate_section $SECTION [list BUFFERSIZE]
validate_section $SECTION [list WAITFORNBYTES]

set tcpdrivername    tcp
set filedrivername   file

set bufsize          $TestEnv($SECTION,BUFFERSIZE)
set waitforbytes     $TestEnv($SECTION,WAITFORNBYTES)

set read_cs         [ lindex $argv 0 ]
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus XIO module
    globus_module_activate $globus_i_xio_module

    # Load transport driver
    lassign [ globus_xio_driver_load $tcpdrivername ] result tcpdriver
    lassign [ globus_xio_driver_load $filedrivername ] result filedriver

    # Attribute initialization
    lassign [ globus_xio_attr_init ] result attr

    # Stack construction
    lassign [ globus_xio_stack_init NULL ] result filestack
    lassign [ globus_xio_stack_init NULL ] result tcpstack

    # Push drivers onto stack
    globus_xio_stack_push_driver $filestack $filedriver 
    globus_xio_stack_push_driver $tcpstack $tcpdriver 

    # File handle
    lassign [ globus_xio_handle_create $filestack ] result filehandle
    globus_xio_open $filehandle $read_cs NULL

    # Server handle
    lassign [ globus_xio_server_create \
		  $attr \
		  $tcpstack \
		 ] result server
    
    lassign [ globus_xio_server_get_contact_string \
		  $server \
		 ] result server_cs
    
    puts "$server_cs"    
	
    lassign [ globus_xio_server_accept $server ] result tcphandle
    
    globus_xio_open $tcphandle $server_cs NULL

    # Blocking read and write
    set remainingbytestoread [ file size $read_cs ]
    set totalbufsizes [ expr $bufsize ]
    do {
	if { [ catch {
	    # Read from file
	    lassign [ globus_xio_read \
			  $filehandle \
			  $bufsize \
			  $waitforbytes \
			  NULL \
			 ] read_status buffer
	    
	    # Write to network
	    lassign [ globus_xio_write \
			  $tcphandle \
			  $buffer \
			  $waitforbytes \
			  NULL \
			 ] write_status nbytes
	} err ] } {
	    switch -regex -- $err {
		{end of file} { 
		    # Reach end of file and exit from the loop
		    break
		}
	    }
	}
	
	set remainingbytestoread [ expr { $remainingbytestoread - \
					      $totalbufsizes } ]
	
    } until { $remainingbytestoread < 0 }
    
    # Close server
    globus_xio_close $tcphandle NULL
    globus_xio_server_close $server
    
    # Close file handle
    globus_xio_close $filehandle NULL
    
    # Destroy stacks
    globus_xio_stack_destroy $tcpstack
    globus_xio_stack_destroy $filestack
    
    # Destroy attributes
    globus_xio_attr_destroy $attr

    # Unload both drivers
    globus_xio_driver_unload $tcpdriver
    globus_xio_driver_unload $filedriver

    # Deactivate Globus XIO module
    globus_module_deactivate $globus_i_xio_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
