#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
for i in tclsh wish 
# execution after the exec line.\
do
# Execute the script using either tclsh or wish \
   $i "$0"
# end of loop \
done
# end of bourne script \
exit 0

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Sanity check to verify that the version of Globus being used supports
# the xio library.
#------------------------------------------------------------------------
if { [ info exists globus_i_xio_module ] == 0 } {
    #--------------------------------------------------------------------
    # No xio extensions as is the case with globus 2.4.x
    #--------------------------------------------------------------------
    puts_info 10 "No XIO extensions as is the case with globus 2.4.x"
    TestExit
}

#------------------------------------------------------------------------
# Initialize necessary variables 
#------------------------------------------------------------------------
set SECTION xio_test
validate_section $SECTION [list BUFFERSIZE]
validate_section $SECTION [list WAITFORNBYTES]

set driver_name   file
set waitforbytes  $TestEnv($SECTION,WAITFORNBYTES)

set sourcedir  [ file join [ file dirname [ pwd ] ] swig ]
puts_info 10 "sourcedir: $sourcedir"

set localdir [file join /tmp \
		  [file join "$env(USER)" \
		       [constructOutputDirectory $argv0]]]
puts_info 10 "localdir: $localdir"

#------------------------------------------------------------------------
# copy - globus_xio_read() and globus_xio_write()
#------------------------------------------------------------------------
proc copy { read_cs write_cs } {
    if { [ catch {
	set bufsize $::TestEnv($::SECTION,BUFFERSIZE)
	# Initialize read handle
	puts_info 10 "globus_xio_handle_create"
	lassign [ globus_xio_handle_create $::stack ] result rhandle
	puts_info 10 "read handle: $rhandle"
	# Initialize write handle
	puts_info 10 "globus_xio_handle_create"
	lassign [ globus_xio_handle_create $::stack ] result whandle
	puts_info 10 "write handle: $whandle"
	# Open both handles
	puts_info 10 "globus_xio_open"
	globus_xio_open $rhandle $read_cs NULL
	puts_info 10 "globus_xio_open"
	globus_xio_open $whandle $write_cs NULL
	
	set remainingbytestoread [ file size $read_cs ]
	set totalbufsizes [ expr $bufsize ]
	
	puts_info 20 "read_cs: $read_cs"
	puts_info 20 "write_cs: $write_cs"
	
	do {
	    # The actual Globus XIO call to read the data
	    lassign [ globus_xio_read \
			  $rhandle \
			  $bufsize \
			  $::waitforbytes \
			  NULL ] status buffer
	    
	    puts_info 20 "globus_xio_read returns $status"
	    puts_info 20 "bytes read: [ string length $buffer ]"
	    
	    # The actual Globus XIO call to write the data
	    lassign [ globus_xio_write \
			  $whandle \
			  $buffer \
			  $::waitforbytes \
			  NULL ] status nbytes
	    
	    puts_info 20 "globus_xio_write returns $status"
	    puts_info 20 "bytes written: $nbytes"
	    
	    set remainingbytestoread [ expr { $remainingbytestoread - \
						  $totalbufsizes } ]
	} until { $remainingbytestoread < 0 }
	
	#----------------------------------------------------------------
	# Run MD5 checksum before exiting
	#----------------------------------------------------------------
	check_md5sum $read_cs $write_cs
	
	# Close the handle
	puts_info 10 "globus_xio_close"
	globus_xio_close $rhandle NULL
	
	puts_info 10 "globus_xio_close"
	globus_xio_close $whandle NULL

    } err ] } {
	return -code error $err
    }
}
#------------------------------------------------------------------------
# Run test loop
#------------------------------------------------------------------------
proc run_test { } {
    # Create temporary directory
    copy_init $::localdir
    # Get file list
    set in_filelist [ constructFileList ]
    puts_info 10 "Number of files: [ llength $in_filelist ]"
    # Loop over every single file, binary and non-binary
    foreach read_cs $in_filelist {
	if { [file isdirectory $read_cs] } {
	    continue
	}
	set write_cs [ file join $::localdir [ file tail $read_cs ] ]
	copy $read_cs $write_cs
    }
    # Remove temporary directory
    copy_deinit $::localdir
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus XIO module
    puts_info 10 "Activating XIO module"
    globus_module_activate $globus_i_xio_module
    # Load the driver
    puts_info 10 "globus_xio_driver_load"
    lassign [ globus_xio_driver_load $driver_name ] result driver
    # Create the driver stack and push the driver onto the stack
    puts_info 10 "globus_xio_stack_init"
    lassign [ globus_xio_stack_init NULL ] result stack
    puts_info 10 "globus_xio_stack_push_driver"
    globus_xio_stack_push_driver $stack $driver
    # Run the test in infinite loop mode or not
    if { [IsInfinite] == 1 } {
	while 1 {
	    run_test
	}
    } else {
	run_test
    }
    # Destroy the stack
    puts_info 10 "globus_xio_stack_destroy"
    globus_xio_stack_destroy $stack
    # Unload the driver
    puts_info 10 "globus_xio_driver_unload"
    globus_xio_driver_unload $driver
    # Deactivate globus module
    puts_info 10 "Deactivating XIO module"
    globus_module_deactivate $globus_i_xio_module
} err ] } {
    err_handler $err
}

puts_info 10 "Done"

#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
