#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"
puts "env(TOPSRCDIR) $env(TOPSRCDIR)"
puts "env(SRCDIR) $env(SRCDIR)"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  gt_xio_socket_swig } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Initialize the necessary variables 
#------------------------------------------------------------------------
set SECTION xio_test
validate_section $SECTION [list FILENAME SERVER PORT]
set filename $TestEnv($SECTION,FILENAME)
set server $TestEnv($SECTION,SERVER)
set port $TestEnv($SECTION,PORT)

set read_cs [file join [ file join [ file dirname [ pwd ] ] swig ] $filename]
set write_cs [file join /tmp $filename]

set ::SERVICE_NAME 	ldas

#verifyServer $server $port
verifyProxy

#------------------------------------------------------------------------
# Read
#------------------------------------------------------------------------
proc Read { sock } {
    if { [ catch {
	if { [eof $sock] } {
	    puts_info 10 "Closing the socket"
	    close $sock
	}
	close $sock
    } err ] } {
	return -code error $err
    }	
}

#------------------------------------------------------------------------
# Write
#------------------------------------------------------------------------
proc Write { sock } {
    if { [ catch {
	set srcdir [ file join [ file dirname [ pwd ] ] swig ]
	puts_info 10 "srcdir $srcdir"
	set srcfile [ file join $srcdir $::filename ]
	puts_info 10 "srcfile $srcfile"
	set fid [ open $srcfile r ]
	fconfigure $fid -buffering full
	fconfigure $fid -translation binary
	fconfigure $fid -encoding binary
	set data [ read $fid ]
	close $fid
	puts_info 10 "data size [string length $data]" 

	set peerInfo [ fconfigure $sock -peername ]
	puts_info 10 "peerInfo: $peerInfo"
	
	fconfigure $sock -translation binary
	fconfigure $sock -encoding binary
	fconfigure $sock -buffering none
	fconfigure $sock -blocking 0

	#while 1 {
	puts_info 10 "puts the data on the socket"
	puts -nonewline $sock $data
	flush $sock
	#}
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Main program - open/close/read/write routines
#------------------------------------------------------------------------
if { [ catch {
    puts "client pid: [pid]"
    
    # With all options plus GSI authentication
    if { 0 } {
	if [ catch { gt_xio_socket -gsi_auth_enabled -myaddr localhost \
			 -myport 40000 $server $port  } sock ] {
	    puts stderr "Error: $sock"
	    exit 1
	}
    }

    # Without GSI authentication
    if { 0 } {
	if [ catch {gt_xio_socket $server $port} sock ] {
	    puts stderr "Error: $sock"
	    exit 1
	}
    }

    while 1 {
	# With GSI authentication
	if { 1 } {
	    if [ catch { gt_xio_socket -gsi_auth_enabled $server $port } sock ] {
		puts stderr "Error: $sock"
		exit 1
	    }
	}
	puts_info 10 "sock: $sock"
	
	#Write $sock
	
	close $sock
	after 1000
    }
    
    # Without options using service cert by default
    if { 0 } {
	if [ catch {gt_xio_socket -cert_type service -service_name ldas \
			$server $port} sock ] {
	    puts stderr "Error: $sock"
	    exit 1
	}
    }

    # Incorrect options
    if { 0 } {
	if [ catch {gt_xio_socket -myaddr localhost -myport 40000 \
			$server $port} sock ] {
	    puts stderr "Error: $sock"
	    exit 1
	}
    }
    
    #puts_info 10 "sock: $sock"

    #Write $sock
    
} err ] } {
    puts "err: $err"
    err_handler $err
}
