#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  gt_xio_socket_swig } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Initialize the necessary variables 
#------------------------------------------------------------------------
set SECTION xio_test
validate_section $SECTION [list FILENAME PORT]
set filename $TestEnv($SECTION,FILENAME)
set port $TestEnv($SECTION,PORT)

set read_cs [file join [ file join [ file dirname [ pwd ] ] swig ] $filename]
set write_cs [file join /tmp $filename]

set ::X509_CERT_DIR	/ldas_outgoing/grid-security/certificates
set ::X509_USER_KEY	/ldas_outgoing/grid-security/ldaskey.pem
set ::X509_USER_CERT	/ldas_outgoing/grid-security/ldascert.pem


#------------------------------------------------------------------------
# Write
#------------------------------------------------------------------------
proc Write { newSock } {
    if { [ catch {
	set srcdir [ file join [ file dirname [ pwd ] ] swig ]
	puts_info 10 "srcdir $srcdir"
	set srcfile [ file join $srcdir $::filename ]
	puts_info 10 "srcfile $srcfile"
	set fid [ open $srcfile r ]
	fconfigure $fid -translation binary -encoding binary
	set data [ read $fid ]
	close $fid 
	puts_info 10 "data size [string length $data]" 
	
	fconfigure $newSock -translation binary 
	fconfigure $newSock -encoding binary
	fconfigure $newSock -buffering none
	fconfigure $newSock -blocking 0

	if [ catch {puts -nonewline $newSock $data} err ] {
	    puts stderr $err
	    exit 1
	}

	close $newSock
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Read
#------------------------------------------------------------------------
proc Read { newSock } {
    if { [ catch {
	puts_info 10 "Entering Read proc"
	puts_info 10 "newSock $newSock"
	
	if { [eof $newSock] } {
	    puts_info 10 "Closing the socket"
	    close $newSock
	} else {
	    fconfigure $newSock -translation binary -encoding binary
	    fconfigure $newSock -buffering full 
	    fconfigure $newSock -blocking 0
	    #fconfigure $newSock -buffersize 32768

	    set peerInfo [ fconfigure $newSock -peername ]
	    puts_info 10 "peerInfo $peerInfo"

	    if { [ catch {
		set peer_x509_subj_name [ fconfigure $newSock -x509_subj_name ]
		puts_info 10 "peer_x509_subj_name $peer_x509_subj_name"
	    } err ] } {
		puts_info 10 "x509_subj_name not supported: $err"
	    }

	    set data [read $newSock]
	    
	    puts_info 10 "data size [string length $data]"
	    
	    # Write the data to a file
	    puts -nonewline $::fid $data
	    flush $::fid
	}
	
	puts_info 10 "Leaving Read proc"
    } err ] } {
	close $newSock
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Close
#------------------------------------------------------------------------
proc Close { newSock } {
    if { [ catch {
	puts_info 10 "Entering Close proc"
	puts_info 10 "newSock $newSock"
	
	puts_info 10 "Closing the socket"
	close $newSock
    } err ] } {
	return -code error $err
    }
}

#------------------------------------------------------------------------
# Accept
#------------------------------------------------------------------------
proc Accept { newSock addr port } {
    if { [ catch {
	#puts_info 10 "newSock $newSock addr $addr port $port"

	puts_info 10 "Accepted newSock $newSock from addr $addr port $port"
	
	
	fileevent $newSock readable [list Close $newSock]
	#close $newSock

	if { 0 } {
	    set ::newdir [ file join /tmp/msamidi $newSock ]
	    puts_info 10 "newdir $::newdir"
	    
	    if { ![file exists $::newdir] } {
		file mkdir $::newdir
	    }
	    
	    puts "Accepted newSock $newSock from addr $addr port $port"
	    
	    set ::fid [ open [file join $::newdir $::filename] w ]
	    puts_info 10  "fid: $::fid"
	    fconfigure $::fid -translation binary -encoding binary
	    
	    fileevent $newSock readable [list Read $newSock]
	}

    } err ] } {
	puts "err: $err"
	return -code error $err
    }	
}

#------------------------------------------------------------------------
# Main program
#------------------------------------------------------------------------
if { [ catch {
    puts_info 10 "server pid: [pid]"

    puts_info 10 "port $port"
    
    # Without GSI authentication
    if { 0 } {
	# GT XIO server is opened for business
	set listenSocket [ gt_xio_socket -server Accept -myaddr localhost \
			       $port ]
    }

    # With GSI authentication
    if { 1 } {
	# GT XIO server is opened for business
	set listenSocket [ gt_xio_socket -server Accept \
			       -gsi_auth_enabled -myaddr localhost $port ]
    }

    vwait forever
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
