#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
exec tclsh "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}

#------------------------------------------------------------------------
# Sanity check to verify that the version of Globus being used supports
# the xio library.
#------------------------------------------------------------------------
if { [ info exists globus_i_xio_module ] == 0 } {
    #--------------------------------------------------------------------
    # No xio extensions as is the case with globus 2.4.x
    #--------------------------------------------------------------------
    puts_info 10 "No XIO extensions as is the case with globus 2.4.x"
    TestExit
}

#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus XIO module
    puts_info 10 "Activating XIO module"
    globus_module_activate $globus_i_xio_module

    # Load the driver
    puts_info 10 "globus_xio_driver_load"
    lassign [ globus_xio_driver_load tcp ] result tcpdriver

    puts_info 10 "globus_xio_attr_init"
    lassign [ globus_xio_attr_init ] result attr
    
    # Create XIO stack and push the drivers onto the stack
    puts_info 10 "globus_xio_stack_init"
    lassign [ globus_xio_stack_init NULL ] result stack
    
    puts_info 10 "globus_xio_stack_push_driver"
    lassign [ globus_xio_stack_push_driver \
		  $stack \
		  $tcpdriver \
		 ] result
    
    puts_info 10 "GLOBUS_XIO_TCP_SET_SERVICE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_SERVICE \
		  "ldas"
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_SERVICE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_SERVICE \
	     ] result service_name
    puts_info 10 "service_name $service_name"

    puts_info 10 "GLOBUS_XIO_TCP_SET_PORT"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_PORT \
		  35000
	     ] result
    
    puts_info 10 "GLOBUS_XIO_TCP_GET_PORT"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_PORT \
	     ] result port
    puts_info 10 "port $port"

    puts_info 10 "GLOBUS_XIO_TCP_SET_BACKLOG"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_BACKLOG \
		  15
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_BACKLOG"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_BACKLOG \
	     ] result listener_backlog
    puts_info 10 "listener_backlog $listener_backlog"

    puts_info 10 "GLOBUS_XIO_TCP_SET_LISTEN_RANGE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_LISTEN_RANGE \
		  35000 \
		  36000
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_LISTEN_RANGE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_LISTEN_RANGE \
	     ] result listener_min_port listener_max_port
    puts_info 10 "listener_min_port $listener_min_port"
    puts_info 10 "listener_max_port $listener_max_port"

    puts_info 10 "GLOBUS_XIO_TCP_SET_HANDLE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_HANDLE \
		  10
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_HANDLE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_HANDLE \
		 ] result handle
    puts_info 10 "handle $handle"

    puts_info 10 "GLOBUS_XIO_TCP_SET_INTERFACE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_INTERFACE \
		  eth0
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_INTERFACE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_INTERFACE \
		 ] result interface
    puts_info 10 "interface $interface"

    puts_info 10 "GLOBUS_XIO_TCP_SET_RESTRICT_PORT"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_RESTRICT_PORT \
		  $GLOBUS_TRUE
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_RESTRICT_PORT"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_RESTRICT_PORT \
	     ] result restrict_port
    puts_info 10 "restrict_port $restrict_port"

    puts_info 10 "GLOBUS_XIO_TCP_SET_REUSEADDR"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_REUSEADDR \
		  $GLOBUS_TRUE
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_REUSEADDR"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_REUSEADDR \
	     ] result reuseaddr
    puts_info 10 "reuseaddr $reuseaddr"

    puts_info 10 "GLOBUS_XIO_TCP_SET_NO_IPV6"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_NO_IPV6 \
		  $GLOBUS_TRUE
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_NO_IPV6"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_NO_IPV6 \
	     ] result no_ipv6
    puts_info 10 "no_ipv6 $no_ipv6"

    puts_info 10 "GLOBUS_XIO_TCP_SET_CONNECT_RANGE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_CONNECT_RANGE \
		  35000 \
		  36000
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_CONNECT_RANGE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_CONNECT_RANGE \
	     ] result connect_min_port connect_max_port
    puts_info 10 "connect_min_port $connect_min_port"
    puts_info 10 "connect_max_port $connect_max_port"

    puts_info 10 "GLOBUS_XIO_TCP_SET_KEEPALIVE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_KEEPALIVE \
		  $GLOBUS_TRUE
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_KEEPALIVE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_KEEPALIVE \
	     ] result keepalive
    puts_info 10 "keepalive $keepalive"

    puts_info 10 "GLOBUS_XIO_TCP_SET_LINGER"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_LINGER \
		  $GLOBUS_TRUE \
		  15
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_LINGER"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_LINGER \
	     ] result linger linger_time
    puts_info 10 "linger $linger"
    puts_info 10 "linger_time $linger_time"

    puts_info 10 "GLOBUS_XIO_TCP_SET_OOBINLINE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_OOBINLINE \
		  $GLOBUS_TRUE \
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_OOBINLINE"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_OOBINLINE \
	     ] result oobinline
    puts_info 10 "oobinline $oobinline"

    puts_info 10 "GLOBUS_XIO_TCP_SET_SNDBUF"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_SNDBUF \
		  1024 \
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_SNDBUF"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_SNDBUF \
	     ] result sndbuf
    puts_info 10 "sndbuf $sndbuf"

    puts_info 10 "GLOBUS_XIO_TCP_SET_RCVBUF"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_RCVBUF \
		  1024 \
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_RCVBUF"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_RCVBUF \
	     ] result rcvbuf
    puts_info 10 "rcvbuf $rcvbuf"

    puts_info 10 "GLOBUS_XIO_TCP_SET_NODELAY"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_SET_NODELAY \
		  $GLOBUS_TRUE \
	     ] result

    puts_info 10 "GLOBUS_XIO_TCP_GET_NODELAY"
    lassign [ globus_xio_attr_cntl \
		  $attr \
		  $tcpdriver \
		  $GLOBUS_XIO_TCP_GET_NODELAY \
	     ] result nodelay
    puts_info 10 "nodelay $nodelay"
















    
    # Destroy the stack
    puts_info 10 "globus_xio_stack_destroy"
    lassign [ globus_xio_stack_destroy \
		  $stack \
		 ] result

    # Destroy both attributes
    puts_info 10 "globus_xio_attr_destroy"
    lassign [ globus_xio_attr_destroy \
		  $attr \
		 ] result
    
    # Unload the driver
    lassign [ globus_xio_driver_unload \
		  $tcpdriver \
		 ] result
    
    # Deactivate globus module
    puts_info 10 "Deactivating XIO module"
    globus_module_deactivate $globus_i_xio_module
} err ] } {
    err_handler $err
}

puts_info 10 "Done"

#------------------------------------------------------------------------
# Exit with normal exit status since test completed without error.
#------------------------------------------------------------------------
TestExit
