#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
case x$PROGRAM in
# ... and otherwise use tclsh. \
*) PROGRAM="tclsh";;
# End of bourne script \
esac
# execution after the exec line. \
exec $PROGRAM "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Sanity check to verify that the version of Globus being used supports
#   the xio library.
#------------------------------------------------------------------------
if { [ info exists globus_i_xio_module ] == 0 } {
    #--------------------------------------------------------------------
    # No xio extensions as is the case with globus 2.4.x
    #--------------------------------------------------------------------
    TestExit
}
#------------------------------------------------------------------------
# Initialize necessary variables 
#------------------------------------------------------------------------
set SECTION xio_test
validate_section $SECTION [list BUFFERSIZE]
validate_section $SECTION [list WAITFORNBYTES]

set tcpdrivername    tcp
set filedrivername   file

set bufsize         $TestEnv($SECTION,BUFFERSIZE)
set waitforbytes    $TestEnv($SECTION,WAITFORNBYTES)

set contact_string  [ lindex $argv 0 ]
set write_cs	    [ lindex $argv 1 ]
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus XIO module
    globus_module_activate $globus_i_xio_module

    # Load transport and transform driver
    lassign [ globus_xio_driver_load $filedrivername ] result filedriver
    lassign [ globus_xio_driver_load $tcpdrivername ] result tcpdriver

    # Stack construction
    lassign [ globus_xio_stack_init NULL ] result filestack
    lassign [ globus_xio_stack_init NULL ] result tcpstack
    
    # Push drivers onto stack
    globus_xio_stack_push_driver $filestack $filedriver
    globus_xio_stack_push_driver $tcpstack $tcpdriver

    # File handle construction
    lassign [ globus_xio_handle_create $filestack ] result filehandle	
    # Network handle construction
    lassign [ globus_xio_handle_create $tcpstack ] result tcphandle

    # Open blocking handle
    globus_xio_open $filehandle $write_cs NULL 
    globus_xio_open $tcphandle $contact_string NULL

    # Blocking read and write
    set nwritten 0
    do {
	if { [ catch {
	    # Read from network
	    lassign [ globus_xio_read \
			  $tcphandle \
			  $bufsize \
			  $waitforbytes \
			  NULL \
			 ] status buffer
	    
	    # Write to file
	    lassign [ globus_xio_write \
			  $filehandle \
			  $buffer \
			  $waitforbytes \
			  NULL \
			 ] status nwritten
	} err ] } {
	    switch -regex -- $err {
		{end of file} { 
		    break
		}
	    }
	}
    } until { $nwritten == 0 }

    # Close both handles 
    globus_xio_close $tcphandle NULL 
    globus_xio_close $filehandle NULL
    
    # Destroy both stacks
    globus_xio_stack_destroy $tcpstack
    globus_xio_stack_destroy $filestack

    # Unload both drivers
    globus_xio_driver_unload $tcpdriver
    globus_xio_driver_unload $filedriver

    # Deactivate Globus XIO module
    globus_module_deactivate $globus_i_xio_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with 0 if no tests failed, 1 otherwise
#------------------------------------------------------------------------
TestExit
