#!/bin/sh
# ALL the continued lines following this one are interpreted
# by the bourne shell and ignored by Tcl, which picks up
# execution after the exec line.\
case x$PROGRAM in
# ... and otherwise use tclsh. \
*) PROGRAM="tclsh";;
# End of bourne script \
esac
# execution after the exec line. \
exec $PROGRAM "$0" ${1+"$@"}

#------------------------------------------------------------------------
# Initialize the test
#------------------------------------------------------------------------
source "$env(SRCDIR)/test_utils.tcl"

foreach lib { globus_module \
		  globus_error \
		  globus_object \
		  globus_xio } {
    load [ eval file join $topbuilddir "swig/.libs/libtcl${lib}.so" ]
}
#------------------------------------------------------------------------
# Sanity check to verify that the version of Globus being used supports
#   the xio library.
#------------------------------------------------------------------------
if { [ info exists globus_i_xio_module ] == 0 } {
    #--------------------------------------------------------------------
    # No xio extensions as is the case with globus 2.4.x
    #--------------------------------------------------------------------
    TestExit
}
#------------------------------------------------------------------------
# Initialize necessary variables 
#------------------------------------------------------------------------
set SECTION xio_test
validate_section $SECTION [list BUFFERSIZE]
validate_section $SECTION [list WAITFORNBYTES]

set tcpdrivername   tcp
set filedrivername  file

set bufsize         $TestEnv($SECTION,BUFFERSIZE)
set waitforbytes    $TestEnv($SECTION,WAITFORNBYTES)

set read_cs         [ lindex $argv 0 ]
#------------------------------------------------------------------------
# Actual demo
#------------------------------------------------------------------------
if { [ catch {
    # Activate Globus XIO module
    globus_module_activate $globus_i_xio_module

    # Load transport and transform driver
    lassign [ globus_xio_driver_load $tcpdrivername ] result tcpdriver
    lassign [ globus_xio_driver_load $filedrivername ] result filedriver

    # Stack construction
    lassign [ globus_xio_stack_init NULL ] result tcpstack
    lassign [ globus_xio_stack_init NULL ] result filestack

    # Push drivers onto stack
    globus_xio_stack_push_driver $filestack $filedriver
    globus_xio_stack_push_driver $tcpstack $tcpdriver

    # File handle construction
    lassign [ globus_xio_handle_create $filestack ] result filehandle
    # Network handle construction
    lassign [ globus_xio_server_create NULL $tcpstack ] result server
    # Get server contact string
    lassign [ globus_xio_server_get_contact_string $server ] result server_cs
    puts "$server_cs"    

    lassign [ globus_xio_server_accept $server ] result tcphandle

    # TCP driver specific attribute initialization
    lassign [ globus_xio_attr_init ] result tcpattr 

    if { 0 } {
	# TCP driver specific attribute settings
	set tcp_sendbuffer 32768
	globus_xio_attr_cntl \
	    $tcpattr \
	    $tcpdriver \
	    $GLOBUS_XIO_TCP_SET_SNDBUF \
	    $tcp_sendbuffer 
	
	set tcp_receivebuffer 32768
	globus_xio_attr_cntl \
	    $tcpattr \
	    $tcpdriver \
	    $GLOBUS_XIO_TCP_SET_RCVBUF \
	    $tcp_receivebuffer

	lassign [ globus_xio_attr_cntl \
		      $tcpattr \
		      $tcpdriver \
		      $GLOBUS_XIO_TCP_GET_SNDBUF \
		     ] result send_buffer
	
	lassign [ globus_xio_attr_cntl \
		      $tcpattr \
		      $tcpdriver \
		      $GLOBUS_XIO_TCP_GET_RCVBUF \
		     ] result recv_buffer
	
	globus_xio_attr_cntl \
	    $tcpattr \
	    $tcpdriver \
	    $GLOBUS_XIO_TCP_SET_NODELAY \
	    $GLOBUS_TRUE
	
	globus_xio_attr_cntl \
	    $tcpattr \
	    $tcpdriver \
	    $GLOBUS_XIO_TCP_SET_KEEPALIVE \
	    $GLOBUS_TRUE
    }
	
    # Open blocking handle
    globus_xio_open $filehandle $read_cs NULL
    globus_xio_open $tcphandle $server_cs $tcpattr

    # Blocking read and write
    set remainingbytestoread [ file size $read_cs ]
    set totalbufsizes [ expr $bufsize ]
    do {
	if { [ catch {
	    # Read from file
	    lassign [ globus_xio_read \
			  $filehandle \
			  $bufsize \
			  $waitforbytes \
			  NULL \
			 ] read_status buffer
	    # Write to network
	    lassign [ globus_xio_write \
			  $tcphandle \
			  $buffer \
			  $waitforbytes \
			  NULL \
			 ] write_status nbytes
	} err ] } {
	    switch -regex -- $err {
		{end of file} { 
		    # Ignore this error
		}
	    }
	}
	
	set remainingbytestoread [ expr { $remainingbytestoread - \
					      $totalbufsizes } ]
    } until { $remainingbytestoread < 0 }
    
    # Close TCP handle and server
    globus_xio_close $tcphandle $tcpattr
    globus_xio_server_close $server
    
    # Close file handle
    globus_xio_close $filehandle NULL
    
    # Destroy stacks
    globus_xio_stack_destroy $filestack
    globus_xio_stack_destroy $tcpstack
    
    # Destroy attributes
    globus_xio_attr_destroy $tcpattr
    
    # Unload both drivers
    globus_xio_driver_unload $filedriver
    globus_xio_driver_unload $tcpdriver
    
    # Deactivate Globus XIO module
    globus_module_deactivate $globus_i_xio_module
} err ] } {
    err_handler $err
}
#------------------------------------------------------------------------
# Exit with 0 if no tests failed, 1 otherwise
#------------------------------------------------------------------------
TestExit
