/* -*- mode: c++; c-basic-offset: 2; -*- */


#ifdef SWIG

%include "../../config.h"

/*
 * Unsupported functions
 */
%ignore globus_ftp_client_operationattr_set_rfc1738_url;
%ignore globus_ftp_client_operationattr_get_rfc1738_url;

/*
 * Internal use
 */
%ignore globus_ftp_client_features_init;
%ignore globus_ftp_client_features_destroy;
%ignore globus_ftp_client_feat;
%ignore globus_ftp_client_is_feature_supported;


/*****************************************************************
 * Hand-wrapped SWIG functions
 *****************************************************************/

%ignore globus_ftp_client_size;
%ignore globus_ftp_client_cksm;
%ignore globus_ftp_client_modification_time;

%ignore globus_ftp_client_operationattr_get_parallelism;
%ignore globus_ftp_client_operationattr_get_layout;
%ignore globus_ftp_client_operationattr_get_tcp_buffer;
%ignore globus_ftp_client_operationattr_get_dcau;

%ignore globus_ftp_client_register_read;
%ignore globus_ftp_client_register_write;

%include "tcl-typemaps.i"

%include "swig/globus_ftp_client.i"

/*
 * To avoid conflict between globus_ftp_client_restart_marker_get_total
 * and globus_ftp_client_size
 */
%clear globus_off_t*;

%{

#include <stdio.h>
#include <tcl.h>
#include "tcl-callback.h"
#include "common/sgdebug.h"
#include "common/sginit.h"
  
#define MAX_UNIQUE_NAME		64

#define MAX_CKSM_STRING		256  
  
  /* Callback-related typedef */
  typedef struct callbackData_ {
    long           total_bytes;
    int            iovec_cnt;
    void*          buffer;
    char           unique_name[MAX_UNIQUE_NAME];
  } callbackData;
  
  /**************************************************************************
   * CALLBACK FOR FTP CLIENT OPERATIONS
   **************************************************************************/
  static void
    FTPClientCompleteCallback
    ( void* 				user_data, 
      globus_ftp_client_handle_t* 	handle,
      globus_object_t*			error )
    {
      char 	str_handle[128];
      char*	msg;

      if ( error )
        {
          msg = globus_object_printable_to_string( error );
        }
      else
        {
          msg = "";
        }
      
      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_client_handle_t" ), 0 );

      SGDEBUG_LVL_FMT_1( 10, "FTPClientCompleteCallback: msg: %s", msg );
      SGDEBUG_LVL_FMT_1( 10, "FTPClientCompleteCallback: str_handle = %s", 
                     str_handle );
      
      CallCallback( (callbackStruct*)user_data,
                    "%s \"%s\"",
                    str_handle, msg );

      if ( error )
      {
	globus_libc_free(msg);
      }
    }


 /**************************************************************************
   *
   * GLOBUS_FTP_CLIENT_OPERATIONATTR_SET_READ_ALL()
   * GLOBUS_FTP_CLIENT_OPERATIONATTR_GET_READ_ALL()
   *
   **************************************************************************/

  static void
    FTPClientDataCallback( void* 			user_data,
                           globus_ftp_client_handle_t* handle,
                           globus_object_t* 		error,
                           globus_byte_t* 		buffer,
                           globus_size_t 		length,
                           globus_off_t 		offset,
                           globus_bool_t 		eof )
    {
      char str_handle[128];
      char str_buffer[128];
      char*	msg;

      if ( error )
        {
          msg = globus_object_printable_to_string( error );
        }
      else
        {
          msg = "";
        }

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_client_handle_t" ), 0 );
      
      SWIG_MakePtr( str_buffer, buffer,
                    SWIG_TypeQuery( "globus_byte_t" ), 0 );

            
      CallCallback( (callbackStruct*)user_data,
                    "%s \"%s\" %s %d %d %d",
                    str_handle,
                    msg,
                    str_buffer,
                    length,
                    offset,
                    eof );

      if ( error )
      {
	globus_libc_free(msg);
      }
    }

  
  /**************************************************************************
   *
   * GLOBUS_FTP_CLIENT_SIZE()
   *
   **************************************************************************/

  static void
    GlobusFTPClientSizeInit( callbackStruct* cb )
    {
      globus_off_t* filesize =
        (globus_off_t*)globus_libc_malloc( sizeof(globus_off_t) );
      
      cb->s_api_data = filesize;
    }


  static int
    GlobusFTPClientSizeCleanup( callbackStruct* cb )
    {
      globus_libc_free( cb->s_api_data );

      cb->s_api_data = NULL;
      return 1;
    }


  static void
    TclFTPClientSizeCallback( void* user_data, 
                              globus_ftp_client_handle_t * handle,
                              globus_object_t* error )
    {
      char str_handle[128];
      char*	msg;

      if ( error )
        {
          msg = globus_object_printable_to_string( error );
        }
      else
        {
          msg = "";
        }

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_client_handle_t" ), 0 );
      
      SGDEBUG_LVL_FMT_1( 10, "TclFTPClientSizeCallback: error msg: %s", msg );
      SGDEBUG_LVL_FMT_1( 10, "TclFTPClientSizeCallback: filesize: %d",
                     *((globus_off_t*)((callbackStruct*)user_data)->s_api_data) );
      
      CallCallback( (callbackStruct*)user_data,
                    "%s \"%s\" %d",
                    str_handle,
                    msg,
                    *((globus_off_t*)((callbackStruct*)user_data)->s_api_data));
      if ( error )
      {
	globus_libc_free(msg);
      }
    }


  /**************************************************************************
   *
   * GLOBUS_FTP_CLIENT_CKSM()
   *
   **************************************************************************/

  static void
    GlobusFTPClientCksmInit( callbackStruct* cb )
    {
      char * cksm =
        (char *)globus_libc_malloc( sizeof(char) * MAX_CKSM_STRING);

      memset(cksm, '\0', MAX_CKSM_STRING);
      
      cb->s_api_data = cksm;
    }
  
  static int
    GlobusFTPClientCksmCleanup( callbackStruct* cb )
    {
      globus_libc_free( cb->s_api_data );

      cb->s_api_data = NULL;
      return 1;
    }


  static void
    TclFTPClientCksmCallback( void* user_data, 
                              globus_ftp_client_handle_t* handle,
                              globus_object_t* error )
    {
      char str_handle[128];
      char*	msg;

      if ( error )
        {
          msg = globus_object_printable_to_string( error );
        }
      else
        {
          msg = "";
        }

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_client_handle_t" ), 0 );

  
      SGDEBUG_LVL_FMT_1( 10, "TclFTPClientCksmCallback: msg: %s", msg );
      SGDEBUG_LVL_FMT_1( 10, "TclFTPClientCksmCallback: cksm: %s",
               ((callbackStruct*)user_data)->s_api_data );
      
      CallCallback( (callbackStruct*)user_data,
                    "%s \"%s\" %s",
                    str_handle,
                    msg,
                    ((callbackStruct*)user_data)->s_api_data );

      if ( error )
      {
	globus_libc_free(msg);
      }
    }

  
  /**************************************************************************
   *
   * GLOBUS_FTP_CLIENT_MODIFICATION_TIME()
   *
   **************************************************************************/

  static void
    GlobusFTPClientMdtmInit( callbackStruct* cb )
    {
      globus_abstime_t * mdtm =
        (globus_abstime_t *)globus_libc_malloc( sizeof(globus_abstime_t) );

      cb->s_api_data = mdtm;
    }
  
  static int
    GlobusFTPClientMdtmCleanup( callbackStruct* cb )
    {
      globus_libc_free( cb->s_api_data );

      cb->s_api_data = NULL;
      return 1;
    }

  
  static void
    TclFTPClientMdtmCallback( void* user_data, 
                              globus_ftp_client_handle_t* handle,
                              globus_object_t* error )
    {
      char str_handle[128];
      char*	msg;

      if ( error )
        {
          msg = globus_object_printable_to_string( error );
	  
        }
      else
        {
          msg = "";
        }

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_client_handle_t" ), 0 );
      

      SGDEBUG_LVL_FMT_1( 10, "TclFTPClientMdtmCallback: msg: %s", msg );
      SGDEBUG_LVL_FMT_1( 10, "TclFTPClientMdtmCallback: mdtm->sec: %ld",
                     ((globus_abstime_t *)((callbackStruct*)user_data)->s_api_data)->tv_sec );
      SGDEBUG_LVL_FMT_1( 10, "TclFTPClientMdtmCallback: mdtm->nsec: %ld\n",
                     ((globus_abstime_t *)((callbackStruct*)user_data)->s_api_data)->tv_nsec );
      
      CallCallback( (callbackStruct*)user_data,
                    "%s \"%s\" %ld %ld",
                    str_handle,
                    msg,
                    ((globus_abstime_t *)((callbackStruct*)user_data)->s_api_data)->tv_sec,
                    ((globus_abstime_t *)((callbackStruct*)user_data)->s_api_data)->tv_nsec );

      if ( error )
      {	
	globus_libc_free(msg);
      }
    }


  /**************************************************************************
   *
   * GLOBUS_FTP_CLIENT_REGISTER_READ()
   *
   **************************************************************************/
  
  static void
    GlobusFTPClientRegisterReadInit( callbackStruct* cb, long buffer_length )
    {
      /* Allocate memory space for our buffer */
      callbackData* data = 
        (callbackData *)globus_libc_malloc( sizeof(callbackData) );
      
      cb->s_api_data = data;

      data->total_bytes = buffer_length; 
      
      /* :TODO: We should not exit if we fail to allocate our buffer */
      /* :TODO: Must return error message instead of just exit */
      if ( ( data->buffer = 
             (globus_byte_t *)globus_libc_malloc( sizeof(globus_byte_t) * 
                                                  buffer_length )) == NULL ) 
        {
          fprintf(stderr, "Memory allocation fails\n");
          exit(1);
        }
      
      memset( data->buffer, '\0', buffer_length );
                                                                               
      TclGlobusCreateUniqueTCLVariableName( data->unique_name );

      SGDEBUG_LVL_FMT_1( 20, "read callback allocation: %p", data );  
      SGDEBUG_LVL_FMT_1( 20, "read buffer allocation: %p", data->buffer );  
      SGDEBUG_LVL_FMT( 10, "read Tcl_SetVar2" );
    }

  
  static int
    GlobusFTPClientRegisterReadCleanup( callbackStruct* cb )
    {
      callbackData* data = (callbackData*)cb->s_api_data;

      SGDEBUG_LVL_FMT( 10, "read Tcl_UnsetVar2" );
    
      Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);

      /* Deallocate memory space */
      if ( data->buffer ) 
        {
          SGDEBUG_LVL_FMT_1
            ( 20, "read buffer deallocation: %p", data->buffer ); 
          globus_libc_free( data->buffer );
        }

      if ( cb->s_api_data ) 
        {
          SGDEBUG_LVL_FMT_1
            ( 20, "read callback deallocation: %p", cb->s_api_data );  
          globus_libc_free(cb->s_api_data);
          cb->s_api_data = NULL;
        }
      return 1;
    }


  static void
    GlobusFTPClientRegisterReadPreCallback( callbackStruct* cb )
    {
      Tcl_Interp* interp;
      Tcl_Obj* obj;
      
      callbackData* data = (callbackData*)(cb->s_api_data);
      
      interp = cb->s_main_tcl_interp;

      obj = Tcl_NewByteArrayObj((char *)(data->buffer), data->total_bytes);
          
      Tcl_SetVar2Ex(interp, data->unique_name, NULL, obj, 0);
    }

  
  static void
    TclFTPClientRRDataCallback( void* user_data,
                                globus_ftp_client_handle_t* handle,
                                globus_object_t* error,
                                globus_byte_t* buffer,
                                globus_size_t length,
                                globus_off_t offset,
                                globus_bool_t eof )
    {
      char str_handle[128];
      char*	msg;

      if ( error )
        {
          msg = globus_object_printable_to_string( error );
        }
      else
        {
          msg = "";
        }
      
      SWIG_MakePtr( str_handle, handle,
                  SWIG_TypeQuery( "globus_ftp_client_handle_t" ), 0 );
      
      callbackData*
        data = (callbackData*)((callbackStruct*)user_data)->s_api_data;
      
      data->total_bytes = length;
      
      CallCallback( (callbackStruct*)user_data,
                    "%s \"%s\" $%s %d %d %d",
                    str_handle,
                    msg,
                    data->unique_name,
                    data->total_bytes,
                    offset,
                    eof );
      
      if ( error )
      {
	globus_libc_free(msg);
      }
    }

  /**************************************************************************
   *
   * GLOBUS_FTP_CLIENT_REGISTER_WRITE()
   *
   **************************************************************************/

  static void
  GlobusFTPClientRegisterWriteInit( callbackStruct* cb, unsigned char* buffer )
  {
    /* Allocate memory space */
    callbackData* data =
      (callbackData *)globus_libc_malloc( sizeof(callbackData) );
    
    SGDEBUG_LVL_FMT_1( 20, "write callback allocation: %p", data );

    cb->s_api_data = data;
    data->buffer = buffer;

    TclGlobusCreateUniqueTCLVariableName( data->unique_name );
  }


  static int
  GlobusFTPClientRegisterWriteCleanup( callbackStruct* cb )
  {
    callbackData* data = (callbackData*)(cb->s_api_data);

    /* Release TCL resources */
    Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);
 
    /* Deallocate memory space */
    if ( data->buffer ) 
      {
        SGDEBUG_LVL_FMT_1
          ( 20, "write buffer deallocation: %p", data->buffer );  
        globus_libc_free( data->buffer );
      }

    /* Deallocate memory space */
    if ( cb->s_api_data ) 
    {
      SGDEBUG_LVL_FMT_1
        ( 20, "write callback deallocation: %p", cb->s_api_data );
      globus_libc_free(cb->s_api_data);
      cb->s_api_data = NULL;
    }
    return 1;
  }


  static void
  GlobusFTPClientRegisterWritePreCallback( callbackStruct* cb )
  {
    Tcl_Interp* interp;
    Tcl_Obj* obj;
    callbackData* data = (callbackData*)(cb->s_api_data);

    obj = Tcl_NewByteArrayObj( (char *)(data->buffer), data->total_bytes );
  
    interp = cb->s_main_tcl_interp;
    Tcl_SetVar2Ex(interp, data->unique_name, NULL, obj, 0);
  }


  static void
    TclFTPClientRWDataCallback( void* user_data,
                                globus_ftp_client_handle_t* handle,
                                globus_object_t* error,
                                globus_byte_t* buffer,
                                globus_size_t length,
                                globus_off_t offset,
                                globus_bool_t eof )
    {
      char str_handle[128];
      char*	msg;

      if ( error )
        {
          msg = globus_object_printable_to_string( error );
        }
      else
        {
          msg = "";
        }
      
      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_ftp_client_handle_t" ), 0 );

      callbackData*
        data = (callbackData*)((callbackStruct*)user_data)->s_api_data;
      
      data->total_bytes = length;

      data->buffer = buffer;
      
      CallCallback( (callbackStruct*)user_data,
                    "%s \"%s\" $%s %d %d %d",
                    str_handle,
                    msg,
                    data->unique_name,
                    data->total_bytes,
                    offset,
                    eof );

      if ( error )
      {
	globus_libc_free(msg);
      }
    }

  
  %}

/**************************************************************************
 *
 *  SWIG INITIALIZATION STUFF
 *
 **************************************************************************/
%init %{

  SGINIT_X509;

  %}

/**************************************************************************
 **************************************************************************
 *
 *				TYPEMAPS SECTION
 *
 **************************************************************************
 **************************************************************************/

/**************************************************************************
 *
 *  GLOBUS_FTP_CLIENT_OPERATIONATTR_GET_PARALLELISM
 *
 **************************************************************************/

%typemap(in,numinputs=0)
  globus_ftp_control_parallelism_t * out_parallelism ($*1_ltype temp) 
{
  $1 = (globus_ftp_control_parallelism_t *)&temp;
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_parallelism_t * out_parallelism
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_client_operationattr_get_parallelism) tcl_globus_ftp_client_operationattr_get_parallelism;

%exception tcl_globus_ftp_client_operationattr_get_parallelism GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_client_operationattr_get_parallelism
    ( const globus_ftp_client_operationattr_t *		attr,
      globus_ftp_control_parallelism_t *		out_parallelism )
    {
      globus_result_t retval;

      retval = globus_ftp_client_operationattr_get_parallelism
        ( attr,
          out_parallelism );
      
      return retval;
    }
  
  %}



/**************************************************************************
 *
 *  GLOBUS_FTP_CLIENT_OPERATIONATTR_GET_LAYOUT
 *
 **************************************************************************/


%typemap(in,numinputs=0)
  globus_ftp_control_layout_t * out_layout ($*1_ltype temp) 
{
  $1 = (globus_ftp_control_layout_t *)&temp;
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_layout_t * out_layout 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_client_operationattr_get_layout) tcl_globus_ftp_client_operationattr_get_layout;

%exception tcl_globus_ftp_client_operationattr_get_layout GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_client_operationattr_get_layout
    ( const globus_ftp_client_operationattr_t *		attr,
      globus_ftp_control_layout_t *			out_layout )
    {
      globus_result_t retval;

      retval = globus_ftp_client_operationattr_get_layout
        ( attr,
          out_layout );
      
      return retval;
    }
  
  %}


/**************************************************************************
 *
 *  GLOBUS_FTP_CLIENT_OPERATIONATTR_GET_TCP_BUFFER
 *
 **************************************************************************/

%typemap(in,numinputs=0)
  globus_ftp_control_tcpbuffer_t * out_tcp_buffer($*1_ltype temp) 
{
  $1 = (globus_ftp_control_tcpbuffer_t *)&temp;
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_tcpbuffer_t * out_tcp_buffer 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_client_operationattr_get_tcp_buffer) tcl_globus_ftp_client_operationattr_get_tcp_buffer;

%exception tcl_globus_ftp_client_operationattr_get_tcp_buffer GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_client_operationattr_get_tcp_buffer
    ( const globus_ftp_client_operationattr_t *		attr,
      globus_ftp_control_tcpbuffer_t *			out_tcp_buffer )
    {
      globus_result_t retval;

      retval = globus_ftp_client_operationattr_get_tcp_buffer
        ( attr,
          out_tcp_buffer );
      
      return retval;
    }
  
  %}


/**************************************************************************
 *
 *  GLOBUS_FTP_CLIENT_OPERATIONATTR_GET_DCAU
 *
 **************************************************************************/

%typemap(in,numinputs=0) globus_ftp_control_dcau_t * out_dcau($*1_ltype temp) 
{
  $1 = (globus_ftp_control_dcau_t *)&temp;
}

%typemap(argout,fragment="t_output_helper")
  globus_ftp_control_dcau_t * out_dcau 
{
  Tcl_Obj* o = SWIG_NewPointerObj((void *)($1), $1_descriptor, 0);
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%rename(globus_ftp_client_operationattr_get_dcau) tcl_globus_ftp_client_operationattr_get_dcau;

%exception tcl_globus_ftp_client_operationattr_get_dcau GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_client_operationattr_get_dcau
    ( const globus_ftp_client_operationattr_t *		attr,
      globus_ftp_control_dcau_t *			out_dcau )
    {
      globus_result_t retval;

      retval = globus_ftp_client_operationattr_get_dcau
        ( attr,
          out_dcau );
      
      return retval;
    }
  
  %}



/**************************************************************************
 *
 *  GLOBUS_FTP_CLIENT_SIZE
 *
 **************************************************************************/

%typemap(in) 
  (globus_ftp_client_complete_callback_t size_complete_callback,
   void * size_user_arg) (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  /* Allocate our buffer */
  GlobusFTPClientSizeInit( cbStruct );

  $1 = TclFTPClientSizeCallback; 
  $2 = cbStruct;

  cbStruct->s_post_process = GlobusFTPClientSizeCleanup;
}


%rename(globus_ftp_client_size) tcl_globus_ftp_client_size;

%exception tcl_globus_ftp_client_size GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_client_size
    ( globus_ftp_client_handle_t *		handle,
      const char *				url,
      globus_ftp_client_operationattr_t *	attr,
      globus_ftp_client_complete_callback_t	size_complete_callback,
      void *					size_user_arg )
    {
      globus_result_t retval;
      
      retval = globus_ftp_client_size
        (handle,
         url,
         attr,
         (globus_off_t*)((callbackStruct*)size_user_arg)->s_api_data,
         size_complete_callback,
         size_user_arg );
      
      return retval;
    }

  %}


/**************************************************************************
 *
 *  GLOBUS_FTP_CLIENT_CKSM
 *
 **************************************************************************/

%typemap(in) 
  (globus_ftp_client_complete_callback_t cksm_complete_callback,
   void * cksm_user_arg) (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  /* Allocate our buffer */
  GlobusFTPClientCksmInit( cbStruct );

  $1 = TclFTPClientCksmCallback; 
  $2 = cbStruct;

  cbStruct->s_post_process = GlobusFTPClientCksmCleanup;
}

%rename(globus_ftp_client_cksm) tcl_globus_ftp_client_cksm;

%exception tcl_globus_ftp_client_cksm GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_client_cksm
    ( globus_ftp_client_handle_t *		handle,
      const char *				url,
      globus_ftp_client_operationattr_t *	attr,
      globus_off_t				offset,
      globus_off_t				length,
      const char *				algorithm,
      globus_ftp_client_complete_callback_t	cksm_complete_callback,
      void *					cksm_user_arg )
    {
      globus_result_t retval;
      
      retval = globus_ftp_client_cksm
        (handle,
         url,
         attr,
         ((callbackStruct*)cksm_user_arg)->s_api_data,
         offset,
         length,
         algorithm,
         cksm_complete_callback,
         cksm_user_arg );
      
      return retval;
    }

  %}


/**************************************************************************
 *
 *  GLOBUS_FTP_CLIENT_MODIFICATION_TIME
 *
 **************************************************************************/

%typemap(in) 
  (globus_ftp_client_complete_callback_t mdtm_complete_callback,
   void * mdtm_user_arg) (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  /* Allocate our buffer */
  GlobusFTPClientMdtmInit( cbStruct );

  $1 = TclFTPClientMdtmCallback; 
  $2 = cbStruct;

  cbStruct->s_post_process = GlobusFTPClientMdtmCleanup;
}


%rename(globus_ftp_client_modification_time) tcl_globus_ftp_client_modification_time;

%exception tcl_globus_ftp_client_modification_time GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_ftp_client_modification_time
    ( globus_ftp_client_handle_t *		handle,
      const char *				url,
      globus_ftp_client_operationattr_t *	attr,
      globus_ftp_client_complete_callback_t	mdtm_complete_callback,
      void *					mdtm_user_arg )
    {
      globus_result_t retval;
      
      retval = globus_ftp_client_modification_time
        (handle,
         url,
         attr,
         (globus_abstime_t *)((callbackStruct*)mdtm_user_arg)->s_api_data,
         mdtm_complete_callback,
         mdtm_user_arg );
      
      return retval;
    }

  %}


/**************************************************************************
 *
 *  GLOBUS_FTP_CLIENT_REGISTER_READ
 *
 **************************************************************************/

%typemap(in) 
  (globus_ftp_client_data_callback_t rr_cb, void * rr_user_arg) 
  (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  /* Allocate our buffer */
  GlobusFTPClientRegisterReadInit( cbStruct, arg2 );

  $1 = TclFTPClientRRDataCallback; 
  $2 = cbStruct;

  cbStruct->s_pre_process = GlobusFTPClientRegisterReadPreCallback;
  cbStruct->s_post_process = GlobusFTPClientRegisterReadCleanup;
}


%rename(globus_ftp_client_register_read) tcl_globus_ftp_client_register_read;

%exception tcl_globus_ftp_client_register_read {

  globus_result_t error = result =
    tcl_globus_ftp_client_register_read(arg1,arg2,arg3,arg4);

  if ( error != 0 )
    {
      globus_object_t* error_obj = globus_error_get( error );

      /*
       * We don't need to catch end-of-file flag and
       * falls through to the next execution
       */
      if ( error == 28 )
      {
        
      }
      else
      {
        Tcl_Obj*	tcl_result = Tcl_GetObjResult( interp );
        
        char*	msg = globus_object_printable_to_string( error_obj );
                
        Tcl_SetStringObj( tcl_result, msg, strlen( msg ) );
        
	globus_libc_free( msg );

        return TCL_ERROR;
      }
    }
}

/****f* globus_ftp_client/globus_ftp_client_register_read
 * NAME
 *   globus_ftp_client_register_read
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_register_read
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer_length
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to register a data buffer to handle as a part of
 *   the FTP data transfer.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - FTP client handle which contains state information about the get
 *   operation.
 *
 *   |html <b>
 *   buffer_length
 *   |html </b>
 *   - The maximum amount of data that can be stored into the buffer.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked to perform actual read. The command will be
 *   executed exactly once in global scope. If an error occurs while
 *   executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_register_read returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_register_read $handle $buffer_length
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_operationattr_set_read_all
 *****/

%inline %{

  typedef struct {
    globus_byte_t*	buffer;
    globus_size_t	length;
  } tcl_globus_buffer_t;


  globus_result_t
    tcl_globus_ftp_client_register_read
    ( globus_ftp_client_handle_t* 	handle,
      long 				max_nbytes,
      globus_ftp_client_data_callback_t rr_cb,
      void* 				rr_user_arg )
    {
      globus_result_t	retval;
      
      retval = 
        globus_ftp_client_register_read
        (
         handle,
         (globus_byte_t*)
         ((callbackData*)(((callbackStruct*)rr_user_arg)->s_api_data))->buffer,
         max_nbytes,
         rr_cb,
         rr_user_arg 
         );

      return retval;
    }
  %}


/**************************************************************************
 *
 *  GLOBUS_FTP_CLIENT_REGISTER_WRITE
 *
 **************************************************************************/

%typemap(in) 
  tcl_globus_buffer_t *rw_in_buffer( $*1_ltype temp ) 
{
  unsigned char*   tbuffer;

  tbuffer = 
    (unsigned char *)Tcl_GetByteArrayFromObj( $input, &(temp.length) );
  temp.buffer = globus_libc_malloc( temp.length );
  SGDEBUG_LVL_FMT_1( 20, "write buffer allocation: = %p", temp.buffer );  
  memcpy( temp.buffer, tbuffer, temp.length );

  $1 = &temp;
}

%typemap(in) 
  (globus_ftp_client_data_callback_t rw_cb, void * rw_user_arg) 
  (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );
  GlobusFTPClientRegisterWriteInit(cbStruct, arg2->buffer);
  
  $1 = TclFTPClientRWDataCallback; 
  $2 = cbStruct;
  
  cbStruct->s_pre_process = GlobusFTPClientRegisterWritePreCallback;
  cbStruct->s_post_process = GlobusFTPClientRegisterWriteCleanup;
}


%rename(globus_ftp_client_register_write) tcl_globus_ftp_client_register_write;

%exception tcl_globus_ftp_client_register_write GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_ftp_client/globus_ftp_client_register_write
 * NAME
 *   globus_ftp_client_register_write
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_register_write
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   offset
 *   eof
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to register a data buffer to handle as a part of
 *   the FTP data transfer in the background.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - FTP client handle which contains state information about the put
 *   operation.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - A user-supplied buffer containing the data to write to the server.
 *
 *   |html <b>
 *   offset
 *   |html </b>
 *   - The offset of the buffer to be written. In extended-block mode, the
 *   data does not need to be sent in order. In stream mode (the default),
 *   data must be sent in sequential order. The behavior is undefined if
 *   multiple writes overlap.
 *
 *   |html <b>
 *   eof
 *   |html </b>
 *   - A variable to indicate the end-of-file.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked to perform actual write. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_register_write returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_register_write $handle $buffer $offset
 *   $eof { callback $callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/

%inline %{

  globus_result_t
    tcl_globus_ftp_client_register_write
    ( globus_ftp_client_handle_t * 	handle,
      tcl_globus_buffer_t * 		rw_in_buffer,
      globus_off_t   	    	  	offset,
      globus_bool_t  	   	  	eof,
      globus_ftp_client_data_callback_t rw_cb,
      void * 				rw_user_arg )
    {
      globus_result_t	retval;
      
      retval = globus_ftp_client_register_write( handle,
                                                 rw_in_buffer->buffer,
                                                 rw_in_buffer->length,
                                                 offset,
                                                 eof,
                                                 rw_cb,
                                                 rw_user_arg );

      
      return retval;
    }
  %}


#endif /* SWIG */



/**************************************************************************
 *
 * ROBODOC
 *
 **************************************************************************/


////////////////////////////////////////////////////////////////////
//
// RESTART MARKERS
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_restart_marker_init
 * NAME
 *   globus_ftp_client_restart_marker_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_restart_marker_init
 *   |html </b>
 *   |html <i>
 *   marker
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a restart marker.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   marker
 *   |html </b>
 *   - New restart marker.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_restart_marker_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_restart_marker_init $marker ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_restart_marker_destroy.
 *****/


/****f* globus_ftp_client/globus_ftp_client_restart_marker_copy
 * NAME
 *   globus_ftp_client_restart_marker_copy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_restart_marker_copy
 *   |html </b>
 *   |html <i>
 *   new_marker
 *   old_marker
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create a copy of a restart marker.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   new_marker
 *   |html </b>
 *   - A pointer to a new restart marker.
 *
 *   |html <b>
 *   old_marker
 *   |html </b>
 *   - The marker to copy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_restart_marker_copy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_restart_marker_copy $new_marker
 *   $old_marker ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_restart_marker_init,
 *   globus_ftp_client_restart_marker_destroy.
 *****/


/****f* globus_ftp_client/globus_ftp_client_restart_marker_destroy
 * NAME
 *   globus_ftp_client_restart_marker_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_restart_marker_destroy
 *   |html </b>
 *   |html <i>
 *   marker
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a restart marker.
 *
 *   This marker must be initialized by either calling
 *   globus_ftp_client_restart_marker_init or
 *   globus_ftp_client_restart_marker_copy.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   marker
 *   |html </b>
 *   - Restart marker.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_restart_marker_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_restart_marker_destroy $marker ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_restart_marker_init,
 *   globus_ftp_client_restart_marker_copy.
 *****/



/****f* globus_ftp_client/globus_ftp_client_restart_marker_insert_range
 * NAME
 *   globus_ftp_client_restart_marker_insert_range
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_restart_marker_insert_range
 *   |html </b>
 *   |html <i>
 *   marker
 *   offset
 *   end_offset
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to updates a restart marker with a new byte range,
 *   suitable for using to restart an extended block mode transfer. Adjacent
 *   ranges within the marker will be combined into a single entry in the
 *   marker.
 *
 *   This marker must be initialized by either calling
 *   globus_ftp_client_restart_marker_init or
 *   globus_ftp_client_restart_marker_copy.
 *
 *   A marker can only hold a range list or a stream offset. Calling this
 *   command after calling globus_ftp_client_restart_marker_set_offset will
 *   result in a marker suitable only for use restarting an extended block
 *   mode transfer.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   marker
 *   |html </b>
 *   - Restart marker.
 *
 *   |html <b>
 *   offset
 *   |html </b>
 *   - The starting offset of the range.
 *
 *   |html <b>
 *   end_offset
 *   |html </b>
 *   - The ending offset of the range.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_restart_marker_insert_range returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_restart_marker_insert_range $marker
 *   $offset $end_offset ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_restart_marker_set_offset,
 *   globus_ftp_client_operationattr_set_mode.
 *****/


/****f* globus_ftp_client/globus_ftp_client_restart_marker_set_ascii_offset
 * NAME
 *   globus_ftp_client_restart_marker_set_ascii_offset
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_restart_marker_set_ascii_offset
 *   |html </b>
 *   |html <i>
 *   marker
 *   offset
 *   ascii_offset
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to modify a restart marker to contain a stream
 *   offset, suitable for using to restart a steam mode transfer.
 *
 *   The marker must first be initialized by calling
 *   globus_ftp_client_restart_marker_init or
 *   globus_ftp_client_restart_marker_copy.
 *
 *   A marker can only hold a range list or a stream offset. Calling this
 *   function after calling globus_ftp_client_restart_marker_insert_range
 *   will delete the ranges associated with the marker, and replace it with
 *   a marker suitable only for use restarting a stream mode transfer.
 *
 *   When restarting an ASCII type transfer, use
 *   globus_ftp_client_restart_marker_set_ascii_offset to set both the offset
 *   used in the local representation of an ACSII file, and the network
 *   representation of the ASCII file. For UNIX systems, the former includes
 *   counts newlines as one character towards the file offset, and the latter
 *   counts them as 2 characters (CRLF).
 *
 * PARAMETERS
 *   |html <ul><b>
 *   marker
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   offset
 *   |html </b>
 *   - The local stream offset.
 *
 *   |html <b>
 *   ascii_offset
 *   |html </b>
 *   - The network ASCII representation of the offset.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_restart_marker_set_ascii_offset returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_restart_marker_set_ascii_offset $marker
 *   $offset $ascii_offset ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_restart_marker_insert_range,
 *   globus_ftp_client_restart_marker_set_offset,
 *   globus_ftp_client_operationattr_set_mode,
 *   globus_ftp_client_operationattr_set_type.
 *****/



/****f* globus_ftp_client/globus_ftp_client_restart_marker_set_offset
 * NAME
 *   globus_ftp_client_restart_marker_set_offset
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_restart_marker_set_offset
 *   |html </b>
 *   |html <i>
 *   marker
 *   offset
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to modify a restart marker to contain a stream
 *   offset, suitable for using to restart a steam mode transfer.
 *
 *   The marker must first be initialized by calling
 *   globus_ftp_client_restart_marker_init or
 *   globus_ftp_client_restart_marker_copy.
 *
 *   A marker can only hold a range list or a stream offset. Calling this
 *   function after calling globus_ftp_client_restart_marker_insert_range
 *   will delete the ranges associated with the marker, and replace it with
 *   a marker suitable only for use restarting a stream mode transfer.
 *
 *   When restarting an ASCII type transfer, the offset must take into
 *   account the additional carriage return characters added to the data
 *   stream.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   marker
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   offset
 *   |html </b>
 *   - The stream offset.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_restart_marker_set_offset returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_restart_marker_set_offset $marker $offset ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_restart_marker_insert_range,
 *   globus_ftp_client_operationattr_set_mode,
 *   globus_ftp_client_operationattr_set_type.
 *****/



/****f* globus_ftp_client/globus_ftp_client_restart_marker_get_total
 * NAME
 *   globus_ftp_client_restart_marker_get_total
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_restart_marker_get_total
 *   |html </b>
 *   |html <i>
 *   marker
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to get the total bytes accounted for in restart
 *   marker.
 *
 *   If this restart marker contains a stream offset then this value is the
 *   same as the offset (not the ASCII offset) that it was set with. If it is
 *   a range list, it returns a sum of all the bytes in the ranges.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   marker
 *   |html </b>
 *   - A previously initialized or copied restart marker.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_restart_marker_get_total returns 0 and
 *   the total bytes accounted for in restart marker. On error, this command
 *   will raise an error and must be caught using the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_restart_marker_get_total $marker ]
 *   result total_bytes
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_client/globus_ftp_client_restart_marker_to_string
 * NAME
 *   globus_ftp_client_restart_marker_to_string
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_restart_marker_to_string
 *   |html </b>
 *   |html <i>
 *   marker
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to create a string representation of a restart
 *   marker.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   marker
 *   |html </b>
 *   - An initialized FTP client restart marker.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_restart_marker_to_string returns 0 and
 *   the string representation of a restart marker. On error, this command
 *   will raise an error and must be caught using the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_restart_marker_to_string $marker ]
 *   result marker_string
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_client/globus_ftp_client_restart_marker_from_string
 * NAME
 *   globus_ftp_client_restart_marker_from_string
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_restart_marker_from_string
 *   |html </b>
 *   |html <i>
 *   marker
 *   marker_string
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to initializes a new restart, marker, based on
 *   the marker_string parameter. The string may be either a single offset
 *   for a stream-mode restart marker, or a comma-separated list of start-end
 *   ranges.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   marker
 *   |html </b>
 *   - The restart marker to be unitialized.
 *
 *   |html <ul><b>
 *   marker_string
 *   |html </b>
 *   - The string containing a textual representation of a restart marker.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_restart_marker_from_string returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_restart_marker_from_string $marker
 *   $marker_string ] result 
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



////////////////////////////////////////////////////////////////////
//
// HANDLE MANAGEMENT
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_handle_init
 * NAME
 *   globus_ftp_client_handle_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handle_init
 *   |html </b>
 *   |html <i>
 *   handle
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a client FTP handle which can be
 *   used in subsequent get, put, or transfer requests. A handle may have
 *   at most one get, put, or third-party transfer in progress.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to be initialized.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Initial attributes to be used to create this handle. This can be
 *   NULL.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handle_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handle_init $handle NULL ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_handle_destroy.
 *****/



/****f* globus_ftp_client/globus_ftp_client_handle_destroy
 * NAME
 *   globus_ftp_client_handle_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handle_destroy
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a client FTP handle.
 *
 *   A FTP client handle may not be destroyed if a get, put, or third-party
 *   transfer is in progress.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The handle to be destroyed.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handle_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handle_destroy $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_handle_init.
 *****/


/****f* globus_ftp_client/globus_ftp_client_handle_cache_url_state
 * NAME
 *   globus_ftp_client_handle_cache_url_state
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handle_cache_url_state
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to explicitly cache connections to URL server in
 *   an FTP handle. When an URL is cached, the client library will not close
 *   the connection to the URL server after a file transfer completes.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Handle which will contain a cached connection to the URL server.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL of the FTP or GSIFTP server to cache.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handle_cache_url_state returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handle_cache_url_state $handle $url ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_flush_url_state.
 *****/



/****f* globus_ftp_client/globus_ftp_client_handle_flush_url_state
 * NAME
 *   globus_ftp_client_handle_flush_url_state
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handle_flush_url_state
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to explicitly remove a cached connection to an
 *   FTP server from the FTP handle. If an idle connection to an FTP server
 *   exists, it will be closed.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Handle which will contain a cached connection to the URL server.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL of the FTP or GSIFTP server to cache.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handle_flush_url_state returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handle_flush_url_state $handle $url ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_handle_cache_url_state.
 *****/


/****f* globus_ftp_client/globus_ftp_client_handle_set_user_pointer
 * NAME
 *   globus_ftp_client_handle_set_user_pointer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handle_set_user_pointer
 *   |html </b>
 *   |html <i>
 *   handle
 *   user_pointer
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the user pointer field from an FTP client
 *   handle.
 *
 *   The user pointer is provided to all the user of the FTP client library
 *   to associate a pointer to any application-specific data to an FTP client
 *   handle. This pointer is never internally used by the client library.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP client handle to set.
 *
 *   |html <b>
 *   user_pointer
 *   |html </b>
 *   - The value of the user pointer.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handle_set_user_pointer returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handle_set_user_pointer $handle
 *   $user_pointer ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_client/globus_ftp_client_handle_get_user_pointer
 * NAME
 *   globus_ftp_client_handle_get_user_pointer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handle_get_user_pointer
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the user pointer field from an FTP client
 *   handle.
 *
 *   The user pointer is provided to all the user of the FTP client library
 *   to associate a pointer to any application-specific data to an FTP client
 *   handle. This pointer is never internally used by the client library.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP client handle to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handle_get_user_pointer returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handle_get_user_pointer $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_client/globus_ftp_client_handle_add_plugin
 * NAME
 *   globus_ftp_client_handle_add_plugin
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handle_add_plugin
 *   |html </b>
 *   |html <i>
 *   handle
 *   plugin
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to add a plugin to an FTP client handle after it
 *   has been created. Plugins may be added to an FTP client handle whenever
 *   an operation is not in progress. The plugin will be appended to the list
 *   of plugins present in the handle, and will be invoked during any
 *   subsequent operations processed with this handle.
 *
 *   Only one instance of a particular plugin may be added to a particular
 *   handle.
 *
 *   Plugins may be removed from a handle by calling
 *   globus_ftp_client_handle_remove_plugin.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP client handle.
 *
 *   |html <b>
 *   plugin
 *   |html </b>
 *   - A pointer to the plugin structure to add to this handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handle_add_plugin returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handle_add_plugin $handle $plugin ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_remove_plugin, globus_ftp_clent_handleattr_add_plugin,
 *   globus_ftp_clent_handleattr_remove_plugin.
 *****/



/****f* globus_ftp_client/globus_ftp_client_handle_remove_plugin
 * NAME
 *   globus_ftp_client_handle_remove_plugin
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handle_remove_plugin
 *   |html </b>
 *   |html <i>
 *   handle
 *   plugin
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to remove a plugin to an FTP client handle.
 *
 *   This command removes a plugin from an FTP client handle after it has
 *   been created. Plugins may be removed from an FTP client handle whenever
 *   an operation is not in progress. The plugin will be removed from the
 *   list of plugins, and will not be used during any subsequent operations
 *   processed with this handle.
 *
 *   This command can remove plugins which were added at
 *   globus_ftp_client_handle_init "handle initialization time" or by
 *   calling globus_ftp_client_handle_add_plugin.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - The FTP client handle.
 *
 *   |html <b>
 *   plugin
 *   |html </b>
 *   - A pointer to the plugin structure to remove from this handle.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handle_remove_plugin returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handle_add_plugin $handle $plugin ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_handleattr_add_plugin,
 *   globus_ftp_client_handleattr_remove_plugin.
 *****/

////////////////////////////////////////////////////////////////////
//
// HANDLE ATTRIBUTES
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_handleattr_init
 * NAME
 *   globus_ftp_client_handleattr_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_init
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize an FTP client handle attribute set.
 *
 *   This command creates an empty FTP client handle attribute set. This
 *   command must be called on each attribute set before any of the other
 *   methods may be called.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The new handle attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_init $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_handleattr_destroy.
 *****/



/****f* globus_ftp_client/globus_ftp_client_handleattr_destroy
 * NAME
 *   globus_ftp_client_handleattr_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_destroy
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a FTP client handle attribute set.
 *
 *   This command destroys a FTP client handle attribute set. All attributes
 *   on this set will be lost. The user must call
 *   globus_ftp_client_handleattr_init again on this attribute set before
 *   calling any other handle attribute functions on it.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to destroy.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_destroy $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_handleattr_init.
 *****/



/****f* globus_ftp_client/globus_ftp_client_handleattr_copy
 * NAME
 *   globus_ftp_client_handleattr_copy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_copy
 *   |html </b>
 *   |html <i>
 *   dest
 *   src
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create a duplicate of a handle attribute set.
 *
 *   The duplicated attribute set has a deep copy of all data in the
 *   attribute set, so the original may be destroyed while the copy is
 *   still valid.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   dest
 *   |html </b>
 *   - The attribute set to be initialized to the same values as src.
 *
 *   |html <b>
 *   src
 *   |html </b>
 *   - The original attribute set to duplicate.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_copy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_copy $dest $src ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// Connection Caching
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_handleattr_set_cache_all
 * NAME
 *   globus_ftp_client_handleattr_set_cache_all
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_set_cache_all
 *   |html </b>
 *   |html <i>
 *   attr
 *   cache_all
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the cache all connections attribute for
 *   an FTP client handle attribute set.
 *
 *   This attribute allows the user to cause all control connections to be
 *   cached between FTP operations. When this is enabled, the user skips
 *   the authentication handshake and connection establishment overhead
 *   for multiple subsequent ftp operations to the same server.
 *
 *   Memory and network connections associated with the caching will be
 *   used until the handle is destroyed. If fine grained caching is needed,
 *   then the user should disable this attribute and explicitly cache
 *   specific URLs.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Attribute to query or modify.
 *
 *   |html <b>
 *   cache_all
 *   |html </b>
 *   - Boolean value of the cache_all attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_set_cache_all returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_set_cache_all $attr $cache_all ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_handleattr_add_cached_url,
 *   globus_ftp_client_handleattr_remove_cached_url,
 *   globus_ftp_client_handle_cache_url_state,
 *   globus_ftp_client_handle_flush_url_state
 *****/


/****f* globus_ftp_client/globus_ftp_client_handleattr_get_cache_all
 * NAME
 *   globus_ftp_client_handleattr_get_cache_all
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_get_cache_all
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the cache all connections attribute for
 *   an FTP client handle attribute set.
 *
 *   This attribute allows the user to cause all control connections to be
 *   cached between FTP operations. When this is enabled, the user skips
 *   the authentication handshake and connection establishment overhead
 *   for multiple subsequent ftp operations to the same server.
 *
 *   Memory and network connections associated with the caching will be
 *   used until the handle is destroyed. If fine grained caching is needed,
 *   then the user should disable this attribute and explicitly cache
 *   specific URLs.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Attribute to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_get_cache_all returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_get_cache_all $attr ]
 *   result cache_all
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_handleattr_add_cached_url,
 *   globus_ftp_client_handleattr_remove_cached_url,
 *   globus_ftp_client_handle_cache_url_state,
 *   globus_ftp_client_handle_flush_url_state
 *****/



////////////////////////////////////////////////////////////////////
//
// Non-root relative URLs
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_handleattr_set_rfc1738_url
 * NAME
 *   globus_ftp_client_handleattr_set_rfc1738_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_set_rfc1738_url
 *   |html </b>
 *   |html <i>
 *   attr
 *   rfc1738_url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to enable/disable RFC1738 support for
 *   non-root relative URLs.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Attribute to modify.
 *
 *   |html <b>
 *   rfc1738_url
 *   |html </b>
 *   - Set to GLOBUS_TRUE to enable non-root relative URLs.
 *   Default of GLOBUS_FALSE specifies root-relative URLs.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_set_rfc1738_url returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_set_rfc1738_url $attr
 *   $GLOBUS_TRUE ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_client/globus_ftp_client_handleattr_get_rfc1738_url
 * NAME
 *   globus_ftp_client_handleattr_get_rfc1738_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_get_rfc1738_url
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to query the value of RFC1738 support
 *   for non-root relative URLs.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Attribute to modify.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_get_rfc1738_url returns 0
 *   and the value of rfc1738_url flag set by
 *   globus_ftp_client_handleattr_set_rfc1738_url. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_get_rfc1738_url $attr ]
 *   result rfc1738_url
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// URL Caching
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_handleattr_add_cached_url
 * NAME
 *   globus_ftp_client_handleattr_add_cached_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_add_cached_url
 *   |html </b>
 *   |html <i>
 *   attr
 *   url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to enable caching for a specific URL.
 *
 *   This command adds the specified URL into the default cache for a handle
 *   attribute. Handles initialized with this attr will keep connections to
 *   FTP servers associated with the URLs in its cache open between
 *   operations.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Attribute to modify.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL string to cache.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_add_cached_url returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_add_cached_url $attr $url ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_client/globus_ftp_client_handleattr_remove_cached_url
 * NAME
 *   globus_ftp_client_handleattr_remove_cached_url
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_remove_cached_url
 *   |html </b>
 *   |html <i>
 *   attr
 *   url
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to disable caching for a specific URL.
 *
 *   This command removes the specified URL into the default cache for a
 *   handle attribute. Handles initialized with this attr will keep
 *   connections to FTP servers associated with the URLs in its cache open
 *   between operations.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Attribute to modify.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - URL string to cache.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_remove_cached_url returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_remove_cached_url $attr $url ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/

////////////////////////////////////////////////////////////////////
//
// Plugin Management
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_handleattr_add_plugin
 * NAME
 *   globus_ftp_client_handleattr_add_plugin
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_add_plugin
 *   |html </b>
 *   |html <i>
 *   attr
 *   plugin
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to add a plugin to a handle attribute set.
 *
 *   Each handle attribute set contains a list of plugins associated with it.
 *   When a handle is created with a particular attribute set, it will be
 *   associated with a copy of those plugins.
 *
 *   Only one instance of a specific plugin may be added to an attribute set.
 *   Each plugin must have a different name.
 *
 *   A copy of the plugin is created via the plugins 'copy' method when it
 *   is added to an attribute set. Thus, any changes to a particular plugin
 *   must be done before the plugin is added to an attribute set, and before
 *   the attribute set is used to create handles.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   plugin
 *   |html </b>
 *   - The plugin to add to the list.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_add_plugin returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_add_plugin $attr $plugin ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_client/globus_ftp_client_handleattr_remove_plugin
 * NAME
 *   globus_ftp_client_handleattr_remove_plugin
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_remove_plugin
 *   |html </b>
 *   |html <i>
 *   attr
 *   plugin
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to remove a plugin from a handle attribute set.
 *
 *   Each handle attribute set contains a list of plugins associated with it.
 *   When a handle is created with a particular attribute set, it will be
 *   associated with a copy of those plugins.
 *
 *   Only one instance of a specific plugin may be added to an attribute set.
 *   Each plugin must have a different name.
 *
 *   A copy of the plugin is created via the plugins 'copy' method when it
 *   is added to an attribute set. Thus, any changes to a particular plugin
 *   must be done before the plugin is added to an attribute set, and before
 *   the attribute set is used to create handles.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   plugin
 *   |html </b>
 *   - The plugin to remove from the list.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_remove_plugin returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_handleattr_remove_plugin $attr $plugin ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/

////////////////////////////////////////////////////////////////////
//
// Netlogger management
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_handleattr_set_netlogger
 * NAME
 *   globus_ftp_client_handleattr_set_netlogger
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_set_netlogger
 *   |html </b>
 *   |html <i>
 *   attr
 *   nl_handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the netlogger handle used with this transfer.
 *   Each handle can have a netlogger handle associated with it for logging
 *   its data. Only 1 netlogger handle can be associated with a client handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   nl_handle
 *   |html </b>
 *   - The open netlogger handle to be associated with this attribute set.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_set_netlogger returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *  
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_client/globus_ftp_client_handleattr_set_netlogger_ftp_io
 * NAME
 *   globus_ftp_client_handleattr_set_netlogger_ftp_io
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_handleattr_set_netlogger_ftp_io
 *   |html </b>
 *   |html <i>
 *   attr
 *   nl_handle
 *   ftp
 *   io
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the netlogger handle used with
 *   this transfer.
 *
 *   Each handle can have a netlogger handle associated with it for
 *   logging its data.
 *
 *   Only 1 netlogger handle can be associated with a client handle.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   nl_handle
 *   |html </b>
 *   - The open netlogger handle to be associated with this attribute set.
 *
 *   |html <b>
 *   ftp
 *   |html </b>
 *   - |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *   |html <b>
 *   io
 *   |html </b>
 *   - |html <font color="Red">
 *   TBD
 *   |html </font>
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_handleattr_set_netlogger_ftp_io returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *  
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/




////////////////////////////////////////////////////////////////////
//
// FTP OPERATIONS
//
////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////
//
// Filesize
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_size
 * NAME
 *   globus_ftp_client_size
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_size
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to request the size of a file from an FTP server.
 *
 *   When the filesize request is completed or aborted, the
 *   script will be invoked with the final status of the operation. If the
 *   callback is returns without an error, the filesize will be
 *   stored in the filesize parameter of the callback script.
 *
 *   Note:
 *   In ASCII mode, the size will be the size of the file after conversion
 *   to ASCII mode. The actual amount of data which is returned in the data
 *   callbacks may be less than this amount.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP client handle to use for the operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to list. The URL may be a FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - 	Attributes for this operation.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the size check is completed.
 *   The command will be executed exactly once in
 *   global scope. If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_size returns 0 and the total size of the
 *   file, if it exists. On error, this command will raise an error and
 *   must be caught using the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_size $handle $url $attr
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// File or Directory Existence
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_exists
 * NAME
 *   globus_ftp_client_exists
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_exists
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   operation_attr
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to check for the existence of a file or
 *   directory on an FTP server.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP client handle to use for the existence check operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL of the directory or file to check. The URL may be a FTP
 *   or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes to use for this operation.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the command is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_exists returns 0.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_exists $handle $url $operation_attr
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// Make Directory
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_mkdir
 * NAME
 *   globus_ftp_client_mkdir
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_mkdir
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   operation_attr
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to make a directory on an FTP server.
 *
 *   When the response to the "mkdir" request has been received
 *   the script will be invoked with the result of the "mkdir" operation.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP client handle to use for the mkdir operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL for the directory to create. The URL may be a FTP
 *   or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes to use for this operation.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the command is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_mkdir returns 0.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_mkdir $handle $url $operation_attr
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// Remove Directory
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_rmdir
 * NAME
 *   globus_ftp_client_rmdir
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_rmdir
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   operation_attr
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to remove a directory on an FTP server.
 *
 *   When the response to the "rmdir" request has been received
 *   the script will be invoked with the result of the "rmdir" operation.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP client handle to use for the rmdir operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL for the directory to be removed. The URL may be a FTP
 *   or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes to use for this operation.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the command is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_rmdir returns 0.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_rmdir $handle $url $operation_attr
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/

////////////////////////////////////////////////////////////////////
//
// Delete
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_delete
 * NAME
 *   globus_ftp_client_delete
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_delete
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_delete
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   operation_attr
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command starts a delete operation on a FTP server. When the
 *   response to the delete request has been received the script
 *   will be invoked with the result of the delete operation.
 *
 *   |html <font color="Red">
 *   Note that this functions will only delete files, not directories.
 *   |html </font>
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP client handle to use for the delete operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL for the file to delete. The URL may be a FTP
 *   or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes to use for this operation.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the command is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_delete returns 0.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_delete $handle $url $operation_attr
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/

////////////////////////////////////////////////////////////////////
//
// List
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_list
 * NAME
 *   globus_ftp_client_list
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_list
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   operation_attr
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to get a file listing from an FTP server.
 *
 *   This function starts a "NLST" transfer from an FTP server. If
 *   this function returns no error, then the user may immediately
 *   begin calling globus_ftp_client_read to retrieve the data
 *   associated with this listing.
 *
 *   When all of the data associated with the listing is retrieved,
 *   and all of the data callbacks have been called, or if the list
 *   request is aborted, the script will be invoked with the final
 *   status of the list.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP client handle to use for the list operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to list. The URL may be a FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - 	Attributes for this file transfer.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the "list" is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_list returns 0.
 *   On error, this command will raise an error and
 *   must be caught using the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_list $handle $url $operation_attr
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_client/globus_ftp_client_verbose_list
 * NAME
 *   globus_ftp_client_verbose_list
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_verbose_list
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   operation_attr
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to get a file listing from an FTP server.
 *
 *   This function starts a "LIST" transfer from an FTP server. If
 *   this function returns no error, then the user may immediately
 *   begin calling globus_ftp_client_read to retrieve the data
 *   associated with this listing.
 *
 *   When all of the data associated with the listing is retrieved,
 *   and all of the data callbacks have been called, or if the list
 *   request is aborted, the script will be invoked with the final
 *   status of the list.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP client handle to use for the list operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to list. The URL may be a FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - 	Attributes for this file transfer.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the "list" is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_verbose_list returns 0.
 *   On error, this command will raise an error and
 *   must be caught using the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_verbose_list $handle $url $operation_attr
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_client/globus_ftp_client_machine_list
 * NAME
 *   globus_ftp_client_machine_list
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_machine_list
 *   |html </b>
 *   |html <i>
 *   u_handle
 *   url
 *   attr
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to get a machine parseable file listing from
 *   an FTP server.
 *
 *   This command starts a "MLSD" transfer from an FTP server. If this
 *   command returns GLOBUS_SUCCESS, then the user may immediately begin
 *   calling globus_ftp_client_register_read to retrieve the data associated
 *   with this listing.
 *
 *   When all of the data associated with the listing is retrieved, and all
 *   of the data callbacks have been called, or if the list request is
 *   aborted, the script will be invoked with the final status
 *   of the list.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   u_handle
 *   |html </b>
 *   - An FTP client handle to use for the list operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to list. The URL may be a FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - 	Attributes for this file transfer.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the "list" is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_machine_list returns 0.
 *   On error, this command will raise an error and
 *   must be caught using the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_machine_list $handle $url $attr
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



////////////////////////////////////////////////////////////////////
//
// MLST
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_mlst
 * NAME
 *   globus_ftp_client_mlst
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_mlst
 *   |html </b>
 *   |html <i>
 *   u_handle
 *   url
 *   attr
 *   mlst_buffer
 *   mlst_buffer_length
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to get information about a file or directory
 *   from a FTP server.
 *
 *   This command requests the MLST fact string of a file or directory
 *   from an FTP server.
 * 
 *   When the MLST request is completed or aborted, the script will be
 *   invoked with the final status of the operation. If the callback
 *   is returns without an error, the MLST fact string will be stored
 *   in the mlst_buffer parameter to this command.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   u_handle
 *   |html </b>
 *   - An FTP client handle to use for the list operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to list. The URL may be a FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes for this file transfer.
 *
 *   |html <b>
 *   mlst_buffer
 *   |html </b>
 *   - It contains the MLST fact string of the file, if it exists.
 *   Otherwise, the value pointed to by it is undefined. 
 *
 *   |html <b>
 *   mlst_buffer_length
 *   |html </b>
 *   - It contains the length of the data in mlst_buffer.	
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the "list" is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_mlst returns 0.
 *   On error, this command will raise an error and
 *   must be caught using the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_mlst $handle $url $attr $mlst_buffer
 *   $mlst_buffer_length { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



////////////////////////////////////////////////////////////////////
//
// Move
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_move
 * NAME
 *   globus_ftp_client_move
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_move
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   restart
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to start a move (rename) operation on an
 *   FTP server. Note that this function does not move files between
 *   FTP servers and that the host:port part of the destination url
 *   is ignored.
 *
 *   When the response to the move request has been received the
 *   script will be invoked with the result of the move operation.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP Client handle to use for the move operation.
 *
 *   |html <b>
 *   source_url
 *   |html </b>
 *   - The URL for the file to move.
 *
 *   |html <b>
 *   dest_url
 *   |html </b>
 *   - The URL for the target of the move. The host:port part of this
 *   URL is ignored.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes for this operation.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the "move" operation is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_move returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_move $handle $source_url $dest_url $attr
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// Get
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_get
 * NAME
 *   globus_ftp_client_get
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_get
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   restart
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a "get" file transfer from a FTP server.
 *   If this command returns no error, then the user may immediately
 *   begin calling globus_ftp_client_register_read to retrieve the data
 *   associated with this URL.
 *
 *   When all of the data associated with this URL is retrieved, and all of the
 *   data callbacks have been called, or if the get request is aborted, the
 *   script will be invoked with the final status of the get.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - FTP client handle to use for the get operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to download. The URL may be FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes for this file transfer.
 *
 *   |html <b>
 *   restart
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the "get" is completed. The command will be
 *   executed exactly once in global scope. If an error occurs while
 *   executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_get returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_get $handle $url NULL NULL
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_register_read.
 *****/


/****f* globus_ftp_client/globus_ftp_client_partial_get
 * NAME
 *   globus_ftp_client_partial_get
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_partial_get
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   restart
 *   partial_offset
 *   partial_end_offset
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a "get" file transfer from a FTP server.
 *   If this command returns no error, then the user may immediately
 *   begin calling globus_ftp_client_read to retrieve the data associated with
 *   this URL.
 *
 *   When all of the data associated with this URL is retrieved, and all of the
 *   data callbacks have been called, or if the get request is aborted, the
 *   script will be invoked with the final status of the get.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - FTP client handle to use for the get operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to download. The URL may be FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes for this file transfer.
 *
 *   |html <b>
 *   restart
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   partial_offset
 *   |html </b>
 *   - Starting offset for a partial file get.
 *
 *   |html <b>
 *   partial_end_offset
 *   |html </b>
 *   - Ending offset for a partial file get.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the "get" is completed. The command will be
 *   executed exactly once in global scope. If an error occurs while
 *   executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_partial_get returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_register_read.
 *****/


/****f* globus_ftp_client/globus_ftp_client_extended_get
 * NAME
 *   globus_ftp_client_extended_get
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_extended_get
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   restart
 *   eret_alg_str
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a "get" file transfer from a FTP server
 *   with server-side processing.
 *   If this command returns no error, then the user may immediately
 *   begin calling globus_ftp_client_read to retrieve the data associated with
 *   this URL.
 *
 *   When all of the data associated with this URL is retrieved, and all of the
 *   data callbacks have been called, or if the get request is aborted, the
 *   script will be invoked with the final status of the get.
 *
 *   This function differs from the globus_ftp_client_get command by
 *   allowing the user to invoke server-side data processing algorithms.
 *   GridFTP servers may support support algorithms for data reduction or
 *   other customized data storage requirements. There is no client-side
 *   verification done on the algorithm string provided by the user. If the
 *   server does not understand the requested algorithm, the transfer will
 *   fail.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - FTP client handle to use for the get operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to download. The URL may be FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes for this file transfer.
 *
 *   |html <b>
 *   restart
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   eret_alg_str
 *   |html </b>
 *   - The ERET algorithm string. This string contains information needed
 *   to invoke a server-specific data reduction algorithm on the file being
 *   retrieved.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the "get" is completed. The command will be
 *   executed exactly once in global scope. If an error occurs while
 *   executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_extended_get returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_register_read.
 *****/


////////////////////////////////////////////////////////////////////
//
// Put
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_put
 * NAME
 *   globus_ftp_client_put
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_put
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   restart
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a "put" file transfer to a FTP server.
 *   If this command returns no error, then the user may immediately
 *   begin calling globus_ftp_client_write to send the data associated with
 *   this URL.
 *
 *   When all of the data associated with this URL is sent, and all of the
 *   data callbacks have been called, or if the put request is aborted, the
 *   script will be invoked with the final status of the put.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - FTP client handle to use for the put operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to store the data to. The URL may be FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes for this file transfer.
 *
 *   |html <b>
 *   restart
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the "put" is completed. The command will be
 *   executed exactly once in global scope. If an error occurs while
 *   executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_put returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_put $handle $url NULL NULL
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_register_write.
 *****/



/****f* globus_ftp_client/globus_ftp_client_partial_put
 * NAME
 *   globus_ftp_client_partial_put
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_partial_put
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   restart
 *   partial_offset
 *   partial_end_offset
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a "put" file transfer from a FTP server.
 *   If this command returns no error, then the user may immediately
 *   begin calling globus_ftp_client_write to send the data associated with
 *   this URL.
 *
 *   When all of the data associated with this URL is retrieved, and all of the
 *   data callbacks have been called, or if the put request is aborted, the
 *   script will be invoked with the final status of the put.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - FTP client handle to use for the put operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to download. The URL may be FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes for this file transfer.
 *
 *   |html <b>
 *   restart
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   partial_offset
 *   |html </b>
 *   - Starting offset for a partial file put.
 *
 *   |html <b>
 *   partial_end_offset
 *   |html </b>
 *   - Ending offset for a partial file put.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the "put" is completed. The command will be
 *   executed exactly once in global scope. If an error occurs while
 *   executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_partial_put returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_register_write.
 *****/


/****f* globus_ftp_client/globus_ftp_client_extended_put
 * NAME
 *   globus_ftp_client_extended_put
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_extended_put
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   restart
 *   eret_alg_str
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initiate a "put" file transfer to a FTP server
 *   with server-side processing.
 *   If this command returns no error, then the user may immediately
 *   begin calling globus_ftp_client_write to send the data associated with
 *   this URL.
 *
 *   When all of the data associated with this URL is sent, and all of the
 *   data callbacks have been called, or if the get request is aborted, the
 *   script will be invoked with the final status of the put.
 *
 *   This function differs from the globus_ftp_client_put command by
 *   allowing the user to invoke server-side data processing algorithms.
 *   GridFTP servers may support support algorithms for data reduction or
 *   other customized data storage requirements. There is no client-side
 *   verification done on the algorithm string provided by the user. If the
 *   server does not understand the requested algorithm, the transfer will
 *   fail.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - FTP client handle to use for the get operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to download. The URL may be FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes for this file transfer.
 *
 *   |html <b>
 *   restart
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   eret_alg_str
 *   |html </b>
 *   - The ERET algorithm string. This string contains information needed
 *   to invoke a server-specific data reduction algorithm on the file being
 *   retrieved.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the "put" is completed. The command will be
 *   executed exactly once in global scope. If an error occurs while
 *   executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_extended_put returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_register_write.
 *****/

////////////////////////////////////////////////////////////////////
//
// 3rd Party Transfer
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_third_party_transfer
 * NAME
 *   globus_ftp_client_third_party_transfer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_third_party_transfer
 *   |html </b>
 *   |html <i>
 *   handle
 *   source_url
 *   source_attr
 *   dest_url
 *   dest_attr
 *   restart
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to transfer a file between two FTP servers.
 *
 *   When the transfer is completed or if the transfer is aborted, the
 *   script will be invoked with the final status of the transfer.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP client handle to use for the transfer operation.
 *
 *   |html <b>
 *   source_url
 *   |html </b>
 *   - The URL to transfer. The URL may be an FTP or GSIFTP URL.
 *
 *   |html <b>
 *   source_attr
 *   |html </b>
 *   - Attributes for the souce URL.
 *
 *   |html <b>
 *   dest_url
 *   |html </b>
 *   - The destination URL for the transfer. The URL may be an FTP or
 *   GSIFTP URL.
 *
 *   |html <b>
 *   dest_attr
 *   |html </b>
 *   - Attributes for the destination URL.
 *
 *   |html <b>
 *   restart
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the "transfer" operation is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_third_party_transfer returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


/****f* globus_ftp_client/globus_ftp_client_partial_third_party_transfer
 * NAME
 *   globus_ftp_client_partial_third_party_transfer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_partial_third_party_transfer
 *   |html </b>
 *   |html <i>
 *   handle
 *   source_url
 *   source_attr
 *   dest_url
 *   dest_attr
 *   restart
 *   partial_offset
 *   partial_end_offset
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to transfer a file between two FTP servers.
 *
 *   When the transfer is completed or if the transfer is aborted, the
 *   script will be invoked with the final status of the transfer.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP client handle to use for the transfer operation.
 *
 *   |html <b>
 *   source_url
 *   |html </b>
 *   - The URL to transfer. The URL may be an FTP or GSIFTP URL.
 *
 *   |html <b>
 *   source_attr
 *   |html </b>
 *   - Attributes for the souce URL.
 *
 *   |html <b>
 *   dest_url
 *   |html </b>
 *   - The destination URL for the transfer. The URL may be an FTP or
 *   GSIFTP URL.
 *
 *   |html <b>
 *   dest_attr
 *   |html </b>
 *   - Attributes for the destination URL.
 *
 *   |html <b>
 *   restart
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   partial_offset
 *   |html </b>
 *   - Starting offset for a partial file get.
 *
 *   |html <b>
 *   partial_end_offset
 *   |html </b>
 *   - Ending offset for a partial file get.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the "transfer" operation is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_partial_third_party_transfer returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_client/globus_ftp_client_extended_third_party_transfer
 * NAME
 *   globus_ftp_client_extended_third_party_transfer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_extended_third_party_transfer
 *   |html </b>
 *   |html <i>
 *   handle
 *   source_url
 *   source_attr
 *   eret_alg_str
 *   dest_url
 *   dest_attr
 *   eret_alg_str
 *   restart
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to transfer a file between two FTP servers
 *   with server-side processing.
 *
 *   When the transfer is completed or if the transfer is aborted, the
 *   script will be invoked with the final status of the transfer.
 *
 *   This function differs from the globus_ftp_client_third_party_transfer
 *   command by allowing the user to invoke server-side data processing
 *   algorithms. GridFTP servers may support algorithms for data reduction
 *   or other customized data storage requirements. There is no client-side
 *   verification done on the algorithm string provided by the user. If the
 *   server does not understand the requested algorithm, the transfer will
 *   fail.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - An FTP client handle to use for the transfer operation.
 *
 *   |html <b>
 *   source_url
 *   |html </b>
 *   - The URL to transfer. The URL may be an FTP or GSIFTP URL.
 *
 *   |html <b>
 *   source_attr
 *   |html </b>
 *   - Attributes for the souce URL.
 *
 *   |html <b>
 *   eret_alg_str
 *   |html </b>
 *   - The ERET algorithm string. This string contains information needed
 *   to invoke a server-specific data reduction algorithm on the file being
 *   retrieved.
 *
 *   |html <b>
 *   dest_url
 *   |html </b>
 *   - The destination URL for the transfer. The URL may be an FTP or
 *   GSIFTP URL.
 *
 *   |html <b>
 *   dest_attr
 *   |html </b>
 *   - Attributes for the destination URL.
 *
 *   |html <b>
 *   eret_alg_str
 *   |html </b>
 *   - The ERET algorithm string. This string contains information needed
 *   to invoke a server-specific data reduction algorithm on the file being
 *   retrieved.
 *
 *   |html <b>
 *   restart
 *   |html </b>
 *   - A restart marker.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the "transfer" operation is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_extended_third_party_transfer returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// Modification Time
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_modification_time
 * NAME
 *   globus_ftp_client_modification_time
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_modification_time
 *   |html </b>
 *   |html <i>
 *   handle
 *   url
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to request the modification time of a file from
 *   an FTP server.
 *
 *   When the modification time request is completed or aborted, the
 *   script will be invoked with the final status of the operation. If the
 *   callback script is returned without an error, the modification time
 *   will be stored in the modification_time parameter of the callback
 *   script.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - FTP client handle to use for the operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL to store the data to. The URL may be FTP or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes for this file transfer.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the operation is completed. The command will be
 *   executed exactly once in global scope. If an error occurs while
 *   executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_modification_time returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_modification_time $handle $url
 *   $operation_attr { callback_script callback_arg } ] result  
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



////////////////////////////////////////////////////////////////////
//
// Chmod
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_chmod
 * NAME
 *   globus_ftp_client_chmod
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_chmod
 *   |html </b>
 *   |html <i>
 *   u_handle
 *   url
 *   mode
 *   attr
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to change permissions on a file.
 *
 *   When the response to the chmod request has been received the
 *   script will be invoked with the result of the operation.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   u_handle
 *   |html </b>
 *   - An FTP client handle to use for the chmod operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL for the file to delete. The URL may be a FTP
 *   or GSIFTP URL.
 *
 *   |html <b>
 *   mode
 *   |html </b>
 *   - The file mode to change to.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes to use for this operation.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the command is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_chmod returns 0.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_chmod $handle $url $mode $attr
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



////////////////////////////////////////////////////////////////////
//
// Cksm
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_cksm
 * NAME
 *   globus_ftp_client_cksm
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_cksm
 *   |html </b>
 *   |html <i>
 *   u_handle
 *   url
 *   attr
 *   offset
 *   length
 *   algorithm
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to request the checksum of a file from an
 *   FTP server.
 *
 *   When the checksum request is completed or aborted, the script will be
 *   invoked with the final status of the operation. If the callback is
 *   returned without an error, the checksum will be stored in the
 *   cksm parameter of the callback script.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   u_handle
 *   |html </b>
 *   - An FTP client handle to use for the chmod operation.
 *
 *   |html <b>
 *   url
 *   |html </b>
 *   - The URL for the file to delete. The URL may be a FTP
 *   or GSIFTP URL.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Attributes to use for this operation.
 *
 *   |html <b>
 *   offset
 *   |html </b>
 *   - File offset to start calculating checksum.
 *
 *   |html <b>
 *   length
 *   |html </b>
 *   - Length of data to read from the starting offset.
 *   Use -1 to read the entire file.
 *
 *   |html <b>
 *   algorithm
 *   |html </b>
 *   - Currently the command only supports MD5.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the command is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_cksm returns 0.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_cksm $handle $url $operation_attr
 *   $offset $length $algorithm { callback_script callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// Abort
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_abort
 * NAME
 *   globus_ftp_client_abort
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_abort
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to abort the operation currently in progress.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Handle which to abort.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_abort returns 0.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_abort $handle ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/




////////////////////////////////////////////////////////////////////
//
// FTP OPERATION ATTRIBUTES
//
////////////////////////////////////////////////////////////////////



/****f* globus_ftp_client/globus_ftp_client_operationattr_init
 * NAME
 *   globus_ftp_client_operationattr_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_init
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize an FTP client attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The new attribute set.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_init returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_init $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_destroy
 * NAME
 *   globus_ftp_client_operationattr_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_destroy
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a FTP client attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The new attribute set.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_destroy $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_operationattr_init.
 *****/


/****f* globus_ftp_client/globus_ftp_client_operationattr_copy
 * NAME
 *   globus_ftp_client_operationattr_copy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_copy
 *   |html </b>
 *   |html <i>
 *   dest
 *   src
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create a duplicate of an attribute set.
 *
 *   The duplicated attribute set has a deep copy of all data in the
 *   attribute set, so the original may be destroyed, while the copy
 *   is still valid.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   dest
 *   |html </b>
 *   - The attribute set to be initialized to the same values as src.
 *
 *   |html <b>
 *   src
 *   |html </b>
 *   - The original attribute set to duplicate.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_copy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_copy $dest $src ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// Parallelism
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_operationattr_set_parallelism
 * NAME
 *   globus_ftp_client_operationattr_set_parallelism
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_parallelism
 *   |html </b>
 *   |html <i>
 *   attr
 *   parallelism
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the parallelism attribute for an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to control the level of parallelism to be
 *   used on an extended block mode file transfer. Currently, only a "fixed"
 *   parallelism level is supported. This is interpreted by the FTP server as
 *   the number of parallel data connections to be allowed for each stripe of
 *   data. Currently, only the "fixed" parallelism type is.
 *
 *   This attribute is ignored in stream mode.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query or modify.
 *
 *   |html <b>
 *   parallelism
 *   |html </b>
 *   - The value of parallelism attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_parallelism returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_parallelism $attr
 *   $parallelism ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gsiftp_control_parallelism_t,
 *   globus_ftp_client_operationattr_set_layout,
 *   globus_ftp_client_operationattr_set_mode.
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_parallelism
 * NAME
 *   globus_ftp_client_operationattr_get_parallelism
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_parallelism
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the parallelism attribute from an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to control the level of parallelism to be
 *   used on an extended block mode file transfer. Currently, only a "fixed"
 *   parallelism level is supported. This is interpreted by the FTP server as
 *   the number of parallel data connections to be allowed for each stripe of
 *   data. Currently, only the "fixed" parallelism type is.
 *
 *   This attribute is ignored in stream mode.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query or modify.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_parallelism returns 0 and
 *   the parallelism attribute. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_get_parallelism $attr ] 
 *   result parallelism
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gsiftp_control_parallelism_t,
 *   globus_ftp_client_operationattr_set_layout,
 *   globus_ftp_client_operationattr_set_mode.
 *****/


////////////////////////////////////////////////////////////////////
//
// Striped Data Movement
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_operationattr_set_striped
 * NAME
 *   globus_ftp_client_operationattr_set_striped
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_striped
 *   |html </b>
 *   |html <i>
 *   attr
 *   striped
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the striped attribute for an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to force the client library to used
 *   the FTP commands to do a striped data transfer, even when the user
 *   has not requested a specific file layout via the layout attribute.
 *   This is useful when transferring files between servers which use the
 *   server side processing commands ERET or ESTO to transform data and
 *   send it to particular stripes on the destination server.
 *
 *   The layout attribute is used only when the data is being stored the
 *   server (on a put or 3rd party transfer). This attribute is ignored
 *   for stream mode data transfers.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   striped
 *   |html </b>
 *   - The value of striped attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_striped returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_striped $attr striped ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_operationattr_set_parallelism,
 *   globus_ftp_client_operationattr_set_layout,
 *   globus_ftp_client_operationattr_set_mode.
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_striped
 * NAME
 *   globus_ftp_client_operationattr_get_striped
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_striped
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the striped attribute from an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to force the client library to used
 *   the FTP commands to do a striped data transfer, even when the user
 *   has not requested a specific file layout via the layout attribute.
 *   This is useful when transferring files between servers which use
 *   the server side processing commands ERET or ESTO to transform data
 *   and send it to particular stripes on the destination server.
 *
 *   The layout attribute is used only when the data is being stored the
 *   server (on a put or 3rd party transfer). This attribute is ignored
 *   for stream mode data transfers.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_striped returns 0 and
 *   the striped attribute. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_get_striped $attr ]
 *   result striped
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_client_operationattr_set_parallelism,
 *   globus_ftp_client_operationattr_set_layout,
 *   globus_ftp_client_operationattr_set_mode.
 *****/


////////////////////////////////////////////////////////////////////
//
// Striped File Layout
//
////////////////////////////////////////////////////////////////////



/****f* globus_ftp_client/globus_ftp_client_operationattr_set_layout
 * NAME
 *   globus_ftp_client_operationattr_set_layout
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_layout
 *   |html </b>
 *   |html <i>
 *   attr
 *   layout
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the layout attribute for an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to control the layout of a file being
 *   transfered to a striped Grid-FTP server. The striping layout defines
 *   what regions of a file will be stored on each stripe of a
 *   multiple-striped FTP server.
 *
 *   The layout attribute is used only when the data is being stored on
 *   the server (on a put or 3rd party transfer). This attribute is ignored
 *   for stream mode data transfers.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   layout
 *   |html </b>
 *   - The value of layout attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_striped returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_striped $attr striped ]
 *   result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_layout_t,
 *   globus_ftp_client_operationattr_get_layout,
 *   globus_ftp_client_operationattr_set_parallelism,
 *   globus_ftp_client_operationattr_set_mode.
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_layout
 * NAME
 *   globus_ftp_client_operationattr_get_layout
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_layout
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the layout attribute for an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to control the layout of a file being
 *   transfered to a striped Grid-FTP server. The striping layout defines
 *   what regions of a file will be stored on each stripe of a
 *   multiple-striped FTP server.
 *
 *   The layout attribute is used only when the data is being stored on
 *   the server (on a put or 3rd party transfer). This attribute is ignored
 *   for stream mode data transfers.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_layout returns 0 and
 *   the layout attribute. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_get_layout $attr ]
 *   result layout
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_layout_t,
 *   globus_ftp_client_operationattr_set_layout,
 *   globus_ftp_client_operationattr_set_mode.
 *****/


////////////////////////////////////////////////////////////////////
//
// TCP Buffer
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_operationattr_set_tcp_buffer
 * NAME
 *   globus_ftp_client_operationattr_set_tcp_buffer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_tcp_buffer
 *   |html </b>
 *   |html <i>
 *   attr
 *   tcp_buffer
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the TCP buffer attribute for an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to control the TCP buffer size used for
 *   all data channels used in a file transfer. The size of the TCP buffer
 *   can make a significant impact on the performance of a file transfer.
 *   The user may set the buffer to either a system-dependent default value,
 *   or to a fixed value.
 *
 *   The actual implementation of this attribute is designed to be as widely
 *   interoperable as possible. In addition to supporting the SBUF command
 *   described in the GridFTP protocol extensions document, it also supports
 *   other commands and site commands which are used by other servers to set
 *   TCP buffer sizes. These are:
 *
 *   |html <ul>
 *   |html <li>SITE RETRBUFSIZE</li>
 *   |html <li>SITE RBUFSZ</li>
 *   |html <li>SITE RBUFSIZ</li>
 *   |html <li>SITE STORBUFIZE</li>
 *   |html <li>SITE SBUFSZ</li>
 *   |html <li>SITE SBUFSIZ</li>
 *   |html <li>SITE BUFSIZE</li>
 *   |html </ul>
 *
 *   This attribute is affects any type of data transfer done with the
 *   FTP client library.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   tcp_buffer
 *   |html </b>
 *   - The value of tcp_buffer attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_tcp_buffer returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_tcp_buffer $attr
 *   $tcp_buffer ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gsiftp_control_tcpbuffer_t
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_tcp_buffer
 * NAME
 *   globus_ftp_client_operationattr_get_tcp_buffer
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_tcp_buffer
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the TCP buffer attribute for an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to control the TCP buffer size used for
 *   all data channels used in a file transfer. The size of the TCP buffer
 *   can make a significant impact on the performance of a file transfer.
 *   The user may set the buffer to either a system-dependent default value,
 *   or to a fixed value.
 *
 *   The actual implementation of this attribute is designed to be as widely
 *   interoperable as possible. In addition to supporting the SBUF command
 *   described in the GridFTP protocol extensions document, it also supports
 *   other commands and site commands which are used by other servers to set
 *   TCP buffer sizes. These are:
 *
 *   |html <ul>
 *   |html <li>SITE RETRBUFSIZE</li>
 *   |html <li>SITE RBUFSZ</li>
 *   |html <li>SITE RBUFSIZ</li>
 *   |html <li>SITE STORBUFIZE</li>
 *   |html <li>SITE SBUFSZ</li>
 *   |html <li>SITE SBUFSIZ</li>
 *   |html <li>SITE BUFSIZE</li>
 *   |html </ul>
 *
 *   This attribute is affects any type of data transfer done with the
 *   FTP client library.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_tcp_buffer returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_get_tcp_buffer $attr ]
 *   result tcp_buffer
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_gsiftp_control_tcpbuffer_t
 *****/


////////////////////////////////////////////////////////////////////
//
// File Type
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_operationattr_set_type
 * NAME
 *   globus_ftp_client_operationattr_set_type
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_type
 *   |html </b>
 *   |html <i>
 *   attr
 *   type
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the file representation type attribute
 *   for an FTP client attribute set.
 *
 *   This attribute allows the user to choose the file type used for an
 *   FTP file transfer. The file may be transferred as either ASCII or
 *   a binary image.
 *
 *   When transferring an ASCII file, the data will be transformed in
 *   the following way:
 *
 *   |html <ul>
 *   |html <li>the high-order bit will be set to zero.</li>
 *   |html <li>end-of-line characters will be converted to a CRLF pair for
 *   the data transfer, and then converted to native format before being
 *   returned to the user's data callbacks.
 *   |html </ul>
 *
 *   The default type for the ftp client libary is binary.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   type
 *   |html </b>
 *   - The value of type attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_type returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_type $attr $type ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_type_t
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_type
 * NAME
 *   globus_ftp_client_operationattr_get_type
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_type
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the file representation type attribute
 *   for an FTP client attribute set.
 *
 *   This attribute allows the user to choose the file type used for an
 *   FTP file transfer. The file may be transferred as either ASCII or
 *   a binary image.
 *
 *   When transferring an ASCII file, the data will be transformed in
 *   the following way:
 *
 *   |html <ul>
 *   |html <li>the high-order bit will be set to zero.</li>
 *   |html <li>end-of-line characters will be converted to a CRLF pair for
 *   the data transfer, and then converted to native format before being
 *   returned to the user's data callbacks.
 *   |html </ul>
 *
 *   The default type for the ftp client libary is binary.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_type returns 0 and
 *   the value of type attribute. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_get_type $attr ]
 *   result type
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_type_t,
 *****/


////////////////////////////////////////////////////////////////////
//
// Transfer Mode
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_operationattr_set_mode
 * NAME
 *   globus_ftp_client_operationattr_set_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   mode
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the file transfer mode attribute for
 *   an FTP client attribute set.
 *
 *   This attribute allows the user to choose the data channel protocol
 *   used to transfer a file. There are two modes supported by this library:
 *   |html <ul>
 *   |html <li>stream mode
 *   |html <li>extended block mode
 *   |html </ul>
 *
 *   Stream mode is a file transfer mode where all data is sent over a
 *   single TCP socket, without any data framing. In stream mode, data will
 *   arrive in sequential order. This mode is supported by nearly all FTP
 *   servers.
 *
 *   Extended block mode is a file transfer mode where data can be sent
 *   over multiple parallel connections and to multiple data storage nodes
 *   to provide a high-performance data transfer. In extended block mode,
 *   data may arrive out-of-order.
 *   |html <font color="Red"> ASCII type files
 *   are not supported in extended block mode
 *   |html </font>. 
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   mode
 *   |html </b>
 *   - The value of mode attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_mode returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_mode $attr $type ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_type_t
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_set_list_uses_data_mode
 * NAME
 *   globus_ftp_client_operationattr_set_list_uses_data_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_list_uses_data_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   list_uses_data_mode
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set whether or not list data will use the
 *   current data mode.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   list_uses_data_mode
 *   |html </b>
 *   - Boolean value.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_list_uses_data_mode
 *   returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_list_uses_data_mode
 *   $attr $GLOBUS_TRUE ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/




/****f* globus_ftp_client/globus_ftp_client_operationattr_get_mode
 * NAME
 *   globus_ftp_client_operationattr_get_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the file transfer mode attribute for
 *   an FTP client attribute set.
 *
 *   This attribute allows the user to choose the data channel protocol
 *   used to transfer a file. There are two modes supported by this library:
 *   |html <ul>
 *   |html <li>stream mode
 *   |html <li>extended block mode
 *   |html </ul>
 *
 *   Stream mode is a file transfer mode where all data is sent over a
 *   single TCP socket, without any data framing. In stream mode, data will
 *   arrive in sequential order. This mode is supported by nearly all FTP
 *   servers.
 *
 *   Extended block mode is a file transfer mode where data can be sent
 *   over multiple parallel connections and to multiple data storage nodes
 *   to provide a high-performance data transfer. In extended block mode,
 *   data may arrive out-of-order.
 *   |html <font color="Red"> ASCII type files
 *   are not supported in extended block mode
 *   |html </font>. 
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_mode returns 0 and
 *   the value of mode attribute. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_get_mode $attr ]
 *   result mode
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_type_t,
 *   globus_ftp_client_operationattr_set_parallelism,
 *   globus_ftp_client_operationattr_set_layout.
 *****/


/****f* globus_ftp_client/globus_ftp_client_operationattr_get_list_uses_data_mode
 * NAME
 *   globus_ftp_client_operationattr_get_list_uses_data_mode
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_list_uses_data_mode
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get whether or not list data will use the
 *   current data mode.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_list_uses_data_mode
 *   returns 0 and the value of list_uses_data_mode. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_get_list_uses_data_mode
 *   $attr ] result list_uses_data_mode
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



////////////////////////////////////////////////////////////////////
//
// Authorization
//
////////////////////////////////////////////////////////////////////

/****f* globus_ftp_client/globus_ftp_client_operationattr_set_authorization
 * NAME
 *   globus_ftp_client_operationattr_set_authorization
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_authorization
 *   |html </b>
 *   |html <i>
 *   attr
 *   credential
 *   user
 *   password
 *   account
 *   subject
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the authorization attribute for an FTP
 *   client attribute set.
 *
 *   This attribute allows the user to pass authentication information
 *   to the FTP client library. This information is used to authenticate
 *   with the FTP server.
 *
 *   The Globus FTP client library supports authentication using either
 *   the GSSAPI, or standard plaintext username and passwords. The type
 *   of authentication is determined by the URL scheme which is used for
 *   the individual get, put, or 3rd party transfer calls.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   credential
 *   |html </b>
 *   - The credential to use for authenticating with a GSIFTP server.
 *   This may be GSS_C_NO_CREDENTIAL to use the default credential.
 *
 *   |html <b>
 *   user
 *   |html </b>
 *   - The user name to send to the FTP server. When doing a GSIFTP transfer,
 *   this may be set to NULL, and the default globusmap entry for the user's
 *   GSI identity will be used.
 *
 *   |html <b>
 *   password
 *   |html </b>
 *   - The password to send to the FTP server. When doing a GSIFTP transfer,
 *   this may be set to NULL.
 *
 *   |html <b>
 *   account
 *   |html </b>
 *   - The account to use for the data transfer. Most FTP servers do not
 *   require this.
 *
 *   |html <b>
 *   subject
 *   |html </b>
 *   - The subject name of the FTP server. This is only used when doing a
 *   GSIFTP transfer, and then only when the security subject name does not
 *   match the hostname of the server (ie, when the server is being run by
 *   a user).
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_authorization returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_authorization
 * NAME
 *   globus_ftp_client_operationattr_get_authorization
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_authorization
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the authorization attribute for an FTP
 *   client attribute set.
 *
 *   This attribute allows the user to pass authentication information
 *   to the FTP client library. This information is used to authenticate
 *   with the FTP server.
 *
 *   The Globus FTP client library supports authentication using either
 *   the GSSAPI, or standard plaintext username and passwords. The type
 *   of authentication is determined by the URL scheme which is used for
 *   the individual get, put, or 3rd party transfer calls.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_authorization returns 0
 *   and the following variables:
 *   |html <ul><b>
 *   credential
 *   |html </b>
 *   - The credential to use for authenticating with a GSIFTP server.
 *   This may be GSS_C_NO_CREDENTIAL to use the default credential.
 *
 *   |html <b>
 *   user
 *   |html </b>
 *   - The user name to send to the FTP server. When doing a GSIFTP transfer,
 *   this may be set to NULL, and the default globusmap entry for the user's
 *   GSI identity will be used.
 *
 *   |html <b>
 *   password
 *   |html </b>
 *   - The password to send to the FTP server. When doing a GSIFTP transfer,
 *   this may be set to NULL.
 *
 *   |html <b>
 *   account
 *   |html </b>
 *   - The account to use for the data transfer. Most FTP servers do not
 *   require this.
 *
 *   |html <b>
 *   subject
 *   |html </b>
 *   - The subject name of the FTP server. This is only used when doing a
 *   GSIFTP transfer, and then only when the security subject name does not
 *   match the hostname of the server (ie, when the server is being run by
 *   a user).
 *   |html </ul>
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// Data Channel Authentication
//
////////////////////////////////////////////////////////////////////



/****f* globus_ftp_client/globus_ftp_client_operationattr_set_dcau
 * NAME
 *   globus_ftp_client_operationattr_set_dcau
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_dcau
 *   |html </b>
 *   |html <i>
 *   attr
 *   dcau
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the data channel authentication attribute
 *   for an FTP client attribute set.
 *
 *   Data channel authentication is a GridFTP extension, and may not be
 *   supported by all servers. If a server supports it, then the default
 *   is to delegate a credential to the server, and authenticate all data
 *   channels with that delegated credential.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   dcau
 *   |html </b>
 *   - The value of mode attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_dcau returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_dcau $attr $dcau ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_dcau_t
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_dcau
 * NAME
 *   globus_ftp_client_operationattr_get_dcau
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_dcau
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the data channel authentication attribute
 *   for an FTP client attribute set.
 *
 *   Data channel authentication is a GridFTP extension, and may not be
 *   supported by all servers. If a server supports it, then the default
 *   is to delegate a credential to the server, and authenticate all data
 *   channels with that delegated credential.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_dcau returns 0 and
 *   the value of data channel authentication attribute. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_get_dcau $attr ] result dcau
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_dcau_t
 *****/

////////////////////////////////////////////////////////////////////
//
// Data Channel Protection
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_operationattr_set_data_protection
 * NAME
 *   globus_ftp_client_operationattr_set_data_protection
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_data_protection
 *   |html </b>
 *   |html <i>
 *   attr
 *   protection
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the data channel protection attribute
 *   for an FTP client attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   protection
 *   |html </b>
 *   - The value of data channel protection attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_data_protection returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_data_protection $attr
 *   $protection ] result
 *
 * BUGS
 *   |html <font color="Red">
 *   Only safe and private protection levels are supported by GSIFTP.
 *   |html </font>
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_protection_t
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_data_protection
 * NAME
 *   globus_ftp_client_operationattr_get_data_protection
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_data_protection
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the data channel protection attribute
 *   for an FTP client attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_data_protection returns 0
 *   and the value of data channel protection attribute. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_get_data_protection $attr ]
 *   result protection
 *
 * BUGS
 *   |html <font color="Red">
 *   Only safe and private protection levels are supported by GSIFTP.
 *   |html </font>
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_protection_t
 *****/


////////////////////////////////////////////////////////////////////
//
// Control Channel Protection
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_operationattr_set_control_protection
 * NAME
 *   globus_ftp_client_operationattr_set_control_protection
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_control_protection
 *   |html </b>
 *   |html <i>
 *   attr
 *   protection
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the control channel protection attribute
 *   for an FTP client attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   protection
 *   |html </b>
 *   - The value of control channel protection attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_control_protection returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_control_protection $attr
 *   $protection ] result
 *
 * BUGS
 *   |html <font color="Red">
 *   The clear and safe protection levels are treated identically, with the
 *   client integrity checking all commands. The confidential and private
 *   protection levels are treated identically, with the client encrypting
 *   all commands.
 *   |html </font>
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_protection_t
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_control_protection
 * NAME
 *   globus_ftp_client_operationattr_get_control_protection
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_control_protection
 *   |html </b>
 *   |html <i>
 *   attr
 *   protection
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the control channel protection attribute
 *   for an FTP client attribute set.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_control_protection returns 0
 *   and The value of control channel protection attribute. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_get_control_protection $attr ]
 *   result protection
 *
 * BUGS
 *   |html <font color="Red">
 *   The clear and safe protection levels are treated identically, with the
 *   client integrity checking all commands. The confidential and private
 *   protection levels are treated identically, with the client encrypting
 *   all commands.
 *   |html </font>
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_ftp_control_protection_t
 *****/


////////////////////////////////////////////////////////////////////
//
// Append
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_operationattr_set_append
 * NAME
 *   globus_ftp_client_operationattr_set_append
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_append
 *   |html </b>
 *   |html <i>
 *   attr
 *   append
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to set the append attribute for an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to append to a file on an FTP server,
 *   instead of replacing the existing file when doing a globus_ftp_client_put
 *   or globus_ftp_client_transfer.
 *
 *   This attribute is ignored on the retrieving side of a transfer, or a
 *   globus_ftp_client_get.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   append
 *   |html </b>
 *   - The value of append attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_append returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_append $attr TRUE ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_append
 * NAME
 *   globus_ftp_client_operationattr_get_append
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_append
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get the append attribute for an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to append to a file on an FTP server,
 *   instead of replacing the existing file when doing a globus_ftp_client_put
 *   or globus_ftp_client_transfer.
 *
 *   This attribute is ignored on the retrieving side of a transfer, or a
 *   globus_ftp_client_get.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query.
 *
 *   |html <b>
 *   append
 *   |html </b>
 *   - The value of append attribute.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_append returns 0 and
 *   The value of append attribute. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_append $attr ] result append
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/


////////////////////////////////////////////////////////////////////
//
// Read into a Single Buffer
//
////////////////////////////////////////////////////////////////////


/****f* globus_ftp_client/globus_ftp_client_operationattr_set_read_all
 * NAME
 *   globus_ftp_client_operationattr_set_read_all
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_set_read_all
 *   |html </b>
 *   |html <i>
 *   attr
 *   read_all
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to set the read_all attribute for an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to pass in a single buffer to receive
 *   all of the data for the current transfer. This buffer must be large
 *   enough to hold all of the data for the transfer. Only one buffer may
 *   be registered with globus_ftp_client_register_read when this attribute
 *   is used for a get.
 *
 *   In extended block mode, this attribute will cause data to be stored
 *   directly into the buffer from multiple streams without any extra data
 *   copies.
 *
 *   If the user sets the script to a non-null value, this script will be
 *   called whenever an intermediate sub-section of the data is received
 *   into the buffer.
 *
 *   This attribute is ignored for globus_ftp_client_put or
 *   globus_ftp_client_third_party_transfer operations.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to modify.
 *
 *   |html <b>
 *   read_all
 *   |html </b>
 *   - The value of read_all attribute.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked when the command is completed.
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_set_read_all returns 0.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_ftp_client_operationattr_set_read_all $attr TRUE
 *   { callback callback_arg } ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/



/****f* globus_ftp_client/globus_ftp_client_operationattr_get_read_all
 * NAME
 *   globus_ftp_client_operationattr_get_read_all
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_ftp_client
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_ftp_client_operationattr_get_read_all
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to get the read_all attribute for an FTP client
 *   attribute set.
 *
 *   This attribute allows the user to pass in a single buffer to receive
 *   all of the data for the current transfer. This buffer must be large
 *   enough to hold all of the data for the transfer. Only one buffer may
 *   be registered with globus_ftp_client_register_read when this attribute
 *   is used for a get.
 *
 *   In extended block mode, this attribute will cause data to be stored
 *   directly into the buffer from multiple streams without any extra data
 *   copies.
 *
 *   If the user sets the script to a non-null value, this script will be
 *   called whenever an intermediate sub-section of the data is received
 *   into the buffer.
 *
 *   This attribute is ignored for globus_ftp_client_put or
 *   globus_ftp_client_third_party_transfer operations.
 *
 * PARAMETERS
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute set to query.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_ftp_client_operationattr_get_read_all returns 0 and
 *   the value of read_all attribute.
 *   On error, this command will raise an error and must be caught using
 *   the appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *****/
