/* -*- mode: c++; c-basic-offset: 2; -*- */

#ifdef SWIG

%include "../../config.h"

%include "tcl-typemaps.i"

%inline %{
  
#include "tcl_vararg.h"
#include "xio_driver.h"  
#include "xio_driver_file.h"
#include "xio_driver_gsi.h"
#include "xio_driver_tcp.h"
#include "xio_driver_udp.h"
#include "xio_driver_http.h"
  
  %}

%ignore globus_xio_driver_load;
%ignore globus_xio_driver_unload;

%ignore globus_xio_attr_cntl;
%ignore globus_xio_handle_cntl;
%ignore globus_xio_server_cntl;
%ignore globus_xio_data_descriptor_cntl;

%ignore globus_xio_read;
%ignore globus_xio_write;

%ignore globus_xio_readv;
%ignore globus_xio_writev;

%ignore globus_xio_register_read;
%ignore globus_xio_register_write;

%ignore globus_xio_register_readv;
%ignore globus_xio_register_writev;

#if HAVE_GLOBUS_XIO_H

%include "swig/globus_xio.i"

%{

#include "../../config.h"

#include <tcl.h>

#include "tcl-callback.h"

#include "common/sgdebug.h"
#include "common/sginit.h"  

#define MAX_HASH_KEY    64
#define MAX_HASH_VALUE  64

#define MAX_DRIVER_NAME 64
#define MAX_UNIQUE_NAME 64
  
/* 
 * 4 indicates that SWIG sees 4 arguments before getting the actual
 * variable arguments ?...?
 * i.e., globus_xio_attr_cntl attr driver cmd
 */
#define ATTR_ARG_LENGTH 4

  /* Callback-related typedef */
  typedef struct callbackData_ {
    long           total_bytes;
    int            iovec_cnt;
    void*          buffer;
    char           unique_name[MAX_UNIQUE_NAME];
  } callbackData;

  struct swig_type_info *ty;
  
  /*************************************************
   * To expose SWIG_Tcl_TypeQuery() at Globus scope
   *************************************************/ 
  swig_type_info *
    GlobusXio_SWIG_TypeQuery(const char *ty)
    {
      return SWIG_TypeQuery(ty);
    }

  /**************************************************
   * To expose SWIG_Tcl_ConvertPtr() at Globus scope
   **************************************************/ 
  int
    GlobusXio_SWIG_ConvertPtr(Tcl_Interp *interp,
                              Tcl_Obj *oc,
                              void **ptr,
                              swig_type_info *type,
                              int flags)
    {
      return SWIG_Tcl_ConvertPtr(interp, oc, ptr, type, flags); 
    }

  /******************************************************
   * To expose SWIG_Tcl_NewInstanceObj() at Globus scope
   ******************************************************/   
  Tcl_Obj *
    GlobusXio_SWIG_NewInstanceObj(Tcl_Interp *interp,
                                 void *thisvalue,
                                 swig_type_info *type,
                                 int flags)
    {
      return SWIG_Tcl_NewInstanceObj(interp, thisvalue, type, flags);
    }
  
  /* 
   * :TODO: We need to make thread-safe hash tables 
   */

  /* Driver type hash table */
  static Tcl_HashTable tcl_globus_xio_driver_type_hashtbl;

  /* Driver file function map hash table */
  static Tcl_HashTable tcl_globus_xio_driver_func_map_hashtbl;

  /* Forward references */
  static void GlobusXioRemoveFunctionMapHashTbl(char *);

  static void
    GlobusXioInsertDriverFuncMapHashTbl(char *, XIODriverFunctionMap_type *);

  /**************************************************************************
   *
   * GLOBUS_XIO_DRIVER_LOAD() & GLOBUS_XIO_DRIVER_UNLOAD()
   *
   **************************************************************************/

  /****if* globus_xio/GlobusXioInsertDriverTypeHashTbl
   * NAME
   *  GlobusXioInsertDriverTypeHashTbl
   *
   * SYNOPSIS
   *    |html <b>
   *    static void 
   *    |html </b>
   *    GlobusXioInsertDriverTypeHashTbl
   *    |html <b>
   *    (
   *    |html </b>
   *    |html <b>
   *    char *
   *    |html </b>
   *    |html <i>
   *    driver_name
   *    |html </i>
   *    ,
   *    |html <b>
   *    globus_xio_driver_t
   *    |html </b>
   *    |html <i>
   *	driver_type
   *    |html </i>
   *	);
   *    |html </b>
   *
   * DESCRIPTION
   *    Hash key = address of driver structure converted into hex string
   *    Hash value = driver name
   *
   * RETURN VALUE
   *    Return value
   *
   * BUGS
   *    Bugs
   *
   * TODO
   *    Todo
   *
   * SEE ALSO
   *    See also
   ****/
  static void
    GlobusXioInsertDriverTypeHashTbl(char * driver_name,
                                     globus_xio_driver_t driver_type)
    {
      char key[MAX_HASH_KEY];

      Tcl_HashEntry * entry_ptr = NULL;
      int new_entry;
      char * value;
      unsigned int value_len;

      static char func_name[] = "GlobusXioInsertDriverTypeHashTbl";

      SGDEBUG_LVL_FMT_1( 10, "Entering %s", func_name );
      
      memset(key, '\0', MAX_HASH_KEY);
      sprintf(key, "%p", driver_type);

      SGDEBUG_LVL_FMT_1( 10, "key = %s", key );
      SGDEBUG_LVL_FMT_1( 10, "driver_name = %s", driver_name );

      entry_ptr = Tcl_CreateHashEntry( &tcl_globus_xio_driver_type_hashtbl, 
                                       key,
                                       &new_entry );
      
      value_len = strlen(driver_name);

      /*
       * Allocate memory for holding hash value
       */
      value = (char *)ckalloc(sizeof(char) * value_len);
      
      memset(value, '\0', value_len);
      strcpy(value, driver_name);

      Tcl_SetHashValue(entry_ptr, (ClientData)value);

      /*
       * Insert appropriate driver into Tcl function map hash table
       */
      if ( strcmp(driver_name, "file") == 0 )
        {
          GlobusXioInsertDriverFuncMapHashTbl
            ("file", (XIODriverFunctionMap_type *)DriverFileInit());          
        }
      else if ( strcmp(driver_name, "gsi") == 0 )
        {
          GlobusXioInsertDriverFuncMapHashTbl
            ("gsi", (XIODriverFunctionMap_type *)DriverGSIInit());
        }
      else if ( strcmp(driver_name, "tcp") == 0 )
        {
          GlobusXioInsertDriverFuncMapHashTbl
            ("tcp", (XIODriverFunctionMap_type *)DriverTCPInit());
        }
      else if ( strcmp(driver_name, "udp") == 0 )
        {
          GlobusXioInsertDriverFuncMapHashTbl
            ("udp", (XIODriverFunctionMap_type *)DriverUDPInit());
        }
      else if ( strcmp(driver_name, "http") == 0 )
        {
          GlobusXioInsertDriverFuncMapHashTbl
            ("http", (XIODriverFunctionMap_type *)DriverHTTPInit());
        }
      else
        {
          /*
           * :TODO: Invalid driver name
           */
        }
      
      SGDEBUG_LVL_FMT_1( 10, "Leaving %s", func_name );
    }

  /****if* globus_xio/GlobusXioRemoveDriverTypeHashTbl
   * NAME
   *  GlobusXioRemoveDriverTypeHashTbl
   *
   * SYNOPSIS
   *    |html <b>
   *    static void 
   *    |html </b>
   *    GlobusXioRemoveDriverTypeHashTbl
   *    |html <b>
   *    (
   *    |html </b>
   *    |html <b>
   *    globus_xio_driver_t
   *    |html </b>
   *    |html <i>
   *	driver_type
   *    |html </i>
   *	);
   *    |html </b>
   *
   * DESCRIPTION
   *    Find hash key corresponding to the address of the driver structure
   *    and remove hash value from the hash table.
   *
   * RETURN VALUE
   *    Return value
   *
   * BUGS
   *    Bugs
   *
   * TODO
   *    Todo
   *
   * SEE ALSO
   *    See also
   ****/
  static void
    GlobusXioRemoveDriverTypeHashTbl(globus_xio_driver_t driver_type)
    {
      char key[MAX_HASH_KEY];
      Tcl_HashEntry *entry_ptr = NULL;
      char *data;

      memset(key, '\0', MAX_HASH_KEY);
      sprintf(key, "%p", driver_type);

      SGDEBUG_LVL_FMT_1
        ( 10, "GlobusXioRemoveDriverTypeHashTbl: key = %s", key );
      
      /* Find the entry based on hash key */
      entry_ptr = Tcl_FindHashEntry(&tcl_globus_xio_driver_type_hashtbl, 
                                    key);

      /* Found a match */
      if ( entry_ptr != NULL )
        {
          data = (char *)Tcl_GetHashValue(entry_ptr);
          
          SGDEBUG_LVL_FMT_1
            ( 10, "GlobusXioRemoveDriverTypeHashTbl: value: %s", data );
          
          /* Remove the entry in the function map hash table */
          GlobusXioRemoveFunctionMapHashTbl(data);
          
          /*
           * :TODO: Fix this to prevent memory leaks
           */
#if 0
          ckfree((char *)data);
#endif
          
          /* Delete the entry */
          Tcl_DeleteHashEntry(entry_ptr);
        }
    }


  /* 
   * Hash key = driver name
   * Hash value = driver attr & cntl function map
   */
  static void
    GlobusXioInsertDriverFuncMapHashTbl(char *driver_name,
                                  XIODriverFunctionMap_type *driver_func)
    {
      char key[MAX_HASH_KEY];
      Tcl_HashEntry *entry_ptr = NULL;
      int new_entry;
      XIODriverFunctionMap_type *value;

      memset(key, '\0', MAX_HASH_KEY);
      sprintf(key, "%s", driver_name);
      
      entry_ptr = Tcl_CreateHashEntry
        ( &tcl_globus_xio_driver_func_map_hashtbl, 
          key, 
          &new_entry );

      value = (XIODriverFunctionMap_type *)
        ckalloc(sizeof(XIODriverFunctionMap_type));

      value = driver_func;

      Tcl_SetHashValue(entry_ptr, (ClientData)value);
    }
  
  /*
   * Find hash key corresponding to the address of the driver structure
   * and remove hash value from the hash table.
   */
  static void
    GlobusXioRemoveFunctionMapHashTbl(char* key)
    {
      Tcl_HashEntry *entry_ptr = NULL;
      XIODriverFunctionMap_type *data;

      SGDEBUG_LVL_FMT_1
        ( 10, "GlobusXioRemoveFunctionMapHashTbl: key: %s", key );
      
      entry_ptr = Tcl_FindHashEntry( &tcl_globus_xio_driver_func_map_hashtbl, 
                                     key );

      if ( entry_ptr != NULL )
        {
          data = (XIODriverFunctionMap_type *)Tcl_GetHashValue(entry_ptr);

          SGDEBUG_LVL_FMT_1
            ( 10, "GlobusXioRemoveFunctionMapHashTbl: value: %p", data);

          /*
           * :TODO: Fix this to prevent memory leaks
           */
#if 0
          ckfree((char*)data);
#endif
          
          Tcl_DeleteHashEntry(entry_ptr);
        }
    }

  
  /**************************************************************************
   *
   * GLOBUS_XIO_REGISTER_READ()
   *
   **************************************************************************/

  /****if* globus_xio/GlobusXioRegisterReadInit
   * NAME
   *  GlobusXioRegisterReadInit
   *
   * SYNOPSIS
   *    |html <b>
   *    static void 
   *    |html </b>
   *    GlobusXioRegisterReadInit
   *    |html <b>
   *    (
   *    |html </b>
   *    |html <b>
   *    callbackStruct*
   *    |html </b>
   *    |html <i>
   *	cb
   *    |html </i>
   *    |html <b>
   *    long
   *    |html </b>
   *    |html <i>
   *	buffer_length
   *    |html </i>
   *    |html <b>
   *	);
   *    |html </b>
   *
   * DESCRIPTION
   *    Description
   * 
   * RETURN VALUE
   *    Return value
   *
   * BUGS
   *    Bugs
   *
   * TODO
   *    Todo
   *
   * SEE ALSO
   *    See also
   ****/
  static void
  GlobusXioRegisterReadInit( callbackStruct* cb, long buffer_length )
  {
    /* Allocate memory space for our buffer */
    callbackData* data = 
      (callbackData *)globus_libc_malloc( sizeof(callbackData) );
    
    cb->s_api_data = data;

    data->total_bytes = buffer_length; 

    SGDEBUG_LVL_FMT_1( 20, "GlobusXioRegisterReadInit: data->total_bytes: %d", 
                       data->total_bytes );
    /* :TODO: */
    /* We should not exit if we fail to allocate our buffer */
    /* Must return error message instead of just exit */
    if ( ( data->buffer = 
	   (globus_byte_t *)globus_libc_malloc( sizeof(globus_byte_t) * 
					   buffer_length )) == NULL ) 
    {
      fprintf(stderr, "Memory allocation fails\n");
      exit(1);
    }

    memset( data->buffer, '\0', buffer_length );

    TclGlobusCreateUniqueTCLVariableName( data->unique_name );
  }


  /****if* globus_xio/GlobusXioRegisterReadCleanup
   * NAME
   *  GlobusXioRegisterReadCleanup
   *
   * SYNOPSIS
   *    |html <b>
   *    static void 
   *    |html </b>
   *    GlobusXioRegisterReadCleanup
   *    |html <b>
   *    (
   *    |html </b>
   *    |html <b>
   *    callbackStruct*
   *    |html </b>
   *    |html <i>
   *	cb
   *    |html </i>
   *    |html <b>
   *	);
   *    |html </b>
   *
   * DESCRIPTION
   *    Description
   * 
   * RETURN VALUE
   *    Return value
   *
   * BUGS
   *    Bugs
   *
   * TODO
   *    Todo
   *
   * SEE ALSO
   *    See also
   ****/
  static int
  GlobusXioRegisterReadCleanup( callbackStruct* cb )
  {
    int result;

    callbackData* data = (callbackData*)cb->s_api_data;

    SGDEBUG_LVL_FMT_1( 20, "GlobusXioRegisterReadCleanup: UnsetVar2: %s", 
                       data->unique_name );

    /* Release TCL resources */
    result = Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);
    
    SGDEBUG_LVL_FMT_2
      ( 20, 
        "GlobusXioRegisterReadCleanup: Tcl_UnsetVar2 return value: %d unique_name: %s", 
        result, data->unique_name );

    /* Deallocate memory space */
    if ( data->buffer ) 
      {
        SGDEBUG_LVL_FMT( 10, "GlobusXioRegisterReadCleanup: buffer" );
        globus_libc_free( data->buffer );
      }

    if ( cb->s_api_data )
      {
        SGDEBUG_LVL_FMT( 10, "GlobusXioRegisterReadCleanup: s_api_data" );
        globus_libc_free(cb->s_api_data);
        cb->s_api_data = NULL;
      }

    return 1;
  }

  /****if* globus_xio/GlobusXioRegisterReadPreCallback
   * NAME
   *  GlobusXioRegisterReadPreCallback
   *
   * SYNOPSIS
   *    |html <b>
   *    static void 
   *    |html </b>
   *    GlobusXioRegisterReadPreCallback
   *    |html <b>
   *    (
   *    |html </b>
   *    |html <b>
   *    callbackStruct*
   *    |html </b>
   *    |html <i>
   *	cb
   *    |html </i>
   *    |html <b>
   *	);
   *    |html </b>
   ****/
  static void
  GlobusXioRegisterReadPreCallback( callbackStruct* cb )
  {
    Tcl_Interp* interp;
    Tcl_Obj* obj;
  
    callbackData* data = (callbackData*)(cb->s_api_data);

    interp = cb->s_main_tcl_interp;

    obj = Tcl_NewByteArrayObj((char *)(data->buffer), data->total_bytes);

    Tcl_SetVar2Ex(interp, data->unique_name, NULL, obj, 0);
    SGDEBUG_LVL_FMT_1
      ( 20, 
	"GlobusXioRegisterReadPreCallback: unique_name: %s", 
	data->unique_name );
  }

  /**************************************************************************
   *
   * GLOBUS_XIO_REGISTER_WRITE()
   *
   **************************************************************************/

  /****if* globus_xio/GlobusXioRegisterWriteInit
   * NAME
   *  GlobusXioRegisterWriteInit
   *
   * SYNOPSIS
   *    |html <b>
   *    static void 
   *    |html </b>
   *    GlobusXioRegisterWriteInit
   *    |html <b>
   *    (
   *    |html </b>
   *    |html <b>
   *    callbackStruct*
   *    |html </b>
   *    |html <i>
   *	cb
   *    |html </i>
   *    |html <b>
   *	);
   *    |html </b>
   ****/
  static void
  GlobusXioRegisterWriteInit( callbackStruct* cb, unsigned char* buffer )
  {
    /* Allocate memory space */
    callbackData* data = 
      (callbackData *)globus_libc_malloc( sizeof(callbackData) );

    cb->s_api_data = data;

    data->buffer = buffer; 

    TclGlobusCreateUniqueTCLVariableName( data->unique_name );
  }

  /****if* globus_xio/GlobusXioRegisterWriteCleanup
   * NAME
   *  GlobusXioRegisterWriteCleanup
   *
   * SYNOPSIS
   *    |html <b>
   *    static void 
   *    |html </b>
   *    GlobusXioRegisterWriteCleanup
   *    |html <b>
   *    (
   *    |html </b>
   *    |html <b>
   *    callbackStruct*
   *    |html </b>
   *    |html <i>
   *	cb
   *    |html </i>
   *    |html <b>
   *	);
   *    |html </b>
   ****/
  static int
  GlobusXioRegisterWriteCleanup( callbackStruct* cb )
  {
    int result;

    callbackData* data = (callbackData*)(cb->s_api_data);

    /* Release TCL resources */
    result = Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);

    SGDEBUG_LVL_FMT_2
      ( 20, 
        "GlobusXioRegisterWriteCleanup: Tcl_UnsetVar2 return value: %d unique_name: %s", 
        result, data->unique_name );

    /* Deallocate memory space */
    if ( data->buffer ) 
      {
        SGDEBUG_LVL_FMT( 10, "GlobusXioRegisterWriteCleanup: buffer" );
        globus_libc_free( data->buffer );
      }

    /* Deallocate memory space */
    if ( cb->s_api_data ) 
    {
      globus_libc_free(cb->s_api_data);
      cb->s_api_data = NULL;
    }

    return 1;
  }

  /****if* globus_xio/GlobusXioRegisterWritePreCallback
   * NAME
   *  GlobusXioRegisterWritePreCallback
   *
   * SYNOPSIS
   *    |html <b>
   *    static void 
   *    |html </b>
   *    GlobusXioRegisterWritePreCallback
   *    |html <b>
   *    (
   *    |html </b>
   *    |html <b>
   *    callbackStruct*
   *    |html </b>
   *    |html <i>
   *	cb
   *    |html </i>
   *    |html <b>
   *	);
   *    |html </b>
   ****/
  static void
  GlobusXioRegisterWritePreCallback( callbackStruct* cb )
  {

    Tcl_Interp* interp;
    Tcl_Obj* obj;

    callbackData* data = (callbackData*)(cb->s_api_data);

    SGDEBUG_LVL_FMT_1
      ( 20, 
        "GlobusXioRegisterWritePreCallback: data->total_bytes: %d", 
        data->total_bytes );

    obj = Tcl_NewByteArrayObj( (char *)(data->buffer), data->total_bytes );
  
    interp = cb->s_main_tcl_interp;
    Tcl_SetVar2Ex(interp, data->unique_name, NULL, obj, 0);
    SGDEBUG_LVL_FMT_1
      ( 20, 
	"GlobusXioRegisterWritePreCallback: unique_name: %s", 
	data->unique_name );
  }

  /**************************************************************************
   *
   * GLOBUS_XIO_REGISTER_READV()
   *
   **************************************************************************/

  static void
  GlobusXioRegisterReadVectorInit( callbackStruct* 	cb,
				   int 			iovec_cnt, 
                                   char** 		bufarray )
  {
    int indx, len;
    
    callbackData* data =
      (callbackData *)globus_libc_malloc(sizeof(callbackData));
    
    cb->s_api_data = data;

    data->total_bytes = 0;
    data->iovec_cnt = iovec_cnt;

    data->buffer = (globus_xio_iovec_t *)
      globus_libc_malloc( sizeof(struct iovec) * iovec_cnt );
  
    indx = 0;
    while ( bufarray[indx] )
      {
        len = atoi(bufarray[indx]);
        
        (((globus_xio_iovec_t*)(data->buffer))[indx]).iov_base = 
          (globus_byte_t *)globus_libc_malloc( sizeof(globus_byte_t) * len );
        
        memset((((globus_xio_iovec_t*)(data->buffer))[indx]).iov_base, 
               '\0', len);
        
        (((globus_xio_iovec_t*)(data->buffer))[indx]).iov_len = len;
        
        indx++;
      }
    
    /*
     * Deallocate memory reserved for holding a list of buffer size
     * in typemap(in)
     */
    if ( bufarray ) 
      {
        globus_libc_free(bufarray);
      }

    TclGlobusCreateUniqueTCLVariableName( data->unique_name );
  }

  /**********************************************************/
  /**********************************************************/

  static int
  GlobusXioRegisterReadVectorCleanup( callbackStruct* cb )
  {
    int i;

    callbackData* data = (callbackData*)(cb->s_api_data);

    /* Release TCL resources */
    Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);

    /*
     *Iterate our buffer list to free up memory
     */
    
    SGDEBUG_LVL_FMT_1( 10, "iovec_cnt: %ld", data->iovec_cnt );
    i = 0;
    while ( i < data->iovec_cnt )
    {
      globus_libc_free( (((globus_xio_iovec_t*)(data->buffer))[i]).iov_base );
      SGDEBUG_LVL_FMT( 10, "globus_libc_free is completed" );
      i++;
    }

    /* Deallocate memory space */
    if ( data->buffer ) 
    {
      SGDEBUG_LVL_FMT( 10, "Deallocating buffer" );
      globus_libc_free( data->buffer );
    }

    if ( cb->s_api_data ) 
      {
        SGDEBUG_LVL_FMT( 10, "Deallocating s_api_data" );
        globus_libc_free(cb->s_api_data);
        cb->s_api_data = NULL;
      }

    return 1;
  }

  /**********************************************************/
  /**********************************************************/

  static void
  GlobusXioRegisterReadVectorPreCallback( callbackStruct* cb ) 
  {
    int indx = 0;
    Tcl_Interp* interp;
    Tcl_Obj* obj;
    Tcl_Obj* listptr;
    globus_xio_iovec_t iov;
    long totbytes = 0;
    long curbytes = 0;

    callbackData* data = (callbackData*)(cb->s_api_data);

    listptr = Tcl_NewListObj(0, NULL);

    interp = cb->s_main_tcl_interp;

    totbytes = data->total_bytes;

    SGDEBUG_LVL_FMT_1
      ( 20, 
	"GlobusXioRegisterReadVectorPreCallback: readv: totbytes = %d", 
	totbytes );

    while ( indx < data->iovec_cnt ) 
    {  
      iov = (((globus_xio_iovec_t*)(data->buffer))[indx]);
	  
      /* nbytes <= the size of iov[0].iov_len */
      if ( indx == 0 && totbytes <= iov.iov_len ) 
      {
	curbytes = totbytes;

	Tcl_ListObjAppendElement( interp, 
				  listptr, 
				  Tcl_NewStringObj( (char *)iov.iov_base, 
						    curbytes ) );
	totbytes = totbytes - curbytes;
      }
      /* otherwise */
      else 
      {
	if ( totbytes <= iov.iov_len )
	{
	  curbytes = totbytes;
	  totbytes = totbytes - curbytes;
	}
	if ( totbytes >= iov.iov_len )
	{
	  curbytes = iov.iov_len;
	  totbytes = totbytes - curbytes;
	}

	Tcl_ListObjAppendElement( interp, 
				  listptr, 
				  Tcl_NewStringObj( (char *)iov.iov_base, 
						    curbytes ) );
      }
	  
      indx++;

    }
    Tcl_SetVar2Ex(interp, data->unique_name, NULL, listptr, 0);

  }

  /**************************************************************************
   *
   * GLOBUS_XIO_REGISTER_WRITEV()
   *
   **************************************************************************/

  static void
    GlobusXioRegisterWriteVectorInit( callbackStruct* 		cb, 
                                      globus_xio_iovec_t* 	iovec,
                                      int 			iovec_cnt )
    {
      /* Allocate memory space */
      callbackData* data = (callbackData *)
        globus_libc_malloc(sizeof(callbackData));
      
      cb->s_api_data = data;
      data->buffer = iovec;
      data->iovec_cnt = iovec_cnt;

      TclGlobusCreateUniqueTCLVariableName( data->unique_name );      
    }


  /**********************************************************/
  /**********************************************************/
  
  static int
    GlobusXioRegisterWriteVectorCleanup( callbackStruct* cb )
    {
      int i;
      callbackData* data = (callbackData*)(cb->s_api_data);
      
      /* Release TCL resources */
      Tcl_UnsetVar2(cb->s_main_tcl_interp, data->unique_name, NULL, 0);

      SGDEBUG_LVL_FMT_1( 20, "iovec_cnt: %ld", data->iovec_cnt );

      for(i = 0; i < data->iovec_cnt; i++)
        {
	  if ( (((globus_xio_iovec_t*)(data->buffer))[i]).iov_base )
	    {
	      globus_libc_free
		( (((globus_xio_iovec_t*)(data->buffer))[i]).iov_base );
	      SGDEBUG_LVL_FMT( 10, "Deallocating iovec" );
	    }
        }
      
      /* Deallocate memory space */
      if ( data->buffer ) 
        {
          SGDEBUG_LVL_FMT( 10, "Deallocating buffer" );
          globus_libc_free( data->buffer );
        }

      if ( cb->s_api_data ) 
        {
          SGDEBUG_LVL_FMT( 10, "Deallocating s_api_data" );
          globus_libc_free(cb->s_api_data);
          cb->s_api_data = NULL;
        }

      return 1;
    }

  /**********************************************************/
  /**********************************************************/
  
  static void
    GlobusXioRegisterWriteVectorPreCallback( callbackStruct* cb ) 
    {
      int indx = 0;
      Tcl_Interp* interp;
      Tcl_Obj* listptr;
      callbackData*	data = (callbackData*)(cb->s_api_data);
      globus_xio_iovec_t* iovec_buf;
      long totbytes = 0;
      long curbytes = 0;
      
      listptr = Tcl_NewListObj(0, NULL);
      
      iovec_buf = (globus_xio_iovec_t *)data->buffer;
      
      interp = cb->s_main_tcl_interp;
          
      totbytes = data->total_bytes;

      SGDEBUG_LVL_FMT_1( 20, "writev: totbytes = %d", totbytes );
          
      while ( indx < data->iovec_cnt ) 
      {
	SGDEBUG_LVL_FMT_1( 20, "GlobusXioRegisterWriteVectorPreCallback: iov_len = %d", (iovec_buf[indx]).iov_len );
              
	Tcl_ListObjAppendElement
	  ( interp, listptr, 
	    Tcl_NewByteArrayObj((char *)(iovec_buf[indx]).iov_base,  
				(iovec_buf[indx]).iov_len ) 
	    );
              
	indx++;
      }
          
      Tcl_SetVar2Ex(interp, data->unique_name, NULL, listptr, 0);
    }
  
  /**************************************************************************
   *
   * GLOBUS_XIO_SERVER_REGISTER_ACCEPT()
   *  
   **************************************************************************/
  
  static void
    TclXioAcceptCallback( globus_xio_server_t server,
                          globus_xio_handle_t handle,
                          globus_result_t     result,
                          void *              user_arg)
    {
      char 	handle_ptr_buf[256];
      char 	str_server[128];
      char * 	msg;
	
      if ( result != 0 )
	{
	  globus_object_t * error_obj = globus_error_get( result );
	  msg = globus_object_printable_to_string( error_obj );
	}
      else
	{
	  msg = "";
	}
      
      /* Special trick for server handle */
      SWIG_MakePtr( handle_ptr_buf, handle, 
                    SWIG_TypeQuery( "globus_xio_handle_t" ), 0 );

      SWIG_MakePtr( str_server, server,
                    SWIG_TypeQuery( "globus_xio_server_t" ), 0 );
      
      CallCallback( (callbackStruct*)user_arg,
                    "%s %s \"%s\"",
                    str_server, handle_ptr_buf, msg );

      if ( result != 0 )
	{
	  globus_libc_free(msg);
	}
    }
  
  /**************************************************************************
   *
   * GLOBUS_XIO_SERVER_REGISTER_CLOSE()
   *
   **************************************************************************/
  
  static void
    TclXioServerCallback( globus_xio_server_t server,
                          void *              user_arg)
    {
      char 	str_server[128];
      
      SWIG_MakePtr( str_server, server,
                    SWIG_TypeQuery( "globus_xio_server_t" ), 0 );
      
      CallCallback( (callbackStruct*)user_arg,
                    "%s",
                    str_server );
    }
  
  /**************************************************************************
   *
   * GLOBUS_XIO_REGISTER_OPEN() & GLOBUS_XIO_REGISTER_CLOSE()
   *
   **************************************************************************/
  static void
    TclXioCallback( globus_xio_handle_t handle,
                    globus_result_t     result,
                    void *              user_arg)
    {
      char 	str_handle[128];
      char *	msg;
      
      if ( result != 0 )
	{
	  globus_object_t * error_obj = globus_error_get( result );
	  msg = globus_object_printable_to_string( error_obj );
	}
      else
	{
	  msg = "";
	}

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_xio_handle_t" ), 0 );

      CallCallback( (callbackStruct*)user_arg,
                    "%s \"%s\"", 
                    str_handle, msg );

      if ( result != 0 )
	{
	  globus_libc_free(msg);
	}
    }
  
  /**************************************************************************
   *
   * GLOBUS_XIO_REGISTER_READ()
   *
   **************************************************************************/
  static void
    TclXioRRDataCallback( globus_xio_handle_t          handle,
                          globus_result_t              result,
                          globus_byte_t *              buffer,
                          globus_size_t                len,
                          globus_size_t                nbytes,
                          globus_xio_data_descriptor_t data_desc,
                          void *                       user_arg)
    {
      char 	str_handle[128];
      char	str_data_desc[128];
      char *	msg;
      
      if ( result != 0 )
	{
	  globus_object_t * error_obj = globus_error_get( result );
	  msg = globus_object_printable_to_string( error_obj );
	}
      else
	{
	  msg = "";
	}

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_xio_handle_t" ), 0 );

      SWIG_MakePtr( str_data_desc, data_desc,
                    SWIG_TypeQuery( "globus_xio_data_descriptor_t" ), 0 );

      callbackData*
        data = (callbackData*)((callbackStruct*)user_arg)->s_api_data;
      
      data->total_bytes = nbytes;

      SGDEBUG_LVL_FMT_1( 20, "TclXioRRDataCallback: handle: %s", 
                         str_handle );
      SGDEBUG_LVL_FMT_1( 20, "TclXioRRDataCallback: result: %d", 
                         result );
      SGDEBUG_LVL_FMT_1( 20, "TclXioRRDataCallback: len: %d", 
                         len );
      SGDEBUG_LVL_FMT_1( 20, "TclXioRRDataCallback: nbytes: %d", 
                         nbytes );
      SGDEBUG_LVL_FMT_1( 20, "TclXioRRDataCallback: data_desc: %s", 
                         str_data_desc );

      CallCallback( (callbackStruct*)user_arg,
                    "%s \"%s\" $%s %s",
                    str_handle, msg, data->unique_name, str_data_desc );

      if ( result != 0 )
	{
	  globus_libc_free(msg);
	}
    }

  /**************************************************************************
   *
   * GLOBUS_XIO_REGISTER_WRITE()
   *
   **************************************************************************/
  static void
    TclXioRWDataCallback( globus_xio_handle_t          handle,
                          globus_result_t              result,
                          globus_byte_t *              buffer,
                          globus_size_t                len,
                          globus_size_t                nbytes,
                          globus_xio_data_descriptor_t data_desc,
                          void *                       user_arg )
    {
      char 	str_handle[128];
      char	str_data_desc[128];
      char *	msg;
      
      if ( result != 0 )
	{
	  globus_object_t * error_obj = globus_error_get( result );
	  msg = globus_object_printable_to_string( error_obj );
	}
      else
	{
	  msg = "";
	}

      callbackData*
        data = (callbackData*)((callbackStruct*)user_arg)->s_api_data;

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_xio_handle_t" ), 0 );

      SWIG_MakePtr( str_data_desc, data_desc,
                    SWIG_TypeQuery( "globus_xio_data_descriptor_t" ), 0 );

      data->total_bytes = nbytes;
      data->buffer = buffer;

      SGDEBUG_LVL_FMT_1( 20, "TclXioRWDataCallback: handle: %s", 
                         str_handle );
      SGDEBUG_LVL_FMT_1( 20, "TclXioRWDataCallback: result: %d", 
                         result );
      SGDEBUG_LVL_FMT_1( 20, "TclXioRWDataCallback: len: %d", 
                         len );
      SGDEBUG_LVL_FMT_1( 20, "TclXioRWDataCallback: nbytes: %d", 
                         nbytes );
      SGDEBUG_LVL_FMT_1( 20, "TclXioRWDataCallback: data_desc: %s", 
                         str_data_desc );

      CallCallback( (callbackStruct*)user_arg,
                    "%s \"%s\" $%s %s",
                    str_handle, msg, data->unique_name, str_data_desc );

      if ( result != 0 )
	{
	  globus_libc_free(msg);
	}
    }
  
  /**************************************************************************
   *
   * GLOBUS_XIO_REGISTER_READV()
   *
   **************************************************************************/
  static void
    TclXioRRVectorDataCallback( globus_xio_handle_t                 handle,
                                globus_result_t                     result,
                                globus_xio_iovec_t *                iovec,
                                int                                 count,
                                globus_size_t                       nbytes,
                                globus_xio_data_descriptor_t        data_desc,
                                void *                              user_arg)
    {
      char 	str_handle[128];
      char	str_data_desc[128];
      char *	msg;
      
      if ( result != 0 )
	{
	  globus_object_t * error_obj = globus_error_get( result );
	  msg = globus_object_printable_to_string( error_obj );
	}
      else
	{
	  msg = "";
	}

      callbackData*
        data = (callbackData*)((callbackStruct*)user_arg)->s_api_data;

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_xio_handle_t" ), 0 );

      SWIG_MakePtr( str_data_desc, data_desc,
                    SWIG_TypeQuery( "globus_xio_data_descriptor_t" ), 0 );
      
      data->total_bytes = nbytes;
      
      CallCallback( (callbackStruct*)user_arg,
                    "%s \"%s\" $%s %d %d %s",
                    str_handle, msg, data->unique_name, count, nbytes, 
                    str_data_desc );

      if ( result != 0 )
	{
	  globus_libc_free(msg);
	}
    }
  
  /**************************************************************************
   *
   * GLOBUS_XIO_REGISTER_WRITEV()
   *
   **************************************************************************/
  static void
    TclXioRWVectorDataCallback( globus_xio_handle_t                 handle,
                                globus_result_t                     result,
                                globus_xio_iovec_t *                iovec,
                                int                                 count,
                                globus_size_t                       nbytes,
                                globus_xio_data_descriptor_t        data_desc,
                                void *                              user_arg)
    {
      char 	str_handle[128];
      char	str_data_desc[128];
      char *	msg;
      
      if ( result != 0 )
	{
	  globus_object_t * error_obj = globus_error_get( result );
	  msg = globus_object_printable_to_string( error_obj );
	}
      else
	{
	  msg = "";
	}

      callbackData*
        data = (callbackData*)((callbackStruct*)user_arg)->s_api_data;

      SWIG_MakePtr( str_handle, handle,
                    SWIG_TypeQuery( "globus_xio_handle_t" ), 0 );

      SWIG_MakePtr( str_data_desc, data_desc,
                    SWIG_TypeQuery( "globus_xio_data_descriptor_t" ), 0 );
      
      data->iovec_cnt = count;
      data->buffer = iovec;
      data->total_bytes = nbytes;
      
      CallCallback( (callbackStruct*)user_arg,
                    "%s \"%s\" $%s %d %d %s",
                    str_handle, msg, data->unique_name, count, nbytes, 
                    str_data_desc );

      if ( result != 0 )
	{
	  globus_libc_free(msg);
	}
    }
  
  %}


/**************************************************************************
 *
 *  XIO INITIALIZATION STUFF
 *
 **************************************************************************/

%init %{

  SGINIT_X509;
  
  /* 
   * Initialize driver type hash table 
   */
  Tcl_InitHashTable(&tcl_globus_xio_driver_type_hashtbl, TCL_STRING_KEYS);

  /* 
   * Initialize driver function map hash table 
   */
  Tcl_InitHashTable(&tcl_globus_xio_driver_func_map_hashtbl, TCL_STRING_KEYS);


#if 0  
  /* 
   * Create entry in tcl_globus_xio_func_map_hashtbl with the results 
   * from this call DriverFileInit();
   */

  GlobusXioInsertDriverFuncMapHashTbl
    ("file", (XIODriverFunctionMap_type *)DriverFileInit());

  /* 
   * Create entry in tcl_globus_xio_func_map_hashtbl with the results 
   * from this call DriverGSIInit();
   */

  GlobusXioInsertDriverFuncMapHashTbl
    ("gsi", (XIODriverFunctionMap_type *)DriverGSIInit());
  
  /* 
   * Create entry in tcl_globus_xio_func_map_hashtbl with the results 
   * from this call DriverTCPInit();
   */

  GlobusXioInsertDriverFuncMapHashTbl
    ("tcp", (XIODriverFunctionMap_type *)DriverTCPInit());

  /* 
   * Create entry in tcl_globus_xio_func_map_hashtbl with the results 
   * from this call DriverUDPInit();
   */

  GlobusXioInsertDriverFuncMapHashTbl
    ("udp", (XIODriverFunctionMap_type *)DriverUDPInit());
  
  /* 
   * Create entry in tcl_globus_xio_func_map_hashtbl with the results 
   * from this call DriverHTTPInit();
   */

  GlobusXioInsertDriverFuncMapHashTbl
    ("http", (XIODriverFunctionMap_type *)DriverHTTPInit());

  /* 
   * Create entry in tcl_globus_xio_func_map_hashtbl with the results 
   * from this call DriverModeEInit();
   */

  GlobusXioInsertDriverFuncMapHashTbl
    ("mode_e", 
     (XIODriverFunctionMap_type *)DriverModeEInit());

#endif /* 0 */
  
%}



/**************************************************************************
 *  GLOBUS_XIO_DRIVER_LOAD()
 **************************************************************************/

%typemap(in,numinputs=0) tcl_globus_xio_driver_t* out_driver 
(tcl_globus_xio_driver_t driver) 
{
  $1 = &driver;
}

%typemap( argout ) tcl_globus_xio_driver_t* out_driver {

  char driver_ptr_buf[256];
      
  /* Special trick for driver type pointer */
  SWIG_MakePtr( driver_ptr_buf, $1->driver_type, 
		SWIG_TypeQuery( "globus_xio_driver_t" ), 0 );

  SGDEBUG_LVL_FMT_1( 10, "globus_xio_driver_load: driver_ptr_buf: %s", 
		     driver_ptr_buf);

  /* Return the address of driver structure */
  Tcl_Obj* o = Tcl_NewStringObj(driver_ptr_buf, strlen(driver_ptr_buf));
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
  
}

%rename(globus_xio_driver_load) tcl_globus_xio_driver_load;

%exception tcl_globus_xio_driver_load GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_xio/globus_xio_driver_load
 * NAME
 *   globus_xio_driver_load
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_driver_load
 *   |html </b>
 *   |html <i>
 *   driver_name
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to load a driver. In order to successfully 
 *   use this command the user must know the name of all the drivers 
 *   he/she wishes to load. Those drivers are "file", "gsi", "http", 
 *   "mode_e", "tcp" or "udp".
 *
 *   |html <ul><b>
 *   driver_name
 *   |html </b>
 *   - The name of the driver the user wishes to load.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_driver_load returns 0 and a driver object that
 *   can be pushed onto stack by calling globus_xio_stack_push_driver. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_driver_load $driver_name ] result driver_object
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_driver_unload, globus_xio_stack_push_driver
 *
 *****/

%inline %{

  /* The driver structure is created for each driver */
  typedef struct {
    char                driver_name[MAX_DRIVER_NAME];
    globus_xio_driver_t driver_type;
  } tcl_globus_xio_driver_t;


  globus_result_t
    tcl_globus_xio_driver_load( char *driver_name,
				tcl_globus_xio_driver_t* out_driver )
  {
    globus_result_t	retval;

    memset(out_driver->driver_name, '\0', MAX_DRIVER_NAME); 
    strcpy(out_driver->driver_name, driver_name);
    
    retval = globus_xio_driver_load(out_driver->driver_name,
				    &out_driver->driver_type);

    SGDEBUG_LVL_FMT_1( 10, "tcl_globus_xio_driver_load: retval = %d", retval);
    SGDEBUG_LVL_FMT_1( 10, "tcl_globus_xio_driver_load: driver_name = %s", 
                   out_driver->driver_name );
    SGDEBUG_LVL_FMT_1( 10, "tcl_globus_xio_driver_load: driver_type = %ld", 
                   out_driver->driver_type );
    
    /* Insert driver structure into driver_type hash table */
    GlobusXioInsertDriverTypeHashTbl(out_driver->driver_name,
				     out_driver->driver_type);    

    return retval;
  }
%}


/**************************************************************************
 *  GLOBUS_XIO_DRIVER_UNLOAD()
 **************************************************************************/

%typemap(in) globus_xio_driver_t in_driver
{
  static void *descr = 0;
  if ( !descr )
    {
      descr = SWIG_TypeQuery( "tcl_globus_xio_driver_t *" );
      assert( descr );
    }
  SWIG_ConvertPtr( $input, (void **) &$1, descr,
		   SWIG_POINTER_EXCEPTION );
}

%rename(globus_xio_driver_unload) tcl_globus_xio_driver_unload;

%exception tcl_globus_xio_driver_unload GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_xio/globus_xio_driver_unload
 * NAME
 *   globus_xio_driver_unload
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_driver_unload
 *   |html </b>
 *   |html <i>
 *   driver_object
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to unload the driver_object. 
 *
 *   |html <ul><b>
 *   driver_object
 *   |html </b>
 *   - The driver object as returned by globus_xio_driver_load 
 *   the user wishes to unload.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_driver_unload returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_driver_unload $driver_object ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_driver_load
 *
 *****/

%inline %{

  globus_result_t
    tcl_globus_xio_driver_unload( globus_xio_driver_t in_driver )
  {
    globus_result_t	retval;
    
    /* The actual Globus XIO function call */
    retval = globus_xio_driver_unload(in_driver);

    SGDEBUG_LVL_FMT_1
      ( 20, "tcl_globus_xio_driver_unload: retval: %d", retval );
    
    /* Remove the entry from hash table */
    GlobusXioRemoveDriverTypeHashTbl(in_driver);

    return retval;
  }
%}


/**************************************************************************
 *  GLOBUS_XIO_READ()
 **************************************************************************/

/****if* globus_xio.typemap/tcl_globus_buffer_t*__buffer
 * NAME
 *  tcl_globus_buffer_t* buffer
 *
 * SOURCE
 */
%typemap(in,numinputs=0) 
  tcl_globus_buffer_t* buffer( tcl_globus_buffer_t buffer ) 
{
  $1 = &buffer;
}

%typemap( argout ) tcl_globus_buffer_t* buffer 
{
  /* Return the data buffer */
  Tcl_Obj* o = Tcl_NewByteArrayObj( $1->buffer, $1->length );
  Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
}

%typemap( freearg ) tcl_globus_buffer_t* buffer 
{
  globus_libc_free( $1->buffer );
}
/****/

%rename(globus_xio_read) tcl_globus_xio_read;

%exception tcl_globus_xio_read GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_xio/globus_xio_read
 * NAME
 *   globus_xio_read
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_read
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer_length
 *   waitforbytes
 *   data_descriptor
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to script the reading of a block of
 *   data from a handle in blocking mode.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   buffer_length
 *   |html </b>
 *   - Maximum number of bytes to read.
 *
 *   |html <b>
 *   waitforbytes
 *   |html </b>
 *   - Minimum number of bytes to write before invoking the script.
 *
 *   |html <b>
 *   data_descriptor
 *   |html </b>
 *   - Globus XIO data descriptor as returned by 
 *   globus_xio_data_descriptor_cntl. It can be NULL.
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_read returns 0 and a block of data.
 *   On error, this command will raise an error and must be 
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_read $handle $buffer_length 
 *   $waitforbytes $data_descriptor ] status buffer
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_cntl, globus_xio_handle_create, 
 *   globus_xio_write
 *******
 */

%inline %{
  typedef struct {
    globus_byte_t *	buffer;
    globus_size_t	length;
  } tcl_globus_buffer_t;
  
  globus_result_t
    tcl_globus_xio_read( globus_xio_handle_t handle,
                         long buffer_length,
                         long waitforbytes,
                         globus_xio_data_descriptor_t data_desc,
                         tcl_globus_buffer_t *buffer )
    {
      globus_result_t	retval;
      
      buffer->buffer = 
        (globus_byte_t*)globus_libc_malloc( sizeof( globus_byte_t ) *
                                            buffer_length );

      if ( buffer->buffer == GLOBUS_NULL )
	{
	  fprintf(stderr, "globus_libc_malloc fails\n");
	  exit(1);
	}
      
      memset( buffer->buffer, '\0', buffer_length );

      retval = globus_xio_read( handle,
                                buffer->buffer,
                                buffer_length,
                                waitforbytes,
                                &(buffer->length),
                                data_desc );

#if 0      
      SGDEBUG_LVL_FMT_1( 10, "tcl_globus_xio_read: buffer->buffer: %s", 
                         buffer->buffer );
#endif
      
      SGDEBUG_LVL_FMT_1( 20, "tcl_globus_xio_read: buffer->length: %d", 
                         buffer->length );
      SGDEBUG_LVL_FMT_1( 20, "tcl_globus_xio_read: data_desc: %p", 
                         data_desc );

      return retval;
  }
%}

/**************************************************************************
 *  GLOBUS_XIO_WRITE()
 **************************************************************************/

%typemap(in) 
  tcl_globus_buffer_t *in_buffer( $*1_ltype temp ) 
{
  temp.buffer = 
    (globus_byte_t *)Tcl_GetByteArrayFromObj( $input, &(temp.length) );
  $1 = &temp;
}

%rename(globus_xio_write) tcl_globus_xio_write;

%exception tcl_globus_xio_write GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_xio/globus_xio_write
 * NAME
 *   globus_xio_write
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_write
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   waitforbytes
 *   data_desc
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to script the writing of a block of
 *   data in blocking mode.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - Buffer to write.
 *
 *   |html <b>
 *   waitforbytes
 *   |html </b>
 *   - Minimum number of bytes to write before invoking the script.
 *
 *   |html <b>
 *   data_desc
 *   |html </b>
 *   - To be initialized by globus_xio_data_descriptor_init.
 *
 *   |html </ul>
 *
 * RESULT
 *   On success globus_xio_write returns 0 and number of bytes written.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_write $handle $buffer $waitforbytes NULL ] status 
 *   nbytes
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_init, globus_xio_handle_create, 
 *   globus_xio_read
 *******
 */

%inline %{

  globus_result_t
    tcl_globus_xio_write( globus_xio_handle_t handle,
			  tcl_globus_buffer_t *in_buffer,
			  long waitforbytes,
			  globus_xio_data_descriptor_t data_desc,
			  globus_size_t *nbytes )
    {
      globus_result_t retval;

      retval = globus_xio_write( handle,
				 in_buffer->buffer,
				 in_buffer->length,
				 waitforbytes,
				 nbytes,
				 data_desc );

      return retval;
    }
  
  %}

/**************************************************************************
 *  GLOBUS_XIO_READV()
 **************************************************************************/

%typemap(in) 
  tcl_globus_iovec_t* inout_iovec ( $*1_type temp ) 
{
  Tcl_Obj **listobjv;
  int nitems;
  int i = 0;

  /* Pass in Tcl list of buffer sizes */
  if ( Tcl_ListObjGetElements(interp, $input, &nitems, &listobjv) == 
       TCL_ERROR )
    return TCL_ERROR;

  temp.bufsizearray = (char **) globus_libc_malloc((nitems+1)*sizeof(char *));
  
  for (i = 0; i < nitems; i++) 
    {
      temp.bufsizearray[i] = Tcl_GetStringFromObj(listobjv[i], 0);
    }
  temp.bufsizearray[i] = 0;

  SGDEBUG_LVL_FMT_1( 20, "typemap(in): nitems: %ld", nitems );

  temp.iovcnt = nitems;

  temp.iovec = 
    (globus_xio_iovec_t *)globus_libc_malloc( sizeof(struct iovec) * nitems );

  i = 0;
  while ( temp.bufsizearray[i] ) {
    temp.iovec[i].iov_base = 
      (void *)globus_libc_malloc( sizeof( globus_byte_t ) * 
			     atoi(temp.bufsizearray[i]) );

    /* Clear up our memory buffer */
    memset( temp.iovec[i].iov_base, '\0', atoi(temp.bufsizearray[i]) );
    temp.iovec[i].iov_len = atoi(temp.bufsizearray[i]);
    i++;
  }

  $1 = &temp;
}

%typemap( argout ) tcl_globus_iovec_t* inout_iovec 
{  
  int i = 0;
  long totbytes = 0;
  long curbytes = 0;
  
  /* Loop over as many as the specified number of buffers */
  /* Return the string iovec */
  totbytes = $1->nbytes;
  
  for(i = 0; i < $1->iovcnt; i++) 
    {
      if ( totbytes == 0 ) 
        curbytes = 0;
      
      if ( totbytes > 0 ) 
        {
          /* 
           * remaining bytes is smaller or equal to the size of the first 
           * buffer 
           */
          if ( i == 0 && totbytes <= $1->iovec[i].iov_len ) 
            {
              curbytes = $1->nbytes;
              totbytes = totbytes - curbytes; 
            }
          
          /* otherwise */
          else {
            if ( totbytes <= $1->iovec[i].iov_len )
              {
                curbytes = totbytes;
                totbytes = totbytes - curbytes;
              }
            if ( totbytes >= $1->iovec[i].iov_len )
              {
                curbytes = $1->iovec[i].iov_len;
                totbytes = totbytes - curbytes; 
              }
          }
        }
      
      Tcl_Obj* o = Tcl_NewStringObj( $1->iovec[i].iov_base, curbytes );
      
      Tcl_ListObjAppendElement( interp, Tcl_GetObjResult(interp), o );
    }
}

%typemap( freearg ) 
  tcl_globus_iovec_t* inout_iovec 
{
  int i;

  SGDEBUG_LVL_FMT_1( 20, "freearg: iovcnt: %ld", sizeof($1->iovcnt) );
  SGDEBUG_LVL_FMT_1( 20, "freearg: bufsizearray: %ld",
		     sizeof($1->bufsizearray) );
  SGDEBUG_LVL_FMT_1( 20, "freearg: iovec: %ld", sizeof($1->iovec) );

  /* 
   * Iterate the list to release memory 
   */
  i = 0;
  while ( $1->bufsizearray[i] ) 
    {
      globus_libc_free(  $1->iovec[i].iov_base );
      i++;
    }

  globus_libc_free( $1->iovec );
  globus_libc_free( $1->bufsizearray );
}

%rename(globus_xio_readv) tcl_globus_xio_readv;

%exception tcl_globus_xio_readv GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_xio/globus_xio_readv
 * NAME
 *   globus_xio_readv
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_readv
 *   |html </b>
 *   |html <i>
 *   handle
 *   list_of_buffer_sizes
 *   waitforbytes
 *   data_descriptor
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to script the reading of a block of
 *   data from a handle and put the data into several buffers
 *   at once.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   list_of_buffer_sizes
 *   |html </b>
 *   - TCL list of buffer sizes.
 *
 *   |html <b>
 *   waitforbytes
 *   |html </b>
 *   - Minimum number of bytes to write before invoking the script.
 *
 *   |html <b>
 *   data_descriptor
 *   |html </b>
 *   - Globus XIO data descriptor as returned by 
 *   globus_xio_data_descriptor_cntl. It can be NULL.
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_readv returns 0 and puts the data into TCL 
 *   buffers. On error, this command will raise an error and must be 
 *   caught using the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_xio_readv $handle [list $bufsize1 $bufsize2] 
 *   $waitforbytes NULL ] status buf1 buf2
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_cntl, globus_xio_handle_create, 
 *   globus_xio_writev
 *
 *****/

%inline %{

  typedef struct {
    char** bufsizearray;
    int iovcnt;
    globus_xio_iovec_t	*iovec;
    globus_size_t	nbytes;
  } tcl_globus_iovec_t;
  
  globus_result_t
  tcl_globus_xio_readv( globus_xio_handle_t handle,
			tcl_globus_iovec_t *inout_iovec,
			long waitforbytes,
			globus_xio_data_descriptor_t data_desc )
  {
    globus_result_t	retval;
    
    retval = globus_xio_readv( handle,
			       inout_iovec->iovec,
			       inout_iovec->iovcnt,
			       waitforbytes,
			       &(inout_iovec->nbytes),
			       data_desc );
    
    return retval;

  }
%}

/**************************************************************************
 *  GLOBUS_XIO_WRITEV()
 **************************************************************************/

%typemap(in) 
  tcl_globus_iovec_t *in_buflist( $*1_ltype temp ) 
{

  Tcl_Obj **listobjv;
  int nitems;
  int i = 0;

  /* Pass in Tcl list of data buffer */
  if ( Tcl_ListObjGetElements(interp, $input, &nitems, &listobjv) == 
       TCL_ERROR )
    return TCL_ERROR;

  temp.iovec = 
    (globus_xio_iovec_t *)globus_libc_malloc( sizeof(struct iovec) * nitems);
  
  temp.iovcnt = nitems;
  
  for (i = 0; i < nitems; i++) 
    {
      temp.iovec[i].iov_base = 
	Tcl_GetStringFromObj(listobjv[i], (int*)&(temp.iovec[i].iov_len));
    }

  $1 = &temp;
}

%typemap( freearg ) 
  tcl_globus_iovec_t* in_buflist 
{
  globus_libc_free( $1->iovec );
}

%rename(globus_xio_writev) tcl_globus_xio_writev;

%exception tcl_globus_xio_writev GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_xio/globus_xio_writev
 * NAME
 *   globus_xio_writev
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_writev
 *   |html </b>
 *   |html <i>
 *   handle
 *   list_of_buffers
 *   waitforbytes
 *   data_desc
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to script the writing of a block of
 *   data to several memory addresses at once.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   list_of_buffers
 *   |html </b>
 *   - TCL list of data buffers.
 *
 *   |html <b>
 *   waitforbytes
 *   |html </b>
 *   - Minimum number of bytes to write before invoking the script.
 *
 *   |html <b>
 *   data_desc
 *   |html </b>
 *   - To be initialized by globus_xio_data_descriptor_init.
 *
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_xio_writev returns 0 and number of bytes written.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_xio_writev $handle [list $buf1 $buf2] 
 *   $waitforbytes NULL ] status nbytes
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_init, globus_xio_handle_create, 
 *   globus_xio_readv
 *
 *****/

%inline %{

  globus_result_t
    tcl_globus_xio_writev( globus_xio_handle_t handle,
			   tcl_globus_iovec_t *in_buflist,
			   long waitforbytes,
			   globus_xio_data_descriptor_t data_desc,
			   globus_size_t *nbytes )
    {
      globus_result_t retval;

      retval = globus_xio_writev( handle,
				  in_buflist->iovec,
				  in_buflist->iovcnt,
				  waitforbytes,
				  nbytes,
				  data_desc );

      return retval;
    }
  
  %}

/**************************************************************************
 *
 * GLOBUS_XIO_REGISTER_READ() 
 *
 **************************************************************************/

%typemap(in) 
  (globus_xio_data_callback_t rr_cb, void * rr_user_arg) 
  (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  /* It's the time to allocate our buffer */
  GlobusXioRegisterReadInit( cbStruct, arg2 );

  $1 = TclXioRRDataCallback; 
  $2 = cbStruct;

  cbStruct->s_pre_process = GlobusXioRegisterReadPreCallback;
  cbStruct->s_post_process = GlobusXioRegisterReadCleanup; 
}

%rename(globus_xio_register_read) tcl_globus_xio_register_read;

%exception tcl_globus_xio_register_read GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_xio/globus_xio_register_read
 * NAME
 *   globus_xio_register_read
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_register_read
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer_length
 *   waitforbytes
 *   data_desc
 *   script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to script the reading of a block of
 *   data in the background.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   buffer_length
 *   |html </b>
 *   - Maximum number of bytes to read.
 *
 *   |html <b>
 *   waitforbytes
 *   |html </b>
 *   - Minimum number of bytes to read before invoking the script.
 *
 *   |html <b>
 *   data_desc
 *   |html </b>
 *   - To be initialized by globus_xio_data_descriptor_init. It can be NULL.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked to perform actual read. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_xio_register_read returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command. 
 *
 * EXAMPLE
 *   lassign [ globus_xio_register_read $handle $buffer_length $waitforbytes
 *   $data_desc [ subst { callback user_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_init, globus_xio_handle_create, 
 *   globus_xio_register_write
 *
 *****/
%inline %{

  globus_result_t
    tcl_globus_xio_register_read( globus_xio_handle_t handle,
                                  long buffer_length,
                                  long waitforbytes,
                                  globus_xio_data_descriptor_t data_desc,
                                  globus_xio_data_callback_t rr_cb,
                                  void * rr_user_arg
                                  )
    {
      globus_result_t	retval;
      
      retval = 
        globus_xio_register_read
        (
         handle,
         (globus_byte_t*)
         ((callbackData*)(((callbackStruct*)rr_user_arg)->s_api_data))->buffer,
         buffer_length,
         waitforbytes,
         data_desc,
         rr_cb,
         rr_user_arg 
         );
      
      return retval;
      
  }
%}

/**************************************************************************
 *
 * GLOBUS_XIO_REGISTER_WRITE()
 *
 **************************************************************************/

%typemap(in) 
  tcl_globus_buffer_t *rw_in_buffer( $*1_ltype temp ) 
{
  unsigned char*   tbuffer;

  tbuffer = 
    (unsigned char *)Tcl_GetByteArrayFromObj( $input, &(temp.length) );
  temp.buffer = globus_libc_malloc( temp.length );
  memcpy( temp.buffer, tbuffer, temp.length );
  
  SGDEBUG_LVL_FMT_1( 10, "globus_xio_register_write: typemap(in): length: %d", 
                     temp.length );

#if 0
  SGDEBUG_LVL_FMT_3
    ( 30, "globus_xio_register_write: typemap(in): buffer: %*.*s", 
      temp.length, temp.length, temp.buffer );
#endif 

  $1 = &temp;
}

%typemap(in) 
  (globus_xio_data_callback_t rw_cb, void * rw_user_arg) 
  (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );
  GlobusXioRegisterWriteInit(cbStruct, arg2->buffer);
  
  $1 = TclXioRWDataCallback; 
  $2 = cbStruct;
  
  cbStruct->s_pre_process = GlobusXioRegisterWritePreCallback;
  cbStruct->s_post_process = GlobusXioRegisterWriteCleanup;
}

%rename(globus_xio_register_write) tcl_globus_xio_register_write;

%exception tcl_globus_xio_register_write GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_xio/globus_xio_register_write
 * NAME
 *   globus_xio_register_write
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_register_write
 *   |html </b>
 *   |html <i>
 *   handle
 *   buffer
 *   waitforbytes
 *   data_descriptor
 *   callback_script
 *   |html </i>
 * 
 * DESCRIPTION
 *   This command is used to script the writing of a block of
 *   data to a handle in the background.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   buffer
 *   |html </b>
 *   - Buffer to write.
 *
 *   |html <b>
 *   waitforbytes
 *   |html </b>
 *   - Minimum number of bytes to write before invoking the script.
 *
 *   |html <b>
 *   data_desc
 *   |html </b>
 *   - To be initialized by globus_xio_data_descriptor_init. It can be NULL.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked to perform actual write. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_register_write returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_register_write $handle $buffer $waitforbytes
 *   $data_desc [ subst { callback_proc $user_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_init, globus_xio_handle_create, 
 *   globus_xio_register_read
 *
 *****/
%inline %{

  /****if* globus_xio/tcl_globus_xio_register_write
   * NAME
   *   tcl_globus_xio_register_write
   *
   * SYNOPSIS
   *  |html <b>
   *  globus_result_t
   *  tcl_globus_xio_register_write(
   *  globus_xio_handle_t
   *  |html </b>
   *  |html <i>
   *  handle,
   *  |html </i>
   *  |html <b>
   *  tcl_globus_buffer_t*
   *  |html </b>
   *  |html <i>
   *  rw_in_buffer,
   *  |html </i>
   *  |html <b>
   *  long
   *  |html </b>
   *  |html <i>
   *  waitforbytes,
   *  |html </i>
   *  |html <b>
   *  globus_xio_data_descriptor_t
   *  |html </b>
   *  |html <i>
   *  data_desc,
   *  |html </i>
   *  |html <b>
   *  globus_xio_data_callback_t
   *  |html </b>
   *  |html <i>
   *  rw_cb,
   *  |html </i>
   *  |html <b>
   *  void*
   *  |html </b>
   *  |html <i>
   *  rw_user_arg
   *  |html </i>
   *  |html <b>
   *  );
   *  |html </b>
   *
   * DESCRIPTION
   *  This customized version of the globus_xio_register_write function
   *  allows for passing and converting of TCL data to
   *  globus_xio_register_write. This is the function that gets
   *  invoked from the TCL layer.
   *
   * SEE ALSO
   *
   ****/

  globus_result_t
    tcl_globus_xio_register_write( globus_xio_handle_t handle,
                                   tcl_globus_buffer_t *rw_in_buffer,
                                   long waitforbytes,
                                   globus_xio_data_descriptor_t data_desc,
                                   globus_xio_data_callback_t rw_cb,
                                   void * rw_user_arg)
    {
      globus_result_t	retval;
      
      SGDEBUG_LVL_FMT_1( 20, "data_desc: %p", data_desc );
      
      retval = globus_xio_register_write( handle,
                                          rw_in_buffer->buffer,
                                          rw_in_buffer->length,
                                          waitforbytes,
                                          data_desc,
                                          rw_cb,
                                          rw_user_arg );
      
      return retval;
    }
%}
 
/**************************************************************************
 *
 * GLOBUS_XIO_REGISTER_READV()
 *
 **************************************************************************/

%typemap(in) 
  tcl_globus_iovec_t* rrv_list ( $*1_type temp ) 
{
  Tcl_Obj **listobjv;
  int nitems;
  int i;

  /* Pass in Tcl list of buffer sizes */
  if ( Tcl_ListObjGetElements(interp, $input, &nitems, &listobjv) == 
       TCL_ERROR )
    return TCL_ERROR;

  temp.bufsizearray = (char **) globus_libc_malloc((nitems+1)*sizeof(char *));

  for (i = 0; i < nitems; i++) 
    {
      temp.bufsizearray[i] = Tcl_GetStringFromObj(listobjv[i], 0);
    }

  temp.bufsizearray[i] = 0;
  temp.iovcnt = nitems;

  $1 = &temp;
}


%typemap(in) 
  (globus_xio_iovec_callback_t rrv_cb, void * rrv_user_arg) 
  (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  /* It's the time to allocate our iovec buffer */
  GlobusXioRegisterReadVectorInit( cbStruct, 
                                   arg2->iovcnt, 
                                   arg2->bufsizearray );

  $1 = TclXioRRVectorDataCallback; 
  $2 = cbStruct;
  
  /* For clean up after callback */
  cbStruct->s_post_process = GlobusXioRegisterReadVectorCleanup; 
  
  /* We assign global unique name to iovec buffer */
  cbStruct->s_pre_process = GlobusXioRegisterReadVectorPreCallback;
}

%rename(globus_xio_register_readv) tcl_globus_xio_register_readv;

%exception tcl_globus_xio_register_readv GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_xio/globus_xio_register_readv
 * NAME
 *   globus_xio_register_readv
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_register_readv
 *   |html </b>
 *   |html <i>
 *   handle
 *   list_of_buffer_size
 *   waitforbytes
 *   data_desc
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to script the reading of a block of
 *   data from several memory addresses at once in the background.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   list_of_buffer_size
 *   |html </b>
 *   - TCL list of buffer size
 *
 *   |html <b>
 *   waitforbytes
 *   |html </b>
 *   - Minimum number of bytes to read before invoking the script.
 *
 *   |html <b>
 *   data_desc
 *   |html </b>
 *   - To be initialized by globus_xio_data_descriptor_init. It can be NULL.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked to perform actual read. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_register_readv returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_register_readv $handle [list $bufsize1 $bufsize2]
 *   $waitforbytes $data_desc [ subst { callback_proc $user_arg } ] ] result  
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_init, globus_xio_handle_create, 
 *   globus_xio_register_writev
 *
 *****/

%inline %{
  
  globus_result_t
  tcl_globus_xio_register_readv( globus_xio_handle_t handle,
				 tcl_globus_iovec_t* rrv_list,
				 long waitforbytes,
				 globus_xio_data_descriptor_t data_desc,
				 globus_xio_iovec_callback_t rrv_cb,
				 void * rrv_user_arg )
    {
      globus_result_t	retval;
      
      retval = globus_xio_register_readv
        (handle,
         (globus_xio_iovec_t*)
         ((callbackData*)
          (((callbackStruct*)rrv_user_arg)->s_api_data))->buffer,
         rrv_list->iovcnt,
         waitforbytes,
         data_desc,
         rrv_cb,
         rrv_user_arg
         );
            
      return retval;
    }
  %}

/**************************************************************************
 *
 * GLOBUS_XIO_REGISTER_WRITEV()
 *
 **************************************************************************/

%typemap(in) 
  tcl_globus_iovec_t *rwv_list( $*1_ltype temp ) 
{
  Tcl_Obj 	**listobjv;
  int 		nitems;
  int 		i;

  char*		tbuffer;

  /* Pass in Tcl list of buffer containing actual data */
  if ( Tcl_ListObjGetElements(interp, $input, &nitems, &listobjv) == 
       TCL_ERROR )
    return TCL_ERROR;

  temp.iovec = 
    (globus_xio_iovec_t *)globus_libc_malloc( sizeof(struct iovec) * nitems);

  temp.iovcnt = nitems;
  
  /* nitems == iovec count */
  for (i = 0; i < nitems; i++) 
    {     
      /*
      temp.iovec[i].iov_base = 
	Tcl_GetStringFromObj(listobjv[i], &(temp.iovec[i].iov_len));    
      */
      
      tbuffer = (char*)
        Tcl_GetStringFromObj(listobjv[i], (int*)&(temp.iovec[i].iov_len));
      
      if ( temp.iovec[i].iov_len > 0 )
	{
	  temp.iovec[i].iov_base =
	    (char*)globus_libc_malloc( temp.iovec[i].iov_len );

	  memcpy(temp.iovec[i].iov_base, tbuffer, temp.iovec[i].iov_len);
	}
      else
	{
	  temp.iovec[i].iov_base = NULL;
	}
    }

  $1 = &temp;
}

%typemap(in) 
  (globus_xio_iovec_callback_t rwv_cb, void * rwv_user_arg) 
  (callbackStruct *cbStruct) 
{
  cbStruct = CreateCallback( interp, $input );

  GlobusXioRegisterWriteVectorInit( cbStruct,
                                    (globus_xio_iovec_t *)arg2->iovec, 
                                    arg2->iovcnt );
  
  $1 = TclXioRWVectorDataCallback; 
  $2 = cbStruct;

  cbStruct->s_pre_process = GlobusXioRegisterWriteVectorPreCallback;
  cbStruct->s_post_process = GlobusXioRegisterWriteVectorCleanup;
}

%rename(globus_xio_register_writev) tcl_globus_xio_register_writev;

%exception tcl_globus_xio_register_writev GLOBUS_EXCEPTION_RESULT_T;

/****f* globus_xio/globus_xio_register_writev
 * NAME
 *   globus_xio_register_writev
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_register_writev
 *   |html </b>
 *   |html <i>
 *   handle
 *   list_of_buffer
 *   waitforbytes
 *   data_descriptor
 *   callback_script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to script the writing of a block of
 *   data to several memory addresses at once in the background.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   list_of_buffer_size
 *   |html </b>
 *   - TCL list of buffer
 *
 *   |html <b>
 *   waitforbytes
 *   |html </b>
 *   - Minimum number of bytes to read before invoking the script.
 *
 *   |html <b>
 *   data_desc
 *   |html </b>
 *   - To be initialized by globus_xio_data_descriptor_init. It can be NULL.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked to perform actual write. The command will be
 *   executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * EXAMPLE
 *   lassign [ globus_xio_register_writev $handle [list $buffer1 $buffer2] 
 *   $waitforbytes $data_desc [ subst { callback_proc $user_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_init, globus_xio_handle_create, 
 *   globus_xio_register_readv
 *
 *****/

%inline %{
  
  globus_result_t
    tcl_globus_xio_register_writev( globus_xio_handle_t handle,
                                    tcl_globus_iovec_t *rwv_list,
                                    long waitforbytes,
                                    globus_xio_data_descriptor_t data_desc,
                                    globus_xio_iovec_callback_t rwv_cb,
                                    void * rwv_user_arg)
    {
      globus_result_t retval;
      callbackData * data;

      SGDEBUG_LVL_FMT_1
        ( 20, "tcl_globus_xio_register_writev: rwv_list->iovec: %p", 
          rwv_list->iovec );

      SGDEBUG_LVL_FMT_1
        ( 20, "tcl_globus_xio_register_writev: rwv_list->iovcnt: %d", 
          rwv_list->iovcnt );

      /* 
       * Stick iovec and iovcnt information to callback data structure
       * so we can reclaim the space after the callback is completed.
       */
      data = (callbackData*)((callbackStruct *)rwv_user_arg)->s_api_data;
      
      data->buffer = (globus_xio_iovec_t*)rwv_list->iovec;
      data->iovec_cnt = rwv_list->iovcnt;

      retval = globus_xio_register_writev( handle,
                                           rwv_list->iovec,
                                           rwv_list->iovcnt,
                                           waitforbytes,
                                           data_desc,
                                           rwv_cb,
                                           rwv_user_arg );
      
      return retval;
    }
  
  %}


/**************************************************************************
 *
 *  TYPEMAPS - GLOBUS_XIO_ATTR_CNTL()
 *             GLOBUS_XIO_HANDLE_CNTL()
 *             GLOBUS_XIO_DATA_DESCRIPTOR_CNTL()
 *             GLOBUS_XIO_SERVER_CNTL()
 *
 **************************************************************************/

%typemap( in ) ( int cmd, ... ) ( driver_call_info_type driver_attr )
{
  Tcl_HashEntry *entry_ptr;
  ClientData *value;
  char* driver_type_key;
  char driver_key[MAX_HASH_KEY];
  int cmd;
  TclInterpInfo_type* interp_info;

  if (Tcl_GetIntFromObj(interp,$input,&$1) == TCL_ERROR) {
    /*
     * :TODO:
     * May need to expand the error message.
     * Try by sending non int value
     */
    SWIG_fail;
  }

  sprintf(driver_key, "%p", arg2);

  SGDEBUG_LVL_FMT_1( 10, "typemap: in: driver_key: %s", driver_key );

  /* :TODO: Need routine to reclaim this memory */
  interp_info = globus_libc_malloc( sizeof( TclInterpInfo_type ) );
  interp_info->m_interp = interp;
  interp_info->m_results = Tcl_NewListObj(0,0);

  driver_attr.m_driver_type = driver_key;
  driver_attr.m_cmd = $1;
  driver_attr.m_var_argc = ( objc - ATTR_ARG_LENGTH );
  driver_attr.m_var_argv = (void *)&objv[4];
  driver_attr.m_interp_info = interp_info;
  
  SGDEBUG_LVL_FMT_1( 10, "typemap: in: objc: %d", objc );
  SGDEBUG_LVL_FMT_1( 10, "typemap: in: cmd: %d", $input );
  SGDEBUG_LVL_FMT_1
    ( 10, "typemap: in: m_var_argc: %d", driver_attr.m_var_argc );
  SGDEBUG_LVL_FMT_1
    ( 10, "typemap: in: m_var_argv: %p", driver_attr.m_var_argv );
  SGDEBUG_LVL_FMT_1( 10, "typemap: in: m_interp_info: %p", 
                     driver_attr.m_interp_info );

  /*
   * ---------------------------------------------------------------------
   * Look up the driver type
   * ---------------------------------------------------------------------
   */
  entry_ptr = Tcl_FindHashEntry(&tcl_globus_xio_driver_type_hashtbl, 
				driver_key);
  
  if ( entry_ptr != NULL )
    {
      /* Get driver name from the first hash table */
      driver_type_key = (char *)Tcl_GetHashValue(entry_ptr);
      SGDEBUG_LVL_FMT_1
        ( 10, "typemap: in: driver type key: %s", driver_type_key );
    }
  else
    {
      /*
       * TODO: Add text describing error
       */
      globus_libc_fprintf(stderr, "Driver type not found\n");
      return TCL_ERROR;
    }

  //---------------------------------------------------------------------
  // Look up the routines for this driver type
  //---------------------------------------------------------------------

  entry_ptr = Tcl_FindHashEntry(&tcl_globus_xio_driver_func_map_hashtbl,
				driver_type_key);
  
  if ( entry_ptr != NULL )
    {      
      value = Tcl_GetHashValue(entry_ptr);
      driver_attr.m_driver_info = (XIODriverFunctionMap_type *)value;
    }
  else
    {
      /*
       * :TODO: Add text describing error
       */
      globus_libc_fprintf(stderr, "Conversion routines not found\n");
      return TCL_ERROR;
    }

  $2 = &driver_attr;  

} // typemap( in, numinputs=0 )


%typemap( argout ) ( ... ) 
{
  
  TclInterpInfo_type* interp_info =
    ((TclInterpInfo_type*)((driver_call_info_type *)$1)->m_interp_info);

  SGDEBUG_LVL_FMT_1( 10, "typemap: argout: m_interp_info: %p",
                     (void*)interp_info );
  
  SGDEBUG_LVL_FMT_1( 10, "typemap: argout: m_results: %p",
                     (void*)(interp_info->m_results) );

  Tcl_ListObjAppendList( interp, Tcl_GetObjResult(interp),
                         interp_info->m_results );
  
  SGDEBUG_LVL_FMT( 10, "Tcl_ListObjAppendList" );

} // typemap( argout )


/**************************************************************************
 *
 *  GLOBUS_XIO_ATTR_CNTL()
 *
 **************************************************************************/

%rename(globus_xio_attr_cntl) tcl_globus_xio_attr_cntl;

%exception tcl_globus_xio_attr_cntl GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_xio_attr_cntl( globus_xio_attr_t attr,
                              globus_xio_driver_t driver,
                              int cmd,
                              ... )
    {
      driver_call_info_type* parms = NULL;
      
      va_list arguments;
      
      va_start( arguments, cmd );
      
      parms = va_arg(arguments, driver_call_info_type *);
      
      va_end( arguments );
      
      return (*(parms->m_driver_info->m_attr_cntl.m_function_call))
        ( attr,
          driver,
          parms );
    }
  %}


/**************************************************************************
 *
 *  GLOBUS_XIO_HANDLE_CNTL()
 *
 **************************************************************************/

%rename(globus_xio_handle_cntl) tcl_globus_xio_handle_cntl;

%exception tcl_globus_xio_handle_cntl GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_xio_handle_cntl( globus_xio_handle_t handle,
                                globus_xio_driver_t driver,
                                int cmd,
                                ... )
    {
      driver_call_info_type* parms = NULL;
      
      va_list arguments;
      
      va_start( arguments, cmd );
      
      parms = va_arg(arguments, driver_call_info_type *);
      
      va_end( arguments );
      
      return (*(parms->m_driver_info->m_handle_cntl.m_function_call))
        ( handle,
          driver,
          parms );
    }
  %}


/**************************************************************************
 *
 *  GLOBUS_XIO_DATA_DESCRIPTOR_CNTL()
 *
 **************************************************************************/

%rename(globus_xio_data_descriptor_cntl) tcl_globus_xio_data_descriptor_cntl;

%exception tcl_globus_xio_data_descriptor_cntl GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t tcl_globus_xio_data_descriptor_cntl
    ( globus_xio_data_descriptor_t data_descriptor,
      globus_xio_driver_t driver,
      int cmd,
      ... )
    {
      driver_call_info_type* parms = NULL;      
      va_list arguments;
      
      va_start( arguments, cmd );
      
      parms = va_arg(arguments, driver_call_info_type *);
      
      va_end( arguments );
      
      return (*(parms->m_driver_info->m_data_desc_cntl.m_function_call))
        ( data_descriptor,
          driver,
          parms );
    }
  %}


/**************************************************************************
 *
 *  GLOBUS_XIO_SERVER_CNTL()
 *
 **************************************************************************/

%rename(globus_xio_server_cntl) tcl_globus_xio_server_cntl;

%exception tcl_globus_xio_server_cntl GLOBUS_EXCEPTION_RESULT_T;

%inline %{

  globus_result_t
    tcl_globus_xio_server_cntl( globus_xio_server_t server,
                                globus_xio_driver_t driver,
                                int cmd,
                                ... )
    {
      driver_call_info_type* parms = NULL;      
      va_list arguments;
      
      va_start( arguments, cmd );
      
      parms = va_arg(arguments, driver_call_info_type *);
      
      va_end( arguments );
      
      return (*(parms->m_driver_info->m_server_cntl.m_function_call))
        ( server,
          driver,
          parms );
    }
%}


#endif /* HAVE_GLOBUS_XIO_H */
#endif /* SWIG */















/****f* globus_xio/globus_xio_attr_init
 * NAME
 *   globus_xio_attr_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_init
 *   |html </b>
 *
 * DESCRIPTION
 *   This command is used to initialize Globus XIO driver attribute. Upon
 *   return from this command,
 *   |html <i>
 *   attr
 *   |html </i>
 *   will be initialized. This attribute must be destroyed in order to
 *   free resources associated with it.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_init returns 0 and the driver attribute. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_attr_init ] result driver_attr
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_attr_cntl, globus_xio_attr_copy, globus_xio_attr_destroy
 *
 *****/


/****f* globus_xio/globus_xio_attr_cntl
 * NAME
 *   globus_xio_attr_cntl
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   cmd
 *   var_args
 *   |html </i>
 *
 * DESCRIPTION
 *   This command provides a means to access the 
 *   |html <i>
 *   attr
 *   |html </i> structure. 
 *   What exactly this command does is determined by the value in 
 *   the parameter 
 *   |html <i>
 *   cmd
 *   |html </i> and the value of the parameter 
 *   |html <i>
 *   driver.
 *   |html </i>
 *   When the driver parameter is NULL it indicates that this command
 *   applies to general globus_xio values. If it is not NULL it 
 *   indicates that the command will effect driver specific values. 
 *   Each driver is responsible for defining its own enumeration of 
 *   values for
 *   |html <i>
 *   cmd
 *   |html </i>
 *   and the 
 *   |html <i>
 *   var_args
 *   |html </i>
 *   associated with that command.
 *
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - The attribute structure to be manipulated
 *
 *   |html <b>
 *   driver
 *   |html </b>
 *   - This parameter indicates which driver the user would like 
 *   to perform the requested operation. If this parameter is NULL 
 *   this request will be scoped to general attribute functions.
 *
 *   |html <b>
 *   cmd
 *   |html </b>
 *   - An enum that determines what specific operation the user is 
 *   requesting. Each driver will determine the value for this 
 *   enumeration. 
 *
 *   |html <b>
 *   var_args
 *   |html </b>
 *   - These values are associated with the cmd values defined by the driver.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   |html <ul>
 *   TCL sample script to illustrate how to use this command.
 *
 *   More explanation about cmd and var args.
 *   |html </ul>
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_attr_init, globus_xio_attr_copy, globus_xio_attr_destroy
 *
 *****/


/****f* globus_xio/globus_xio_attr_copy
 * NAME
 *   globus_xio_attr_copy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_copy
 *   |html </b>
 *   |html <i>
 *   src
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to copy an attribute structure.
 *
 *   |html <ul><b>
 *   src
 *   |html </b>
 *   - Source attribute structure as returned by globus_xio_attr_cntl.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_copy returns 0 and the target attribute 
 *   structure. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_attr_copy $src ] result dst
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_attr_init, globus_xio_attr_cntl, globus_xio_attr_destroy
 *
 *****/


/****f* globus_xio/globus_xio_attr_destroy
 * NAME
 *   globus_xio_attr_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_destroy
 *   |html </b>
 *   |html <i>
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to clean up resources associated with an attribute
 *
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - Globus XIO attribute as returned by globus_xio_attr_init and 
 *   upon completion of this command all resources associated with this 
 *   attribute will be returned to the system and the 
 *   |html <i>
 *   attr 
 *   |html </i> will no longer be valid.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_attr_destroy $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_attr_init, globus_xio_attr_cntl, globus_xio_attr_copy
 *
 *****/


/****f* globus_xio/globus_xio_stack_init
 * NAME
 *   globus_xio_stack_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_stack_init
 *   |html </b>
 *   |html <i>
 *   stack
 *   stack_attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a stack object. 
 *
 *   |html <ul><b>
 *   stack
 *   |html </b>
 *   - The name of all the drivers the user wishes to load.
 *
 *   |html <b>
 *   stack_attr
 *   |html </b>
 *   -
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_stack_init returns 0 and the stack object. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_stack_init $stack_attr ] result stack
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_stack_push_driver, globus_xio_stack_destroy
 *
 *****/


/****f* globus_xio/globus_xio_stack_destroy
 * NAME
 *   globus_xio_stack_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_stack_destroy
 *   |html </b>
 *   |html <i>
 *   stack
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a stack object. 
 *
 *   |html <ul><b>
 *   stack
 *   |html </b>
 *   - The stack object the user wishes to destroy.
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_stack_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_stack_destroy $stack ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_stack_push_driver, globus_xio_stack_init
 *
 *****/


/****f* globus_xio/globus_xio_stack_push_driver
 * NAME
 *   globus_xio_stack_push_driver
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_stack_push_driver
 *   |html </b>
 *   |html <i>
 *   stack
 *   driver
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to push a driver onto a stack. 
 *
 *   |html <ul><b>
 *   stack
 *   |html </b>
 *   - The stack object represents the ordered lists of transform drivers.
 *
 *   |html <b>
 *   driver
 *   |html </b>
 *   - The driver object.
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_stack_push_driver returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_stack_push_driver $stack $driver] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_driver_load, globus_xio_stack_init
 *
 *****/


/****f* globus_xio/globus_xio_handle_create
 * NAME
 *   globus_xio_handle_create
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_create
 *   |html </b>
 *   |html <i>
 *   stack
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a handle for active opens
 *   (client-side connections).
 *
 *   |html <ul><b>
 *   stack
 *   |html </b>
 *   - A stack object as returned by globus_xio_stack_init.
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_create returns 0 and a handle. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_handle_create $stack ] result handle
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_stack_init, globus_xio_stack_push_driver
 *
 *****/


/****f* globus_xio/globus_xio_handle_cntl
 * NAME
 *   globus_xio_handle_cntl
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   cmd
 *   ...
 *   |html </i>
 *
 * DESCRIPTION
 *   This command allows the user to comunicate directly with a driver 
 *   in association with a handle object. The driver defines what 
 *   operations can be performed.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create
 *
 *   |html <b>
 *   driver
 *   |html </b>
 *   - This parameter indicates which driver the user would like 
 *   to perform the requested operation. If this parameter is NULL 
 *   this request will be scoped to general attribute functions.
 *
 *   |html <b>
 *   cmd
 *   |html </b>
 *   - An enum that determines what specific operation the user is 
 *   requesting. Each driver will determine the value for this 
 *   enumeration.
 *
 *   |html <b>
 *   var args
 *   |html </b>
 *   - These values are associated with the cmd values defined by the driver.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_handle_create
 *
 *****/



/****f* globus_xio/globus_xio_data_descriptor_init
 * NAME
 *   globus_xio_data_descriptor_init
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   globus_xio_data_descriptor_init
 *   |html </b>
 *   |html <i>
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to initialize a data descriptor.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_data_descriptor_init returns 0 and data
 *   descriptor used with the handle. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_data_descriptor_init $handle ] result data_desc
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_destroy, globus_xio_data_descriptor_cntl
 *
 *****/


/****f* globus_xio/globus_xio_data_descriptor_destroy
 * NAME
 *   globus_xio_data_descriptor_destroy
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   globus_xio_data_descriptor_destroy
 *   |html </b>
 *   |html <i>
 *   data_desc
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy a data descriptor.
 *
 *   |html <ul><b>
 *   data_desc
 *   |html </b>
 *   - A data descriptor as returned by globus_xio_data_descriptor_init.
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_data_descriptor_destroy returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_data_descriptor_destroy $data_desc ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_init, globus_xio_data_descriptor_cntl
 *
 *****/


/****f* globus_xio/globus_xio_data_descriptor_cntl
 * NAME
 *   globus_xio_data_descriptor_cntl
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_data_descriptor_cntl
 *   |html </b>
 *   |html <i>
 *   data_desc
 *   driver
 *   cmd
 *   var_args
 *   |html </i>
 *
 * DESCRIPTION
 *   This command allows the user to comunicate directly with a driver 
 *   in association with a data descriptor. The driver defines what 
 *   operations can be performed.
 *
 *   |html <ul><b>
 *   data_desc
 *   |html </b>
 *   - The data descriptor to be manipulated
 *
 *   |html <b>
 *   driver
 *   |html </b>
 *   - This parameter indicates which driver the user would like 
 *   to perform the requested operation. If this parameter is NULL 
 *   this request will be scoped to all drivers.
 *
 *   |html <b>
 *   cmd
 *   |html </b>
 *   - An enum that determines what specific operation the user is 
 *   requesting. Each driver will determine the value for this 
 *   enumeration.
 *
 *   |html <b>
 *   var_args
 *   |html </b>
 *   - These values are associated with the 
 *   |html <i>
 *   cmd
 *   |html </i>
 *   values defined by the driver.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_data_descriptor_cntl returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_init, globus_xio_data_descriptor_destroy
 *
 *****/



/****f* globus_xio/globus_xio_open
 * NAME
 *   globus_xio_open
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_open
 *   |html </b>
 *   |html <i>
 *   handle
 *   contact_string
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create an open handle in blocking mode.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   contact_string
 *   |html </b>
 *   - This string represents the resources that the user wishes to 
 *   contact.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - This attribute can be used to change the behavior of a handle and
 *   globus_xio_attr_cntl should precede the calling of this command in
 *   this case. This attribute can be NULL. 
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_open returns 0. On error, this command will 
 *   raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_open $handle $contact_string $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_close, globus_xio_handle_create, globus_xio_attr_cntl
 *
 *****/


/****f* globus_xio/globus_xio_close
 * NAME
 *   globus_xio_close
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_close
 *   |html </b>
 *   |html <i>
 *   handle
 *   attr
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to close the handle in blocking mode.
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Globus XIO attributes as returned by globus_xio_attr_cnt.l 
 *   It can be NULL.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_close returns 0. On error, this command will 
 *   raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_close $handle $attr ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_open, globus_xio_attr_cntl
 *
 *****/



/****f* globus_xio/globus_xio_register_open
 * NAME
 *   globus_xio_register_open
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_register_open
 *   |html </b>
 *   |html <i>
 *   handle
 *   contact_string
 *   attr
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to script the opening of a handle based on 
 *   the state contained in the given stack in the background. As soon 
 *   as the operation completes, the script is
 *   called.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create or 
 *   globus_xio_server_register_accept.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Determine how to open the handle. It can be NULL.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the operation completes.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_register_open returns 0. On error, this 
 *   command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> mechanism.

 * EXAMPLE
 *   lassign [ globus_xio_register_open $handle $contact_string $attr
 *   [ subst { callback user_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_register_close
 *
 *****/



/****f* globus_xio/globus_xio_register_close
 * NAME
 *   globus_xio_register_close
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_register_close
 *   |html </b>
 *   |html <i>
 *   handle
 *   attr
 *   callback_script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to destroy for a handle in the 
 *   background. As soon as the operation completes, the callback 
 *   script is called.
 *
 *   |html <ul><b>
 *   handle
 *   |html </b>
 *   - Globus XIO handle as returned by globus_xio_handle_create.
 *
 *   |html <b>
 *   attr
 *   |html </b>
 *   - Globus XIO attributes as returned by globus_xio_attr_cntl and
 *   it determines how to close the handle. It can be NULL.
 *
 *   |html <b>
 *   callback_script
 *   |html </b>
 *   - TCL script invoked once the operation completes.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 * 
 * RETURN VALUE
 *   On success globus_xio_register_close returns 0. On error, this 
 *   command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> mechanism.
 *
 * EXAMPLE
 *   lassign [ globus_xio_register_close $handle $attr 
 *   [ subst { callback_proc user_arg } ] ] result
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_register_open
 *
 *****/



/****f* globus_xio/globus_xio_server_create
 * NAME
 *   globus_xio_server_create
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_server_create
 *   |html </b>
 *   |html <i>
 *   attr
 *   stack
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to create a server object which can then be used 
 *   to accept connections.
 *
 *   |html <ul><b>
 *   attr
 *   |html </b>
 *   - An attributre structure used to alter the default server 
 *   initialization. This will mostly be used in a driver specific manner. 
 *   It can be NULL.
 *
 *   |html <b>
 *   stack
 *   |html </b>
 *   - Globus XIO stack as returned by globus_xio_stack_init
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_server_create returns 0 and a server object.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_server_create $attr $stack ] result server_obj  
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_server_register_accept, globus_xio_server_get_contact_string
 *
 *****/


/****f* globus_xio/globus_xio_server_accept
 * NAME
 *   globus_xio_server_accept
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_server_accept
 *   |html </b>
 *   |html <i>
 *   server_object
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is usde to accept a connection on the given server object. 
 *
 *   |html <ul><b>
 *   server_object
 *   |html </b>
 *   - A server object as returned by globus_xio_server_create.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_server_close returns 0 and a server handle.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_server_accept $server_object ] result server_handle  
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_server_create, globus_xio_server_close
 *
 *****/


/****f* globus_xio/globus_xio_server_close
 * NAME
 *   globus_xio_server_close
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_server_close
 *   |html </b>
 *   |html <i>
 *   server_object
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to post a blocking server close.
 *
 *   |html <ul><b>
 *   server_object
 *   |html </b>
 *   - A server object as returned by globus_xio_server_create.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_server_close returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_server_close $server_object ] result  
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_server_accept
 *
 *****/


/****f* globus_xio/globus_xio_server_cntl
 * NAME
 *   globus_xio_server_cntl
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_server_cntl
 *   |html </b>
 *   |html <i>
 *   server
 *   driver
 *   cmd
 *   ...
 *   |html </i>
 *
 * DESCRIPTION
 *   This command allows the user to comunicate directly with 
 *   a driver in association with a server object. The driver 
 *   defines what operations can be performed.
 *
 *   |html <ul><b>
 *   server
 *   |html </b>
 *   - A server object as returned by globus_xio_server_create.
 *
 *   |html <b>
 *   driver
 *   |html </b>
 *   - This parameter indicates which driver the user would like 
 *   to perform the requested operation.
 *
 *   |html <b>
 *   cmd
 *   |html </b>
 *   - An enum that determines what specific operation the user is 
 *   requesting. Each driver will determine the value for this enumeration.
 *
 *   |html <b>
 *   var args
 *   |html </b>
 *   - These values are associated with the cmd values defined by the driver.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_server_cntl returns 0. 
 *   On error, this command will raise an error and must be caught using 
 *   the appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_server_create
 *
 *****/


/****f* globus_xio/globus_xio_server_register_accept
 * NAME
 *   globus_xio_server_register_accept
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_server_register_accept
 *   |html </b>
 *   |html <i>
 *   server_obj
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to post asynchronous accept. Once the 
 *   operation has completed the script will be called.
 *
 *   |html <ul><b>
 *   server_obj
 *   |html </b>
 *   - Globus XIO server object as returned by globus_xio_server_create.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked after the operation has been completed. 
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_server_register_accept returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_server_register_accept $server_obj 
 *   { callback_proc $user_arg } ] result   
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_server_create, globus_xio_server_get_contact_string,
 *   globus_xio_server_register_close
 *
 *****/


/****f* globus_xio/globus_xio_server_register_close
 * NAME
 *   globus_xio_server_register_close
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_server_register_close
 *   |html </b>
 *   |html <i>
 *   server_obj
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to post asynchronous close to a server_obj. 
 *   Once the operation has completed the script will be called.
 *
 *   |html <ul><b>
 *   server_obj
 *   |html </b>
 *   - Globus XIO server object as returned by globus_xio_server_create.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked after the operation has been completed. 
 *   The command will be executed exactly once in global scope.
 *   If an error occurs while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_server_register_close returns 0.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   bgerror
 *   |html </i> command.
 *
 * EXAMPLE
 *   lassign [ globus_xio_server_register_close $server_obj 
 *   { callback_proc $user_arg } ] result   
 *
 * BUGS
 *   None
 *
 * TODO
 *   |html <font color="Red">
 *   TCL sample script to illustrate how to use this command.
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_server_create, globus_xio_server_register_accept,
 *   globus_xio_server_get_contact_string
 *
 *****/


/****f* globus_xio/globus_xio_server_get_contact_string
 * NAME
 *   globus_xio_server_get_contact_string
 *
 * SYNOPSIS
 *   |html <b>
 *   package require  globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_server_get_contact_string
 *   |html </b>
 *   |html <i>
 *   server_obj
 *   |html </i>
 *
 * DESCRIPTION
 *   This command is used to get contact string for a server. 
 *
 *   |html <ul><b>
 *   server_obj
 *   |html </b>
 *   - Globus XIO server object as returned by globus_xio_server_create.
 *
 *   |html </ul>
 *
 * RETURN VALUE
 *
 *   On success globus_xio_server_get_contact_string returns 0 and a string
 *   containing contact_string information.
 *   On error, this command will raise an error and must be caught using the
 *   appropriate
 *   |html <i>
 *   catch
 *   |html </i> command.

 * EXAMPLE
 *   lassign [ globus_xio_server_get_contact_string $server_obj ] result
 *   contact_string
 *
 * BUGS
 *   None
 *
 * TODO
 *   None
 *
 * SEE ALSO
 *   globus_xio_server_create
 *
 *****/



/****************************************************************************
 *
 *
 * ROBODOC TYPE USER DOCUMENTATION
 *
 *
 ****************************************************************************/


/****************************************************************************
 *
 * GLOBUS XIO FILE DRIVER - ATTRIBUTES AND CNTLS
 *
 ****************************************************************************/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_FILE_SET_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_FILE_SET_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_FILE_SET_MODE
 *   mode
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the file create mode (default mode is 0644). Use this to set 
 *   the permissions a non-existent file.
 *
 * PARAMETERS
 *   mode - A bitwise OR of all the modes desired
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_file_mode_t
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_FILE_GET_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_FILE_GET_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_FILE_GET_MODE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the file create mode.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the current mode.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_FILE_SET_FLAGS
 * ATTRIBUTES
 *   GLOBUS_XIO_FILE_SET_FLAGS
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_FILE_SET_FLAGS
 *   flags
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the file open flags. The default flags specify to create the 
 *   file if it doesn't exist, open it for reading and writing, and 
 *   interpret it as a binary file. 
 *
 * PARAMETERS
 *   flags - A bitwise OR of all the flags desired
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_file_flag_t
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_FILE_GET_FLAGS
 * ATTRIBUTES
 *   GLOBUS_XIO_FILE_GET_FLAGS
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_FILE_GET_FLAGS
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the file open flags.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the current flags.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_FILE_SET_TRUNC_OFFSET
 * ATTRIBUTES
 *   GLOBUS_XIO_FILE_SET_TRUNC_OFFSET
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_FILE_SET_TRUNC_OFFSET
 *   offset
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the file truncate offset. Use this in conjunction with the 
 *   GLOBUS_XIO_FILE_TRUNC flag to truncate a file to a non-zero offset. 
 *   If the file was larger than offset bytes, the extra data is lost. 
 *   If the file was shorter or non-existent, it is extended and the 
 *   extended part reads as zeros. (default is 0)
 *
 * PARAMETERS
 *   offset - The desired size of the file.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 * SEE ALSO
 *   globus_xio_file_flag_t/GLOBUS_XIO_FILE_TRUNC
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_FILE_GET_TRUNC_OFFSET
 * ATTRIBUTES
 *   GLOBUS_XIO_FILE_GET_TRUNC_OFFSET
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_FILE_GET_TRUNC_OFFSET
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the file truncate offset.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the offset.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_FILE_SET_HANDLE
 * ATTRIBUTES
 *   GLOBUS_XIO_FILE_SET_HANDLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_FILE_SET_HANDLE
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the file handle to use. Do not open a new file, use this 
 *   preopened handle instead.
 *
 * PARAMETERS
 *   handle - Use this handle for the file. Note: close() will not be 
 *   called on this handle.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_FILE_GET_HANDLE
 * ATTRIBUTES
 *   GLOBUS_XIO_FILE_GET_HANDLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_FILE_GET_HANDLE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the file handle in use or in attr.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the file handle.
 *   If none is set, GLOBUS_XIO_TCP_INVALID_HANDLE will be set.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_FILE_GET_HANDLE
 * ATTRIBUTES
 *   GLOBUS_XIO_FILE_GET_HANDLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_FILE_GET_HANDLE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the file handle in use or in attr.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the file handle. 
 *   If none is set, GLOBUS_XIO_TCP_INVALID_HANDLE will be set. 
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_FILE_SEEK
 * ATTRIBUTES
 *   GLOBUS_XIO_FILE_SEEK
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_FILE_SEEK
 *   whence
 *   |html </i>
 *
 * DESCRIPTION
 *   Reposition read/write file offset.
 *
 * PARAMETERS
 *   whence - Specify how offset should be interpreted.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the actual file offset.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_file_whence_t
 *****/


/****t* globus_xio/globus_xio_file_flag_t
 * NAME
 *   globus_xio_file_flag_t
 *
 * DESCRIPTION
 *   File driver open flags. OR together all the flags you want.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_CREAT = O_CREAT
 *   |html </b>
 *   - Create a new file if it doesn't exist (default).
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_EXCL = O_EXCL
 *   |html </b>
 *   - Fail if file already exists.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_RDONLY = O_RDONLY
 *   |html </b>
 *   - Open for read only.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_WRONLY = O_WRONLY
 *   |html </b>
 *   - Open for write only.
 * 
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_RDWR = O_RDWR
 *   |html </b>
 *   - Open for reading and writing (default).
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_TRUNC = O_TRUNC
 *   |html </b>
 *   - Truncate file. 
 *   See also globus_xio_attr_cntl/GLOBUS_XIO_FILE_SET_TRUNC_OFFSET.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_APPEND = O_APPEND
 *   |html </b>
 *   - Open file for appending.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_BINARY = 0
 *   |html </b>
 *   - File is binary (default).
 * 
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_TEXT = 0
 *   |html </b>
 *   - File is text.
 *   |html </ul>
 *
 * SEE ALSO
 *   globus_xio_attr_cntl/GLOBUS_XIO_FILE_SET_FLAGS,
 *   globus_xio_attr_cntl/GLOBUS_XIO_FILE_SET_TRUNC_OFFSET
 *****/


/****t* globus_xio/globus_xio_file_mode_t
 * NAME
 *   globus_xio_file_mode_t
 *
 * DESCRIPTION
 *   File driver create mode. OR these modes together to get the mode 
 *   you want. 
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IRWXU
 *   |html </b>
 *   - User read, write, and execute.
 *   
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IRUSR
 *   |html </b>
 *   - User read.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IWUSR
 *   |html </b>
 *   - User write.
 *   
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IXUSR
 *   |html </b>
 *   - User execute.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IRWXG
 *   |html </b>
 *   - Group read, write, and execute.
 *   
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IRGRP
 *   |html </b>
 *   - Group read.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IWGRP
 *   |html </b>
 *   - Group write.
 *   
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IXGRP
 *   |html </b>
 *   - Group execute.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IRWXO
 *   |html </b>
 *   - Others read, write, and execute.
 *   
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IROTH
 *   |html </b>
 *   - Others read.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IWOTH
 *   |html </b>
 *   - Others write.
 *   
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_IXOTH
 *   |html </b>
 *   - Others execute.
 *   |html </ul>
 *
 * SEE ALSO
 *   globus_xio_attr_cntl/GLOBUS_XIO_FILE_SET_MODE
 *****/




/****t* globus_xio/globus_xio_file_whence_t
 * NAME
 *   globus_xio_file_whence_t
 *
 * DESCRIPTION
 *   File driver seek options.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_SEEK_SET = SEEK_SET
 *   |html </b>
 *   - Set the file pointer at the specified offset.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_SEEK_CUR = SEEK_CUR
 *   |html </b>
 *   - Set the file pointer at current position + offset.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_FILE_SEEK_END = SEEK_END
 *   |html </b>
 *   - Set the file pointer at size of file + offest.
 *   |html </ul>
 *
 * SEE ALSO
 *   globus_xio_handle_cntl/GLOBUS_XIO_FILE_SEEK
 *****/


/****************************************************************************
 *
 * GLOBUS XIO GSI DRIVER - ATTRIBUTES AND CNTLS
 *
 ****************************************************************************/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_CREDENTIAL
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_CREDENTIAL
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_CREDENTIAL
 *   credential
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the credential to be used.
 *
 * PARAMETERS
 *   credential - The credential to set. The credential structure 
 *   needs to remain valid for the lifetime of any XIO data structure 
 *   it is used by.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_CREDENTIAL
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_CREDENTIAL
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_GET_CREDENTIAL
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the credential to be used
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and a credential if a 
 *   credential was explicitly set prior to this call. It will not 
 *   return any credential automatically acquired during context 
 *   initizalization.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_GSSAPI_REQ_FLAGS
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_GSSAPI_REQ_FLAGS
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_GSSAPI_REQ_FLAGS
 *   req_flags
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the GSSAPI req_flags to be used.
 *
 * PARAMETERS
 *   req_flags - The req_flags to set.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/




/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_GSSAPI_REQ_FLAGS
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_GSSAPI_REQ_FLAGS
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_GET_GSSAPI_REQ_FLAGS
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the GSSAPI req_flags to be used
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the req_flags currently 
 *   in effect.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_PROXY_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_PROXY_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_PROXY_MODE
 *   proxy_mode
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the proxy mode. Note that changing the proxy mode changes the 
 *   req_flags.
 *
 * PARAMETERS
 *   proxy_mode - The proxy mode to set
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_PROXY_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_PROXY_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_GET_PROXY_MODE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the proxy mode. Note that changing the proxy mode changes the 
 *   req_flags.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the proxy mode currently 
 *   in effect.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_AUTHORIZATION_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_AUTHORIZATION_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_AUTHORIZATION_MODE
 *   authz_mode
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the authorization mode. 
 *
 * PARAMETERS
 *   authz_mode - The authorization mode to set
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_AUTHORIZATION_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_AUTHORIZATION_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_GET_AUTHORIZATION_MODE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the authorization mode. 
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the authorization mode 
 *   currently in effect.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_DELEGATION_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_DELEGATION_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_DELEGATION_MODE
 *   delegation_mode
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the delegation mode. Note that: changing the delegation mode 
 *   changes the req_flags.
 *
 * PARAMETERS
 *   delegation_mode - The delegation mode to set.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_DELEGATION_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_DELEGATION_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_GET_DELEGATION_MODE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the delegation mode. Note that: changing the delegation mode 
 *   changes the req_flags.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the delegation mode 
 *   currently in effect.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_SSL_COMPATIBLE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_SSL_COMPATIBLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_SSL_COMPATIBLE
 *   ssl_mode
 *   |html </i>
 *
 * DESCRIPTION
 *   Make the wire protocol SSL compatible. This implies no wrapping of 
 *   security tokens and no delegation. Note that: changing the SSL
 *   compatibility mode changes the req_flags.
 *
 * PARAMETERS
 *   ssl_mode - The ssl compatibility mode to use.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_ANON
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_ANON
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_ANON
 *   anon_mode
 *   |html </i>
 *
 * DESCRIPTION
 *   Do anonymous authentication. Note that: changing the SSL
 *   compatibility mode changes the req_flags and the wrapping mode.
 *
 * PARAMETERS
 *   anon_mode - The ssl compatibility mode to use.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_WRAP_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_WRAP_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_WRAP_MODE
 *   wrap_mode
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the wrapping mode. This mode determines whether tokens will be 
 *   wrapped with a Globus IO style header or not.
 *
 * PARAMETERS
 *   wrap_mode - The wrapping mode to use.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_WRAP_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_WRAP_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_GET_WRAP_MODE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the wrapping mode. This mode determines whether tokens will be 
 *   wrapped with a Globus IO style header or not.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the wrapping mode 
 *   currently in use.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_BUFFER_SIZE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_BUFFER_SIZE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_BUFFER_SIZE
 *   buffer_size
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the read buffer size. The read buffer is used for buffering 
 *   wrapped data, is initialized with a default size of 128K and 
 *   scaled dynamically to always be able to fit whole tokens.
 *
 * PARAMETERS
 *   buffer_size - The size of the read buffer.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_BUFFER_SIZE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_BUFFER_SIZE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_GET_BUFFER_SIZE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the read buffer size. The read buffer is used for buffering 
 *   wrapped data, is initialized with a default size of 128K and 
 *   scaled dynamically to always be able to fit whole tokens.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the buffer size 
 *   currently in use.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_PROTECTION_LEVEL
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_PROTECTION_LEVEL
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_PROTECTION_LEVEL
 *   protection_level
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the protection level. Note: changing the proxy mode changes 
 *   the req_flags
 *
 * PARAMETERS
 *   protection_level - The protection level to set.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_PROTECTION_LEVEL
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_PROTECTION_LEVEL
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_GET_PROTECTION_LEVEL
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the protection level
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the current protection
 *   level.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_TARGET_NAME
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_TARGET_NAME
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_SET_TARGET_NAME
 *   target_name
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the expected peer name.
 *
 * PARAMETERS
 *   target_name - The expected peer name.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_TARGET_NAME
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_TARGET_NAME
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_GET_TARGET_NAME
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the expected peer name.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the expected peer name.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_GSI_FORCE_SERVER_MODE
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_FORCE_SERVER_MODE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_GSI_FORCE_SERVER_MODE
 *   server_mode
 *   |html </i>
 *
 * DESCRIPTION
 *   Force the server mode setting. This explicitly sets the directionality 
 *   of context establishment and delegation.
 *
 * PARAMETERS
 *   server_mode - The server mode.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_GSI_SET_CREDENTIAL
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_SET_CREDENTIAL
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_GSI_SET_CREDENTIAL
 *   credential
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the credential to be used. Note: If this is called with the 
 *   handle_cntl, there must be no outstanding operations on the handle.
 *
 * PARAMETERS
 *   credential - The credential to set. The credential structure 
 *   needs to remain valid for the lifetime of any XIO data structure 
 *   it is used by.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_GSI_GET_CREDENTIAL
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_CREDENTIAL
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_GSI_GET_CREDENTIAL
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the credential to be used
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and a credential if a 
 *   credential was explicitly set prior to this call. It will not 
 *   return any credential automatically acquired during context 
 *   initizalization.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_GSI_GET_CONTEXT
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_CONTEXT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_GSI_GET_CONTEXT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the GSS context.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the GSS context.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_GSI_GET_DELEGATED_CRED
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_DELEGATED_CRED
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_GSI_GET_DELEGATED_CRED
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the delegated credential.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the delegated credential.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_GSI_GET_PEER_NAME
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_PEER_NAME
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_GSI_GET_PEER_NAME
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the name of the peer.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the GSS name of the peer.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_GSI_GET_LOCAL_NAME
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_GET_LOCAL_NAME
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_GSI_GET_LOCAL_NAME
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the GSS name associated with the local credentials.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the GSS name of the local 
 *   credentials.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/




/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_GSI_INIT_DELEGATION
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_INIT_DELEGATION
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_GSI_INIT_DELEGATION
 *   credential
 *   restriction_oids
 *   restriction_buffers
 *   time_req
 *   |html </i>
 *
 * DESCRIPTION
 *   Initialize delegation-at-any-time process
 *
 *   |html <ul><b>
 *   credential
 *   |html </b>
 *   - The GSS credential to delegate.
 *
 *   |html <b>
 *   restriction_oids
 *   |html </b>
 *   - The OIDS for X.509 extensions to embed in the delegated credential.
 *
 *   |html <b>
 *   restriction_buffers
 *   |html </b>
 *   - The corresponding bodies for the X.509 extensions.
 *
 *   |html <b>
 *   time_req
 *   |html </b>
 *   - 	The lifetime of the delegated credential.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_GSI_REGISTER_INIT_DELEGATION
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_REGISTER_INIT_DELEGATION
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_GSI_REGISTER_INIT_DELEGATION
 *   credential
 *   restriction_oids
 *   restriction_buffers
 *   time_req
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   Initialize non-blocking delegation-at-any-time process.
 *
 *   |html <ul><b>
 *   credential
 *   |html </b>
 *   - The GSS credential to delegate.
 *
 *   |html <b>
 *   restriction_oids
 *   |html </b>
 *   - The OIDS for X.509 extensions to embed in the delegated credential.
 *
 *   |html <b>
 *   restriction_buffers
 *   |html </b>
 *   - The corresponding bodies for the X.509 extensions.
 *
 *   |html <b>
 *   time_req
 *   |html </b>
 *   - 	The lifetime of the delegated credential.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the operation completes. If an error occurs 
 *   while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_GSI_ACCEPT_DELEGATION
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_ACCEPT_DELEGATION
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_GSI_ACCEPT_DELEGATION
 *   restriction_oids
 *   restriction_buffers
 *   time_req
 *   |html </i>
 *
 * DESCRIPTION
 *   Accept delegation-at-any-time process.
 *
 *   |html <ul><b>
 *   restriction_oids
 *   |html </b>
 *   - The OIDS for X.509 extensions to embed in the delegated credential.
 *
 *   |html <b>
 *   restriction_buffers
 *   |html </b>
 *   - The corresponding bodies for the X.509 extensions.
 *
 *   |html <b>
 *   time_req
 *   |html </b>
 *   - 	The lifetime of the delegated credential.
 *   |html </ul>

 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the delegated GSS 
 *   credential.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_GSI_REGISTER_ACCEPT_DELEGATION
 * ATTRIBUTES
 *   GLOBUS_XIO_GSI_REGISTER_ACCEPT_DELEGATION
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_GSI_REGISTER_ACCEPT_DELEGATION
 *   restriction_oids
 *   restriction_buffers
 *   time_req
 *   script
 *   |html </i>
 *
 * DESCRIPTION
 *   Accept non-blocking delegation-at-any-time process.
 *
 *   |html <b>
 *   restriction_oids
 *   |html </b>
 *   - The OIDS for X.509 extensions to embed in the delegated credential.
 *
 *   |html <b>
 *   restriction_buffers
 *   |html </b>
 *   - The corresponding bodies for the X.509 extensions.
 *
 *   |html <b>
 *   time_req
 *   |html </b>
 *   - 	The lifetime of the delegated credential.
 *
 *   |html <b>
 *   script
 *   |html </b>
 *   - TCL script invoked once the operation completes. If an error occurs 
 *   while executing the command, then the
 *   |html <b>
 *   bgerror
 *   |html </b>
 *   mechanism is used to report the error.
 *   |html </ul>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_gsi_protection_level_t
 * NAME
 *   globus_xio_gsi_protection_level_t
 *
 * DESCRIPTION
 *   Globus XIO GSI protection levels.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_PROTECTION_LEVEL_NONE 
 *   |html </b>
 *   - No security.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_PROTECTION_LEVEL_INTEGRITY
 *   |html </b>
 *   - Messages are signed.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_PROTECTION_LEVEL_PRIVACY
 *   |html </b>
 *   - Messages are signed and encrypted. 
 *   |html </ul>
 *
 * SEE ALSO
 *   globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_PROTECTION_LEVEL,
 *   globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_PROTECTION_LEVEL.
 *****/


/****t* globus_xio/globus_xio_gsi_delegation_mode_t
 * NAME
 *   globus_xio_gsi_delegation_mode_t
 *
 * DESCRIPTION
 *   Globus XIO GSI delegation modes.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_DELEGATION_MODE_NONE 
 *   |html </b>
 *   - No delegation.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_DELEGATION_MODE_LIMITED 
 *   |html </b>
 *   - Delegate a limited proxy.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_DELEGATION_MODE_FULL 
 *   |html </b>
 *   - Delegate a full proxy.
 *   |html </ul>
 *
 * SEE ALSO
 *   globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_DELEGATION_MODE,
 *   globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_DELEGATION_MODE.
 *****/



/****t* globus_xio/globus_xio_gsi_proxy_mode_t
 * NAME
 *   globus_xio_gsi_proxy_mode_t
 *
 * DESCRIPTION
 *   Globus XIO GSI proxy modes.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_PROXY_MODE_FULL
 *   |html </b>
 *   - Accept only full proxies.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_PROXY_MODE_LIMITED 
 *   |html </b>
 *   - Accept full proxies and limited proxies if they are the only 
 *   limited proxy in the cert chain.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_PROXY_MODE_MANY 
 *   |html </b>
 *   - Accept both full and limited proxies unconditionally.
 *   |html </ul>
 *
 * SEE ALSO
 *   globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_PROXY_MODE,
 *   globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_PROXY_MODE.
 *****/



/****t* globus_xio/globus_xio_gsi_authorization_mode_t
 * NAME
 *   globus_xio_gsi_authorization_mode_t
 *
 * DESCRIPTION
 *   Globus XIO GSI authorization modes.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_NO_AUTHORIZATION
 *   |html </b>
 *   - Do not perform any authorization. This will cause a error when 
 *   used in conjunction with delegation on the init/client side. 
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_SELF_AUTHORIZATION 
 *   |html </b>
 *   - Authorize the peer if the peer has the same identity as ourselves.
 *
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_IDENTITY_AUTHORIZATION
 *   |html </b>
 *   - Authorize the peer if the peer identity matches the identity 
 *   set in the target name.
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_GSI_HOST_AUTHORIZATION
 *   |html </b>
 *   - Authorize the peer if the identity of the peer matches the 
 *   identity of the peer hostname.
 *   |html </ul>
 *
 * SEE ALSO
 *   globus_xio_attr_cntl/GLOBUS_XIO_GSI_SET_AUTHORIZATION_MODE,
 *   globus_xio_attr_cntl/GLOBUS_XIO_GSI_GET_AUTHORIZATION_MODE.
 *****/


/****************************************************************************
 *
 * GLOBUS XIO TCP DRIVER - ATTRIBUTES AND CNTLS
 *
 ****************************************************************************/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_SERVICE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_SERVICE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_SERVICE
 *   service_name
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the tcp service name to bind to. Used only on attrs for
 *   globus_xio_server_create().
 *
 * PARAMETERS
 *   service_name - The service name to use when setting up the listener.
 *   If the service name cannot be resolved, the port (if one is set) will
 *   be used instead.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/




/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_SERVICE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_SERVICE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_SERVICE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP service name to bind to.
 *
 * PARAMETERS
 *   Parameters
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the service name.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_PORT
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_PORT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_PORT
 *   listener_port
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the TCP port number to bind to. Used only on attrs for
 *   globus_xio_server_create(). The default port number is 0
 *   (system assigned)
 *
 * PARAMETERS
 *   listener_port - The port number to use when setting up the listener.
 *   If the service name is also set, this will only be used if that can't
 *   be resolved.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_PORT
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_PORT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_PORT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP port number to bind to.
 *
 * PARAMETERS
 *   Parameters
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the port number.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_BACKLOG
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_BACKLOG
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_BACKLOG
 *   listener_backlog
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the listener backlog on a server. Used only on attrs for
 *   globus_xio_server_create(). The default backlog is -1 (system maximum).
 *
 * PARAMETERS
 *   This indicates the maximum length of the system's queue of pending
 *   connections. Any connection attempts when the queue is full will fail.
 *   If backlog is equal to -1, then the system-specific maximum queue
 *   length will be used.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_BACKLOG
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_BACKLOG
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_BACKLOG
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the listener backlog on an attr.
 *
 * PARAMETERS
 *   Parameters
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the listener backlog.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_LISTEN_RANGE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_LISTEN_RANGE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_LISTEN_RANGE
 *   listener_min_port
 *   listener_max_port
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the TCP port range to confine the server to. Used only on attrs
 *   for globus_xio_server_create() where no specific service or port has
 *   been set. It overrides the range set in the GLOBUS_TCP_PORT_RANGE env
 *   variable. If 'restrict port' is true, the server's listening port will
 *   be constrained to the range specified.
 *
 * PARAMETERS
 *   |html <i>
 *   listener_min_port - The lower bound on the listener port.
 *   (default 0 -- no bound).
 *
 *   listener_max_port - The upper bound on the listener port.
 *   (default 0 -- no bound)
 *   |html </i>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_LISTEN_RANGE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_LISTEN_RANGE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_LISTEN_RANGE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP port range on an attr.
 *
 * PARAMETERS
 *   |html <i>
 *   listener_min_port_out - The lower bound will be stored here.
 *
 *   listener_max_port_out - The upper bound will be stored here.
 *   |html </i>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0, listener_min_port_out
 *   and listener_max_port_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_HANDLE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_HANDLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_HANDLE
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the TCP socket to use for a handle or server. Used only on attrs
 *   for globus_xio_server_create() or globus_xio_register_open().
 *
 * PARAMETERS
 *   handle - Use this handle (fd or SOCKET) for the listener or connection.
 *   Note: close() will not be called on this handle.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_HANDLE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_HANDLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_HANDLE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP socket handle on an attr, handle, or server.
 *
 * PARAMETERS
 *   handle_out - The TCP socket will be stored here. If none is set,
 *   GLOBUS_XIO_TCP_INVALID_HANDLE will be set.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and handle.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_INTERFACE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_INTERFACE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_INTERFACE
 *   interface
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the interface to bind a listener or connection to. Used only on
 *   attrs for globus_xio_server_create() or globus_xio_register_open.
 *
 * PARAMETERS
 *   interface - The interface to be used. Can be a hostname or numeric IP.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_INTERFACE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_INTERFACE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_INTERFACE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the interface on the attr.
 *
 * PARAMETERS
 *   interface_out - A pointer to the interface will be stored here.
 *   If one is set, NULL will be passed back. Otherwise, the interface
 *   will be duplicated with strdup() and the user should call free() on it.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and interface_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_RESTRICT_PORT
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_RESTRICT_PORT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_RESTRICT_PORT
 *   restrict_port
 *   |html </i>
 *
 * DESCRIPTION
 *   Enable or disable the listener or connector range constraints.
 *   Used only on attrs for globus_xio_server_create() or
 *   globus_xio_register_open(). This enables or ignores the port range
 *   found in the attr or in then env. By default, those ranges are enabled.
 *
 * PARAMETERS
 *   restrict_port - GLOBUS_TRUE to enable (default), GLOBUS_FALSE to
 *   disable.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_LISTEN_RANGE,
 *   globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_CONNECT_RANGE.
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_RESTRICT_PORT
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_RESTRICT_PORT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_RESTRICT_PORT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the restrict port flag.
 *
 * PARAMETERS
 *   restrict_port_out - The restrict port flag will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and restrict_port_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_REUSEADDR
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_REUSEADDR
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_REUSEADDR
 *   reuseaddr
 *   |html </i>
 *
 * DESCRIPTION
 *   Reuse addr when binding. Used only on attrs for
 *   globus_xio_server_create() or globus_xio_register_open() to determine
 *   whether or not to allow reuse of addresses when binding a socket to
 *   a port number.
 *
 * PARAMETERS
 *   resuseaddr - GLOBUS_TRUE to allow, GLOBUS_FALSE to disallow (default).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_REUSEADDR
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_REUSEADDR
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_REUSEADDR
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the reuseaddr flag on an attr.
 *
 * PARAMETERS
 *   resuseaddr_out - The reuseaddr flag will be stored here. 
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_NO_IPV6
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_NO_IPV6
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_NO_IPV6
 *   no_ipv6
 *   |html </i>
 *
 * DESCRIPTION
 *   Restrict to IPV4 only. Used only on attrs for globus_xio_server_create()
 *   or globus_xio_register_open(). Disallow IPV6 sockets from being used
 *   (default is to use either IPV4 or IPV6).
 *
 * PARAMETERS
 *   no_ipv6 - GLOBUS_TRUE to disallow ipv6, GLOBUS_FALSE to allow (default).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_NO_IPV6
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_NO_IPV6
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_NO_IPV6
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the no IPV6 flag on an attr.
 *
 * PARAMETERS
 *   no_ipv6_out - The no IPV6 flag will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and no_ipv6_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_CONNECT_RANGE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_CONNECT_RANGE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_CONNECT_RANGE
 *   connector_min_port
 *   connector_max_port
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the TCP port range to confine the server to. Used only on attrs
 *   for globus_xio_register_open(). It overrides the range set in the
 *   GLOBUS_TCP_SOURCE_RANGE env variable. If 'restrict port' is true,
 *   the connecting socket's local port will be constrained to the range
 *   specified.
 *
 * PARAMETERS
 *   |html <i>
 *   connector_min_port - The lower bound on the listener port.
 *   (default 0 -- no bound).
 *
 *   connector_max_port - The upper bound on the listener port.
 *   (default 0 -- no bound)
 *   |html </i>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_CONNECT_RANGE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_CONNECT_RANGE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_CONNECT_RANGE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP source port range on an attr.
 *
 * PARAMETERS
 *   |html <i>
 *   connector_min_port_out - The lower bound will be stored here.
 *
 *   connector_max_port_out - The upper bound will be stored here.
 *   |html </i>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0, connector_min_port_out
 *   and connector_max_port_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_KEEPALIVE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_KEEPALIVE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_KEEPALIVE
 *   keepalive
 *   |html </i>
 *
 * DESCRIPTION
 *   Enable TCP keepalive. Used on attrs for globus_xio_server_create(),
 *   globus_xio_register_open() and with globus_xio_handle_cntl() to
 *   determine whether or not to periodically send "keepalive" messages
 *   on a connected socket handle. This may enable earlier detection of
 *   broken connections.
 *
 * PARAMETERS
 *   keepalive - GLOBUS_TRUE to enable, GLOBUS_FALSE to disable (default).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_KEEPALIVE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_KEEPALIVE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_KEEPALIVE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP keepalive flag.
 *
 * PARAMETERS
 *   keepalive_out - The TCP keepalive flag will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and keepalive_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_LINGER
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_LINGER
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_LINGER
 *   linger
 *   linger_time
 *   |html </i>
 *
 * DESCRIPTION
 *   Set TCP linger. Used on attrs for globus_xio_server_create(),
 *   globus_xio_register_open() and with globus_xio_handle_cntl() to
 *   determine what to do when data is in the socket's buffer when the
 *   socket is closed. If linger is set to true, then the close operation
 *   will block until the socket buffers are empty, or the linger_time has
 *   expired. If this is enabled, any data remaining after the linger time
 *   has expired, will be discarded. If this is disabled, close finishes
 *   immediately, but the OS will still attempt to transmit the remaining
 *   data.
 *
 * PARAMETERS
 *   |html <i>
 *   linger - GLOBUS_TRUE to enable, GLOBUS_FALSE to disable (default).   
 *
 *   linger_time - The time (in seconds) to block at close time if linger
 *   is true and data is queued in the socket buffer.
 *   |html </i>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_LINGER
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_LINGER
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_LINGER
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP linger flag and time.
 *
 * PARAMETERS
 *   |html <i>
 *   linger_out - The linger flag will be stored here.
 *
 *   linger_time_out - The linger time will be set here.
 *   |html </i>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0, linger_out and
 *   linger_time_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_OOBINLINE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_OOBINLINE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_OOBINLINE
 *   oobinline
 *   |html </i>
 *
 * DESCRIPTION
 *   Receive out of band data (TCP urgent data) in normal stream. Used on
 *   attrs for globus_xio_server_create(), globus_xio_register_open() and
 *   with globus_xio_handle_cntl() to choose whether out-of-band data is
 *   received in the normal data queue. (Currently, there is no other way
 *   to receive OOB data).
 *
 * PARAMETERS
 *   oobinline - GLOBUS_TRUE to enable, GLOBUS_FALSE to disable (default).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_OOBINLINE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_OOBINLINE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_OOBINLINE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the oobinline flag.
 *
 * PARAMETERS
 *   oobinline_out - The oobinline flag will be stored here. 
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and oobinline_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_SNDBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_SNDBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_SNDBUF
 *   sndbuf
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the TCP socket send buffer size. Used on attrs for
 *   globus_xio_server_create(), globus_xio_register_open() and with
 *   globus_xio_handle_cntl() to set the size of the send buffer used
 *   on the socket.
 *
 * PARAMETERS
 *   sndbuf - The send buffer size in bytes to use. (default is system
 *   specific).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_SNDBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_SNDBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_SNDBUF
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP send buffer size on the attr or handle.
 *
 * PARAMETERS
 *   sndbuf_out - The send buffer size will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and sndbuf_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_RCVBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_RCVBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_RCVBUF
 *   rcvbuf
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the TCP socket receive buffer size. Used on attrs for
 *   globus_xio_server_create(), globus_xio_register_open() and with
 *   globus_xio_handle_cntl() to set the size of the receive buffer
 *   used on the socket. The receive buffer size is often used by the
 *   operating system to choose the appropriate TCP window size.
 *
 * PARAMETERS
 *   rcvbuf - The receive buffer size in bytes. (default is system specific).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_RCVBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_RCVBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_RCVBUF
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP receive buffer size on the attr or handle.
 *
 * PARAMETERS
 *   rcvbuf_out - The receive buffer size will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and rcvbuf_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_SET_NODELAY
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_NODELAY
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_SET_NODELAY
 *   nodelay
 *   |html </i>
 *
 * DESCRIPTION
 *   Disable Nagle's algorithm. Used on attrs for globus_xio_server_create(),
 *   globus_xio_register_open() and with globus_xio_handle_cntl() to determine
 *   whether or not to disable Nagle's algorithm. If set to GLOBUS_TRUE, the
 *   socket will send packets as soon as possible with no unnecessary delays
 *   introduced.
 *
 * PARAMETERS
 *   nodelay - GLOBUS_TRUE to disable nagle, GLOBUS_FALSE to enable (default).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_TCP_GET_NODELAY
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_NODELAY
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_TCP_GET_NODELAY
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP nodelay flag.
 *
 * PARAMETERS
 *   nodelay_out - The no delay flag will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and nodelay_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_HANDLE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_HANDLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_HANDLE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP socket handle on an attr, handle, or server.
 *
 * PARAMETERS
 *   handle_out - The TCP socket will be stored here. If none is set,
 *   GLOBUS_XIO_TCP_INVALID_HANDLE will be set.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and handle.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_SET_KEEPALIVE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_KEEPALIVE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_SET_KEEPALIVE
 *   keepalive
 *   |html </i>
 *
 * DESCRIPTION
 *   Enable TCP keepalive. Used on attrs for globus_xio_server_create(),
 *   globus_xio_register_open() and with globus_xio_handle_cntl() to
 *   determine whether or not to periodically send "keepalive" messages
 *   on a connected socket handle. This may enable earlier detection of
 *   broken connections.
 *
 * PARAMETERS
 *   keepalive - GLOBUS_TRUE to enable, GLOBUS_FALSE to disable (default).
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_KEEPALIVE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_KEEPALIVE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_KEEPALIVE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP keepalive flag.
 *
 * PARAMETERS
 *   keepalive_out - The TCP keepalive flag will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and keepalive_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_SET_LINGER
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_LINGER
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_SET_LINGER
 *   linger
 *   linger_time
 *   |html </i>
 *
 * DESCRIPTION
 *   Set TCP linger. Used on attrs for globus_xio_server_create(),
 *   globus_xio_register_open() and with globus_xio_handle_cntl() to
 *   determine what to do when data is in the socket's buffer when the
 *   socket is closed. If linger is set to true, then the close operation
 *   will block until the socket buffers are empty, or the linger_time has
 *   expired. If this is enabled, any data remaining after the linger time
 *   has expired, will be discarded. If this is disabled, close finishes
 *   immediately, but the OS will still attempt to transmit the remaining
 *   data.
 *
 * PARAMETERS
 *   |html <i>
 *   linger - GLOBUS_TRUE to enable, GLOBUS_FALSE to disable (default).   
 *
 *   linger_time - The time (in seconds) to block at close time if linger
 *   is true and data is queued in the socket buffer.
 *   |html </i>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_LINGER
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_LINGER
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_LINGER
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP linger flag and time.
 *
 * PARAMETERS
 *   |html <i>
 *   linger_out - The linger flag will be stored here.
 *
 *   linger_time_out - The linger time will be set here.
 *   |html </i>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0, linger_out and
 *   linger_time_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_SET_OOBINLINE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_OOBINLINE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_SET_OOBINLINE
 *   oobinline
 *   |html </i>
 *
 * DESCRIPTION
 *   Receive out of band data (TCP urgent data) in normal stream. Used on
 *   attrs for globus_xio_server_create(), globus_xio_register_open() and
 *   with globus_xio_handle_cntl() to choose whether out-of-band data is
 *   received in the normal data queue. (Currently, there is no other way
 *   to receive OOB data).
 *
 * PARAMETERS
 *   oobinline - GLOBUS_TRUE to enable, GLOBUS_FALSE to disable (default).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_OOBINLINE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_OOBINLINE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_OOBINLINE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the oobinline flag.
 *
 * PARAMETERS
 *   oobinline_out - The oobinline flag will be stored here. 
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and oobinline_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_SET_SNDBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_SNDBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_SET_SNDBUF
 *   sndbuf
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the TCP socket send buffer size. Used on attrs for
 *   globus_xio_server_create(), globus_xio_register_open() and with
 *   globus_xio_handle_cntl() to set the size of the send buffer used
 *   on the socket.
 *
 * PARAMETERS
 *   sndbuf - The send buffer size in bytes to use. (default is system
 *   specific).
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_SNDBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_SNDBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_SNDBUF
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP send buffer size on the attr or handle.
 *
 * PARAMETERS
 *   sndbuf_out - The send buffer size will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and sndbuf_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_SET_RCVBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_RCVBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_SET_RCVBUF
 *   rcvbuf
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the TCP socket receive buffer size. Used on attrs for
 *   globus_xio_server_create(), globus_xio_register_open() and with
 *   globus_xio_handle_cntl() to set the size of the receive buffer
 *   used on the socket. The receive buffer size is often used by the
 *   operating system to choose the appropriate TCP window size.
 *
 * PARAMETERS
 *   rcvbuf - The receive buffer size in bytes. (default is system specific).
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_RCVBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_RCVBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_RCVBUF
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP receive buffer size on the attr or handle.
 *
 * PARAMETERS
 *   rcvbuf_out - The receive buffer size will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and rcvbuf_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_SET_NODELAY
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_SET_NODELAY
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_SET_NODELAY
 *   nodelay
 *   |html </i>
 *
 * DESCRIPTION
 *   Disable Nagle's algorithm. Used on attrs for globus_xio_server_create(),
 *   globus_xio_register_open() and with globus_xio_handle_cntl() to determine
 *   whether or not to disable Nagle's algorithm. If set to GLOBUS_TRUE, the
 *   socket will send packets as soon as possible with no unnecessary delays
 *   introduced.
 *
 * PARAMETERS
 *   nodelay - GLOBUS_TRUE to disable nagle, GLOBUS_FALSE to enable (default).
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_NODELAY
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_NODELAY
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_NODELAY
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP nodelay flag.
 *
 * PARAMETERS
 *   nodelay_out - The no delay flag will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and nodelay_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_LOCAL_CONTACT
 * ATTRIBUTES
 *   GLOBUS_XIO_TCP_GET_LOCAL_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_LOCAL_CONTACT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get local socket info.
 *
 * PARAMETERS
 *   contact_string_out - A pointer to a contact string for the local end
 *   of a connected socket or listener will be stored here. The user should
 *   free() it when done with it. It will be in the format: <hostname>:<port>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the contact_string_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_server_get_contact_string
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_LOCAL_NUMERIC_CONTACT
 * ATTRIBUTES
 *   GLOBUS_XIO_TCP_GET_LOCAL_NUMERIC_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_LOCAL_NUMERIC_CONTACT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get local socket info.
 *
 * PARAMETERS
 *   contact_string_out - A pointer to a contact string for the local end
 *   of a connected socket or listener will be stored here. The user should
 *   free() it when done with it. It will be in the format: <ip>:<port>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the contact_string_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_REMOTE_CONTACT
 * ATTRIBUTES
 *   GLOBUS_XIO_TCP_GET_REMOTE_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_REMOTE_CONTACT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get remote socket info.
 *
 * PARAMETERS
 *   contact_string_out - A pointer to a contact string for the remote end
 *   of a connected socket will be stored here. The user should free() it
 *   when done with it. It will be in the format: <hostname>:<port>.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the contact_string_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_REMOTE_NUMERIC_CONTACT
 * ATTRIBUTES
 *   GLOBUS_XIO_TCP_GET_REMOTE_NUMERIC_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_REMOTE_NUMERIC_CONTACT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get remote socket info.
 *
 * PARAMETERS
 *   contact_string_out - A pointer to a contact string for the remote end
 *   of a connected socket will be stored here. The user should free() it
 *   when done with it. It will be in the format: <ip>:<port>.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the contact_string_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_server_cntl/GLOBUS_XIO_TCP_GET_HANDLE
 * ATTRIBUTES
 *     GLOBUS_XIO_TCP_GET_HANDLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_server_cntl
 *   |html </b>
 *   |html <i>
 *   server
 *   driver
 *   GLOBUS_XIO_TCP_GET_HANDLE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the TCP socket handle on an attr, handle, or server.
 *
 * PARAMETERS
 *   handle_out - The TCP socket will be stored here. If none is set,
 *   GLOBUS_XIO_TCP_INVALID_HANDLE will be set.
 *
 * RETURN VALUE
 *   On success globus_xio_server_cntl returns 0 and handle.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_server_cntl/GLOBUS_XIO_TCP_GET_LOCAL_CONTACT
 * ATTRIBUTES
 *   GLOBUS_XIO_TCP_GET_LOCAL_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_server_cntl
 *   |html </b>
 *   |html <i>
 *   server
 *   driver
 *   GLOBUS_XIO_TCP_GET_LOCAL_CONTACT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get local socket info.
 *
 * PARAMETERS
 *   contact_string_out - A pointer to a contact string for the local end
 *   of a connected socket or listener will be stored here. The user should
 *   free() it when done with it. It will be in the format: <hostname>:<port>
 *
 * RETURN VALUE
 *   On success globus_xio_server_cntl returns 0 and the contact_string_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_server_get_contact_string
 *****/


/****t* globus_xio/globus_xio_server_cntl/GLOBUS_XIO_TCP_GET_LOCAL_NUMERIC_CONTACT
 * ATTRIBUTES
 *   GLOBUS_XIO_TCP_GET_LOCAL_NUMERIC_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_server_cntl
 *   |html </b>
 *   |html <i>
 *   server
 *   driver
 *   GLOBUS_XIO_TCP_GET_LOCAL_NUMERIC_CONTACT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get local socket info.
 *
 * PARAMETERS
 *   contact_string_out - A pointer to a contact string for the local end
 *   of a connected socket or listener will be stored here. The user should
 *   free() it when done with it. It will be in the format: <ip>:<port>
 *
 * RETURN VALUE
 *   On success globus_xio_server_cntl returns 0 and the contact_string_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_tcp_send_flags_t
 * NAME
 *   globus_xio_tcp_send_flags_t
 *
 * DESCRIPTION
 *   TCP driver specific types.
 *
 * ATTRIBUTES
 *   |html <ul>
 *   o 
 *   |html <b>
 *   GLOBUS_XIO_TCP_SEND_OOB
 *   |html </b>
 *   - Use this with GLOBUS_XIO_TCP_SET_SEND_FLAGS to send a TCP message
 *   out of band (Urgent data flag set).
 *   |html </ul>
 *
 * SEE ALSO
 *   globus_xio_data_descriptor_cntl/GLOBUS_XIO_TCP_SET_SEND_FLAGS
 *****/


/****t* globus_xio/globus_xio_data_descriptor_cntl/GLOBUS_XIO_TCP_SET_SEND_FLAGS
 * ATTRIBUTES
 *   GLOBUS_XIO_TCP_SET_SEND_FLAGS
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_data_descriptor_cntl
 *   |html </b>
 *   |html <i>
 *   dd
 *   driver
 *   GLOBUS_XIO_TCP_SET_SEND_FLAGS
 *   send_flags
 *   |html </i>
 *
 * DESCRIPTION
 *   Set TCP send flags. Used only for data descriptors to write calls.
 *
 * PARAMETERS
 *   send_flags - The flags to use when sending data.
 *
 * RETURN VALUE
 *   On success globus_xio_data_descriptor_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio/globus_xio_tcp_send_flags_t
 *****/


/****t* globus_xio/globus_xio_data_descriptor_cntl/GLOBUS_XIO_TCP_GET_SEND_FLAGS
 * ATTRIBUTES
 *   GLOBUS_XIO_TCP_GET_SEND_FLAGS
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_data_descriptor_cntl
 *   |html </b>
 *   |html <i>
 *   dd
 *   driver
 *   GLOBUS_XIO_TCP_GET_SEND_FLAGS
 *   |html </i>
 *
 * DESCRIPTION
 *   Get TCP send flags.
 *
 * PARAMETERS
 *   send_flags_out - The flags to use will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_data_descriptor_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****************************************************************************
 *
 * GLOBUS XIO UDP DRIVER - ATTRIBUTES AND CNTLS
 *
 ****************************************************************************/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_HANDLE
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_HANDLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_SET_HANDLE
 *   handle
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the UDP socket to use.
 *
 * PARAMETERS
 *   handle - Use this handle (fd or SOCKET). Note: close() will not be
 *   called on this handle.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_SERVICE
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_SERVICE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_SET_SERVICE
 *   service_name
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the UDP service name to listen on.
 *
 * PARAMETERS
 *   service_name - The service name to use when setting up the listener.
 *   If the service name cannot be resolved, the port (if one is set) will
 *   be used instead.
* 
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_GET_SERVICE
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_SERVICE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_GET_SERVICE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the service name to listen on.
 *
 * PARAMETERS
 *   service_name_out - A pointer to the service name will be stored here.
 *   If none is set, NULL will be passed back. Otherwise, the name will be
 *   duplicated with strdup() and the user should call free() on it.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the service name.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_PORT
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_PORT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_SET_PORT
 *   listener_port
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the port number to listen on. The default is 0 (system assigned).
 *
 * PARAMETERS
 *   listener_port - The port number to use when setting up the listener.
 *   If the service name is also set, this will only be used if that can't
 *   be resolved. 
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_GET_PORT
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_PORT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_GET_PORT
 *   |html </i>
 *
 * DESCRIPTION
 *   The port number to listen on.
 *
 * PARAMETERS
 *   listener_port_out - The port will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and the port number.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_LISTEN_RANGE
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_LISTEN_RANGE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_SET_LISTEN_RANGE
 *   listener_min_port
 *   listener_max_port
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the port range to confine the listener to. Used only where no
 *   specific service or port has been set. It overrides the range set
 *   in the GLOBUS_UDP_PORT_RANGE env variable. If 'restrict port' is
 *   true, the listening port will be constrained to the range specified.
 *
 * PARAMETERS
 *   |html <i>
 *   listener_min_port - The lower bound on the listener port.
 *   (default 0 -- no bound).
 *
 *   listener_max_port - The upper bound on the listener port.
 *   (default 0 -- no bound)
 *   |html </i>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 & SEE ALSO
 *   globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_RESTRICT_PORT
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_GET_LISTEN_RANGE
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_LISTEN_RANGE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_GET_LISTEN_RANGE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the UDP port range on an attr.
 *
 * PARAMETERS
 *   |html <i>
 *   listener_min_port_out - The lower bound will be stored here.
 *
 *   listener_max_port_out - The upper bound will be stored here.
 *   |html </i>
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0, listener_min_port_out
 *   and listener_max_port_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_INTERFACE
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_INTERFACE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_SET_INTERFACE
 *   interface
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the interface to bind the socket to.
 *
 * PARAMETERS
 *   interface - The interface to be used. Can be a hostname or numeric IP.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_GET_INTERFACE
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_INTERFACE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_GET_INTERFACE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the interface on the attr.
 *
 * PARAMETERS
 *   interface_out - A pointer to the interface will be stored here. If
 *   one is set, NULL will be passed back. Otherwise, the interface will
 *   be duplicated with strdup() and the user should call free() on it.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and interface_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_RESTRICT_PORT
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_RESTRICT_PORT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_SET_RESTRICT_PORT
 *   restrict_port
 *   |html </i>
 *
 * DESCRIPTION
 *   Enable or disable the listener range constraints. This enables or
 *   ignores the port range found in the attr or in then env. By default,
 *   those ranges are enabled.
 *
 * PARAMETERS
 *   restrict_port - GLOBUS_TRUE to enable (default), GLOBUS_FALSE to
 *   disable.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 * SEE ALSO
 *   globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_LISTEN_RANGE
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_GET_RESTRICT_PORT
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_RESTRICT_PORT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_GET_RESTRICT_PORT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the restrict port flag.
 *
 * PARAMETERS
 *   restrict_port_out - The restrict port flag will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and restrict_port_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_REUSEADDR
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_REUSEADDR
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_SET_REUSEADDR
 *   reuseaddr
 *   |html </i>
 *
 * DESCRIPTION
 *   Reuse addr when binding. Used to determine whether or not to allow
 *   reuse of addresses when binding a socket to a port number.
 *
 * PARAMETERS
 *   resuseaddr - GLOBUS_TRUE to allow, GLOBUS_FALSE to disallow (default).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_GET_REUSEADDR
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_REUSEADDR
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_GET_REUSEADDR
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the reuseaddr flag on an attr.
 *
 * PARAMETERS
 *   resuseaddr_out - The reuseaddr flag will be stored here. 
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and reuseaddr_out flag.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_NO_IPV6
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_NO_IPV6
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_SET_NO_IPV6
 *   no_ipv6
 *   |html </i>
 *
 * DESCRIPTION
 *   Restrict to IPV4 only. Disallow IPV6 sockets from being used (default
 *   is to use either IPV4 or IPV6).
 *
 * PARAMETERS
 *   no_ipv6 - GLOBUS_TRUE to disallow IPV6, GLOBUS_FALSE to allow (default).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_GET_NO_IPV6
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_NO_IPV6
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_GET_NO_IPV6
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the no IPV6 flag on an attr.
 *
 * PARAMETERS
 *   no_ipv6_out - The no IPV6 flag will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and no_ipv6_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_GET_HANDLE
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_HANDLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_GET_HANDLE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the socket handle on an attr or handle.
 *
 * PARAMETERS
 *   handle_out - The UDP socket will be stored here. If none is set,
 *   GLOBUS_XIO_UDP_INVALID_HANDLE will be set.
 *
 * RETURN VALUE
 *   On success globus_xio_server_cntl returns 0 and handle_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_SNDBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_SNDBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_SET_SNDBUF
 *   sndbuf
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the socket send buffer size. Used to set the size of the send
 *   buffer used on the socket.
 *
 * PARAMETERS
 *   sndbuf - The send buffer size in bytes to use. (default is system
 *   specific).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_GET_SNDBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_SNDBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_GET_SNDBUF
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the send buffer size on the attr or handle.
 *
 * PARAMETERS
 *   sndbuf_out - The send buffer size will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and sndbuf_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_SET_RCVBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_RCVBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_SET_RCVBUF
 *   rcvbuf
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the socket receive buffer size. Used to set the size of the receive
 *   buffer used on the socket.
 *
 * PARAMETERS
 *   rcvbuf - The receive buffer size in bytes. (default is system specific).
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_attr_cntl/GLOBUS_XIO_UDP_GET_RCVBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_RCVBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_attr_cntl
 *   |html </b>
 *   |html <i>
 *   attr
 *   driver
 *   GLOBUS_XIO_UDP_GET_RCVBUF
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the UDP receive buffer size on the attr or handle.
 *
 * PARAMETERS
 *   rcvbuf_out - The receive buffer size will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_attr_cntl returns 0 and rcvbuf_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_UDP_GET_HANDLE
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_HANDLE
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_UDP_GET_HANDLE
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the socket handle on an attr or handle.
 *
 * PARAMETERS
 *   handle_out - The UDP socket will be stored here. If none is set,
 *   GLOBUS_XIO_UDP_INVALID_HANDLE will be set.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and handle_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_UDP_SET_SNDBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_SNDBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_UDP_SET_SNDBUF
 *   sndbuf
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the socket send buffer size. Used to set the size of the send
 *   buffer used on the socket.
 *
 * PARAMETERS
 *   sndbuf - The send buffer size in bytes to use. (default is system
 *   specific).
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_UDP_GET_SNDBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_SNDBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_UDP_GET_SNDBUF
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the send buffer size on the attr or handle.
 *
 * PARAMETERS
 *   sndbuf_out - The send buffer size will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and sndbuf_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_UDP_SET_RCVBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_SET_RCVBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_UDP_SET_RCVBUF
 *   rcvbuf
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the socket receive buffer size. Used to set the size of the
 *   receive buffer used on the socket.
 *
 * PARAMETERS
 *   rcvbuf - The receive buffer size in bytes. (default is system specific).
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_UDP_GET_RCVBUF
 * ATTRIBUTES
 *     GLOBUS_XIO_UDP_GET_RCVBUF
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_UDP_GET_RCVBUF
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the receive buffer size on the attr or handle.
 *
 * PARAMETERS
 *   rcvbuf_out - The receive buffer size will be stored here.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and rcvbuf_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_TCP_GET_CONTACT
 * ATTRIBUTES
 *   GLOBUS_XIO_TCP_GET_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_TCP_GET_CONTACT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the contact string associated with a handle or data descriptor.
 *   Use with globus_xio_handle_cntl() to get a contact string for the UDP
 *   listener. Use with globus_xio_data_descriptor_cntl() to get the sender's
 *   contact string from a data descriptor passed to
 *   globus_xio_register_read().
 *
 * PARAMETERS
 *   contact_string_out - A pointer to a contact string will be stored here.
 *   The user should free() it when done with it. It will be in the format:
 *   <hostname>:<port>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the contact_string_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_UDP_GET_NUMERIC_CONTACT
 * ATTRIBUTES
 *   GLOBUS_XIO_UDP_GET_NUMERIC_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_UDP_GET_NUMERIC_CONTACT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the contact string associated with a handle or data descriptor.
 *   Use with globus_xio_handle_cntl() to get a contact string for the UDP
 *   listener. Use with globus_xio_data_descriptor_cntl() to get the sender's
 *   contact string from a data descriptor passed to
 *   globus_xio_register_read().
 *
 * PARAMETERS
 *   contact_string_out - A pointer to a contact string will be stored here.
 *   The user should free() it when done with it. It will be in the format:
 *   <ip>:<port>.
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0 and the contact_string_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_handle_cntl/GLOBUS_XIO_UDP_CONNECT
 * ATTRIBUTES
 *   GLOBUS_XIO_UDP_CONNECT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_handle_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_UDP_CONNECT
 *   contact_string
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the default destination contact. Connecting a handle to a specific
 *   contact blocks packets from any other contact. It also sets the default
 *   destination of all outgoing packets so, using GLOBUS_XIO_UDP_SET_CONTACT
 *   is unnecessary.
 *
 * PARAMETERS
 *   contact_string - A pointer to a contact string of the format
 *   <hostname/ip>:<port/service>
 *
 * RETURN VALUE
 *   On success globus_xio_handle_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_data_descriptor_cntl/GLOBUS_XIO_UDP_GET_CONTACT
 * ATTRIBUTES
 *   GLOBUS_XIO_UDP_GET_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_data_descriptor_cntl
 *   |html </b>
 *   |html <i>
 *   dd
 *   driver
 *   GLOBUS_XIO_UDP_GET_CONTACT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the contact string associated with a handle or data descriptor.
 *   Use with globus_xio_handle_cntl() to get a contact string for the UDP
 *   listener. Use with globus_xio_data_descriptor_cntl() to get the sender's
 *   contact string from a data descriptor passed to
 *   globus_xio_register_read().
 *
 * PARAMETERS
 *   contact_string_out - A pointer to a contact string will be stored here.
 *   The user should free() it when done with it. It will be in the format:
 *   <hostname>:<port>
 *
 * RETURN VALUE
 *   On success globus_xio_data_descriptor_cntl returns 0 and the
 *   contact_string_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/



/****t* globus_xio/globus_xio_data_descriptor_cntl/GLOBUS_XIO_UDP_GET_NUMERIC_CONTACT
 * ATTRIBUTES
 *    GLOBUS_XIO_UDP_GET_NUMERIC_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_data_descriptor_cntl
 *   |html </b>
 *   |html <i>
 *   dd
 *   driver
 *   GLOBUS_XIO_UDP_GET_NUMERIC_CONTACT
 *   |html </i>
 *
 * DESCRIPTION
 *   Get the contact string associated with a handle or data descriptor.
 *   Use with globus_xio_handle_cntl() to get a contact string for the UDP
 *   listener. Use with globus_xio_data_descriptor_cntl() to get the sender's
 *   contact string from a data descriptor passed to
 *   globus_xio_register_read().
 *
 * PARAMETERS
 *   contact_string_out - A pointer to a contact string will be stored here.
 *   The user should free() it when done with it. It will be in the format:
 *   <ip>:<port>.
 *
 * RETURN VALUE
 *   On success globus_xio_data_descriptor_cntl returns 0 and the
 *   contact_string_out.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****t* globus_xio/globus_xio_data_descriptor_cntl/GLOBUS_XIO_UDP_SET_CONTACT
 * ATTRIBUTES
 *   GLOBUS_XIO_UDP_SET_CONTACT
 *
 * SYNOPSIS
 *   |html <b>
 *   package require globus_xio
 *   |html <br>
 *   |html <br>
 *   |html </b>
 *   |html <b>
 *   globus_xio_data_descriptor_cntl
 *   |html </b>
 *   |html <i>
 *   handle
 *   driver
 *   GLOBUS_XIO_UDP_SET_CONTACT
 *   contact_string
 *   |html </i>
 *
 * DESCRIPTION
 *   Set the destination contact. Use on a data descriptor passed to
 *   globus_xio_register_write() to specify the recipient of the data.
 *   This is necessary with unconnected handles or to send to recipients
 *   other than the connected one.
 *
 * PARAMETERS
 *   contact_string - A pointer to a contact string of the format
 *   <hostname/ip>:<port/service>
 *
 * RETURN VALUE
 *   On success globus_xio_data_descriptor_cntl returns 0.
 *
 * EXAMPLE
 *   |html <font color="Red">
 *   TBD
 *   |html </font>
 *****/


/****************************************************************************
 *
 * GLOBUS XIO HTTP DRIVER - ATTRIBUTES AND CNTLS
 *
 ****************************************************************************/
